export as namespace fhir5;
// <auto-generated/>
// Contents of: hl7.fhir.r5.core version: 5.0.0-ballot
// Primitive Naming Style: None
// Complex Type / Resource Naming Style: PascalCase
// Interaction Naming Style: None
// Extension Support: NonPrimitive
// Minimum TypeScript Version: 3.7
/**
 * An address expressed using postal conventions (as opposed to GPS or other location definition formats).  This data type may be used to convey addresses for use in delivering mail as well as for visiting locations which might not be valid for mail delivery.  There are a variety of postal address formats defined around the world.
 * The ISO21090-codedString may be used to provide a coded representation of the contents of strings in an Address.
 */
export interface Address extends DataType {
    /**
     * The name of the city, town, suburb, village or other community or delivery center.
     */
    city?: string | undefined;
    _city?: Element | undefined;
    /**
     * ISO 3166 2- or 3- letter codes MAY be used in place of a human readable country name.
     */
    country?: string | undefined;
    _country?: Element | undefined;
    /**
     * District is sometimes known as county, but in some regions 'county' is used in place of city (municipality), so county name should be conveyed in city instead.
     */
    district?: string | undefined;
    _district?: Element | undefined;
    /**
     * This component contains the house number, apartment number, street name, street direction,  P.O. Box number, delivery hints, and similar address information.
     */
    line?: string[] | undefined;
    _line?: Element[] | undefined;
    /**
     * Time period when address was/is in use.
     */
    period?: Period | undefined;
    /**
     * A postal code designating a region defined by the postal service.
     */
    postalCode?: string | undefined;
    _postalCode?: Element | undefined;
    /**
     * Sub-unit of a country with limited sovereignty in a federally organized country. A code may be used if codes are in common use (e.g. US 2 letter state codes).
     */
    state?: string | undefined;
    _state?: Element | undefined;
    /**
     * Can provide both a text representation and parts. Applications updating an address SHALL ensure that  when both text and parts are present,  no content is included in the text that isn't found in a part.
     */
    text?: string | undefined;
    _text?: Element | undefined;
    /**
     * The definition of Address states that "address is intended to describe postal addresses, not physical locations". However, many applications track whether an address has a dual purpose of being a location that can be visited as well as being a valid delivery destination, and Postal addresses are often used as proxies for physical locations (also see the [Location](location.html#) resource).
     */
    type?: ('postal' | 'physical' | 'both') | undefined;
    _type?: Element | undefined;
    /**
     * Applications can assume that an address is current unless it explicitly says that it is temporary or old.
     */
    use?: ('home' | 'work' | 'temp' | 'old' | 'billing') | undefined;
    _use?: Element | undefined;
}
/**
 * A duration of time during which an organism (or a process) has existed.
 */
export interface Age extends Quantity {}
/**
 * A  text note which also  contains information about who made the statement and when.
 */
export interface Annotation extends DataType {
    /**
     * Organization is used when there's no need for specific attribution as to who made the comment.
     */
    authorReference?: Reference | undefined;
    /**
     * Organization is used when there's no need for specific attribution as to who made the comment.
     */
    authorString?: string | undefined;
    _authorString?: Element | undefined;
    /**
     * The text of the annotation in markdown format.
     */
    text: string;
    _text?: Element | undefined;
    /**
     * Indicates when this particular annotation was made.
     */
    time?: string | undefined;
    _time?: Element | undefined;
}
/**
 * For referring to data content defined in other formats.
 */
export interface Attachment extends DataType {
    /**
     * Identifies the type of the data in the attachment and allows a method to be chosen to interpret or render the data. Includes mime type parameters such as charset where appropriate.
     */
    contentType?: string | undefined;
    _contentType?: Element | undefined;
    /**
     * The date that the attachment was first created.
     */
    creation?: string | undefined;
    _creation?: Element | undefined;
    /**
     * The base64-encoded data SHALL be expressed in the same character set as the base resource XML or JSON.
     */
    data?: string | undefined;
    _data?: Element | undefined;
    /**
     * The duration might differ from occurrencePeriod if recording was paused.
     */
    duration?: number | undefined;
    /**
     * if the number of frames is not supplied, the value may be unknown. Applications should not assume that there is only one frame unless it is explicitly stated.
     */
    frames?: number | undefined;
    /**
     * The hash is calculated on the data prior to base64 encoding, if the data is based64 encoded. The hash is not intended to support digital signatures. Where protection against malicious threats a digital signature should be considered, see [Provenance.signature](provenance-definitions.html#Provenance.signature) for mechanism to protect a resource with a digital signature.
     */
    hash?: string | undefined;
    _hash?: Element | undefined;
    /**
     * Height of the image in pixels (photo/video).
     */
    height?: number | undefined;
    /**
     * The human language of the content. The value can be any valid value according to BCP 47.
     */
    language?: string | undefined;
    _language?: Element | undefined;
    /**
     * The number of pages when printed.
     */
    pages?: number | undefined;
    /**
     * The number of bytes is redundant if the data is provided as a base64binary, but is useful if the data is provided as a url reference.
     */
    size?: string | undefined;
    _size?: Element | undefined;
    /**
     * May sometimes be derived from the source filename.
     */
    title?: string | undefined;
    _title?: Element | undefined;
    /**
     * If both data and url are provided, the url SHALL point to the same content as the data contains. Urls may be relative references or may reference transient locations such as a wrapping envelope using cid: though this has ramifications for using signatures. Relative URLs are interpreted relative to the service url, like a resource reference, rather than relative to the resource itself. If a URL is provided, it SHALL resolve to actual data.
     */
    url?: string | undefined;
    _url?: Element | undefined;
    /**
     * Width of the image in pixels (photo/video).
     */
    width?: number | undefined;
}
/**
 * Times the {item} is available.
 */
export interface AvailabilityAvailableTime extends Element {
    /**
     * Always available? i.e. 24 hour service.
     */
    allDay?: boolean | undefined;
    _allDay?: Element | undefined;
    /**
     * The time zone is expected to be specified or implied by the context this datatype is used.
     */
    availableEndTime?: string | undefined;
    _availableEndTime?: Element | undefined;
    /**
     * The time zone is expected to be specified or implied by the context this datatype is used.
     */
    availableStartTime?: string | undefined;
    _availableStartTime?: Element | undefined;
    /**
     * mon | tue | wed | thu | fri | sat | sun.
     */
    daysOfWeek?: ('mon' | 'tue' | 'wed' | 'thu' | 'fri' | 'sat' | 'sun')[] | undefined;
    _daysOfWeek?: Element[] | undefined;
}
/**
 * Not available during this time due to provided reason.
 */
export interface AvailabilityNotAvailableTime extends Element {
    /**
     * The reason will generally be provided to give the textual reason for displaying when the {item} is not available, e.g. 'Closed public holidays' or 'Independence Day'. In cases such as this, the `during` might not be included and local knowledge would be required in such cases (as don't desire to keep updating when the holiday occurs each year).
     * e.g.2: 'Closed for maintenance over the summer' for this example you would want to include the `during` period, unless this was a university hospital and the "summer" period was well known, but would recommend its inclusion anyway.
     */
    description?: string | undefined;
    _description?: Element | undefined;
    /**
     * Service not available during this period.
     */
    during?: Period | undefined;
}
/**
 * Availability data for an {item}.
 */
export interface Availability extends DataType {
    /**
     * Times the {item} is available.
     */
    availableTime?: AvailabilityAvailableTime[] | undefined;
    /**
     * Not available during this time due to provided reason.
     */
    notAvailableTime?: AvailabilityNotAvailableTime[] | undefined;
}
/**
 * Base definition for all elements that are defined inside a resource - but not those in a data type.
 */
export interface BackboneElement extends Element {
    /**
     * There can be no stigma associated with the use of extensions by any application, project, or standard - regardless of the institution or jurisdiction that uses or defines the extensions.  The use of extensions is what allows the FHIR specification to retain a core level of simplicity for everyone.
     */
    modifierExtension?: Extension[] | undefined;
}
/**
 * Base definition for the few data types that are allowed to carry modifier extensions.
 */
export interface BackboneType extends DataType {
    /**
     * There can be no stigma associated with the use of extensions by any application, project, or standard - regardless of the institution or jurisdiction that uses or defines the extensions.  The use of extensions is what allows the FHIR specification to retain a core level of simplicity for everyone.
     */
    modifierExtension?: Extension[] | undefined;
}
/**
 * Base definition for all types defined in FHIR type system.
 */
export interface Base {}
/**
 * A concept that may be defined by a formal reference to a terminology or ontology or may be provided by text.
 */
export interface CodeableConcept extends DataType {
    /**
     * Codes may be defined very casually in enumerations, or code lists, up to very formal definitions such as SNOMED CT - see the HL7 v3 Core Principles for more information.  Ordering of codings is undefined and SHALL NOT be used to infer meaning. Generally, at most only one of the coding values will be labeled as UserSelected = true.
     */
    coding?: Coding[] | undefined;
    /**
     * Very often the text is the same as a displayName of one of the codings.
     */
    text?: string | undefined;
    _text?: Element | undefined;
}
/**
 * A reference to a resource (by instance), or instead, a reference to a concept defined in a terminology or ontology (by class).
 */
export interface CodeableReference extends DataType {
    /**
     * A reference to a concept - e.g. the information is identified by its general class to the degree of precision found in the terminology.
     */
    concept?: CodeableConcept | undefined;
    /**
     * A reference to a resource the provides exact details about the information being referenced.
     */
    reference?: Reference | undefined;
}
/**
 * A reference to a code defined by a terminology system.
 */
export interface Coding extends DataType {
    /**
     * A symbol in syntax defined by the system. The symbol may be a predefined code or an expression in a syntax defined by the coding system (e.g. post-coordination).
     */
    code?: string | undefined;
    _code?: Element | undefined;
    /**
     * A representation of the meaning of the code in the system, following the rules of the system.
     */
    display?: string | undefined;
    _display?: Element | undefined;
    /**
     * The URI may be an OID (urn:oid:...) or a UUID (urn:uuid:...).  OIDs and UUIDs SHALL be references to the HL7 OID registry. Otherwise, the URI should come from HL7's list of FHIR defined special URIs or it should be an absolute reference to some definition that establishes the system clearly and unambiguously.
     */
    system?: string | undefined;
    _system?: Element | undefined;
    /**
     * Amongst a set of alternatives, a directly chosen code is the most appropriate starting point for new translations. There is some ambiguity about what exactly 'directly chosen' implies, and trading partner agreement may be needed to clarify the use of this element and its consequences more completely.
     */
    userSelected?: boolean | undefined;
    _userSelected?: Element | undefined;
    /**
     * Where the terminology does not clearly define what string should be used to identify code system versions, the recommendation is to use the date (expressed in FHIR date format) on which that version was officially published as the version date.
     */
    version?: string | undefined;
    _version?: Element | undefined;
}
/**
 * Specifies contact information for a person or organization.
 */
export interface ContactDetail extends DataType {
    /**
     * If there is no named individual, the telecom information is for the organization as a whole.
     */
    name?: string | undefined;
    _name?: Element | undefined;
    /**
     * The contact details for the individual (if a name was provided) or the organization.
     */
    telecom?: ContactPoint[] | undefined;
}
/**
 * Details for all kinds of technology mediated contact points for a person or organization, including telephone, email, etc.
 */
export interface ContactPoint extends DataType {
    /**
     * Time period when the contact point was/is in use.
     */
    period?: Period | undefined;
    /**
     * Note that rank does not necessarily follow the order in which the contacts are represented in the instance.
     */
    rank?: number | undefined;
    /**
     * Telecommunications form for contact point - what communications system is required to make use of the contact.
     */
    system?: ('phone' | 'fax' | 'email' | 'pager' | 'url' | 'sms' | 'other') | undefined;
    _system?: Element | undefined;
    /**
     * Applications can assume that a contact is current unless it explicitly says that it is temporary or old.
     */
    use?: ('home' | 'work' | 'temp' | 'old' | 'mobile') | undefined;
    _use?: Element | undefined;
    /**
     * Additional text data such as phone extension numbers, or notes about use of the contact are sometimes included in the value.
     */
    value?: string | undefined;
    _value?: Element | undefined;
}
/**
 * A contributor to the content of a knowledge asset, including authors, editors, reviewers, and endorsers.
 */
export interface Contributor extends DataType {
    /**
     * Contact details to assist a user in finding and communicating with the contributor.
     */
    contact?: ContactDetail[] | undefined;
    /**
     * The name of the individual or organization responsible for the contribution.
     */
    name: string;
    _name?: Element | undefined;
    /**
     * The type of contributor.
     */
    type: 'author' | 'editor' | 'reviewer' | 'endorser';
    _type?: Element | undefined;
}
/**
 * A measured amount (or an amount that can potentially be measured). Note that measured amounts include amounts that are not precisely quantified, including amounts involving arbitrary units and floating currencies.
 */
export interface Count extends Quantity {}
/**
 * Code filters specify additional constraints on the data, specifying the value set of interest for a particular element of the data. Each code filter defines an additional constraint on the data, i.e. code filters are AND'ed, not OR'ed.
 */
export interface DataRequirementCodeFilter extends Element {
    /**
     * The codes for the code filter. If values are given, the filter will return only those data items for which the code-valued attribute specified by the path has a value that is one of the specified codes. If codes are specified in addition to a value set, the filter returns items matching a code in the value set or one of the specified codes.
     */
    code?: Coding[] | undefined;
    /**
     * The path attribute contains a [Simple FHIRPath Subset](fhirpath.html#simple) that allows path traversal, but not calculation.
     */
    path?: string | undefined;
    _path?: Element | undefined;
    /**
     * A token parameter that refers to a search parameter defined on the specified type of the DataRequirement, and which searches on elements of type code, Coding, or CodeableConcept.
     */
    searchParam?: string | undefined;
    _searchParam?: Element | undefined;
    /**
     * The valueset for the code filter. The valueSet and code elements are additive. If valueSet is specified, the filter will return only those data items for which the value of the code-valued element specified in the path is a member of the specified valueset.
     */
    valueSet?: string | undefined;
    _valueSet?: Element | undefined;
}
/**
 * Date filters specify additional constraints on the data in terms of the applicable date range for specific elements. Each date filter specifies an additional constraint on the data, i.e. date filters are AND'ed, not OR'ed.
 */
export interface DataRequirementDateFilter extends Element {
    /**
     * The path attribute contains a [Simple FHIR Subset](fhirpath.html#simple) that allows path traversal, but not calculation.
     */
    path?: string | undefined;
    _path?: Element | undefined;
    /**
     * A date parameter that refers to a search parameter defined on the specified type of the DataRequirement, and which searches on elements of type date, dateTime, Period, Schedule, or Timing.
     */
    searchParam?: string | undefined;
    _searchParam?: Element | undefined;
    /**
     * The value of the filter. If period is specified, the filter will return only those data items that fall within the bounds determined by the Period, inclusive of the period boundaries. If dateTime is specified, the filter will return only those data items that are equal to the specified dateTime. If a Duration is specified, the filter will return only those data items that fall within Duration before now.
     */
    valueDateTime?: string | undefined;
    _valueDateTime?: Element | undefined;
    /**
     * The value of the filter. If period is specified, the filter will return only those data items that fall within the bounds determined by the Period, inclusive of the period boundaries. If dateTime is specified, the filter will return only those data items that are equal to the specified dateTime. If a Duration is specified, the filter will return only those data items that fall within Duration before now.
     */
    valuePeriod?: Period | undefined;
    /**
     * The value of the filter. If period is specified, the filter will return only those data items that fall within the bounds determined by the Period, inclusive of the period boundaries. If dateTime is specified, the filter will return only those data items that are equal to the specified dateTime. If a Duration is specified, the filter will return only those data items that fall within Duration before now.
     */
    valueDuration?: Duration | undefined;
}
/**
 * Value filters specify additional constraints on the data for elements other than code-valued or date-valued. Each value filter specifies an additional constraint on the data (i.e. valueFilters are AND'ed, not OR'ed).
 */
export interface DataRequirementValueFilter extends Element {
    /**
     * The comparator to be used to determine whether the value is matching.
     */
    comparator?: ('eq' | 'gt' | 'lt' | 'ge' | 'le' | 'sa' | 'eb') | undefined;
    _comparator?: Element | undefined;
    /**
     * The path attribute contains a [Simple FHIR Subset](fhirpath.html#simple) that allows path traversal, but not calculation.
     */
    path?: string | undefined;
    _path?: Element | undefined;
    /**
     * A search parameter defined on the specified type of the DataRequirement, and which searches on elements of a type compatible with the type of the valueFilter.value[x] for the filter.
     */
    searchParam?: string | undefined;
    _searchParam?: Element | undefined;
    /**
     * The value of the filter.
     */
    valueDateTime?: string | undefined;
    _valueDateTime?: Element | undefined;
    /**
     * The value of the filter.
     */
    valuePeriod?: Period | undefined;
    /**
     * The value of the filter.
     */
    valueDuration?: Duration | undefined;
}
/**
 * This element can be used in combination with the sort element to specify quota requirements such as "the most recent 5" or "the highest 5". When multiple sorts are specified, they are applied in the order they appear in the resource.
 */
export interface DataRequirementSort extends Element {
    /**
     * The direction of the sort, ascending or descending.
     */
    direction: 'ascending' | 'descending';
    _direction?: Element | undefined;
    /**
     * The attribute of the sort. The specified path must be resolvable from the type of the required data. The path is allowed to contain qualifiers (.) to traverse sub-elements, as well as indexers ([x]) to traverse multiple-cardinality sub-elements. Note that the index must be an integer constant.
     */
    path: string;
    _path?: Element | undefined;
}
/**
 * Describes a required data item for evaluation in terms of the type of data, and optional code or date-based filters of the data.
 */
export interface DataRequirement extends DataType {
    /**
     * Code filters specify additional constraints on the data, specifying the value set of interest for a particular element of the data. Each code filter defines an additional constraint on the data, i.e. code filters are AND'ed, not OR'ed.
     */
    codeFilter?: DataRequirementCodeFilter[] | undefined;
    /**
     * Date filters specify additional constraints on the data in terms of the applicable date range for specific elements. Each date filter specifies an additional constraint on the data, i.e. date filters are AND'ed, not OR'ed.
     */
    dateFilter?: DataRequirementDateFilter[] | undefined;
    /**
     * This element can be used in combination with the sort element to specify quota requirements such as "the most recent 5" or "the highest 5".
     */
    limit?: number | undefined;
    /**
     * Indicates that specific elements of the type are referenced by the knowledge module and must be supported by the consumer in order to obtain an effective evaluation. This does not mean that a value is required for this element, only that the consuming system must understand the element and be able to provide values for it if they are available.
     * The value of mustSupport SHALL be a FHIRPath resolveable on the type of the DataRequirement. The path SHALL consist only of identifiers, constant indexers, and .resolve() (see the [Simple FHIRPath Profile](fhirpath.html#simple) for full details).
     */
    mustSupport?: string[] | undefined;
    _mustSupport?: Element[] | undefined;
    /**
     * The profile of the required data, specified as the uri of the profile definition.
     */
    profile?: string[] | undefined;
    _profile?: Element[] | undefined;
    /**
     * This element can be used in combination with the sort element to specify quota requirements such as "the most recent 5" or "the highest 5". When multiple sorts are specified, they are applied in the order they appear in the resource.
     */
    sort?: DataRequirementSort[] | undefined;
    /**
     * The subject of a data requirement is critical, as the data being specified is determined with respect to a particular subject. This corresponds roughly to the notion of a Compartment in that it limits what data is available based on its relationship to the subject. In CQL, this corresponds to the context declaration.
     */
    subjectCodeableConcept?: CodeableConcept | undefined;
    /**
     * The subject of a data requirement is critical, as the data being specified is determined with respect to a particular subject. This corresponds roughly to the notion of a Compartment in that it limits what data is available based on its relationship to the subject. In CQL, this corresponds to the context declaration.
     */
    subjectReference?: Reference | undefined;
    /**
     * The type of the required data, specified as the type name of a resource. For profiles, this value is set to the type of the base resource of the profile.
     */
    type: string;
    _type?: Element | undefined;
    /**
     * Value filters specify additional constraints on the data for elements other than code-valued or date-valued. Each value filter specifies an additional constraint on the data (i.e. valueFilters are AND'ed, not OR'ed).
     */
    valueFilter?: DataRequirementValueFilter[] | undefined;
}
/**
 * The base class for all re-useable types defined as part of the FHIR Specification.
 */
export interface DataType extends Element {}
/**
 * A length - a value with a unit that is a physical distance.
 */
export interface Distance extends Quantity {}
/**
 * Depending on the resource,this is the amount of medication administered, to  be administered or typical amount to be administered.
 */
export interface DosageDoseAndRate extends Element {
    /**
     * Note that this specifies the quantity of the specified medication, not the quantity for each active ingredient(s). Each ingredient amount can be communicated in the Medication resource. For example, if one wants to communicate that a tablet was 375 mg, where the dose was one tablet, you can use the Medication resource to document that the tablet was comprised of 375 mg of drug XYZ. Alternatively if the dose was 375 mg, then you may only need to use the Medication resource to indicate this was a tablet. If the example were an IV such as dopamine and you wanted to communicate that 400mg of dopamine was mixed in 500 ml of some IV solution, then this would all be communicated in the Medication resource. If the administration is not intended to be instantaneous (rate is present or timing has a duration), this can be specified to convey the total amount to be administered over the period of time as indicated by the schedule e.g. 500 ml in dose, with timing used to convey that this should be done over 4 hours.
     */
    doseRange?: Range | undefined;
    /**
     * Note that this specifies the quantity of the specified medication, not the quantity for each active ingredient(s). Each ingredient amount can be communicated in the Medication resource. For example, if one wants to communicate that a tablet was 375 mg, where the dose was one tablet, you can use the Medication resource to document that the tablet was comprised of 375 mg of drug XYZ. Alternatively if the dose was 375 mg, then you may only need to use the Medication resource to indicate this was a tablet. If the example were an IV such as dopamine and you wanted to communicate that 400mg of dopamine was mixed in 500 ml of some IV solution, then this would all be communicated in the Medication resource. If the administration is not intended to be instantaneous (rate is present or timing has a duration), this can be specified to convey the total amount to be administered over the period of time as indicated by the schedule e.g. 500 ml in dose, with timing used to convey that this should be done over 4 hours.
     */
    doseQuantity?: Quantity | undefined;
    /**
     * It is possible to supply both a rate and a doseQuantity to provide full details about how the medication is to be administered and supplied. If the rate is intended to change over time, depending on local rules/regulations, each change should be captured as a new version of the MedicationRequest with an updated rate, or captured with a new MedicationRequest with the new rate.
     * It is possible to specify a rate over time (for example, 100 ml/hour) using either the rateRatio and rateQuantity.  The rateQuantity approach requires systems to have the capability to parse UCUM grammer where ml/hour is included rather than a specific ratio where the time is specified as the denominator.  Where a rate such as 500ml over 2 hours is specified, the use of rateRatio may be more semantically correct than specifying using a rateQuantity of 250 mg/hour.
     */
    rateRatio?: Ratio | undefined;
    /**
     * It is possible to supply both a rate and a doseQuantity to provide full details about how the medication is to be administered and supplied. If the rate is intended to change over time, depending on local rules/regulations, each change should be captured as a new version of the MedicationRequest with an updated rate, or captured with a new MedicationRequest with the new rate.
     * It is possible to specify a rate over time (for example, 100 ml/hour) using either the rateRatio and rateQuantity.  The rateQuantity approach requires systems to have the capability to parse UCUM grammer where ml/hour is included rather than a specific ratio where the time is specified as the denominator.  Where a rate such as 500ml over 2 hours is specified, the use of rateRatio may be more semantically correct than specifying using a rateQuantity of 250 mg/hour.
     */
    rateRange?: Range | undefined;
    /**
     * It is possible to supply both a rate and a doseQuantity to provide full details about how the medication is to be administered and supplied. If the rate is intended to change over time, depending on local rules/regulations, each change should be captured as a new version of the MedicationRequest with an updated rate, or captured with a new MedicationRequest with the new rate.
     * It is possible to specify a rate over time (for example, 100 ml/hour) using either the rateRatio and rateQuantity.  The rateQuantity approach requires systems to have the capability to parse UCUM grammer where ml/hour is included rather than a specific ratio where the time is specified as the denominator.  Where a rate such as 500ml over 2 hours is specified, the use of rateRatio may be more semantically correct than specifying using a rateQuantity of 250 mg/hour.
     */
    rateQuantity?: Quantity | undefined;
    /**
     * The kind of dose or rate specified, for example, ordered or calculated.
     */
    type?: CodeableConcept | undefined;
}
/**
 * Indicates how the medication is/was taken or should be taken by the patient.
 */
export interface Dosage extends BackboneType {
    /**
     * Information about administration or preparation of the medication (e.g. "infuse as rapidly as possibly via intraperitoneal port" or "immediately following drug x") should be populated in dosage.text.
     */
    additionalInstruction?: CodeableConcept[] | undefined;
    /**
     * Can express "as needed" without a reason by setting the Boolean = True.  In this case the CodeableConcept is not populated.
     */
    asNeeded?: boolean | undefined;
    _asNeeded?: Element | undefined;
    /**
     * Can express "as needed" with a reason by including the CodeableConcept.  In this case the Boolean is assumed to be False, then the dose is given according to the schedule and is not "prn" or "as needed".
     */
    asNeededFor?: CodeableConcept[] | undefined;
    /**
     * Depending on the resource,this is the amount of medication administered, to  be administered or typical amount to be administered.
     */
    doseAndRate?: DosageDoseAndRate[] | undefined;
    /**
     * This is intended for use as an adjunct to the dosage when there is an upper cap.  For example, a body surface area related dose with a maximum amount, such as 1.5 mg/m2 (maximum 2 mg) IV over 5 – 10 minutes would have doseQuantity of 1.5 mg/m2 and maxDosePerAdministration of 2 mg.
     */
    maxDosePerAdministration?: Quantity | undefined;
    /**
     * Upper limit on medication per lifetime of the patient.
     */
    maxDosePerLifetime?: Quantity | undefined;
    /**
     * This is intended for use as an adjunct to the dosage when there is an upper cap.  For example "2 tablets every 4 hours to a maximum of 8/day".
     */
    maxDosePerPeriod?: Ratio[] | undefined;
    /**
     * Terminologies used often pre-coordinate this term with the route and or form of administration.
     */
    method?: CodeableConcept | undefined;
    /**
     * Instructions in terms that are understood by the patient or consumer.
     */
    patientInstruction?: string | undefined;
    _patientInstruction?: Element | undefined;
    /**
     * How drug should enter body.
     */
    route?: CodeableConcept | undefined;
    /**
     * Indicates the order in which the dosage instructions should be applied or interpreted.
     */
    sequence?: number | undefined;
    /**
     * If the use case requires attributes from the BodySite resource (e.g. to identify and track separately) then use the standard extension [bodySite](extension-bodysite.html).  May be a summary code, or a reference to a very precise definition of the location, or both.
     */
    site?: CodeableConcept | undefined;
    /**
     * Free text dosage instructions e.g. SIG.
     */
    text?: string | undefined;
    _text?: Element | undefined;
    /**
     * This attribute might not always be populated while the Dosage.text is expected to be populated.  If both are populated, then the Dosage.text should reflect the content of the Dosage.timing.
     */
    timing?: Timing | undefined;
}
/**
 * A length of time.
 */
export interface Duration extends Quantity {}
/**
 * Base definition for all elements in a resource.
 */
export interface Element {
    /**
     * There can be no stigma associated with the use of extensions by any application, project, or standard - regardless of the institution or jurisdiction that uses or defines the extensions.  The use of extensions is what allows the FHIR specification to retain a core level of simplicity for everyone.
     */
    extension?: Extension[] | undefined;
    /**
     * Unique id for the element within a resource (for internal references). This may be any string value that does not contain spaces.
     */
    id?: string | undefined;
    _id?: Element | undefined;
}
/**
 * If there is no discriminator, the content is hard to process, so this should be avoided.
 */
export interface ElementDefinitionSlicingDiscriminator extends Element {
    /**
     * A FHIRPath expression, using [the simple subset of FHIRPath](fhirpath.html#simple), that is used to identify the element on which discrimination is based.
     */
    path: string;
    _path?: Element | undefined;
    /**
     * pattern' is deprecated - it works exactly the same as 'value'.
     */
    type: 'value' | 'exists' | 'type' | 'profile' | 'position' | 'pattern';
    _type?: Element | undefined;
}
/**
 * The first element in the sequence, the one that carries the slicing, is the definition that applies to all the slices. This is based on the unconstrained element, but can apply any constraints as appropriate. This may include the common constraints on the children of the element.
 */
export interface ElementDefinitionSlicing extends Element {
    /**
     * If it's really not possible to differentiate them, the design should be re-evaluated to make the content usable.
     */
    description?: string | undefined;
    _description?: Element | undefined;
    /**
     * If there is no discriminator, the content is hard to process, so this should be avoided.
     */
    discriminator?: ElementDefinitionSlicingDiscriminator[] | undefined;
    /**
     * Order should only be required when it is a pressing concern for presentation. Profile authors should consider making the order a feature of the rules about the narrative, not the rules about the data - requiring ordered data makes the profile much less re-usable.
     */
    ordered?: boolean | undefined;
    _ordered?: Element | undefined;
    /**
     * Allowing additional elements makes for a much for flexible template - it's open for use in wider contexts, but also means that the content of the resource is not closed, and applications have to decide how to handle content not described by the profile.
     */
    rules: 'closed' | 'open' | 'openAtEnd';
    _rules?: Element | undefined;
}
/**
 * The base information does not carry any information that could not be determined from the path and related profiles, but making this determination requires both that the related profiles are available, and that the algorithm to determine them be available. For tooling simplicity, the base information must always be populated in element definitions in snap shots, even if it is the same.
 */
export interface ElementDefinitionBase extends Element {
    /**
     * This is provided to code generation, since the serialization representation in JSON differs depending on whether the base element has max > 1. Also, some forms of code generation may differ.
     */
    max: string;
    _max?: Element | undefined;
    /**
     * This is provided for consistency with max, and may affect code generation of mandatory elements of the base resource are generated differently (some reference implementations have done this).
     */
    min: number;
    /**
     * The Path that identifies the base element - this matches the ElementDefinition.path for that element. Across FHIR, there is only one base definition of any element - that is, an element definition on a [StructureDefinition](structuredefinition.html#) without a StructureDefinition.base.
     */
    path: string;
    _path?: Element | undefined;
}
/**
 * The Type of the element can be left blank in a differential constraint, in which case the type is inherited from the resource. Abstract types are not permitted to appear as a type when multiple types are listed.  (I.e. Abstract types cannot be part of a choice).
 */
export interface ElementDefinitionType extends Element {
    /**
     * See [Aggregation Rules](elementdefinition.html#aggregation) for further clarification.
     */
    aggregation?: ('contained' | 'referenced' | 'bundled')[] | undefined;
    _aggregation?: Element[] | undefined;
    /**
     * If the element is a reference to another resource, this element contains "Reference", and the targetProfile element defines what resources can be referenced. The targetProfile may be a reference to the general definition of a resource (e.g. http://hl7.org/fhir/StructureDefinition/Patient).
     */
    code: string;
    _code?: Element | undefined;
    /**
     * It is possible to profile  backbone element (e.g. part of a resource), using the [profile-element](extension-elementdefinition-profile-element.html) extension.
     */
    profile?: string[] | undefined;
    _profile?: Element[] | undefined;
    /**
     * Used when the type is "Reference" or "canonical", and identifies a profile structure or implementation Guide that applies to the target of the reference this element refers to. If any profiles are specified, then the content must conform to at least one of them. The URL can be a local reference - to a contained StructureDefinition, or a reference to another StructureDefinition or Implementation Guide by a canonical URL. When an implementation guide is specified, the target resource SHALL conform to at least one profile defined in the implementation guide.
     */
    targetProfile?: string[] | undefined;
    _targetProfile?: Element[] | undefined;
    /**
     * The base specification never makes a rule as to which form is allowed, but implementation guides may do this. See [Aggregation Rules](elementdefinition.html#aggregation) for further clarification.
     */
    versioning?: ('either' | 'independent' | 'specific') | undefined;
    _versioning?: Element | undefined;
}
/**
 * Examples will most commonly be present for data where it's not implicitly obvious from either the data type or value set what the values might be.  (I.e. Example values for dates or quantities would generally be unnecessary.)  If the example value is fully populated, the publication tool can generate an instance automatically.
 */
export interface ElementDefinitionExample extends Element {
    /**
     * Describes the purpose of this example amoung the set of examples.
     */
    label: string;
    _label?: Element | undefined;
    /**
     * The actual value for the element, which must be one of the types allowed for this element.
     */
    valueBase64Binary?: string | undefined;
    _valueBase64Binary?: Element | undefined;
    /**
     * The actual value for the element, which must be one of the types allowed for this element.
     */
    valueBoolean?: boolean | undefined;
    _valueBoolean?: Element | undefined;
    /**
     * The actual value for the element, which must be one of the types allowed for this element.
     */
    valueCanonical?: string | undefined;
    _valueCanonical?: Element | undefined;
    /**
     * The actual value for the element, which must be one of the types allowed for this element.
     */
    valueCode?: string | undefined;
    _valueCode?: Element | undefined;
    /**
     * The actual value for the element, which must be one of the types allowed for this element.
     */
    valueDate?: string | undefined;
    _valueDate?: Element | undefined;
    /**
     * The actual value for the element, which must be one of the types allowed for this element.
     */
    valueDateTime?: string | undefined;
    _valueDateTime?: Element | undefined;
    /**
     * The actual value for the element, which must be one of the types allowed for this element.
     */
    valueDecimal?: number | undefined;
    /**
     * The actual value for the element, which must be one of the types allowed for this element.
     */
    valueId?: string | undefined;
    _valueId?: Element | undefined;
    /**
     * The actual value for the element, which must be one of the types allowed for this element.
     */
    valueInstant?: string | undefined;
    _valueInstant?: Element | undefined;
    /**
     * The actual value for the element, which must be one of the types allowed for this element.
     */
    valueInteger?: number | undefined;
    /**
     * The actual value for the element, which must be one of the types allowed for this element.
     */
    valueInteger64?: string | undefined;
    _valueInteger64?: Element | undefined;
    /**
     * The actual value for the element, which must be one of the types allowed for this element.
     */
    valueMarkdown?: string | undefined;
    _valueMarkdown?: Element | undefined;
    /**
     * The actual value for the element, which must be one of the types allowed for this element.
     */
    valueOid?: string | undefined;
    _valueOid?: Element | undefined;
    /**
     * The actual value for the element, which must be one of the types allowed for this element.
     */
    valuePositiveInt?: number | undefined;
    /**
     * The actual value for the element, which must be one of the types allowed for this element.
     */
    valueString?: string | undefined;
    _valueString?: Element | undefined;
    /**
     * The actual value for the element, which must be one of the types allowed for this element.
     */
    valueTime?: string | undefined;
    _valueTime?: Element | undefined;
    /**
     * The actual value for the element, which must be one of the types allowed for this element.
     */
    valueUnsignedInt?: number | undefined;
    /**
     * The actual value for the element, which must be one of the types allowed for this element.
     */
    valueUri?: string | undefined;
    _valueUri?: Element | undefined;
    /**
     * The actual value for the element, which must be one of the types allowed for this element.
     */
    valueUrl?: string | undefined;
    _valueUrl?: Element | undefined;
    /**
     * The actual value for the element, which must be one of the types allowed for this element.
     */
    valueUuid?: string | undefined;
    _valueUuid?: Element | undefined;
    /**
     * The actual value for the element, which must be one of the types allowed for this element.
     */
    valueAddress?: Address | undefined;
    /**
     * The actual value for the element, which must be one of the types allowed for this element.
     */
    valueAge?: Age | undefined;
    /**
     * The actual value for the element, which must be one of the types allowed for this element.
     */
    valueAnnotation?: Annotation | undefined;
    /**
     * The actual value for the element, which must be one of the types allowed for this element.
     */
    valueAttachment?: Attachment | undefined;
    /**
     * The actual value for the element, which must be one of the types allowed for this element.
     */
    valueCodeableConcept?: CodeableConcept | undefined;
    /**
     * The actual value for the element, which must be one of the types allowed for this element.
     */
    valueCodeableReference?: CodeableReference | undefined;
    /**
     * The actual value for the element, which must be one of the types allowed for this element.
     */
    valueCoding?: Coding | undefined;
    /**
     * The actual value for the element, which must be one of the types allowed for this element.
     */
    valueContactPoint?: ContactPoint | undefined;
    /**
     * The actual value for the element, which must be one of the types allowed for this element.
     */
    valueCount?: Count | undefined;
    /**
     * The actual value for the element, which must be one of the types allowed for this element.
     */
    valueDistance?: Distance | undefined;
    /**
     * The actual value for the element, which must be one of the types allowed for this element.
     */
    valueDuration?: Duration | undefined;
    /**
     * The actual value for the element, which must be one of the types allowed for this element.
     */
    valueHumanName?: HumanName | undefined;
    /**
     * The actual value for the element, which must be one of the types allowed for this element.
     */
    valueIdentifier?: Identifier | undefined;
    /**
     * The actual value for the element, which must be one of the types allowed for this element.
     */
    valueMoney?: Money | undefined;
    /**
     * The actual value for the element, which must be one of the types allowed for this element.
     */
    valuePeriod?: Period | undefined;
    /**
     * The actual value for the element, which must be one of the types allowed for this element.
     */
    valueQuantity?: Quantity | undefined;
    /**
     * The actual value for the element, which must be one of the types allowed for this element.
     */
    valueRange?: Range | undefined;
    /**
     * The actual value for the element, which must be one of the types allowed for this element.
     */
    valueRatio?: Ratio | undefined;
    /**
     * The actual value for the element, which must be one of the types allowed for this element.
     */
    valueRatioRange?: RatioRange | undefined;
    /**
     * The actual value for the element, which must be one of the types allowed for this element.
     */
    valueReference?: Reference | undefined;
    /**
     * The actual value for the element, which must be one of the types allowed for this element.
     */
    valueSampledData?: SampledData | undefined;
    /**
     * The actual value for the element, which must be one of the types allowed for this element.
     */
    valueSignature?: Signature | undefined;
    /**
     * The actual value for the element, which must be one of the types allowed for this element.
     */
    valueTiming?: Timing | undefined;
    /**
     * The actual value for the element, which must be one of the types allowed for this element.
     */
    valueContactDetail?: ContactDetail | undefined;
    /**
     * The actual value for the element, which must be one of the types allowed for this element.
     */
    valueDataRequirement?: DataRequirement | undefined;
    /**
     * The actual value for the element, which must be one of the types allowed for this element.
     */
    valueExpression?: Expression | undefined;
    /**
     * The actual value for the element, which must be one of the types allowed for this element.
     */
    valueParameterDefinition?: ParameterDefinition | undefined;
    /**
     * The actual value for the element, which must be one of the types allowed for this element.
     */
    valueRelatedArtifact?: RelatedArtifact | undefined;
    /**
     * The actual value for the element, which must be one of the types allowed for this element.
     */
    valueTriggerDefinition?: TriggerDefinition | undefined;
    /**
     * The actual value for the element, which must be one of the types allowed for this element.
     */
    valueUsageContext?: UsageContext | undefined;
    /**
     * The actual value for the element, which must be one of the types allowed for this element.
     */
    valueAvailability?: Availability | undefined;
    /**
     * The actual value for the element, which must be one of the types allowed for this element.
     */
    valueExtendedContactDetail?: ExtendedContactDetail | undefined;
    /**
     * The actual value for the element, which must be one of the types allowed for this element.
     */
    valueDosage?: Dosage | undefined;
    /**
     * The actual value for the element, which must be one of the types allowed for this element.
     */
    valueMeta?: Meta | undefined;
}
/**
 * Constraints should be declared on the "context" element - the lowest element in the hierarchy that is common to all nodes referenced by the constraint.
 */
export interface ElementDefinitionConstraint extends Element {
    /**
     * In the absense of an expression, the expression is likely not enforceable by validators, and might be missed by many systems.
     */
    expression?: string | undefined;
    _expression?: Element | undefined;
    /**
     * Should be expressed in business terms as much as possible.
     */
    human: string;
    _human?: Element | undefined;
    /**
     * Allows identification of which elements have their cardinalities impacted by the constraint.  Will not be referenced for constraints that do not affect cardinality.
     */
    key: string;
    _key?: Element | undefined;
    /**
     * To be used if the reason for the constraint might not be intuitive to all implementers.
     */
    requirements?: string | undefined;
    _requirements?: Element | undefined;
    /**
     * This allows constraints to be asserted as "shall" (error) and "should" (warning).
     */
    severity: 'error' | 'warning';
    _severity?: Element | undefined;
    /**
     * This is used when, e.g. rendering, where it is not useful to present inherited constraints when rendering the snapshot.
     */
    source?: string | undefined;
    _source?: Element | undefined;
    /**
     * This element should only be present in a derived profile where a warning or hint has been determined to be spurious/incorrect. E.g. a warning encouraging the inclusion of an element that the profile explicitly prohibits.
     */
    suppress?: boolean | undefined;
    _suppress?: Element | undefined;
    /**
     * Elements SHALL use "f" as the namespace prefix for the FHIR namespace, and "x" for the xhtml namespace, and SHALL NOT use any other prefixes.     Note: XPath is generally considered not useful because it does not apply to JSON and other formats and because of XSLT implementation issues, and may be removed in the future.
     */
    xpath?: string | undefined;
    _xpath?: Element | undefined;
}
/**
 * For a CodeableConcept, when no codes are allowed - only text, use a binding of strength "required" with a description explaining that no coded values are allowed and what sort of information to put in the "text" element.
 */
export interface ElementDefinitionBinding extends Element {
    /**
     * Describes the intended use of this particular set of codes.
     */
    description?: string | undefined;
    _description?: Element | undefined;
    /**
     * For further discussion, see [Using Terminologies](terminologies.html).
     */
    strength: 'required' | 'extensible' | 'preferred' | 'example';
    _strength?: Element | undefined;
    /**
     * The reference may be version-specific or not (e.g. have a |[version] at the end of the canonical URL).
     */
    valueSet?: string | undefined;
    _valueSet?: Element | undefined;
}
/**
 * Mappings are not necessarily specific enough for safe translation.
 */
export interface ElementDefinitionMapping extends Element {
    /**
     * Comments that provide information about the mapping or its use.
     */
    comment?: string | undefined;
    _comment?: Element | undefined;
    /**
     * An internal reference to the definition of a mapping.
     */
    identity: string;
    _identity?: Element | undefined;
    /**
     * If omitted, then there can be no expectation of computational interpretation of the mapping.
     */
    language?: string | undefined;
    _language?: Element | undefined;
    /**
     * For most mappings, the syntax is undefined.  Syntax will be provided for mappings to the RIM.  Multiple mappings may be possible and may include constraints on other resource elements that identify when a particular mapping applies.
     */
    map: string;
    _map?: Element | undefined;
}
/**
 * Captures constraints on each element within the resource, profile, or extension.
 */
export interface ElementDefinition extends BackboneType {
    /**
     * Identifies additional names by which this element might also be known.
     */
    alias?: string[] | undefined;
    _alias?: Element[] | undefined;
    /**
     * The base information does not carry any information that could not be determined from the path and related profiles, but making this determination requires both that the related profiles are available, and that the algorithm to determine them be available. For tooling simplicity, the base information must always be populated in element definitions in snap shots, even if it is the same.
     */
    base?: ElementDefinitionBase | undefined;
    /**
     * For a CodeableConcept, when no codes are allowed - only text, use a binding of strength "required" with a description explaining that no coded values are allowed and what sort of information to put in the "text" element.
     */
    binding?: ElementDefinitionBinding | undefined;
    /**
     * The concept SHALL be properly aligned with the data element definition and other constraints, as defined in the code system, including relationships, of any code listed here.  Where multiple codes exist in a terminology that could correspond to the data element, the most granular code(s) should be selected, so long as they are not more restrictive than the data element itself. The mappings may be used to provide more or less granular or structured equivalences in the code system.
     */
    code?: Coding[] | undefined;
    /**
     * If it is possible to capture usage rules using constraints, that mechanism should be used in preference to this element.
     */
    comment?: string | undefined;
    _comment?: Element | undefined;
    /**
     * A reference to an invariant that may make additional statements about the cardinality or value in the instance.
     */
    condition?: string[] | undefined;
    _condition?: Element[] | undefined;
    /**
     * Constraints should be declared on the "context" element - the lowest element in the hierarchy that is common to all nodes referenced by the constraint.
     */
    constraint?: ElementDefinitionConstraint[] | undefined;
    /**
     * ContentReferences can only be defined in specializations, not constrained types, and they cannot be changed and always reference the non-constrained definition.
     */
    contentReference?: string | undefined;
    _contentReference?: Element | undefined;
    /**
     * Specifying a default value means that the property can never been unknown - it must always have a value. Further, the default value can never be changed, or changed in constraints on content models. Defining default values creates many difficulties in implementation (e.g. when is a value missing?). For these reasons, default values are (and should be) used extremely sparingly.
     * No default values are ever defined in the FHIR specification, nor can they be defined in constraints ("profiles") on data types or resources. This element only exists so that default values may be defined in logical models.
     */
    defaultValueBase64Binary?: string | undefined;
    _defaultValueBase64Binary?: Element | undefined;
    /**
     * Specifying a default value means that the property can never been unknown - it must always have a value. Further, the default value can never be changed, or changed in constraints on content models. Defining default values creates many difficulties in implementation (e.g. when is a value missing?). For these reasons, default values are (and should be) used extremely sparingly.
     * No default values are ever defined in the FHIR specification, nor can they be defined in constraints ("profiles") on data types or resources. This element only exists so that default values may be defined in logical models.
     */
    defaultValueBoolean?: boolean | undefined;
    _defaultValueBoolean?: Element | undefined;
    /**
     * Specifying a default value means that the property can never been unknown - it must always have a value. Further, the default value can never be changed, or changed in constraints on content models. Defining default values creates many difficulties in implementation (e.g. when is a value missing?). For these reasons, default values are (and should be) used extremely sparingly.
     * No default values are ever defined in the FHIR specification, nor can they be defined in constraints ("profiles") on data types or resources. This element only exists so that default values may be defined in logical models.
     */
    defaultValueCanonical?: string | undefined;
    _defaultValueCanonical?: Element | undefined;
    /**
     * Specifying a default value means that the property can never been unknown - it must always have a value. Further, the default value can never be changed, or changed in constraints on content models. Defining default values creates many difficulties in implementation (e.g. when is a value missing?). For these reasons, default values are (and should be) used extremely sparingly.
     * No default values are ever defined in the FHIR specification, nor can they be defined in constraints ("profiles") on data types or resources. This element only exists so that default values may be defined in logical models.
     */
    defaultValueCode?: string | undefined;
    _defaultValueCode?: Element | undefined;
    /**
     * Specifying a default value means that the property can never been unknown - it must always have a value. Further, the default value can never be changed, or changed in constraints on content models. Defining default values creates many difficulties in implementation (e.g. when is a value missing?). For these reasons, default values are (and should be) used extremely sparingly.
     * No default values are ever defined in the FHIR specification, nor can they be defined in constraints ("profiles") on data types or resources. This element only exists so that default values may be defined in logical models.
     */
    defaultValueDate?: string | undefined;
    _defaultValueDate?: Element | undefined;
    /**
     * Specifying a default value means that the property can never been unknown - it must always have a value. Further, the default value can never be changed, or changed in constraints on content models. Defining default values creates many difficulties in implementation (e.g. when is a value missing?). For these reasons, default values are (and should be) used extremely sparingly.
     * No default values are ever defined in the FHIR specification, nor can they be defined in constraints ("profiles") on data types or resources. This element only exists so that default values may be defined in logical models.
     */
    defaultValueDateTime?: string | undefined;
    _defaultValueDateTime?: Element | undefined;
    /**
     * Specifying a default value means that the property can never been unknown - it must always have a value. Further, the default value can never be changed, or changed in constraints on content models. Defining default values creates many difficulties in implementation (e.g. when is a value missing?). For these reasons, default values are (and should be) used extremely sparingly.
     * No default values are ever defined in the FHIR specification, nor can they be defined in constraints ("profiles") on data types or resources. This element only exists so that default values may be defined in logical models.
     */
    defaultValueDecimal?: number | undefined;
    /**
     * Specifying a default value means that the property can never been unknown - it must always have a value. Further, the default value can never be changed, or changed in constraints on content models. Defining default values creates many difficulties in implementation (e.g. when is a value missing?). For these reasons, default values are (and should be) used extremely sparingly.
     * No default values are ever defined in the FHIR specification, nor can they be defined in constraints ("profiles") on data types or resources. This element only exists so that default values may be defined in logical models.
     */
    defaultValueId?: string | undefined;
    _defaultValueId?: Element | undefined;
    /**
     * Specifying a default value means that the property can never been unknown - it must always have a value. Further, the default value can never be changed, or changed in constraints on content models. Defining default values creates many difficulties in implementation (e.g. when is a value missing?). For these reasons, default values are (and should be) used extremely sparingly.
     * No default values are ever defined in the FHIR specification, nor can they be defined in constraints ("profiles") on data types or resources. This element only exists so that default values may be defined in logical models.
     */
    defaultValueInstant?: string | undefined;
    _defaultValueInstant?: Element | undefined;
    /**
     * Specifying a default value means that the property can never been unknown - it must always have a value. Further, the default value can never be changed, or changed in constraints on content models. Defining default values creates many difficulties in implementation (e.g. when is a value missing?). For these reasons, default values are (and should be) used extremely sparingly.
     * No default values are ever defined in the FHIR specification, nor can they be defined in constraints ("profiles") on data types or resources. This element only exists so that default values may be defined in logical models.
     */
    defaultValueInteger?: number | undefined;
    /**
     * Specifying a default value means that the property can never been unknown - it must always have a value. Further, the default value can never be changed, or changed in constraints on content models. Defining default values creates many difficulties in implementation (e.g. when is a value missing?). For these reasons, default values are (and should be) used extremely sparingly.
     * No default values are ever defined in the FHIR specification, nor can they be defined in constraints ("profiles") on data types or resources. This element only exists so that default values may be defined in logical models.
     */
    defaultValueInteger64?: string | undefined;
    _defaultValueInteger64?: Element | undefined;
    /**
     * Specifying a default value means that the property can never been unknown - it must always have a value. Further, the default value can never be changed, or changed in constraints on content models. Defining default values creates many difficulties in implementation (e.g. when is a value missing?). For these reasons, default values are (and should be) used extremely sparingly.
     * No default values are ever defined in the FHIR specification, nor can they be defined in constraints ("profiles") on data types or resources. This element only exists so that default values may be defined in logical models.
     */
    defaultValueMarkdown?: string | undefined;
    _defaultValueMarkdown?: Element | undefined;
    /**
     * Specifying a default value means that the property can never been unknown - it must always have a value. Further, the default value can never be changed, or changed in constraints on content models. Defining default values creates many difficulties in implementation (e.g. when is a value missing?). For these reasons, default values are (and should be) used extremely sparingly.
     * No default values are ever defined in the FHIR specification, nor can they be defined in constraints ("profiles") on data types or resources. This element only exists so that default values may be defined in logical models.
     */
    defaultValueOid?: string | undefined;
    _defaultValueOid?: Element | undefined;
    /**
     * Specifying a default value means that the property can never been unknown - it must always have a value. Further, the default value can never be changed, or changed in constraints on content models. Defining default values creates many difficulties in implementation (e.g. when is a value missing?). For these reasons, default values are (and should be) used extremely sparingly.
     * No default values are ever defined in the FHIR specification, nor can they be defined in constraints ("profiles") on data types or resources. This element only exists so that default values may be defined in logical models.
     */
    defaultValuePositiveInt?: number | undefined;
    /**
     * Specifying a default value means that the property can never been unknown - it must always have a value. Further, the default value can never be changed, or changed in constraints on content models. Defining default values creates many difficulties in implementation (e.g. when is a value missing?). For these reasons, default values are (and should be) used extremely sparingly.
     * No default values are ever defined in the FHIR specification, nor can they be defined in constraints ("profiles") on data types or resources. This element only exists so that default values may be defined in logical models.
     */
    defaultValueString?: string | undefined;
    _defaultValueString?: Element | undefined;
    /**
     * Specifying a default value means that the property can never been unknown - it must always have a value. Further, the default value can never be changed, or changed in constraints on content models. Defining default values creates many difficulties in implementation (e.g. when is a value missing?). For these reasons, default values are (and should be) used extremely sparingly.
     * No default values are ever defined in the FHIR specification, nor can they be defined in constraints ("profiles") on data types or resources. This element only exists so that default values may be defined in logical models.
     */
    defaultValueTime?: string | undefined;
    _defaultValueTime?: Element | undefined;
    /**
     * Specifying a default value means that the property can never been unknown - it must always have a value. Further, the default value can never be changed, or changed in constraints on content models. Defining default values creates many difficulties in implementation (e.g. when is a value missing?). For these reasons, default values are (and should be) used extremely sparingly.
     * No default values are ever defined in the FHIR specification, nor can they be defined in constraints ("profiles") on data types or resources. This element only exists so that default values may be defined in logical models.
     */
    defaultValueUnsignedInt?: number | undefined;
    /**
     * Specifying a default value means that the property can never been unknown - it must always have a value. Further, the default value can never be changed, or changed in constraints on content models. Defining default values creates many difficulties in implementation (e.g. when is a value missing?). For these reasons, default values are (and should be) used extremely sparingly.
     * No default values are ever defined in the FHIR specification, nor can they be defined in constraints ("profiles") on data types or resources. This element only exists so that default values may be defined in logical models.
     */
    defaultValueUri?: string | undefined;
    _defaultValueUri?: Element | undefined;
    /**
     * Specifying a default value means that the property can never been unknown - it must always have a value. Further, the default value can never be changed, or changed in constraints on content models. Defining default values creates many difficulties in implementation (e.g. when is a value missing?). For these reasons, default values are (and should be) used extremely sparingly.
     * No default values are ever defined in the FHIR specification, nor can they be defined in constraints ("profiles") on data types or resources. This element only exists so that default values may be defined in logical models.
     */
    defaultValueUrl?: string | undefined;
    _defaultValueUrl?: Element | undefined;
    /**
     * Specifying a default value means that the property can never been unknown - it must always have a value. Further, the default value can never be changed, or changed in constraints on content models. Defining default values creates many difficulties in implementation (e.g. when is a value missing?). For these reasons, default values are (and should be) used extremely sparingly.
     * No default values are ever defined in the FHIR specification, nor can they be defined in constraints ("profiles") on data types or resources. This element only exists so that default values may be defined in logical models.
     */
    defaultValueUuid?: string | undefined;
    _defaultValueUuid?: Element | undefined;
    /**
     * Specifying a default value means that the property can never been unknown - it must always have a value. Further, the default value can never be changed, or changed in constraints on content models. Defining default values creates many difficulties in implementation (e.g. when is a value missing?). For these reasons, default values are (and should be) used extremely sparingly.
     * No default values are ever defined in the FHIR specification, nor can they be defined in constraints ("profiles") on data types or resources. This element only exists so that default values may be defined in logical models.
     */
    defaultValueAddress?: Address | undefined;
    /**
     * Specifying a default value means that the property can never been unknown - it must always have a value. Further, the default value can never be changed, or changed in constraints on content models. Defining default values creates many difficulties in implementation (e.g. when is a value missing?). For these reasons, default values are (and should be) used extremely sparingly.
     * No default values are ever defined in the FHIR specification, nor can they be defined in constraints ("profiles") on data types or resources. This element only exists so that default values may be defined in logical models.
     */
    defaultValueAge?: Age | undefined;
    /**
     * Specifying a default value means that the property can never been unknown - it must always have a value. Further, the default value can never be changed, or changed in constraints on content models. Defining default values creates many difficulties in implementation (e.g. when is a value missing?). For these reasons, default values are (and should be) used extremely sparingly.
     * No default values are ever defined in the FHIR specification, nor can they be defined in constraints ("profiles") on data types or resources. This element only exists so that default values may be defined in logical models.
     */
    defaultValueAnnotation?: Annotation | undefined;
    /**
     * Specifying a default value means that the property can never been unknown - it must always have a value. Further, the default value can never be changed, or changed in constraints on content models. Defining default values creates many difficulties in implementation (e.g. when is a value missing?). For these reasons, default values are (and should be) used extremely sparingly.
     * No default values are ever defined in the FHIR specification, nor can they be defined in constraints ("profiles") on data types or resources. This element only exists so that default values may be defined in logical models.
     */
    defaultValueAttachment?: Attachment | undefined;
    /**
     * Specifying a default value means that the property can never been unknown - it must always have a value. Further, the default value can never be changed, or changed in constraints on content models. Defining default values creates many difficulties in implementation (e.g. when is a value missing?). For these reasons, default values are (and should be) used extremely sparingly.
     * No default values are ever defined in the FHIR specification, nor can they be defined in constraints ("profiles") on data types or resources. This element only exists so that default values may be defined in logical models.
     */
    defaultValueCodeableConcept?: CodeableConcept | undefined;
    /**
     * Specifying a default value means that the property can never been unknown - it must always have a value. Further, the default value can never be changed, or changed in constraints on content models. Defining default values creates many difficulties in implementation (e.g. when is a value missing?). For these reasons, default values are (and should be) used extremely sparingly.
     * No default values are ever defined in the FHIR specification, nor can they be defined in constraints ("profiles") on data types or resources. This element only exists so that default values may be defined in logical models.
     */
    defaultValueCodeableReference?: CodeableReference | undefined;
    /**
     * Specifying a default value means that the property can never been unknown - it must always have a value. Further, the default value can never be changed, or changed in constraints on content models. Defining default values creates many difficulties in implementation (e.g. when is a value missing?). For these reasons, default values are (and should be) used extremely sparingly.
     * No default values are ever defined in the FHIR specification, nor can they be defined in constraints ("profiles") on data types or resources. This element only exists so that default values may be defined in logical models.
     */
    defaultValueCoding?: Coding | undefined;
    /**
     * Specifying a default value means that the property can never been unknown - it must always have a value. Further, the default value can never be changed, or changed in constraints on content models. Defining default values creates many difficulties in implementation (e.g. when is a value missing?). For these reasons, default values are (and should be) used extremely sparingly.
     * No default values are ever defined in the FHIR specification, nor can they be defined in constraints ("profiles") on data types or resources. This element only exists so that default values may be defined in logical models.
     */
    defaultValueContactPoint?: ContactPoint | undefined;
    /**
     * Specifying a default value means that the property can never been unknown - it must always have a value. Further, the default value can never be changed, or changed in constraints on content models. Defining default values creates many difficulties in implementation (e.g. when is a value missing?). For these reasons, default values are (and should be) used extremely sparingly.
     * No default values are ever defined in the FHIR specification, nor can they be defined in constraints ("profiles") on data types or resources. This element only exists so that default values may be defined in logical models.
     */
    defaultValueCount?: Count | undefined;
    /**
     * Specifying a default value means that the property can never been unknown - it must always have a value. Further, the default value can never be changed, or changed in constraints on content models. Defining default values creates many difficulties in implementation (e.g. when is a value missing?). For these reasons, default values are (and should be) used extremely sparingly.
     * No default values are ever defined in the FHIR specification, nor can they be defined in constraints ("profiles") on data types or resources. This element only exists so that default values may be defined in logical models.
     */
    defaultValueDistance?: Distance | undefined;
    /**
     * Specifying a default value means that the property can never been unknown - it must always have a value. Further, the default value can never be changed, or changed in constraints on content models. Defining default values creates many difficulties in implementation (e.g. when is a value missing?). For these reasons, default values are (and should be) used extremely sparingly.
     * No default values are ever defined in the FHIR specification, nor can they be defined in constraints ("profiles") on data types or resources. This element only exists so that default values may be defined in logical models.
     */
    defaultValueDuration?: Duration | undefined;
    /**
     * Specifying a default value means that the property can never been unknown - it must always have a value. Further, the default value can never be changed, or changed in constraints on content models. Defining default values creates many difficulties in implementation (e.g. when is a value missing?). For these reasons, default values are (and should be) used extremely sparingly.
     * No default values are ever defined in the FHIR specification, nor can they be defined in constraints ("profiles") on data types or resources. This element only exists so that default values may be defined in logical models.
     */
    defaultValueHumanName?: HumanName | undefined;
    /**
     * Specifying a default value means that the property can never been unknown - it must always have a value. Further, the default value can never be changed, or changed in constraints on content models. Defining default values creates many difficulties in implementation (e.g. when is a value missing?). For these reasons, default values are (and should be) used extremely sparingly.
     * No default values are ever defined in the FHIR specification, nor can they be defined in constraints ("profiles") on data types or resources. This element only exists so that default values may be defined in logical models.
     */
    defaultValueIdentifier?: Identifier | undefined;
    /**
     * Specifying a default value means that the property can never been unknown - it must always have a value. Further, the default value can never be changed, or changed in constraints on content models. Defining default values creates many difficulties in implementation (e.g. when is a value missing?). For these reasons, default values are (and should be) used extremely sparingly.
     * No default values are ever defined in the FHIR specification, nor can they be defined in constraints ("profiles") on data types or resources. This element only exists so that default values may be defined in logical models.
     */
    defaultValueMoney?: Money | undefined;
    /**
     * Specifying a default value means that the property can never been unknown - it must always have a value. Further, the default value can never be changed, or changed in constraints on content models. Defining default values creates many difficulties in implementation (e.g. when is a value missing?). For these reasons, default values are (and should be) used extremely sparingly.
     * No default values are ever defined in the FHIR specification, nor can they be defined in constraints ("profiles") on data types or resources. This element only exists so that default values may be defined in logical models.
     */
    defaultValuePeriod?: Period | undefined;
    /**
     * Specifying a default value means that the property can never been unknown - it must always have a value. Further, the default value can never be changed, or changed in constraints on content models. Defining default values creates many difficulties in implementation (e.g. when is a value missing?). For these reasons, default values are (and should be) used extremely sparingly.
     * No default values are ever defined in the FHIR specification, nor can they be defined in constraints ("profiles") on data types or resources. This element only exists so that default values may be defined in logical models.
     */
    defaultValueQuantity?: Quantity | undefined;
    /**
     * Specifying a default value means that the property can never been unknown - it must always have a value. Further, the default value can never be changed, or changed in constraints on content models. Defining default values creates many difficulties in implementation (e.g. when is a value missing?). For these reasons, default values are (and should be) used extremely sparingly.
     * No default values are ever defined in the FHIR specification, nor can they be defined in constraints ("profiles") on data types or resources. This element only exists so that default values may be defined in logical models.
     */
    defaultValueRange?: Range | undefined;
    /**
     * Specifying a default value means that the property can never been unknown - it must always have a value. Further, the default value can never be changed, or changed in constraints on content models. Defining default values creates many difficulties in implementation (e.g. when is a value missing?). For these reasons, default values are (and should be) used extremely sparingly.
     * No default values are ever defined in the FHIR specification, nor can they be defined in constraints ("profiles") on data types or resources. This element only exists so that default values may be defined in logical models.
     */
    defaultValueRatio?: Ratio | undefined;
    /**
     * Specifying a default value means that the property can never been unknown - it must always have a value. Further, the default value can never be changed, or changed in constraints on content models. Defining default values creates many difficulties in implementation (e.g. when is a value missing?). For these reasons, default values are (and should be) used extremely sparingly.
     * No default values are ever defined in the FHIR specification, nor can they be defined in constraints ("profiles") on data types or resources. This element only exists so that default values may be defined in logical models.
     */
    defaultValueRatioRange?: RatioRange | undefined;
    /**
     * Specifying a default value means that the property can never been unknown - it must always have a value. Further, the default value can never be changed, or changed in constraints on content models. Defining default values creates many difficulties in implementation (e.g. when is a value missing?). For these reasons, default values are (and should be) used extremely sparingly.
     * No default values are ever defined in the FHIR specification, nor can they be defined in constraints ("profiles") on data types or resources. This element only exists so that default values may be defined in logical models.
     */
    defaultValueReference?: Reference | undefined;
    /**
     * Specifying a default value means that the property can never been unknown - it must always have a value. Further, the default value can never be changed, or changed in constraints on content models. Defining default values creates many difficulties in implementation (e.g. when is a value missing?). For these reasons, default values are (and should be) used extremely sparingly.
     * No default values are ever defined in the FHIR specification, nor can they be defined in constraints ("profiles") on data types or resources. This element only exists so that default values may be defined in logical models.
     */
    defaultValueSampledData?: SampledData | undefined;
    /**
     * Specifying a default value means that the property can never been unknown - it must always have a value. Further, the default value can never be changed, or changed in constraints on content models. Defining default values creates many difficulties in implementation (e.g. when is a value missing?). For these reasons, default values are (and should be) used extremely sparingly.
     * No default values are ever defined in the FHIR specification, nor can they be defined in constraints ("profiles") on data types or resources. This element only exists so that default values may be defined in logical models.
     */
    defaultValueSignature?: Signature | undefined;
    /**
     * Specifying a default value means that the property can never been unknown - it must always have a value. Further, the default value can never be changed, or changed in constraints on content models. Defining default values creates many difficulties in implementation (e.g. when is a value missing?). For these reasons, default values are (and should be) used extremely sparingly.
     * No default values are ever defined in the FHIR specification, nor can they be defined in constraints ("profiles") on data types or resources. This element only exists so that default values may be defined in logical models.
     */
    defaultValueTiming?: Timing | undefined;
    /**
     * Specifying a default value means that the property can never been unknown - it must always have a value. Further, the default value can never be changed, or changed in constraints on content models. Defining default values creates many difficulties in implementation (e.g. when is a value missing?). For these reasons, default values are (and should be) used extremely sparingly.
     * No default values are ever defined in the FHIR specification, nor can they be defined in constraints ("profiles") on data types or resources. This element only exists so that default values may be defined in logical models.
     */
    defaultValueContactDetail?: ContactDetail | undefined;
    /**
     * Specifying a default value means that the property can never been unknown - it must always have a value. Further, the default value can never be changed, or changed in constraints on content models. Defining default values creates many difficulties in implementation (e.g. when is a value missing?). For these reasons, default values are (and should be) used extremely sparingly.
     * No default values are ever defined in the FHIR specification, nor can they be defined in constraints ("profiles") on data types or resources. This element only exists so that default values may be defined in logical models.
     */
    defaultValueDataRequirement?: DataRequirement | undefined;
    /**
     * Specifying a default value means that the property can never been unknown - it must always have a value. Further, the default value can never be changed, or changed in constraints on content models. Defining default values creates many difficulties in implementation (e.g. when is a value missing?). For these reasons, default values are (and should be) used extremely sparingly.
     * No default values are ever defined in the FHIR specification, nor can they be defined in constraints ("profiles") on data types or resources. This element only exists so that default values may be defined in logical models.
     */
    defaultValueExpression?: Expression | undefined;
    /**
     * Specifying a default value means that the property can never been unknown - it must always have a value. Further, the default value can never be changed, or changed in constraints on content models. Defining default values creates many difficulties in implementation (e.g. when is a value missing?). For these reasons, default values are (and should be) used extremely sparingly.
     * No default values are ever defined in the FHIR specification, nor can they be defined in constraints ("profiles") on data types or resources. This element only exists so that default values may be defined in logical models.
     */
    defaultValueParameterDefinition?: ParameterDefinition | undefined;
    /**
     * Specifying a default value means that the property can never been unknown - it must always have a value. Further, the default value can never be changed, or changed in constraints on content models. Defining default values creates many difficulties in implementation (e.g. when is a value missing?). For these reasons, default values are (and should be) used extremely sparingly.
     * No default values are ever defined in the FHIR specification, nor can they be defined in constraints ("profiles") on data types or resources. This element only exists so that default values may be defined in logical models.
     */
    defaultValueRelatedArtifact?: RelatedArtifact | undefined;
    /**
     * Specifying a default value means that the property can never been unknown - it must always have a value. Further, the default value can never be changed, or changed in constraints on content models. Defining default values creates many difficulties in implementation (e.g. when is a value missing?). For these reasons, default values are (and should be) used extremely sparingly.
     * No default values are ever defined in the FHIR specification, nor can they be defined in constraints ("profiles") on data types or resources. This element only exists so that default values may be defined in logical models.
     */
    defaultValueTriggerDefinition?: TriggerDefinition | undefined;
    /**
     * Specifying a default value means that the property can never been unknown - it must always have a value. Further, the default value can never be changed, or changed in constraints on content models. Defining default values creates many difficulties in implementation (e.g. when is a value missing?). For these reasons, default values are (and should be) used extremely sparingly.
     * No default values are ever defined in the FHIR specification, nor can they be defined in constraints ("profiles") on data types or resources. This element only exists so that default values may be defined in logical models.
     */
    defaultValueUsageContext?: UsageContext | undefined;
    /**
     * Specifying a default value means that the property can never been unknown - it must always have a value. Further, the default value can never be changed, or changed in constraints on content models. Defining default values creates many difficulties in implementation (e.g. when is a value missing?). For these reasons, default values are (and should be) used extremely sparingly.
     * No default values are ever defined in the FHIR specification, nor can they be defined in constraints ("profiles") on data types or resources. This element only exists so that default values may be defined in logical models.
     */
    defaultValueAvailability?: Availability | undefined;
    /**
     * Specifying a default value means that the property can never been unknown - it must always have a value. Further, the default value can never be changed, or changed in constraints on content models. Defining default values creates many difficulties in implementation (e.g. when is a value missing?). For these reasons, default values are (and should be) used extremely sparingly.
     * No default values are ever defined in the FHIR specification, nor can they be defined in constraints ("profiles") on data types or resources. This element only exists so that default values may be defined in logical models.
     */
    defaultValueExtendedContactDetail?: ExtendedContactDetail | undefined;
    /**
     * Specifying a default value means that the property can never been unknown - it must always have a value. Further, the default value can never be changed, or changed in constraints on content models. Defining default values creates many difficulties in implementation (e.g. when is a value missing?). For these reasons, default values are (and should be) used extremely sparingly.
     * No default values are ever defined in the FHIR specification, nor can they be defined in constraints ("profiles") on data types or resources. This element only exists so that default values may be defined in logical models.
     */
    defaultValueDosage?: Dosage | undefined;
    /**
     * Specifying a default value means that the property can never been unknown - it must always have a value. Further, the default value can never be changed, or changed in constraints on content models. Defining default values creates many difficulties in implementation (e.g. when is a value missing?). For these reasons, default values are (and should be) used extremely sparingly.
     * No default values are ever defined in the FHIR specification, nor can they be defined in constraints ("profiles") on data types or resources. This element only exists so that default values may be defined in logical models.
     */
    defaultValueMeta?: Meta | undefined;
    /**
     * It is easy for a different definition to change the meaning of an element and this can have nasty downstream consequences. Please be careful when providing definitions in a profile.
     */
    definition?: string | undefined;
    _definition?: Element | undefined;
    /**
     * Examples will most commonly be present for data where it's not implicitly obvious from either the data type or value set what the values might be.  (I.e. Example values for dates or quantities would generally be unnecessary.)  If the example value is fully populated, the publication tool can generate an instance automatically.
     */
    example?: ElementDefinitionExample[] | undefined;
    /**
     * This is not recommended for Coding and CodeableConcept since these often have highly contextual properties such as version or display.
     */
    fixedBase64Binary?: string | undefined;
    _fixedBase64Binary?: Element | undefined;
    /**
     * This is not recommended for Coding and CodeableConcept since these often have highly contextual properties such as version or display.
     */
    fixedBoolean?: boolean | undefined;
    _fixedBoolean?: Element | undefined;
    /**
     * This is not recommended for Coding and CodeableConcept since these often have highly contextual properties such as version or display.
     */
    fixedCanonical?: string | undefined;
    _fixedCanonical?: Element | undefined;
    /**
     * This is not recommended for Coding and CodeableConcept since these often have highly contextual properties such as version or display.
     */
    fixedCode?: string | undefined;
    _fixedCode?: Element | undefined;
    /**
     * This is not recommended for Coding and CodeableConcept since these often have highly contextual properties such as version or display.
     */
    fixedDate?: string | undefined;
    _fixedDate?: Element | undefined;
    /**
     * This is not recommended for Coding and CodeableConcept since these often have highly contextual properties such as version or display.
     */
    fixedDateTime?: string | undefined;
    _fixedDateTime?: Element | undefined;
    /**
     * This is not recommended for Coding and CodeableConcept since these often have highly contextual properties such as version or display.
     */
    fixedDecimal?: number | undefined;
    /**
     * This is not recommended for Coding and CodeableConcept since these often have highly contextual properties such as version or display.
     */
    fixedId?: string | undefined;
    _fixedId?: Element | undefined;
    /**
     * This is not recommended for Coding and CodeableConcept since these often have highly contextual properties such as version or display.
     */
    fixedInstant?: string | undefined;
    _fixedInstant?: Element | undefined;
    /**
     * This is not recommended for Coding and CodeableConcept since these often have highly contextual properties such as version or display.
     */
    fixedInteger?: number | undefined;
    /**
     * This is not recommended for Coding and CodeableConcept since these often have highly contextual properties such as version or display.
     */
    fixedInteger64?: string | undefined;
    _fixedInteger64?: Element | undefined;
    /**
     * This is not recommended for Coding and CodeableConcept since these often have highly contextual properties such as version or display.
     */
    fixedMarkdown?: string | undefined;
    _fixedMarkdown?: Element | undefined;
    /**
     * This is not recommended for Coding and CodeableConcept since these often have highly contextual properties such as version or display.
     */
    fixedOid?: string | undefined;
    _fixedOid?: Element | undefined;
    /**
     * This is not recommended for Coding and CodeableConcept since these often have highly contextual properties such as version or display.
     */
    fixedPositiveInt?: number | undefined;
    /**
     * This is not recommended for Coding and CodeableConcept since these often have highly contextual properties such as version or display.
     */
    fixedString?: string | undefined;
    _fixedString?: Element | undefined;
    /**
     * This is not recommended for Coding and CodeableConcept since these often have highly contextual properties such as version or display.
     */
    fixedTime?: string | undefined;
    _fixedTime?: Element | undefined;
    /**
     * This is not recommended for Coding and CodeableConcept since these often have highly contextual properties such as version or display.
     */
    fixedUnsignedInt?: number | undefined;
    /**
     * This is not recommended for Coding and CodeableConcept since these often have highly contextual properties such as version or display.
     */
    fixedUri?: string | undefined;
    _fixedUri?: Element | undefined;
    /**
     * This is not recommended for Coding and CodeableConcept since these often have highly contextual properties such as version or display.
     */
    fixedUrl?: string | undefined;
    _fixedUrl?: Element | undefined;
    /**
     * This is not recommended for Coding and CodeableConcept since these often have highly contextual properties such as version or display.
     */
    fixedUuid?: string | undefined;
    _fixedUuid?: Element | undefined;
    /**
     * This is not recommended for Coding and CodeableConcept since these often have highly contextual properties such as version or display.
     */
    fixedAddress?: Address | undefined;
    /**
     * This is not recommended for Coding and CodeableConcept since these often have highly contextual properties such as version or display.
     */
    fixedAge?: Age | undefined;
    /**
     * This is not recommended for Coding and CodeableConcept since these often have highly contextual properties such as version or display.
     */
    fixedAnnotation?: Annotation | undefined;
    /**
     * This is not recommended for Coding and CodeableConcept since these often have highly contextual properties such as version or display.
     */
    fixedAttachment?: Attachment | undefined;
    /**
     * This is not recommended for Coding and CodeableConcept since these often have highly contextual properties such as version or display.
     */
    fixedCodeableConcept?: CodeableConcept | undefined;
    /**
     * This is not recommended for Coding and CodeableConcept since these often have highly contextual properties such as version or display.
     */
    fixedCodeableReference?: CodeableReference | undefined;
    /**
     * This is not recommended for Coding and CodeableConcept since these often have highly contextual properties such as version or display.
     */
    fixedCoding?: Coding | undefined;
    /**
     * This is not recommended for Coding and CodeableConcept since these often have highly contextual properties such as version or display.
     */
    fixedContactPoint?: ContactPoint | undefined;
    /**
     * This is not recommended for Coding and CodeableConcept since these often have highly contextual properties such as version or display.
     */
    fixedCount?: Count | undefined;
    /**
     * This is not recommended for Coding and CodeableConcept since these often have highly contextual properties such as version or display.
     */
    fixedDistance?: Distance | undefined;
    /**
     * This is not recommended for Coding and CodeableConcept since these often have highly contextual properties such as version or display.
     */
    fixedDuration?: Duration | undefined;
    /**
     * This is not recommended for Coding and CodeableConcept since these often have highly contextual properties such as version or display.
     */
    fixedHumanName?: HumanName | undefined;
    /**
     * This is not recommended for Coding and CodeableConcept since these often have highly contextual properties such as version or display.
     */
    fixedIdentifier?: Identifier | undefined;
    /**
     * This is not recommended for Coding and CodeableConcept since these often have highly contextual properties such as version or display.
     */
    fixedMoney?: Money | undefined;
    /**
     * This is not recommended for Coding and CodeableConcept since these often have highly contextual properties such as version or display.
     */
    fixedPeriod?: Period | undefined;
    /**
     * This is not recommended for Coding and CodeableConcept since these often have highly contextual properties such as version or display.
     */
    fixedQuantity?: Quantity | undefined;
    /**
     * This is not recommended for Coding and CodeableConcept since these often have highly contextual properties such as version or display.
     */
    fixedRange?: Range | undefined;
    /**
     * This is not recommended for Coding and CodeableConcept since these often have highly contextual properties such as version or display.
     */
    fixedRatio?: Ratio | undefined;
    /**
     * This is not recommended for Coding and CodeableConcept since these often have highly contextual properties such as version or display.
     */
    fixedRatioRange?: RatioRange | undefined;
    /**
     * This is not recommended for Coding and CodeableConcept since these often have highly contextual properties such as version or display.
     */
    fixedReference?: Reference | undefined;
    /**
     * This is not recommended for Coding and CodeableConcept since these often have highly contextual properties such as version or display.
     */
    fixedSampledData?: SampledData | undefined;
    /**
     * This is not recommended for Coding and CodeableConcept since these often have highly contextual properties such as version or display.
     */
    fixedSignature?: Signature | undefined;
    /**
     * This is not recommended for Coding and CodeableConcept since these often have highly contextual properties such as version or display.
     */
    fixedTiming?: Timing | undefined;
    /**
     * This is not recommended for Coding and CodeableConcept since these often have highly contextual properties such as version or display.
     */
    fixedContactDetail?: ContactDetail | undefined;
    /**
     * This is not recommended for Coding and CodeableConcept since these often have highly contextual properties such as version or display.
     */
    fixedDataRequirement?: DataRequirement | undefined;
    /**
     * This is not recommended for Coding and CodeableConcept since these often have highly contextual properties such as version or display.
     */
    fixedExpression?: Expression | undefined;
    /**
     * This is not recommended for Coding and CodeableConcept since these often have highly contextual properties such as version or display.
     */
    fixedParameterDefinition?: ParameterDefinition | undefined;
    /**
     * This is not recommended for Coding and CodeableConcept since these often have highly contextual properties such as version or display.
     */
    fixedRelatedArtifact?: RelatedArtifact | undefined;
    /**
     * This is not recommended for Coding and CodeableConcept since these often have highly contextual properties such as version or display.
     */
    fixedTriggerDefinition?: TriggerDefinition | undefined;
    /**
     * This is not recommended for Coding and CodeableConcept since these often have highly contextual properties such as version or display.
     */
    fixedUsageContext?: UsageContext | undefined;
    /**
     * This is not recommended for Coding and CodeableConcept since these often have highly contextual properties such as version or display.
     */
    fixedAvailability?: Availability | undefined;
    /**
     * This is not recommended for Coding and CodeableConcept since these often have highly contextual properties such as version or display.
     */
    fixedExtendedContactDetail?: ExtendedContactDetail | undefined;
    /**
     * This is not recommended for Coding and CodeableConcept since these often have highly contextual properties such as version or display.
     */
    fixedDosage?: Dosage | undefined;
    /**
     * This is not recommended for Coding and CodeableConcept since these often have highly contextual properties such as version or display.
     */
    fixedMeta?: Meta | undefined;
    /**
     * Only the definition of an element can set IsModifier true - either the specification itself or where an extension is originally defined. Once set, it cannot be changed in derived profiles. An element/extension that has isModifier=true SHOULD also have a minimum cardinality of 1, so that there is no lack of clarity about what to do if it is missing. If it can be missing, the definition SHALL make the meaning of a missing element clear.
     */
    isModifier?: boolean | undefined;
    _isModifier?: Element | undefined;
    /**
     * Explains how that element affects the interpretation of the resource or element that contains it.
     */
    isModifierReason?: string | undefined;
    _isModifierReason?: Element | undefined;
    /**
     * Some resources include a set of simple metadata, and some very large data. This element is used to reduce the quantity of data returned in searches. Note that servers may pre-cache summarized resources for optimal performance. When a request is made with _summary=true, serializers only include elements marked as 'isSummary = true'. With a few exceptions (listed below), all datatype properties are included in the summary form. In resource and datatype definitions, if an element has a descendant marked as isSummary=true or if it satisfies both of the following conditions, it must be marked as isSummary=true:
     * * is at the root or has a parent that is 'mustSupport'
     * * has a minimum cardinality of 1 or is a modifier element
     * The following datatype properties are exceptions, and are not marked as isSummary:
     * * Attachment.data
     * * Signature.data, Signature.targetFormat, Signature.sigFormat
     * * Narrative.status, Narrative.div
     * * SampledData.data.
     */
    isSummary?: boolean | undefined;
    _isSummary?: Element | undefined;
    /**
     * See also the extension (http://hl7.org/fhir/StructureDefinition/elementdefinition-question)[extension-elementdefinition-question.html].
     */
    label?: string | undefined;
    _label?: Element | undefined;
    /**
     * Mappings are not necessarily specific enough for safe translation.
     */
    mapping?: ElementDefinitionMapping[] | undefined;
    /**
     * The maximum number of times this element is permitted to appear in the instance.
     */
    max?: string | undefined;
    _max?: Element | undefined;
    /**
     * Receivers are not required to reject instances that exceed the maximum length.  The full length could be stored.  In some cases, data might be truncated, though truncation should be undertaken with care and an understanding of the consequences of doing so. If not specified, there is no conformance expectation for length support.
     */
    maxLength?: number | undefined;
    /**
     * Except for date/date/instant, the type of the maxValue[x] SHALL be the same as the specified type of the element. For the date/dateTime/instant values, the type of maxValue[x] SHALL be either the same, or a [Duration](datatypes.html#Duration) which specifies a relative time limit to the current time. The duration value is positive, and is added to the current clock to determine the maximum allowable value.   A maximum value for a Quantity is interpreted as an canonical maximum - e.g. you cannot provide 10g if the maximum value is 50mg.
     */
    maxValueDate?: string | undefined;
    _maxValueDate?: Element | undefined;
    /**
     * Except for date/date/instant, the type of the maxValue[x] SHALL be the same as the specified type of the element. For the date/dateTime/instant values, the type of maxValue[x] SHALL be either the same, or a [Duration](datatypes.html#Duration) which specifies a relative time limit to the current time. The duration value is positive, and is added to the current clock to determine the maximum allowable value.   A maximum value for a Quantity is interpreted as an canonical maximum - e.g. you cannot provide 10g if the maximum value is 50mg.
     */
    maxValueDateTime?: string | undefined;
    _maxValueDateTime?: Element | undefined;
    /**
     * Except for date/date/instant, the type of the maxValue[x] SHALL be the same as the specified type of the element. For the date/dateTime/instant values, the type of maxValue[x] SHALL be either the same, or a [Duration](datatypes.html#Duration) which specifies a relative time limit to the current time. The duration value is positive, and is added to the current clock to determine the maximum allowable value.   A maximum value for a Quantity is interpreted as an canonical maximum - e.g. you cannot provide 10g if the maximum value is 50mg.
     */
    maxValueInstant?: string | undefined;
    _maxValueInstant?: Element | undefined;
    /**
     * Except for date/date/instant, the type of the maxValue[x] SHALL be the same as the specified type of the element. For the date/dateTime/instant values, the type of maxValue[x] SHALL be either the same, or a [Duration](datatypes.html#Duration) which specifies a relative time limit to the current time. The duration value is positive, and is added to the current clock to determine the maximum allowable value.   A maximum value for a Quantity is interpreted as an canonical maximum - e.g. you cannot provide 10g if the maximum value is 50mg.
     */
    maxValueTime?: string | undefined;
    _maxValueTime?: Element | undefined;
    /**
     * Except for date/date/instant, the type of the maxValue[x] SHALL be the same as the specified type of the element. For the date/dateTime/instant values, the type of maxValue[x] SHALL be either the same, or a [Duration](datatypes.html#Duration) which specifies a relative time limit to the current time. The duration value is positive, and is added to the current clock to determine the maximum allowable value.   A maximum value for a Quantity is interpreted as an canonical maximum - e.g. you cannot provide 10g if the maximum value is 50mg.
     */
    maxValueDecimal?: number | undefined;
    /**
     * Except for date/date/instant, the type of the maxValue[x] SHALL be the same as the specified type of the element. For the date/dateTime/instant values, the type of maxValue[x] SHALL be either the same, or a [Duration](datatypes.html#Duration) which specifies a relative time limit to the current time. The duration value is positive, and is added to the current clock to determine the maximum allowable value.   A maximum value for a Quantity is interpreted as an canonical maximum - e.g. you cannot provide 10g if the maximum value is 50mg.
     */
    maxValueInteger?: number | undefined;
    /**
     * Except for date/date/instant, the type of the maxValue[x] SHALL be the same as the specified type of the element. For the date/dateTime/instant values, the type of maxValue[x] SHALL be either the same, or a [Duration](datatypes.html#Duration) which specifies a relative time limit to the current time. The duration value is positive, and is added to the current clock to determine the maximum allowable value.   A maximum value for a Quantity is interpreted as an canonical maximum - e.g. you cannot provide 10g if the maximum value is 50mg.
     */
    maxValueInteger64?: string | undefined;
    _maxValueInteger64?: Element | undefined;
    /**
     * Except for date/date/instant, the type of the maxValue[x] SHALL be the same as the specified type of the element. For the date/dateTime/instant values, the type of maxValue[x] SHALL be either the same, or a [Duration](datatypes.html#Duration) which specifies a relative time limit to the current time. The duration value is positive, and is added to the current clock to determine the maximum allowable value.   A maximum value for a Quantity is interpreted as an canonical maximum - e.g. you cannot provide 10g if the maximum value is 50mg.
     */
    maxValuePositiveInt?: number | undefined;
    /**
     * Except for date/date/instant, the type of the maxValue[x] SHALL be the same as the specified type of the element. For the date/dateTime/instant values, the type of maxValue[x] SHALL be either the same, or a [Duration](datatypes.html#Duration) which specifies a relative time limit to the current time. The duration value is positive, and is added to the current clock to determine the maximum allowable value.   A maximum value for a Quantity is interpreted as an canonical maximum - e.g. you cannot provide 10g if the maximum value is 50mg.
     */
    maxValueUnsignedInt?: number | undefined;
    /**
     * Except for date/date/instant, the type of the maxValue[x] SHALL be the same as the specified type of the element. For the date/dateTime/instant values, the type of maxValue[x] SHALL be either the same, or a [Duration](datatypes.html#Duration) which specifies a relative time limit to the current time. The duration value is positive, and is added to the current clock to determine the maximum allowable value.   A maximum value for a Quantity is interpreted as an canonical maximum - e.g. you cannot provide 10g if the maximum value is 50mg.
     */
    maxValueQuantity?: Quantity | undefined;
    /**
     * Implicit meanings for missing values can only be specified on a resource, data type, or extension definition, and never in a profile that applies to one of these. An implicit meaning for a missing value can never be changed, and specifying one has the consequence that constraining its use in profiles eliminates use cases as possibilities, not merely moving them out of scope.
     */
    meaningWhenMissing?: string | undefined;
    _meaningWhenMissing?: Element | undefined;
    /**
     * The minimum number of times this element SHALL appear in the instance.
     */
    min?: number | undefined;
    /**
     * Except for date/date/instant, the type of the minValue[x] SHALL be the same as the specified type of the element. For the date/dateTime/instant values, the type of minValue[x] SHALL be either the same, or a [Duration](datatypes.html#Duration) which specifies a relative time limit to the current time. The duration value is positive, and is subtracted from the current clock to determine the minimum allowable value.   A minimum value for a Quantity is interpreted as an canonical minimum - e.g. you cannot provide 100mg if the minimum value is 10g.
     */
    minValueDate?: string | undefined;
    _minValueDate?: Element | undefined;
    /**
     * Except for date/date/instant, the type of the minValue[x] SHALL be the same as the specified type of the element. For the date/dateTime/instant values, the type of minValue[x] SHALL be either the same, or a [Duration](datatypes.html#Duration) which specifies a relative time limit to the current time. The duration value is positive, and is subtracted from the current clock to determine the minimum allowable value.   A minimum value for a Quantity is interpreted as an canonical minimum - e.g. you cannot provide 100mg if the minimum value is 10g.
     */
    minValueDateTime?: string | undefined;
    _minValueDateTime?: Element | undefined;
    /**
     * Except for date/date/instant, the type of the minValue[x] SHALL be the same as the specified type of the element. For the date/dateTime/instant values, the type of minValue[x] SHALL be either the same, or a [Duration](datatypes.html#Duration) which specifies a relative time limit to the current time. The duration value is positive, and is subtracted from the current clock to determine the minimum allowable value.   A minimum value for a Quantity is interpreted as an canonical minimum - e.g. you cannot provide 100mg if the minimum value is 10g.
     */
    minValueInstant?: string | undefined;
    _minValueInstant?: Element | undefined;
    /**
     * Except for date/date/instant, the type of the minValue[x] SHALL be the same as the specified type of the element. For the date/dateTime/instant values, the type of minValue[x] SHALL be either the same, or a [Duration](datatypes.html#Duration) which specifies a relative time limit to the current time. The duration value is positive, and is subtracted from the current clock to determine the minimum allowable value.   A minimum value for a Quantity is interpreted as an canonical minimum - e.g. you cannot provide 100mg if the minimum value is 10g.
     */
    minValueTime?: string | undefined;
    _minValueTime?: Element | undefined;
    /**
     * Except for date/date/instant, the type of the minValue[x] SHALL be the same as the specified type of the element. For the date/dateTime/instant values, the type of minValue[x] SHALL be either the same, or a [Duration](datatypes.html#Duration) which specifies a relative time limit to the current time. The duration value is positive, and is subtracted from the current clock to determine the minimum allowable value.   A minimum value for a Quantity is interpreted as an canonical minimum - e.g. you cannot provide 100mg if the minimum value is 10g.
     */
    minValueDecimal?: number | undefined;
    /**
     * Except for date/date/instant, the type of the minValue[x] SHALL be the same as the specified type of the element. For the date/dateTime/instant values, the type of minValue[x] SHALL be either the same, or a [Duration](datatypes.html#Duration) which specifies a relative time limit to the current time. The duration value is positive, and is subtracted from the current clock to determine the minimum allowable value.   A minimum value for a Quantity is interpreted as an canonical minimum - e.g. you cannot provide 100mg if the minimum value is 10g.
     */
    minValueInteger?: number | undefined;
    /**
     * Except for date/date/instant, the type of the minValue[x] SHALL be the same as the specified type of the element. For the date/dateTime/instant values, the type of minValue[x] SHALL be either the same, or a [Duration](datatypes.html#Duration) which specifies a relative time limit to the current time. The duration value is positive, and is subtracted from the current clock to determine the minimum allowable value.   A minimum value for a Quantity is interpreted as an canonical minimum - e.g. you cannot provide 100mg if the minimum value is 10g.
     */
    minValueInteger64?: string | undefined;
    _minValueInteger64?: Element | undefined;
    /**
     * Except for date/date/instant, the type of the minValue[x] SHALL be the same as the specified type of the element. For the date/dateTime/instant values, the type of minValue[x] SHALL be either the same, or a [Duration](datatypes.html#Duration) which specifies a relative time limit to the current time. The duration value is positive, and is subtracted from the current clock to determine the minimum allowable value.   A minimum value for a Quantity is interpreted as an canonical minimum - e.g. you cannot provide 100mg if the minimum value is 10g.
     */
    minValuePositiveInt?: number | undefined;
    /**
     * Except for date/date/instant, the type of the minValue[x] SHALL be the same as the specified type of the element. For the date/dateTime/instant values, the type of minValue[x] SHALL be either the same, or a [Duration](datatypes.html#Duration) which specifies a relative time limit to the current time. The duration value is positive, and is subtracted from the current clock to determine the minimum allowable value.   A minimum value for a Quantity is interpreted as an canonical minimum - e.g. you cannot provide 100mg if the minimum value is 10g.
     */
    minValueUnsignedInt?: number | undefined;
    /**
     * Except for date/date/instant, the type of the minValue[x] SHALL be the same as the specified type of the element. For the date/dateTime/instant values, the type of minValue[x] SHALL be either the same, or a [Duration](datatypes.html#Duration) which specifies a relative time limit to the current time. The duration value is positive, and is subtracted from the current clock to determine the minimum allowable value.   A minimum value for a Quantity is interpreted as an canonical minimum - e.g. you cannot provide 100mg if the minimum value is 10g.
     */
    minValueQuantity?: Quantity | undefined;
    /**
     * "Something meaningful" is context dependent and impossible to describe in the base FHIR specification. For this reason, the mustSupport flag is never set to true by the FHIR specification itself - it is only set to true in profiles.  A profile on a type can always make mustSupport = true if it is false in the base type but cannot make mustSupport = false if it is true in the base type.   This is done in [Resource Profiles](profiling.html#mustsupport), where the profile labels an element as mustSupport=true. When a profile does this, it SHALL also make clear exactly what kind of "support" is required, as this can mean many things.    Note that an element that has the property IsModifier is not necessarily a "key" element (e.g. one of the important elements to make use of the resource), nor is it automatically mustSupport - however both of these things are more likely to be true for IsModifier elements than for other elements.
     */
    mustSupport?: boolean | undefined;
    _mustSupport?: Element | undefined;
    /**
     * This element can only be asserted on repeating elements and can only be introduced when defining resources or data types.  It can be further refined profiled elements but if absent in the base type, a profile cannot assert meaning.
     */
    orderMeaning?: string | undefined;
    _orderMeaning?: Element | undefined;
    /**
     * The path identifies the element and is expressed as a "."-separated list of ancestor elements, beginning with the name of the resource or extension.
     */
    path: string;
    _path?: Element | undefined;
    /**
     * Mostly used for fixing values of CodeableConcept. In general, pattern[x] is not intended for use with primitive types, where is has the same meaning as fixed[x].
     */
    patternBase64Binary?: string | undefined;
    _patternBase64Binary?: Element | undefined;
    /**
     * Mostly used for fixing values of CodeableConcept. In general, pattern[x] is not intended for use with primitive types, where is has the same meaning as fixed[x].
     */
    patternBoolean?: boolean | undefined;
    _patternBoolean?: Element | undefined;
    /**
     * Mostly used for fixing values of CodeableConcept. In general, pattern[x] is not intended for use with primitive types, where is has the same meaning as fixed[x].
     */
    patternCanonical?: string | undefined;
    _patternCanonical?: Element | undefined;
    /**
     * Mostly used for fixing values of CodeableConcept. In general, pattern[x] is not intended for use with primitive types, where is has the same meaning as fixed[x].
     */
    patternCode?: string | undefined;
    _patternCode?: Element | undefined;
    /**
     * Mostly used for fixing values of CodeableConcept. In general, pattern[x] is not intended for use with primitive types, where is has the same meaning as fixed[x].
     */
    patternDate?: string | undefined;
    _patternDate?: Element | undefined;
    /**
     * Mostly used for fixing values of CodeableConcept. In general, pattern[x] is not intended for use with primitive types, where is has the same meaning as fixed[x].
     */
    patternDateTime?: string | undefined;
    _patternDateTime?: Element | undefined;
    /**
     * Mostly used for fixing values of CodeableConcept. In general, pattern[x] is not intended for use with primitive types, where is has the same meaning as fixed[x].
     */
    patternDecimal?: number | undefined;
    /**
     * Mostly used for fixing values of CodeableConcept. In general, pattern[x] is not intended for use with primitive types, where is has the same meaning as fixed[x].
     */
    patternId?: string | undefined;
    _patternId?: Element | undefined;
    /**
     * Mostly used for fixing values of CodeableConcept. In general, pattern[x] is not intended for use with primitive types, where is has the same meaning as fixed[x].
     */
    patternInstant?: string | undefined;
    _patternInstant?: Element | undefined;
    /**
     * Mostly used for fixing values of CodeableConcept. In general, pattern[x] is not intended for use with primitive types, where is has the same meaning as fixed[x].
     */
    patternInteger?: number | undefined;
    /**
     * Mostly used for fixing values of CodeableConcept. In general, pattern[x] is not intended for use with primitive types, where is has the same meaning as fixed[x].
     */
    patternInteger64?: string | undefined;
    _patternInteger64?: Element | undefined;
    /**
     * Mostly used for fixing values of CodeableConcept. In general, pattern[x] is not intended for use with primitive types, where is has the same meaning as fixed[x].
     */
    patternMarkdown?: string | undefined;
    _patternMarkdown?: Element | undefined;
    /**
     * Mostly used for fixing values of CodeableConcept. In general, pattern[x] is not intended for use with primitive types, where is has the same meaning as fixed[x].
     */
    patternOid?: string | undefined;
    _patternOid?: Element | undefined;
    /**
     * Mostly used for fixing values of CodeableConcept. In general, pattern[x] is not intended for use with primitive types, where is has the same meaning as fixed[x].
     */
    patternPositiveInt?: number | undefined;
    /**
     * Mostly used for fixing values of CodeableConcept. In general, pattern[x] is not intended for use with primitive types, where is has the same meaning as fixed[x].
     */
    patternString?: string | undefined;
    _patternString?: Element | undefined;
    /**
     * Mostly used for fixing values of CodeableConcept. In general, pattern[x] is not intended for use with primitive types, where is has the same meaning as fixed[x].
     */
    patternTime?: string | undefined;
    _patternTime?: Element | undefined;
    /**
     * Mostly used for fixing values of CodeableConcept. In general, pattern[x] is not intended for use with primitive types, where is has the same meaning as fixed[x].
     */
    patternUnsignedInt?: number | undefined;
    /**
     * Mostly used for fixing values of CodeableConcept. In general, pattern[x] is not intended for use with primitive types, where is has the same meaning as fixed[x].
     */
    patternUri?: string | undefined;
    _patternUri?: Element | undefined;
    /**
     * Mostly used for fixing values of CodeableConcept. In general, pattern[x] is not intended for use with primitive types, where is has the same meaning as fixed[x].
     */
    patternUrl?: string | undefined;
    _patternUrl?: Element | undefined;
    /**
     * Mostly used for fixing values of CodeableConcept. In general, pattern[x] is not intended for use with primitive types, where is has the same meaning as fixed[x].
     */
    patternUuid?: string | undefined;
    _patternUuid?: Element | undefined;
    /**
     * Mostly used for fixing values of CodeableConcept. In general, pattern[x] is not intended for use with primitive types, where is has the same meaning as fixed[x].
     */
    patternAddress?: Address | undefined;
    /**
     * Mostly used for fixing values of CodeableConcept. In general, pattern[x] is not intended for use with primitive types, where is has the same meaning as fixed[x].
     */
    patternAge?: Age | undefined;
    /**
     * Mostly used for fixing values of CodeableConcept. In general, pattern[x] is not intended for use with primitive types, where is has the same meaning as fixed[x].
     */
    patternAnnotation?: Annotation | undefined;
    /**
     * Mostly used for fixing values of CodeableConcept. In general, pattern[x] is not intended for use with primitive types, where is has the same meaning as fixed[x].
     */
    patternAttachment?: Attachment | undefined;
    /**
     * Mostly used for fixing values of CodeableConcept. In general, pattern[x] is not intended for use with primitive types, where is has the same meaning as fixed[x].
     */
    patternCodeableConcept?: CodeableConcept | undefined;
    /**
     * Mostly used for fixing values of CodeableConcept. In general, pattern[x] is not intended for use with primitive types, where is has the same meaning as fixed[x].
     */
    patternCodeableReference?: CodeableReference | undefined;
    /**
     * Mostly used for fixing values of CodeableConcept. In general, pattern[x] is not intended for use with primitive types, where is has the same meaning as fixed[x].
     */
    patternCoding?: Coding | undefined;
    /**
     * Mostly used for fixing values of CodeableConcept. In general, pattern[x] is not intended for use with primitive types, where is has the same meaning as fixed[x].
     */
    patternContactPoint?: ContactPoint | undefined;
    /**
     * Mostly used for fixing values of CodeableConcept. In general, pattern[x] is not intended for use with primitive types, where is has the same meaning as fixed[x].
     */
    patternCount?: Count | undefined;
    /**
     * Mostly used for fixing values of CodeableConcept. In general, pattern[x] is not intended for use with primitive types, where is has the same meaning as fixed[x].
     */
    patternDistance?: Distance | undefined;
    /**
     * Mostly used for fixing values of CodeableConcept. In general, pattern[x] is not intended for use with primitive types, where is has the same meaning as fixed[x].
     */
    patternDuration?: Duration | undefined;
    /**
     * Mostly used for fixing values of CodeableConcept. In general, pattern[x] is not intended for use with primitive types, where is has the same meaning as fixed[x].
     */
    patternHumanName?: HumanName | undefined;
    /**
     * Mostly used for fixing values of CodeableConcept. In general, pattern[x] is not intended for use with primitive types, where is has the same meaning as fixed[x].
     */
    patternIdentifier?: Identifier | undefined;
    /**
     * Mostly used for fixing values of CodeableConcept. In general, pattern[x] is not intended for use with primitive types, where is has the same meaning as fixed[x].
     */
    patternMoney?: Money | undefined;
    /**
     * Mostly used for fixing values of CodeableConcept. In general, pattern[x] is not intended for use with primitive types, where is has the same meaning as fixed[x].
     */
    patternPeriod?: Period | undefined;
    /**
     * Mostly used for fixing values of CodeableConcept. In general, pattern[x] is not intended for use with primitive types, where is has the same meaning as fixed[x].
     */
    patternQuantity?: Quantity | undefined;
    /**
     * Mostly used for fixing values of CodeableConcept. In general, pattern[x] is not intended for use with primitive types, where is has the same meaning as fixed[x].
     */
    patternRange?: Range | undefined;
    /**
     * Mostly used for fixing values of CodeableConcept. In general, pattern[x] is not intended for use with primitive types, where is has the same meaning as fixed[x].
     */
    patternRatio?: Ratio | undefined;
    /**
     * Mostly used for fixing values of CodeableConcept. In general, pattern[x] is not intended for use with primitive types, where is has the same meaning as fixed[x].
     */
    patternRatioRange?: RatioRange | undefined;
    /**
     * Mostly used for fixing values of CodeableConcept. In general, pattern[x] is not intended for use with primitive types, where is has the same meaning as fixed[x].
     */
    patternReference?: Reference | undefined;
    /**
     * Mostly used for fixing values of CodeableConcept. In general, pattern[x] is not intended for use with primitive types, where is has the same meaning as fixed[x].
     */
    patternSampledData?: SampledData | undefined;
    /**
     * Mostly used for fixing values of CodeableConcept. In general, pattern[x] is not intended for use with primitive types, where is has the same meaning as fixed[x].
     */
    patternSignature?: Signature | undefined;
    /**
     * Mostly used for fixing values of CodeableConcept. In general, pattern[x] is not intended for use with primitive types, where is has the same meaning as fixed[x].
     */
    patternTiming?: Timing | undefined;
    /**
     * Mostly used for fixing values of CodeableConcept. In general, pattern[x] is not intended for use with primitive types, where is has the same meaning as fixed[x].
     */
    patternContactDetail?: ContactDetail | undefined;
    /**
     * Mostly used for fixing values of CodeableConcept. In general, pattern[x] is not intended for use with primitive types, where is has the same meaning as fixed[x].
     */
    patternDataRequirement?: DataRequirement | undefined;
    /**
     * Mostly used for fixing values of CodeableConcept. In general, pattern[x] is not intended for use with primitive types, where is has the same meaning as fixed[x].
     */
    patternExpression?: Expression | undefined;
    /**
     * Mostly used for fixing values of CodeableConcept. In general, pattern[x] is not intended for use with primitive types, where is has the same meaning as fixed[x].
     */
    patternParameterDefinition?: ParameterDefinition | undefined;
    /**
     * Mostly used for fixing values of CodeableConcept. In general, pattern[x] is not intended for use with primitive types, where is has the same meaning as fixed[x].
     */
    patternRelatedArtifact?: RelatedArtifact | undefined;
    /**
     * Mostly used for fixing values of CodeableConcept. In general, pattern[x] is not intended for use with primitive types, where is has the same meaning as fixed[x].
     */
    patternTriggerDefinition?: TriggerDefinition | undefined;
    /**
     * Mostly used for fixing values of CodeableConcept. In general, pattern[x] is not intended for use with primitive types, where is has the same meaning as fixed[x].
     */
    patternUsageContext?: UsageContext | undefined;
    /**
     * Mostly used for fixing values of CodeableConcept. In general, pattern[x] is not intended for use with primitive types, where is has the same meaning as fixed[x].
     */
    patternAvailability?: Availability | undefined;
    /**
     * Mostly used for fixing values of CodeableConcept. In general, pattern[x] is not intended for use with primitive types, where is has the same meaning as fixed[x].
     */
    patternExtendedContactDetail?: ExtendedContactDetail | undefined;
    /**
     * Mostly used for fixing values of CodeableConcept. In general, pattern[x] is not intended for use with primitive types, where is has the same meaning as fixed[x].
     */
    patternDosage?: Dosage | undefined;
    /**
     * Mostly used for fixing values of CodeableConcept. In general, pattern[x] is not intended for use with primitive types, where is has the same meaning as fixed[x].
     */
    patternMeta?: Meta | undefined;
    /**
     * In resources, this is rarely used except for special cases where the representation deviates from the normal, and can only be done in the base standard (and profiles must reproduce what the base standard does). This element is used quite commonly in Logical models when the logical models represent a specific serialization format (e.g. CDA, v2 etc.).
     */
    representation?: ('xmlAttr' | 'xmlText' | 'typeAttr' | 'cdaText' | 'xhtml')[] | undefined;
    _representation?: Element[] | undefined;
    /**
     * This element does not describe the usage of the element (that's done in comments), rather it's for traceability of *why* the element is either needed or why the constraints exist as they do.  This may be used to point to source materials or specifications that drove the structure of this data element.
     */
    requirements?: string | undefined;
    _requirements?: Element | undefined;
    /**
     * It is easy for a different short definition to change the meaning of an element and this can have nasty downstream consequences. Please be careful when providing short definitions in a profile.
     */
    short?: string | undefined;
    _short?: Element | undefined;
    /**
     * If set to true, an ancestor profile SHALL have a slicing definition with this name.  If set to false, no ancestor profile is permitted to have a slicing definition with this name.
     */
    sliceIsConstraining?: boolean | undefined;
    _sliceIsConstraining?: Element | undefined;
    /**
     * The name SHALL be unique within the structure within the context of the constrained resource element.  (Though to avoid confusion, uniqueness across all elements is recommended.).
     */
    sliceName?: string | undefined;
    _sliceName?: Element | undefined;
    /**
     * The first element in the sequence, the one that carries the slicing, is the definition that applies to all the slices. This is based on the unconstrained element, but can apply any constraints as appropriate. This may include the common constraints on the children of the element.
     */
    slicing?: ElementDefinitionSlicing | undefined;
    /**
     * The Type of the element can be left blank in a differential constraint, in which case the type is inherited from the resource. Abstract types are not permitted to appear as a type when multiple types are listed.  (I.e. Abstract types cannot be part of a choice).
     */
    type?: ElementDefinitionType[] | undefined;
}
/**
 * A expression that is evaluated in a specified context and returns a value. The context of use of the expression must specify the context in which the expression is evaluated, and how the result of the expression is used.
 */
export interface Expression extends DataType {
    /**
     * A brief, natural language description of the condition that effectively communicates the intended semantics.
     */
    description?: string | undefined;
    _description?: Element | undefined;
    /**
     * If Expression.expression and Expression.reference are both present, the Expression.expression might just be a name pointing something within the referenced content.
     */
    expression?: string | undefined;
    _expression?: Element | undefined;
    /**
     * The media type of the language for the expression.
     */
    language?: string | undefined;
    _language?: Element | undefined;
    /**
     * A short name assigned to the expression to allow for multiple reuse of the expression in the context where it is defined.
     */
    name?: string | undefined;
    _name?: Element | undefined;
    /**
     * If both a reference and an expression is found, the reference SHALL point to the same expression.
     */
    reference?: string | undefined;
    _reference?: Element | undefined;
}
/**
 * Specifies contact information for a specific purpose over a period of time, might be handled/monitored by a specific named person or organization.
 */
export interface ExtendedContactDetail extends DataType {
    /**
     * More than 1 address would be for different purposes, and thus should be entered as a different entry,.
     */
    address?: Address | undefined;
    /**
     * If there is no named individual, the telecom/address information is not generally monitored by a specific individual.
     */
    name?: HumanName[] | undefined;
    /**
     * This contact detail is handled/monitored by a specific organization. If the name is provided in the contact, then it is referring to the named individual within this organization.
     */
    organization?: Reference | undefined;
    /**
     * If the details have multiple periods, then enter in a new ExtendedContact with the new period.
     */
    period?: Period | undefined;
    /**
     * If no purpose is defined, then these contact details may be used for any purpose.
     */
    purpose?: CodeableConcept | undefined;
    /**
     * The contact details application for the purpose defined.
     */
    telecom?: ContactPoint[] | undefined;
}
/**
 * Optional Extension Element - found in all resources.
 */
export interface Extension extends DataType {
    /**
     * The definition may point directly to a computable or human-readable definition of the extensibility codes, or it may be a logical URI as declared in some other specification. The definition SHALL be a URI for the Structure Definition defining the extension.
     */
    url: string;
    _url?: Element | undefined;
    /**
     * Value of extension - must be one of a constrained set of the data types (see [Extensibility](extensibility.html) for a list).
     */
    valueBase64Binary?: string | undefined;
    _valueBase64Binary?: Element | undefined;
    /**
     * Value of extension - must be one of a constrained set of the data types (see [Extensibility](extensibility.html) for a list).
     */
    valueBoolean?: boolean | undefined;
    _valueBoolean?: Element | undefined;
    /**
     * Value of extension - must be one of a constrained set of the data types (see [Extensibility](extensibility.html) for a list).
     */
    valueCanonical?: string | undefined;
    _valueCanonical?: Element | undefined;
    /**
     * Value of extension - must be one of a constrained set of the data types (see [Extensibility](extensibility.html) for a list).
     */
    valueCode?: string | undefined;
    _valueCode?: Element | undefined;
    /**
     * Value of extension - must be one of a constrained set of the data types (see [Extensibility](extensibility.html) for a list).
     */
    valueDate?: string | undefined;
    _valueDate?: Element | undefined;
    /**
     * Value of extension - must be one of a constrained set of the data types (see [Extensibility](extensibility.html) for a list).
     */
    valueDateTime?: string | undefined;
    _valueDateTime?: Element | undefined;
    /**
     * Value of extension - must be one of a constrained set of the data types (see [Extensibility](extensibility.html) for a list).
     */
    valueDecimal?: number | undefined;
    /**
     * Value of extension - must be one of a constrained set of the data types (see [Extensibility](extensibility.html) for a list).
     */
    valueId?: string | undefined;
    _valueId?: Element | undefined;
    /**
     * Value of extension - must be one of a constrained set of the data types (see [Extensibility](extensibility.html) for a list).
     */
    valueInstant?: string | undefined;
    _valueInstant?: Element | undefined;
    /**
     * Value of extension - must be one of a constrained set of the data types (see [Extensibility](extensibility.html) for a list).
     */
    valueInteger?: number | undefined;
    /**
     * Value of extension - must be one of a constrained set of the data types (see [Extensibility](extensibility.html) for a list).
     */
    valueInteger64?: string | undefined;
    _valueInteger64?: Element | undefined;
    /**
     * Value of extension - must be one of a constrained set of the data types (see [Extensibility](extensibility.html) for a list).
     */
    valueMarkdown?: string | undefined;
    _valueMarkdown?: Element | undefined;
    /**
     * Value of extension - must be one of a constrained set of the data types (see [Extensibility](extensibility.html) for a list).
     */
    valueOid?: string | undefined;
    _valueOid?: Element | undefined;
    /**
     * Value of extension - must be one of a constrained set of the data types (see [Extensibility](extensibility.html) for a list).
     */
    valuePositiveInt?: number | undefined;
    /**
     * Value of extension - must be one of a constrained set of the data types (see [Extensibility](extensibility.html) for a list).
     */
    valueString?: string | undefined;
    _valueString?: Element | undefined;
    /**
     * Value of extension - must be one of a constrained set of the data types (see [Extensibility](extensibility.html) for a list).
     */
    valueTime?: string | undefined;
    _valueTime?: Element | undefined;
    /**
     * Value of extension - must be one of a constrained set of the data types (see [Extensibility](extensibility.html) for a list).
     */
    valueUnsignedInt?: number | undefined;
    /**
     * Value of extension - must be one of a constrained set of the data types (see [Extensibility](extensibility.html) for a list).
     */
    valueUri?: string | undefined;
    _valueUri?: Element | undefined;
    /**
     * Value of extension - must be one of a constrained set of the data types (see [Extensibility](extensibility.html) for a list).
     */
    valueUrl?: string | undefined;
    _valueUrl?: Element | undefined;
    /**
     * Value of extension - must be one of a constrained set of the data types (see [Extensibility](extensibility.html) for a list).
     */
    valueUuid?: string | undefined;
    _valueUuid?: Element | undefined;
    /**
     * Value of extension - must be one of a constrained set of the data types (see [Extensibility](extensibility.html) for a list).
     */
    valueAddress?: Address | undefined;
    /**
     * Value of extension - must be one of a constrained set of the data types (see [Extensibility](extensibility.html) for a list).
     */
    valueAge?: Age | undefined;
    /**
     * Value of extension - must be one of a constrained set of the data types (see [Extensibility](extensibility.html) for a list).
     */
    valueAnnotation?: Annotation | undefined;
    /**
     * Value of extension - must be one of a constrained set of the data types (see [Extensibility](extensibility.html) for a list).
     */
    valueAttachment?: Attachment | undefined;
    /**
     * Value of extension - must be one of a constrained set of the data types (see [Extensibility](extensibility.html) for a list).
     */
    valueCodeableConcept?: CodeableConcept | undefined;
    /**
     * Value of extension - must be one of a constrained set of the data types (see [Extensibility](extensibility.html) for a list).
     */
    valueCodeableReference?: CodeableReference | undefined;
    /**
     * Value of extension - must be one of a constrained set of the data types (see [Extensibility](extensibility.html) for a list).
     */
    valueCoding?: Coding | undefined;
    /**
     * Value of extension - must be one of a constrained set of the data types (see [Extensibility](extensibility.html) for a list).
     */
    valueContactPoint?: ContactPoint | undefined;
    /**
     * Value of extension - must be one of a constrained set of the data types (see [Extensibility](extensibility.html) for a list).
     */
    valueCount?: Count | undefined;
    /**
     * Value of extension - must be one of a constrained set of the data types (see [Extensibility](extensibility.html) for a list).
     */
    valueDistance?: Distance | undefined;
    /**
     * Value of extension - must be one of a constrained set of the data types (see [Extensibility](extensibility.html) for a list).
     */
    valueDuration?: Duration | undefined;
    /**
     * Value of extension - must be one of a constrained set of the data types (see [Extensibility](extensibility.html) for a list).
     */
    valueHumanName?: HumanName | undefined;
    /**
     * Value of extension - must be one of a constrained set of the data types (see [Extensibility](extensibility.html) for a list).
     */
    valueIdentifier?: Identifier | undefined;
    /**
     * Value of extension - must be one of a constrained set of the data types (see [Extensibility](extensibility.html) for a list).
     */
    valueMoney?: Money | undefined;
    /**
     * Value of extension - must be one of a constrained set of the data types (see [Extensibility](extensibility.html) for a list).
     */
    valuePeriod?: Period | undefined;
    /**
     * Value of extension - must be one of a constrained set of the data types (see [Extensibility](extensibility.html) for a list).
     */
    valueQuantity?: Quantity | undefined;
    /**
     * Value of extension - must be one of a constrained set of the data types (see [Extensibility](extensibility.html) for a list).
     */
    valueRange?: Range | undefined;
    /**
     * Value of extension - must be one of a constrained set of the data types (see [Extensibility](extensibility.html) for a list).
     */
    valueRatio?: Ratio | undefined;
    /**
     * Value of extension - must be one of a constrained set of the data types (see [Extensibility](extensibility.html) for a list).
     */
    valueRatioRange?: RatioRange | undefined;
    /**
     * Value of extension - must be one of a constrained set of the data types (see [Extensibility](extensibility.html) for a list).
     */
    valueReference?: Reference | undefined;
    /**
     * Value of extension - must be one of a constrained set of the data types (see [Extensibility](extensibility.html) for a list).
     */
    valueSampledData?: SampledData | undefined;
    /**
     * Value of extension - must be one of a constrained set of the data types (see [Extensibility](extensibility.html) for a list).
     */
    valueSignature?: Signature | undefined;
    /**
     * Value of extension - must be one of a constrained set of the data types (see [Extensibility](extensibility.html) for a list).
     */
    valueTiming?: Timing | undefined;
    /**
     * Value of extension - must be one of a constrained set of the data types (see [Extensibility](extensibility.html) for a list).
     */
    valueContactDetail?: ContactDetail | undefined;
    /**
     * Value of extension - must be one of a constrained set of the data types (see [Extensibility](extensibility.html) for a list).
     */
    valueDataRequirement?: DataRequirement | undefined;
    /**
     * Value of extension - must be one of a constrained set of the data types (see [Extensibility](extensibility.html) for a list).
     */
    valueExpression?: Expression | undefined;
    /**
     * Value of extension - must be one of a constrained set of the data types (see [Extensibility](extensibility.html) for a list).
     */
    valueParameterDefinition?: ParameterDefinition | undefined;
    /**
     * Value of extension - must be one of a constrained set of the data types (see [Extensibility](extensibility.html) for a list).
     */
    valueRelatedArtifact?: RelatedArtifact | undefined;
    /**
     * Value of extension - must be one of a constrained set of the data types (see [Extensibility](extensibility.html) for a list).
     */
    valueTriggerDefinition?: TriggerDefinition | undefined;
    /**
     * Value of extension - must be one of a constrained set of the data types (see [Extensibility](extensibility.html) for a list).
     */
    valueUsageContext?: UsageContext | undefined;
    /**
     * Value of extension - must be one of a constrained set of the data types (see [Extensibility](extensibility.html) for a list).
     */
    valueAvailability?: Availability | undefined;
    /**
     * Value of extension - must be one of a constrained set of the data types (see [Extensibility](extensibility.html) for a list).
     */
    valueExtendedContactDetail?: ExtendedContactDetail | undefined;
    /**
     * Value of extension - must be one of a constrained set of the data types (see [Extensibility](extensibility.html) for a list).
     */
    valueDosage?: Dosage | undefined;
    /**
     * Value of extension - must be one of a constrained set of the data types (see [Extensibility](extensibility.html) for a list).
     */
    valueMeta?: Meta | undefined;
}
/**
 * A name, normally of a human, that can be used for other living entities (eg. animals but not organizations) that have been assigned names by a human and may need the use of name parts or the need for usage information.
 */
export interface HumanName extends DataType {
    /**
     * Family Name may be decomposed into specific parts using extensions (de, nl, es related cultures).
     */
    family?: string | undefined;
    _family?: Element | undefined;
    /**
     * If only initials are recorded, they may be used in place of the full name parts. Initials may be separated into multiple given names but often aren't due to paractical limitations.  This element is not called "first name" since given names do not always come first.
     */
    given?: string[] | undefined;
    _given?: Element[] | undefined;
    /**
     * Indicates the period of time when this name was valid for the named person.
     */
    period?: Period | undefined;
    /**
     * Part of the name that is acquired as a title due to academic, legal, employment or nobility status, etc. and that appears at the start of the name.
     */
    prefix?: string[] | undefined;
    _prefix?: Element[] | undefined;
    /**
     * Part of the name that is acquired as a title due to academic, legal, employment or nobility status, etc. and that appears at the end of the name.
     */
    suffix?: string[] | undefined;
    _suffix?: Element[] | undefined;
    /**
     * Can provide both a text representation and parts. Applications updating a name SHALL ensure that when both text and parts are present,  no content is included in the text that isn't found in a part.
     */
    text?: string | undefined;
    _text?: Element | undefined;
    /**
     * Applications can assume that a name is current unless it explicitly says that it is temporary or old.
     */
    use?: ('usual' | 'official' | 'temp' | 'nickname' | 'anonymous' | 'old' | 'maiden') | undefined;
    _use?: Element | undefined;
}
/**
 * An identifier - identifies some entity uniquely and unambiguously. Typically this is used for business identifiers.
 */
export interface Identifier extends DataType {
    /**
     * The Identifier.assigner may omit the .reference element and only contain a .display element reflecting the name or other textual information about the assigning organization.
     */
    assigner?: Reference | undefined;
    /**
     * Time period during which identifier is/was valid for use.
     */
    period?: Period | undefined;
    /**
     * Identifier.system is always case sensitive.
     */
    system?: string | undefined;
    _system?: Element | undefined;
    /**
     * This element deals only with general categories of identifiers.  It SHOULD not be used for codes that correspond 1..1 with the Identifier.system. Some identifiers may fall into multiple categories due to common usage.   Where the system is known, a type is unnecessary because the type is always part of the system definition. However systems often need to handle identifiers where the system is not known. There is not a 1:1 relationship between type and system, since many different systems have the same type.
     */
    type?: CodeableConcept | undefined;
    /**
     * Applications can assume that an identifier is permanent unless it explicitly says that it is temporary.
     */
    use?: ('usual' | 'official' | 'temp' | 'secondary' | 'old') | undefined;
    _use?: Element | undefined;
    /**
     * If the value is a full URI, then the system SHALL be urn:ietf:rfc:3986.  The value's primary purpose is computational mapping.  As a result, it may be normalized for comparison purposes (e.g. removing non-significant whitespace, dashes, etc.)  A value formatted for human display can be conveyed using the [Rendered Value extension](extension-rendered-value.html). Identifier.value is to be treated as case sensitive unless knowledge of the Identifier.system allows the processer to be confident that non-case-sensitive processing is safe.
     */
    value?: string | undefined;
    _value?: Element | undefined;
}
/**
 * The marketing status describes the date when a medicinal product is actually put on the market or the date as of which it is no longer available.
 */
export interface MarketingStatus extends BackboneType {
    /**
     * The country in which the marketing authorisation has been granted shall be specified It should be specified using the ISO 3166 ‑ 1 alpha-2 code elements.
     */
    country?: CodeableConcept | undefined;
    /**
     * The date when the Medicinal Product is placed on the market by the Marketing Authorisation Holder (or where applicable, the manufacturer/distributor) in a country and/or jurisdiction shall be provided A complete date consisting of day, month and year shall be specified using the ISO 8601 date format NOTE “Placed on the market” refers to the release of the Medicinal Product into the distribution chain.
     */
    dateRange?: Period | undefined;
    /**
     * Where a Medicines Regulatory Agency has granted a marketing authorisation for which specific provisions within a jurisdiction apply, the jurisdiction can be specified using an appropriate controlled terminology The controlled term and the controlled term identifier shall be specified.
     */
    jurisdiction?: CodeableConcept | undefined;
    /**
     * The date when the Medicinal Product is placed on the market by the Marketing Authorisation Holder (or where applicable, the manufacturer/distributor) in a country and/or jurisdiction shall be provided A complete date consisting of day, month and year shall be specified using the ISO 8601 date format NOTE “Placed on the market” refers to the release of the Medicinal Product into the distribution chain.
     */
    restoreDate?: string | undefined;
    _restoreDate?: Element | undefined;
    /**
     * This attribute provides information on the status of the marketing of the medicinal product See ISO/TS 20443 for more information and examples.
     */
    status: CodeableConcept;
}
/**
 * The metadata about a resource. This is content in the resource that is maintained by the infrastructure. Changes to the content might not always be associated with version changes to the resource.
 */
export interface Meta extends DataType {
    /**
     * This value is always populated except when the resource is first being created. The server / resource manager sets this value; what a client provides is irrelevant. This is equivalent to the HTTP Last-Modified and SHOULD have the same value on a [read](http.html#read) interaction.
     */
    lastUpdated?: string | undefined;
    _lastUpdated?: Element | undefined;
    /**
     * It is up to the server and/or other infrastructure of policy to determine whether/how these claims are verified and/or updated over time.  The list of profile URLs is a set.
     */
    profile?: string[] | undefined;
    _profile?: Element[] | undefined;
    /**
     * The security labels can be updated without changing the stated version of the resource. The list of security labels is a set. Uniqueness is based the system/code, and version and display are ignored.
     */
    security?: Coding[] | undefined;
    /**
     * The exact use of the source (and the possible implied Provenance.entity.role and agent.role) is left to implementer discretion. Only one nominated source is allowed; for additional provenance details, a full Provenance resource should be used. The source may correspond to Provenance.entity.what[x] or Provenance.agent.who[x], though it may be a more general or abstract reference.
     * This element can be used to indicate where the current master source of a resource that has a canonical URL if the resource is no longer hosted at the canonical URL.
     */
    source?: string | undefined;
    _source?: Element | undefined;
    /**
     * The tags can be updated without changing the stated version of the resource. The list of tags is a set. Uniqueness is based the system/code, and version and display are ignored.
     */
    tag?: Coding[] | undefined;
    /**
     * The server assigns this value, and ignores what the client specifies, except in the case that the server is imposing version integrity on updates/deletes.
     */
    versionId?: string | undefined;
    _versionId?: Element | undefined;
}
/**
 * Availability data for an {item}.
 */
export interface MonetaryComponent extends DataType {
    /**
     * Explicit value amount to be used.
     */
    amount?: Money | undefined;
    /**
     * Codes may be used to differentiate between kinds of taxes, surcharges, discounts etc.
     */
    code?: CodeableConcept | undefined;
    /**
     * Factor used for calculating this component.
     */
    factor?: number | undefined;
    /**
     * base | surcharge | deduction | discount | tax | informational.
     */
    type: 'base' | 'surcharge' | 'deduction' | 'discount' | 'tax' | 'informational';
    _type?: Element | undefined;
}
/**
 * An amount of economic utility in some recognized currency.
 */
export interface Money extends DataType {
    /**
     * ISO 4217 Currency Code.
     */
    currency?: string | undefined;
    _currency?: Element | undefined;
    /**
     * Monetary values have their own rules for handling precision (refer to standard accounting text books).
     */
    value?: number | undefined;
}
/**
 * A human-readable summary of the resource conveying the essential clinical and business information for the resource.
 */
export interface Narrative extends DataType {
    /**
     * The contents of the html element are an XHTML fragment containing only the basic html formatting elements described in chapters 7-11 and 15 of the HTML 4.0 standard, <a> elements (either name or href), images and internally contained stylesheets. The XHTML content SHALL NOT contain a head, a body, external stylesheet references, scripts, forms, base/link/xlink, frames, iframes and objects.
     */
    div: string;
    _div?: Element | undefined;
    /**
     * The status of the narrative - whether it's entirely generated (from just the defined data or the extensions too), or whether a human authored it and it may contain additional data.
     */
    status: 'generated' | 'extensions' | 'additional' | 'empty';
    _status?: Element | undefined;
}
/**
 * The parameters to the module. This collection specifies both the input and output parameters. Input parameters are provided by the caller as part of the $evaluate operation. Output parameters are included in the GuidanceResponse.
 */
export interface ParameterDefinition extends DataType {
    /**
     * A brief discussion of what the parameter is for and how it is used by the module.
     */
    documentation?: string | undefined;
    _documentation?: Element | undefined;
    /**
     * The maximum number of times this element is permitted to appear in the request or response.
     */
    max?: string | undefined;
    _max?: Element | undefined;
    /**
     * The minimum number of times this parameter SHALL appear in the request or response.
     */
    min?: number | undefined;
    /**
     * The name of the parameter used to allow access to the value of the parameter in evaluation contexts.
     */
    name?: string | undefined;
    _name?: Element | undefined;
    /**
     * If specified, this indicates a profile that the input data must conform to, or that the output data will conform to.
     */
    profile?: string | undefined;
    _profile?: Element | undefined;
    /**
     * The type of the parameter.
     */
    type: string;
    _type?: Element | undefined;
    /**
     * Whether the parameter is input or output for the module.
     */
    use: 'in' | 'out';
    _use?: Element | undefined;
}
/**
 * A time period defined by a start and end date and optionally time.
 */
export interface Period extends DataType {
    /**
     * The high value includes any matching date/time. i.e. 2012-02-03T10:00:00 is in a period that has an end value of 2012-02-03.
     */
    end?: string | undefined;
    _end?: Element | undefined;
    /**
     * If the low element is missing, the meaning is that the low boundary is not known.
     */
    start?: string | undefined;
    _start?: Element | undefined;
}
/**
 * A populatioof people with some set of grouping criteria.
 */
export interface Population extends BackboneType {
    /**
     * The age of the specific population.
     */
    ageRange?: Range | undefined;
    /**
     * The age of the specific population.
     */
    ageCodeableConcept?: CodeableConcept | undefined;
    /**
     * The gender of the specific population.
     */
    gender?: CodeableConcept | undefined;
    /**
     * The existing physiological conditions of the specific population to which this applies.
     */
    physiologicalCondition?: CodeableConcept | undefined;
    /**
     * Race of the specific population.
     */
    race?: CodeableConcept | undefined;
}
/**
 * The base type for all re-useable types defined that have a simple property.
 */
export interface PrimitiveType extends DataType {}
/**
 * The shelf-life and storage information for a medicinal product item or container can be described using this class.
 */
export interface ProductShelfLife extends BackboneType {
    /**
     * The shelf life time period can be specified using a numerical value for the period of time and its unit of time measurement The unit of measurement shall be specified in accordance with ISO 11240 and the resulting terminology The symbol and the symbol identifier shall be used.
     */
    periodDuration?: Duration | undefined;
    /**
     * The shelf life time period can be specified using a numerical value for the period of time and its unit of time measurement The unit of measurement shall be specified in accordance with ISO 11240 and the resulting terminology The symbol and the symbol identifier shall be used.
     */
    periodString?: string | undefined;
    _periodString?: Element | undefined;
    /**
     * Special precautions for storage, if any, can be specified using an appropriate controlled vocabulary The controlled term and the controlled term identifier shall be specified.
     */
    specialPrecautionsForStorage?: CodeableConcept[] | undefined;
    /**
     * This describes the shelf life, taking into account various scenarios such as shelf life of the packaged Medicinal Product itself, shelf life after transformation where necessary and shelf life after the first opening of a bottle, etc. The shelf life type shall be specified using an appropriate controlled vocabulary The controlled term and the controlled term identifier shall be specified.
     */
    type?: CodeableConcept | undefined;
}
/**
 * A measured amount (or an amount that can potentially be measured). Note that measured amounts include amounts that are not precisely quantified, including amounts involving arbitrary units and floating currencies.
 */
export interface Quantity extends DataType {
    /**
     * The preferred system is UCUM, but SNOMED CT can also be used (for customary units) or ISO 4217 for currency.  The context of use may additionally require a code from a particular system.
     */
    code?: string | undefined;
    _code?: Element | undefined;
    /**
     * How the value should be understood and represented - whether the actual value is greater or less than the stated value due to measurement issues; e.g. if the comparator is "<" , then the real value is < stated value.
     */
    comparator?: ('<' | '<=' | '>=' | '>' | 'ad') | undefined;
    _comparator?: Element | undefined;
    /**
     * The identification of the system that provides the coded form of the unit.
     */
    system?: string | undefined;
    _system?: Element | undefined;
    /**
     * A human-readable form of the unit.
     */
    unit?: string | undefined;
    _unit?: Element | undefined;
    /**
     * The implicit precision in the value should always be honored. Monetary values have their own rules for handling precision (refer to standard accounting text books).
     */
    value?: number | undefined;
}
/**
 * A set of ordered Quantities defined by a low and high limit.
 */
export interface Range extends DataType {
    /**
     * If the high element is missing, the high boundary is not known.
     */
    high?: Quantity | undefined;
    /**
     * If the low element is missing, the low boundary is not known.
     */
    low?: Quantity | undefined;
}
/**
 * A relationship of two Quantity values - expressed as a numerator and a denominator.
 */
export interface Ratio extends DataType {
    /**
     * The value of the denominator.
     */
    denominator?: Quantity | undefined;
    /**
     * The value of the numerator.
     */
    numerator?: Quantity | undefined;
}
/**
 * A range of ratios expressed as a low and high numerator and a denominator.
 */
export interface RatioRange extends DataType {
    /**
     * The value of the denominator.
     */
    denominator?: Quantity | undefined;
    /**
     * If the high element is missing, the high boundary is not known.
     */
    highNumerator?: Quantity | undefined;
    /**
     * If the low element is missing, the low boundary is not known.
     */
    lowNumerator?: Quantity | undefined;
}
/**
 * A reference from one resource to another.
 */
export interface Reference extends DataType {
    /**
     * This is generally not the same as the Resource.text of the referenced resource.  The purpose is to identify what's being referenced, not to fully describe it.
     */
    display?: string | undefined;
    _display?: Element | undefined;
    /**
     * When an identifier is provided in place of a reference, any system processing the reference will only be able to resolve the identifier to a reference if it understands the business context in which the identifier is used. Sometimes this is global (e.g. a national identifier) but often it is not. For this reason, none of the useful mechanisms described for working with references (e.g. chaining, includes) are possible, nor should servers be expected to be able resolve the reference. Servers may accept an identifier based reference untouched, resolve it, and/or reject it - see CapabilityStatement.rest.resource.referencePolicy.
     * When both an identifier and a literal reference are provided, the literal reference is preferred. Applications processing the resource are allowed - but not required - to check that the identifier matches the literal reference
     * Applications converting a logical reference to a literal reference may choose to leave the logical reference present, or remove it.
     * Reference is intended to point to a structure that can potentially be expressed as a FHIR resource, though there is no need for it to exist as an actual FHIR resource instance - except in as much as an application wishes to actual find the target of the reference. The content referred to be the identifier must meet the logical constraints implied by any limitations on what resource types are permitted for the reference.  For example, it would not be legitimate to send the identifier for a drug prescription if the type were Reference(Observation|DiagnosticReport).  One of the use-cases for Reference.identifier is the situation where no FHIR representation exists (where the type is Reference (Any).
     * This element only allows for a single identifier. In the case where additional identifers are required, use the [additional identifiers](extension-additionalidentifier.html) extension.
     */
    identifier?: Identifier | undefined;
    /**
     * Using absolute URLs provides a stable scalable approach suitable for a cloud/web context, while using relative/logical references provides a flexible approach suitable for use when trading across closed eco-system boundaries.   Absolute URLs do not need to point to a FHIR RESTful server, though this is the preferred approach. If the URL conforms to the structure "[type]/[id]" then it should be assumed that the reference is to a FHIR RESTful server.
     */
    reference?: string | undefined;
    _reference?: Element | undefined;
    /**
     * This element is used to indicate the type of  the target of the reference. This may be used which ever of the other elements are populated (or not). In some cases, the type of the target may be determined by inspection of the reference (e.g. a RESTful URL) or by resolving the target of the reference; if both the type and a reference is provided, the reference SHALL resolve to a resource of the same type as that specified.
     */
    type?: string | undefined;
    _type?: Element | undefined;
}
/**
 * Related artifacts such as additional documentation, justification, or bibliographic references.
 */
export interface RelatedArtifact extends DataType {
    /**
     * Additional structured information about citations should be captured as extensions.
     */
    citation?: string | undefined;
    _citation?: Element | undefined;
    /**
     * Provides additional classifiers of the related artifact.
     */
    classifier?: CodeableConcept[] | undefined;
    /**
     * A brief description of the document or knowledge resource being referenced, suitable for display to a consumer.
     */
    display?: string | undefined;
    _display?: Element | undefined;
    /**
     * The document being referenced, represented as an attachment. This is exclusive with the resource element.
     */
    document?: Attachment | undefined;
    /**
     * A short label that can be used to reference the citation from elsewhere in the containing artifact, such as a footnote index.
     */
    label?: string | undefined;
    _label?: Element | undefined;
    /**
     * The date of publication of the artifact being referred to.
     */
    publicationDate?: string | undefined;
    _publicationDate?: Element | undefined;
    /**
     * The publication status of the artifact being referred to.
     */
    publicationStatus?: ('draft' | 'active' | 'retired' | 'unknown') | undefined;
    _publicationStatus?: Element | undefined;
    /**
     * If the type is predecessor, this is a reference to the succeeding knowledge resource. If the type is successor, this is a reference to the prior knowledge resource.
     */
    resource?: string | undefined;
    _resource?: Element | undefined;
    /**
     * If both resource and resourceReference are present, they SHOULD be consistent and reference the same resource. Although relatedArtifact is intended to support references to definitional resources, there are cases where non-definitional resources can be definitional (such as Location where the kind is mode). Related artifacts SHOULD be used to reference definitional resources, and profiles SHOULD be used to make that explicit for particular use cases.
     */
    resourceReference?: Reference | undefined;
    /**
     * The presence of both sides of a relationship type (e.g. successor and predecessor) is required to support use cases where one side of a relationship is not represented in FHIR. However, this feature SHALL NOT be used to create bi-directional resource links in FHIR instances. Specifically, following the methodology of "new points to old" and "many points to one", when using related artifact elements to describe and reference FHIR resources, the type element SHALL be drawn from the fhir-related-artifact-type ValueSet.
     */
    type:
        | 'documentation'
        | 'justification'
        | 'citation'
        | 'predecessor'
        | 'successor'
        | 'derived-from'
        | 'depends-on'
        | 'composed-of'
        | 'part-of'
        | 'amends'
        | 'amended-with'
        | 'appends'
        | 'appended-with'
        | 'cites'
        | 'cited-by'
        | 'comments-on'
        | 'comment-in'
        | 'contains'
        | 'contained-in'
        | 'corrects'
        | 'correction-in'
        | 'replaces'
        | 'replaced-with'
        | 'retracts'
        | 'retracted-by'
        | 'signs'
        | 'similar-to'
        | 'supports'
        | 'supported-with'
        | 'transforms'
        | 'transformed-into'
        | 'transformed-with'
        | 'documents'
        | 'specification-of'
        | 'created-with'
        | 'cite-as';
    _type?: Element | undefined;
}
/**
 * A series of measurements taken by a device, with upper and lower limits. There may be more than one dimension in the data.
 */
export interface SampledData extends DataType {
    /**
     * Data may be missing if it is omitted for summarization purposes. In general, data is required for any actual use of a SampledData.
     */
    data?: string | undefined;
    _data?: Element | undefined;
    /**
     * If there is more than one dimension, the code for the type of data will define the meaning of the dimensions (typically ECG data).
     */
    dimensions: number;
    /**
     * A correction factor that is applied to the sampled data points before they are added to the origin.
     */
    factor?: number | undefined;
    /**
     * This is usually a whole number.
     */
    interval: number;
    /**
     * The measurement unit in which the sample interval is expressed.
     */
    intervalUnit: string;
    _intervalUnit?: Element | undefined;
    /**
     * The lower limit of detection of the measured points. This is needed if any of the data points have the value "L" (lower than detection limit).
     */
    lowerLimit?: number | undefined;
    /**
     * The base quantity that a measured value of zero represents. In addition, this provides the units of the entire measurement series.
     */
    origin: Quantity;
    /**
     * The upper limit of detection of the measured points. This is needed if any of the data points have the value "U" (higher than detection limit).
     */
    upperLimit?: number | undefined;
}
/**
 * A signature along with supporting context. The signature may be a digital signature that is cryptographic in nature, or some other signature acceptable to the domain. This other signature may be as simple as a graphical image representing a hand-written signature, or a signature ceremony Different signature approaches have different utilities.
 */
export interface Signature extends DataType {
    /**
     * Where the signature type is an XML DigSig, the signed content is a FHIR Resource(s), the signature is of the XML form of the Resource(s) using  XML-Signature (XMLDIG) "Detached Signature" form.
     */
    data?: string | undefined;
    _data?: Element | undefined;
    /**
     * The party that can't sign. For example a child.
     */
    onBehalfOf?: Reference | undefined;
    /**
     * A mime type that indicates the technical format of the signature. Important mime types are application/signature+xml for X ML DigSig, application/jose for JWS, and image/* for a graphical image of a signature, etc.
     */
    sigFormat?: string | undefined;
    _sigFormat?: Element | undefined;
    /**
     * "xml", "json" and "ttl" are allowed, which describe the simple encodings described in the specification (and imply appropriate bundle support). Otherwise, mime types are legal here.
     */
    targetFormat?: string | undefined;
    _targetFormat?: Element | undefined;
    /**
     * Examples include attesting to: authorship, correct transcription, and witness of specific event. Also known as a &quot;Commitment Type Indication&quot;.
     */
    type?: Coding[] | undefined;
    /**
     * This should agree with the information in the signature.
     */
    when?: string | undefined;
    _when?: Element | undefined;
    /**
     * This should agree with the information in the signature.
     */
    who?: Reference | undefined;
}
/**
 * A set of rules that describe when the event is scheduled.
 */
export interface TimingRepeat extends Element {
    /**
     * Either a duration for the length of the timing schedule, a range of possible length, or outer bounds for start and/or end limits of the timing schedule.
     */
    boundsDuration?: Duration | undefined;
    /**
     * Either a duration for the length of the timing schedule, a range of possible length, or outer bounds for start and/or end limits of the timing schedule.
     */
    boundsRange?: Range | undefined;
    /**
     * Either a duration for the length of the timing schedule, a range of possible length, or outer bounds for start and/or end limits of the timing schedule.
     */
    boundsPeriod?: Period | undefined;
    /**
     * If you have both bounds and count, then this should be understood as within the bounds period, until count times happens.
     */
    count?: number | undefined;
    /**
     * If present, indicates that the count is a range - so to perform the action between [count] and [countMax] times.
     */
    countMax?: number | undefined;
    /**
     * If no days are specified, the action is assumed to happen every day as otherwise specified.
     */
    dayOfWeek?: ('mon' | 'tue' | 'wed' | 'thu' | 'fri' | 'sat' | 'sun')[] | undefined;
    _dayOfWeek?: Element[] | undefined;
    /**
     * For some events the duration is part of the definition of the event (e.g. IV infusions, where the duration is implicit in the specified quantity and rate). For others, it's part of the timing specification (e.g. exercise).
     */
    duration?: number | undefined;
    /**
     * For some events the duration is part of the definition of the event (e.g. IV infusions, where the duration is implicit in the specified quantity and rate). For others, it's part of the timing specification (e.g. exercise).
     */
    durationMax?: number | undefined;
    /**
     * The units of time for the duration, in UCUM units
     * Normal practice is to use the 'mo' code as a calendar month when calculating the next occurrence.
     */
    durationUnit?: ('s' | 'min' | 'h' | 'd' | 'wk' | 'mo' | 'a') | undefined;
    _durationUnit?: Element | undefined;
    /**
     * The number of times to repeat the action within the specified period. If frequencyMax is present, this element indicates the lower bound of the allowed range of the frequency.
     */
    frequency?: number | undefined;
    /**
     * If present, indicates that the frequency is a range - so to repeat between [frequency] and [frequencyMax] times within the period or period range.
     */
    frequencyMax?: number | undefined;
    /**
     * The number of minutes from the event. If the event code does not indicate whether the minutes is before or after the event, then the offset is assumed to be after the event.
     */
    offset?: number | undefined;
    /**
     * Indicates the duration of time over which repetitions are to occur; e.g. to express "3 times per day", 3 would be the frequency and "1 day" would be the period. If periodMax is present, this element indicates the lower bound of the allowed range of the period length.
     */
    period?: number | undefined;
    /**
     * If present, indicates that the period is a range from [period] to [periodMax], allowing expressing concepts such as "do this once every 3-5 days.
     */
    periodMax?: number | undefined;
    /**
     * The units of time for the period in UCUM units
     * Normal practice is to use the 'mo' code as a calendar month when calculating the next occurrence.
     */
    periodUnit?: ('s' | 'min' | 'h' | 'd' | 'wk' | 'mo' | 'a') | undefined;
    _periodUnit?: Element | undefined;
    /**
     * When time of day is specified, it is inferred that the action happens every day (as filtered by dayofWeek) on the specified times. If there's a timeOfDay, there cannot be a when, or vice versa.
     */
    timeOfDay?: string[] | undefined;
    _timeOfDay?: Element[] | undefined;
    /**
     * When more than one event is listed, the event is tied to the union of the specified events.
     */
    when?: string[] | undefined;
    _when?: Element[] | undefined;
}
/**
 * Specifies an event that may occur multiple times. Timing schedules are used to record when things are planned, expected or requested to occur. The most common usage is in dosage instructions for medications. They are also used when planning care of various kinds, and may be used for reporting the schedule to which past regular activities were carried out.
 */
export interface Timing extends BackboneType {
    /**
     * BID etc. are defined as 'at institutionally specified times'. For example, an institution may choose that BID is "always at 7am and 6pm".  If it is inappropriate for this choice to be made, the code BID should not be used. Instead, a distinct organization-specific code should be used in place of the HL7-defined BID code and/or a structured representation should be used (in this case, specifying the two event times).
     */
    code?: CodeableConcept | undefined;
    /**
     * Identifies specific times when the event occurs.
     */
    event?: string[] | undefined;
    _event?: Element[] | undefined;
    /**
     * A set of rules that describe when the event is scheduled.
     */
    repeat?: TimingRepeat | undefined;
}
/**
 * A description of a triggering event. Triggering events can be named events, data events, or periodic, as determined by the type element.
 */
export interface TriggerDefinition extends DataType {
    /**
     * A code that identifies the event.
     */
    code?: CodeableConcept | undefined;
    /**
     * This element can be only be specified for data type triggers and provides additional semantics for the trigger. The context available within the condition is based on the type of data event. For all events, the current resource will be available as context. In addition, for modification events, the previous resource will also be available. The expression may be inlined, or may be a simple absolute URI, which is a reference to a named expression within a logic library referenced by a library element or extension within the containing resource. If the expression is a FHIR Path expression, it evaluates in the context of a resource of one of the type identified in the data requirement, and may also refer to the variable %previous for delta comparisons on events of type data-changed, data-modified, and data-deleted which will always have the same type.
     */
    condition?: Expression | undefined;
    /**
     * This element shall be present for any data type trigger.
     */
    data?: DataRequirement[] | undefined;
    /**
     * An event name can be provided for all event types, but is required for named events. If a name is provided for a type other than named events, it is considered to be a shorthand for the semantics described by the formal description of the event.
     */
    name?: string | undefined;
    _name?: Element | undefined;
    /**
     * A reference to a SubscriptionTopic resource that defines the event. If this element is provided, no other information about the trigger definition may be supplied.
     */
    subscriptionTopic?: string | undefined;
    _subscriptionTopic?: Element | undefined;
    /**
     * The timing of the event (if this is a periodic trigger).
     */
    timingTiming?: Timing | undefined;
    /**
     * The timing of the event (if this is a periodic trigger).
     */
    timingReference?: Reference | undefined;
    /**
     * The timing of the event (if this is a periodic trigger).
     */
    timingDate?: string | undefined;
    _timingDate?: Element | undefined;
    /**
     * The timing of the event (if this is a periodic trigger).
     */
    timingDateTime?: string | undefined;
    _timingDateTime?: Element | undefined;
    /**
     * The type of triggering event.
     */
    type:
        | 'named-event'
        | 'periodic'
        | 'data-changed'
        | 'data-added'
        | 'data-modified'
        | 'data-removed'
        | 'data-accessed'
        | 'data-access-ended';
    _type?: Element | undefined;
}
/**
 * Specifies clinical/business/etc. metadata that can be used to retrieve, index and/or categorize an artifact. This metadata can either be specific to the applicable population (e.g., age category, DRG) or the specific context of care (e.g., venue, care setting, provider of care).
 */
export interface UsageContext extends DataType {
    /**
     * A code that identifies the type of context being specified by this usage context.
     */
    code: Coding;
    /**
     * A value that defines the context specified in this context of use. The interpretation of the value is defined by the code.
     */
    valueCodeableConcept?: CodeableConcept | undefined;
    /**
     * A value that defines the context specified in this context of use. The interpretation of the value is defined by the code.
     */
    valueQuantity?: Quantity | undefined;
    /**
     * A value that defines the context specified in this context of use. The interpretation of the value is defined by the code.
     */
    valueRange?: Range | undefined;
    /**
     * A value that defines the context specified in this context of use. The interpretation of the value is defined by the code.
     */
    valueReference?: Reference | undefined;
}
/**
 * Virtual Service Contact Details.
 */
export interface VirtualServiceDetail extends DataType {
    /**
     * This web address can be used to provide additional details on the call, such as alternative/regional call in numbers, or other associated services.
     */
    additionalInfo?: string[] | undefined;
    _additionalInfo?: Element[] | undefined;
    /**
     * What address or number needs to be used for a user to connect to the virtual service to join. The channelType informs as to which datatype is appropriate to use (requires knowledge of the specific type).
     */
    addressUrl?: string | undefined;
    _addressUrl?: Element | undefined;
    /**
     * What address or number needs to be used for a user to connect to the virtual service to join. The channelType informs as to which datatype is appropriate to use (requires knowledge of the specific type).
     */
    addressString?: string | undefined;
    _addressString?: Element | undefined;
    /**
     * What address or number needs to be used for a user to connect to the virtual service to join. The channelType informs as to which datatype is appropriate to use (requires knowledge of the specific type).
     */
    addressContactPoint?: ContactPoint | undefined;
    /**
     * What address or number needs to be used for a user to connect to the virtual service to join. The channelType informs as to which datatype is appropriate to use (requires knowledge of the specific type).
     */
    addressExtendedContactDetail?: ExtendedContactDetail | undefined;
    /**
     * As this type defines what the virtual service is, it then defines what address and additional information might be relevant.
     * Some profiles could be used for specific types to define what the other values could/should be.
     */
    channelType?: Coding | undefined;
    /**
     * Some services have only a limitted number of participants permitted to connect at a time, this is typically used in "shared" virtual services that you might find on location resources.
     * Typically on-demand services might not have these restrictions.
     */
    maxParticipants?: number | undefined;
    /**
     * Some services require a session key to be able to access the service once connected.
     * This could be in cases where a shared number is used, and a session key is added to put into a private line, or to identify the participant in a call.
     */
    sessionKey?: string | undefined;
    _sessionKey?: Element | undefined;
}
/**
 * Typically. this may be some form of insurance, internal charges, or self-pay.
 * Local or jurisdictional business rules may determine which coverage covers which types of billable items charged to the account, and in which order.
 * Where the order is important, a local/jurisdictional extension may be defined to specify the order for the type of charge.
 */
export interface AccountCoverage extends BackboneElement {
    /**
     * The party(s) that contribute to payment (or part of) of the charges applied to this account (including self-pay).
     * A coverage may only be responsible for specific types of charges, and the sequence of the coverages in the account could be important when processing billing.
     */
    coverage: Reference;
    /**
     * It is common in some jurisdictions for there to be multiple coverages allocated to an account, and a sequence is required to order the settling of the account (often with insurance claiming).
     */
    priority?: number | undefined;
}
/**
 * The parties responsible for balancing the account if other payment options fall short.
 */
export interface AccountGuarantor extends BackboneElement {
    /**
     * A guarantor may be placed on credit hold or otherwise have their role temporarily suspended.
     */
    onHold?: boolean | undefined;
    _onHold?: Element | undefined;
    /**
     * The entity who is responsible.
     */
    party: Reference;
    /**
     * The timeframe during which the guarantor accepts responsibility for the account.
     */
    period?: Period | undefined;
}
/**
 * Other associated accounts related to this account.
 */
export interface AccountRelatedAccount extends BackboneElement {
    /**
     * Reference to an associated Account.
     */
    account: Reference;
    /**
     * Relationship of the associated Account.
     */
    relationship?: CodeableConcept | undefined;
}
/**
 * The calculated account balances - these are calculated and processed by the finance system.
 * The balances with a `term` that is not current are usually generated/updated by an invoicing or similar process.
 */
export interface AccountBalance extends BackboneElement {
    /**
     * Who is expected to pay this part of the balance.
     */
    aggregate?: CodeableConcept | undefined;
    /**
     * The actual balance value calculated for the age defined in the term property.
     */
    amount: Money;
    /**
     * The amount is only an estimated value - this is likely common for `current` term balances, but not with known terms (that were generated by a backend process).
     */
    estimate?: boolean | undefined;
    _estimate?: Element | undefined;
    /**
     * The term of the account balances - The balance value is the amount that was outstanding for this age.
     */
    term?: CodeableConcept | undefined;
}
/**
 * A financial tool for tracking value accrued for a particular purpose.  In the healthcare field, used to track charges for a patient, cost centers, etc.
 */
export interface Account extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'Account';
    /**
     * The calculated account balances - these are calculated and processed by the finance system.
     * The balances with a `term` that is not current are usually generated/updated by an invoicing or similar process.
     */
    balance?: AccountBalance[] | undefined;
    /**
     * The BillingStatus tracks the lifecycle of the account through the billing process. It indicates how transactions are treated when they are allocated to the account.
     */
    billingStatus?: CodeableConcept | undefined;
    /**
     * The total(s) are not typically updated using the restful API but more often by back-end processes included in a typical finance system.
     */
    calculatedAt?: string | undefined;
    _calculatedAt?: Element | undefined;
    /**
     * Typically. this may be some form of insurance, internal charges, or self-pay.
     * Local or jurisdictional business rules may determine which coverage covers which types of billable items charged to the account, and in which order.
     * Where the order is important, a local/jurisdictional extension may be defined to specify the order for the type of charge.
     */
    coverage?: AccountCoverage[] | undefined;
    /**
     * The default currency for the account.
     */
    currency?: CodeableConcept | undefined;
    /**
     * Provides additional information about what the account tracks and how it is used.
     */
    description?: string | undefined;
    _description?: Element | undefined;
    /**
     * The parties responsible for balancing the account if other payment options fall short.
     */
    guarantor?: AccountGuarantor[] | undefined;
    /**
     * Unique identifier used to reference the account.  Might or might not be intended for human use (e.g. credit card number).
     */
    identifier?: Identifier[] | undefined;
    /**
     * Name used for the account when displaying it to humans in reports, etc.
     */
    name?: string | undefined;
    _name?: Element | undefined;
    /**
     * Indicates the service area, hospital, department, etc. with responsibility for managing the Account.
     */
    owner?: Reference | undefined;
    /**
     * Other associated accounts related to this account.
     */
    relatedAccount?: AccountRelatedAccount[] | undefined;
    /**
     * It is possible for transactions to be posted outside the service period, as long as the service was provided within the defined service period.
     */
    servicePeriod?: Period | undefined;
    /**
     * This element is labeled as a modifier because the status contains the codes inactive and entered-in-error that mark the Account as not currently valid.
     */
    status: 'active' | 'inactive' | 'entered-in-error' | 'on-hold' | 'unknown';
    _status?: Element | undefined;
    /**
     * Accounts can be applied to non-patients for tracking other non-patient related activities, such as group services (patients not tracked, and costs charged to another body), or might not be allocated.
     */
    subject?: Reference[] | undefined;
    /**
     * Categorizes the account for reporting and searching purposes.
     */
    type?: CodeableConcept | undefined;
}
/**
 * Indicates who should participate in performing the action described.
 */
export interface ActivityDefinitionParticipant extends BackboneElement {
    /**
     * Indicates how the actor will be involved in the action - author, reviewer, witness, etc.
     */
    function?: CodeableConcept | undefined;
    /**
     * The role the participant should play in performing the described action.
     */
    role?: CodeableConcept | undefined;
    /**
     * The type of participant in the action.
     */
    type?:
        | (
              | 'careteam'
              | 'device'
              | 'group'
              | 'healthcareservice'
              | 'location'
              | 'organization'
              | 'patient'
              | 'practitioner'
              | 'practitionerrole'
              | 'relatedperson'
          )
        | undefined;
    _type?: Element | undefined;
    /**
     * The type of participant in the action.
     */
    typeCanonical?: string | undefined;
    _typeCanonical?: Element | undefined;
    /**
     * When this element is a reference, it SHOULD be a reference to a definitional resource (for example, a location type, rather than a specific location).
     */
    typeReference?: Reference | undefined;
}
/**
 * Dynamic values are applied in the order in which they are defined in the ActivityDefinition. Note that if both a transform and dynamic values are specified, the dynamic values will be applied to the result of the transform.
 */
export interface ActivityDefinitionDynamicValue extends BackboneElement {
    /**
     * The expression may be inlined, or may be a reference to a named expression within a logic library referenced by the library element.
     */
    expression: Expression;
    /**
     * The path attribute contains a [Simple FHIRPath Subset](fhirpath.html#simple) that allows path traversal, but not calculation.
     */
    path: string;
    _path?: Element | undefined;
}
/**
 * This resource allows for the definition of some activity to be performed, independent of a particular patient, practitioner, or other performance context.
 */
export interface ActivityDefinition extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'ActivityDefinition';
    /**
     * The 'date' element may be more recent than the approval date because of minor changes or editorial corrections.
     */
    approvalDate?: string | undefined;
    _approvalDate?: Element | undefined;
    /**
     * If a CodeableConcept is present, it indicates the pre-condition for performing the service.  For example "pain", "on flare-up", etc.
     */
    asNeededBoolean?: boolean | undefined;
    _asNeededBoolean?: Element | undefined;
    /**
     * If a CodeableConcept is present, it indicates the pre-condition for performing the service.  For example "pain", "on flare-up", etc.
     */
    asNeededCodeableConcept?: CodeableConcept | undefined;
    /**
     * An individiual or organization primarily involved in the creation and maintenance of the content.
     */
    author?: ContactDetail[] | undefined;
    /**
     * Only used if not implicit in the code found in ServiceRequest.type.
     */
    bodySite?: CodeableConcept[] | undefined;
    /**
     * Tends to be less relevant for activities involving particular products.
     */
    code?: CodeableConcept | undefined;
    /**
     * May be a web site, an email address, a telephone number, etc.
     */
    contact?: ContactDetail[] | undefined;
    /**
     * A copyright statement relating to the activity definition and/or its contents. Copyright statements are generally legal restrictions on the use and publishing of the activity definition.
     */
    copyright?: string | undefined;
    _copyright?: Element | undefined;
    /**
     * Note that this is not the same as the resource last-modified-date, since the resource may be a secondary representation of the activity definition. Additional specific dates may be added as extensions or be found by consulting Provenances associated with past versions of the resource.
     */
    date?: string | undefined;
    _date?: Element | undefined;
    /**
     * This description can be used to capture details such as comments about misuse, instructions for clinical use and interpretation, literature references, examples from the paper world, etc. It is not a rendering of the activity definition as conveyed in the 'text' field of the resource itself. This item SHOULD be populated unless the information is available from context (e.g. the language of the activity definition is presumed to be the predominant language in the place the activity definition was created).
     */
    description?: string | undefined;
    _description?: Element | undefined;
    /**
     * This element is not intended to be used to communicate a decision support response to cancel an order in progress. That should be done with the "remove" type of a PlanDefinition or RequestOrchestration.
     */
    doNotPerform?: boolean | undefined;
    _doNotPerform?: Element | undefined;
    /**
     * If a dosage instruction is used, the definition should not specify timing or quantity.
     */
    dosage?: Dosage[] | undefined;
    /**
     * Dynamic values are applied in the order in which they are defined in the ActivityDefinition. Note that if both a transform and dynamic values are specified, the dynamic values will be applied to the result of the transform.
     */
    dynamicValue?: ActivityDefinitionDynamicValue[] | undefined;
    /**
     * An individual or organization primarily responsible for internal coherence of the content.
     */
    editor?: ContactDetail[] | undefined;
    /**
     * The effective period for a activity definition  determines when the content is applicable for usage and is independent of publication and review dates. For example, a measure intended to be used for the year 2016 might be published in 2015.
     */
    effectivePeriod?: Period | undefined;
    /**
     * An individual or organization responsible for officially endorsing the content for use in some setting.
     */
    endorser?: ContactDetail[] | undefined;
    /**
     * Allows filtering of activity definitions that are appropriate for use versus not.
     */
    experimental?: boolean | undefined;
    _experimental?: Element | undefined;
    /**
     * Typically, this is used for identifiers that can go in an HL7 V3 II (instance identifier) data type, and can then identify this activity definition outside of FHIR, where it is not possible to use the logical URI.
     */
    identifier?: Identifier[] | undefined;
    /**
     * Indicates the level of authority/intentionality associated with the activity and where the request should fit into the workflow chain.
     */
    intent?:
        | (
              | 'proposal'
              | 'plan'
              | 'directive'
              | 'order'
              | 'original-order'
              | 'reflex-order'
              | 'filler-order'
              | 'instance-order'
              | 'option'
          )
        | undefined;
    _intent?: Element | undefined;
    /**
     * It may be possible for the activity definition to be used in jurisdictions other than those for which it was originally designed or intended.
     */
    jurisdiction?: CodeableConcept[] | undefined;
    /**
     * May determine what types of extensions are permitted.
     */
    kind?: string | undefined;
    _kind?: Element | undefined;
    /**
     * If specified, this date follows the original approval date.
     */
    lastReviewDate?: string | undefined;
    _lastReviewDate?: Element | undefined;
    /**
     * A reference to a Library resource containing any formal logic used by the activity definition.
     */
    library?: string[] | undefined;
    _library?: Element[] | undefined;
    /**
     * May reference a specific clinical location or may just identify a type of location.
     */
    location?: CodeableReference | undefined;
    /**
     * The name is not expected to be globally unique. The name should be a simple alphanumeric type name to ensure that it is machine-processing friendly.
     */
    name?: string | undefined;
    _name?: Element | undefined;
    /**
     * Defines observation requirements for the action to be performed, such as body weight or surface area.
     */
    observationRequirement?: string[] | undefined;
    _observationRequirement?: Element[] | undefined;
    /**
     * Defines the observations that are expected to be produced by the action.
     */
    observationResultRequirement?: string[] | undefined;
    _observationResultRequirement?: Element[] | undefined;
    /**
     * Indicates who should participate in performing the action described.
     */
    participant?: ActivityDefinitionParticipant[] | undefined;
    /**
     * Indicates how quickly the activity  should be addressed with respect to other requests.
     */
    priority?: ('routine' | 'urgent' | 'asap' | 'stat') | undefined;
    _priority?: Element | undefined;
    /**
     * Identifies the food, drug or other product being consumed or supplied in the activity.
     */
    productReference?: Reference | undefined;
    /**
     * Identifies the food, drug or other product being consumed or supplied in the activity.
     */
    productCodeableConcept?: CodeableConcept | undefined;
    /**
     * A profile to which the target of the activity definition is expected to conform.
     */
    profile?: string | undefined;
    _profile?: Element | undefined;
    /**
     * Usually an organization but may be an individual. The publisher (or steward) of the activity definition is the organization or individual primarily responsible for the maintenance and upkeep of the activity definition. This is not necessarily the same individual or organization that developed and initially authored the content. The publisher is the primary point of contact for questions or issues with the activity definition. This item SHOULD be populated unless the information is available from context.
     */
    publisher?: string | undefined;
    _publisher?: Element | undefined;
    /**
     * This element does not describe the usage of the activity definition. Instead, it provides traceability of ''why'' the resource is either needed or ''why'' it is defined as it is.  This may be used to point to source materials or specifications that drove the structure of this activity definition.
     */
    purpose?: string | undefined;
    _purpose?: Element | undefined;
    /**
     * Identifies the quantity expected to be consumed at once (per dose, per meal, etc.).
     */
    quantity?: Quantity | undefined;
    /**
     * Each related artifact is either an attachment, or a reference to another resource, but not both.
     */
    relatedArtifact?: RelatedArtifact[] | undefined;
    /**
     * An individual or organization primarily responsible for review of some aspect of the content.
     */
    reviewer?: ContactDetail[] | undefined;
    /**
     * Defines specimen requirements for the action to be performed, such as required specimens for a lab test.
     */
    specimenRequirement?: string[] | undefined;
    _specimenRequirement?: Element[] | undefined;
    /**
     * Allows filtering of activity definitions that are appropriate for use versus not.
     */
    status: 'draft' | 'active' | 'retired' | 'unknown';
    _status?: Element | undefined;
    /**
     * Note that the choice of canonical for the subject element was introduced in R4B to support pharmaceutical quality use cases. To ensure as much backwards-compatibility as possible, it is recommended to only use the new canonical type with these use cases.
     */
    subjectCodeableConcept?: CodeableConcept | undefined;
    /**
     * Note that the choice of canonical for the subject element was introduced in R4B to support pharmaceutical quality use cases. To ensure as much backwards-compatibility as possible, it is recommended to only use the new canonical type with these use cases.
     */
    subjectReference?: Reference | undefined;
    /**
     * Note that the choice of canonical for the subject element was introduced in R4B to support pharmaceutical quality use cases. To ensure as much backwards-compatibility as possible, it is recommended to only use the new canonical type with these use cases.
     */
    subjectCanonical?: string | undefined;
    _subjectCanonical?: Element | undefined;
    /**
     * An explanatory or alternate title for the activity definition giving additional information about its content.
     */
    subtitle?: string | undefined;
    _subtitle?: Element | undefined;
    /**
     * The intent of the timing element is to provide timing when the action should be performed. As a definitional resource, this timing is interpreted as part of an apply operation so that the timing of the result actions in a CarePlan or RequestOrchestration, for example, would be specified by evaluating the timing definition in the context of the apply and setting the resulting timing on the appropriate elements of the target resource. If the timing is an Age, the activity is expected to be performed when the subject is the given Age. When the timing is a Duration, the activity is expected to be performed within the specified duration from the apply. When the timing is a Range, it may be a range of Ages or Durations, providing a range for the expected timing of the resulting activity. When the timing is a Timing, it is establishing a schedule for the timing of the resulting activity.
     */
    timingTiming?: Timing | undefined;
    /**
     * The intent of the timing element is to provide timing when the action should be performed. As a definitional resource, this timing is interpreted as part of an apply operation so that the timing of the result actions in a CarePlan or RequestOrchestration, for example, would be specified by evaluating the timing definition in the context of the apply and setting the resulting timing on the appropriate elements of the target resource. If the timing is an Age, the activity is expected to be performed when the subject is the given Age. When the timing is a Duration, the activity is expected to be performed within the specified duration from the apply. When the timing is a Range, it may be a range of Ages or Durations, providing a range for the expected timing of the resulting activity. When the timing is a Timing, it is establishing a schedule for the timing of the resulting activity.
     */
    timingAge?: Age | undefined;
    /**
     * The intent of the timing element is to provide timing when the action should be performed. As a definitional resource, this timing is interpreted as part of an apply operation so that the timing of the result actions in a CarePlan or RequestOrchestration, for example, would be specified by evaluating the timing definition in the context of the apply and setting the resulting timing on the appropriate elements of the target resource. If the timing is an Age, the activity is expected to be performed when the subject is the given Age. When the timing is a Duration, the activity is expected to be performed within the specified duration from the apply. When the timing is a Range, it may be a range of Ages or Durations, providing a range for the expected timing of the resulting activity. When the timing is a Timing, it is establishing a schedule for the timing of the resulting activity.
     */
    timingRange?: Range | undefined;
    /**
     * The intent of the timing element is to provide timing when the action should be performed. As a definitional resource, this timing is interpreted as part of an apply operation so that the timing of the result actions in a CarePlan or RequestOrchestration, for example, would be specified by evaluating the timing definition in the context of the apply and setting the resulting timing on the appropriate elements of the target resource. If the timing is an Age, the activity is expected to be performed when the subject is the given Age. When the timing is a Duration, the activity is expected to be performed within the specified duration from the apply. When the timing is a Range, it may be a range of Ages or Durations, providing a range for the expected timing of the resulting activity. When the timing is a Timing, it is establishing a schedule for the timing of the resulting activity.
     */
    timingDuration?: Duration | undefined;
    /**
     * This name does not need to be machine-processing friendly and may contain punctuation, white-space, etc.
     */
    title?: string | undefined;
    _title?: Element | undefined;
    /**
     * Descriptive topics related to the content of the activity. Topics provide a high-level categorization of the activity that can be useful for filtering and searching.
     */
    topic?: CodeableConcept[] | undefined;
    /**
     * Note that if both a transform and dynamic values are specified, the dynamic values will be applied to the result of the transform.
     */
    transform?: string | undefined;
    _transform?: Element | undefined;
    /**
     * Can be a urn:uuid: or a urn:oid: but real http: addresses are preferred.  Multiple instances may share the same URL if they have a distinct version.
     * The determination of when to create a new version of a resource (same url, new version) vs. defining a new artifact is up to the author.  Considerations for making this decision are found in [Technical and Business Versions](resource.html#versions).
     * In some cases, the resource can no longer be found at the stated url, but the url itself cannot change. Implementations can use the [meta.source](resource.html#meta) element to indicate where the current master source of the resource can be found.
     */
    url?: string | undefined;
    _url?: Element | undefined;
    /**
     * A detailed description of how the activity definition is used from a clinical perspective.
     */
    usage?: string | undefined;
    _usage?: Element | undefined;
    /**
     * When multiple useContexts are specified, there is no expectation that all or any of the contexts apply.
     */
    useContext?: UsageContext[] | undefined;
    /**
     * There may be different activity definition instances that have the same identifier but different versions.  The version can be appended to the url in a reference to allow a reference to a particular business version of the activity definition with the format [url]|[version]. The version SHOULD NOT contain a '#' - see [Business Version](resource.html#bv-format).
     */
    version?: string | undefined;
    _version?: Element | undefined;
}
/**
 * Describes an actor - a human or an application that plays a role in data exchange, and that may have obligations associated with the role the actor plays.
 */
export interface ActorDefinition extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'ActorDefinition';
    /**
     * The capability statement for the actor (if the concept is applicable).
     */
    capabilities?: string | undefined;
    _capabilities?: Element | undefined;
    /**
     * May be a web site, an email address, a telephone number, etc.
     */
    contact?: ContactDetail[] | undefined;
    /**
     * ...
     */
    copyright?: string | undefined;
    _copyright?: Element | undefined;
    /**
     * The (c) symbol should NOT be included in this string. It will be added by software when rendering the notation. Full details about licensing, restrictions, warrantees, etc. goes in the more general 'copyright' element.
     */
    copyrightLabel?: string | undefined;
    _copyrightLabel?: Element | undefined;
    /**
     * Note that this is not the same as the resource last-modified-date, since the resource may be a secondary representation of the actor definition. Additional specific dates may be added as extensions or be found by consulting Provenances associated with past versions of the resource.
     */
    date?: string | undefined;
    _date?: Element | undefined;
    /**
     * A url that identifies the definition of this actor in another IG (which IG must be listed in the dependencies). This actor inherits all the obligations etc as defined in the other IG.
     */
    derivedFrom?: string[] | undefined;
    _derivedFrom?: Element[] | undefined;
    /**
     * This description can be used for a general description of the actor, and which covers why it was defined.
     */
    description?: string | undefined;
    _description?: Element | undefined;
    /**
     * In general terms, the description desribes what the actor is, and this element describes what the actor does (or is expected to do).
     */
    documentation?: string | undefined;
    _documentation?: Element | undefined;
    /**
     * Allows filtering of actor definitions that are appropriate for use versus not.
     */
    experimental?: boolean | undefined;
    _experimental?: Element | undefined;
    /**
     * A formal identifier that is used to identify this actor definition when it is represented in other formats, or referenced in a specification, model, design or an instance.
     */
    identifier?: Identifier[] | undefined;
    /**
     * It may be possible for the actor definition to be used in jurisdictions other than those for which it was originally designed or intended.
     */
    jurisdiction?: CodeableConcept[] | undefined;
    /**
     * The name is not expected to be globally unique. The name should be a simple alphanumeric type name to ensure that it is machine-processing friendly.
     */
    name?: string | undefined;
    _name?: Element | undefined;
    /**
     * Usually an organization but may be an individual. The publisher (or steward) of the actor definition is the organization or individual primarily responsible for the maintenance and upkeep of the actor definition. This is not necessarily the same individual or organization that developed and initially authored the content. The publisher is the primary point of contact for questions or issues with the actor definition. This item SHOULD be populated unless the information is available from context.
     */
    publisher?: string | undefined;
    _publisher?: Element | undefined;
    /**
     * This element does not describe the usage of the actor definition. Instead, it provides traceability of ''why'' the resource is either needed or ''why'' it is defined as it is.  This may be used to point to source materials or specifications that drove the structure of this actor definition.
     */
    purpose?: string | undefined;
    _purpose?: Element | undefined;
    /**
     * A reference to additional documentation about the actor, but description and documentation.
     */
    reference?: string[] | undefined;
    _reference?: Element[] | undefined;
    /**
     * Allows filtering of actor definitions that are appropriate for use versus not.
     */
    status: 'draft' | 'active' | 'retired' | 'unknown';
    _status?: Element | undefined;
    /**
     * This name does not need to be machine-processing friendly and may contain punctuation, white-space, etc.
     */
    title?: string | undefined;
    _title?: Element | undefined;
    /**
     * Whether the actor represents a human or an appliction.
     */
    type: 'person' | 'system';
    _type?: Element | undefined;
    /**
     * Can be a urn:uuid: or a urn:oid: but real http: addresses are preferred.  Multiple instances may share the same URL if they have a distinct version.
     * The determination of when to create a new version of a resource (same url, new version) vs. defining a new artifact is up to the author.  Considerations for making this decision are found in [Technical and Business Versions](resource.html#versions).
     * In some cases, the resource can no longer be found at the stated url, but the url itself cannot change. Implementations can use the [meta.source](resource.html#meta) element to indicate where the current master source of the resource can be found.
     */
    url?: string | undefined;
    _url?: Element | undefined;
    /**
     * When multiple useContexts are specified, there is no expectation that all or any of the contexts apply.
     */
    useContext?: UsageContext[] | undefined;
    /**
     * There may be different actor definition instances that have the same identifier but different versions.  The version can be appended to the url in a reference to allow a reference to a particular business version of the actor definition with the format [url]|[version]. The version SHOULD NOT contain a '#' - see [Business Version](resource.html#bv-format).
     */
    version?: string | undefined;
    _version?: Element | undefined;
}
/**
 * Characteristics e.g. a product's onset of action.
 */
export interface AdministrableProductDefinitionProperty extends BackboneElement {
    /**
     * The status of characteristic e.g. assigned or pending.
     */
    status?: CodeableConcept | undefined;
    /**
     * A code expressing the type of characteristic.
     */
    type: CodeableConcept;
    /**
     * A value for the characteristic.
     */
    valueCodeableConcept?: CodeableConcept | undefined;
    /**
     * A value for the characteristic.
     */
    valueQuantity?: Quantity | undefined;
    /**
     * A value for the characteristic.
     */
    valueDate?: string | undefined;
    _valueDate?: Element | undefined;
    /**
     * A value for the characteristic.
     */
    valueBoolean?: boolean | undefined;
    _valueBoolean?: Element | undefined;
    /**
     * A value for the characteristic.
     */
    valueAttachment?: Attachment | undefined;
}
/**
 * A species specific time during which consumption of animal product is not appropriate.
 */
export interface AdministrableProductDefinitionRouteOfAdministrationTargetSpeciesWithdrawalPeriod
    extends BackboneElement {
    /**
     * Extra information about the withdrawal period.
     */
    supportingInformation?: string | undefined;
    _supportingInformation?: Element | undefined;
    /**
     * Coded expression for the type of tissue for which the withdrawal period applies, e.g. meat, milk.
     */
    tissue: CodeableConcept;
    /**
     * A value for the time.
     */
    value: Quantity;
}
/**
 * A species for which this route applies.
 */
export interface AdministrableProductDefinitionRouteOfAdministrationTargetSpecies extends BackboneElement {
    /**
     * Coded expression for the species.
     */
    code: CodeableConcept;
    /**
     * A species specific time during which consumption of animal product is not appropriate.
     */
    withdrawalPeriod?: AdministrableProductDefinitionRouteOfAdministrationTargetSpeciesWithdrawalPeriod[] | undefined;
}
/**
 * The path by which the product is taken into or makes contact with the body. In some regions this is referred to as the licenced or approved route. RouteOfAdministration cannot be used when the 'formOf' product already uses MedicinalProductDefinition.route (and vice versa).
 */
export interface AdministrableProductDefinitionRouteOfAdministration extends BackboneElement {
    /**
     * Coded expression for the route.
     */
    code: CodeableConcept;
    /**
     * The first dose (dose quantity) administered can be specified for the product, using a numerical value and its unit of measurement.
     */
    firstDose?: Quantity | undefined;
    /**
     * The maximum dose per day (maximum dose quantity to be administered in any one 24-h period) that can be administered.
     */
    maxDosePerDay?: Quantity | undefined;
    /**
     * The maximum dose per treatment period that can be administered.
     */
    maxDosePerTreatmentPeriod?: Ratio | undefined;
    /**
     * The maximum single dose that can be administered, specified using a numerical value and its unit of measurement.
     */
    maxSingleDose?: Quantity | undefined;
    /**
     * The maximum treatment period during which the product can be administered.
     */
    maxTreatmentPeriod?: Duration | undefined;
    /**
     * A species for which this route applies.
     */
    targetSpecies?: AdministrableProductDefinitionRouteOfAdministrationTargetSpecies[] | undefined;
}
/**
 * A medicinal product in the final form which is suitable for administering to a patient (after any mixing of multiple components, dissolution etc. has been performed).
 */
export interface AdministrableProductDefinition extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'AdministrableProductDefinition';
    /**
     * The dose form of the final product after necessary reconstitution or processing. Contrasts to the manufactured dose form (see ManufacturedItemDefinition). If the manufactured form was 'powder for solution for injection', the administrable dose form could be 'solution for injection' (once mixed with another item having manufactured form 'solvent for solution for injection').
     */
    administrableDoseForm?: CodeableConcept | undefined;
    /**
     * A device that is integral to the medicinal product, in effect being considered as an "ingredient" of the medicinal product. This is not intended for devices that are just co-packaged.
     */
    device?: Reference | undefined;
    /**
     * References a product from which one or more of the constituent parts of that product can be prepared and used as described by this administrable product.  If this administrable product describes the administration of a crushed tablet, the 'formOf' would be the product representing a distribution containing tablets and possibly also a cream.  This is distinct from the 'producedFrom' which refers to the specific components of the product that are used in this preparation, rather than the product as a whole.
     */
    formOf?: Reference[] | undefined;
    /**
     * An identifier for the administrable product.
     */
    identifier?: Identifier[] | undefined;
    /**
     * The ingredients of this administrable medicinal product. This is only needed if the ingredients are not specified either using ManufacturedItemDefiniton (via AdministrableProductDefinition.producedFrom) to state which component items are used to make this, or using by incoming references from the Ingredient resource, to state in detail which substances exist within this. This element allows a basic coded ingredient to be used.
     */
    ingredient?: CodeableConcept[] | undefined;
    /**
     * Indicates the specific manufactured items that are part of the 'formOf' product that are used in the preparation of this specific administrable form.  In some cases, an administrable form might use all of the items from the overall product (or there might only be one item), while in other cases, an administrable form might use only a subset of the items available in the overall product.  For example, an administrable form might involve combining a liquid and a powder available as part of an overall product, but not involve applying the also supplied cream.
     */
    producedFrom?: Reference[] | undefined;
    /**
     * Characteristics e.g. a product's onset of action.
     */
    property?: AdministrableProductDefinitionProperty[] | undefined;
    /**
     * The path by which the product is taken into or makes contact with the body. In some regions this is referred to as the licenced or approved route. RouteOfAdministration cannot be used when the 'formOf' product already uses MedicinalProductDefinition.route (and vice versa).
     */
    routeOfAdministration: AdministrableProductDefinitionRouteOfAdministration[];
    /**
     * Allows filtering of administrable products that are appropriate for use versus not.
     */
    status: 'draft' | 'active' | 'retired' | 'unknown';
    _status?: Element | undefined;
    /**
     * The presentation type in which this item is given to a patient. e.g. for a spray - 'puff' (as in 'contains 100 mcg per puff'), or for a liquid - 'vial' (as in 'contains 5 ml per vial').
     */
    unitOfPresentation?: CodeableConcept | undefined;
}
/**
 * Indicates who or what participated in the adverse event and how they were involved.
 */
export interface AdverseEventParticipant extends BackboneElement {
    /**
     * For example, the physician prescribing a drug, a nurse administering the drug, a device that administered the drug, a witness to the event, or an informant of clinical history.
     */
    actor: Reference;
    /**
     * Distinguishes the type of involvement of the actor in the adverse event, such as contributor or informant.
     */
    function?: CodeableConcept | undefined;
}
/**
 * Information on the possible cause of the event.
 */
export interface AdverseEventSuspectEntityCausality extends BackboneElement {
    /**
     * The method of evaluating the relatedness of the suspected entity to the event.
     */
    assessmentMethod?: CodeableConcept | undefined;
    /**
     * The author of the information on the possible cause of the event.
     */
    author?: Reference | undefined;
    /**
     * The result of the assessment regarding the relatedness of the suspected entity to the event.
     */
    entityRelatedness?: CodeableConcept | undefined;
}
/**
 * Describes the entity that is suspected to have caused the adverse event.
 */
export interface AdverseEventSuspectEntity extends BackboneElement {
    /**
     * Information on the possible cause of the event.
     */
    causality?: AdverseEventSuspectEntityCausality | undefined;
    /**
     * Identifies the actual instance of what caused the adverse event.  May be a substance, medication, medication administration, medication statement or a device.
     */
    instanceCodeableConcept?: CodeableConcept | undefined;
    /**
     * Identifies the actual instance of what caused the adverse event.  May be a substance, medication, medication administration, medication statement or a device.
     */
    instanceReference?: Reference | undefined;
}
/**
 * The contributing factors suspected to have increased the probability or severity of the adverse event.
 */
export interface AdverseEventContributingFactor extends BackboneElement {
    /**
     * The item that is suspected to have increased the probability or severity of the adverse event.
     */
    itemReference?: Reference | undefined;
    /**
     * The item that is suspected to have increased the probability or severity of the adverse event.
     */
    itemCodeableConcept?: CodeableConcept | undefined;
}
/**
 * Preventive actions that contributed to avoiding the adverse event.
 */
export interface AdverseEventPreventiveAction extends BackboneElement {
    /**
     * The action that contributed to avoiding the adverse event.
     */
    itemReference?: Reference | undefined;
    /**
     * The action that contributed to avoiding the adverse event.
     */
    itemCodeableConcept?: CodeableConcept | undefined;
}
/**
 * The ameliorating action taken after the adverse event occured in order to reduce the extent of harm.
 */
export interface AdverseEventMitigatingAction extends BackboneElement {
    /**
     * The ameliorating action taken after the adverse event occured in order to reduce the extent of harm.
     */
    itemReference?: Reference | undefined;
    /**
     * The ameliorating action taken after the adverse event occured in order to reduce the extent of harm.
     */
    itemCodeableConcept?: CodeableConcept | undefined;
}
/**
 * Supporting information relevant to the event.
 */
export interface AdverseEventSupportingInfo extends BackboneElement {
    /**
     * Relevant past history for the subject. In a clinical care context, an example being a patient had an adverse event following a pencillin administration and the patient had a previously documented penicillin allergy. In a clinical trials context, an example is a bunion or rash that was present prior to the study. Additionally, the supporting item can be a document that is relevant to this instance of the adverse event that is not part of the subject's medical history. For example, a clinical note, staff list, or material safety data sheet (MSDS).  Supporting information is not a contributing factor, preventive action, or mitigating action.
     */
    itemReference?: Reference | undefined;
    /**
     * Relevant past history for the subject. In a clinical care context, an example being a patient had an adverse event following a pencillin administration and the patient had a previously documented penicillin allergy. In a clinical trials context, an example is a bunion or rash that was present prior to the study. Additionally, the supporting item can be a document that is relevant to this instance of the adverse event that is not part of the subject's medical history. For example, a clinical note, staff list, or material safety data sheet (MSDS).  Supporting information is not a contributing factor, preventive action, or mitigating action.
     */
    itemCodeableConcept?: CodeableConcept | undefined;
}
/**
 * An event (i.e. any change to current patient status) that may be related to unintended effects on a patient or research subject. The unintended effects may require additional monitoring, treatment, hospitalization, or may result in death. The AdverseEvent resource also extends to potential or avoided events that could have had such effects. There are two major domains where the AdverseEvent resource is expected to be used. One is in clinical care reported adverse events and the other is in reporting adverse events in clinical  research trial management. Given the differences between these two arenas, we recommend consulting the domain specific implementation guides when implementing the AdverseEvent Resource. The implementation guides include specific extensions, value sets and constraints.
 */
export interface AdverseEvent extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'AdverseEvent';
    /**
     * AllergyIntolerance should be used for the initial capture or recording of the individual's propensity to an adverse reaction to a substance.  If an AllergyIntolerance does not exist, then an adverse reaction should be recorded as an AllergyIntolerance. If an AllergyIntolerance does exist and the substance was given, then an adverse reaction should be recorded as an AdverseEvent due to the aberrant workflow.
     */
    actuality: 'actual' | 'potential';
    _actuality?: Element | undefined;
    /**
     * The overall type of event, intended for search and filtering purposes.
     */
    category?: CodeableConcept[] | undefined;
    /**
     * Specific event that occurred or that was averted, such as patient fall, wrong organ removed, or wrong blood transfused.
     */
    code?: CodeableConcept | undefined;
    /**
     * The contributing factors suspected to have increased the probability or severity of the adverse event.
     */
    contributingFactor?: AdverseEventContributingFactor[] | undefined;
    /**
     * Estimated or actual date the AdverseEvent began, in the opinion of the reporter.
     */
    detected?: string | undefined;
    _detected?: Element | undefined;
    /**
     * This will typically be the encounter the event occurred within, but some activities may be initiated prior to or after the official completion of an encounter but still be tied to the context of the encounter. For example, if a medication administration was considered an adverse event because it resulted in a rash, then the encounter when the medication administration was given is the associated encounter. If the patient reports the AdverseEvent during a second encounter, that second encounter is not the associated encounter.
     */
    encounter?: Reference | undefined;
    /**
     * Considered likely or probable or anticipated in the research study.  Whether the reported event matches any of the outcomes for the patient that are considered by the study as known or likely.
     */
    expectedInResearchStudy?: boolean | undefined;
    _expectedInResearchStudy?: Element | undefined;
    /**
     * This is a business identifier, not a resource identifier (see [discussion](resource.html#identifiers)).  It is best practice for the identifier to only appear on a single resource instance, however business practices may occasionally dictate that multiple resource instances with the same identifier can exist - possibly even with different resource types.  For example, multiple Patient and a Person resource instance might share the same social insurance number.
     */
    identifier?: Identifier[] | undefined;
    /**
     * The information about where the adverse event occurred.
     */
    location?: Reference | undefined;
    /**
     * The ameliorating action taken after the adverse event occured in order to reduce the extent of harm.
     */
    mitigatingAction?: AdverseEventMitigatingAction[] | undefined;
    /**
     * Comments made about the adverse event by the performer, subject or other participants.
     */
    note?: Annotation[] | undefined;
    /**
     * The date (and perhaps time) when the adverse event occurred.
     */
    occurrenceDateTime?: string | undefined;
    _occurrenceDateTime?: Element | undefined;
    /**
     * The date (and perhaps time) when the adverse event occurred.
     */
    occurrencePeriod?: Period | undefined;
    /**
     * The date (and perhaps time) when the adverse event occurred.
     */
    occurrenceTiming?: Timing | undefined;
    /**
     * Describes the type of outcome from the adverse event, such as resolved, recovering, ongoing, resolved-with-sequelae, or fatal.
     */
    outcome?: CodeableConcept[] | undefined;
    /**
     * Indicates who or what participated in the adverse event and how they were involved.
     */
    participant?: AdverseEventParticipant[] | undefined;
    /**
     * Preventive actions that contributed to avoiding the adverse event.
     */
    preventiveAction?: AdverseEventPreventiveAction[] | undefined;
    /**
     * The recordedDate represents the date when this particular AdverseEvent record was created in the system, not the date of the most recent update.  The date of the last record modification can be retrieved from the resource metadata.
     */
    recordedDate?: string | undefined;
    _recordedDate?: Element | undefined;
    /**
     * Information on who recorded the adverse event.  May be the patient or a practitioner.
     */
    recorder?: Reference | undefined;
    /**
     * Information about the condition that occurred as a result of the adverse event, such as hives due to the exposure to a substance (for example, a drug or a chemical) or a broken leg as a result of the fall.
     */
    resultingCondition?: Reference[] | undefined;
    /**
     * Using an example, a rash can have an AdverseEvent.resultingCondition.severity = severe, yet an AdverseEvent.seriousness = non-serious.  Alternatively, a medication given with the wrong dose (chemotherapy given with too low of a dose) can have an AdverseEvent.resultingCondition.severity = mild, yet a AdverseEvent.seriousness = serious.  Another example would be a beta blocker clinical trial where patients with asthma should be excluded, yet a patient with asthma was included and had an asthmatic episode where  AdverseEvent.resultingCondition.severity = mild, yet a AdverseEvent.seriousness = serious.
     */
    seriousness?: CodeableConcept | undefined;
    /**
     * This is not the reporting of the event to any regulatory or quality organization.  This is not the outcome of the patient's condition.
     */
    status: 'in-progress' | 'completed' | 'entered-in-error' | 'unknown';
    _status?: Element | undefined;
    /**
     * The research study that the subject is enrolled in.
     */
    study?: Reference[] | undefined;
    /**
     * If AdverseEvent.resultingCondition differs among members of the group, then use Patient as the subject.
     */
    subject: Reference;
    /**
     * Supporting information relevant to the event.
     */
    supportingInfo?: AdverseEventSupportingInfo[] | undefined;
    /**
     * Describes the entity that is suspected to have caused the adverse event.
     */
    suspectEntity?: AdverseEventSuspectEntity[] | undefined;
}
/**
 * Indicates who or what participated in the activities related to the allergy or intolerance and how they were involved.
 */
export interface AllergyIntoleranceParticipant extends BackboneElement {
    /**
     * Indicates who or what participated in the activities related to the allergy or intolerance.
     */
    actor: Reference;
    /**
     * Distinguishes the type of involvement of the actor in the activities related to the allergy or intolerance.
     */
    function?: CodeableConcept | undefined;
}
/**
 * Details about each adverse reaction event linked to exposure to the identified substance.
 */
export interface AllergyIntoleranceReaction extends BackboneElement {
    /**
     * Use the description to provide any details of a particular event of the occurred reaction such as circumstances, reaction specifics, what happened before/after. Information, related to the event, but not describing a particular care should be captured in the note field. For example: at the age of four, the patient was given penicillin for strep throat and subsequently developed severe hives.
     */
    description?: string | undefined;
    _description?: Element | undefined;
    /**
     * Coding of the route of exposure with a terminology should be used wherever possible.
     */
    exposureRoute?: CodeableConcept | undefined;
    /**
     * Manifestation can be expressed as a single word, phrase or brief description. For example: nausea, rash or no reaction. It is preferable that manifestation should be coded with a terminology, where possible. The values entered here may be used to display on an application screen as part of a list of adverse reactions, as recommended in the UK NHS CUI guidelines.  Terminologies commonly used include, but are not limited to, SNOMED CT or ICD10.
     */
    manifestation: CodeableReference[];
    /**
     * Use this field to record information indirectly related to a particular event and not captured in the description. For example: Clinical records are no longer available, recorded based on information provided to the patient by her mother and her mother is deceased.
     */
    note?: Annotation[] | undefined;
    /**
     * Record of the date and/or time of the onset of the Reaction.
     */
    onset?: string | undefined;
    _onset?: Element | undefined;
    /**
     * It is acknowledged that this assessment is very subjective. There may be some specific practice domains where objective scales have been applied. Objective scales can be included in this model as extensions.
     */
    severity?: ('mild' | 'moderate' | 'severe') | undefined;
    _severity?: Element | undefined;
    /**
     * Coding of the specific substance (or pharmaceutical product) with a terminology capable of triggering decision support should be used wherever possible.  The 'code' element allows for the use of a specific substance or pharmaceutical product, or a group or class of substances. In the case of an allergy or intolerance to a class of substances, (for example, "penicillins"), the 'reaction.substance' element could be used to code the specific substance that was identified as having caused the reaction (for example, "amoxycillin"). Duplication of the value in the 'code' and 'reaction.substance' elements is acceptable when a specific substance has been recorded in 'code'.
     */
    substance?: CodeableConcept | undefined;
}
/**
 * Risk of harmful or undesirable physiological response which is specific to an individual and associated with exposure to a substance.
 */
export interface AllergyIntolerance extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'AllergyIntolerance';
    /**
     * This data element has been included because it is currently being captured in some clinical systems. This data can be derived from the substance where coding systems are used, and is effectively redundant in that situation.  When searching on category, consider the implications of AllergyIntolerance resources without a category.  For example, when searching on category = medication, medication allergies that don't have a category valued will not be returned.  Refer to [search](search.html) for more information on how to search category with a :missing modifier to get allergies that don't have a category.  Additionally, category should be used with caution because category can be subjective based on the sender.
     */
    category?: ('food' | 'medication' | 'environment' | 'biologic')[] | undefined;
    _category?: Element[] | undefined;
    /**
     * AllergyIntolerance.clinicalStatus should be present if verificationStatus is not entered-in-error and the AllergyIntolerance.code isn't negated (No Known Allergy, No Drug Allergy, No Food Allergy, No Latex Allergy).
     * Refer to [discussion](extensibility.html#Special-Case) if clincalStatus is missing data.
     * The data type is CodeableConcept because clinicalStatus has some clinical judgment involved, such that there might need to be more specificity than the required FHIR value set allows. For example, a SNOMED coding might allow for additional specificity.
     */
    clinicalStatus?: CodeableConcept | undefined;
    /**
     * It is strongly recommended that this element be populated using a terminology, where possible. For example, some terminologies used include RxNorm, SNOMED CT, DM+D, NDFRT, ICD-9, IDC-10, UNII, and ATC. Plain text should only be used if there is no appropriate terminology available. Additional details can be specified in the text.
     * When a substance or product code is specified for the 'code' element, the "default" semantic context is that this is a positive statement of an allergy or intolerance (depending on the value of the 'type' element, if present) condition to the specified substance/product.  In the corresponding SNOMED CT allergy model, the specified substance/product is the target (destination) of the "Causative agent" relationship.
     * The 'substanceExposureRisk' extension is available as a structured and more flexible alternative to the 'code' element for making positive or negative allergy or intolerance statements.  This extension provides the capability to make "no known allergy" (or "no risk of adverse reaction") statements regarding any coded substance/product (including cases when a pre-coordinated "no allergy to x" concept for that substance/product does not exist).  If the 'substanceExposureRisk' extension is present, the AllergyIntolerance.code element SHALL be omitted.
     */
    code?: CodeableConcept | undefined;
    /**
     * The default criticality value for any propensity to an adverse reaction should be 'Low Risk', indicating at the very least a relative contraindication to deliberate or voluntary exposure to the substance. 'High Risk' is flagged if the clinician has identified a propensity for a more serious or potentially life-threatening reaction, such as anaphylaxis, and implies an absolute contraindication to deliberate or voluntary exposure to the substance. If this element is missing, the criticality is unknown (though it may be known elsewhere).  Systems that capture a severity at the condition level are actually representing the concept of criticality whereas the severity documented at the reaction level is representing the true reaction severity.  Existing systems that are capturing both condition criticality and reaction severity may use the term "severity" to represent both.  Criticality is the worst it could be in the future (i.e. situation-agnostic) whereas severity is situation-dependent.
     */
    criticality?: ('low' | 'high' | 'unable-to-assess') | undefined;
    _criticality?: Element | undefined;
    /**
     * The encounter when the allergy or intolerance was asserted.
     */
    encounter?: Reference | undefined;
    /**
     * This is a business identifier, not a resource identifier (see [discussion](resource.html#identifiers)).  It is best practice for the identifier to only appear on a single resource instance, however business practices may occasionally dictate that multiple resource instances with the same identifier can exist - possibly even with different resource types.  For example, multiple Patient and a Person resource instance might share the same social insurance number.
     */
    identifier?: Identifier[] | undefined;
    /**
     * This date may be replicated by one of the Onset of Reaction dates. Where a textual representation of the date of last occurrence is required e.g. 'In Childhood, '10 years ago' the AllergyIntolerance.note element should be used.
     */
    lastOccurrence?: string | undefined;
    _lastOccurrence?: Element | undefined;
    /**
     * For example: including reason for flagging a seriousness of 'High Risk'; and instructions related to future exposure or administration of the substance, such as administration within an Intensive Care Unit or under corticosteroid cover. The notes should be related to an allergy or intolerance as a condition in general and not related to any particular episode of it. For episode notes and descriptions, use AllergyIntolerance.event.description and  AllergyIntolerance.event.notes.
     */
    note?: Annotation[] | undefined;
    /**
     * Age is generally used when the patient reports an age at which the AllergyIntolerance was noted. Period is generally used to convey an imprecise onset that occurred within the time period.  Range is generally used to convey an imprecise age range (e.g. 4 to 6 years old).
     */
    onsetDateTime?: string | undefined;
    _onsetDateTime?: Element | undefined;
    /**
     * Age is generally used when the patient reports an age at which the AllergyIntolerance was noted. Period is generally used to convey an imprecise onset that occurred within the time period.  Range is generally used to convey an imprecise age range (e.g. 4 to 6 years old).
     */
    onsetAge?: Age | undefined;
    /**
     * Age is generally used when the patient reports an age at which the AllergyIntolerance was noted. Period is generally used to convey an imprecise onset that occurred within the time period.  Range is generally used to convey an imprecise age range (e.g. 4 to 6 years old).
     */
    onsetPeriod?: Period | undefined;
    /**
     * Age is generally used when the patient reports an age at which the AllergyIntolerance was noted. Period is generally used to convey an imprecise onset that occurred within the time period.  Range is generally used to convey an imprecise age range (e.g. 4 to 6 years old).
     */
    onsetRange?: Range | undefined;
    /**
     * Age is generally used when the patient reports an age at which the AllergyIntolerance was noted. Period is generally used to convey an imprecise onset that occurred within the time period.  Range is generally used to convey an imprecise age range (e.g. 4 to 6 years old).
     */
    onsetString?: string | undefined;
    _onsetString?: Element | undefined;
    /**
     * Indicates who or what participated in the activities related to the allergy or intolerance and how they were involved.
     */
    participant?: AllergyIntoleranceParticipant[] | undefined;
    /**
     * The patient who has the allergy or intolerance.
     */
    patient: Reference;
    /**
     * Details about each adverse reaction event linked to exposure to the identified substance.
     */
    reaction?: AllergyIntoleranceReaction[] | undefined;
    /**
     * When onset date is unknown, recordedDate can be used to establish if the allergy or intolerance was present on or before a given date.  If the recordedDate is known and provided by a sending system, it is preferred that the receiving system preserve that recordedDate value. If the recordedDate is not provided by the sending system, the receipt timestamp is sometimes used as the recordedDate.
     */
    recordedDate?: string | undefined;
    _recordedDate?: Element | undefined;
    /**
     * Allergic (typically immune-mediated) reactions have been traditionally regarded as an indicator for potential escalation to significant future risk. Contemporary knowledge suggests that some reactions previously thought to be immune-mediated are, in fact, non-immune, but in some cases can still pose a life threatening risk. It is acknowledged that many clinicians might not be in a position to distinguish the mechanism of a particular reaction. Often the term "allergy" is used rather generically and may overlap with the use of "intolerance" - in practice the boundaries between these two concepts might not be well-defined or understood. This data element is included nevertheless, because many legacy systems have captured this attribute. Immunologic testing may provide supporting evidence for the basis of the reaction and the causative substance, but no tests are 100% sensitive or specific for sensitivity to a particular substance. If, as is commonly the case, it is unclear whether the reaction is due to an allergy or an intolerance, then the type element should be omitted from the resource.
     */
    type?: ('allergy' | 'intolerance') | undefined;
    _type?: Element | undefined;
    /**
     * The data type is CodeableConcept because verificationStatus has some clinical judgment involved, such that there might need to be more specificity than the required FHIR value set allows. For example, a SNOMED coding might allow for additional specificity.
     */
    verificationStatus?: CodeableConcept | undefined;
}
/**
 * List of participants involved in the appointment.
 */
export interface AppointmentParticipant extends BackboneElement {
    /**
     * Where a CareTeam is provided, this does not imply that the entire team is included, just a single member from the group with the appropriate role. Where multiple members are required, please include the CareTeam the required number of times.
     */
    actor?: Reference | undefined;
    /**
     * Participation period of the actor.
     */
    period?: Period | undefined;
    /**
     * For the use-case where two doctors need to meet to discuss the results for a specific patient, and the patient is not required to be present include the patient in the subject field, but do not include them as a participant - this was formerly done prior to R5 with required='information-only'.
     */
    required?: boolean | undefined;
    _required?: Element | undefined;
    /**
     * Participation status of the actor.
     */
    status: 'accepted' | 'declined' | 'tentative' | 'needs-action';
    _status?: Element | undefined;
    /**
     * The role of the participant can be used to declare what the actor will be doing in the scope of this appointment.
     * If the actor is not specified, then it is expected that the actor will be filled in at a later stage of planning.
     * This value SHALL be the same when creating an AppointmentResponse so that they can be matched, and subsequently update the Appointment.
     */
    type?: CodeableConcept[] | undefined;
}
/**
 * Information about weekly recurring appointments.
 */
export interface AppointmentRecurrenceTemplateWeeklyTemplate extends BackboneElement {
    /**
     * Indicates that recurring appointments should occur on Fridays.
     */
    friday?: boolean | undefined;
    _friday?: Element | undefined;
    /**
     * Indicates that recurring appointments should occur on Mondays.
     */
    monday?: boolean | undefined;
    _monday?: Element | undefined;
    /**
     * Indicates that recurring appointments should occur on Saturdays.
     */
    saturday?: boolean | undefined;
    _saturday?: Element | undefined;
    /**
     * Indicates that recurring appointments should occur on Sundays.
     */
    sunday?: boolean | undefined;
    _sunday?: Element | undefined;
    /**
     * Indicates that recurring appointments should occur on Thursdays.
     */
    thursday?: boolean | undefined;
    _thursday?: Element | undefined;
    /**
     * Indicates that recurring appointments should occur on Tuesdays.
     */
    tuesday?: boolean | undefined;
    _tuesday?: Element | undefined;
    /**
     * Indicates that recurring appointments should occur on Wednesdays.
     */
    wednesday?: boolean | undefined;
    _wednesday?: Element | undefined;
    /**
     * The interval defines if the recurrence is every nth week. The default is every week, so it is expected that this value will be 2 or more.
     * e.g. For recurring every second week this interval would be 2, or every third week the interval would be 3.
     */
    weekInterval?: number | undefined;
}
/**
 * Information about monthly recurring appointments.
 */
export interface AppointmentRecurrenceTemplateMonthlyTemplate extends BackboneElement {
    /**
     * Indicates that appointments in the series of recurring appointments should occur on a specific day of the month.
     */
    dayOfMonth?: number | undefined;
    /**
     * This property is intended to be used with Appointment.recurrenceTemplate.monthly.nthWeek.
     */
    dayOfWeek?: Coding | undefined;
    /**
     * Indicates that recurring appointments should occur every nth month.
     */
    monthInterval: number;
    /**
     * Indicates which week within a month the appointments in the series of recurring appointments should occur on.
     */
    nthWeekOfMonth?: Coding | undefined;
}
/**
 * Information about yearly recurring appointments.
 */
export interface AppointmentRecurrenceTemplateYearlyTemplate extends BackboneElement {
    /**
     * Appointment recurs every nth year.
     */
    yearInterval: number;
}
/**
 * The details of the recurrence pattern or template that is used to generate recurring appointments.
 */
export interface AppointmentRecurrenceTemplate extends BackboneElement {
    /**
     * excludingDate is an alternative to excludingRecurrenceId.  You may say either that 12/25/XXXX should be excluded using excludingDate, or you may indicate the recurrenceId of the appointment that would fall on the excluded date in the excludingRecurrenceId property.
     */
    excludingDate?: string[] | undefined;
    _excludingDate?: Element[] | undefined;
    /**
     * excludingRecurrenceId is an alternative to excludingDate.  You may say either that 12/25/XXXX should be excluded using excludingDate, or you may indicate the recurrenceId of the appointment that would fall on the excluded date in the excludingRecurrenceId property.
     */
    excludingRecurrenceId?: number[] | undefined;
    /**
     * Recurring appointments will not occur after this date.
     */
    lastOccurrenceDate?: string | undefined;
    _lastOccurrenceDate?: Element | undefined;
    /**
     * Information about monthly recurring appointments.
     */
    monthlyTemplate?: AppointmentRecurrenceTemplateMonthlyTemplate | undefined;
    /**
     * How many appointments are planned in the recurrence.
     */
    occurrenceCount?: number | undefined;
    /**
     * The list of specific dates that will have appointments generated.
     */
    occurrenceDate?: string[] | undefined;
    _occurrenceDate?: Element[] | undefined;
    /**
     * How often the appointment series should recur.
     */
    recurrenceType: CodeableConcept;
    /**
     * The timezone of the recurring appointment occurrences.
     */
    timezone?: CodeableConcept | undefined;
    /**
     * Information about weekly recurring appointments.
     */
    weeklyTemplate?: AppointmentRecurrenceTemplateWeeklyTemplate | undefined;
    /**
     * Information about yearly recurring appointments.
     */
    yearlyTemplate?: AppointmentRecurrenceTemplateYearlyTemplate | undefined;
}
/**
 * A booking of a healthcare event among patient(s), practitioner(s), related person(s) and/or device(s) for a specific date/time. This may result in one or more Encounter(s).
 */
export interface Appointment extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'Appointment';
    /**
     * The specified account(s) could be those identified during pre-registration workflows in preparation for an upcoming Encounter.
     */
    account?: Reference[] | undefined;
    /**
     * The style of appointment or patient that has been booked in the slot (not service type).
     */
    appointmentType?: CodeableConcept | undefined;
    /**
     * The request this appointment is allocated to assess (e.g. incoming referral or procedure request).
     */
    basedOn?: Reference[] | undefined;
    /**
     * The coded reason for the appointment being cancelled. This is often used in reporting/billing/futher processing to determine if further actions are required, or specific fees apply.
     */
    cancellationReason?: CodeableConcept | undefined;
    /**
     * Concepts representing classification of patient encounter such as ambulatory (outpatient), inpatient, emergency, home health or others due to local variations.
     */
    class?: CodeableConcept[] | undefined;
    /**
     * This property is required for many use cases where the age of an appointment is considered in processing workflows for scheduling and billing of appointments.
     */
    created?: string | undefined;
    _created?: Element | undefined;
    /**
     * The brief description of the appointment as would be shown on a subject line in a meeting request, or appointment list. Detailed or expanded information should be put in the comment field.
     */
    description?: string | undefined;
    _description?: Element | undefined;
    /**
     * Date/Time that the appointment is to conclude.
     */
    end?: string | undefined;
    _end?: Element | undefined;
    /**
     * This records identifiers associated with this appointment concern that are defined by business processes and/or used to refer to it when a direct URL reference to the resource itself is not appropriate (e.g. in CDA documents, or in written / printed documentation).
     */
    identifier?: Identifier[] | undefined;
    /**
     * Number of minutes that the appointment is to take. This can be less than the duration between the start and end times.  For example, where the actual time of appointment is only an estimate or if a 30 minute appointment is being requested, but any time would work.  Also, if there is, for example, a planned 15 minute break in the middle of a long appointment, the duration may be 15 minutes less than the difference between the start and end.
     */
    minutesDuration?: number | undefined;
    /**
     * Additional text to aid in facilitating the appointment. For instance, a note might be, "patient should proceed immediately to infusion room upon arrival"
     * Where this is a planned appointment and the start/end dates are not set then this field can be used to provide additional guidance on the details of the appointment request, including any restrictions on when to book it.
     * Typically only the concept.text will be used, however occasionally a reference to some generic documentation (or specific) and also supports coded instructions if/when they are required.
     */
    note?: Annotation[] | undefined;
    /**
     * For example, if a patient has physical therapy appointments every Tuesday and Thursday, but due to a national holiday, an single occurrence was rescheduled to a different day, while the rest of the recurring series remained on Tuesday / Thursday.
     */
    occurrenceChanged?: boolean | undefined;
    _occurrenceChanged?: Element | undefined;
    /**
     * This property is intended for use when representing a recurring set of related appointments.
     * For example, a patient undergoing physical therapy may have a recurring appointment every Tuesday and Thursday.  Each occurrence of the set will refer to the originating appointment, which contains the recurring template information.  For representing appointment series, see the guidance on recurring vs. series appointments.
     */
    originatingAppointment?: Reference | undefined;
    /**
     * List of participants involved in the appointment.
     */
    participant: AppointmentParticipant[];
    /**
     * While Appointment.note contains information for internal use, Appointment.patientInstructions is used to capture patient facing information about the Appointment (e.g. please bring your referral or fast from 8pm night before).
     */
    patientInstruction?: CodeableReference[] | undefined;
    /**
     * This property is intended for use when representing a series of related appointments.  For example, in a nuclear medicine procedure, where there is an appointment for the injection of the isotopes, and then a subsequent appointment for the scan, the scan appointment would refer to the injection appointment via Appointment.previousAppointment.  For representing recurring appointments, see the guidance on recurring vs. series appointments.
     */
    previousAppointment?: Reference | undefined;
    /**
     * Seeking implementer feedback on this property and how interoperable it is.
     * Using an extension to record a CodeableConcept for named values may be tested at a future connectathon.
     */
    priority?: CodeableConcept | undefined;
    /**
     * The reason that this appointment is being scheduled. This is more clinical than administrative. This can be coded, or as specified using information from another resource. When the patient arrives and the encounter begins it may be used as the admission diagnosis. The indication will typically be a Condition (with other resources referenced in the evidence.detail), or a Procedure.
     */
    reason?: CodeableReference[] | undefined;
    /**
     * The sequence number that identifies a specific appointment in a recurring pattern.
     */
    recurrenceId?: number | undefined;
    /**
     * The details of the recurrence pattern or template that is used to generate recurring appointments.
     */
    recurrenceTemplate?: AppointmentRecurrenceTemplate[] | undefined;
    /**
     * Appointment replaced by this Appointment in cases where there is a cancellation, the details of the cancellation can be found in the cancellationReason property (on the referenced resource).
     */
    replaces?: Reference[] | undefined;
    /**
     * This does not introduce a capacity for recurring appointments.
     */
    requestedPeriod?: Period[] | undefined;
    /**
     * A broad categorization of the service that is to be performed during this appointment.
     */
    serviceCategory?: CodeableConcept[] | undefined;
    /**
     * For a provider to provider appointment the code "FOLLOWUP" may be appropriate, as this is expected to be discussing some patient that was seen in the past.
     */
    serviceType?: CodeableReference[] | undefined;
    /**
     * The slots from the participants' schedules that will be filled by the appointment.
     */
    slot?: Reference[] | undefined;
    /**
     * The specialty of a practitioner that would be required to perform the service requested in this appointment.
     */
    specialty?: CodeableConcept[] | undefined;
    /**
     * Date/Time that the appointment is to take place.
     */
    start?: string | undefined;
    _start?: Element | undefined;
    /**
     * If the Appointment's status is "cancelled" then all participants are expected to have their calendars released for the appointment period, and as such any Slots that were marked as BUSY can be re-set to FREE.
     * This element is labeled as a modifier because the status contains the code entered-in-error that mark the Appointment as not currently valid.
     */
    status:
        | 'proposed'
        | 'pending'
        | 'booked'
        | 'arrived'
        | 'fulfilled'
        | 'cancelled'
        | 'noshow'
        | 'entered-in-error'
        | 'checked-in'
        | 'waitlist';
    _status?: Element | undefined;
    /**
     * The patient or group associated with the appointment, if they are to be present (usually) then they should also be included in the participant backbone element.
     */
    subject?: Reference | undefined;
    /**
     * Additional information to support the appointment provided when making the appointment.
     */
    supportingInformation?: Reference[] | undefined;
    /**
     * There are two types of virtual meetings that often exist:
     * * a persistent, virtual meeting room that can only be used for a single purpose at a time,
     * * and a dynamic virtual meeting room that is generated on demand for a specific purpose.
     * Implementers may consider using Location.virtualService for persistent meeting rooms.
     * If each participant would have a different meeting link, an extension using the VirtualServiceContactDetail  can be applied to the Appointment.participant BackboneElement.
     */
    virtualService?: VirtualServiceDetail[] | undefined;
}
/**
 * A reply to an appointment request for a patient and/or practitioner(s), such as a confirmation or rejection.
 */
export interface AppointmentResponse extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'AppointmentResponse';
    /**
     * A Person, Location, HealthcareService, or Device that is participating in the appointment.
     */
    actor?: Reference | undefined;
    /**
     * Appointment that this response is replying to.
     */
    appointment: Reference;
    /**
     * This comment is particularly important when the responder is declining, tentatively accepting or requesting another time to indicate the reasons why.
     */
    comment?: string | undefined;
    _comment?: Element | undefined;
    /**
     * This may be either the same as the appointment request to confirm the details of the appointment, or alternately a new time to request a re-negotiation of the end time.
     */
    end?: string | undefined;
    _end?: Element | undefined;
    /**
     * This records identifiers associated with this appointment response concern that are defined by business processes and/ or used to refer to it when a direct URL reference to the resource itself is not appropriate.
     */
    identifier?: Identifier[] | undefined;
    /**
     * The original date within a recurring request. This could be used in place of the recurrenceId to be more direct (or where the template is provided through the simple list of dates in `Appointment.occurrenceDate`).
     */
    occurrenceDate?: string | undefined;
    _occurrenceDate?: Element | undefined;
    /**
     * This element is labeled as a modifier because the status contains the code entered-in-error that marks the participant as not currently valid.
     */
    participantStatus: 'accepted' | 'declined' | 'tentative' | 'needs-action';
    _participantStatus?: Element | undefined;
    /**
     * The role of the participant can be used to declare what the actor will be doing in the scope of the referenced appointment.
     * If the actor is not specified, then it is expected that the actor will be filled in at a later stage of planning.
     * This value SHALL be the same as specified on the referenced Appointment so that they can be matched, and subsequently updated.
     */
    participantType?: CodeableConcept[] | undefined;
    /**
     * Indicates that the response is proposing a different time that was initially requested.  The new proposed time will be indicated in the start and end properties.
     */
    proposedNewTime?: boolean | undefined;
    _proposedNewTime?: Element | undefined;
    /**
     * If the recurrence template was defined using the list of occurrenceDates then this property might not be used.
     */
    recurrenceId?: number | undefined;
    /**
     * When a recurring appointment is requested, the participant may choose to respond to each individual occurrence, in which case AppointmentResponse.recurring should be false.  If the participant chooses to respond the same way for all occurrences, they may instead use a single AppointmentResponse with recurring set to true.  These may be combined as the participant chooses.  For example, they may accept all occurrences (recurring = true), but then send a decline for a specific occurrence (recurring=false).
     */
    recurring?: boolean | undefined;
    _recurring?: Element | undefined;
    /**
     * This may be either the same as the appointment request to confirm the details of the appointment, or alternately a new time to request a re-negotiation of the start time.
     */
    start?: string | undefined;
    _start?: Element | undefined;
}
/**
 * A component comment, classifier, or rating of the artifact.
 */
export interface ArtifactAssessmentContent extends BackboneElement {
    /**
     * Indicates who or what authored the content.
     */
    author?: Reference | undefined;
    /**
     * Represents a rating, classifier, or assessment of the artifact.
     */
    classifier?: CodeableConcept[] | undefined;
    /**
     * If the informationType is container, the components of the content.
     */
    component?: ArtifactAssessmentContent[] | undefined;
    /**
     * Acceptable to publicly share the comment, classifier or rating.
     */
    freeToShare?: boolean | undefined;
    _freeToShare?: Element | undefined;
    /**
     * The type of information this component of the content represents.
     */
    informationType?: ('comment' | 'classifier' | 'rating' | 'container' | 'response' | 'change-request') | undefined;
    _informationType?: Element | undefined;
    /**
     * The target element is used to point the comment to aspect of the artifact, such as a text range within a CQL library (e.g. #content?0:0-120:80).
     */
    path?: string[] | undefined;
    _path?: Element[] | undefined;
    /**
     * Additional related artifacts that provide supporting documentation, additional evidence, or further information related to the content.
     */
    relatedArtifact?: RelatedArtifact[] | undefined;
    /**
     * A brief summary of the content of this component.
     */
    summary?: string | undefined;
    _summary?: Element | undefined;
    /**
     * Indicates what type of content this component represents.
     */
    type?: CodeableConcept | undefined;
}
/**
 * This Resource provides one or more comments, classifiers or ratings about a Resource and supports attribution and rights management metadata for the added content.
 */
export interface ArtifactAssessment extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'ArtifactAssessment';
    /**
     * The 'date' element may be more recent than the approval date because of minor changes or editorial corrections.
     */
    approvalDate?: string | undefined;
    _approvalDate?: Element | undefined;
    /**
     * A reference to a resource, canonical resource, or non-FHIR resource which the comment or assessment is about.
     */
    artifactReference?: Reference | undefined;
    /**
     * A reference to a resource, canonical resource, or non-FHIR resource which the comment or assessment is about.
     */
    artifactCanonical?: string | undefined;
    _artifactCanonical?: Element | undefined;
    /**
     * A reference to a resource, canonical resource, or non-FHIR resource which the comment or assessment is about.
     */
    artifactUri?: string | undefined;
    _artifactUri?: Element | undefined;
    /**
     * Display of or reference to the bibliographic citation of the comment, classifier, or rating.
     */
    citeAsReference?: Reference | undefined;
    /**
     * Display of or reference to the bibliographic citation of the comment, classifier, or rating.
     */
    citeAsMarkdown?: string | undefined;
    _citeAsMarkdown?: Element | undefined;
    /**
     * A component comment, classifier, or rating of the artifact.
     */
    content?: ArtifactAssessmentContent[] | undefined;
    /**
     * A copyright statement relating to the artifact assessment and/or its contents. Copyright statements are generally legal restrictions on the use and publishing of the artifact assessment.
     */
    copyright?: string | undefined;
    _copyright?: Element | undefined;
    /**
     * Note that this is not the same as the resource last-modified-date, since the resource may be a secondary representation of the artifact assessment. Additional specific dates may be added as extensions or be found by consulting Provenances associated with past versions of the resource.
     */
    date?: string | undefined;
    _date?: Element | undefined;
    /**
     * Indicates the disposition of the responsible party to the comment or change request.
     */
    disposition?:
        | (
              | 'unresolved'
              | 'not-persuasive'
              | 'persuasive'
              | 'persuasive-with-modification'
              | 'not-persuasive-with-modification'
          )
        | undefined;
    _disposition?: Element | undefined;
    /**
     * Typically, this is used for identifiers that can go in an HL7 V3 II (instance identifier) data type, and can then identify this activity definition outside of FHIR, where it is not possible to use the logical URI.
     */
    identifier?: Identifier[] | undefined;
    /**
     * If specified, this date follows the original approval date.
     */
    lastReviewDate?: string | undefined;
    _lastReviewDate?: Element | undefined;
    /**
     * Indicates the workflow status of the comment or change request.
     */
    workflowStatus?:
        | (
              | 'submitted'
              | 'triaged'
              | 'waiting-for-input'
              | 'resolved-no-change'
              | 'resolved-change-required'
              | 'deferred'
              | 'duplicate'
              | 'applied'
              | 'published'
          )
        | undefined;
    _workflowStatus?: Element | undefined;
}
/**
 * In some cases a "success" may be partial, for example, an incomplete or interrupted transfer of a radiological study. For the purpose of establishing accountability, these distinctions are not relevant.
 */
export interface AuditEventOutcome extends BackboneElement {
    /**
     * In some cases a "success" may be partial, for example, an incomplete or interrupted transfer of a radiological study. For the purpose of establishing accountability, these distinctions are not relevant.
     */
    code: Coding;
    /**
     * A human readable description of the error issue SHOULD be placed in details.text.
     */
    detail?: CodeableConcept[] | undefined;
}
/**
 * Several agents may be associated (i.e. have some responsibility for an activity) with an event or activity.
 * For example, an activity may be initiated by one user for other users or involve more than one user. However, only one user may be the initiator/requestor for the activity.
 * When a network are used in an event being recorded, there should be distinct agent elements for the known actors using the network. The agent with a network detail would be the responsible agent for use of that network.
 */
export interface AuditEventAgent extends BackboneElement {
    /**
     * Use AuditEvent.agent.authorization when you know that is specific to the agent, otherwise use AuditEvent.authorization. For example, during a machine-to-machine transfer it might not be obvious to the audit system who caused the event, but it does know why.
     */
    authorization?: CodeableConcept[] | undefined;
    /**
     * Where the agent location is known, the agent location when the event occurred.
     */
    location?: Reference | undefined;
    /**
     * When remote network endpoint is known, another agent representing the remote agent would indicate the remote network endpoint used. Convention is to indicate data flowing from Source to Destination. The convention for Search, given data flows both ways (query parameters vs results), is to have the Source as the initiator of the Search Transaction, and the Destination the responder to the Search transaction.
     */
    networkReference?: Reference | undefined;
    /**
     * When remote network endpoint is known, another agent representing the remote agent would indicate the remote network endpoint used. Convention is to indicate data flowing from Source to Destination. The convention for Search, given data flows both ways (query parameters vs results), is to have the Source as the initiator of the Search Transaction, and the Destination the responder to the Search transaction.
     */
    networkUri?: string | undefined;
    _networkUri?: Element | undefined;
    /**
     * When remote network endpoint is known, another agent representing the remote agent would indicate the remote network endpoint used. Convention is to indicate data flowing from Source to Destination. The convention for Search, given data flows both ways (query parameters vs results), is to have the Source as the initiator of the Search Transaction, and the Destination the responder to the Search transaction.
     */
    networkString?: string | undefined;
    _networkString?: Element | undefined;
    /**
     * For example: Where an OAuth token authorizes, the unique identifier from the OAuth token is placed into the policy element Where a policy engine (e.g. XACML) holds policy logic, the unique policy identifier is placed into the policy element.
     */
    policy?: string[] | undefined;
    _policy?: Element[] | undefined;
    /**
     * There can only be one initiator. If the initiator is not clear, then do not choose any one agent as the initiator.
     */
    requestor?: boolean | undefined;
    _requestor?: Element | undefined;
    /**
     * For example: Chief-of-Radiology, Nurse, Physician, Medical-Student, etc.
     */
    role?: CodeableConcept[] | undefined;
    /**
     * For example: assembler, author, prescriber, signer, investigator, etc.
     */
    type?: CodeableConcept | undefined;
    /**
     * Where a User ID is available it will go into who.identifier. Where a name of the user (human readable) it will go into who.display.
     */
    who: Reference;
}
/**
 * Events are reported by the actor that detected them. This may be one of the participating actors, but may also be different. The actor may be a human such as a medical-records clerk disclosing data manually, that clerk would be the source for the record of disclosure.
 */
export interface AuditEventSource extends BackboneElement {
    /**
     * Identifier of the source where the event was detected.
     */
    observer: Reference;
    /**
     * Logical source location within the healthcare enterprise network.  For example, a hospital or other provider location within a multi-entity provider group.
     */
    site?: Reference | undefined;
    /**
     * Code specifying the type of source where event originated.
     */
    type?: CodeableConcept[] | undefined;
}
/**
 * Tagged value pairs for conveying additional information about the entity.
 */
export interface AuditEventEntityDetail extends BackboneElement {
    /**
     * The type of extra detail provided in the value.
     */
    type: CodeableConcept;
    /**
     * The  value of the extra detail.
     */
    valueQuantity?: Quantity | undefined;
    /**
     * The  value of the extra detail.
     */
    valueCodeableConcept?: CodeableConcept | undefined;
    /**
     * The  value of the extra detail.
     */
    valueString?: string | undefined;
    _valueString?: Element | undefined;
    /**
     * The  value of the extra detail.
     */
    valueBoolean?: boolean | undefined;
    _valueBoolean?: Element | undefined;
    /**
     * The  value of the extra detail.
     */
    valueInteger?: number | undefined;
    /**
     * The  value of the extra detail.
     */
    valueRange?: Range | undefined;
    /**
     * The  value of the extra detail.
     */
    valueRatio?: Ratio | undefined;
    /**
     * The  value of the extra detail.
     */
    valueTime?: string | undefined;
    _valueTime?: Element | undefined;
    /**
     * The  value of the extra detail.
     */
    valueDateTime?: string | undefined;
    _valueDateTime?: Element | undefined;
    /**
     * The  value of the extra detail.
     */
    valuePeriod?: Period | undefined;
    /**
     * The  value of the extra detail.
     */
    valueBase64Binary?: string | undefined;
    _valueBase64Binary?: Element | undefined;
}
/**
 * Required unless the values for event identification, agent identification, and audit source identification are sufficient to document the entire auditable event. Because events may have more than one entity, this group can be a repeating set of values.
 */
export interface AuditEventEntity extends BackboneElement {
    /**
     * A usecase where one AuditEvent.entity.agent is used where the Entity that was used in the creation/updating of a target resource, is not in the context of the same custodianship as the target resource, and thus the meaning of AuditEvent.entity.agent is to say that the entity referenced is managed elsewhere and that this Agent provided access to it.  This would be similar to where the Entity being referenced is managed outside FHIR, such as through HL7 v2, v3, or XDS. This might be where the Entity being referenced is managed in another FHIR resource server. Thus it explains the provenance of that Entity's use in the context of this AuditEvent activity.
     */
    agent?: AuditEventAgent[] | undefined;
    /**
     * Tagged value pairs for conveying additional information about the entity.
     */
    detail?: AuditEventEntityDetail[] | undefined;
    /**
     * The meaning and secondary-encoding of the content of base64 encoded blob is specific to the AuditEvent.type, AuditEvent.subtype, and AuditEvent.entity.role.  The base64 is a general-use and safe container for event specific data blobs regardless of the encoding used by the transaction being recorded.  An AuditEvent consuming application must understand the event it is consuming and the formats used by the event. For example, if auditing an Oracle network database access, the Oracle formats must be understood as they will be simply encoded in the base64binary blob.
     * The DICOM AuditMessage schema does not support both .name and .query being populated.
     */
    query?: string | undefined;
    _query?: Element | undefined;
    /**
     * Code representing the role the entity played in the event being audited.
     */
    role?: CodeableConcept | undefined;
    /**
     * Copied from entity meta security tags.
     */
    securityLabel?: CodeableConcept[] | undefined;
    /**
     * Use .what.display when all you have is a string (e.g. ParticipantObjectName).
     */
    what?: Reference | undefined;
}
/**
 * A record of an event relevant for purposes such as operations, privacy, security, maintenance, and performance analysis.
 */
export interface AuditEvent extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'AuditEvent';
    /**
     * Indicator for type of action performed during the event that generated the audit.
     */
    action?: string | undefined;
    _action?: Element | undefined;
    /**
     * Several agents may be associated (i.e. have some responsibility for an activity) with an event or activity.
     * For example, an activity may be initiated by one user for other users or involve more than one user. However, only one user may be the initiator/requestor for the activity.
     * When a network are used in an event being recorded, there should be distinct agent elements for the known actors using the network. The agent with a network detail would be the responsible agent for use of that network.
     */
    agent: AuditEventAgent[];
    /**
     * Use AuditEvent.agent.authorization when you know that it is specific to the agent, otherwise use AuditEvent.authorization. For example, during a machine-to-machine transfer it might not be obvious to the audit system who caused the event, but it does know why.
     */
    authorization?: CodeableConcept[] | undefined;
    /**
     * Allows tracing of authorizatino for the events and tracking whether proposals/recommendations were acted upon.
     */
    basedOn?: Reference[] | undefined;
    /**
     * Classification of the type of event.
     */
    category?: CodeableConcept[] | undefined;
    /**
     * Describes what happened. The most specific code for the event.
     */
    code: CodeableConcept;
    /**
     * This will typically be the encounter the event occurred, but some events may be initiated prior to or after the official completion of an encounter but still be tied to the context of the encounter (e.g. pre-admission lab tests).
     */
    encounter?: Reference | undefined;
    /**
     * Required unless the values for event identification, agent identification, and audit source identification are sufficient to document the entire auditable event. Because events may have more than one entity, this group can be a repeating set of values.
     */
    entity?: AuditEventEntity[] | undefined;
    /**
     * The time or period can be a little arbitrary; where possible, the time should correspond to human assessment of the activity time.
     */
    occurredPeriod?: Period | undefined;
    /**
     * The time or period can be a little arbitrary; where possible, the time should correspond to human assessment of the activity time.
     */
    occurredDateTime?: string | undefined;
    _occurredDateTime?: Element | undefined;
    /**
     * In some cases a "success" may be partial, for example, an incomplete or interrupted transfer of a radiological study. For the purpose of establishing accountability, these distinctions are not relevant.
     */
    outcome?: AuditEventOutcome | undefined;
    /**
     * The patient element is available to enable deterministic tracking of activities that involve the patient as the subject of the data used in an activity.
     */
    patient?: Reference | undefined;
    /**
     * In a distributed system, some sort of common time base (e.g. an NTP [RFC1305] server) is a good implementation tactic.
     */
    recorded: string;
    _recorded?: Element | undefined;
    /**
     * ATNA will map this to the SYSLOG PRI element.
     */
    severity?:
        | ('emergency' | 'alert' | 'critical' | 'error' | 'warning' | 'notice' | 'informational' | 'debug')
        | undefined;
    _severity?: Element | undefined;
    /**
     * Events are reported by the actor that detected them. This may be one of the participating actors, but may also be different. The actor may be a human such as a medical-records clerk disclosing data manually, that clerk would be the source for the record of disclosure.
     */
    source: AuditEventSource;
}
/**
 * Basic is used for handling concepts not yet defined in FHIR, narrative-only resources that don't map to an existing resource, and custom resources not appropriate for inclusion in the FHIR specification.
 */
export interface Basic extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'Basic';
    /**
     * Indicates who was responsible for creating the resource instance.
     */
    author?: Reference | undefined;
    /**
     * Because resource references will only be able to indicate 'Basic', the type of reference will need to be specified in a Profile identified as part of the resource.  Refer to the resource notes section for information on appropriate terminologies for this code.
     * This element is labeled as a modifier because it defines the meaning of the resource and cannot be ignored.
     */
    code: CodeableConcept;
    /**
     * Identifies when the resource was first created.
     */
    created?: string | undefined;
    _created?: Element | undefined;
    /**
     * Identifier assigned to the resource for business purposes, outside the context of FHIR.
     */
    identifier?: Identifier[] | undefined;
    /**
     * Optional as not all potential resources will have subjects.  Resources associated with multiple subjects can handle this via extension.
     */
    subject?: Reference | undefined;
}
/**
 * A resource that represents the data of a single raw artifact as digital content accessible in its native format.  A Binary resource can contain any content, whether text, image, pdf, zip archive, etc.
 */
export interface Binary extends Resource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'Binary';
    /**
     * MimeType of the binary content represented as a standard MimeType (BCP 13).
     */
    contentType: string;
    _contentType?: Element | undefined;
    /**
     * If the content type is itself base64 encoding, then this will be base64 encoded twice - what is created by un-base64ing the content must be the specified content type.
     */
    data?: string | undefined;
    _data?: Element | undefined;
    /**
     * Very often, a server will also know of a resource that references the binary, and can automatically apply the appropriate access rules based on that reference. However, there are some circumstances where this is not appropriate, e.g. the binary is uploaded directly to the server without any linking resource, the binary is referred to from multiple different resources, and/or the binary is content such as an application logo that has less protection than any of the resources that reference it.
     */
    securityContext?: Reference | undefined;
}
/**
 * How this product was collected.
 */
export interface BiologicallyDerivedProductCollection extends BackboneElement {
    /**
     * Time of product collection.
     */
    collectedDateTime?: string | undefined;
    _collectedDateTime?: Element | undefined;
    /**
     * Time of product collection.
     */
    collectedPeriod?: Period | undefined;
    /**
     * Healthcare professional who is performing the collection.
     */
    collector?: Reference | undefined;
    /**
     * The patient or entity, such as a hospital or vendor in the case of a processed/manipulated/manufactured product, providing the product.
     */
    source?: Reference | undefined;
}
/**
 * Property can be used to provide information on a wide range of additional information specific to a particular biologicallyDerivedProduct.
 */
export interface BiologicallyDerivedProductProperty extends BackboneElement {
    /**
     * The element is identified by name and system URI in the type. Some types may have multiple occurrences.
     */
    type: Coding;
    /**
     * The value should be provided as a boolean, integer, CodeableConcept, period, quantity, range, ratio, or attachment. The description can be a string only when these others are not available. The type of value will depend on the property type and is specified in ST-027.
     */
    valueBoolean?: boolean | undefined;
    _valueBoolean?: Element | undefined;
    /**
     * The value should be provided as a boolean, integer, CodeableConcept, period, quantity, range, ratio, or attachment. The description can be a string only when these others are not available. The type of value will depend on the property type and is specified in ST-027.
     */
    valueInteger?: number | undefined;
    /**
     * The value should be provided as a boolean, integer, CodeableConcept, period, quantity, range, ratio, or attachment. The description can be a string only when these others are not available. The type of value will depend on the property type and is specified in ST-027.
     */
    valueCodeableConcept?: CodeableConcept | undefined;
    /**
     * The value should be provided as a boolean, integer, CodeableConcept, period, quantity, range, ratio, or attachment. The description can be a string only when these others are not available. The type of value will depend on the property type and is specified in ST-027.
     */
    valuePeriod?: Period | undefined;
    /**
     * The value should be provided as a boolean, integer, CodeableConcept, period, quantity, range, ratio, or attachment. The description can be a string only when these others are not available. The type of value will depend on the property type and is specified in ST-027.
     */
    valueQuantity?: Quantity | undefined;
    /**
     * The value should be provided as a boolean, integer, CodeableConcept, period, quantity, range, ratio, or attachment. The description can be a string only when these others are not available. The type of value will depend on the property type and is specified in ST-027.
     */
    valueRange?: Range | undefined;
    /**
     * The value should be provided as a boolean, integer, CodeableConcept, period, quantity, range, ratio, or attachment. The description can be a string only when these others are not available. The type of value will depend on the property type and is specified in ST-027.
     */
    valueRatio?: Ratio | undefined;
    /**
     * The value should be provided as a boolean, integer, CodeableConcept, period, quantity, range, ratio, or attachment. The description can be a string only when these others are not available. The type of value will depend on the property type and is specified in ST-027.
     */
    valueString?: string | undefined;
    _valueString?: Element | undefined;
    /**
     * The value should be provided as a boolean, integer, CodeableConcept, period, quantity, range, ratio, or attachment. The description can be a string only when these others are not available. The type of value will depend on the property type and is specified in ST-027.
     */
    valueAttachment?: Attachment | undefined;
}
/**
 * This resource reflects an instance of a biologically derived product. A material substance originating from a biological entity intended to be transplanted or infused
 * into another (possibly the same) biological entity.
 */
export interface BiologicallyDerivedProduct extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'BiologicallyDerivedProduct';
    /**
     * Necessary to support mandatory requirements for traceability from donor/source to recipient and vice versa, while also satisfying donor anonymity requirements. The element is defined consistently across BiologicallyDerivedProduct, NutritionProduct, and Device.  The identifier references an event that links to a single biological entity such as a blood donor, or to multiple biological entities (e.g. when the product is an embryo or a pooled platelet product).  A single biologicalSourceEvent identifier may appear on multiple products of many types derived from a single donation event or source extraction.  As an example, a single donation event may provide 2 kidneys and a liver for organ transplantation, 2 corneas for eye surgery, heart valves and arterial tissue for cardiovascular surgery, multiple skin grafts, tendons, multiple shaped bone grafts and a large number of bone putty/paste products; and each of them may be assigned to the same biological source event identifier.
     */
    biologicalSourceEvent?: Identifier | undefined;
    /**
     * How this product was collected.
     */
    collection?: BiologicallyDerivedProductCollection | undefined;
    /**
     * A unique identifier for an aliquot of a product.  Used to distinguish individual aliquots of a product carrying the same biologicalSource and productCode identifiers.
     */
    division?: string | undefined;
    _division?: Element | undefined;
    /**
     * Date, and where relevant time, of expiration.
     */
    expirationDate?: string | undefined;
    _expirationDate?: Element | undefined;
    /**
     * This identifier should uniquely identify the product instance in the business domain.  Ideally it should be a globally unique identifier under the control of an ISO/IEC 15459 Issuing Agency.
     */
    identifier?: Identifier[] | undefined;
    /**
     * For products that have multiple collections. For example Peripheral Blood Stem Cells may be collected over several days from a single donor and the donation split into in multiple containers which must be linked to the parent donation.
     */
    parent?: Reference[] | undefined;
    /**
     * Processing facilities responsible for the labeling and distribution of this biologically derived product.
     */
    processingFacility?: Reference[] | undefined;
    /**
     * Broad category of this product.
     */
    productCategory?: Coding | undefined;
    /**
     * A codified value that systematically supports characterization and classification of medical products of human origin inclusive of processing conditions such as additives, volumes and handling conditions.
     */
    productCode?: CodeableConcept | undefined;
    /**
     * Whether the product is currently available.
     */
    productStatus?: Coding | undefined;
    /**
     * Property can be used to provide information on a wide range of additional information specific to a particular biologicallyDerivedProduct.
     */
    property?: BiologicallyDerivedProductProperty[] | undefined;
    /**
     * Request to obtain and/or infuse this biologically derived product.
     */
    request?: Reference[] | undefined;
    /**
     * May be extracted from information held in the Product Description Code.
     */
    storageTempRequirements?: Range | undefined;
}
/**
 * The anatomical location(s) or region(s) of the specimen, lesion, or body structure.
 */
export interface BodyStructureIncludedStructure extends BackboneElement {
    /**
     * Code that represents the included structure laterality.
     */
    laterality?: CodeableConcept | undefined;
    /**
     * Code that represents the included structure qualifier.
     */
    qualifier?: CodeableConcept[] | undefined;
    /**
     * Code that represents the included structure.
     */
    structure: CodeableConcept;
}
/**
 * The anatomical location(s) or region(s) not occupied or represented by the specimen, lesion, or body structure.
 */
export interface BodyStructureExcludedStructure extends BackboneElement {
    /**
     * Code that represents the excluded structure laterality.
     */
    laterality?: CodeableConcept | undefined;
    /**
     * Code that represents the excluded structure qualifier.
     */
    qualifier?: CodeableConcept[] | undefined;
    /**
     * Code that represents the excluded structure.
     */
    structure: CodeableConcept;
}
/**
 * Record details about an anatomical structure.  This resource may be used when a coded concept does not provide the necessary detail needed for the use case.
 */
export interface BodyStructure extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'BodyStructure';
    /**
     * This element is labeled as a modifier because it may be used to mark that the resource was created in error.
     */
    active?: boolean | undefined;
    _active?: Element | undefined;
    /**
     * This description could include any visual markings used to orientate the viewer e.g. external reference points, special sutures, ink markings.
     */
    description?: string | undefined;
    _description?: Element | undefined;
    /**
     * The anatomical location(s) or region(s) not occupied or represented by the specimen, lesion, or body structure.
     */
    excludedStructure?: BodyStructureExcludedStructure[] | undefined;
    /**
     * Identifier for this instance of the anatomical structure.
     */
    identifier?: Identifier[] | undefined;
    /**
     * Image or images used to identify a location.
     */
    image?: Attachment[] | undefined;
    /**
     * The anatomical location(s) or region(s) of the specimen, lesion, or body structure.
     */
    includedStructure: BodyStructureIncludedStructure[];
    /**
     * The minimum cardinality of 0 supports the use case of specifying a location without defining a morphology.
     */
    morphology?: CodeableConcept | undefined;
    /**
     * The person to which the body site belongs.
     */
    patient: Reference;
}
/**
 * Both Bundle.link and Bundle.entry.link are defined to support providing additional context when Bundles are used (e.g. [HATEOAS](http://en.wikipedia.org/wiki/HATEOAS)).
 * Bundle.entry.link corresponds to links found in the HTTP header if the resource in the entry was [read](http.html#read) directly.
 * This specification defines some specific uses of Bundle.link for [searching](search.html#conformance) and [paging](http.html#paging), but no specific uses for Bundle.entry.link, and no defined function in a transaction - the meaning is implementation specific.
 */
export interface BundleLink extends BackboneElement {
    /**
     * A name which details the functional use for this link - see [http://www.iana.org/assignments/link-relations/link-relations.xhtml#link-relations-1](http://www.iana.org/assignments/link-relations/link-relations.xhtml#link-relations-1).
     */
    relation: string;
    _relation?: Element | undefined;
    /**
     * The reference details for the link.
     */
    url: string;
    _url?: Element | undefined;
}
/**
 * Information about the search process that lead to the creation of this entry.
 */
export interface BundleEntrySearch extends BackboneElement {
    /**
     * There is only one mode. In some corner cases, a resource may be included because it is both a match and an include. In these circumstances, 'match' takes precedence.
     */
    mode?: ('match' | 'include' | 'outcome') | undefined;
    _mode?: Element | undefined;
    /**
     * Servers are not required to return a ranking score. 1 is most relevant, and 0 is least relevant. Often, search results are sorted by score, but the client may specify a different sort order.
     * See [Patient Match](patient-operation-match.html) for the EMPI search which relates to this element.
     */
    score?: number | undefined;
}
/**
 * Additional information about how this entry should be processed as part of a transaction or batch.  For history, it shows how the entry was processed to create the version contained in the entry.
 */
export interface BundleEntryRequest extends BackboneElement {
    /**
     * Only perform the operation if the Etag value matches. For more information, see the API section ["Managing Resource Contention"](http.html#concurrency).
     */
    ifMatch?: string | undefined;
    _ifMatch?: Element | undefined;
    /**
     * Only perform the operation if the last updated date matches. See the API documentation for ["Conditional Read"](http.html#cread).
     */
    ifModifiedSince?: string | undefined;
    _ifModifiedSince?: Element | undefined;
    /**
     * Instruct the server not to perform the create if a specified resource already exists. For further information, see the API documentation for ["Conditional Create"](http.html#ccreate). This is just the query portion of the URL - what follows the "?" (not including the "?").
     */
    ifNoneExist?: string | undefined;
    _ifNoneExist?: Element | undefined;
    /**
     * If the ETag values match, return a 304 Not Modified status. See the API documentation for ["Conditional Read"](http.html#cread).
     */
    ifNoneMatch?: string | undefined;
    _ifNoneMatch?: Element | undefined;
    /**
     * In a transaction or batch, this is the HTTP action to be executed for this entry. In a history bundle, this indicates the HTTP action that occurred.
     */
    method: 'GET' | 'HEAD' | 'POST' | 'PUT' | 'DELETE' | 'PATCH';
    _method?: Element | undefined;
    /**
     * E.g. for a Patient Create, the method would be "POST" and the URL would be "Patient". For a Patient Update, the method would be PUT and the URL would be "Patient/[id]".
     */
    url: string;
    _url?: Element | undefined;
}
/**
 * Indicates the results of processing the corresponding 'request' entry in the batch or transaction being responded to or what the results of an operation where when returning history.
 */
export interface BundleEntryResponse extends BackboneElement {
    /**
     * Etags match the Resource.meta.versionId. The ETag has to match the version id in the header if a resource is included.
     */
    etag?: string | undefined;
    _etag?: Element | undefined;
    /**
     * This has to match the same time in the meta header (meta.lastUpdated) if a resource is included.
     */
    lastModified?: string | undefined;
    _lastModified?: Element | undefined;
    /**
     * The location header created by processing this operation, populated if the operation returns a location.
     */
    location?: string | undefined;
    _location?: Element | undefined;
    /**
     * For a POST/PUT operation, this is the equivalent outcome that would be returned for prefer = operationoutcome - except that the resource is always returned whether or not the outcome is returned.
     * This outcome is not used for error responses in batch/transaction, only for hints and warnings. In a batch operation, the error will be in Bundle.entry.response, and for transaction, there will be a single OperationOutcome instead of a bundle in the case of an error.
     */
    outcome?: FhirResource | undefined;
    /**
     * The status code returned by processing this entry. The status SHALL start with a 3 digit HTTP code (e.g. 404) and may contain the standard HTTP description associated with the status code.
     */
    status: string;
    _status?: Element | undefined;
}
/**
 * An entry in a bundle resource - will either contain a resource or information about a resource (transactions and history only).
 */
export interface BundleEntry<BundleContentType = FhirResource> extends BackboneElement {
    /**
     * fullUrl might not be [unique in the context of a resource](bundle.html#bundle-unique). Note that since [FHIR resources do not need to be served through the FHIR API](references.html), the fullURL might be a URN or an absolute URL that does not end with the logical id of the resource (Resource.id). However, but if the fullUrl does look like a RESTful server URL (e.g. meets the [regex](references.html#regex), then the 'id' portion of the fullUrl SHALL end with the Resource.id.
     * Note that the fullUrl is not the same as the canonical URL - it's an absolute url for an endpoint serving the resource (these will happen to have the same value on the canonical server for the resource with the canonical URL).
     */
    fullUrl?: string | undefined;
    _fullUrl?: Element | undefined;
    /**
     * A series of links that provide context to this entry.
     */
    link?: BundleLink[] | undefined;
    /**
     * Additional information about how this entry should be processed as part of a transaction or batch.  For history, it shows how the entry was processed to create the version contained in the entry.
     */
    request?: BundleEntryRequest | undefined;
    /**
     * The Resource for the entry. The purpose/meaning of the resource is determined by the Bundle.type. This is allowed to be a Parameters resource if and only if it is referenced by something else within the Bundle that provides context/meaning.
     */
    resource?: BundleContentType | undefined;
    /**
     * Indicates the results of processing the corresponding 'request' entry in the batch or transaction being responded to or what the results of an operation where when returning history.
     */
    response?: BundleEntryResponse | undefined;
    /**
     * Information about the search process that lead to the creation of this entry.
     */
    search?: BundleEntrySearch | undefined;
}
/**
 * A container for a collection of resources.
 */
export interface Bundle<BundleContentType = FhirResource> extends Resource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'Bundle';
    /**
     * An entry in a bundle resource - will either contain a resource or information about a resource (transactions and history only).
     */
    entry?: BundleEntry<BundleContentType>[] | undefined;
    /**
     * Persistent identity generally only matters for batches of type Document, Message, and Collection. It would not normally be populated for search and history results and servers ignore Bundle.identifier when processing batches and transactions. For Documents  the .identifier SHALL be populated such that the .identifier is globally unique.
     */
    identifier?: Identifier | undefined;
    /**
     * Usage notes: These issues and warnings must apply to the Bundle as a whole, not to individual entries.Messages relating to the processing of individual entries (e.g. in a batch or transaction) SHALL be reported in the entry.response.outcome for that entry.  If there are errors that arise in the creation of the Bundle, then that should be handled by an OperationOutcome being returned instead of the Bundle.
     */
    issues?: FhirResource | undefined;
    /**
     * Both Bundle.link and Bundle.entry.link are defined to support providing additional context when Bundles are used (e.g. [HATEOAS](http://en.wikipedia.org/wiki/HATEOAS)).
     * Bundle.entry.link corresponds to links found in the HTTP header if the resource in the entry was [read](http.html#read) directly.
     * This specification defines some specific uses of Bundle.link for [searching](search.html#conformance) and [paging](http.html#paging), but no specific uses for Bundle.entry.link, and no defined function in a transaction - the meaning is implementation specific.
     */
    link?: BundleLink[] | undefined;
    /**
     * The signature could be created by the "author" of the bundle or by the originating device.   Requirements around inclusion of a signature, verification of signatures and treatment of signed/non-signed bundles is implementation-environment specific.
     */
    signature?: Signature | undefined;
    /**
     * For many bundles, the timestamp is equal to .meta.lastUpdated, because they are not stored (e.g. search results). When a bundle is placed in a persistent store, .meta.lastUpdated will be usually be changed by the server. When the bundle is a message, a middleware agent altering the message (even if not stored) SHOULD update .meta.lastUpdated. .timestamp is used to track the original time of the Bundle, and SHOULD be populated.
     * Usage:
     * * document : the date the document was created. Note: the composition may predate the document, or be associated with multiple documents. The date of the composition - the authoring time - may be earlier than the document assembly time
     * * message : the date that the content of the message was assembled. This date is not changed by middleware engines unless they add additional data that changes the meaning of the time of the message
     * * history : the date that the history was assembled. This time would be used as the _since time to ask for subsequent updates
     * * searchset : the time that the search set was assembled. Note that different pages MAY have different timestamps but need not. Having different timestamps does not imply that subsequent pages will represent or include changes made since the initial query
     * * transaction | transaction-response | batch | batch-response | collection : no particular assigned meaning
     * The timestamp value should be greater than the lastUpdated and other timestamps in the resources in the bundle, and it should be equal or earlier than the .meta.lastUpdated on the Bundle itself.
     */
    timestamp?: string | undefined;
    _timestamp?: Element | undefined;
    /**
     * Only used if the bundle is a search result set. The total does not include resources such as OperationOutcome and included resources, only the total number of matching resources.
     */
    total?: number | undefined;
    /**
     * It's possible to use a bundle for other purposes (e.g. a document can be accepted as a transaction). This is primarily defined so that there can be specific rules for some of the bundle types.
     */
    type:
        | 'document'
        | 'message'
        | 'transaction'
        | 'transaction-response'
        | 'batch'
        | 'batch-response'
        | 'history'
        | 'searchset'
        | 'collection'
        | 'subscription-notification';
    _type?: Element | undefined;
}
/**
 * Common Interface declaration for conformance and knowledge artifact resources.
 */
export interface CanonicalResource extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: string;
    /**
     * May be a web site, an email address, a telephone number, etc.
     */
    contact?: ContactDetail[] | undefined;
    /**
     * The short copyright declaration (e.g. (c) '2015+ xyz organization' should be sent in the copyrightLabel element.
     */
    copyright?: string | undefined;
    _copyright?: Element | undefined;
    /**
     * The (c) symbol should NOT be included in this string. It will be added by software when rendering the notation. Full details about licensing, restrictions, warrantees, etc. goes in the more general 'copyright' element.
     */
    copyrightLabel?: string | undefined;
    _copyrightLabel?: Element | undefined;
    /**
     * Note that this is not the same as the resource last-modified-date, since the resource may be a secondary representation of the {{title}}. Additional specific dates may be added as extensions or be found by consulting Provenances associated with past versions of the resource.
     */
    date?: string | undefined;
    _date?: Element | undefined;
    /**
     * This description can be used to capture details such as comments about misuse, instructions for clinical use and interpretation, literature references, examples from the paper world, etc. It is not a rendering of the {{title}} as conveyed in the 'text' field of the resource itself. This item SHOULD be populated unless the information is available from context (e.g. the language of the {{title}} is presumed to be the predominant language in the place the {{title}} was created).
     */
    description?: string | undefined;
    _description?: Element | undefined;
    /**
     * Allows filtering of {{title}}s that are appropriate for use versus not.
     */
    experimental?: boolean | undefined;
    _experimental?: Element | undefined;
    /**
     * Typically, this is used for identifiers that can go in an HL7 V3 II (instance identifier) data type, and can then identify this {{title}} outside of FHIR, where it is not possible to use the logical URI.
     */
    identifier?: Identifier[] | undefined;
    /**
     * It may be possible for the {{title}} to be used in jurisdictions other than those for which it was originally designed or intended.
     */
    jurisdiction?: CodeableConcept[] | undefined;
    /**
     * The name is not expected to be globally unique. The name should be a simple alphanumeric type name to ensure that it is machine-processing friendly.
     */
    name?: string | undefined;
    _name?: Element | undefined;
    /**
     * Usually an organization but may be an individual. The publisher (or steward) of the {{title}} is the organization or individual primarily responsible for the maintenance and upkeep of the {{title}}. This is not necessarily the same individual or organization that developed and initially authored the content. The publisher is the primary point of contact for questions or issues with the {{title}}. This item SHOULD be populated unless the information is available from context.
     */
    publisher?: string | undefined;
    _publisher?: Element | undefined;
    /**
     * This element does not describe the usage of the {{title}}. Instead, it provides traceability of ''why'' the resource is either needed or ''why'' it is defined as it is.  This may be used to point to source materials or specifications that drove the structure of this {{title}}.
     */
    purpose?: string | undefined;
    _purpose?: Element | undefined;
    /**
     * Allows filtering of {{title}}s that are appropriate for use versus not.
     */
    status: 'draft' | 'active' | 'retired' | 'unknown';
    _status?: Element | undefined;
    /**
     * This name does not need to be machine-processing friendly and may contain punctuation, white-space, etc.
     */
    title?: string | undefined;
    _title?: Element | undefined;
    /**
     * Can be a urn:uuid: or a urn:oid: but real http: addresses are preferred.  Multiple instances may share the same URL if they have a distinct version.
     * The determination of when to create a new version of a resource (same url, new version) vs. defining a new artifact is up to the author.  Considerations for making this decision are found in [Technical and Business Versions](resource.html#versions).
     * In some cases, the resource can no longer be found at the stated url, but the url itself cannot change. Implementations can use the [meta.source](resource.html#meta) element to indicate where the current master source of the resource can be found.
     */
    url?: string | undefined;
    _url?: Element | undefined;
    /**
     * When multiple useContexts are specified, there is no expectation that all or any of the contexts apply.
     */
    useContext?: UsageContext[] | undefined;
    /**
     * There may be different {{title}} instances that have the same identifier but different versions.  The version can be appended to the url in a reference to allow a reference to a particular business version of the {{title}} with the format [url]|[version]. The version SHOULD NOT contain a '#' - see [Business Version](resource.html#bv-format).
     */
    version?: string | undefined;
    _version?: Element | undefined;
    /**
     * If set as a string, this is a FHIRPath expression that has two additional context variables passed in - %version1 and %version2 and will return a negative number if version1 is newer, a positive number if version2 and a 0 if the version ordering can't be successfully be determined.
     */
    versionAlgorithmString?: string | undefined;
    _versionAlgorithmString?: Element | undefined;
    /**
     * If set as a string, this is a FHIRPath expression that has two additional context variables passed in - %version1 and %version2 and will return a negative number if version1 is newer, a positive number if version2 and a 0 if the version ordering can't be successfully be determined.
     */
    versionAlgorithmCoding?: Coding | undefined;
}
/**
 * Software that is covered by this capability statement.  It is used when the capability statement describes the capabilities of a particular software version, independent of an installation.
 */
export interface CapabilityStatementSoftware extends BackboneElement {
    /**
     * Name the software is known by.
     */
    name: string;
    _name?: Element | undefined;
    /**
     * Date this version of the software was released.
     */
    releaseDate?: string | undefined;
    _releaseDate?: Element | undefined;
    /**
     * If possible, a version should be specified, as statements are likely to be different for different versions of software.
     */
    version?: string | undefined;
    _version?: Element | undefined;
}
/**
 * Identifies a specific implementation instance that is described by the capability statement - i.e. a particular installation, rather than the capabilities of a software program.
 */
export interface CapabilityStatementImplementation extends BackboneElement {
    /**
     * The organization responsible for the management of the instance and oversight of the data on the server at the specified URL.
     */
    custodian?: Reference | undefined;
    /**
     * Information about the specific installation that this capability statement relates to.
     */
    description: string;
    _description?: Element | undefined;
    /**
     * An absolute base URL for the implementation.  This forms the base for REST interfaces as well as the mailbox and document interfaces.
     */
    url?: string | undefined;
    _url?: Element | undefined;
}
/**
 * Information about security implementation from an interface perspective - what a client needs to know.
 */
export interface CapabilityStatementRestSecurity extends BackboneElement {
    /**
     * The easiest CORS headers to add are Access-Control-Allow-Origin: * & Access-Control-Request-Method: GET, POST, PUT, DELETE. All servers SHOULD support CORS.
     */
    cors?: boolean | undefined;
    _cors?: Element | undefined;
    /**
     * General description of how security works.
     */
    description?: string | undefined;
    _description?: Element | undefined;
    /**
     * Types of security services that are supported/required by the system.
     */
    service?: CodeableConcept[] | undefined;
}
/**
 * In general, a Resource will only appear in a CapabilityStatement if the server actually has some capabilities - e.g. there is at least one interaction supported. However interactions can be omitted to support summarization (_summary = true).
 */
export interface CapabilityStatementRestResourceInteraction extends BackboneElement {
    /**
     * Coded identifier of the operation, supported by the system resource.
     */
    code:
        | 'read'
        | 'vread'
        | 'update'
        | 'patch'
        | 'delete'
        | 'history-instance'
        | 'history-type'
        | 'create'
        | 'search-type';
    _code?: Element | undefined;
    /**
     * Guidance specific to the implementation of this operation, such as 'delete is a logical delete' or 'updates are only allowed with version id' or 'creates permitted from pre-authorized certificates only'.
     */
    documentation?: string | undefined;
    _documentation?: Element | undefined;
}
/**
 * The search parameters should include the control search parameters such as _sort, _count, etc. that also apply to this resource (though many will be listed at [CapabilityStatement.rest.searchParam](capabilitystatement-definitions.html#CapabilityStatement.rest.searchParam)). The behavior of some search parameters may be further described by other code or extension elements, or narrative within the capability statement or linked [SearchParameter](searchparameter.html#) definitions.
 */
export interface CapabilityStatementRestResourceSearchParam extends BackboneElement {
    /**
     * This SHOULD be present, and matches refers to a SearchParameter by its canonical URL. If systems wish to document their support for modifiers, comparators, target resource types, and chained parameters, they should do using a search parameter resource. This element SHALL be populated if the search parameter refers to a SearchParameter defined by the FHIR core specification or externally defined IGs.
     */
    definition?: string | undefined;
    _definition?: Element | undefined;
    /**
     * This allows documentation of any distinct behaviors about how the search parameter is used.  For example, text matching algorithms.
     */
    documentation?: string | undefined;
    _documentation?: Element | undefined;
    /**
     * Parameter names cannot overlap with standard parameter names, and standard parameters cannot be redefined. There is no correspondence whatsoever between CapabilityStatement's searchParam.name and SearchParameter.name - the latter is used as a class name when generating code for the search parameter.
     */
    name: string;
    _name?: Element | undefined;
    /**
     * While this can be looked up from the definition, it is included here as a convenience for systems that autogenerate a query interface based on the server capability statement.  It SHALL be the same as the type in the search parameter definition.
     */
    type: 'number' | 'date' | 'string' | 'token' | 'reference' | 'composite' | 'quantity' | 'uri' | 'special';
    _type?: Element | undefined;
}
/**
 * Operations linked from CapabilityStatement.rest.resource.operation must have OperationDefinition.type = true or OperationDefinition.instance = true.
 * If an operation that is listed in multiple CapabilityStatement.rest.resource.operation (e.g. for different resource types), then clients should understand that the operation is only supported on the specified resource types, and that may be a subset of those listed in OperationDefinition.resource.
 */
export interface CapabilityStatementRestResourceOperation extends BackboneElement {
    /**
     * This can be used to build an HTML form to invoke the operation, for instance.
     */
    definition: string;
    _definition?: Element | undefined;
    /**
     * Documentation that describes anything special about the operation behavior, possibly detailing different behavior for system, type and instance-level invocation of the operation.
     */
    documentation?: string | undefined;
    _documentation?: Element | undefined;
    /**
     * The name here SHOULD be the same as the OperationDefinition.code in the referenced OperationDefinition, unless there is a name clash and the OperationDefinition.code cannot be used. The name does not include the "$" portion that is always included in the URL. There is no correspondence whatsoever between CapabilityStatement's operation.name and OperationDefinition.name - the latter is used as a class name when generating code for the operation. HL7 will never define operations that have conflicting names.
     */
    name: string;
    _name?: Element | undefined;
}
/**
 * Max of one repetition per resource type.
 */
export interface CapabilityStatementRestResource extends BackboneElement {
    /**
     * Conditional Create is mainly appropriate for interface engine scripts converting from other formats, such as v2.
     */
    conditionalCreate?: boolean | undefined;
    _conditionalCreate?: Element | undefined;
    /**
     * Conditional Delete is mainly appropriate for interface engine scripts converting from other formats, such as v2.
     */
    conditionalDelete?: ('not-supported' | 'single' | 'multiple') | undefined;
    _conditionalDelete?: Element | undefined;
    /**
     * Conditional Patch is mainly appropriate for interface engine scripts converting from other formats, such as v2.
     */
    conditionalPatch?: boolean | undefined;
    _conditionalPatch?: Element | undefined;
    /**
     * Conditional Read is mainly appropriate for interface engine scripts converting from other formats, such as v2.
     */
    conditionalRead?: ('not-supported' | 'modified-since' | 'not-match' | 'full-support') | undefined;
    _conditionalRead?: Element | undefined;
    /**
     * Conditional Update is mainly appropriate for interface engine scripts converting from other formats, such as v2.
     */
    conditionalUpdate?: boolean | undefined;
    _conditionalUpdate?: Element | undefined;
    /**
     * Additional information about the resource type used by the system.
     */
    documentation?: string | undefined;
    _documentation?: Element | undefined;
    /**
     * In general, a Resource will only appear in a CapabilityStatement if the server actually has some capabilities - e.g. there is at least one interaction supported. However interactions can be omitted to support summarization (_summary = true).
     */
    interaction?: CapabilityStatementRestResourceInteraction[] | undefined;
    /**
     * Operations linked from CapabilityStatement.rest.resource.operation must have OperationDefinition.type = true or OperationDefinition.instance = true.
     * If an operation that is listed in multiple CapabilityStatement.rest.resource.operation (e.g. for different resource types), then clients should understand that the operation is only supported on the specified resource types, and that may be a subset of those listed in OperationDefinition.resource.
     */
    operation?: CapabilityStatementRestResourceOperation[] | undefined;
    /**
     * All other profiles for this type that are listed in `.rest.resource.supportedProfile` must conform to this profile.
     */
    profile?: string | undefined;
    _profile?: Element | undefined;
    /**
     * It is useful to support the vRead operation for current operations, even if past versions aren't available.
     */
    readHistory?: boolean | undefined;
    _readHistory?: Element | undefined;
    /**
     * A set of flags that defines how references are supported.
     */
    referencePolicy?: ('literal' | 'logical' | 'resolves' | 'enforced' | 'local')[] | undefined;
    _referencePolicy?: Element[] | undefined;
    /**
     * Documenting [`_include`](http://hl7.org/fhir/R5/search.html#revinclude) support helps set conformance expectations for the desired system. Still, it is a level of detail that might not be exposed by production servers or clients when using CapabilityStatement to describe an actual implementation. If this list is empty, the server does not support includes. Support for *iterative* (a.k.a., recursive) `_include` is communicated by listing the iterative includes values supported by the server in the `searchInclude` element of the "root" resource type. For example, to support the following search:
     * `GET [base]/CarePlan?_include=CarePlan:activity-reference:DeviceRequest&_include:iterate=DeviceRequest:device`
     * These values would be listed as part of capabilities for "CarePlan":
     * "searchInclude" : ["CarePlan:activity-reference:DeviceRequest","DeviceRequest:device"],
     *
     */
    searchInclude?: string[] | undefined;
    _searchInclude?: Element[] | undefined;
    /**
     * The search parameters should include the control search parameters such as _sort, _count, etc. that also apply to this resource (though many will be listed at [CapabilityStatement.rest.searchParam](capabilitystatement-definitions.html#CapabilityStatement.rest.searchParam)). The behavior of some search parameters may be further described by other code or extension elements, or narrative within the capability statement or linked [SearchParameter](searchparameter.html#) definitions.
     */
    searchParam?: CapabilityStatementRestResourceSearchParam[] | undefined;
    /**
     * See `CapabilityStatement.rest.resource.searchInclude` comments.
     */
    searchRevInclude?: string[] | undefined;
    _searchRevInclude?: Element[] | undefined;
    /**
     * Supported profiles must conform to the resource profile in the `.rest.resource.profile` element if it is present. The resource profile is a system-wide profile applied across *all* instances of the resource supported by the system. A supported profile is a statement about the functionality of the data and services provided by the server (or used by the client) for a particular set of use cases and will not necessarily apply to all data consumed or exposed by the server.
     */
    supportedProfile?: string[] | undefined;
    _supportedProfile?: Element[] | undefined;
    /**
     * A type of resource exposed via the restful interface.
     */
    type: string;
    _type?: Element | undefined;
    /**
     * Allowing the clients to create new identities on the server means that the system administrator needs to have confidence that the clients do not create clashing identities between them. Obviously, if there is only one client, this won't happen. While creating identities on the client means that the clients need to be managed, it's much more convenient for many scenarios if such management can be put in place.
     */
    updateCreate?: boolean | undefined;
    _updateCreate?: Element | undefined;
    /**
     * If a server supports versionIds correctly, it SHOULD support vread too, but is not required to do so.
     */
    versioning?: ('no-version' | 'versioned' | 'versioned-update') | undefined;
    _versioning?: Element | undefined;
}
/**
 * A specification of restful operations supported by the system.
 */
export interface CapabilityStatementRestInteraction extends BackboneElement {
    /**
     * A coded identifier of the operation, supported by the system.
     */
    code: 'transaction' | 'batch' | 'search-system' | 'history-system';
    _code?: Element | undefined;
    /**
     * Guidance specific to the implementation of this operation, such as limitations on the kind of transactions allowed, or information about system wide search is implemented.
     */
    documentation?: string | undefined;
    _documentation?: Element | undefined;
}
/**
 * Multiple repetitions allow definition of both client and/or server behaviors or possibly behaviors under different configuration settings (for software or requirements statements).
 */
export interface CapabilityStatementRest extends BackboneElement {
    /**
     * At present, the only defined compartments are at [CompartmentDefinition](compartmentdefinition.html).
     */
    compartment?: string[] | undefined;
    _compartment?: Element[] | undefined;
    /**
     * Information about the system's restful capabilities that apply across all applications, such as security.
     */
    documentation?: string | undefined;
    _documentation?: Element | undefined;
    /**
     * A specification of restful operations supported by the system.
     */
    interaction?: CapabilityStatementRestInteraction[] | undefined;
    /**
     * Identifies whether this portion of the statement is describing the ability to initiate or receive restful operations.
     */
    mode: 'client' | 'server';
    _mode?: Element | undefined;
    /**
     * CapabilityStatement.rest.operation is for operations invoked at the system level, or for operations that are supported across multiple resource types. Operations linked from CapabilityStatement.rest.operation must have OperationDefinition.system = true, or more than one Operation.resource.
     */
    operation?: CapabilityStatementRestResourceOperation[] | undefined;
    /**
     * Max of one repetition per resource type.
     */
    resource?: CapabilityStatementRestResource[] | undefined;
    /**
     * Typically, the only search parameters supported for all searches are those that apply to all resources - tags, profiles, text search etc. These search parameters should include the control search parameters such as _sort, _count, etc. that also apply to this resource (though many will be listed at [CapabilityStatement.rest.searchParam](capabilitystatement-definitions.html#CapabilityStatement.rest.searchParam)). The behavior of some search parameters may be further described by other code or extension elements, or narrative within the capability statement or linked [SearchParameter](searchparameter.html#) definitions.
     */
    searchParam?: CapabilityStatementRestResourceSearchParam[] | undefined;
    /**
     * Information about security implementation from an interface perspective - what a client needs to know.
     */
    security?: CapabilityStatementRestSecurity | undefined;
}
/**
 * An endpoint (network accessible address) to which messages and/or replies are to be sent.
 */
export interface CapabilityStatementMessagingEndpoint extends BackboneElement {
    /**
     * The network address of the endpoint. For solutions that do not use network addresses for routing, it can be just an identifier.
     */
    address: string;
    _address?: Element | undefined;
    /**
     * A list of the messaging transport protocol(s) identifiers, supported by this endpoint.
     */
    protocol: Coding;
}
/**
 * This is a proposed alternative to the messaging.event structure.
 */
export interface CapabilityStatementMessagingSupportedMessage extends BackboneElement {
    /**
     * Points to a message definition that identifies the messaging event, message structure, allowed responses, etc.
     */
    definition: string;
    _definition?: Element | undefined;
    /**
     * The mode of this event declaration - whether application is sender or receiver.
     */
    mode: 'sender' | 'receiver';
    _mode?: Element | undefined;
}
/**
 * Multiple repetitions allow the documentation of multiple endpoints per solution.
 */
export interface CapabilityStatementMessaging extends BackboneElement {
    /**
     * Documentation about the system's messaging capabilities for this endpoint not otherwise documented by the capability statement.  For example, the process for becoming an authorized messaging exchange partner.
     */
    documentation?: string | undefined;
    _documentation?: Element | undefined;
    /**
     * An endpoint (network accessible address) to which messages and/or replies are to be sent.
     */
    endpoint?: CapabilityStatementMessagingEndpoint[] | undefined;
    /**
     * If this value is missing then the application does not implement (receiver) or depend on (sender) reliable messaging.
     */
    reliableCache?: number | undefined;
    /**
     * This is a proposed alternative to the messaging.event structure.
     */
    supportedMessage?: CapabilityStatementMessagingSupportedMessage[] | undefined;
}
/**
 * A document definition.
 */
export interface CapabilityStatementDocument extends BackboneElement {
    /**
     * A description of how the application supports or uses the specified document profile.  For example, when documents are created, what action is taken with consumed documents, etc.
     */
    documentation?: string | undefined;
    _documentation?: Element | undefined;
    /**
     * Mode of this document declaration - whether an application is a producer or consumer.
     */
    mode: 'producer' | 'consumer';
    _mode?: Element | undefined;
    /**
     * The profile is actually on the Bundle.
     */
    profile: string;
    _profile?: Element | undefined;
}
/**
 * A Capability Statement documents a set of capabilities (behaviors) of a FHIR Server or Client for a particular version of FHIR that may be used as a statement of actual server functionality or a statement of required or desired server implementation.
 */
export interface CapabilityStatement extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'CapabilityStatement';
    /**
     * In general, if a server gets a request with an Accept-Language that it doesn't support, it should still reutrn the resource, just in it's default language for the resource.
     */
    acceptLanguage?: string[] | undefined;
    _acceptLanguage?: Element[] | undefined;
    /**
     * May be a web site, an email address, a telephone number, etc.
     */
    contact?: ContactDetail[] | undefined;
    /**
     * ...
     */
    copyright?: string | undefined;
    _copyright?: Element | undefined;
    /**
     * The (c) symbol should NOT be included in this string. It will be added by software when rendering the notation. Full details about licensing, restrictions, warrantees, etc. goes in the more general 'copyright' element.
     */
    copyrightLabel?: string | undefined;
    _copyrightLabel?: Element | undefined;
    /**
     * Note that this is not the same as the resource last-modified-date, since the resource may be a secondary representation of the capability statement. Additional specific dates may be added as extensions or be found by consulting Provenances associated with past versions of the resource.
     */
    date: string;
    _date?: Element | undefined;
    /**
     * This description can be used to capture details such as comments about misuse, instructions for clinical use and interpretation, literature references, examples from the paper world, etc. It is not a rendering of the capability statement as conveyed in the 'text' field of the resource itself. This item SHOULD be populated unless the information is available from context (e.g. the language of the capability statement is presumed to be the predominant language in the place the capability statement was created).This does not need to be populated if the description is adequately implied by the software or implementation details.
     */
    description?: string | undefined;
    _description?: Element | undefined;
    /**
     * A document definition.
     */
    document?: CapabilityStatementDocument[] | undefined;
    /**
     * Allows filtering of capability statements that are appropriate for use versus not.
     */
    experimental?: boolean | undefined;
    _experimental?: Element | undefined;
    /**
     * Servers may implement multiple versions (see [Managing Multiple Versions](versioning.html), and the [$versions](capabilitystatement-operation-versions.html) operation). If they do, and the CapabilityStatement is requested from the server, then this fhirVersion will be either the version requested, or the server's default version.
     */
    fhirVersion: string;
    _fhirVersion?: Element | undefined;
    /**
     * "xml", "json" and "ttl" are allowed, which describe the simple encodings described in the specification (and imply appropriate bundle support). Otherwise, mime types are legal here.
     */
    format: string[];
    _format?: Element[] | undefined;
    /**
     * Identifies a specific implementation instance that is described by the capability statement - i.e. a particular installation, rather than the capabilities of a software program.
     */
    implementation?: CapabilityStatementImplementation | undefined;
    /**
     * Note: this is primarily only relevant in terms of ImplementationGuides that don't define specific CapabilityStatements declaring the expectation of distinct roles.  (E.g. generic IGs that establish privacy policies.)  If CapabilityStatements are defined by the IG, then it is better to use CapabilityStatement.instantiates to identify the specific role(s) supported.
     */
    implementationGuide?: string[] | undefined;
    _implementationGuide?: Element[] | undefined;
    /**
     * the contents of any directly or indirectly imported CapabilityStatements SHALL NOT overlap, i.e. they cannot refer to the same rest/resource, operations/name, searchparam/name, interaction/code, messaging/endpoint, document/mode pair.
     * A capability statement that imports another CapabilityStatement automatically instantiates it too (though this is often not a very useful statement for the kinds of CapabilityStatements that are suitable for importing).
     */
    imports?: string[] | undefined;
    _imports?: Element[] | undefined;
    /**
     * HL7 defines the following Services: [Terminology Service](terminology-service.html).
     * Many [Implementation Guides](http://fhir.org/guides/registry) define additional services.
     */
    instantiates?: string[] | undefined;
    _instantiates?: Element[] | undefined;
    /**
     * It may be possible for the capability statement to be used in jurisdictions other than those for which it was originally designed or intended.
     */
    jurisdiction?: CodeableConcept[] | undefined;
    /**
     * The way that this statement is intended to be used, to describe an actual running instance of software, a particular product (kind, not instance of software) or a class of implementation (e.g. a desired purchase).
     */
    kind: 'instance' | 'capability' | 'requirements';
    _kind?: Element | undefined;
    /**
     * Multiple repetitions allow the documentation of multiple endpoints per solution.
     */
    messaging?: CapabilityStatementMessaging[] | undefined;
    /**
     * The name is not expected to be globally unique. The name should be a simple alphanumeric type name to ensure that it is machine-processing friendly.
     */
    name?: string | undefined;
    _name?: Element | undefined;
    /**
     * At present, the patch mime types application/json-patch+json and application/xml-patch+xml are legal. Generally, if a server supports PATCH, it would be expected to support the patch formats and match the formats it supports, but this is not always possible or necessary.
     */
    patchFormat?: string[] | undefined;
    _patchFormat?: Element[] | undefined;
    /**
     * Usually an organization but may be an individual. The publisher (or steward) of the capability statement is the organization or individual primarily responsible for the maintenance and upkeep of the capability statement. This is not necessarily the same individual or organization that developed and initially authored the content. The publisher is the primary point of contact for questions or issues with the capability statement. This item SHOULD be populated unless the information is available from context.
     */
    publisher?: string | undefined;
    _publisher?: Element | undefined;
    /**
     * This element does not describe the usage of the capability statement. Instead, it provides traceability of ''why'' the resource is either needed or ''why'' it is defined as it is.  This may be used to point to source materials or specifications that drove the structure of this capability statement.
     */
    purpose?: string | undefined;
    _purpose?: Element | undefined;
    /**
     * Multiple repetitions allow definition of both client and/or server behaviors or possibly behaviors under different configuration settings (for software or requirements statements).
     */
    rest?: CapabilityStatementRest[] | undefined;
    /**
     * Software that is covered by this capability statement.  It is used when the capability statement describes the capabilities of a particular software version, independent of an installation.
     */
    software?: CapabilityStatementSoftware | undefined;
    /**
     * Allows filtering of capability statements that are appropriate for use versus not.This is not intended for use with actual capability statements, but where capability statements are used to describe possible or desired systems.
     */
    status: 'draft' | 'active' | 'retired' | 'unknown';
    _status?: Element | undefined;
    /**
     * This name does not need to be machine-processing friendly and may contain punctuation, white-space, etc.
     */
    title?: string | undefined;
    _title?: Element | undefined;
    /**
     * Can be a urn:uuid: or a urn:oid: but real http: addresses are preferred.  Multiple instances may share the same URL if they have a distinct version.
     * The determination of when to create a new version of a resource (same url, new version) vs. defining a new artifact is up to the author.  Considerations for making this decision are found in [Technical and Business Versions](resource.html#versions).
     * In some cases, the resource can no longer be found at the stated url, but the url itself cannot change. Implementations can use the [meta.source](resource.html#meta) element to indicate where the current master source of the resource can be found.
     */
    url?: string | undefined;
    _url?: Element | undefined;
    /**
     * When multiple useContexts are specified, there is no expectation that all or any of the contexts apply.
     */
    useContext?: UsageContext[] | undefined;
    /**
     * There may be different capability statement instances that have the same identifier but different versions.  The version can be appended to the url in a reference to allow a reference to a particular business version of the capability statement with the format [url]|[version]. The version SHOULD NOT contain a '#' - see [Business Version](resource.html#bv-format).
     */
    version?: string | undefined;
    _version?: Element | undefined;
    /**
     * If set as a string, this is a FHIRPath expression that has two additional context variables passed in - %version1 and %version2 and will return a negative number if version1 is newer, a positive number if version2 and a 0 if the version ordering can't be successfully be determined.
     */
    versionAlgorithmString?: string | undefined;
    _versionAlgorithmString?: Element | undefined;
    /**
     * If set as a string, this is a FHIRPath expression that has two additional context variables passed in - %version1 and %version2 and will return a negative number if version1 is newer, a positive number if version2 and a 0 if the version ordering can't be successfully be determined.
     */
    versionAlgorithmCoding?: Coding | undefined;
}
/**
 * A simple summary of a planned activity suitable for a general care plan system (e.g. form driven) that doesn't know about specific resources such as procedure etc.
 */
export interface CarePlanActivityPlannedActivityDetail extends BackboneElement {
    /**
     * Tends to be less relevant for activities involving particular products.  Codes should not convey negation - use "prohibited" instead.
     */
    code?: CodeableConcept | undefined;
    /**
     * Identifies the quantity expected to be consumed in a given day.
     */
    dailyAmount?: Quantity | undefined;
    /**
     * This provides a textual description of constraints on the intended activity occurrence, including relation to other activities.  It may also include objectives, pre-conditions and end-conditions.  Finally, it may convey specifics about the activity such as body site, method, route, etc.
     */
    description?: string | undefined;
    _description?: Element | undefined;
    /**
     * This element is labeled as a modifier because it marks an activity as an activity that is not to be performed.
     */
    doNotPerform?: boolean | undefined;
    _doNotPerform?: Element | undefined;
    /**
     * Internal reference that identifies the goals that this activity is intended to contribute towards meeting.
     */
    goal?: Reference[] | undefined;
    /**
     * The URL pointing to a FHIR-defined protocol, guideline, questionnaire or other definition that is adhered to in whole or in part by this CarePlan activity.
     */
    instantiatesCanonical?: string[] | undefined;
    _instantiatesCanonical?: Element[] | undefined;
    /**
     * This might be an HTML page, PDF, etc. or could just be a non-resolvable URI identifier.
     */
    instantiatesUri?: string[] | undefined;
    _instantiatesUri?: Element[] | undefined;
    /**
     * A description of the kind of resource the in-line definition of a care plan activity is representing.  The CarePlan.activity.detail is an in-line definition when a resource is not referenced using CarePlan.activity.reference.  For example, a MedicationRequest, a ServiceRequest, or a CommunicationRequest.
     */
    kind?:
        | (
              | 'Appointment'
              | 'CommunicationRequest'
              | 'DeviceRequest'
              | 'MedicationRequest'
              | 'NutritionOrder'
              | 'Task'
              | 'ServiceRequest'
              | 'VisionPrescription'
          )
        | undefined;
    _kind?: Element | undefined;
    /**
     * May reference a specific clinical location or may identify a type of location.
     */
    location?: CodeableReference | undefined;
    /**
     * A performer MAY also be a participant in the care plan.
     */
    performer?: Reference[] | undefined;
    /**
     * Identifies the food, drug or other product to be consumed or supplied in the activity.
     */
    productCodeableConcept?: CodeableConcept | undefined;
    /**
     * Identifies the food, drug or other product to be consumed or supplied in the activity.
     */
    productReference?: Reference | undefined;
    /**
     * Identifies the quantity expected to be supplied, administered or consumed by the subject.
     */
    quantity?: Quantity | undefined;
    /**
     * This could be a diagnosis code.  If a full condition record exists, us reason.reference instead. Conditions can be identified at the activity level that are not identified as reasons for the overall plan.
     */
    reason?: CodeableReference[] | undefined;
    /**
     * Indicates if this record was captured as a secondary 'reported' record rather than as an original primary source-of-truth record.  It may also indicate the source of the report.
     */
    reportedBoolean?: boolean | undefined;
    _reportedBoolean?: Element | undefined;
    /**
     * Indicates if this record was captured as a secondary 'reported' record rather than as an original primary source-of-truth record.  It may also indicate the source of the report.
     */
    reportedReference?: Reference | undefined;
    /**
     * The period, timing or frequency upon which the described activity is to occur.
     */
    scheduledTiming?: Timing | undefined;
    /**
     * The period, timing or frequency upon which the described activity is to occur.
     */
    scheduledPeriod?: Period | undefined;
    /**
     * The period, timing or frequency upon which the described activity is to occur.
     */
    scheduledString?: string | undefined;
    _scheduledString?: Element | undefined;
    /**
     * Some aspects of status can be inferred based on the resources linked in actionTaken.  Note that "status" is only as current as the plan was most recently updated.
     * The unknown code is not to be used to convey other statuses.  The unknown code should be used when one of the statuses applies, but the authoring system doesn't know the current state of the activity.
     */
    status:
        | 'not-started'
        | 'scheduled'
        | 'in-progress'
        | 'on-hold'
        | 'completed'
        | 'cancelled'
        | 'stopped'
        | 'unknown'
        | 'entered-in-error';
    _status?: Element | undefined;
    /**
     * Will generally not be present if status is "complete".  Be sure to prompt to update this (or at least remove the existing value) if the status is changed.
     */
    statusReason?: CodeableConcept | undefined;
}
/**
 * Identifies an action that has occurred or is a planned action to occur as part of the plan. For example, a medication to be used, lab tests to perform, self-monitoring that has occurred, education etc.
 */
export interface CarePlanActivity extends BackboneElement {
    /**
     * Note that this should not duplicate the activity status (e.g. completed or in progress). The activity performed is independent of the outcome of the related goal(s). For example, if the goal is to achieve a target body weight of 150 lbs and an activity is defined to exercise, then the activity performed could be amount and intensity of exercise performed whereas the goal outcome is an observation for the actual body weight measured.
     */
    performedActivity?: CodeableReference[] | undefined;
    /**
     * A simple summary of a planned activity suitable for a general care plan system (e.g. form driven) that doesn't know about specific resources such as procedure etc.
     */
    plannedActivityDetail?: CarePlanActivityPlannedActivityDetail | undefined;
    /**
     * Standard extension exists ([resource-pertainsToGoal](extension-resource-pertainstogoal.html)) that allows goals to be referenced from any of the referenced resources in CarePlan.activity.plannedActivityReference.
     * The goal should be visible when the resource referenced by CarePlan.activity.plannedActivityReference is viewed independently from the CarePlan.  Requests that are pointed to by a CarePlan using this element should *not* point to this CarePlan using the "basedOn" element.  i.e. Requests that are part of a CarePlan are not "based on" the CarePlan.
     */
    plannedActivityReference?: Reference | undefined;
    /**
     * This element should NOT be used to describe the activity to be performed - that occurs either within the resource pointed to by activity.detail.reference or in activity.detail.description.
     */
    progress?: Annotation[] | undefined;
}
/**
 * Describes the intention of how one or more practitioners intend to deliver care for a particular patient, group or community for a period of time, possibly limited to care for a specific condition or set of conditions.
 */
export interface CarePlan extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'CarePlan';
    /**
     * Identifies an action that has occurred or is a planned action to occur as part of the plan. For example, a medication to be used, lab tests to perform, self-monitoring that has occurred, education etc.
     */
    activity?: CarePlanActivity[] | undefined;
    /**
     * Use CarePlan.addresses.concept when a code sufficiently describes the concern (e.g. condition, problem, diagnosis, risk). Use CarePlan.addresses.reference when referencing a resource, which allows more information to be conveyed, such as onset date. CarePlan.addresses.concept and CarePlan.addresses.reference are not meant to be duplicative. For a single concern, either CarePlan.addresses.concept or CarePlan.addresses.reference can be used. CarePlan.addresses.concept may be a summary code, or CarePlan.addresses.reference may be used to reference a very precise definition of the concern using Condition. Both CarePlan.addresses.concept and CarePlan.addresses.reference can be used if they are describing different concerns for the care plan.
     */
    addresses?: CodeableReference[] | undefined;
    /**
     * A care plan that is fulfilled in whole or in part by this care plan.
     */
    basedOn?: Reference[] | undefined;
    /**
     * Identifies all people and organizations who are expected to be involved in the care envisioned by this plan.
     */
    careTeam?: Reference[] | undefined;
    /**
     * There may be multiple axes of categorization and one plan may serve multiple purposes.  In some cases, this may be redundant with references to CarePlan.concern.
     */
    category?: CodeableConcept[] | undefined;
    /**
     * Collaborative care plans may have multiple contributors.
     */
    contributor?: Reference[] | undefined;
    /**
     * Represents when this particular CarePlan record was created in the system, which is often a system-generated date.
     */
    created?: string | undefined;
    _created?: Element | undefined;
    /**
     * The custodian might or might not be a contributor.
     */
    custodian?: Reference | undefined;
    /**
     * A description of the scope and nature of the plan.
     */
    description?: string | undefined;
    _description?: Element | undefined;
    /**
     * This will typically be the encounter the event occurred within, but some activities may be initiated prior to or after the official completion of an encounter but still be tied to the context of the encounter. CarePlan activities conducted as a result of the care plan may well occur as part of other encounters.
     */
    encounter?: Reference | undefined;
    /**
     * Goal can be achieving a particular change or merely maintaining a current state or even slowing a decline.
     */
    goal?: Reference[] | undefined;
    /**
     * This is a business identifier, not a resource identifier (see [discussion](resource.html#identifiers)).  It is best practice for the identifier to only appear on a single resource instance, however business practices may occasionally dictate that multiple resource instances with the same identifier can exist - possibly even with different resource types.  For example, multiple Patient and a Person resource instance might share the same social insurance number.
     */
    identifier?: Identifier[] | undefined;
    /**
     * The URL pointing to a FHIR-defined protocol, guideline, questionnaire or other definition that is adhered to in whole or in part by this CarePlan.
     */
    instantiatesCanonical?: string[] | undefined;
    _instantiatesCanonical?: Element[] | undefined;
    /**
     * This might be an HTML page, PDF, etc. or could just be a non-resolvable URI identifier.
     */
    instantiatesUri?: string[] | undefined;
    _instantiatesUri?: Element[] | undefined;
    /**
     * This element is labeled as a modifier because the intent alters when and how the resource is actually applicable.
     * This element is expected to be immutable. E.g. A "proposal" instance should never change to be a "plan" instance or "order" instance. Instead, a new instance 'basedOn' the prior instance should be created with the new 'intent' value.
     */
    intent: 'proposal' | 'plan' | 'order' | 'option' | 'directive';
    _intent?: Element | undefined;
    /**
     * General notes about the care plan not covered elsewhere.
     */
    note?: Annotation[] | undefined;
    /**
     * Each care plan is an independent request, such that having a care plan be part of another care plan can cause issues with cascading statuses.  As such, this element is still being discussed.
     */
    partOf?: Reference[] | undefined;
    /**
     * Any activities scheduled as part of the plan should be constrained to the specified period regardless of whether the activities are planned within a single encounter/episode or across multiple encounters/episodes (e.g. the longitudinal management of a chronic condition).
     */
    period?: Period | undefined;
    /**
     * The replacement could be because the initial care plan was immediately rejected (due to an issue) or because the previous care plan was completed, but the need for the action described by the care plan remains ongoing.
     */
    replaces?: Reference[] | undefined;
    /**
     * The unknown code is not to be used to convey other statuses.  The unknown code should be used when one of the statuses applies, but the authoring system doesn't know the current state of the care plan.
     * This element is labeled as a modifier because the status contains the code entered-in-error that marks the plan as not currently valid.
     */
    status: 'draft' | 'active' | 'on-hold' | 'revoked' | 'completed' | 'entered-in-error' | 'unknown';
    _status?: Element | undefined;
    /**
     * Identifies the patient or group whose intended care is described by the plan.
     */
    subject: Reference;
    /**
     * Use "concern" to identify specific conditions addressed by the care plan.  supportingInfo can be used to convey one or more Advance Directives or Medical Treatment Consent Directives by referencing Consent or any other request resource with intent = directive.
     */
    supportingInfo?: Reference[] | undefined;
    /**
     * Human-friendly name for the care plan.
     */
    title?: string | undefined;
    _title?: Element | undefined;
}
/**
 * Identifies all people and organizations who are expected to be involved in the care team.
 */
export interface CareTeamParticipant extends BackboneElement {
    /**
     * This is populated while creating / managing the CareTeam to ensure there is coverage when servicing CarePlan activities from the Schedule.
     */
    coveragePeriod?: Period | undefined;
    /**
     * This is populated while creating / managing the CareTeam to ensure there is coverage when servicing CarePlan activities from the Schedule.
     */
    coverageTiming?: Timing | undefined;
    /**
     * Patient only needs to be listed if they have a role other than "subject of care".
     * Member is optional because some participants may be known only by their role, particularly in draft plans.
     */
    member?: Reference | undefined;
    /**
     * The organization of the practitioner.
     */
    onBehalfOf?: Reference | undefined;
    /**
     * Roles may sometimes be inferred by type of Practitioner.  These are relationships that hold only within the context of the care team.  General relationships should be handled as properties of the Patient resource directly.  If a participant has multiple roles within the team, then there should be multiple participants.
     */
    role?: CodeableConcept | undefined;
}
/**
 * The Care Team includes all the people and organizations who plan to participate in the coordination and delivery of care.
 */
export interface CareTeam extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'CareTeam';
    /**
     * There may be multiple axis of categorization and one team may serve multiple purposes.
     */
    category?: CodeableConcept[] | undefined;
    /**
     * This is a business identifier, not a resource identifier (see [discussion](resource.html#identifiers)).  It is best practice for the identifier to only appear on a single resource instance, however business practices may occasionally dictate that multiple resource instances with the same identifier can exist - possibly even with different resource types.  For example, multiple Patient and a Person resource instance might share the same social insurance number.
     */
    identifier?: Identifier[] | undefined;
    /**
     * The organization responsible for the care team.
     */
    managingOrganization?: Reference[] | undefined;
    /**
     * The meaning/purpose of the team is conveyed in CareTeam.category.  This element may also convey semantics of the team (e.g. "Red trauma team"), but its primary purpose is to distinguish between identical teams in a human-friendly way.  ("Team 18735" isn't as friendly.).
     */
    name?: string | undefined;
    _name?: Element | undefined;
    /**
     * Comments made about the CareTeam.
     */
    note?: Annotation[] | undefined;
    /**
     * Identifies all people and organizations who are expected to be involved in the care team.
     */
    participant?: CareTeamParticipant[] | undefined;
    /**
     * Indicates when the team did (or is intended to) come into effect and end.
     */
    period?: Period | undefined;
    /**
     * Describes why the care team exists.
     */
    reason?: CodeableReference[] | undefined;
    /**
     * This element is labeled as a modifier because the status contains the code entered-in-error that marks the care team as not currently valid.
     */
    status?: ('proposed' | 'active' | 'suspended' | 'inactive' | 'entered-in-error') | undefined;
    _status?: Element | undefined;
    /**
     * Identifies the patient or group whose intended care is handled by the team.
     */
    subject?: Reference | undefined;
    /**
     * The ContactPoint.use code of home is not appropriate to use. These contacts are not the contact details of individual care team members.
     */
    telecom?: ContactPoint[] | undefined;
}
/**
 * Indicates who or what performed or participated in the charged service.
 */
export interface ChargeItemPerformer extends BackboneElement {
    /**
     * The device, practitioner, etc. who performed or participated in the service.
     */
    actor: Reference;
    /**
     * Describes the type of performance or participation(e.g. primary surgeon, anesthesiologiest, etc.).
     */
    function?: CodeableConcept | undefined;
}
/**
 * The resource ChargeItem describes the provision of healthcare provider products for a certain patient, therefore referring not only to the product, but containing in addition details of the provision, like date, time, amounts and participating organizations and persons. Main Usage of the ChargeItem is to enable the billing process and internal cost allocation.
 */
export interface ChargeItem extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'ChargeItem';
    /**
     * Systems posting the ChargeItems might not always be able to determine, which accounts the Items need to be places into. It is up to the postprocessing Financial System to apply internal rules to decide based on the Encounter/EpisodeOfCare/Patient/Coverage context and the type of ChargeItem, which Account is appropriate.
     */
    account?: Reference[] | undefined;
    /**
     * Only used if not implicit in code found in Condition.code. If the use case requires attributes from the BodySite resource (e.g. to identify and track separately) then use the standard extension [bodySite](extension-bodysite.html).  May be a summary code, or a reference to a very precise definition of the location, or both.
     */
    bodysite?: CodeableConcept[] | undefined;
    /**
     * A code that identifies the charge, like a billing code.
     */
    code: CodeableConcept;
    /**
     * The costCenter could either be given as a reference to an Organization(Role) resource or as the identifier of the cost center determined by Reference.identifier.value and Reference.identifier.system, depending on use case requirements.
     */
    costCenter?: Reference | undefined;
    /**
     * References the source of pricing information, rules of application for the code this ChargeItem uses.
     */
    definitionCanonical?: string[] | undefined;
    _definitionCanonical?: Element[] | undefined;
    /**
     * References the (external) source of pricing information, rules of application for the code this ChargeItem uses.
     */
    definitionUri?: string[] | undefined;
    _definitionUri?: Element[] | undefined;
    /**
     * This ChargeItem may be recorded during planning, execution or after the actual encounter takes place.
     */
    encounter?: Reference | undefined;
    /**
     * The actual date when the service associated with the charge has been rendered is captured in occurrence[x].
     */
    enteredDate?: string | undefined;
    _enteredDate?: Element | undefined;
    /**
     * The enterer is also the person considered responsible for factor/price overrides if applicable.
     */
    enterer?: Reference | undefined;
    /**
     * Identifiers assigned to this event performer or other systems.
     */
    identifier?: Identifier[] | undefined;
    /**
     * Comments made about the event by the performer, subject or other participants.
     */
    note?: Annotation[] | undefined;
    /**
     * The list of types may be constrained as appropriate for the type of charge item.
     */
    occurrenceDateTime?: string | undefined;
    _occurrenceDateTime?: Element | undefined;
    /**
     * The list of types may be constrained as appropriate for the type of charge item.
     */
    occurrencePeriod?: Period | undefined;
    /**
     * The list of types may be constrained as appropriate for the type of charge item.
     */
    occurrenceTiming?: Timing | undefined;
    /**
     * If the list price or the rule-based factor associated with the code is overridden, this attribute can capture a text to indicate the  reason for this action.
     */
    overrideReason?: CodeableConcept | undefined;
    /**
     * ChargeItems can be grouped to larger ChargeItems covering the whole set.
     */
    partOf?: Reference[] | undefined;
    /**
     * Indicates who or what performed or participated in the charged service.
     */
    performer?: ChargeItemPerformer[] | undefined;
    /**
     * Practitioners and Devices can be associated with multiple organizations. It has to be made clear, on behalf of which Organization the services have been rendered.
     */
    performingOrganization?: Reference | undefined;
    /**
     * Identifies the device, food, drug or other product being charged either by type code or reference to an instance.
     */
    product?: CodeableReference[] | undefined;
    /**
     * In many cases this may just be a value, if the underlying units are implicit in the definition of the charge item code.
     */
    quantity?: Quantity | undefined;
    /**
     * If the application of the charge item requires a reason to be given, it can be captured here. Textual reasons can be captured using reasonCode.text.
     */
    reason?: CodeableConcept[] | undefined;
    /**
     * The rendered Service might not be associated with a Request. This property indicates which Organization requested the services to be rendered. (In many cases, this may just be the Department associated with the Encounter.location).
     */
    requestingOrganization?: Reference | undefined;
    /**
     * Indicated the rendered service that caused this charge.
     */
    service?: CodeableReference[] | undefined;
    /**
     * Unknown does not represent "other" - one of the defined statuses must apply.  Unknown is used when the authoring system is not sure what the current status is.
     * This element is labeled as a modifier because the status contains the code entered-in-error that marks the charge item as not currently valid.
     */
    status: 'planned' | 'billable' | 'not-billable' | 'aborted' | 'billed' | 'entered-in-error' | 'unknown';
    _status?: Element | undefined;
    /**
     * The individual or set of individuals the action is being or was performed on.
     */
    subject: Reference;
    /**
     * Further information supporting this charge.
     */
    supportingInformation?: Reference[] | undefined;
    /**
     *  Often, the total price may be be calculated and recorded on the Invoice, but if a calculated total price must be exchanged prior to Invoice creation, it can be communicated in this property.
     */
    totalPriceComponent?: MonetaryComponent | undefined;
    /**
     * This could be communicated in ChargeItemDefinition. But if ChargeItemDefinition is not supported or not available, the unit price can be communicated in this property.
     */
    unitPriceComponent?: MonetaryComponent | undefined;
}
/**
 * The applicability conditions can be used to ascertain whether a billing item is allowed in a specific context. E.g. some billing codes may only be applicable in out-patient settings, only to male/female patients or only to children.
 */
export interface ChargeItemDefinitionApplicability extends BackboneElement {
    /**
     * Please note that FHIRPath Expressions can only be evaluated in the scope of the current ChargeItem resource to which this definition is being applied.
     * FHIRPath expressions can traverse into other resources linked from the ChargeItem resource, however, testing rules such as that a billing code may be billed only once per encounter need a wider scope. In such scenarios, CQL may be the appropriate choice.
     */
    condition?: Expression | undefined;
    /**
     * The effective period for a charge item definition  determines when the content is applicable for usage and is independent of publication and review dates. For example, a measure intended to be used for the year 2016 might be published in 2015.
     */
    effectivePeriod?: Period | undefined;
    /**
     * Reference to / quotation of the external source of the group of properties.
     */
    relatedArtifact?: RelatedArtifact | undefined;
}
/**
 * The price for a ChargeItem may be calculated as a base price with surcharges/deductions that apply in certain conditions. A ChargeItemDefinition resource that defines the prices, factors and conditions that apply to a billing code is currently under development. The priceComponent element can be used to offer transparency to the recipient of the Invoice of how the prices have been calculated.
 */
export interface ChargeItemDefinitionPropertyGroupPriceComponent extends MonetaryComponent {
    /**
     * The amount calculated for this component.
     */
    amount?: Money | undefined;
    /**
     * A code that identifies the component. Codes may be used to differentiate between kinds of taxes, surcharges, discounts etc.
     */
    code?: CodeableConcept | undefined;
    /**
     * The factor that has been applied on the base price for calculating this component.
     */
    factor?: number | undefined;
    /**
     * This code identifies the type of the component.
     */
    type: 'base' | 'surcharge' | 'deduction' | 'discount' | 'tax' | 'informational';
    _type?: Element | undefined;
}
/**
 * Group of properties which are applicable under the same conditions. If no applicability rules are established for the group, then all properties always apply.
 */
export interface ChargeItemDefinitionPropertyGroup extends BackboneElement {
    /**
     * The applicability conditions can be used to ascertain whether a billing item is allowed in a specific context. E.g. some billing codes may only be applicable in out-patient settings, only to male/female patients or only to children.
     */
    applicability?: ChargeItemDefinitionApplicability[] | undefined;
    /**
     * The price for a ChargeItem may be calculated as a base price with surcharges/deductions that apply in certain conditions. A ChargeItemDefinition resource that defines the prices, factors and conditions that apply to a billing code is currently under development. The priceComponent element can be used to offer transparency to the recipient of the Invoice of how the prices have been calculated.
     */
    priceComponent?: ChargeItemDefinitionPropertyGroupPriceComponent[] | undefined;
}
/**
 * The ChargeItemDefinition resource provides the properties that apply to the (billing) codes necessary to calculate costs and prices. The properties may differ largely depending on type and realm, therefore this resource gives only a rough structure and requires profiling for each type of billing code system.
 */
export interface ChargeItemDefinition extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'ChargeItemDefinition';
    /**
     * The applicability conditions can be used to ascertain whether a billing item is allowed in a specific context. E.g. some billing codes may only be applicable in out-patient settings, only to male/female patients or only to children.
     */
    applicability?: ChargeItemDefinitionApplicability[] | undefined;
    /**
     * The 'date' element may be more recent than the approval date because of minor changes or editorial corrections.
     */
    approvalDate?: string | undefined;
    _approvalDate?: Element | undefined;
    /**
     * The defined billing details in this resource pertain to the given billing code.
     */
    code?: CodeableConcept | undefined;
    /**
     * May be a web site, an email address, a telephone number, etc.
     */
    contact?: ContactDetail[] | undefined;
    /**
     * A copyright statement relating to the charge item definition and/or its contents. Copyright statements are generally legal restrictions on the use and publishing of the charge item definition.
     */
    copyright?: string | undefined;
    _copyright?: Element | undefined;
    /**
     * Note that this is not the same as the resource last-modified-date, since the resource may be a secondary representation of the charge item definition. Additional specific dates may be added as extensions or be found by consulting Provenances associated with past versions of the resource.
     */
    date?: string | undefined;
    _date?: Element | undefined;
    /**
     * The URL pointing to an externally-defined charge item definition that is adhered to in whole or in part by this definition.
     */
    derivedFromUri?: string[] | undefined;
    _derivedFromUri?: Element[] | undefined;
    /**
     * This description can be used to capture details such as comments about misuse, instructions for clinical use and interpretation, literature references, examples from the paper world, etc. It is not a rendering of the charge item definition as conveyed in the 'text' field of the resource itself. This item SHOULD be populated unless the information is available from context (e.g. the language of the charge item definition is presumed to be the predominant language in the place the charge item definition was created).
     */
    description?: string | undefined;
    _description?: Element | undefined;
    /**
     * Allows filtering of charge item definitions that are appropriate for use versus not.
     */
    experimental?: boolean | undefined;
    _experimental?: Element | undefined;
    /**
     * Typically, this is used for identifiers that can go in an HL7 V3 II (instance identifier) data type, and can then identify this charge item definition outside of FHIR, where it is not possible to use the logical URI.
     */
    identifier?: Identifier[] | undefined;
    /**
     * In case of highly customized, individually produced or fitted devices/substances, the pricing information may be different for each instance of the product. This reference links pricing details to specific product instances.
     */
    instance?: Reference[] | undefined;
    /**
     * It may be possible for the charge item definition to be used in jurisdictions other than those for which it was originally designed or intended.
     */
    jurisdiction?: CodeableConcept[] | undefined;
    /**
     * If specified, this date follows the original approval date.
     */
    lastReviewDate?: string | undefined;
    _lastReviewDate?: Element | undefined;
    /**
     * The name is not expected to be globally unique. The name should be a simple alphanumeric type name to ensure that it is machine-processing friendly.
     */
    name?: string | undefined;
    _name?: Element | undefined;
    /**
     * A larger definition of which this particular definition is a component or step.
     */
    partOf?: string[] | undefined;
    _partOf?: Element[] | undefined;
    /**
     * Group of properties which are applicable under the same conditions. If no applicability rules are established for the group, then all properties always apply.
     */
    propertyGroup?: ChargeItemDefinitionPropertyGroup[] | undefined;
    /**
     * Usually an organization but may be an individual. The publisher (or steward) of the charge item definition is the organization or individual primarily responsible for the maintenance and upkeep of the charge item definition. This is not necessarily the same individual or organization that developed and initially authored the content. The publisher is the primary point of contact for questions or issues with the charge item definition. This item SHOULD be populated unless the information is available from context.
     */
    publisher?: string | undefined;
    _publisher?: Element | undefined;
    /**
     * This element does not describe the usage of the charge item definition. Instead, it provides traceability of ''why'' the resource is either needed or ''why'' it is defined as it is.  This may be used to point to source materials or specifications that drove the structure of this charge item definition.
     */
    purpose?: string | undefined;
    _purpose?: Element | undefined;
    /**
     * As new versions of a protocol or guideline are defined, allows identification of what versions are replaced by a new instance.
     */
    replaces?: string[] | undefined;
    _replaces?: Element[] | undefined;
    /**
     * Allows filtering of charge item definitions that are appropriate for use versus not.
     */
    status: 'draft' | 'active' | 'retired' | 'unknown';
    _status?: Element | undefined;
    /**
     * This name does not need to be machine-processing friendly and may contain punctuation, white-space, etc.
     */
    title?: string | undefined;
    _title?: Element | undefined;
    /**
     * Can be a urn:uuid: or a urn:oid: but real http: addresses are preferred.  Multiple instances may share the same URL if they have a distinct version.
     * The determination of when to create a new version of a resource (same url, new version) vs. defining a new artifact is up to the author.  Considerations for making this decision are found in [Technical and Business Versions](resource.html#versions).
     * In some cases, the resource can no longer be found at the stated url, but the url itself cannot change. Implementations can use the [meta.source](resource.html#meta) element to indicate where the current master source of the resource can be found.
     */
    url: string;
    _url?: Element | undefined;
    /**
     * When multiple useContexts are specified, there is no expectation that all or any of the contexts apply.
     */
    useContext?: UsageContext[] | undefined;
    /**
     * There may be different charge item definition instances that have the same identifier but different versions.  The version can be appended to the url in a reference to allow a reference to a particular business version of the charge item definition with the format [url]|[version]. The version SHOULD NOT contain a '#' - see [Business Version](resource.html#bv-format).
     */
    version?: string | undefined;
    _version?: Element | undefined;
}
/**
 * A human-readable display of the citation.
 */
export interface CitationSummary extends BackboneElement {
    /**
     * Format for display of the citation.
     */
    style?: CodeableConcept | undefined;
    /**
     * The human-readable display of the citation.
     */
    text: string;
    _text?: Element | undefined;
}
/**
 * The assignment to an organizing scheme.
 */
export interface CitationClassification extends BackboneElement {
    /**
     * The specific classification value.
     */
    classifier?: CodeableConcept[] | undefined;
    /**
     * The kind of classifier (e.g. publication type, keyword).
     */
    type?: CodeableConcept | undefined;
}
/**
 * An effective date or period for a status of the citation.
 */
export interface CitationStatusDate extends BackboneElement {
    /**
     * Classification of the status.
     */
    activity: CodeableConcept;
    /**
     * Either occurred or expected.
     */
    actual?: boolean | undefined;
    _actual?: Element | undefined;
    /**
     * For an instance, place the same value in both start and end elements.
     */
    period: Period;
}
/**
 * The defined version of the cited artifact.
 */
export interface CitationCitedArtifactVersion extends BackboneElement {
    /**
     * Citation for the main version of the cited artifact.
     */
    baseCitation?: Reference | undefined;
    /**
     * The version number or other version identifier.
     */
    value: string;
    _value?: Element | undefined;
}
/**
 * An effective date or period for a status of the cited artifact.
 */
export interface CitationCitedArtifactStatusDate extends BackboneElement {
    /**
     * Classification of the status.
     */
    activity: CodeableConcept;
    /**
     * Either occurred or expected.
     */
    actual?: boolean | undefined;
    _actual?: Element | undefined;
    /**
     * For an instance, place the same value in both start and end elements.
     */
    period: Period;
}
/**
 * The title details of the article or artifact.
 */
export interface CitationCitedArtifactTitle extends BackboneElement {
    /**
     * Used to express the specific language.
     */
    language?: CodeableConcept | undefined;
    /**
     * The title of the article or artifact.
     */
    text: string;
    _text?: Element | undefined;
    /**
     * Used to express the reason or specific aspect for the title.
     */
    type?: CodeableConcept[] | undefined;
}
/**
 * Summary of the article or artifact.
 */
export interface CitationCitedArtifactAbstract extends BackboneElement {
    /**
     * Copyright notice for the abstract.
     */
    copyright?: string | undefined;
    _copyright?: Element | undefined;
    /**
     * Used to express the specific language.
     */
    language?: CodeableConcept | undefined;
    /**
     * Abstract content.
     */
    text: string;
    _text?: Element | undefined;
    /**
     * Used to express the reason or specific aspect for the abstract.
     */
    type?: CodeableConcept | undefined;
}
/**
 * The component of the article or artifact.
 */
export interface CitationCitedArtifactPart extends BackboneElement {
    /**
     * The citation for the full article or artifact.
     */
    baseCitation?: Reference | undefined;
    /**
     * The kind of component.
     */
    type?: CodeableConcept | undefined;
    /**
     * The specification of the component.
     */
    value?: string | undefined;
    _value?: Element | undefined;
}
/**
 * The artifact related to the cited artifact.
 */
export interface CitationCitedArtifactRelatesTo extends BackboneElement {
    /**
     * Additional structured information about citations should be captured as extensions.
     */
    citation?: string | undefined;
    _citation?: Element | undefined;
    /**
     * Provides additional classifiers of the related artifact.
     */
    classifier?: CodeableConcept[] | undefined;
    /**
     * A brief description of the document or knowledge resource being referenced, suitable for display to a consumer.
     */
    display?: string | undefined;
    _display?: Element | undefined;
    /**
     * The document being referenced, represented as an attachment. This is exclusive with the resource element.
     */
    document?: Attachment | undefined;
    /**
     * A short label that can be used to reference the citation from elsewhere in the containing artifact, such as a footnote index.
     */
    label?: string | undefined;
    _label?: Element | undefined;
    /**
     * If the type is predecessor, this is a reference to the succeeding knowledge resource. If the type is successor, this is a reference to the prior knowledge resource
     */
    resource?: string | undefined;
    _resource?: Element | undefined;
    /**
     * If both resource and resourceReference are present, they SHOULD be consistent and reference the same resource. Although relatedArtifact is intended to support references to definitional resources, there are cases where non-definitional resources can be definitional (such as Location where the kind is mode). Related artifacts SHOULD be used to reference definitional resources, and profiles SHOULD be used to make that explicit for particular use cases.
     */
    resourceReference?: Reference | undefined;
    /**
     * The type of relationship to the related artifact.
     */
    type:
        | 'documentation'
        | 'justification'
        | 'citation'
        | 'predecessor'
        | 'successor'
        | 'derived-from'
        | 'depends-on'
        | 'composed-of'
        | 'part-of'
        | 'amends'
        | 'amended-with'
        | 'appends'
        | 'appended-with'
        | 'cites'
        | 'cited-by'
        | 'comments-on'
        | 'comment-in'
        | 'contains'
        | 'contained-in'
        | 'corrects'
        | 'correction-in'
        | 'replaces'
        | 'replaced-with'
        | 'retracts'
        | 'retracted-by'
        | 'signs'
        | 'similar-to'
        | 'supports'
        | 'supported-with'
        | 'transforms'
        | 'transformed-into'
        | 'transformed-with'
        | 'documents'
        | 'specification-of'
        | 'created-with'
        | 'cite-as'
        | 'reprint'
        | 'reprint-of';
    _type?: Element | undefined;
}
/**
 * The collection the cited article or artifact is published in.
 */
export interface CitationCitedArtifactPublicationFormPublishedIn extends BackboneElement {
    /**
     * Journal identifiers include ISSN, ISO Abbreviation and NLMuniqueID; Book identifiers include ISBN.
     */
    identifier?: Identifier[] | undefined;
    /**
     * Name of the publisher.
     */
    publisher?: Reference | undefined;
    /**
     * Geographic location of the publisher.
     */
    publisherLocation?: string | undefined;
    _publisherLocation?: Element | undefined;
    /**
     * ClinicalTrials.gov is a name of a database.
     */
    title?: string | undefined;
    _title?: Element | undefined;
    /**
     * Journals and newspapers are coded as Periodical.
     */
    type?: CodeableConcept | undefined;
}
/**
 * A common use is a journal article with a publication date and pagination for a print version and a different publication date for the online version of the same article.
 */
export interface CitationCitedArtifactPublicationForm extends BackboneElement {
    /**
     * Entry number or identifier for inclusion in a database.
     */
    accessionNumber?: string | undefined;
    _accessionNumber?: Element | undefined;
    /**
     * The date the article was added to the database, or the date the article was released (which may differ from the journal issue publication date).
     */
    articleDate?: string | undefined;
    _articleDate?: Element | undefined;
    /**
     * National Library of Medicine uses the codes "Internet" or "Print".
     */
    citedMedium?: CodeableConcept | undefined;
    /**
     * Copyright notice for the full article or artifact.
     */
    copyright?: string | undefined;
    _copyright?: Element | undefined;
    /**
     * Used for isolated representation of first page.
     */
    firstPage?: string | undefined;
    _firstPage?: Element | undefined;
    /**
     * Issue, part or supplement of journal in which the article is published.
     */
    issue?: string | undefined;
    _issue?: Element | undefined;
    /**
     * Language is provided to support indexing and accessibility (typically, services such as text to speech use the language tag). The html language tag in the narrative applies  to the narrative. The language tag on the resource may be used to specify the language of other presentations generated from the data in the resource. Not all the content has to be in the base language. The Resource.language should not be assumed to apply to the narrative automatically. If a language is specified, it should it also be specified on the div element in the html (see rules in HTML5 for information about the relationship between xml:lang and the html lang attribute).
     */
    language?: CodeableConcept[] | undefined;
    /**
     * Used for isolated representation of last page.
     */
    lastPage?: string | undefined;
    _lastPage?: Element | undefined;
    /**
     * The date the article was last revised or updated in the database.
     */
    lastRevisionDate?: string | undefined;
    _lastRevisionDate?: Element | undefined;
    /**
     * Actual or approximate number of pages or screens.
     */
    pageCount?: string | undefined;
    _pageCount?: Element | undefined;
    /**
     * Used for full display of pagination.
     */
    pageString?: string | undefined;
    _pageString?: Element | undefined;
    /**
     * Day on which the issue of the journal was published.
     */
    publicationDateDay?: string | undefined;
    _publicationDateDay?: Element | undefined;
    /**
     * Month on which the issue of the journal was published.
     */
    publicationDateMonth?: string | undefined;
    _publicationDateMonth?: Element | undefined;
    /**
     * Spring, Summer, Fall/Autumn, Winter.
     */
    publicationDateSeason?: string | undefined;
    _publicationDateSeason?: Element | undefined;
    /**
     * Text representation of the date of which the issue of the journal was published.
     */
    publicationDateText?: string | undefined;
    _publicationDateText?: Element | undefined;
    /**
     * Year on which the issue of the journal was published.
     */
    publicationDateYear?: string | undefined;
    _publicationDateYear?: Element | undefined;
    /**
     * The collection the cited article or artifact is published in.
     */
    publishedIn?: CitationCitedArtifactPublicationFormPublishedIn | undefined;
    /**
     * Volume number of journal in which the article is published.
     */
    volume?: string | undefined;
    _volume?: Element | undefined;
}
/**
 * Used for any URL for the article or artifact cited.
 */
export interface CitationCitedArtifactWebLocation extends BackboneElement {
    /**
     * Code the reason for different URLs, e.g. abstract and full-text.
     */
    classifier?: CodeableConcept[] | undefined;
    /**
     * Persistent URLs, like doi, are preferred.
     */
    url?: string | undefined;
    _url?: Element | undefined;
}
/**
 * The assignment to an organizing scheme.
 */
export interface CitationCitedArtifactClassification extends BackboneElement {
    /**
     * Complex or externally created classification.
     */
    artifactAssessment?: Reference[] | undefined;
    /**
     * The specific classification value.
     */
    classifier?: CodeableConcept[] | undefined;
    /**
     * The kind of classifier (e.g. publication type, keyword).
     */
    type?: CodeableConcept | undefined;
}
/**
 * Contributions with accounting for time or number.
 */
export interface CitationCitedArtifactContributorshipEntryContributionInstance extends BackboneElement {
    /**
     * The time that the contribution was made.
     */
    time?: string | undefined;
    _time?: Element | undefined;
    /**
     * The specific contribution.
     */
    type: CodeableConcept;
}
/**
 * Used to report contributorship in individualized ways.
 */
export interface CitationCitedArtifactContributorshipEntry extends BackboneElement {
    /**
     * Organization affiliated with the entity.
     */
    affiliation?: Reference[] | undefined;
    /**
     * Contributions with accounting for time or number.
     */
    contributionInstance?: CitationCitedArtifactContributorshipEntryContributionInstance[] | undefined;
    /**
     * The CreDiT taxonomy provides a way to encode contribution information inside article XML files. The purpose of CRediT is to provide transparency in contributions to scholarly published work, to enable improved systems of attribution, credit, and accountability.
     */
    contributionType?: CodeableConcept[] | undefined;
    /**
     * The identity of the individual entity.
     */
    contributor: Reference;
    /**
     * Indication of which contributor is the corresponding contributor for the role.
     */
    correspondingContact?: boolean | undefined;
    _correspondingContact?: Element | undefined;
    /**
     * This element is used in Medline based citations, but not supported fully by FHIR HumanName datatype.
     */
    forenameInitials?: string | undefined;
    _forenameInitials?: Element | undefined;
    /**
     * Provides a numerical ranking to represent the degree of contributorship relative to other contributors, such as 1 for first author and 2 for second author.
     */
    rankingOrder?: number | undefined;
    /**
     * The role of the contributor (e.g. author, editor, reviewer).
     */
    role?: CodeableConcept | undefined;
}
/**
 * Used to record a display of the author/contributor list without separate coding for each list member.
 */
export interface CitationCitedArtifactContributorshipSummary extends BackboneElement {
    /**
     * Used to code the producer or rule for creating the display string.
     */
    source?: CodeableConcept | undefined;
    /**
     * The format for the display string.
     */
    style?: CodeableConcept | undefined;
    /**
     * Used most commonly to express an author list or a contributorship statement.
     */
    type?: CodeableConcept | undefined;
    /**
     * The display string for the author list, contributor list, or contributorship statement.
     */
    value: string;
    _value?: Element | undefined;
}
/**
 * This element is used to list authors and other contributors, their contact information, specific contributions, and summary statements.
 */
export interface CitationCitedArtifactContributorship extends BackboneElement {
    /**
     * If list is incomplete "et al" can be appended to Contributorship.summary.value for display purposes.
     */
    complete?: boolean | undefined;
    _complete?: Element | undefined;
    /**
     * Used to report contributorship in individualized ways.
     */
    entry?: CitationCitedArtifactContributorshipEntry[] | undefined;
    /**
     * Used to record a display of the author/contributor list without separate coding for each list member.
     */
    summary?: CitationCitedArtifactContributorshipSummary[] | undefined;
}
/**
 * The article or artifact being described.
 */
export interface CitationCitedArtifact extends BackboneElement {
    /**
     * Summary of the article or artifact.
     */
    abstract?: CitationCitedArtifactAbstract[] | undefined;
    /**
     * The assignment to an organizing scheme.
     */
    classification?: CitationCitedArtifactClassification[] | undefined;
    /**
     * This element is used to list authors and other contributors, their contact information, specific contributions, and summary statements.
     */
    contributorship?: CitationCitedArtifactContributorship | undefined;
    /**
     * The status of the cited artifact.
     */
    currentState?: CodeableConcept[] | undefined;
    /**
     * When the cited artifact was accessed.
     */
    dateAccessed?: string | undefined;
    _dateAccessed?: Element | undefined;
    /**
     * May include DOI, PMID, PMCID, etc. This element will contain unique identifiers that support de-duplication of citations. This identifier can be valid for only one citation resource.
     */
    identifier?: Identifier[] | undefined;
    /**
     * Any additional information or content for the article or artifact.
     */
    note?: Annotation[] | undefined;
    /**
     * The component of the article or artifact.
     */
    part?: CitationCitedArtifactPart | undefined;
    /**
     * A common use is a journal article with a publication date and pagination for a print version and a different publication date for the online version of the same article.
     */
    publicationForm?: CitationCitedArtifactPublicationForm[] | undefined;
    /**
     * May include trial registry identifiers, e.g. NCT04372602 from clinicaltrials.gov. This identifier can be valid for multiple citation resources.
     */
    relatedIdentifier?: Identifier[] | undefined;
    /**
     * The artifact related to the cited artifact.
     */
    relatesTo?: CitationCitedArtifactRelatesTo[] | undefined;
    /**
     * An effective date or period for a status of the cited artifact.
     */
    statusDate?: CitationCitedArtifactStatusDate[] | undefined;
    /**
     * The title details of the article or artifact.
     */
    title?: CitationCitedArtifactTitle[] | undefined;
    /**
     * The defined version of the cited artifact.
     */
    version?: CitationCitedArtifactVersion | undefined;
    /**
     * Used for any URL for the article or artifact cited.
     */
    webLocation?: CitationCitedArtifactWebLocation[] | undefined;
}
/**
 * The Citation Resource enables reference to any knowledge artifact for purposes of identification and attribution. The Citation Resource supports existing reference structures and developing publication practices such as versioning, expressing complex contributorship roles, and referencing computable resources.
 */
export interface Citation extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'Citation';
    /**
     * The 'date' element may be more recent than the approval date because of minor changes or editorial corrections.
     */
    approvalDate?: string | undefined;
    _approvalDate?: Element | undefined;
    /**
     * Who authored the Citation.
     */
    author?: ContactDetail[] | undefined;
    /**
     * The article or artifact being described.
     */
    citedArtifact?: CitationCitedArtifact | undefined;
    /**
     * The assignment to an organizing scheme.
     */
    classification?: CitationClassification[] | undefined;
    /**
     * May be a web site, an email address, a telephone number, etc.
     */
    contact?: ContactDetail[] | undefined;
    /**
     * Use and/or publishing restrictions for the Citation, not for the cited artifact.
     */
    copyright?: string | undefined;
    _copyright?: Element | undefined;
    /**
     * The status of the citation.
     */
    currentState?: CodeableConcept[] | undefined;
    /**
     * Note that this is not the same as the resource last-modified-date, since the resource may be a secondary representation of the citation. Additional specific dates may be added as extensions or be found by consulting Provenances associated with past versions of the resource.
     */
    date?: string | undefined;
    _date?: Element | undefined;
    /**
     * This description can be used to capture details such as comments about misuse, instructions for clinical use and interpretation, literature references, examples from the paper world, etc. It is not a rendering of the citation as conveyed in the 'text' field of the resource itself. This item SHOULD be populated unless the information is available from context (e.g. the language of the citation is presumed to be the predominant language in the place the citation was created).
     */
    description?: string | undefined;
    _description?: Element | undefined;
    /**
     * Who edited the Citation.
     */
    editor?: ContactDetail[] | undefined;
    /**
     * The effective period for a citation  determines when the content is applicable for usage and is independent of publication and review dates. For example, a measure intended to be used for the year 2016 might be published in 2015.
     */
    effectivePeriod?: Period | undefined;
    /**
     * Who endorsed the Citation.
     */
    endorser?: ContactDetail[] | undefined;
    /**
     * Allows filtering of citations that are appropriate for use versus not.
     */
    experimental?: boolean | undefined;
    _experimental?: Element | undefined;
    /**
     * May include DOI, PMID, PMCID, etc. This element will contain unique identifiers that support de-duplication of citations. This identifier can be valid for only one citation resource.
     */
    identifier?: Identifier[] | undefined;
    /**
     * It may be possible for the citation to be used in jurisdictions other than those for which it was originally designed or intended.
     */
    jurisdiction?: CodeableConcept[] | undefined;
    /**
     * If specified, this date follows the original approval date.
     */
    lastReviewDate?: string | undefined;
    _lastReviewDate?: Element | undefined;
    /**
     * The name is not expected to be globally unique. The name should be a simple alphanumeric type name to ensure that it is machine-processing friendly.
     */
    name?: string | undefined;
    _name?: Element | undefined;
    /**
     * Used for general notes and annotations not coded elsewhere.
     */
    note?: Annotation[] | undefined;
    /**
     * Usually an organization but may be an individual. The publisher (or steward) of the citation is the organization or individual primarily responsible for the maintenance and upkeep of the citation. This is not necessarily the same individual or organization that developed and initially authored the content. The publisher is the primary point of contact for questions or issues with the citation. This item SHOULD be populated unless the information is available from context.
     */
    publisher?: string | undefined;
    _publisher?: Element | undefined;
    /**
     * This element does not describe the usage of the citation. Instead, it provides traceability of ''why'' the resource is either needed or ''why'' it is defined as it is.  This may be used to point to source materials or specifications that drove the structure of this citation.
     */
    purpose?: string | undefined;
    _purpose?: Element | undefined;
    /**
     * Artifact related to the Citation Resource.
     */
    relatedArtifact?: RelatedArtifact[] | undefined;
    /**
     * Who reviewed the Citation.
     */
    reviewer?: ContactDetail[] | undefined;
    /**
     * Allows filtering of summarys s that are appropriate for use versus not.
     */
    status: 'draft' | 'active' | 'retired' | 'unknown';
    _status?: Element | undefined;
    /**
     * An effective date or period for a status of the citation.
     */
    statusDate?: CitationStatusDate[] | undefined;
    /**
     * A human-readable display of the citation.
     */
    summary?: CitationSummary[] | undefined;
    /**
     * This name does not need to be machine-processing friendly and may contain punctuation, white-space, etc.
     */
    title?: string | undefined;
    _title?: Element | undefined;
    /**
     * Can be a urn:uuid: or a urn:oid: but real http: addresses are preferred. Multiple instances may share the same URL if they have a distinct version.
     */
    url?: string | undefined;
    _url?: Element | undefined;
    /**
     * When multiple useContexts are specified, there is no expectation that all or any of the contexts apply.
     */
    useContext?: UsageContext[] | undefined;
    /**
     * There may be different citation instances that have the same identifier but different versions.  The version can be appended to the url in a reference to allow a reference to a particular business version of the citation with the format [url]|[version]. The version SHOULD NOT contain a '#' - see [Business Version](resource.html#bv-format).
     */
    version?: string | undefined;
    _version?: Element | undefined;
}
/**
 * For example,  for the original treatment and follow-up exams.
 */
export interface ClaimRelated extends BackboneElement {
    /**
     * Reference to a related claim.
     */
    claim?: Reference | undefined;
    /**
     * For example, Property/Casualty insurer claim # or Workers Compensation case # .
     */
    reference?: Identifier | undefined;
    /**
     * For example, prior claim or umbrella.
     */
    relationship?: CodeableConcept | undefined;
}
/**
 * Often providers agree to receive the benefits payable to reduce the near-term costs to the patient. The insurer may decline to pay the provider and choose to pay the subscriber instead.
 */
export interface ClaimPayee extends BackboneElement {
    /**
     * Not required if the payee is 'subscriber' or 'provider'.
     */
    party?: Reference | undefined;
    /**
     * Type of Party to be reimbursed: subscriber, provider, other.
     */
    type: CodeableConcept;
}
/**
 * The members of the team who provided the products and services.
 */
export interface ClaimCareTeam extends BackboneElement {
    /**
     * Member of the team who provided the product or service.
     */
    provider: Reference;
    /**
     * Responsible might not be required when there is only a single provider listed.
     */
    responsible?: boolean | undefined;
    _responsible?: Element | undefined;
    /**
     * Role might not be required when there is only a single provider listed.
     */
    role?: CodeableConcept | undefined;
    /**
     * A number to uniquely identify care team entries.
     */
    sequence: number;
    /**
     * The specialization of the practitioner or provider which is applicable for this service.
     */
    specialty?: CodeableConcept | undefined;
}
/**
 * Often there are multiple jurisdiction specific valuesets which are required.
 */
export interface ClaimSupportingInfo extends BackboneElement {
    /**
     * This may contain a category for the local bill type codes.
     */
    category: CodeableConcept;
    /**
     * System and code pertaining to the specific information regarding special conditions relating to the setting, treatment or patient  for which care is sought.
     */
    code?: CodeableConcept | undefined;
    /**
     * For example: the reason for the additional stay, or why a tooth is  missing.
     */
    reason?: CodeableConcept | undefined;
    /**
     * A number to uniquely identify supporting information entries.
     */
    sequence: number;
    /**
     * The date when or period to which this information refers.
     */
    timingDate?: string | undefined;
    _timingDate?: Element | undefined;
    /**
     * The date when or period to which this information refers.
     */
    timingPeriod?: Period | undefined;
    /**
     * Could be used to provide references to other resources, document. For example could contain a PDF in an Attachment of the Police Report for an Accident.
     */
    valueBoolean?: boolean | undefined;
    _valueBoolean?: Element | undefined;
    /**
     * Could be used to provide references to other resources, document. For example could contain a PDF in an Attachment of the Police Report for an Accident.
     */
    valueString?: string | undefined;
    _valueString?: Element | undefined;
    /**
     * Could be used to provide references to other resources, document. For example could contain a PDF in an Attachment of the Police Report for an Accident.
     */
    valueQuantity?: Quantity | undefined;
    /**
     * Could be used to provide references to other resources, document. For example could contain a PDF in an Attachment of the Police Report for an Accident.
     */
    valueAttachment?: Attachment | undefined;
    /**
     * Could be used to provide references to other resources, document. For example could contain a PDF in an Attachment of the Police Report for an Accident.
     */
    valueReference?: Reference | undefined;
    /**
     * Could be used to provide references to other resources, document. For example could contain a PDF in an Attachment of the Police Report for an Accident.
     */
    valueIdentifier?: Identifier | undefined;
}
/**
 * Information about diagnoses relevant to the claim items.
 */
export interface ClaimDiagnosis extends BackboneElement {
    /**
     * The nature of illness or problem in a coded form or as a reference to an external defined Condition.
     */
    diagnosisCodeableConcept?: CodeableConcept | undefined;
    /**
     * The nature of illness or problem in a coded form or as a reference to an external defined Condition.
     */
    diagnosisReference?: Reference | undefined;
    /**
     * Indication of whether the diagnosis was present on admission to a facility.
     */
    onAdmission?: CodeableConcept | undefined;
    /**
     * Diagnosis are presented in list order to their expected importance: primary, secondary, etc.
     */
    sequence: number;
    /**
     * For example: admitting, primary, secondary, discharge.
     */
    type?: CodeableConcept[] | undefined;
}
/**
 * Procedures performed on the patient relevant to the billing items with the claim.
 */
export interface ClaimProcedure extends BackboneElement {
    /**
     * Date and optionally time the procedure was performed.
     */
    date?: string | undefined;
    _date?: Element | undefined;
    /**
     * The code or reference to a Procedure resource which identifies the clinical intervention performed.
     */
    procedureCodeableConcept?: CodeableConcept | undefined;
    /**
     * The code or reference to a Procedure resource which identifies the clinical intervention performed.
     */
    procedureReference?: Reference | undefined;
    /**
     * A number to uniquely identify procedure entries.
     */
    sequence: number;
    /**
     * For example: primary, secondary.
     */
    type?: CodeableConcept[] | undefined;
    /**
     * Unique Device Identifiers associated with this line item.
     */
    udi?: Reference[] | undefined;
}
/**
 * All insurance coverages for the patient which may be applicable for reimbursement, of the products and services listed in the claim, are typically provided in the claim to allow insurers to confirm the ordering of the insurance coverages relative to local 'coordination of benefit' rules. One coverage (and only one) with 'focal=true' is to be used in the adjudication of this claim. Coverages appearing before the focal Coverage in the list, and where 'Coverage.subrogation=false', should provide a reference to the ClaimResponse containing the adjudication results of the prior claim.
 */
export interface ClaimInsurance extends BackboneElement {
    /**
     * A business agreement number established between the provider and the insurer for special business processing purposes.
     */
    businessArrangement?: string | undefined;
    _businessArrangement?: Element | undefined;
    /**
     * Must not be specified when 'focal=true' for this insurance.
     */
    claimResponse?: Reference | undefined;
    /**
     * Reference to the insurance card level information contained in the Coverage resource. The coverage issuing insurer will use these details to locate the patient's actual coverage within the insurer's information system.
     */
    coverage: Reference;
    /**
     * A patient may (will) have multiple insurance policies which provide reimbursement for healthcare services and products. For example a person may also be covered by their spouse's policy and both appear in the list (and may be from the same insurer). This flag will be set to true for only one of the listed policies and that policy will be used for adjudicating this claim. Other claims would be created to request adjudication against the other listed policies.
     */
    focal: boolean;
    _focal?: Element | undefined;
    /**
     * Only required in jurisdictions where insurers, rather than the provider, are required to send claims to  insurers that appear after them in the list. This element is not required when 'subrogation=true'.
     */
    identifier?: Identifier | undefined;
    /**
     * This value is an alphanumeric string that may be provided over the phone, via text, via paper, or within a ClaimResponse resource and is not a FHIR Identifier.
     */
    preAuthRef?: string[] | undefined;
    _preAuthRef?: Element[] | undefined;
    /**
     * A number to uniquely identify insurance entries and provide a sequence of coverages to convey coordination of benefit order.
     */
    sequence: number;
}
/**
 * Details of an accident which resulted in injuries which required the products and services listed in the claim.
 */
export interface ClaimAccident extends BackboneElement {
    /**
     * The date of the accident has to precede the dates of the products and services but within a reasonable timeframe.
     */
    date: string;
    _date?: Element | undefined;
    /**
     * The physical location of the accident event.
     */
    locationAddress?: Address | undefined;
    /**
     * The physical location of the accident event.
     */
    locationReference?: Reference | undefined;
    /**
     * The type or context of the accident event for the purposes of selection of potential insurance coverages and determination of coordination between insurers.
     */
    type?: CodeableConcept | undefined;
}
/**
 * Physical location where the service is performed or applies.
 */
export interface ClaimItemBodySite extends BackboneElement {
    /**
     * For example: Providing a tooth code, allows an insurer to identify a provider performing a filling on a tooth that was previously removed.
     */
    site: CodeableReference[];
    /**
     * A region or surface of the bodySite, e.g. limb region or tooth surface(s).
     */
    subSite?: CodeableConcept[] | undefined;
}
/**
 * A claim detail line. Either a simple (a product or service) or a 'group' of sub-details which are simple items.
 */
export interface ClaimItemDetailSubDetail extends BackboneElement {
    /**
     * Examples include Medical Care, Periodontics, Renal Dialysis, Vision Coverage.
     */
    category?: CodeableConcept | undefined;
    /**
     * To show a 10% senior's discount, the value entered is: 0.90 (1.00 - 0.10).
     */
    factor?: number | undefined;
    /**
     * For example in Oral whether the treatment is cosmetic or associated with TMJ, or for Medical whether the treatment was outside the clinic or out of office hours.
     */
    modifier?: CodeableConcept[] | undefined;
    /**
     * For example, the formula: quantity * unitPrice * factor  = net. Quantity and factor are assumed to be 1 if not supplied.
     */
    net?: Money | undefined;
    /**
     * The amount paid by the patient, in total at the claim claim level or specifically for the item and detail level, to the provider for goods and services.
     */
    patientPaid?: Money | undefined;
    /**
     * If this is an actual service or product line, i.e. not a Group, then use code to indicate the Professional Service or Product supplied (e.g. CTP, HCPCS, USCLS, ICD10, NCPDP, DIN, RxNorm, ACHI, CCI). If a grouping item then use a group code to indicate the type of thing being grouped e.g. 'glasses' or 'compound'.
     */
    productOrService?: CodeableConcept | undefined;
    /**
     * This contains the end of a range of product, service, drug or other billing codes for the item. This element is not used when the .productOrService is a group code. This value may only be present when a .productOfService code has been provided to convey the start of the range. Typically this value may be used only with preauthorizations and not with claims.
     */
    productOrServiceEnd?: CodeableConcept | undefined;
    /**
     * For example: Neonatal program, child dental program or drug users recovery program.
     */
    programCode?: CodeableConcept[] | undefined;
    /**
     * The number of repetitions of a service or product.
     */
    quantity?: Quantity | undefined;
    /**
     * The type of revenue or cost center providing the product and/or service.
     */
    revenue?: CodeableConcept | undefined;
    /**
     * A number to uniquely identify item entries.
     */
    sequence: number;
    /**
     * The total of taxes applicable for this product or service.
     */
    tax?: Money | undefined;
    /**
     * Unique Device Identifiers associated with this line item.
     */
    udi?: Reference[] | undefined;
    /**
     * If the item is not a group then this is the fee for the product or service, otherwise this is the total of the fees for the details of the group.
     */
    unitPrice?: Money | undefined;
}
/**
 * A claim detail line. Either a simple (a product or service) or a 'group' of sub-details which are simple items.
 */
export interface ClaimItemDetail extends BackboneElement {
    /**
     * Examples include Medical Care, Periodontics, Renal Dialysis, Vision Coverage.
     */
    category?: CodeableConcept | undefined;
    /**
     * To show a 10% senior's discount, the value entered is: 0.90 (1.00 - 0.10).
     */
    factor?: number | undefined;
    /**
     * For example in Oral whether the treatment is cosmetic or associated with TMJ, or for Medical whether the treatment was outside the clinic or out of office hours.
     */
    modifier?: CodeableConcept[] | undefined;
    /**
     * For example, the formula: quantity * unitPrice * factor  = net. Quantity and factor are assumed to be 1 if not supplied.
     */
    net?: Money | undefined;
    /**
     * The amount paid by the patient, in total at the claim claim level or specifically for the item and detail level, to the provider for goods and services.
     */
    patientPaid?: Money | undefined;
    /**
     * If this is an actual service or product line, i.e. not a Group, then use code to indicate the Professional Service or Product supplied (e.g. CTP, HCPCS, USCLS, ICD10, NCPDP, DIN, RxNorm, ACHI, CCI). If a grouping item then use a group code to indicate the type of thing being grouped e.g. 'glasses' or 'compound'.
     */
    productOrService?: CodeableConcept | undefined;
    /**
     * This contains the end of a range of product, service, drug or other billing codes for the item. This element is not used when the .productOrService is a group code. This value may only be present when a .productOfService code has been provided to convey the start of the range. Typically this value may be used only with preauthorizations and not with claims.
     */
    productOrServiceEnd?: CodeableConcept | undefined;
    /**
     * For example: Neonatal program, child dental program or drug users recovery program.
     */
    programCode?: CodeableConcept[] | undefined;
    /**
     * The number of repetitions of a service or product.
     */
    quantity?: Quantity | undefined;
    /**
     * The type of revenue or cost center providing the product and/or service.
     */
    revenue?: CodeableConcept | undefined;
    /**
     * A number to uniquely identify item entries.
     */
    sequence: number;
    /**
     * A claim detail line. Either a simple (a product or service) or a 'group' of sub-details which are simple items.
     */
    subDetail?: ClaimItemDetailSubDetail[] | undefined;
    /**
     * The total of taxes applicable for this product or service.
     */
    tax?: Money | undefined;
    /**
     * Unique Device Identifiers associated with this line item.
     */
    udi?: Reference[] | undefined;
    /**
     * If the item is not a group then this is the fee for the product or service, otherwise this is the total of the fees for the details of the group.
     */
    unitPrice?: Money | undefined;
}
/**
 * A claim line. Either a simple  product or service or a 'group' of details which can each be a simple items or groups of sub-details.
 */
export interface ClaimItem extends BackboneElement {
    /**
     * Physical location where the service is performed or applies.
     */
    bodySite?: ClaimItemBodySite[] | undefined;
    /**
     * CareTeam members related to this service or product.
     */
    careTeamSequence?: number[] | undefined;
    /**
     * Examples include Medical Care, Periodontics, Renal Dialysis, Vision Coverage.
     */
    category?: CodeableConcept | undefined;
    /**
     * A claim detail line. Either a simple (a product or service) or a 'group' of sub-details which are simple items.
     */
    detail?: ClaimItemDetail[] | undefined;
    /**
     * Diagnosis applicable for this service or product.
     */
    diagnosisSequence?: number[] | undefined;
    /**
     * This will typically be the encounter the event occurred within, but some activities may be initiated prior to or after the official completion of an encounter but still be tied to the context of the encounter.
     */
    encounter?: Reference[] | undefined;
    /**
     * To show a 10% senior's discount, the value entered is: 0.90 (1.00 - 0.10).
     */
    factor?: number | undefined;
    /**
     * Exceptions, special conditions and supporting information applicable for this service or product.
     */
    informationSequence?: number[] | undefined;
    /**
     * Where the product or service was provided.
     */
    locationCodeableConcept?: CodeableConcept | undefined;
    /**
     * Where the product or service was provided.
     */
    locationAddress?: Address | undefined;
    /**
     * Where the product or service was provided.
     */
    locationReference?: Reference | undefined;
    /**
     * For example in Oral whether the treatment is cosmetic or associated with TMJ, or for Medical whether the treatment was outside the clinic or outside of office hours.
     */
    modifier?: CodeableConcept[] | undefined;
    /**
     * For example, the formula: quantity * unitPrice * factor  = net. Quantity and factor are assumed to be 1 if not supplied.
     */
    net?: Money | undefined;
    /**
     * The amount paid by the patient, in total at the claim claim level or specifically for the item and detail level, to the provider for goods and services.
     */
    patientPaid?: Money | undefined;
    /**
     * Procedures applicable for this service or product.
     */
    procedureSequence?: number[] | undefined;
    /**
     * If this is an actual service or product line, i.e. not a Group, then use code to indicate the Professional Service or Product supplied (e.g. CTP, HCPCS, USCLS, ICD10, NCPDP, DIN, RxNorm, ACHI, CCI). If a grouping item then use a group code to indicate the type of thing being grouped e.g. 'glasses' or 'compound'.
     */
    productOrService?: CodeableConcept | undefined;
    /**
     * This contains the end of a range of product, service, drug or other billing codes for the item. This element is not used when the .productOrService is a group code. This value may only be present when a .productOfService code has been provided to convey the start of the range. Typically this value may be used only with preauthorizations and not with claims.
     */
    productOrServiceEnd?: CodeableConcept | undefined;
    /**
     * For example: Neonatal program, child dental program or drug users recovery program.
     */
    programCode?: CodeableConcept[] | undefined;
    /**
     * The number of repetitions of a service or product.
     */
    quantity?: Quantity | undefined;
    /**
     * The type of revenue or cost center providing the product and/or service.
     */
    revenue?: CodeableConcept | undefined;
    /**
     * A number to uniquely identify item entries.
     */
    sequence: number;
    /**
     * The date or dates when the service or product was supplied, performed or completed.
     */
    servicedDate?: string | undefined;
    _servicedDate?: Element | undefined;
    /**
     * The date or dates when the service or product was supplied, performed or completed.
     */
    servicedPeriod?: Period | undefined;
    /**
     * The total of taxes applicable for this product or service.
     */
    tax?: Money | undefined;
    /**
     * Unique Device Identifiers associated with this line item.
     */
    udi?: Reference[] | undefined;
    /**
     * If the item is not a group then this is the fee for the product or service, otherwise this is the total of the fees for the details of the group.
     */
    unitPrice?: Money | undefined;
}
/**
 * A provider issued list of professional services and products which have been provided, or are to be provided, to a patient which is sent to an insurer for reimbursement.
 */
export interface Claim extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'Claim';
    /**
     * Details of an accident which resulted in injuries which required the products and services listed in the claim.
     */
    accident?: ClaimAccident | undefined;
    /**
     * Typically this would be today or in the past for a claim, and today or in the future for preauthorizations and predeterminations. Typically line item dates of service should fall within the billing period if one is specified.
     */
    billablePeriod?: Period | undefined;
    /**
     * The members of the team who provided the products and services.
     */
    careTeam?: ClaimCareTeam[] | undefined;
    /**
     * This field is independent of the date of creation of the resource as it may reflect the creation date of a source document prior to digitization. Typically for claims all services must be completed as of this date.
     */
    created: string;
    _created?: Element | undefined;
    /**
     * Information about diagnoses relevant to the claim items.
     */
    diagnosis?: ClaimDiagnosis[] | undefined;
    /**
     * For example DRG (Diagnosis Related Group) or a bundled billing code. A patient may have a diagnosis of a Myocardial Infarction and a DRG for HeartAttack would be assigned. The Claim item (and possible subsequent claims) would refer to the DRG for those line items that were for services related to the heart attack event.
     */
    diagnosisRelatedGroup?: CodeableConcept | undefined;
    /**
     * This will typically be the encounter the event occurred within, but some activities may be initiated prior to or after the official completion of an encounter but still be tied to the context of the encounter.
     */
    encounter?: Reference[] | undefined;
    /**
     * Individual who created the claim, predetermination or preauthorization.
     */
    enterer?: Reference | undefined;
    /**
     * Facility where the services were provided.
     */
    facility?: Reference | undefined;
    /**
     * This field is only used for preauthorizations.
     */
    fundsReserve?: CodeableConcept | undefined;
    /**
     * A unique identifier assigned to this claim.
     */
    identifier?: Identifier[] | undefined;
    /**
     * All insurance coverages for the patient which may be applicable for reimbursement, of the products and services listed in the claim, are typically provided in the claim to allow insurers to confirm the ordering of the insurance coverages relative to local 'coordination of benefit' rules. One coverage (and only one) with 'focal=true' is to be used in the adjudication of this claim. Coverages appearing before the focal Coverage in the list, and where 'Coverage.subrogation=false', should provide a reference to the ClaimResponse containing the adjudication results of the prior claim.
     */
    insurance?: ClaimInsurance[] | undefined;
    /**
     * The Insurer who is target of the request.
     */
    insurer?: Reference | undefined;
    /**
     * A claim line. Either a simple  product or service or a 'group' of details which can each be a simple items or groups of sub-details.
     */
    item?: ClaimItem[] | undefined;
    /**
     * For example, a physician may prescribe a medication which the pharmacy determines is contraindicated, or for which the patient has an intolerance, and therefore issues a new prescription for an alternate medication which has the same therapeutic intent. The prescription from the pharmacy becomes the 'prescription' and that from the physician becomes the 'original prescription'.
     */
    originalPrescription?: Reference | undefined;
    /**
     * The party to whom the professional services and/or products have been supplied or are being considered and for whom actual or forecast reimbursement is sought.
     */
    patient: Reference;
    /**
     * The amount paid by the patient, in total at the claim claim level or specifically for the item and detail level, to the provider for goods and services.
     */
    patientPaid?: Money | undefined;
    /**
     * Often providers agree to receive the benefits payable to reduce the near-term costs to the patient. The insurer may decline to pay the provider and choose to pay the subscriber instead.
     */
    payee?: ClaimPayee | undefined;
    /**
     * Prescription is the document/authorization given to the claim author for them to provide products and services for which consideration (reimbursement) is sought. Could be a RX for medications, an 'order' for oxygen or wheelchair or physiotherapy treatments.
     */
    prescription?: Reference | undefined;
    /**
     * If a claim processor is unable to complete the processing as per the priority then they should generate an error and not process the request.
     */
    priority?: CodeableConcept | undefined;
    /**
     * Procedures performed on the patient relevant to the billing items with the claim.
     */
    procedure?: ClaimProcedure[] | undefined;
    /**
     * This party is responsible for the claim but not necessarily professionally responsible for the provision of the individual products and services listed below. This field is the Billing Provider, for example, a facility, provider group, lab or practitioner.
     */
    provider?: Reference | undefined;
    /**
     * The referral resource which lists the date, practitioner, reason and other supporting information.
     */
    referral?: Reference | undefined;
    /**
     * For example,  for the original treatment and follow-up exams.
     */
    related?: ClaimRelated[] | undefined;
    /**
     * This element is labeled as a modifier because the status contains codes that mark the resource as not currently valid.
     */
    status: 'active' | 'cancelled' | 'draft' | 'entered-in-error';
    _status?: Element | undefined;
    /**
     * This may contain the local bill type codes, for example the US UB-04 bill type code or the CMS bill type.
     */
    subType?: CodeableConcept | undefined;
    /**
     * Often there are multiple jurisdiction specific valuesets which are required.
     */
    supportingInfo?: ClaimSupportingInfo[] | undefined;
    /**
     * The total value of the all the items in the claim.
     */
    total?: Money | undefined;
    /**
     * The code system provides oral, pharmacy, vision, professional and institutional claim types. Those supported depends on the requirements of the jurisdiction. The valueset is extensible to accommodate other types of claims as required by the jurisdiction.
     */
    type: CodeableConcept;
    /**
     * A code to indicate whether the nature of the request is: Claim - A request to an Insurer to adjudicate the supplied charges for health care goods and services under the identified policy and to pay the determined Benefit amount, if any; Preauthorization - A request to an Insurer to adjudicate the supplied proposed future charges for health care goods and services under the identified policy and to approve the services and provide the expected benefit amounts and potentially to reserve funds to pay the benefits when Claims for the indicated services are later submitted; or, Pre-determination - A request to an Insurer to adjudicate the supplied 'what if' charges for health care goods and services under the identified policy and report back what the Benefit payable would be had the services actually been provided.
     */
    use: 'claim' | 'preauthorization' | 'predetermination';
    _use?: Element | undefined;
}
/**
 * If this item is a group then the values here are a summary of the adjudication of the detail items. If this item is a simple product or service then this is the result of the adjudication of this item.
 */
export interface ClaimResponseItemAdjudication extends BackboneElement {
    /**
     * For example: amount submitted, eligible amount, co-payment, and benefit payable.
     */
    amount?: Money | undefined;
    /**
     * For example codes indicating: Co-Pay, deductible, eligible, benefit, tax, etc.
     */
    category: CodeableConcept;
    /**
     * For example may indicate that the funds for this benefit type have been exhausted.
     */
    reason?: CodeableConcept | undefined;
    /**
     * For example: eligible percentage or co-payment percentage.
     */
    value?: number | undefined;
}
/**
 * A sub-detail adjudication of a simple product or service.
 */
export interface ClaimResponseItemDetailSubDetail extends BackboneElement {
    /**
     * The adjudication results.
     */
    adjudication?: ClaimResponseItemAdjudication[] | undefined;
    /**
     * The element is used to indicate the current state of the adjudication overall for the claim resource, for example: the request has been held (pended) for adjudication processing, for manual review or other reasons; that it has been processed and will be paid, or the outstanding paid, as submitted (approved); that no amount will be paid (denied); or that some amount between zero and the submitted amoutn will be paid (partial).
     */
    decision?: CodeableConcept | undefined;
    /**
     * The numbers associated with notes below which apply to the adjudication of this item.
     */
    noteNumber?: number[] | undefined;
    /**
     * A number to uniquely reference the claim sub-detail entry.
     */
    subDetailSequence: number;
}
/**
 * A claim detail. Either a simple (a product or service) or a 'group' of sub-details which are simple items.
 */
export interface ClaimResponseItemDetail extends BackboneElement {
    /**
     * The adjudication results.
     */
    adjudication?: ClaimResponseItemAdjudication[] | undefined;
    /**
     * The element is used to indicate the current state of the adjudication overall for the claim resource, for example: the request has been held (pended) for adjudication processing, for manual review or other reasons; that it has been processed and will be paid, or the outstanding paid, as submitted (approved); that no amount will be paid (denied); or that some amount between zero and the submitted amoutn will be paid (partial).
     */
    decision?: CodeableConcept | undefined;
    /**
     * A number to uniquely reference the claim detail entry.
     */
    detailSequence: number;
    /**
     * The numbers associated with notes below which apply to the adjudication of this item.
     */
    noteNumber?: number[] | undefined;
    /**
     * A sub-detail adjudication of a simple product or service.
     */
    subDetail?: ClaimResponseItemDetailSubDetail[] | undefined;
}
/**
 * A claim line. Either a simple (a product or service) or a 'group' of details which can also be a simple items or groups of sub-details.
 */
export interface ClaimResponseItem extends BackboneElement {
    /**
     * If this item is a group then the values here are a summary of the adjudication of the detail items. If this item is a simple product or service then this is the result of the adjudication of this item.
     */
    adjudication?: ClaimResponseItemAdjudication[] | undefined;
    /**
     * The element is used to indicate the current state of the adjudication overall for the claim resource, for example: the request has been held (pended) for adjudication processing, for manual review or other reasons; that it has been processed and will be paid, or the outstanding paid, as submitted (approved); that no amount will be paid (denied); or that some amount between zero and the submitted amoutn will be paid (partial).
     */
    decision?: CodeableConcept | undefined;
    /**
     * A claim detail. Either a simple (a product or service) or a 'group' of sub-details which are simple items.
     */
    detail?: ClaimResponseItemDetail[] | undefined;
    /**
     * A number to uniquely reference the claim item entries.
     */
    itemSequence: number;
    /**
     * The numbers associated with notes below which apply to the adjudication of this item.
     */
    noteNumber?: number[] | undefined;
}
/**
 * Physical location where the service is performed or applies.
 */
export interface ClaimResponseAddItemBodySite extends BackboneElement {
    /**
     * For example: Providing a tooth code allows an insurer to identify a provider performing a filling on a tooth that was previously removed.
     */
    site: CodeableReference[];
    /**
     * A region or surface of the bodySite, e.g. limb region or tooth surface(s).
     */
    subSite?: CodeableConcept[] | undefined;
}
/**
 * The third-tier service adjudications for payor added services.
 */
export interface ClaimResponseAddItemDetailSubDetail extends BackboneElement {
    /**
     * The adjudication results.
     */
    adjudication?: ClaimResponseItemAdjudication[] | undefined;
    /**
     * The element is used to indicate the current state of the adjudication overall for the claim resource, for example: the request has been held (pended) for adjudication processing, for manual review or other reasons; that it has been processed and will be paid, or the outstanding paid, as submitted (approved); that no amount will be paid (denied); or that some amount between zero and the submitted amoutn will be paid (partial).
     */
    decision?: CodeableConcept | undefined;
    /**
     * To show a 10% senior's discount, the value entered is: 0.90 (1.00 - 0.10).
     */
    factor?: number | undefined;
    /**
     * For example in Oral whether the treatment is cosmetic or associated with TMJ, or for Medical whether the treatment was outside the clinic or outside of office hours.
     */
    modifier?: CodeableConcept[] | undefined;
    /**
     * For example, the formula: quantity * unitPrice * factor  = net. Quantity and factor are assumed to be 1 if not supplied.
     */
    net?: Money | undefined;
    /**
     * The numbers associated with notes below which apply to the adjudication of this item.
     */
    noteNumber?: number[] | undefined;
    /**
     * If this is an actual service or product line, i.e. not a Group, then use code to indicate the Professional Service or Product supplied (e.g. CTP, HCPCS, USCLS, ICD10, NCPDP, DIN, RxNorm, ACHI, CCI). If a grouping item then use a group code to indicate the type of thing being grouped e.g. 'glasses' or 'compound'.
     */
    productOrService?: CodeableConcept | undefined;
    /**
     * This contains the end of a range of product, service, drug or other billing codes for the item. This element is not used when the .productOrService is a group code. This value may only be present when a .productOfService code has been provided to convey the start of the range. Typically this value may be used only with preauthorizations and not with claims.
     */
    productOrServiceEnd?: CodeableConcept | undefined;
    /**
     * The number of repetitions of a service or product.
     */
    quantity?: Quantity | undefined;
    /**
     * The type of revenue or cost center providing the product and/or service.
     */
    revenue?: CodeableConcept | undefined;
    /**
     * The total of taxes applicable for this product or service.
     */
    tax?: Money | undefined;
    /**
     * If the item is not a group then this is the fee for the product or service, otherwise this is the total of the fees for the details of the group.
     */
    unitPrice?: Money | undefined;
}
/**
 * The second-tier service adjudications for payor added services.
 */
export interface ClaimResponseAddItemDetail extends BackboneElement {
    /**
     * The adjudication results.
     */
    adjudication?: ClaimResponseItemAdjudication[] | undefined;
    /**
     * The element is used to indicate the current state of the adjudication overall for the claim resource, for example: the request has been held (pended) for adjudication processing, for manual review or other reasons; that it has been processed and will be paid, or the outstanding paid, as submitted (approved); that no amount will be paid (denied); or that some amount between zero and the submitted amoutn will be paid (partial).
     */
    decision?: CodeableConcept | undefined;
    /**
     * To show a 10% senior's discount, the value entered is: 0.90 (1.00 - 0.10).
     */
    factor?: number | undefined;
    /**
     * For example in Oral whether the treatment is cosmetic or associated with TMJ, or for Medical whether the treatment was outside the clinic or outside of office hours.
     */
    modifier?: CodeableConcept[] | undefined;
    /**
     * For example, the formula: quantity * unitPrice * factor  = net. Quantity and factor are assumed to be 1 if not supplied.
     */
    net?: Money | undefined;
    /**
     * The numbers associated with notes below which apply to the adjudication of this item.
     */
    noteNumber?: number[] | undefined;
    /**
     * If this is an actual service or product line, i.e. not a Group, then use code to indicate the Professional Service or Product supplied (e.g. CTP, HCPCS, USCLS, ICD10, NCPDP, DIN, RxNorm, ACHI, CCI). If a grouping item then use a group code to indicate the type of thing being grouped e.g. 'glasses' or 'compound'.
     */
    productOrService?: CodeableConcept | undefined;
    /**
     * This contains the end of a range of product, service, drug or other billing codes for the item. This element is not used when the .productOrService is a group code. This value may only be present when a .productOfService code has been provided to convey the start of the range. Typically this value may be used only with preauthorizations and not with claims.
     */
    productOrServiceEnd?: CodeableConcept | undefined;
    /**
     * The number of repetitions of a service or product.
     */
    quantity?: Quantity | undefined;
    /**
     * The type of revenue or cost center providing the product and/or service.
     */
    revenue?: CodeableConcept | undefined;
    /**
     * The third-tier service adjudications for payor added services.
     */
    subDetail?: ClaimResponseAddItemDetailSubDetail[] | undefined;
    /**
     * The total of taxes applicable for this product or service.
     */
    tax?: Money | undefined;
    /**
     * If the item is not a group then this is the fee for the product or service, otherwise this is the total of the fees for the details of the group.
     */
    unitPrice?: Money | undefined;
}
/**
 * The first-tier service adjudications for payor added product or service lines.
 */
export interface ClaimResponseAddItem extends BackboneElement {
    /**
     * The adjudication results.
     */
    adjudication?: ClaimResponseItemAdjudication[] | undefined;
    /**
     * Physical location where the service is performed or applies.
     */
    bodySite?: ClaimResponseAddItemBodySite[] | undefined;
    /**
     * The element is used to indicate the current state of the adjudication overall for the claim resource, for example: the request has been held (pended) for adjudication processing, for manual review or other reasons; that it has been processed and will be paid, or the outstanding paid, as submitted (approved); that no amount will be paid (denied); or that some amount between zero and the submitted amoutn will be paid (partial).
     */
    decision?: CodeableConcept | undefined;
    /**
     * The second-tier service adjudications for payor added services.
     */
    detail?: ClaimResponseAddItemDetail[] | undefined;
    /**
     * The sequence number of the details within the claim item which this line is intended to replace.
     */
    detailSequence?: number[] | undefined;
    /**
     * To show a 10% senior's discount, the value entered is: 0.90 (1.00 - 0.10).
     */
    factor?: number | undefined;
    /**
     * Claim items which this service line is intended to replace.
     */
    itemSequence?: number[] | undefined;
    /**
     * Where the product or service was provided.
     */
    locationCodeableConcept?: CodeableConcept | undefined;
    /**
     * Where the product or service was provided.
     */
    locationAddress?: Address | undefined;
    /**
     * Where the product or service was provided.
     */
    locationReference?: Reference | undefined;
    /**
     * For example in Oral whether the treatment is cosmetic or associated with TMJ, or for Medical whether the treatment was outside the clinic or outside of office hours.
     */
    modifier?: CodeableConcept[] | undefined;
    /**
     * For example, the formula: quantity * unitPrice * factor  = net. Quantity and factor are assumed to be 1 if not supplied.
     */
    net?: Money | undefined;
    /**
     * The numbers associated with notes below which apply to the adjudication of this item.
     */
    noteNumber?: number[] | undefined;
    /**
     * If this is an actual service or product line, i.e. not a Group, then use code to indicate the Professional Service or Product supplied (e.g. CTP, HCPCS, USCLS, ICD10, NCPDP, DIN, RxNorm, ACHI, CCI). If a grouping item then use a group code to indicate the type of thing being grouped e.g. 'glasses' or 'compound'.
     */
    productOrService?: CodeableConcept | undefined;
    /**
     * This contains the end of a range of product, service, drug or other billing codes for the item. This element is not used when the .productOrService is a group code. This value may only be present when a .productOfService code has been provided to convey the start of the range. Typically this value may be used only with preauthorizations and not with claims.
     */
    productOrServiceEnd?: CodeableConcept | undefined;
    /**
     * For example: Neonatal program, child dental program or drug users recovery program.
     */
    programCode?: CodeableConcept[] | undefined;
    /**
     * The providers who are authorized for the services rendered to the patient.
     */
    provider?: Reference[] | undefined;
    /**
     * The number of repetitions of a service or product.
     */
    quantity?: Quantity | undefined;
    /**
     * The type of revenue or cost center providing the product and/or service.
     */
    revenue?: CodeableConcept | undefined;
    /**
     * The date or dates when the service or product was supplied, performed or completed.
     */
    servicedDate?: string | undefined;
    _servicedDate?: Element | undefined;
    /**
     * The date or dates when the service or product was supplied, performed or completed.
     */
    servicedPeriod?: Period | undefined;
    /**
     * The sequence number of the sub-details within the details within the claim item which this line is intended to replace.
     */
    subdetailSequence?: number[] | undefined;
    /**
     * The total of taxes applicable for this product or service.
     */
    tax?: Money | undefined;
    /**
     * If the item is not a group then this is the fee for the product or service, otherwise this is the total of the fees for the details of the group.
     */
    unitPrice?: Money | undefined;
}
/**
 * Totals for amounts submitted, co-pays, benefits payable etc.
 */
export interface ClaimResponseTotal extends BackboneElement {
    /**
     * Monetary total amount associated with the category.
     */
    amount: Money;
    /**
     * For example codes indicating: Co-Pay, deductible, eligible, benefit, tax, etc.
     */
    category: CodeableConcept;
}
/**
 * Payment details for the adjudication of the claim.
 */
export interface ClaimResponsePayment extends BackboneElement {
    /**
     * Insurers will deduct amounts owing from the provider (adjustment), such as a prior overpayment, from the amount owing to the provider (benefits payable) when payment is made to the provider.
     */
    adjustment?: Money | undefined;
    /**
     * Reason for the payment adjustment.
     */
    adjustmentReason?: CodeableConcept | undefined;
    /**
     * Benefits payable less any payment adjustment.
     */
    amount: Money;
    /**
     * Estimated date the payment will be issued or the actual issue date of payment.
     */
    date?: string | undefined;
    _date?: Element | undefined;
    /**
     * For example: EFT number or check number.
     */
    identifier?: Identifier | undefined;
    /**
     * Whether this represents partial or complete payment of the benefits payable.
     */
    type: CodeableConcept;
}
/**
 * A note that describes or explains adjudication results in a human readable form.
 */
export interface ClaimResponseProcessNote extends BackboneElement {
    /**
     * Only required if the language is different from the resource language.
     */
    language?: CodeableConcept | undefined;
    /**
     * A number to uniquely identify a note entry.
     */
    number?: number | undefined;
    /**
     * The explanation or description associated with the processing.
     */
    text: string;
    _text?: Element | undefined;
    /**
     * The business purpose of the note text.
     */
    type?: ('display' | 'print' | 'printoper') | undefined;
    _type?: Element | undefined;
}
/**
 * All insurance coverages for the patient which may be applicable for reimbursement, of the products and services listed in the claim, are typically provided in the claim to allow insurers to confirm the ordering of the insurance coverages relative to local 'coordination of benefit' rules. One coverage (and only one) with 'focal=true' is to be used in the adjudication of this claim. Coverages appearing before the focal Coverage in the list, and where 'subrogation=false', should provide a reference to the ClaimResponse containing the adjudication results of the prior claim.
 */
export interface ClaimResponseInsurance extends BackboneElement {
    /**
     * A business agreement number established between the provider and the insurer for special business processing purposes.
     */
    businessArrangement?: string | undefined;
    _businessArrangement?: Element | undefined;
    /**
     * Must not be specified when 'focal=true' for this insurance.
     */
    claimResponse?: Reference | undefined;
    /**
     * Reference to the insurance card level information contained in the Coverage resource. The coverage issuing insurer will use these details to locate the patient's actual coverage within the insurer's information system.
     */
    coverage: Reference;
    /**
     * A patient may (will) have multiple insurance policies which provide reimbursement for healthcare services and products. For example a person may also be covered by their spouse's policy and both appear in the list (and may be from the same insurer). This flag will be set to true for only one of the listed policies and that policy will be used for adjudicating this claim. Other claims would be created to request adjudication against the other listed policies.
     */
    focal: boolean;
    _focal?: Element | undefined;
    /**
     * A number to uniquely identify insurance entries and provide a sequence of coverages to convey coordination of benefit order.
     */
    sequence: number;
}
/**
 * If the request contains errors then an error element should be provided and no adjudication related sections (item, addItem, or payment) should be present.
 */
export interface ClaimResponseError extends BackboneElement {
    /**
     * An error code, from a specified code system, which details why the claim could not be adjudicated.
     */
    code: CodeableConcept;
    /**
     * The sequence number of the detail within the line item submitted which contains the error. This value is omitted when the error occurs outside of the item structure.
     */
    detailSequence?: number | undefined;
    /**
     * The sequence number of the line item submitted which contains the error. This value is omitted when the error occurs outside of the item structure.
     */
    itemSequence?: number | undefined;
    /**
     * The sequence number of the sub-detail within the detail within the line item submitted which contains the error. This value is omitted when the error occurs outside of the item structure.
     */
    subDetailSequence?: number | undefined;
}
/**
 * This resource provides the adjudication details from the processing of a Claim resource.
 */
export interface ClaimResponse extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'ClaimResponse';
    /**
     * The first-tier service adjudications for payor added product or service lines.
     */
    addItem?: ClaimResponseAddItem[] | undefined;
    /**
     * The adjudication results which are presented at the header level rather than at the line-item or add-item levels.
     */
    adjudication?: ClaimResponseItemAdjudication[] | undefined;
    /**
     * For example: professional reports, documents, images, clinical resources, or accident reports.
     */
    communicationRequest?: Reference[] | undefined;
    /**
     * The date this resource was created.
     */
    created: string;
    _created?: Element | undefined;
    /**
     * The element is used to indicate the current state of the adjudication overall for the claim resource, for example: the request has been held (pended) for adjudication processing, for manual review or other reasons; that it has been processed and will be paid, or the outstanding paid, as submitted (approved); that no amount will be paid (denied); or that some amount between zero and the submitted amoutn will be paid (partial).
     */
    decision?: CodeableConcept | undefined;
    /**
     * For example DRG (Diagnosis Related Group) or a bundled billing code. A patient may have a diagnosis of a Myocardial Infarction and a DRG for HeartAttack would be assigned. The Claim item (and possible subsequent claims) would refer to the DRG for those line items that were for services related to the heart attack event.
     */
    diagnosisRelatedGroup?: CodeableConcept | undefined;
    /**
     * A human readable description of the status of the adjudication.
     */
    disposition?: string | undefined;
    _disposition?: Element | undefined;
    /**
     * This will typically be the encounter the event occurred within, but some activities may be initiated prior to or after the official completion of an encounter but still be tied to the context of the encounter.
     */
    encounter?: Reference[] | undefined;
    /**
     * If the request contains errors then an error element should be provided and no adjudication related sections (item, addItem, or payment) should be present.
     */
    error?: ClaimResponseError[] | undefined;
    /**
     * Needed to permit insurers to include the actual form.
     */
    form?: Attachment | undefined;
    /**
     * May be needed to identify specific jurisdictional forms.
     */
    formCode?: CodeableConcept | undefined;
    /**
     * Fund would be release by a future claim quoting the preAuthRef of this response. Examples of values include: provider, patient, none.
     */
    fundsReserve?: CodeableConcept | undefined;
    /**
     * A unique identifier assigned to this claim response.
     */
    identifier?: Identifier[] | undefined;
    /**
     * All insurance coverages for the patient which may be applicable for reimbursement, of the products and services listed in the claim, are typically provided in the claim to allow insurers to confirm the ordering of the insurance coverages relative to local 'coordination of benefit' rules. One coverage (and only one) with 'focal=true' is to be used in the adjudication of this claim. Coverages appearing before the focal Coverage in the list, and where 'subrogation=false', should provide a reference to the ClaimResponse containing the adjudication results of the prior claim.
     */
    insurance?: ClaimResponseInsurance[] | undefined;
    /**
     * The party responsible for authorization, adjudication and reimbursement.
     */
    insurer?: Reference | undefined;
    /**
     * A claim line. Either a simple (a product or service) or a 'group' of details which can also be a simple items or groups of sub-details.
     */
    item?: ClaimResponseItem[] | undefined;
    /**
     * The resource may be used to indicate that the Claim/Preauthorization/Pre-determination has been received but processing has not begun (queued); that it has been processed and one or more errors have been detected (error); no errors were detected and some of the adjudication processing has been performed (partial); or all of the adjudication processing has completed without errors (complete).
     */
    outcome: 'queued' | 'complete' | 'error' | 'partial';
    _outcome?: Element | undefined;
    /**
     * The party to whom the professional services and/or products have been supplied or are being considered and for whom actual for facast reimbursement is sought.
     */
    patient: Reference;
    /**
     * Type of Party to be reimbursed: subscriber, provider, other.
     */
    payeeType?: CodeableConcept | undefined;
    /**
     * Payment details for the adjudication of the claim.
     */
    payment?: ClaimResponsePayment | undefined;
    /**
     * The time frame during which this authorization is effective.
     */
    preAuthPeriod?: Period | undefined;
    /**
     * This value is only present on preauthorization adjudications.
     */
    preAuthRef?: string | undefined;
    _preAuthRef?: Element | undefined;
    /**
     * A note that describes or explains adjudication results in a human readable form.
     */
    processNote?: ClaimResponseProcessNote[] | undefined;
    /**
     * Original request resource reference.
     */
    request?: Reference | undefined;
    /**
     * This party is responsible for the claim but not necessarily professionally responsible for the provision of the individual products and services listed below. This field is the Billing Provider, for example, a facility, provider group, lab or practitioner.
     */
    requestor?: Reference | undefined;
    /**
     * This element is labeled as a modifier because the status contains codes that mark the resource as not currently valid.
     */
    status: 'active' | 'cancelled' | 'draft' | 'entered-in-error';
    _status?: Element | undefined;
    /**
     * This may contain the local bill type codes, for example the US UB-04 bill type code or the CMS bill type.
     */
    subType?: CodeableConcept | undefined;
    /**
     * Totals for amounts submitted, co-pays, benefits payable etc.
     */
    total?: ClaimResponseTotal[] | undefined;
    /**
     * This may contain the local bill type codes, for example the US UB-04 bill type code or the CMS bill type.
     */
    type: CodeableConcept;
    /**
     * A code to indicate whether the nature of the request is: Claim - A request to an Insurer to adjudicate the supplied charges for health care goods and services under the identified policy and to pay the determined Benefit amount, if any; Preauthorization - A request to an Insurer to adjudicate the supplied proposed future charges for health care goods and services under the identified policy and to approve the services and provide the expected benefit amounts and potentially to reserve funds to pay the benefits when Claims for the indicated services are later submitted; or, Pre-determination - A request to an Insurer to adjudicate the supplied 'what if' charges for health care goods and services under the identified policy and report back what the Benefit payable would be had the services actually been provided.
     */
    use: 'claim' | 'preauthorization' | 'predetermination';
    _use?: Element | undefined;
}
/**
 * Specific findings or diagnoses that were considered likely or relevant to ongoing treatment.
 */
export interface ClinicalImpressionFinding extends BackboneElement {
    /**
     * Which investigations support finding or diagnosis.
     */
    basis?: string | undefined;
    _basis?: Element | undefined;
    /**
     * Specific text, code or reference for finding or diagnosis, which may include ruled-out or resolved conditions.
     */
    item?: CodeableReference | undefined;
}
/**
 * A record of a clinical assessment performed to determine what problem(s) may affect the patient and before planning the treatments or management strategies that are best to manage a patient's condition. Assessments are often 1:1 with a clinical consultation / encounter,  but this varies greatly depending on the clinical workflow. This resource is called "ClinicalImpression" rather than "ClinicalAssessment" to avoid confusion with the recording of assessment tools such as Apgar score.
 */
export interface ClinicalImpression extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'ClinicalImpression';
    /**
     * Change in the status/pattern of a subject's condition since previously assessed, such as worsening, improving, or no change.  It is a subjective assessment of the direction of the change.
     */
    changePattern?: CodeableConcept | undefined;
    /**
     * Indicates when the documentation of the assessment was complete.
     */
    date?: string | undefined;
    _date?: Element | undefined;
    /**
     * A summary of the context and/or cause of the assessment - why / where it was performed, and what patient events/status prompted it.
     */
    description?: string | undefined;
    _description?: Element | undefined;
    /**
     * This SHOULD be accurate to at least the minute, though some assessments only have a known date.
     */
    effectiveDateTime?: string | undefined;
    _effectiveDateTime?: Element | undefined;
    /**
     * This SHOULD be accurate to at least the minute, though some assessments only have a known date.
     */
    effectivePeriod?: Period | undefined;
    /**
     * This will typically be the encounter the event occurred within, but some activities may be initiated prior to or after the official completion of an encounter but still be tied to the context of the encounter.
     */
    encounter?: Reference | undefined;
    /**
     * Specific findings or diagnoses that were considered likely or relevant to ongoing treatment.
     */
    finding?: ClinicalImpressionFinding[] | undefined;
    /**
     * This is a business identifier, not a resource identifier (see [discussion](resource.html#identifiers)).  It is best practice for the identifier to only appear on a single resource instance, however business practices may occasionally dictate that multiple resource instances with the same identifier can exist - possibly even with different resource types.  For example, multiple Patient and a Person resource instance might share the same social insurance number.
     */
    identifier?: Identifier[] | undefined;
    /**
     * Don't use this element for content that should more properly appear as one of the specific elements of the impression.
     */
    note?: Annotation[] | undefined;
    /**
     * The clinician performing the assessment.
     */
    performer?: Reference | undefined;
    /**
     * It is always likely that multiple previous assessments exist for a patient. The point of quoting a previous assessment is that this assessment is relative to it (see resolved).
     */
    previous?: Reference | undefined;
    /**
     * e.g. The patient is a pregnant, has congestive heart failure, has an ‎Adenocarcinoma, and is allergic to penicillin.
     */
    problem?: Reference[] | undefined;
    /**
     * Estimate of likely outcome.
     */
    prognosisCodeableConcept?: CodeableConcept[] | undefined;
    /**
     * RiskAssessment expressing likely outcome.
     */
    prognosisReference?: Reference[] | undefined;
    /**
     * Reference to a specific published clinical protocol that was followed during this assessment, and/or that provides evidence in support of the diagnosis.
     */
    protocol?: string[] | undefined;
    _protocol?: Element[] | undefined;
    /**
     * This element is labeled as a modifier because the status contains the code entered-in-error that marks the clinical impression as not currently valid.
     */
    status:
        | 'preparation'
        | 'in-progress'
        | 'not-done'
        | 'on-hold'
        | 'stopped'
        | 'completed'
        | 'entered-in-error'
        | 'unknown';
    _status?: Element | undefined;
    /**
     * This is generally only used for "exception" statuses such as "not-done", "on-hold" or "stopped".
     */
    statusReason?: CodeableConcept | undefined;
    /**
     * The patient or group of individuals assessed as part of this record.
     */
    subject: Reference;
    /**
     * A text summary of the investigations and the diagnosis.
     */
    summary?: string | undefined;
    _summary?: Element | undefined;
    /**
     * Information supporting the clinical impression, which can contain investigation results.
     */
    supportingInfo?: Reference[] | undefined;
}
/**
 * Information about the use of the medicinal product in relation to other therapies described as part of the contraindication.
 */
export interface ClinicalUseDefinitionContraindicationOtherTherapy extends BackboneElement {
    /**
     * The type of relationship between the medicinal product indication or contraindication and another therapy.
     */
    relationshipType: CodeableConcept;
    /**
     * Reference to a specific medication (active substance, medicinal product or class of products) as part of an indication or contraindication.
     */
    treatment: CodeableReference;
}
/**
 * Specifics for when this is a contraindication.
 */
export interface ClinicalUseDefinitionContraindication extends BackboneElement {
    /**
     * A comorbidity (concurrent condition) or coinfection.
     */
    comorbidity?: CodeableReference[] | undefined;
    /**
     * The status of the disease or symptom for the contraindication, for example "chronic" or "metastatic".
     */
    diseaseStatus?: CodeableReference | undefined;
    /**
     * The situation that is being documented as contraindicating against this item.
     */
    diseaseSymptomProcedure?: CodeableReference | undefined;
    /**
     * The indication which this is a contraidication for.
     */
    indication?: Reference[] | undefined;
    /**
     * Information about the use of the medicinal product in relation to other therapies described as part of the contraindication.
     */
    otherTherapy?: ClinicalUseDefinitionContraindicationOtherTherapy[] | undefined;
}
/**
 * Specifics for when this is an indication.
 */
export interface ClinicalUseDefinitionIndication extends BackboneElement {
    /**
     * A comorbidity (concurrent condition) or coinfection as part of the indication.
     */
    comorbidity?: CodeableReference[] | undefined;
    /**
     * The status of the disease or symptom for the indication, for example "chronic" or "metastatic".
     */
    diseaseStatus?: CodeableReference | undefined;
    /**
     * The situation that is being documented as an indicaton for this item.
     */
    diseaseSymptomProcedure?: CodeableReference | undefined;
    /**
     * Timing or duration information, that may be associated with use with the indicated condition e.g. Adult patients suffering from myocardial infarction (from a few days until less than 35 days), ischaemic stroke (from 7 days until less than 6 months).
     */
    durationRange?: Range | undefined;
    /**
     * Timing or duration information, that may be associated with use with the indicated condition e.g. Adult patients suffering from myocardial infarction (from a few days until less than 35 days), ischaemic stroke (from 7 days until less than 6 months).
     */
    durationString?: string | undefined;
    _durationString?: Element | undefined;
    /**
     * The intended effect, aim or strategy to be achieved.
     */
    intendedEffect?: CodeableReference | undefined;
    /**
     * Information about the use of the medicinal product in relation to other therapies described as part of the indication.
     */
    otherTherapy?: ClinicalUseDefinitionContraindicationOtherTherapy[] | undefined;
    /**
     * An unwanted side effect or negative outcome that may happen if you use the drug (or other subject of this resource) for this indication.
     */
    undesirableEffect?: Reference[] | undefined;
}
/**
 * The specific medication, food, substance or laboratory test that interacts.
 */
export interface ClinicalUseDefinitionInteractionInteractant extends BackboneElement {
    /**
     * The specific medication, food or laboratory test that interacts.
     */
    itemReference?: Reference | undefined;
    /**
     * The specific medication, food or laboratory test that interacts.
     */
    itemCodeableConcept?: CodeableConcept | undefined;
}
/**
 * Specifics for when this is an interaction.
 */
export interface ClinicalUseDefinitionInteraction extends BackboneElement {
    /**
     * The effect of the interaction, for example "reduced gastric absorption of primary medication".
     */
    effect?: CodeableReference | undefined;
    /**
     * The incidence of the interaction, e.g. theoretical, observed.
     */
    incidence?: CodeableConcept | undefined;
    /**
     * The specific medication, food, substance or laboratory test that interacts.
     */
    interactant?: ClinicalUseDefinitionInteractionInteractant[] | undefined;
    /**
     * Actions for managing the interaction.
     */
    management?: CodeableConcept[] | undefined;
    /**
     * The type of the interaction e.g. drug-drug interaction, drug-food interaction, drug-lab test interaction.
     */
    type?: CodeableConcept | undefined;
}
/**
 * Describe the possible undesirable effects (negative outcomes) from the use of the medicinal product as treatment.
 */
export interface ClinicalUseDefinitionUndesirableEffect extends BackboneElement {
    /**
     * High level classification of the effect.
     */
    classification?: CodeableConcept | undefined;
    /**
     * How often the effect is seen.
     */
    frequencyOfOccurrence?: CodeableConcept | undefined;
    /**
     * The situation in which the undesirable effect may manifest.
     */
    symptomConditionEffect?: CodeableReference | undefined;
}
/**
 * A critical piece of information about environmental, health or physical risks or hazards that serve as caution to the user. For example 'Do not operate heavy machinery', 'May cause drowsiness', or 'Get medical advice/attention if you feel unwell'.
 */
export interface ClinicalUseDefinitionWarning extends BackboneElement {
    /**
     * A coded or unformatted textual definition of this warning.
     */
    code?: CodeableConcept | undefined;
    /**
     * A textual definition of this warning, with formatting.
     */
    description?: string | undefined;
    _description?: Element | undefined;
}
/**
 * A single issue - either an indication, contraindication, interaction or an undesirable effect for a medicinal product, medication, device or procedure.
 */
export interface ClinicalUseDefinition extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'ClinicalUseDefinition';
    /**
     * A categorisation of the issue, primarily for dividing warnings into subject heading areas such as "Pregnancy and Lactation", "Overdose", "Effects on Ability to Drive and Use Machines".
     */
    category?: CodeableConcept[] | undefined;
    /**
     * Specifics for when this is a contraindication.
     */
    contraindication?: ClinicalUseDefinitionContraindication | undefined;
    /**
     * Business identifier for this issue.
     */
    identifier?: Identifier[] | undefined;
    /**
     * Specifics for when this is an indication.
     */
    indication?: ClinicalUseDefinitionIndication | undefined;
    /**
     * Specifics for when this is an interaction.
     */
    interaction?: ClinicalUseDefinitionInteraction | undefined;
    /**
     * The population group to which this applies.
     */
    population?: Reference[] | undefined;
    /**
     * Whether this is a current issue or one that has been retired etc.
     */
    status?: CodeableConcept | undefined;
    /**
     * The medication or procedure for which this is an indication.
     */
    subject?: Reference[] | undefined;
    /**
     * indication | contraindication | interaction | undesirable-effect | warning.
     */
    type: 'indication' | 'contraindication' | 'interaction' | 'undesirable-effect' | 'warning';
    _type?: Element | undefined;
    /**
     * Describe the possible undesirable effects (negative outcomes) from the use of the medicinal product as treatment.
     */
    undesirableEffect?: ClinicalUseDefinitionUndesirableEffect | undefined;
    /**
     * A critical piece of information about environmental, health or physical risks or hazards that serve as caution to the user. For example 'Do not operate heavy machinery', 'May cause drowsiness', or 'Get medical advice/attention if you feel unwell'.
     */
    warning?: ClinicalUseDefinitionWarning | undefined;
}
/**
 * Note that filters defined in code systems usually require custom code on the part of any terminology engine that will make them available for use in value set filters. For this reason, they are generally only seen in high value published terminologies.
 */
export interface CodeSystemFilter extends BackboneElement {
    /**
     * The code that identifies this filter when it is used as a filter in [ValueSet](valueset.html#).compose.include.filter.
     */
    code: string;
    _code?: Element | undefined;
    /**
     * A description of how or why the filter is used.
     */
    description?: string | undefined;
    _description?: Element | undefined;
    /**
     * A list of operators that can be used with the filter.
     */
    operator: (
        | '='
        | 'is-a'
        | 'descendent-of'
        | 'is-not-a'
        | 'regex'
        | 'in'
        | 'not-in'
        | 'generalizes'
        | 'child-of'
        | 'descendent-leaf'
        | 'exists'
    )[];
    _operator?: Element[] | undefined;
    /**
     * A description of what the value for the filter should be.
     */
    value: string;
    _value?: Element | undefined;
}
/**
 * Multiple occurrences of CodeSystem.concept.property may occur for a CodeSystem.property where     CodeSystem.concept.property.code is the same and CodeSystem.concept.property.value differs. For example: multiple designations for a single concept.
 */
export interface CodeSystemProperty extends BackboneElement {
    /**
     * A code that is used to identify the property. The code is used internally (in CodeSystem.concept.property.code) and also externally, such as in property filters.
     */
    code: string;
    _code?: Element | undefined;
    /**
     * A description of the property- why it is defined, and how its value might be used.
     */
    description?: string | undefined;
    _description?: Element | undefined;
    /**
     * The type of the property value. Properties of type "code" contain a code defined by the code system (e.g. a reference to another defined concept).
     */
    type: 'code' | 'Coding' | 'string' | 'integer' | 'boolean' | 'dateTime' | 'decimal';
    _type?: Element | undefined;
    /**
     * Reference to the formal meaning of the property. One possible source of meaning is the [Concept Properties](codesystem-concept-properties.html) code system.
     */
    uri?: string | undefined;
    _uri?: Element | undefined;
}
/**
 * Concepts have both a ```display``` and an array of ```designation```. The display is equivalent to a special designation with an implied ```designation.use``` of "primary code" and a language equal to the [Resource Language](resource.html#language).
 */
export interface CodeSystemConceptDesignation extends BackboneElement {
    /**
     * This was added rather than increasing the cardinality of .use to 0..* in order to maintain backward compatibility.
     */
    additionalUse?: Coding[] | undefined;
    /**
     * In the absence of a language, the resource language applies.
     */
    language?: string | undefined;
    _language?: Element | undefined;
    /**
     * If no use is provided, the designation can be assumed to be suitable for general display to a human user.
     */
    use?: Coding | undefined;
    /**
     * The text value for this designation.
     */
    value: string;
    _value?: Element | undefined;
}
/**
 * A property value for this concept.
 */
export interface CodeSystemConceptProperty extends BackboneElement {
    /**
     * A code that is a reference to CodeSystem.property.code.
     */
    code: string;
    _code?: Element | undefined;
    /**
     * The value of this property.
     */
    valueCode?: string | undefined;
    _valueCode?: Element | undefined;
    /**
     * The value of this property.
     */
    valueCoding?: Coding | undefined;
    /**
     * The value of this property.
     */
    valueString?: string | undefined;
    _valueString?: Element | undefined;
    /**
     * The value of this property.
     */
    valueInteger?: number | undefined;
    /**
     * The value of this property.
     */
    valueBoolean?: boolean | undefined;
    _valueBoolean?: Element | undefined;
    /**
     * The value of this property.
     */
    valueDateTime?: string | undefined;
    _valueDateTime?: Element | undefined;
    /**
     * The value of this property.
     */
    valueDecimal?: number | undefined;
}
/**
 * If this is empty, it means that the code system resource does not represent the content of the code system.
 */
export interface CodeSystemConcept extends BackboneElement {
    /**
     * A code - a text symbol - that uniquely identifies the concept within the code system.
     */
    code: string;
    _code?: Element | undefined;
    /**
     * Defines children of a concept to produce a hierarchy of concepts. The nature of the relationships is variable (is-a/contains/categorizes) - see hierarchyMeaning.
     */
    concept?: CodeSystemConcept[] | undefined;
    /**
     * The formal definition of the concept. The code system resource does not make formal definitions required, because of the prevalence of legacy systems. However, they are highly recommended, as without them there is no formal meaning associated with the concept.
     */
    definition?: string | undefined;
    _definition?: Element | undefined;
    /**
     * Concepts have both a ```display``` and an array of ```designation```. The display is equivalent to a special designation with an implied ```designation.use``` of "primary code" and a language equal to the [Resource Language](resource.html#language).
     */
    designation?: CodeSystemConceptDesignation[] | undefined;
    /**
     * A human readable string that is the recommended default way to present this concept to a user.
     */
    display?: string | undefined;
    _display?: Element | undefined;
    /**
     * A property value for this concept.
     */
    property?: CodeSystemConceptProperty[] | undefined;
}
/**
 * The CodeSystem resource is used to declare the existence of and describe a code system or code system supplement and its key properties, and optionally define a part or all of its content.
 */
export interface CodeSystem extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'CodeSystem';
    /**
     * The 'date' element may be more recent than the approval date because of minor changes or editorial corrections.
     */
    approvalDate?: string | undefined;
    _approvalDate?: Element | undefined;
    /**
     * An individiual or organization primarily involved in the creation and maintenance of the CodeSystem.
     */
    author?: ContactDetail[] | undefined;
    /**
     * If this value is missing, then it is not specified whether a code system is case sensitive or not. When the rule is not known, Postel's law should be followed: produce codes with the correct case, and accept codes in any case. This element is primarily provided to support validation software.
     */
    caseSensitive?: boolean | undefined;
    _caseSensitive?: Element | undefined;
    /**
     * Note that the code system resource does not define what the compositional grammar is, only whether or not there is one.
     */
    compositional?: boolean | undefined;
    _compositional?: Element | undefined;
    /**
     * If this is empty, it means that the code system resource does not represent the content of the code system.
     */
    concept?: CodeSystemConcept[] | undefined;
    /**
     * May be a web site, an email address, a telephone number, etc.
     */
    contact?: ContactDetail[] | undefined;
    /**
     * The extent of the content of the code system (the concepts and codes it defines) are represented in this resource instance.
     */
    content: 'not-present' | 'example' | 'fragment' | 'complete' | 'supplement';
    _content?: Element | undefined;
    /**
     * ... Sometimes, the copyright differs between the code system and the codes that are included. The copyright statement should clearly differentiate between these when required.
     */
    copyright?: string | undefined;
    _copyright?: Element | undefined;
    /**
     * The count of concepts defined in this resource cannot be more than this value but may be less for several reasons - see the content element.
     */
    count?: number | undefined;
    /**
     * Note that this is not the same as the resource last-modified-date, since the resource may be a secondary representation of the code system. Additional specific dates may be added as extensions or be found by consulting Provenances associated with past versions of the resource.
     */
    date?: string | undefined;
    _date?: Element | undefined;
    /**
     * This description can be used to capture details such as comments about misuse, instructions for clinical use and interpretation, literature references, examples from the paper world, etc. It is not a rendering of the code system as conveyed in the 'text' field of the resource itself. This item SHOULD be populated unless the information is available from context (e.g. the language of the code system is presumed to be the predominant language in the place the code system was created).
     */
    description?: string | undefined;
    _description?: Element | undefined;
    /**
     * An individual or organization primarily responsible for internal coherence of the CodeSystem.
     */
    editor?: ContactDetail[] | undefined;
    /**
     * The effective period for a CodeSystem  determines when the content is applicable for usage and is independent of publication and review dates. For example, a measure intended to be used for the year 2016 might be published in 2015.
     */
    effectivePeriod?: Period | undefined;
    /**
     * An individual or organization responsible for officially endorsing the CodeSystem for use in some setting.
     */
    endorser?: ContactDetail[] | undefined;
    /**
     * Allows filtering of code systems that are appropriate for use versus not.
     */
    experimental?: boolean | undefined;
    _experimental?: Element | undefined;
    /**
     * Note that filters defined in code systems usually require custom code on the part of any terminology engine that will make them available for use in value set filters. For this reason, they are generally only seen in high value published terminologies.
     */
    filter?: CodeSystemFilter[] | undefined;
    /**
     * Note that other representations might have a different hierarchy or none at all, and represent the information using properties.
     */
    hierarchyMeaning?: ('grouped-by' | 'is-a' | 'part-of' | 'classified-with') | undefined;
    _hierarchyMeaning?: Element | undefined;
    /**
     * Typically, this is used for identifiers that can go in an HL7 V3 II (instance identifier) data type, and can then identify this code system outside of FHIR, where it is not possible to use the logical URI.  Note that HL7 defines at least three identifiers for many of its code systems - the FHIR canonical URL, the OID and the V2 Table 0396 mnemonic code.
     */
    identifier?: Identifier[] | undefined;
    /**
     * It may be possible for the code system to be used in jurisdictions other than those for which it was originally designed or intended.
     */
    jurisdiction?: CodeableConcept[] | undefined;
    /**
     * If specified, this date follows the original approval date.
     */
    lastReviewDate?: string | undefined;
    _lastReviewDate?: Element | undefined;
    /**
     * The name is not expected to be globally unique. The name should be a simple alphanumeric type name to ensure that it is machine-processing friendly.
     */
    name?: string | undefined;
    _name?: Element | undefined;
    /**
     * Multiple occurrences of CodeSystem.concept.property may occur for a CodeSystem.property where     CodeSystem.concept.property.code is the same and CodeSystem.concept.property.value differs. For example: multiple designations for a single concept.
     */
    property?: CodeSystemProperty[] | undefined;
    /**
     * Usually an organization but may be an individual. The publisher (or steward) of the code system is the organization or individual primarily responsible for the maintenance and upkeep of the code system. This is not necessarily the same individual or organization that developed and initially authored the content. The publisher is the primary point of contact for questions or issues with the code system. This item SHOULD be populated unless the information is available from context.
     */
    publisher?: string | undefined;
    _publisher?: Element | undefined;
    /**
     * This element does not describe the usage of the code system. Instead, it provides traceability of ''why'' the resource is either needed or ''why'' it is defined as it is.  This may be used to point to source materials or specifications that drove the structure of this code system.
     */
    purpose?: string | undefined;
    _purpose?: Element | undefined;
    /**
     * Each related artifact is either an attachment, or a reference to another resource, but not both.
     */
    relatedArtifact?: RelatedArtifact[] | undefined;
    /**
     * An individual or organization primarily responsible for review of some aspect of the CodeSystem.
     */
    reviewer?: ContactDetail[] | undefined;
    /**
     * Allows filtering of code systems that are appropriate for use versus not.
     */
    status: 'draft' | 'active' | 'retired' | 'unknown';
    _status?: Element | undefined;
    /**
     * The most common use of a code system supplement is to add additional language support.
     */
    supplements?: string | undefined;
    _supplements?: Element | undefined;
    /**
     * This name does not need to be machine-processing friendly and may contain punctuation, white-space, etc.
     */
    title?: string | undefined;
    _title?: Element | undefined;
    /**
     * This element provides topical categorization and keywords for the CodeSystem, as opposed to the more structured context-of-use information provided in the useContext element
     */
    topic?: CodeableConcept[] | undefined;
    /**
     * Can be a urn:uuid: or a urn:oid: but real http: addresses are preferred.  Multiple instances may share the same URL if they have a distinct version.
     * The determination of when to create a new version of a resource (same url, new version) vs. defining a new artifact is up to the author.  Considerations for making this decision are found in [Technical and Business Versions](resource.html#versions).
     * In some cases, the resource can no longer be found at the stated url, but the url itself cannot change. Implementations can use the [meta.source](resource.html#meta) element to indicate where the current master source of the resource can be found.
     */
    url?: string | undefined;
    _url?: Element | undefined;
    /**
     * When multiple useContexts are specified, there is no expectation that all or any of the contexts apply.
     */
    useContext?: UsageContext[] | undefined;
    /**
     * The definition of the value set SHALL include all codes from this code system, independent of concept status, and only codes from this code system. It SHALL be immutable, and operations SHOULD support validation of any allowed code compositions.
     */
    valueSet?: string | undefined;
    _valueSet?: Element | undefined;
    /**
     * There may be different code system instances that have the same identifier but different versions.  The version can be appended to the url in a reference to allow a reference to a particular business version of the code system with the format [url]|[version]. The version SHOULD NOT contain a '#' - see [Business Version](resource.html#bv-format).
     */
    version?: string | undefined;
    _version?: Element | undefined;
    /**
     * Best practice is that code systems do not redefine concepts, or that if concepts are redefined, a new code system definition is created. But this is not always possible, so some code systems may be defined as 'versionNeeded'.
     * Most code systems occasionally refine the displays defined for concepts between versions. Contexts in which the concept display values are validated may require that the version be specified for some code systems irrespective of the value of this property.
     */
    versionNeeded?: boolean | undefined;
    _versionNeeded?: Element | undefined;
}
/**
 * Text, attachment(s), or resource(s) that was communicated to the recipient.
 */
export interface CommunicationPayload extends BackboneElement {
    /**
     * The content can be codified or textual.  As an example of codified content, when working with machine generated communications, the payload may be drawn from a finite, terminology defined set of communications, especially for short, status update kinds of communications where more of a natural language approach isn't appropriate.
     * If the content isn't codified, contentCodeableConcept.text can be used.
     * When using contentCodeableConcept, the CodeableConcept is what is being communicated and is not a categorization of the content.
     */
    contentAttachment?: Attachment | undefined;
    /**
     * The content can be codified or textual.  As an example of codified content, when working with machine generated communications, the payload may be drawn from a finite, terminology defined set of communications, especially for short, status update kinds of communications where more of a natural language approach isn't appropriate.
     * If the content isn't codified, contentCodeableConcept.text can be used.
     * When using contentCodeableConcept, the CodeableConcept is what is being communicated and is not a categorization of the content.
     */
    contentReference?: Reference | undefined;
    /**
     * The content can be codified or textual.  As an example of codified content, when working with machine generated communications, the payload may be drawn from a finite, terminology defined set of communications, especially for short, status update kinds of communications where more of a natural language approach isn't appropriate.
     * If the content isn't codified, contentCodeableConcept.text can be used.
     * When using contentCodeableConcept, the CodeableConcept is what is being communicated and is not a categorization of the content.
     */
    contentCodeableConcept?: CodeableConcept | undefined;
}
/**
 * A clinical or business level record of information being transmitted or shared; e.g. an alert that was sent to a responsible provider, a public health agency communication to a provider/reporter in response to a case report for a reportable condition.
 */
export interface Communication extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'Communication';
    /**
     * Don't use Communication.about element when a more specific element exists, such as basedOn or reasonReference.
     */
    about?: Reference[] | undefined;
    /**
     * This must point to some sort of a 'Request' resource, such as CarePlan, CommunicationRequest, ServiceRequest, MedicationRequest, etc.
     */
    basedOn?: Reference[] | undefined;
    /**
     * There may be multiple axes of categorization and one communication may serve multiple purposes.
     */
    category?: CodeableConcept[] | undefined;
    /**
     * This will typically be the encounter the event occurred within, but some activities may be initiated prior to or after the official completion of an encounter but still be tied to the context of the encounter.
     */
    encounter?: Reference | undefined;
    /**
     * This is a business identifier, not a resource identifier (see [discussion](resource.html#identifiers)).  It is best practice for the identifier to only appear on a single resource instance, however business practices may occasionally dictate that multiple resource instances with the same identifier can exist - possibly even with different resource types.  For example, multiple Patient and a Person resource instance might share the same social insurance number.
     */
    identifier?: Identifier[] | undefined;
    /**
     * Prior communication that this communication is in response to.
     */
    inResponseTo?: Reference[] | undefined;
    /**
     * The URL pointing to a FHIR-defined protocol, guideline, orderset or other definition that is adhered to in whole or in part by this Communication.
     */
    instantiatesCanonical?: string[] | undefined;
    _instantiatesCanonical?: Element[] | undefined;
    /**
     * This might be an HTML page, PDF, etc. or could just be a non-resolvable URI identifier.
     */
    instantiatesUri?: string[] | undefined;
    _instantiatesUri?: Element[] | undefined;
    /**
     * A channel that was used for this communication (e.g. email, fax).
     */
    medium?: CodeableConcept[] | undefined;
    /**
     * Additional notes or commentary about the communication by the sender, receiver or other interested parties.
     */
    note?: Annotation[] | undefined;
    /**
     * The event the communication was part of - e.g. if you disclosed as "part" of counselling, it would be partOf that Procedure.
     */
    partOf?: Reference[] | undefined;
    /**
     * Text, attachment(s), or resource(s) that was communicated to the recipient.
     */
    payload?: CommunicationPayload[] | undefined;
    /**
     * Used to prioritize workflow (such as which communication to read first) when the communication is planned or in progress.
     */
    priority?: ('routine' | 'urgent' | 'asap' | 'stat') | undefined;
    _priority?: Element | undefined;
    /**
     * Textual reasons can be captured using reason.concept.text.
     */
    reason?: CodeableReference[] | undefined;
    /**
     * The time when this communication arrived at the destination.
     */
    received?: string | undefined;
    _received?: Element | undefined;
    /**
     * If receipts need to be tracked by an individual, a separate resource instance will need to be created for each recipient.  Multiple recipient communications are intended where either receipts are not tracked (e.g. a mass mail-out) or a receipt is captured in aggregate (all emails confirmed received by a particular time).
     */
    recipient?: Reference[] | undefined;
    /**
     * The entity (e.g. person, organization, clinical information system, or device) which is the source of the communication.
     */
    sender?: Reference | undefined;
    /**
     * The time when this communication was sent.
     */
    sent?: string | undefined;
    _sent?: Element | undefined;
    /**
     * This element is labeled as a modifier because the status contains the codes aborted and entered-in-error that mark the communication as not currently valid.
     */
    status:
        | 'preparation'
        | 'in-progress'
        | 'not-done'
        | 'on-hold'
        | 'stopped'
        | 'completed'
        | 'entered-in-error'
        | 'unknown';
    _status?: Element | undefined;
    /**
     * This is generally only used for "exception" statuses such as "not-done", "suspended" or "aborted". The reason for performing the event at all is captured in reasonCode, not here.
     */
    statusReason?: CodeableConcept | undefined;
    /**
     * The patient or group that was the focus of this communication.
     */
    subject?: Reference | undefined;
    /**
     * Communication.topic.text can be used without any codings.
     */
    topic?: CodeableConcept | undefined;
}
/**
 * Text, attachment(s), or resource(s) to be communicated to the recipient.
 */
export interface CommunicationRequestPayload extends BackboneElement {
    /**
     * The content can be codified or textual.  As an example of codified content, when working with machine generated communications, the payload may be drawn from a finite, terminology defined set of communications, especially for short, status update kinds of communications where more of a natural language approach isn't appropriate.
     * If the content isn't codified, contentCodeableConcept.text can be used.
     * When using contentCodeableConcept, the CodeableConcept is what is being communicated and is not a categorization of the content.
     */
    contentAttachment?: Attachment | undefined;
    /**
     * The content can be codified or textual.  As an example of codified content, when working with machine generated communications, the payload may be drawn from a finite, terminology defined set of communications, especially for short, status update kinds of communications where more of a natural language approach isn't appropriate.
     * If the content isn't codified, contentCodeableConcept.text can be used.
     * When using contentCodeableConcept, the CodeableConcept is what is being communicated and is not a categorization of the content.
     */
    contentReference?: Reference | undefined;
    /**
     * The content can be codified or textual.  As an example of codified content, when working with machine generated communications, the payload may be drawn from a finite, terminology defined set of communications, especially for short, status update kinds of communications where more of a natural language approach isn't appropriate.
     * If the content isn't codified, contentCodeableConcept.text can be used.
     * When using contentCodeableConcept, the CodeableConcept is what is being communicated and is not a categorization of the content.
     */
    contentCodeableConcept?: CodeableConcept | undefined;
}
/**
 * A request to convey information; e.g. the CDS system proposes that an alert be sent to a responsible provider, the CDS system proposes that the public health agency be notified about a reportable condition.
 */
export interface CommunicationRequest extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'CommunicationRequest';
    /**
     * Don't use CommunicationRequest.about element when a more specific element exists, such as basedOn, reasonReference, or replaces.
     */
    about?: Reference[] | undefined;
    /**
     * For draft requests, indicates the date of initial creation.  For requests with other statuses, indicates the date of activation.
     */
    authoredOn?: string | undefined;
    _authoredOn?: Element | undefined;
    /**
     * A plan or proposal that is fulfilled in whole or in part by this request.
     */
    basedOn?: Reference[] | undefined;
    /**
     * There may be multiple axes of categorization and one communication request may serve multiple purposes.
     */
    category?: CodeableConcept[] | undefined;
    /**
     * The attributes provided with the request qualify what is not to be done.
     */
    doNotPerform?: boolean | undefined;
    _doNotPerform?: Element | undefined;
    /**
     * This will typically be the encounter the event occurred within, but some activities may be initiated prior to or after the official completion of an encounter but still be tied to the context of the encounter.
     */
    encounter?: Reference | undefined;
    /**
     * Requests are linked either by a "basedOn" relationship (i.e. one request is fulfilling another) or by having a common requisition.  Requests that are part of the same requisition are generally treated independently from the perspective of changing their state or maintaining them after initial creation.
     */
    groupIdentifier?: Identifier | undefined;
    /**
     * This is a business identifier, not a resource identifier (see [discussion](resource.html#identifiers)).  It is best practice for the identifier to only appear on a single resource instance, however business practices may occasionally dictate that multiple resource instances with the same identifier can exist - possibly even with different resource types.  For example, multiple Patient and a Person resource instance might share the same social insurance number.
     */
    identifier?: Identifier[] | undefined;
    /**
     * The entity (e.g. person, organization, clinical information system, or device) which is to be the source of the communication.
     */
    informationProvider?: Reference[] | undefined;
    /**
     * This element is expected to be immutable.  E.g. A "proposal" instance should never change to be a "plan" instance or "order" instance.  Instead, a new instance 'basedOn' the prior instance should be created with the new 'intent' value.
     * One exception to this is that the granularity of CommunicationRequest.intent is allowed to change.  For example, a Request identified as an "order" might later be clarified to be a "filler-order".  Or, in rarer cases (to meet recipient constraints), the reverse might also occur.
     */
    intent:
        | 'proposal'
        | 'plan'
        | 'directive'
        | 'order'
        | 'original-order'
        | 'reflex-order'
        | 'filler-order'
        | 'instance-order'
        | 'option';
    _intent?: Element | undefined;
    /**
     * A channel that was used for this communication (e.g. email, fax).
     */
    medium?: CodeableConcept[] | undefined;
    /**
     * Comments made about the request by the requester, sender, recipient, subject or other participants.
     */
    note?: Annotation[] | undefined;
    /**
     * The time when this communication is to occur.
     */
    occurrenceDateTime?: string | undefined;
    _occurrenceDateTime?: Element | undefined;
    /**
     * The time when this communication is to occur.
     */
    occurrencePeriod?: Period | undefined;
    /**
     * Text, attachment(s), or resource(s) to be communicated to the recipient.
     */
    payload?: CommunicationRequestPayload[] | undefined;
    /**
     * Characterizes how quickly the proposed act must be initiated. Includes concepts such as stat, urgent, routine.
     */
    priority?: ('routine' | 'urgent' | 'asap' | 'stat') | undefined;
    _priority?: Element | undefined;
    /**
     * Textual reasons can be captured using reasonCode.text.
     */
    reason?: CodeableReference[] | undefined;
    /**
     * The entity (e.g. person, organization, clinical information system, device, group, or care team) which is the intended target of the communication.
     */
    recipient?: Reference[] | undefined;
    /**
     * The replacement could be because the initial request was immediately rejected (due to an issue) or because the previous request was completed, but the need for the action described by the request remains ongoing.
     */
    replaces?: Reference[] | undefined;
    /**
     * The device, individual, or organization who asks for the information to be shared.
     */
    requester?: Reference | undefined;
    /**
     * The status of the proposal or order.
     */
    status: 'draft' | 'active' | 'on-hold' | 'revoked' | 'completed' | 'entered-in-error' | 'unknown';
    _status?: Element | undefined;
    /**
     * This is generally only used for "exception" statuses such as "on-hold" or "revoked".  The reason why the CommunicationRequest was created at all is captured in reasonCode, not here.
     */
    statusReason?: CodeableConcept | undefined;
    /**
     * The patient or group that is the focus of this communication request.
     */
    subject?: Reference | undefined;
}
/**
 * Information about how a resource is related to the compartment.
 */
export interface CompartmentDefinitionResource extends BackboneElement {
    /**
     * The name of a resource supported by the server.
     */
    code: string;
    _code?: Element | undefined;
    /**
     * Additional documentation about the resource and compartment.
     */
    documentation?: string | undefined;
    _documentation?: Element | undefined;
    /**
     * Search Parameter for mapping requests made with $everything.end (e.g. on [Patient.$everything](patient-operation-everything.html)).
     */
    endParam?: string | undefined;
    _endParam?: Element | undefined;
    /**
     * If no search parameters are listed, then the resource is not linked to the compartment.
     */
    param?: string[] | undefined;
    _param?: Element[] | undefined;
    /**
     * Search Parameter for mapping requests made with $everything.start (e.g. on [Patient.$everything](patient-operation-everything.html)).
     */
    startParam?: string | undefined;
    _startParam?: Element | undefined;
}
/**
 * A compartment definition that defines how resources are accessed on a server.
 */
export interface CompartmentDefinition extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'CompartmentDefinition';
    /**
     * Only the specification can define the compartments that can exist. Servers can choose to support them.
     */
    code: 'Patient' | 'Encounter' | 'RelatedPerson' | 'Practitioner' | 'Device';
    _code?: Element | undefined;
    /**
     * May be a web site, an email address, a telephone number, etc.
     */
    contact?: ContactDetail[] | undefined;
    /**
     * Note that this is not the same as the resource last-modified-date, since the resource may be a secondary representation of the compartment definition. Additional specific dates may be added as extensions or be found by consulting Provenances associated with past versions of the resource.
     */
    date?: string | undefined;
    _date?: Element | undefined;
    /**
     * This description can be used to capture details such as comments about misuse, instructions for clinical use and interpretation, literature references, examples from the paper world, etc. It is not a rendering of the compartment definition as conveyed in the 'text' field of the resource itself. This item SHOULD be populated unless the information is available from context (e.g. the language of the compartment definition is presumed to be the predominant language in the place the compartment definition was created).
     */
    description?: string | undefined;
    _description?: Element | undefined;
    /**
     * Allows filtering of compartment definitions that are appropriate for use versus not.
     */
    experimental?: boolean | undefined;
    _experimental?: Element | undefined;
    /**
     * The name is not expected to be globally unique. The name should be a simple alphanumeric type name to ensure that it is machine-processing friendly.This is often the same as the code for the parameter, but does not need to be.
     */
    name: string;
    _name?: Element | undefined;
    /**
     * Usually an organization but may be an individual. The publisher (or steward) of the compartment definition is the organization or individual primarily responsible for the maintenance and upkeep of the compartment definition. This is not necessarily the same individual or organization that developed and initially authored the content. The publisher is the primary point of contact for questions or issues with the compartment definition. This item SHOULD be populated unless the information is available from context.
     */
    publisher?: string | undefined;
    _publisher?: Element | undefined;
    /**
     * This element does not describe the usage of the compartment definition. Instead, it provides traceability of ''why'' the resource is either needed or ''why'' it is defined as it is.  This may be used to point to source materials or specifications that drove the structure of this compartment definition.
     */
    purpose?: string | undefined;
    _purpose?: Element | undefined;
    /**
     * Information about how a resource is related to the compartment.
     */
    resource?: CompartmentDefinitionResource[] | undefined;
    /**
     * Servers may define and use compartments to manage logical access without implementing the compartment related syntax.
     */
    search: boolean;
    _search?: Element | undefined;
    /**
     * Allows filtering of compartment definitions that are appropriate for use versus not.
     */
    status: 'draft' | 'active' | 'retired' | 'unknown';
    _status?: Element | undefined;
    /**
     * This name does not need to be machine-processing friendly and may contain punctuation, white-space, etc.
     */
    title?: string | undefined;
    _title?: Element | undefined;
    /**
     * Can be a urn:uuid: or a urn:oid: but real http: addresses are preferred.  Multiple instances may share the same URL if they have a distinct version.
     * The determination of when to create a new version of a resource (same url, new version) vs. defining a new artifact is up to the author.  Considerations for making this decision are found in [Technical and Business Versions](resource.html#versions).
     * In some cases, the resource can no longer be found at the stated url, but the url itself cannot change. Implementations can use the [meta.source](resource.html#meta) element to indicate where the current master source of the resource can be found.
     */
    url: string;
    _url?: Element | undefined;
    /**
     * When multiple useContexts are specified, there is no expectation that all or any of the contexts apply.
     */
    useContext?: UsageContext[] | undefined;
    /**
     * There may be different compartment definition instances that have the same identifier but different versions.  The version can be appended to the url in a reference to allow a reference to a particular business version of the compartment definition with the format [url]|[version]. The version SHOULD NOT contain a '#' - see [Business Version](resource.html#bv-format).
     */
    version?: string | undefined;
    _version?: Element | undefined;
    /**
     * If set as a string, this is a FHIRPath expression that has two additional context variables passed in - %version1 and %version2 and will return a negative number if version1 is newer, a positive number if version2 and a 0 if the version ordering can't be successfully be determined.
     */
    versionAlgorithmString?: string | undefined;
    _versionAlgorithmString?: Element | undefined;
    /**
     * If set as a string, this is a FHIRPath expression that has two additional context variables passed in - %version1 and %version2 and will return a negative number if version1 is newer, a positive number if version2 and a 0 if the version ordering can't be successfully be determined.
     */
    versionAlgorithmCoding?: Coding | undefined;
}
/**
 * Only list each attester once.
 */
export interface CompositionAttester extends BackboneElement {
    /**
     * The type of attestation the authenticator offers.
     */
    mode: CodeableConcept;
    /**
     * Who attested the composition in the specified way.
     */
    party?: Reference | undefined;
    /**
     * When the composition was attested by the party.
     */
    time?: string | undefined;
    _time?: Element | undefined;
}
/**
 * The event needs to be consistent with the type element, though can provide further information if desired.
 */
export interface CompositionEvent extends BackboneElement {
    /**
     * An event can further specialize the act inherent in the typeCode, such as where it is simply "Procedure Report" and the procedure was a "colonoscopy". If one or more eventCodes are included, they SHALL NOT conflict with the values inherent in the classCode, practiceSettingCode or typeCode, as such a conflict would create an ambiguous situation. This short list of codes is provided to be used as key words for certain types of queries.
     */
    code?: CodeableConcept[] | undefined;
    /**
     * The description and/or reference of the event(s) being documented. For example, this could be used to document such a colonoscopy or an appendectomy.
     */
    detail?: Reference[] | undefined;
    /**
     * The period of time covered by the documentation. There is no assertion that the documentation is a complete representation for this period, only that it documents events during this time.
     */
    period?: Period | undefined;
}
/**
 * The root of the sections that make up the composition.
 */
export interface CompositionSection extends BackboneElement {
    /**
     * Identifies who is responsible for the information in this section, not necessarily who typed it in.
     */
    author?: Reference[] | undefined;
    /**
     * The code identifies the section for an automated processor of the document. This is particularly relevant when using profiles to control the structure of the document.
     * If the section has content (instead of sub-sections), the section.code does not change the meaning or interpretation of the resource that is the content of the section in the comments for the section.code.
     */
    code?: CodeableConcept | undefined;
    /**
     * The various reasons for an empty section make a significant interpretation to its interpretation. Note that this code is for use when the entire section content has been suppressed, and not for when individual items are omitted - implementers may consider using a text note or a flag on an entry in these cases.
     */
    emptyReason?: CodeableConcept | undefined;
    /**
     * If there are no entries in the list, an emptyReason SHOULD be provided.
     */
    entry?: Reference[] | undefined;
    /**
     * Typically, sections in a doument are about the subject of the document, whether that is a  patient, or group of patients, location, or device, or whatever. For some kind of documents, some sections actually contain data about related entities. Typical examples are  a section in a newborn discharge summary concerning the mother, or family history documents, with a section about each family member, though there are many other examples.
     */
    focus?: Reference | undefined;
    /**
     * This element is labeled as a modifier because a change list must not be misunderstood as a complete list.
     */
    mode?: ('working' | 'snapshot' | 'changes') | undefined;
    _mode?: Element | undefined;
    /**
     * Applications SHOULD render ordered lists in the order provided, but MAY allow users to re-order based on their own preferences as well. If there is no order specified, the order is unknown, though there may still be some order.
     */
    orderedBy?: CodeableConcept | undefined;
    /**
     * Nested sections are primarily used to help human readers navigate to particular portions of the document.
     */
    section?: CompositionSection[] | undefined;
    /**
     * Document profiles may define what content should be represented in the narrative to ensure clinical safety.
     */
    text?: Narrative | undefined;
    /**
     * The title identifies the section for a human reader. The title must be consistent with the narrative of the resource that is the target of the section.content reference. Generally, sections SHOULD have titles, but in some documents, it is unnecessary or inappropriate. Typically, this is where a section has subsections that have their own adequately distinguishing title,  or documents that only have a single section. Most Implementation Guides will make section title to be a required element.
     */
    title?: string | undefined;
    _title?: Element | undefined;
}
/**
 * A set of healthcare-related information that is assembled together into a single logical package that provides a single coherent statement of meaning, establishes its own context and that has clinical attestation with regard to who is making the statement. A Composition defines the structure and narrative content necessary for a document. However, a Composition alone does not constitute a document. Rather, the Composition must be the first entry in a Bundle where Bundle.type=document, and any other resources referenced from Composition must be included as subsequent entries in the Bundle (for example Patient, Practitioner, Encounter, etc.).
 */
export interface Composition extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'Composition';
    /**
     * Only list each attester once.
     */
    attester?: CompositionAttester[] | undefined;
    /**
     * Identifies who is responsible for the information in the composition, not necessarily who typed it in.
     */
    author: Reference[];
    /**
     * A categorization for the type of the composition - helps for indexing and searching. This may be implied by or derived from the code specified in the Composition Type.
     */
    category?: CodeableConcept[] | undefined;
    /**
     * This is useful when documents are derived from a composition - provides guidance for how to get the latest version of the document. This is optional because this is sometimes not known by the authoring system, and can be inferred by context. However, it is important that this information be known when working with a derived document, so providing a custodian is encouraged.
     */
    custodian?: Reference | undefined;
    /**
     * The Last Modified Date on the composition may be after the date of the document was attested without being changed. This means that the date on an amended document is the date of the amendment, not the date of original authorship.
     */
    date: string;
    _date?: Element | undefined;
    /**
     * Describes the clinical encounter or type of care this documentation is associated with.
     */
    encounter?: Reference | undefined;
    /**
     * The event needs to be consistent with the type element, though can provide further information if desired.
     */
    event?: CompositionEvent[] | undefined;
    /**
     * Similar to ClinicalDocument/setId in CDA. See discussion in resource definition for how these relate.
     */
    identifier?: Identifier[] | undefined;
    /**
     * The name is not expected to be globally unique. The name should be a simple alphanumeric type name to ensure that it is machine-processing friendly.
     */
    name?: string | undefined;
    _name?: Element | undefined;
    /**
     * A comment, general note or annotation not coded elsewhere.
     */
    note?: Annotation[] | undefined;
    /**
     * To reference (using relatesTo) a Bundle of type "document" or another Composition where you only have the identifier, use a resourceReference with an identifier.\nHere is an example of using relatesTo to reference another Bundle of type "document" or Composition where you only have the identifier:
     * ```
     * <relatesTo  >
     *     <type value="replaces" />
     * <resourceReference>
     * <identifier>
     * <system value="..." />
     * <value value="..." />
     * </identifier>
     * </resourceReference>
     * </relatesTo>
     * ```
     */
    relatesTo?: RelatedArtifact[] | undefined;
    /**
     * The root of the sections that make up the composition.
     */
    section?: CompositionSection[] | undefined;
    /**
     * If a composition is marked as withdrawn, the compositions/documents in the series, or data from the composition or document series, should never be displayed to a user without being clearly marked as untrustworthy. The flag "entered-in-error" is why this element is labeled as a modifier of other elements.
     * Some reporting work flows require that the original narrative of a final document never be altered; instead, only new narrative can be added. The composition resource has no explicit status for explicitly noting whether this business rule is in effect. This would be handled by an extension if required.
     */
    status:
        | 'registered'
        | 'partial'
        | 'preliminary'
        | 'final'
        | 'amended'
        | 'corrected'
        | 'appended'
        | 'cancelled'
        | 'entered-in-error'
        | 'deprecated'
        | 'unknown';
    _status?: Element | undefined;
    /**
     * For clinical documents, this is usually the patient.
     */
    subject?: Reference[] | undefined;
    /**
     * For many compositions, the title is the same as the text or a display name of Composition.type (e.g. a "consultation" or "progress note"). Note that CDA does not make title mandatory, but there are no known cases where it is useful for title to be omitted, so it is mandatory here. Feedback on this requirement is welcome during the trial use period.
     */
    title: string;
    _title?: Element | undefined;
    /**
     * For Composition type, LOINC is ubiquitous and strongly endorsed by HL7. Most implementation guides will require a specific LOINC code, or use LOINC as an extensible binding.
     */
    type: CodeableConcept;
    /**
     * Can be a urn:uuid: or a urn:oid: but real http: addresses are preferred. Multiple instances may share the same URL if they have a distinct version.
     */
    url?: string | undefined;
    _url?: Element | undefined;
    /**
     * When multiple useContexts are specified, there is no expectation that all or any of the contexts apply.
     */
    useContext?: UsageContext[] | undefined;
    /**
     * While each resource, including the composition itself, has its own version identifier, this is a formal identifier for the logical version of the Composition as a whole. It would remain constant if the resources were moved to a new server, and all got new individual resource versions, for example.
     */
    version?: string | undefined;
    _version?: Element | undefined;
}
/**
 * A set of additional dependencies for this mapping to hold. This mapping is only applicable if the specified property can be resolved, and it has the specified value.
 */
export interface ConceptMapGroupElementTargetDependsOn extends BackboneElement {
    /**
     * A reference to a property that holds a value the map depends on. This value can be supplied to the $translate operation to select the appropriate targets.
     */
    property: string;
    _property?: Element | undefined;
    /**
     * Property value that the map depends on.
     */
    valueCode?: string | undefined;
    _valueCode?: Element | undefined;
    /**
     * Property value that the map depends on.
     */
    valueCoding?: Coding | undefined;
    /**
     * Property value that the map depends on.
     */
    valueString?: string | undefined;
    _valueString?: Element | undefined;
    /**
     * Property value that the map depends on.
     */
    valueInteger?: number | undefined;
    /**
     * Property value that the map depends on.
     */
    valueBoolean?: boolean | undefined;
    _valueBoolean?: Element | undefined;
    /**
     * Property value that the map depends on.
     */
    valueDateTime?: string | undefined;
    _valueDateTime?: Element | undefined;
    /**
     * Property value that the map depends on.
     */
    valueDecimal?: number | undefined;
    /**
     * Property value that the map depends on.
     */
    valueUri?: string | undefined;
    _valueUri?: Element | undefined;
    /**
     * Property value that the map depends on.
     */
    valueId?: string | undefined;
    _valueId?: Element | undefined;
    /**
     * This mapping applies if the property value is a code from this value set.
     */
    valueSet?: string | undefined;
    _valueSet?: Element | undefined;
}
/**
 * Ideally there would only be one map, with an 'equivalent' mapping. But multiple maps are allowed for several narrower (i.e. source-is-broader-than-target) options, or to assert that other concepts are not related.
 */
export interface ConceptMapGroupElementTarget extends BackboneElement {
    /**
     * Identity (code or path) or the element/item that the map refers to.
     */
    code?: string | undefined;
    _code?: Element | undefined;
    /**
     * A description of status/issues in mapping that conveys additional information not represented in  the structured data.
     */
    comment?: string | undefined;
    _comment?: Element | undefined;
    /**
     * A set of additional dependencies for this mapping to hold. This mapping is only applicable if the specified property can be resolved, and it has the specified value.
     */
    dependsOn?: ConceptMapGroupElementTargetDependsOn[] | undefined;
    /**
     * The display is ignored when processing the map.
     */
    display?: string | undefined;
    _display?: Element | undefined;
    /**
     * Product is the output of a ConceptMap that provides additional values relevant to the interpretation of the mapping target.
     */
    product?: ConceptMapGroupElementTargetDependsOn[] | undefined;
    /**
     * This element is labeled as a modifier because it may indicate that there is no mapping.
     */
    relationship:
        | 'related-to'
        | 'equivalent'
        | 'source-is-narrower-than-target'
        | 'source-is-broader-than-target'
        | 'not-related-to';
    _relationship?: Element | undefined;
    /**
     * The set of codes that the map refers to.
     */
    valueSet?: string | undefined;
    _valueSet?: Element | undefined;
}
/**
 * Generally, the ideal is that there would only be one mapping for each concept in the source value set, but a given concept may be mapped multiple times with different comments or dependencies.
 */
export interface ConceptMapGroupElement extends BackboneElement {
    /**
     * Identity (code or path) or the element/item being mapped.
     */
    code?: string | undefined;
    _code?: Element | undefined;
    /**
     * The display is ignored when processing the map.
     */
    display?: string | undefined;
    _display?: Element | undefined;
    /**
     * If noMap = true this indicates that no mapping to a target concept exists for this source concept.
     */
    noMap?: boolean | undefined;
    _noMap?: Element | undefined;
    /**
     * Ideally there would only be one map, with an 'equivalent' mapping. But multiple maps are allowed for several narrower (i.e. source-is-broader-than-target) options, or to assert that other concepts are not related.
     */
    target?: ConceptMapGroupElementTarget[] | undefined;
    /**
     * The set of codes being mapped.
     */
    valueSet?: string | undefined;
    _valueSet?: Element | undefined;
}
/**
 * This only applies if the source code has a system value that matches the system defined for the group.
 */
export interface ConceptMapGroupUnmapped extends BackboneElement {
    /**
     * The fixed code to use when the mode = 'fixed'  - all unmapped codes are mapped to a single fixed code.
     */
    code?: string | undefined;
    _code?: Element | undefined;
    /**
     * The display is ignored when processing the map.
     */
    display?: string | undefined;
    _display?: Element | undefined;
    /**
     * Defines which action to take if there is no match for the source concept in the target system designated for the group. One of 3 actions are possible: use the unmapped source code (this is useful when doing a mapping between versions, and only a few codes have changed), use a fixed code (a default code), or alternatively, a reference to a different concept map can be provided (by canonical URL).
     */
    mode: 'use-source-code' | 'fixed' | 'other-map';
    _mode?: Element | undefined;
    /**
     * The canonical reference to an additional ConceptMap resource instance to use for mapping if this ConceptMap resource contains no matching mapping for the source concept.
     */
    otherMap?: string | undefined;
    _otherMap?: Element | undefined;
    /**
     * This element is labeled as a modifier because it may indicate that there is no mapping.
     */
    relationship?:
        | (
              | 'related-to'
              | 'equivalent'
              | 'source-is-narrower-than-target'
              | 'source-is-broader-than-target'
              | 'not-related-to'
          )
        | undefined;
    _relationship?: Element | undefined;
    /**
     * The set of fixed codes to use when the mode = 'fixed'  - all unmapped codes are mapped to each of the fixed codes.
     */
    valueSet?: string | undefined;
    _valueSet?: Element | undefined;
}
/**
 * A group of mappings that all have the same source and target system.
 */
export interface ConceptMapGroup extends BackboneElement {
    /**
     * Generally, the ideal is that there would only be one mapping for each concept in the source value set, but a given concept may be mapped multiple times with different comments or dependencies.
     */
    element: ConceptMapGroupElement[];
    /**
     * This is not needed if the source value set is specified and it contains concepts from only a single system.
     */
    source?: string | undefined;
    _source?: Element | undefined;
    /**
     * This is not needed if the target value set is specified and it contains concepts from only a single system. The group target may also be omitted if all of the target element relationship values are 'not-related-to'.
     */
    target?: string | undefined;
    _target?: Element | undefined;
    /**
     * This only applies if the source code has a system value that matches the system defined for the group.
     */
    unmapped?: ConceptMapGroupUnmapped | undefined;
}
/**
 * A statement of relationships from one set of concepts to one or more other concepts - either concepts in code systems, or data element/data element concepts, or classes in class models.
 */
export interface ConceptMap extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'ConceptMap';
    /**
     * The 'date' element may be more recent than the approval date because of minor changes or editorial corrections.
     */
    approvalDate?: string | undefined;
    _approvalDate?: Element | undefined;
    /**
     * An individiual or organization primarily involved in the creation and maintenance of the ConceptMap.
     */
    author?: ContactDetail[] | undefined;
    /**
     * May be a web site, an email address, a telephone number, etc.
     */
    contact?: ContactDetail[] | undefined;
    /**
     * Frequently the copyright differs between the concept map and codes that are included. The copyright statement should clearly differentiate between these when required.
     */
    copyright?: string | undefined;
    _copyright?: Element | undefined;
    /**
     * Note that this is not the same as the resource last-modified-date, since the resource may be a secondary representation of the concept map. Additional specific dates may be added as extensions or be found by consulting Provenances associated with past versions of the resource.
     */
    date?: string | undefined;
    _date?: Element | undefined;
    /**
     * The description is not intended to describe the semantics of the concept map. The description should capture its intended use, which is needed for ensuring integrity for its use in models across future changes.
     */
    description?: string | undefined;
    _description?: Element | undefined;
    /**
     * An individual or organization primarily responsible for internal coherence of the ConceptMap.
     */
    editor?: ContactDetail[] | undefined;
    /**
     * The effective period for a ConceptMap  determines when the content is applicable for usage and is independent of publication and review dates. For example, a measure intended to be used for the year 2016 might be published in 2015.
     */
    effectivePeriod?: Period | undefined;
    /**
     * An individual or organization responsible for officially endorsing the ConceptMap for use in some setting.
     */
    endorser?: ContactDetail[] | undefined;
    /**
     * Allows filtering of concept maps that are appropriate for use versus not.
     */
    experimental?: boolean | undefined;
    _experimental?: Element | undefined;
    /**
     * A group of mappings that all have the same source and target system.
     */
    group?: ConceptMapGroup[] | undefined;
    /**
     * Typically, this is used for identifiers that can go in an HL7 V3 II (instance identifier) data type, and can then identify this concept map outside of FHIR, where it is not possible to use the logical URI.
     */
    identifier?: Identifier[] | undefined;
    /**
     * It may be possible for the concept map to be used in jurisdictions other than those for which it was originally designed or intended.
     */
    jurisdiction?: CodeableConcept[] | undefined;
    /**
     * If specified, this date follows the original approval date.
     */
    lastReviewDate?: string | undefined;
    _lastReviewDate?: Element | undefined;
    /**
     * The name is not expected to be globally unique. The name should be a simple alphanumeric type name to ensure that it is machine-processing friendly.
     */
    name?: string | undefined;
    _name?: Element | undefined;
    /**
     * Usually an organization but may be an individual. The publisher (or steward) of the concept map is the organization or individual primarily responsible for the maintenance and upkeep of the concept map. This is not necessarily the same individual or organization that developed and initially authored the content. The publisher is the primary point of contact for questions or issues with the concept map. This item SHOULD be populated unless the information is available from context.
     */
    publisher?: string | undefined;
    _publisher?: Element | undefined;
    /**
     * This element does not describe the usage of the concept map. Instead, it provides traceability of ''why'' the resource is either needed or ''why'' it is defined as it is.  This may be used to point to source materials or specifications that drove the structure of this concept map.
     */
    purpose?: string | undefined;
    _purpose?: Element | undefined;
    /**
     * Each related artifact is either an attachment, or a reference to another resource, but not both.
     */
    relatedArtifact?: RelatedArtifact[] | undefined;
    /**
     * An individual or organization primarily responsible for review of some aspect of the ConceptMap.
     */
    reviewer?: ContactDetail[] | undefined;
    /**
     * Should be a version specific reference. URIs SHOULD be absolute. If there is no sourceScope or targetScope value set, there is no specified context for the map (not recommended).  The sourceScope value set may select codes from either an explicit (standard or local) or implicit code system.
     */
    sourceScopeUri?: string | undefined;
    _sourceScopeUri?: Element | undefined;
    /**
     * Should be a version specific reference. URIs SHOULD be absolute. If there is no sourceScope or targetScope value set, there is no specified context for the map (not recommended).  The sourceScope value set may select codes from either an explicit (standard or local) or implicit code system.
     */
    sourceScopeCanonical?: string | undefined;
    _sourceScopeCanonical?: Element | undefined;
    /**
     * Allows filtering of concept maps that are appropriate for use versus not.
     */
    status: 'draft' | 'active' | 'retired' | 'unknown';
    _status?: Element | undefined;
    /**
     * Should be a version specific reference. URIs SHOULD be absolute. If there is no sourceScope or targetScope value set, there is no specified context for the map (not recommended).  The targetScope value set may select codes from either an explicit (standard or local) or implicit code system.
     */
    targetScopeUri?: string | undefined;
    _targetScopeUri?: Element | undefined;
    /**
     * Should be a version specific reference. URIs SHOULD be absolute. If there is no sourceScope or targetScope value set, there is no specified context for the map (not recommended).  The targetScope value set may select codes from either an explicit (standard or local) or implicit code system.
     */
    targetScopeCanonical?: string | undefined;
    _targetScopeCanonical?: Element | undefined;
    /**
     * This name does not need to be machine-processing friendly and may contain punctuation, white-space, etc.
     */
    title?: string | undefined;
    _title?: Element | undefined;
    /**
     * This element provides topical categorization and keywords for the ConceptMap, as opposed to the more structured context-of-use information provided in the useContext element
     */
    topic?: CodeableConcept[] | undefined;
    /**
     * Can be a urn:uuid: or a urn:oid: but real http: addresses are preferred.  Multiple instances may share the same URL if they have a distinct version.
     * The determination of when to create a new version of a resource (same url, new version) vs. defining a new artifact is up to the author.  Considerations for making this decision are found in [Technical and Business Versions](resource.html#versions).
     * In some cases, the resource can no longer be found at the stated url, but the url itself cannot change. Implementations can use the [meta.source](resource.html#meta) element to indicate where the current master source of the resource can be found.
     */
    url?: string | undefined;
    _url?: Element | undefined;
    /**
     * When multiple useContexts are specified, there is no expectation that all or any of the contexts apply.
     */
    useContext?: UsageContext[] | undefined;
    /**
     * There may be different concept map instances that have the same identifier but different versions.  The version can be appended to the url in a reference to allow a reference to a particular business version of the concept map with the format [url]|[version]. The version SHOULD NOT contain a '#' - see [Business Version](resource.html#bv-format).
     */
    version?: string | undefined;
    _version?: Element | undefined;
}
/**
 * Indicates who or what participated in the activities related to the condition and how they were involved.
 */
export interface ConditionParticipant extends BackboneElement {
    /**
     * Indicates who or what participated in the activities related to the condition.
     */
    actor: Reference;
    /**
     * Distinguishes the type of involvement of the actor in the activities related to the condition.
     */
    function?: CodeableConcept | undefined;
}
/**
 * A simple summary of the stage such as "Stage 3" or "Early Onset". The determination of the stage is disease-specific, such as cancer, retinopathy of prematurity, kidney diseases, Alzheimer's, or Parkinson disease.
 */
export interface ConditionStage extends BackboneElement {
    /**
     * Reference to a formal record of the evidence on which the staging assessment is based.
     */
    assessment?: Reference[] | undefined;
    /**
     * A simple summary of the stage such as "Stage 3" or "Early Onset". The determination of the stage is disease-specific, such as cancer, retinopathy of prematurity, kidney diseases, Alzheimer's, or Parkinson disease.
     */
    summary?: CodeableConcept | undefined;
    /**
     * The kind of staging, such as pathological or clinical staging.
     */
    type?: CodeableConcept | undefined;
}
/**
 * A clinical condition, problem, diagnosis, or other event, situation, issue, or clinical concept that has risen to a level of concern.
 */
export interface Condition extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'Condition';
    /**
     * There is no explicit distinction between resolution and remission because in many cases the distinction is not clear. Age is generally used when the patient reports an age at which the Condition abated.  If there is no abatement element, it is unknown whether the condition has resolved or entered remission; applications and users should generally assume that the condition is still valid.  When abatementString exists, it implies the condition is abated.
     * Because a Condition.code can represent multiple levels of granularity and can be modified over time, the onset and abatement dates can have ambiguity whether those dates apply to the current Condition.code or an earlier representation of that Condition.code.   For example, if the Condition.code was initially documented as severe asthma, then it is ambiguous whether the onset and abatement dates apply to asthma (overall in that subject's lifetime) or when asthma transitioned to become severe.
     */
    abatementDateTime?: string | undefined;
    _abatementDateTime?: Element | undefined;
    /**
     * There is no explicit distinction between resolution and remission because in many cases the distinction is not clear. Age is generally used when the patient reports an age at which the Condition abated.  If there is no abatement element, it is unknown whether the condition has resolved or entered remission; applications and users should generally assume that the condition is still valid.  When abatementString exists, it implies the condition is abated.
     * Because a Condition.code can represent multiple levels of granularity and can be modified over time, the onset and abatement dates can have ambiguity whether those dates apply to the current Condition.code or an earlier representation of that Condition.code.   For example, if the Condition.code was initially documented as severe asthma, then it is ambiguous whether the onset and abatement dates apply to asthma (overall in that subject's lifetime) or when asthma transitioned to become severe.
     */
    abatementAge?: Age | undefined;
    /**
     * There is no explicit distinction between resolution and remission because in many cases the distinction is not clear. Age is generally used when the patient reports an age at which the Condition abated.  If there is no abatement element, it is unknown whether the condition has resolved or entered remission; applications and users should generally assume that the condition is still valid.  When abatementString exists, it implies the condition is abated.
     * Because a Condition.code can represent multiple levels of granularity and can be modified over time, the onset and abatement dates can have ambiguity whether those dates apply to the current Condition.code or an earlier representation of that Condition.code.   For example, if the Condition.code was initially documented as severe asthma, then it is ambiguous whether the onset and abatement dates apply to asthma (overall in that subject's lifetime) or when asthma transitioned to become severe.
     */
    abatementPeriod?: Period | undefined;
    /**
     * There is no explicit distinction between resolution and remission because in many cases the distinction is not clear. Age is generally used when the patient reports an age at which the Condition abated.  If there is no abatement element, it is unknown whether the condition has resolved or entered remission; applications and users should generally assume that the condition is still valid.  When abatementString exists, it implies the condition is abated.
     * Because a Condition.code can represent multiple levels of granularity and can be modified over time, the onset and abatement dates can have ambiguity whether those dates apply to the current Condition.code or an earlier representation of that Condition.code.   For example, if the Condition.code was initially documented as severe asthma, then it is ambiguous whether the onset and abatement dates apply to asthma (overall in that subject's lifetime) or when asthma transitioned to become severe.
     */
    abatementRange?: Range | undefined;
    /**
     * There is no explicit distinction between resolution and remission because in many cases the distinction is not clear. Age is generally used when the patient reports an age at which the Condition abated.  If there is no abatement element, it is unknown whether the condition has resolved or entered remission; applications and users should generally assume that the condition is still valid.  When abatementString exists, it implies the condition is abated.
     * Because a Condition.code can represent multiple levels of granularity and can be modified over time, the onset and abatement dates can have ambiguity whether those dates apply to the current Condition.code or an earlier representation of that Condition.code.   For example, if the Condition.code was initially documented as severe asthma, then it is ambiguous whether the onset and abatement dates apply to asthma (overall in that subject's lifetime) or when asthma transitioned to become severe.
     */
    abatementString?: string | undefined;
    _abatementString?: Element | undefined;
    /**
     * Only used if not implicit in code found in Condition.code. If the use case requires attributes from the BodyStructure resource (e.g. to identify and track separately) then use the standard extension [bodyStructure](extension-bodysite.html).  May be a summary code, or a reference to a very precise definition of the location, or both.
     */
    bodySite?: CodeableConcept[] | undefined;
    /**
     * The categorization is often highly contextual and may appear poorly differentiated or not very useful in other contexts.
     */
    category?: CodeableConcept[] | undefined;
    /**
     * The data type is CodeableConcept because clinicalStatus has some clinical judgment involved, such that there might need to be more specificity than the required FHIR value set allows. For example, a SNOMED coding might allow for additional specificity.  clinicalStatus is required since it is a modifier element.  For conditions that are problems list items, the clinicalStatus should not be unknown.  For conditions that are not problem list items, the clinicalStatus may be unknown.  For example, conditions derived from a claim are point in time, so those conditions may have a clinicalStatus of unknown
     */
    clinicalStatus: CodeableConcept;
    /**
     * Identification of the condition, problem or diagnosis.
     */
    code?: CodeableConcept | undefined;
    /**
     * This will typically be the encounter the event occurred within, but some activities may be initiated prior to or after the official completion of an encounter but still be tied to the context of the encounter. This record indicates the encounter this particular record is associated with.  In the case of a "new" diagnosis reflecting ongoing/revised information about the condition, this might be distinct from the first encounter in which the underlying condition was first "known".
     */
    encounter?: Reference | undefined;
    /**
     * If the condition was confirmed, but subsequently refuted, then the evidence can be cumulative including all evidence over time.  The evidence may be a simple list of coded symptoms/manifestations, or references to observations or formal assessments, or both.  For example, if the Condition.code is pneumonia, then there could be an evidence list where Condition.evidence.concept = fever (CodeableConcept), Condition.evidence.concept = cough (CodeableConcept), and Condition.evidence.reference = bronchitis (reference to Condition).
     */
    evidence?: CodeableReference[] | undefined;
    /**
     * This is a business identifier, not a resource identifier (see [discussion](resource.html#identifiers)).  It is best practice for the identifier to only appear on a single resource instance, however business practices may occasionally dictate that multiple resource instances with the same identifier can exist - possibly even with different resource types.  For example, multiple Patient and a Person resource instance might share the same social insurance number.
     */
    identifier?: Identifier[] | undefined;
    /**
     * Additional information about the Condition. This is a general notes/comments entry  for description of the Condition, its diagnosis and prognosis.
     */
    note?: Annotation[] | undefined;
    /**
     * Age is generally used when the patient reports an age at which the Condition began to occur.  Period is generally used to convey an imprecise onset that occurred within the time period.  For example, Period is not intended to convey the transition period before the chronic bronchitis or COPD condition was diagnosed, but Period can be used to convey an imprecise diagnosis date.
     * Because a Condition.code can represent multiple levels of granularity and can be modified over time, the onset and abatement dates can have ambiguity whether those dates apply to the current Condition.code or an earlier representation of that Condition.code.   For example, if the Condition.code was initially documented as severe asthma, then it is ambiguous whether the onset and abatement dates apply to asthma (overall in that subject's lifetime) or when asthma transitioned to become severe.
     */
    onsetDateTime?: string | undefined;
    _onsetDateTime?: Element | undefined;
    /**
     * Age is generally used when the patient reports an age at which the Condition began to occur.  Period is generally used to convey an imprecise onset that occurred within the time period.  For example, Period is not intended to convey the transition period before the chronic bronchitis or COPD condition was diagnosed, but Period can be used to convey an imprecise diagnosis date.
     * Because a Condition.code can represent multiple levels of granularity and can be modified over time, the onset and abatement dates can have ambiguity whether those dates apply to the current Condition.code or an earlier representation of that Condition.code.   For example, if the Condition.code was initially documented as severe asthma, then it is ambiguous whether the onset and abatement dates apply to asthma (overall in that subject's lifetime) or when asthma transitioned to become severe.
     */
    onsetAge?: Age | undefined;
    /**
     * Age is generally used when the patient reports an age at which the Condition began to occur.  Period is generally used to convey an imprecise onset that occurred within the time period.  For example, Period is not intended to convey the transition period before the chronic bronchitis or COPD condition was diagnosed, but Period can be used to convey an imprecise diagnosis date.
     * Because a Condition.code can represent multiple levels of granularity and can be modified over time, the onset and abatement dates can have ambiguity whether those dates apply to the current Condition.code or an earlier representation of that Condition.code.   For example, if the Condition.code was initially documented as severe asthma, then it is ambiguous whether the onset and abatement dates apply to asthma (overall in that subject's lifetime) or when asthma transitioned to become severe.
     */
    onsetPeriod?: Period | undefined;
    /**
     * Age is generally used when the patient reports an age at which the Condition began to occur.  Period is generally used to convey an imprecise onset that occurred within the time period.  For example, Period is not intended to convey the transition period before the chronic bronchitis or COPD condition was diagnosed, but Period can be used to convey an imprecise diagnosis date.
     * Because a Condition.code can represent multiple levels of granularity and can be modified over time, the onset and abatement dates can have ambiguity whether those dates apply to the current Condition.code or an earlier representation of that Condition.code.   For example, if the Condition.code was initially documented as severe asthma, then it is ambiguous whether the onset and abatement dates apply to asthma (overall in that subject's lifetime) or when asthma transitioned to become severe.
     */
    onsetRange?: Range | undefined;
    /**
     * Age is generally used when the patient reports an age at which the Condition began to occur.  Period is generally used to convey an imprecise onset that occurred within the time period.  For example, Period is not intended to convey the transition period before the chronic bronchitis or COPD condition was diagnosed, but Period can be used to convey an imprecise diagnosis date.
     * Because a Condition.code can represent multiple levels of granularity and can be modified over time, the onset and abatement dates can have ambiguity whether those dates apply to the current Condition.code or an earlier representation of that Condition.code.   For example, if the Condition.code was initially documented as severe asthma, then it is ambiguous whether the onset and abatement dates apply to asthma (overall in that subject's lifetime) or when asthma transitioned to become severe.
     */
    onsetString?: string | undefined;
    _onsetString?: Element | undefined;
    /**
     * Indicates who or what participated in the activities related to the condition and how they were involved.
     */
    participant?: ConditionParticipant[] | undefined;
    /**
     * When onset date is unknown, recordedDate can be used to establish if the condition was present on or before a given date.  If the recordedDate is known and provided by a sending system, it is preferred that the receiving system preserve that recordedDate value. If the recordedDate is not provided by the sending system, the receipt timestamp is sometimes used as the recordedDate.
     */
    recordedDate?: string | undefined;
    _recordedDate?: Element | undefined;
    /**
     * Coding of the severity with a terminology is preferred, where possible.
     */
    severity?: CodeableConcept | undefined;
    /**
     * A simple summary of the stage such as "Stage 3" or "Early Onset". The determination of the stage is disease-specific, such as cancer, retinopathy of prematurity, kidney diseases, Alzheimer's, or Parkinson disease.
     */
    stage?: ConditionStage[] | undefined;
    /**
     * Indicates the patient or group who the condition record is associated with.
     */
    subject: Reference;
    /**
     * verificationStatus is not required.  For example, when a patient has abdominal pain in the ED, there is not likely going to be a verification status.
     * The data type is CodeableConcept because verificationStatus has some clinical judgment involved, such that there might need to be more specificity than the required FHIR value set allows. For example, a SNOMED coding might allow for additional specificity.
     */
    verificationStatus?: CodeableConcept | undefined;
}
/**
 * Observations particularly relevant to this condition.
 */
export interface ConditionDefinitionObservation extends BackboneElement {
    /**
     * Category that is relevant.
     */
    category?: CodeableConcept | undefined;
    /**
     * Code for relevant Observation.
     */
    code?: CodeableConcept | undefined;
}
/**
 * Medications particularly relevant for this condition.
 */
export interface ConditionDefinitionMedication extends BackboneElement {
    /**
     * Category that is relevant.
     */
    category?: CodeableConcept | undefined;
    /**
     * Code for relevant Medication.
     */
    code?: CodeableConcept | undefined;
}
/**
 * An observation that suggests that this condition applies.
 */
export interface ConditionDefinitionPrecondition extends BackboneElement {
    /**
     * Code for relevant Observation.
     */
    code: CodeableConcept;
    /**
     * Kind of pre-condition.
     */
    type: 'sensitive' | 'specific';
    _type?: Element | undefined;
    /**
     * Value of Observation.
     */
    valueCodeableConcept?: CodeableConcept | undefined;
    /**
     * Value of Observation.
     */
    valueQuantity?: Quantity | undefined;
}
/**
 * Questionnaire for this condition.
 */
export interface ConditionDefinitionQuestionnaire extends BackboneElement {
    /**
     * Use of the questionnaire.
     */
    purpose: 'preadmit' | 'diff-diagnosis' | 'outcome';
    _purpose?: Element | undefined;
    /**
     * Specific Questionnaire.
     */
    reference: Reference;
}
/**
 * Plan that is appropriate.
 */
export interface ConditionDefinitionPlan extends BackboneElement {
    /**
     * The actual plan.
     */
    reference: Reference;
    /**
     * Use for the plan.
     */
    role?: CodeableConcept | undefined;
}
/**
 * A definition of a condition and information relevant to managing it.
 */
export interface ConditionDefinition extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'ConditionDefinition';
    /**
     * Only used if not implicit in code found in ConditionDefinition.code. If the use case requires attributes from the BodySite resource (e.g. to identify and track separately) then use the standard extension [bodySite](extension-bodysite.html).  May be a summary code, or a reference to a very precise definition of the location, or both.
     */
    bodySite?: CodeableConcept | undefined;
    /**
     * Identification of the condition, problem or diagnosis.
     */
    code: CodeableConcept;
    /**
     * May be a web site, an email address, a telephone number, etc.
     */
    contact?: ContactDetail[] | undefined;
    /**
     * Note that this is not the same as the resource last-modified-date, since the resource may be a secondary representation of the condition definition. Additional specific dates may be added as extensions or be found by consulting Provenances associated with past versions of the resource.
     */
    date?: string | undefined;
    _date?: Element | undefined;
    /**
     * Formal definitions of the condition. These may be references to ontologies, published clinical protocols or research papers.
     */
    definition?: string[] | undefined;
    _definition?: Element[] | undefined;
    /**
     * This description can be used to capture details such as comments about misuse, instructions for clinical use and interpretation, literature references, examples from the paper world, etc. It is not a rendering of the condition definition as conveyed in the 'text' field of the resource itself. This item SHOULD be populated unless the information is available from context (e.g. the language of the condition definition is presumed to be the predominant language in the place the condition definition was created).
     */
    description?: string | undefined;
    _description?: Element | undefined;
    /**
     * Allows filtering of condition definitions that are appropriate for use versus not.
     */
    experimental?: boolean | undefined;
    _experimental?: Element | undefined;
    /**
     * Whether bodySite is appropriate to collect for this condition.
     */
    hasBodySite?: boolean | undefined;
    _hasBodySite?: Element | undefined;
    /**
     * Whether Severity is appropriate to collect for this condition.
     */
    hasSeverity?: boolean | undefined;
    _hasSeverity?: Element | undefined;
    /**
     * Whether stage is appropriate to collect for this condition.
     */
    hasStage?: boolean | undefined;
    _hasStage?: Element | undefined;
    /**
     * Typically, this is used for identifiers that can go in an HL7 V3 II (instance identifier) data type, and can then identify this condition definition outside of FHIR, where it is not possible to use the logical URI.
     */
    identifier?: Identifier[] | undefined;
    /**
     * It may be possible for the condition definition to be used in jurisdictions other than those for which it was originally designed or intended.
     */
    jurisdiction?: CodeableConcept[] | undefined;
    /**
     * Medications particularly relevant for this condition.
     */
    medication?: ConditionDefinitionMedication[] | undefined;
    /**
     * The name is not expected to be globally unique. The name should be a simple alphanumeric type name to ensure that it is machine-processing friendly.
     */
    name?: string | undefined;
    _name?: Element | undefined;
    /**
     * Observations particularly relevant to this condition.
     */
    observation?: ConditionDefinitionObservation[] | undefined;
    /**
     * Plan that is appropriate.
     */
    plan?: ConditionDefinitionPlan[] | undefined;
    /**
     * An observation that suggests that this condition applies.
     */
    precondition?: ConditionDefinitionPrecondition[] | undefined;
    /**
     * Usually an organization but may be an individual. The publisher (or steward) of the condition definition is the organization or individual primarily responsible for the maintenance and upkeep of the condition definition. This is not necessarily the same individual or organization that developed and initially authored the content. The publisher is the primary point of contact for questions or issues with the condition definition. This item SHOULD be populated unless the information is available from context.
     */
    publisher?: string | undefined;
    _publisher?: Element | undefined;
    /**
     * Questionnaire for this condition.
     */
    questionnaire?: ConditionDefinitionQuestionnaire[] | undefined;
    /**
     * Coding of the severity with a terminology is preferred, where possible.
     */
    severity?: CodeableConcept | undefined;
    /**
     * Clinical stage or grade of a condition. May include formal severity assessments.
     */
    stage?: CodeableConcept | undefined;
    /**
     * Allows filtering of condition definitions that are appropriate for use versus not.
     */
    status: 'draft' | 'active' | 'retired' | 'unknown';
    _status?: Element | undefined;
    /**
     * An explanatory or alternate title for the event definition giving additional information about its content.
     */
    subtitle?: string | undefined;
    _subtitle?: Element | undefined;
    /**
     * Appropriate team for this condition.
     */
    team?: Reference[] | undefined;
    /**
     * This name does not need to be machine-processing friendly and may contain punctuation, white-space, etc.
     */
    title?: string | undefined;
    _title?: Element | undefined;
    /**
     * Can be a urn:uuid: or a urn:oid: but real http: addresses are preferred.  Multiple instances may share the same URL if they have a distinct version.
     * The determination of when to create a new version of a resource (same url, new version) vs. defining a new artifact is up to the author.  Considerations for making this decision are found in [Technical and Business Versions](resource.html#versions).
     * In some cases, the resource can no longer be found at the stated url, but the url itself cannot change. Implementations can use the [meta.source](resource.html#meta) element to indicate where the current master source of the resource can be found.
     */
    url?: string | undefined;
    _url?: Element | undefined;
    /**
     * When multiple useContexts are specified, there is no expectation that all or any of the contexts apply.
     */
    useContext?: UsageContext[] | undefined;
    /**
     * There may be different condition definition instances that have the same identifier but different versions.  The version can be appended to the url in a reference to allow a reference to a particular business version of the condition definition with the format [url]|[version]. The version SHOULD NOT contain a '#' - see [Business Version](resource.html#bv-format).
     */
    version?: string | undefined;
    _version?: Element | undefined;
}
/**
 * A Reference or URL used to uniquely identify the policy the organization will enforce for this Consent. This Reference or URL should be specific to the version of the policy and should be dereferencable to a computable policy of some form.
 */
export interface ConsentPolicyBasis extends BackboneElement {
    /**
     * While any resource may be used, Consent, PlanDefinition and Contract would be most frequent
     */
    reference?: Reference | undefined;
    /**
     * A URL that links to a computable version of the policy the organization will enforce for this Consent.
     */
    url?: string | undefined;
    _url?: Element | undefined;
}
/**
 * Whether a treatment instruction (e.g. artificial respiration: yes or no) was verified with the patient, his/her family or another authorized person.
 */
export interface ConsentVerification extends BackboneElement {
    /**
     * Allows for history of verification/validation.
     */
    verificationDate?: string[] | undefined;
    _verificationDate?: Element[] | undefined;
    /**
     * This allows the verification element to hold multiple use cases including RelatedPerson verification of the Grantee decision and periodic re-validation of the consent.
     */
    verificationType?: CodeableConcept | undefined;
    /**
     * Has the instruction been verified.
     */
    verified: boolean;
    _verified?: Element | undefined;
    /**
     * The person who conducted the verification/validation of the Grantor decision.
     */
    verifiedBy?: Reference | undefined;
    /**
     * Who verified the instruction (Patient, Relative or other Authorized Person).
     */
    verifiedWith?: Reference | undefined;
}
/**
 * Who or what is controlled by this rule. Use group to identify a set of actors by some property they share (e.g. 'admitting officers').
 */
export interface ConsentProvisionActor extends BackboneElement {
    /**
     * The resource that identifies the actor. To identify actors by type, use group to identify a set of actors by some property they share (e.g. 'admitting officers').
     */
    reference?: Reference | undefined;
    /**
     * How the individual is involved in the resources content that is described in the exception.
     */
    role?: CodeableConcept | undefined;
}
/**
 * The resources controlled by this rule if specific resources are referenced.
 */
export interface ConsentProvisionData extends BackboneElement {
    /**
     * How the resource reference is interpreted when testing consent restrictions.
     */
    meaning: 'instance' | 'related' | 'dependents' | 'authoredby';
    _meaning?: Element | undefined;
    /**
     * A reference to a specific resource that defines which resources are covered by this consent.
     */
    reference: Reference;
}
/**
 * An exception to the base policy of this consent. An exception can be an addition or removal of access permissions.
 */
export interface ConsentProvision extends BackboneElement {
    /**
     * Note that this is the direct action (not the grounds for the action covered in the purpose element). At present, the only action in the understood and tested scope of this resource is 'read'.
     */
    action?: CodeableConcept[] | undefined;
    /**
     * Who or what is controlled by this rule. Use group to identify a set of actors by some property they share (e.g. 'admitting officers').
     */
    actor?: ConsentProvisionActor[] | undefined;
    /**
     * Multiple types are or'ed together. The intention of the contentType element is that the codes refer to profiles or document types defined in a standard or an implementation guide somewhere.
     */
    class?: Coding[] | undefined;
    /**
     * Typical use of this is a Document code with class = CDA.
     */
    code?: CodeableConcept[] | undefined;
    /**
     * The resources controlled by this rule if specific resources are referenced.
     */
    data?: ConsentProvisionData[] | undefined;
    /**
     * This has a different sense to the Consent.period - that is when the consent agreement holds. This is the time period of the data that is controlled by the agreement.
     */
    dataPeriod?: Period | undefined;
    /**
     * Constraining the expression type for a specific implementation via profile is recommended
     */
    expression?: Expression | undefined;
    /**
     * This is the bound effective time of the consent and should be in the base provision in the Consent resource.
     */
    period?: Period | undefined;
    /**
     * Rules which provide exceptions to the base rule or subrules.
     */
    provision?: ConsentProvision[] | undefined;
    /**
     * When the purpose of use tag is on the data, access request purpose of use shall not conflict.
     */
    purpose?: Coding[] | undefined;
    /**
     * If the consent specifies a security label of "R" then it applies to all resources that are labeled "R" or lower. E.g. for Confidentiality, it's a high water mark. For other kinds of security labels, subsumption logic applies. When the purpose of use tag is on the data, access request purpose of use shall not conflict.
     */
    securityLabel?: Coding[] | undefined;
    /**
     * Action  to take - permit or deny - when the rule conditions are met.  Not permitted in root rule, required in all nested rules.
     */
    type?: ('deny' | 'permit') | undefined;
    _type?: Element | undefined;
}
/**
 * A record of a healthcare consumer’s  choices  or choices made on their behalf by a third party, which permits or denies identified recipient(s) or recipient role(s) to perform one or more actions within a given policy context, for specific purposes and periods of time.
 */
export interface Consent extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'Consent';
    /**
     * A classification of the type of consents found in the statement. This element supports indexing and retrieval of consent statements.
     */
    category?: CodeableConcept[] | undefined;
    /**
     * The actor that controls/enforces the access according to the consent.
     */
    controller?: Reference[] | undefined;
    /**
     * Date and time the consent instance was agreed to.
     */
    dateTime?: string | undefined;
    _dateTime?: Element | undefined;
    /**
     * In a fully computable consent, both grantee and grantor  will be listed as actors within the consent. The Grantee and Grantor elements are for ease of search only.
     */
    grantee?: Reference[] | undefined;
    /**
     * The entity responsible for granting the rights listed in a Consent Directive.
     */
    grantor?: Reference[] | undefined;
    /**
     * This identifier identifies this copy of the consent. Where this identifier is also used elsewhere as the identifier for a consent record (e.g. a CDA consent document) then the consent details are expected to be the same.
     */
    identifier?: Identifier[] | undefined;
    /**
     * The actor that manages the consent through its lifecycle.
     */
    manager?: Reference[] | undefined;
    /**
     * A Reference or URL used to uniquely identify the policy the organization will enforce for this Consent. This Reference or URL should be specific to the version of the policy and should be dereferencable to a computable policy of some form.
     */
    policyBasis?: ConsentPolicyBasis | undefined;
    /**
     * A Reference to the human readable policy explaining the basis for the Consent.
     */
    policyText?: Reference[] | undefined;
    /**
     * An exception to the base policy of this consent. An exception can be an addition or removal of access permissions.
     */
    provision?: ConsentProvision | undefined;
    /**
     * A set of codes that indicate the regulatory basis (if any) that this consent supports.
     */
    regulatoryBasis?: CodeableConcept[] | undefined;
    /**
     * The source can be contained inline (Attachment), referenced directly (Consent), referenced in a consent repository (DocumentReference), or simply by an identifier (Identifier), e.g. a CDA document id.
     */
    sourceAttachment?: Attachment[] | undefined;
    /**
     * The source can be contained inline (Attachment), referenced directly (Consent), referenced in a consent repository (DocumentReference), or simply by an identifier (Identifier), e.g. a CDA document id.
     */
    sourceReference?: Reference[] | undefined;
    /**
     * This element is labeled as a modifier because the status contains the codes rejected and entered-in-error that mark the Consent as not currently valid.
     */
    status: 'draft' | 'active' | 'inactive' | 'not-done' | 'entered-in-error' | 'unknown';
    _status?: Element | undefined;
    /**
     * The patient/healthcare practitioner or group of persons to whom this consent applies.
     */
    subject?: Reference | undefined;
    /**
     * Whether a treatment instruction (e.g. artificial respiration: yes or no) was verified with the patient, his/her family or another authorized person.
     */
    verification?: ConsentVerification[] | undefined;
}
/**
 * Precusory content developed with a focus and intent of supporting the formation a Contract instance, which may be associated with and transformable into a Contract.
 */
export interface ContractContentDefinition extends BackboneElement {
    /**
     * A copyright statement relating to Contract precursor content. Copyright statements are generally legal restrictions on the use and publishing of the Contract precursor content.
     */
    copyright?: string | undefined;
    _copyright?: Element | undefined;
    /**
     * The date (and optionally time) when the contract was published. The date must change when the business version changes and it must change if the status code changes. In addition, it should change when the substantive content of the contract changes.
     */
    publicationDate?: string | undefined;
    _publicationDate?: Element | undefined;
    /**
     * amended | appended | cancelled | disputed | entered-in-error | executable +.
     */
    publicationStatus:
        | 'amended'
        | 'appended'
        | 'cancelled'
        | 'disputed'
        | 'entered-in-error'
        | 'executable'
        | 'executed'
        | 'negotiable'
        | 'offered'
        | 'policy'
        | 'rejected'
        | 'renewed'
        | 'revoked'
        | 'resolved'
        | 'terminated';
    _publicationStatus?: Element | undefined;
    /**
     * The  individual or organization that published the Contract precursor content.
     */
    publisher?: Reference | undefined;
    /**
     * Detailed Precusory content type.
     */
    subType?: CodeableConcept | undefined;
    /**
     * Precusory content structure and use, i.e., a boilerplate, template, application for a contract such as an insurance policy or benefits under a program, e.g., workers compensation.
     */
    type: CodeableConcept;
}
/**
 * Security labels that protect the handling of information about the term and its elements, which may be specifically identified..
 */
export interface ContractTermSecurityLabel extends BackboneElement {
    /**
     * Security label privacy tag that species the applicable privacy and security policies governing this term and/or term elements.
     */
    category?: Coding[] | undefined;
    /**
     * Security label privacy tag that species the level of confidentiality protection required for this term and/or term elements.
     */
    classification: Coding;
    /**
     * Security label privacy tag that species the manner in which term and/or term elements are to be protected.
     */
    control?: Coding[] | undefined;
    /**
     * Number used to link this term or term element to the applicable Security Label.
     */
    number?: number[] | undefined;
}
/**
 * Offer Recipient.
 */
export interface ContractTermOfferParty extends BackboneElement {
    /**
     * Participant in the offer.
     */
    reference: Reference[];
    /**
     * How the party participates in the offer.
     */
    role: CodeableConcept;
}
/**
 * Response to offer text.
 */
export interface ContractTermOfferAnswer extends BackboneElement {
    /**
     * Response to an offer clause or question text,  which enables selection of values to be agreed to, e.g., the period of participation, the date of occupancy of a rental, warrently duration, or whether biospecimen may be used for further research.
     */
    valueBoolean?: boolean | undefined;
    _valueBoolean?: Element | undefined;
    /**
     * Response to an offer clause or question text,  which enables selection of values to be agreed to, e.g., the period of participation, the date of occupancy of a rental, warrently duration, or whether biospecimen may be used for further research.
     */
    valueDecimal?: number | undefined;
    /**
     * Response to an offer clause or question text,  which enables selection of values to be agreed to, e.g., the period of participation, the date of occupancy of a rental, warrently duration, or whether biospecimen may be used for further research.
     */
    valueInteger?: number | undefined;
    /**
     * Response to an offer clause or question text,  which enables selection of values to be agreed to, e.g., the period of participation, the date of occupancy of a rental, warrently duration, or whether biospecimen may be used for further research.
     */
    valueDate?: string | undefined;
    _valueDate?: Element | undefined;
    /**
     * Response to an offer clause or question text,  which enables selection of values to be agreed to, e.g., the period of participation, the date of occupancy of a rental, warrently duration, or whether biospecimen may be used for further research.
     */
    valueDateTime?: string | undefined;
    _valueDateTime?: Element | undefined;
    /**
     * Response to an offer clause or question text,  which enables selection of values to be agreed to, e.g., the period of participation, the date of occupancy of a rental, warrently duration, or whether biospecimen may be used for further research.
     */
    valueTime?: string | undefined;
    _valueTime?: Element | undefined;
    /**
     * Response to an offer clause or question text,  which enables selection of values to be agreed to, e.g., the period of participation, the date of occupancy of a rental, warrently duration, or whether biospecimen may be used for further research.
     */
    valueString?: string | undefined;
    _valueString?: Element | undefined;
    /**
     * Response to an offer clause or question text,  which enables selection of values to be agreed to, e.g., the period of participation, the date of occupancy of a rental, warrently duration, or whether biospecimen may be used for further research.
     */
    valueUri?: string | undefined;
    _valueUri?: Element | undefined;
    /**
     * Response to an offer clause or question text,  which enables selection of values to be agreed to, e.g., the period of participation, the date of occupancy of a rental, warrently duration, or whether biospecimen may be used for further research.
     */
    valueAttachment?: Attachment | undefined;
    /**
     * Response to an offer clause or question text,  which enables selection of values to be agreed to, e.g., the period of participation, the date of occupancy of a rental, warrently duration, or whether biospecimen may be used for further research.
     */
    valueCoding?: Coding | undefined;
    /**
     * Response to an offer clause or question text,  which enables selection of values to be agreed to, e.g., the period of participation, the date of occupancy of a rental, warrently duration, or whether biospecimen may be used for further research.
     */
    valueQuantity?: Quantity | undefined;
    /**
     * Response to an offer clause or question text,  which enables selection of values to be agreed to, e.g., the period of participation, the date of occupancy of a rental, warrently duration, or whether biospecimen may be used for further research.
     */
    valueReference?: Reference | undefined;
}
/**
 * The matter of concern in the context of this provision of the agrement.
 */
export interface ContractTermOffer extends BackboneElement {
    /**
     * Response to offer text.
     */
    answer?: ContractTermOfferAnswer[] | undefined;
    /**
     * Type of choice made by accepting party with respect to an offer made by an offeror/ grantee.
     */
    decision?: CodeableConcept | undefined;
    /**
     * How the decision about a Contract was conveyed.
     */
    decisionMode?: CodeableConcept[] | undefined;
    /**
     * Unique identifier for this particular Contract Provision.
     */
    identifier?: Identifier[] | undefined;
    /**
     * The id of the clause or question text of the offer in the referenced questionnaire/response.
     */
    linkId?: string[] | undefined;
    _linkId?: Element[] | undefined;
    /**
     * Offer Recipient.
     */
    party?: ContractTermOfferParty[] | undefined;
    /**
     * Security labels that protects the offer.
     */
    securityLabelNumber?: number[] | undefined;
    /**
     * Human readable form of this Contract Offer.
     */
    text?: string | undefined;
    _text?: Element | undefined;
    /**
     * The Contract.topic may be an application for or offer of a policy or service (e.g., uri to a consent directive form or a health insurance policy), which becomes the Contract once accepted by both the grantor and grantee.
     * The Contract Resource may function simply as the computable representation of the executed contract, which may be the attached to the Contract Resource as the “binding” or as the “friendly” electronic form.  For example, a Contract Resource may be automatically populated with the values expressed in a related QuestionnaireResponse.
     * However, the Contract Resource may be considered the legally binding contract if it is the only “executed” form of this contract, and includes the signatures as *The Contract Resource may function as the computable representation of an application or offer in a pre-executed Contract if the grantor has not entered any values.  In this case, it is populated with values in a “legal” form of the application or offer or by the values in an associated Questionnaire.  If the grantor has filled in the legal form or the associated Questionnaire Response, then these values are used to populate a pre-executed Contract Resource.
     * If the Contract.topic is considered an application or offer, then the policy is often required to be attached as the “legal” basis for the application to ensure “informed consent” to the contract, and that any discrepancy between the application and the policy are interpreted against the policy.  Implementers should check organizational and jurisdictional policies to determine the relationship among multiple representations of a contract pre- and post-execution.
     */
    topic?: Reference | undefined;
    /**
     * Type of Contract Provision such as specific requirements, purposes for actions, obligations, prohibitions, e.g. life time maximum benefit.
     */
    type?: CodeableConcept | undefined;
}
/**
 * Circumstance of the asset.
 */
export interface ContractTermAssetContext extends BackboneElement {
    /**
     * Coded representation of the context generally or of the Referenced entity, such as the asset holder type or location.
     */
    code?: CodeableConcept[] | undefined;
    /**
     * Asset context reference may include the creator, custodian, or owning Person or Organization (e.g., bank, repository),  location held, e.g., building,  jurisdiction.
     */
    reference?: Reference | undefined;
    /**
     * Context description.
     */
    text?: string | undefined;
    _text?: Element | undefined;
}
/**
 * Contract Valued Item List.
 */
export interface ContractTermAssetValuedItem extends BackboneElement {
    /**
     * Indicates the time during which this Contract ValuedItem information is effective.
     */
    effectiveTime?: string | undefined;
    _effectiveTime?: Element | undefined;
    /**
     * Specific type of Contract Valued Item that may be priced.
     */
    entityCodeableConcept?: CodeableConcept | undefined;
    /**
     * Specific type of Contract Valued Item that may be priced.
     */
    entityReference?: Reference | undefined;
    /**
     * A real number that represents a multiplier used in determining the overall value of the Contract Valued Item delivered. The concept of a Factor allows for a discount or surcharge multiplier to be applied to a monetary amount.
     */
    factor?: number | undefined;
    /**
     * Identifies a Contract Valued Item instance.
     */
    identifier?: Identifier | undefined;
    /**
     * Id  of the clause or question text related to the context of this valuedItem in the referenced form or QuestionnaireResponse.
     */
    linkId?: string[] | undefined;
    _linkId?: Element[] | undefined;
    /**
     * Expresses the product of the Contract Valued Item unitQuantity and the unitPriceAmt. For example, the formula: unit Quantity * unit Price (Cost per Point) * factor Number  * points = net Amount. Quantity, factor and points are assumed to be 1 if not supplied.
     */
    net?: Money | undefined;
    /**
     * Terms of valuation.
     */
    payment?: string | undefined;
    _payment?: Element | undefined;
    /**
     * When payment is due.
     */
    paymentDate?: string | undefined;
    _paymentDate?: Element | undefined;
    /**
     * An amount that expresses the weighting (based on difficulty, cost and/or resource intensiveness) associated with the Contract Valued Item delivered. The concept of Points allows for assignment of point values for a Contract Valued Item, such that a monetary amount can be assigned to each point.
     */
    points?: number | undefined;
    /**
     * Specifies the units by which the Contract Valued Item is measured or counted, and quantifies the countable or measurable Contract Valued Item instances.
     */
    quantity?: Quantity | undefined;
    /**
     * Who will receive payment.
     */
    recipient?: Reference | undefined;
    /**
     * Who will make payment.
     */
    responsible?: Reference | undefined;
    /**
     * A set of security labels that define which terms are controlled by this condition.
     */
    securityLabelNumber?: number[] | undefined;
    /**
     * A Contract Valued Item unit valuation measure.
     */
    unitPrice?: Money | undefined;
}
/**
 * Contract Term Asset List.
 */
export interface ContractTermAsset extends BackboneElement {
    /**
     * Response to assets.
     */
    answer?: ContractTermOfferAnswer[] | undefined;
    /**
     * Description of the quality and completeness of the asset that imay be a factor in its valuation.
     */
    condition?: string | undefined;
    _condition?: Element | undefined;
    /**
     * Circumstance of the asset.
     */
    context?: ContractTermAssetContext[] | undefined;
    /**
     * Id [identifier??] of the clause or question text about the asset in the referenced form or QuestionnaireResponse.
     */
    linkId?: string[] | undefined;
    _linkId?: Element[] | undefined;
    /**
     * Asset relevant contractual time period.
     */
    period?: Period[] | undefined;
    /**
     * Type of Asset availability for use or ownership.
     */
    periodType?: CodeableConcept[] | undefined;
    /**
     * Specifies the applicability of the term to an asset resource instance, and instances it refers to orinstances that refer to it, and/or are owned by the offeree.
     */
    relationship?: Coding | undefined;
    /**
     * Differentiates the kind of the asset .
     */
    scope?: CodeableConcept | undefined;
    /**
     * Security labels that protects the asset.
     */
    securityLabelNumber?: number[] | undefined;
    /**
     * May be a subtype or part of an offered asset.
     */
    subtype?: CodeableConcept[] | undefined;
    /**
     * Clause or question text (Prose Object) concerning the asset in a linked form, such as a QuestionnaireResponse used in the formation of the contract.
     */
    text?: string | undefined;
    _text?: Element | undefined;
    /**
     * Target entity type about which the term may be concerned.
     */
    type?: CodeableConcept[] | undefined;
    /**
     * Associated entities.
     */
    typeReference?: Reference[] | undefined;
    /**
     * Time period of asset use.
     */
    usePeriod?: Period[] | undefined;
    /**
     * Contract Valued Item List.
     */
    valuedItem?: ContractTermAssetValuedItem[] | undefined;
}
/**
 * Entity of the action.
 */
export interface ContractTermActionSubject extends BackboneElement {
    /**
     * The entity the action is performed or not performed on or for.
     */
    reference: Reference[];
    /**
     * Role type of agent assigned roles in this Contract.
     */
    role?: CodeableConcept | undefined;
}
/**
 * Several agents may be associated (i.e. has some responsibility for an activity) with an activity and vice-versa.
 * For example, in cases of actions initiated by one user for other users, or in events that involve more than one user, hardware device, software, or system process. However, only one user may be the initiator/requestor for the event.
 */
export interface ContractTermAction extends BackboneElement {
    /**
     * Encounter or Episode with primary association to specified term activity.
     */
    context?: Reference | undefined;
    /**
     * Id [identifier??] of the clause or question text related to the requester of this action in the referenced form or QuestionnaireResponse.
     */
    contextLinkId?: string[] | undefined;
    _contextLinkId?: Element[] | undefined;
    /**
     * True if the term prohibits the  action.
     */
    doNotPerform?: boolean | undefined;
    _doNotPerform?: Element | undefined;
    /**
     * Reason or purpose for the action stipulated by this Contract Provision.
     */
    intent: CodeableConcept;
    /**
     * Id [identifier??] of the clause or question text related to this action in the referenced form or QuestionnaireResponse.
     */
    linkId?: string[] | undefined;
    _linkId?: Element[] | undefined;
    /**
     * Comments made about the term action made by the requester, performer, subject or other participants.
     */
    note?: Annotation[] | undefined;
    /**
     * When action happens.
     */
    occurrenceDateTime?: string | undefined;
    _occurrenceDateTime?: Element | undefined;
    /**
     * When action happens.
     */
    occurrencePeriod?: Period | undefined;
    /**
     * When action happens.
     */
    occurrenceTiming?: Timing | undefined;
    /**
     * Indicates who or what is being asked to perform (or not perform) the ction.
     */
    performer?: Reference | undefined;
    /**
     * Id [identifier??] of the clause or question text related to the reason type or reference of this  action in the referenced form or QuestionnaireResponse.
     */
    performerLinkId?: string[] | undefined;
    _performerLinkId?: Element[] | undefined;
    /**
     * The type of role or competency of an individual desired or required to perform or not perform the action.
     */
    performerRole?: CodeableConcept | undefined;
    /**
     * The type of individual that is desired or required to perform or not perform the action.
     */
    performerType?: CodeableConcept[] | undefined;
    /**
     * Rationale for the action to be performed or not performed. Describes why the action is permitted or prohibited. Either a coded concept, or another resource whose existence justifies permitting or not permitting this action.
     */
    reason?: CodeableReference[] | undefined;
    /**
     * Id [identifier??] of the clause or question text related to the reason type or reference of this  action in the referenced form or QuestionnaireResponse.
     */
    reasonLinkId?: string[] | undefined;
    _reasonLinkId?: Element[] | undefined;
    /**
     * Who or what initiated the action and has responsibility for its activation.
     */
    requester?: Reference[] | undefined;
    /**
     * Id [identifier??] of the clause or question text related to the requester of this action in the referenced form or QuestionnaireResponse.
     */
    requesterLinkId?: string[] | undefined;
    _requesterLinkId?: Element[] | undefined;
    /**
     * Security labels that protects the action.
     */
    securityLabelNumber?: number[] | undefined;
    /**
     * Current state of the term action.
     */
    status: CodeableConcept;
    /**
     * Entity of the action.
     */
    subject?: ContractTermActionSubject[] | undefined;
    /**
     * Activity or service obligation to be done or not done, performed or not performed, effectuated or not by this Contract term.
     */
    type: CodeableConcept;
}
/**
 * One or more Contract Provisions, which may be related and conveyed as a group, and may contain nested groups.
 */
export interface ContractTerm extends BackboneElement {
    /**
     * Several agents may be associated (i.e. has some responsibility for an activity) with an activity and vice-versa.
     * For example, in cases of actions initiated by one user for other users, or in events that involve more than one user, hardware device, software, or system process. However, only one user may be the initiator/requestor for the event.
     */
    action?: ContractTermAction[] | undefined;
    /**
     * Relevant time or time-period when this Contract Provision is applicable.
     */
    applies?: Period | undefined;
    /**
     * Contract Term Asset List.
     */
    asset?: ContractTermAsset[] | undefined;
    /**
     * Nested group of Contract Provisions.
     */
    group?: ContractTerm[] | undefined;
    /**
     * Unique identifier for this particular Contract Provision.
     */
    identifier?: Identifier | undefined;
    /**
     * When this Contract Provision was issued.
     */
    issued?: string | undefined;
    _issued?: Element | undefined;
    /**
     * The matter of concern in the context of this provision of the agrement.
     */
    offer: ContractTermOffer;
    /**
     * Security labels that protect the handling of information about the term and its elements, which may be specifically identified..
     */
    securityLabel?: ContractTermSecurityLabel[] | undefined;
    /**
     * A specialized legal clause or condition based on overarching contract type.
     */
    subType?: CodeableConcept | undefined;
    /**
     * Statement of a provision in a policy or a contract.
     */
    text?: string | undefined;
    _text?: Element | undefined;
    /**
     * The entity that the term applies to.
     */
    topicCodeableConcept?: CodeableConcept | undefined;
    /**
     * The entity that the term applies to.
     */
    topicReference?: Reference | undefined;
    /**
     * A legal clause or condition contained within a contract that requires one or both parties to perform a particular requirement by some specified time or prevents one or both parties from performing a particular requirement by some specified time.
     */
    type?: CodeableConcept | undefined;
}
/**
 * Signers who are principal parties to the contract are bound by the Contract.activity related to the Contract.topic, and the Contract.term(s), which either extend or restrict the overall action on the topic by, for example, stipulating specific policies or obligations constraining actions, action reason, or agents with respect to some or all of the topic.
 * For example, specifying how policies or obligations shall constrain actions and action reasons permitted or denied on all or a subset of the Contract.topic (e.g., all or a portion of property being transferred by the contract), agents (e.g., who can resell, assign interests, or alter the property being transferred by the contract), actions, and action reasons; or with respect to Contract.terms, stipulating, extending, or limiting the Contract.period of applicability or valuation of items under consideration.
 */
export interface ContractSigner extends BackboneElement {
    /**
     * Party which is a signator to this Contract.
     */
    party: Reference;
    /**
     * Legally binding Contract DSIG signature contents in Base64.
     */
    signature: Signature[];
    /**
     * Role of this Contract signer, e.g. notary, grantee.
     */
    type: Coding;
}
/**
 * The "patient friendly language" versionof the Contract in whole or in parts. "Patient friendly language" means the representation of the Contract and Contract Provisions in a manner that is readily accessible and understandable by a layperson in accordance with best practices for communication styles that ensure that those agreeing to or signing the Contract understand the roles, actions, obligations, responsibilities, and implication of the agreement.
 */
export interface ContractFriendly extends BackboneElement {
    /**
     * Human readable rendering of this Contract in a format and representation intended to enhance comprehension and ensure understandability.
     */
    contentAttachment?: Attachment | undefined;
    /**
     * Human readable rendering of this Contract in a format and representation intended to enhance comprehension and ensure understandability.
     */
    contentReference?: Reference | undefined;
}
/**
 * List of Legal expressions or representations of this Contract.
 */
export interface ContractLegal extends BackboneElement {
    /**
     * Contract legal text in human renderable form.
     */
    contentAttachment?: Attachment | undefined;
    /**
     * Contract legal text in human renderable form.
     */
    contentReference?: Reference | undefined;
}
/**
 * List of Computable Policy Rule Language Representations of this Contract.
 */
export interface ContractRule extends BackboneElement {
    /**
     * Computable Contract conveyed using a policy rule language (e.g. XACML, DKAL, SecPal).
     */
    contentAttachment?: Attachment | undefined;
    /**
     * Computable Contract conveyed using a policy rule language (e.g. XACML, DKAL, SecPal).
     */
    contentReference?: Reference | undefined;
}
/**
 * Legally enforceable, formally recorded unilateral or bilateral directive i.e., a policy or agreement.
 */
export interface Contract extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'Contract';
    /**
     * Alternative representation of the title for this Contract definition, derivative, or instance in any legal state., e.g., a domain specific contract number related to legislation.
     */
    alias?: string[] | undefined;
    _alias?: Element[] | undefined;
    /**
     * Relevant time or time-period when this Contract is applicable.
     */
    applies?: Period | undefined;
    /**
     * The individual or organization that authored the Contract definition, derivative, or instance in any legal state.
     */
    author?: Reference | undefined;
    /**
     * A formally or informally recognized grouping of people, principals, organizations, or jurisdictions formed for the purpose of achieving some form of collective action such as the promulgation, administration and enforcement of contracts and policies.
     */
    authority?: Reference[] | undefined;
    /**
     * Precusory content developed with a focus and intent of supporting the formation a Contract instance, which may be associated with and transformable into a Contract.
     */
    contentDefinition?: ContractContentDefinition | undefined;
    /**
     * The minimal content derived from the basal information source at a specific stage in its lifecycle.
     */
    contentDerivative?: CodeableConcept | undefined;
    /**
     * Recognized governance framework or system operating with a circumscribed scope in accordance with specified principles, policies, processes or procedures for managing rights, actions, or behaviors of parties or principals relative to resources.
     */
    domain?: Reference[] | undefined;
    /**
     * Event resulting in discontinuation or termination of this Contract instance by one or more parties to the contract.
     */
    expirationType?: CodeableConcept | undefined;
    /**
     * The "patient friendly language" versionof the Contract in whole or in parts. "Patient friendly language" means the representation of the Contract and Contract Provisions in a manner that is readily accessible and understandable by a layperson in accordance with best practices for communication styles that ensure that those agreeing to or signing the Contract understand the roles, actions, obligations, responsibilities, and implication of the agreement.
     */
    friendly?: ContractFriendly[] | undefined;
    /**
     * Unique identifier for this Contract or a derivative that references a Source Contract.
     */
    identifier?: Identifier[] | undefined;
    /**
     * The URL pointing to a FHIR-defined Contract Definition that is adhered to in whole or part by this Contract.
     */
    instantiatesCanonical?: Reference | undefined;
    /**
     * The URL pointing to an externally maintained definition that is adhered to in whole or in part by this Contract.
     */
    instantiatesUri?: string | undefined;
    _instantiatesUri?: Element | undefined;
    /**
     * When this  Contract was issued.
     */
    issued?: string | undefined;
    _issued?: Element | undefined;
    /**
     * List of Legal expressions or representations of this Contract.
     */
    legal?: ContractLegal[] | undefined;
    /**
     * Legally binding Contract: This is the signed and legally recognized representation of the Contract, which is considered the "source of truth" and which would be the basis for legal action related to enforcement of this Contract.
     */
    legallyBindingAttachment?: Attachment | undefined;
    /**
     * Legally binding Contract: This is the signed and legally recognized representation of the Contract, which is considered the "source of truth" and which would be the basis for legal action related to enforcement of this Contract.
     */
    legallyBindingReference?: Reference | undefined;
    /**
     * Legal states of the formation of a legal instrument, which is a formally executed written document that can be formally attributed to its author, records and formally expresses a legally enforceable act, process, or contractual duty, obligation, or right, and therefore evidences that act, process, or agreement.
     */
    legalState?: CodeableConcept | undefined;
    /**
     * The name is not expected to be globally unique. The name should be a simple alphanumeric type name to ensure that it is machine-processing friendly.
     */
    name?: string | undefined;
    _name?: Element | undefined;
    /**
     * Links to Provenance records for past versions of this Contract definition, derivative, or instance, which identify key state transitions or updates that are likely to be relevant to a user looking at the current version of the Contract.  The Provence.entity indicates the target that was changed in the update. http://build.fhir.org/provenance-definitions.html#Provenance.entity.
     */
    relevantHistory?: Reference[] | undefined;
    /**
     * List of Computable Policy Rule Language Representations of this Contract.
     */
    rule?: ContractRule[] | undefined;
    /**
     * A selector of legal concerns for this Contract definition, derivative, or instance in any legal state.
     */
    scope?: CodeableConcept | undefined;
    /**
     * Signers who are principal parties to the contract are bound by the Contract.activity related to the Contract.topic, and the Contract.term(s), which either extend or restrict the overall action on the topic by, for example, stipulating specific policies or obligations constraining actions, action reason, or agents with respect to some or all of the topic.
     * For example, specifying how policies or obligations shall constrain actions and action reasons permitted or denied on all or a subset of the Contract.topic (e.g., all or a portion of property being transferred by the contract), agents (e.g., who can resell, assign interests, or alter the property being transferred by the contract), actions, and action reasons; or with respect to Contract.terms, stipulating, extending, or limiting the Contract.period of applicability or valuation of items under consideration.
     */
    signer?: ContractSigner[] | undefined;
    /**
     * Sites in which the contract is complied with,  exercised, or in force.
     */
    site?: Reference[] | undefined;
    /**
     * This element is labeled as a modifier because the status contains codes that mark the contract as not currently valid or active.
     */
    status?:
        | (
              | 'amended'
              | 'appended'
              | 'cancelled'
              | 'disputed'
              | 'entered-in-error'
              | 'executable'
              | 'executed'
              | 'negotiable'
              | 'offered'
              | 'policy'
              | 'rejected'
              | 'renewed'
              | 'revoked'
              | 'resolved'
              | 'terminated'
          )
        | undefined;
    _status?: Element | undefined;
    /**
     * The Contract.subject is an entity that has some role with respect to the Contract.topic and Contract.topic.term, which is of focal interest to the parties to the contract and likely impacted in a significant way by the Contract.action/Contract.action.reason and the Contract.term.action/Contract.action.reason.
     * In many cases, the Contract.subject is a Contract.signer if the subject is an adult; has a legal interest in the contract; and incompetent to participate in the contract agreement.
     */
    subject?: Reference[] | undefined;
    /**
     * An explanatory or alternate user-friendly title for this Contract definition, derivative, or instance in any legal state.t giving additional information about its content.
     */
    subtitle?: string | undefined;
    _subtitle?: Element | undefined;
    /**
     * Sub-category for the Contract that distinguishes the kinds of systems that would be interested in the Contract within the context of the Contract's scope.
     */
    subType?: CodeableConcept[] | undefined;
    /**
     * Information that may be needed by/relevant to the performer in their execution of this term action.
     */
    supportingInfo?: Reference[] | undefined;
    /**
     * One or more Contract Provisions, which may be related and conveyed as a group, and may contain nested groups.
     */
    term?: ContractTerm[] | undefined;
    /**
     * A short, descriptive, user-friendly title for this Contract definition, derivative, or instance in any legal state.t giving additional information about its content.
     */
    title?: string | undefined;
    _title?: Element | undefined;
    /**
     * Narrows the range of legal concerns to focus on the achievement of specific contractual objectives.
     */
    topicCodeableConcept?: CodeableConcept | undefined;
    /**
     * Narrows the range of legal concerns to focus on the achievement of specific contractual objectives.
     */
    topicReference?: Reference | undefined;
    /**
     * A high-level category for the legal instrument, whether constructed as a Contract definition, derivative, or instance in any legal state.  Provides additional information about its content within the context of the Contract's scope to distinguish the kinds of systems that would be interested in the contract.
     */
    type?: CodeableConcept | undefined;
    /**
     * Used in a domain that uses a supplied contract repository.
     */
    url?: string | undefined;
    _url?: Element | undefined;
    /**
     * Note -  This is a business versionId, not a resource version id (see discussion http://build.fhir.org/resource.html#versions)
     * Comments - There may be different contract instances that have the same identifier but different versions. The version can be appended to the url in a reference to allow a reference to a particular business version of the plan definition with the format [url]|[version]. The version SHOULD NOT contain a '#' - see [Business Version](resource.html#bv-format).
     */
    version?: string | undefined;
    _version?: Element | undefined;
}
/**
 * Link to the paying party and optionally what specifically they will be responsible to pay.
 */
export interface CoveragePaymentBy extends BackboneElement {
    /**
     * The list of parties providing non-insurance payment for the treatment costs.
     */
    party: Reference;
    /**
     *  Description of the financial responsibility.
     */
    responsibility?: string | undefined;
    _responsibility?: Element | undefined;
}
/**
 * For example, class may be used to identify a class of coverage or employer group, policy, or plan.
 */
export interface CoverageClass extends BackboneElement {
    /**
     * A short description for the class.
     */
    name?: string | undefined;
    _name?: Element | undefined;
    /**
     * The type of classification for which an insurer-specific class label or number and optional name is provided.  For example, type may be used to identify a class of coverage or employer group, policy, or plan.
     */
    type: CodeableConcept;
    /**
     * For example, the Group or Plan number.
     */
    value: Identifier;
}
/**
 * A suite of codes indicating exceptions or reductions to patient costs and their effective periods.
 */
export interface CoverageCostToBeneficiaryException extends BackboneElement {
    /**
     * The timeframe the exception is in force.
     */
    period?: Period | undefined;
    /**
     * The code for the specific exception.
     */
    type: CodeableConcept;
}
/**
 * For example by knowing the patient visit co-pay, the provider can collect the amount prior to undertaking treatment.
 */
export interface CoverageCostToBeneficiary extends BackboneElement {
    /**
     * Examples include Medical Care, Periodontics, Renal Dialysis, Vision Coverage.
     */
    category?: CodeableConcept | undefined;
    /**
     * A suite of codes indicating exceptions or reductions to patient costs and their effective periods.
     */
    exception?: CoverageCostToBeneficiaryException[] | undefined;
    /**
     * Is a flag to indicate whether the benefits refer to in-network providers or out-of-network providers.
     */
    network?: CodeableConcept | undefined;
    /**
     * The term or period of the values such as 'maximum lifetime benefit' or 'maximum annual visits'.
     */
    term?: CodeableConcept | undefined;
    /**
     * For example visit, specialist visits, emergency, inpatient care, etc.
     */
    type?: CodeableConcept | undefined;
    /**
     * Indicates if the benefits apply to an individual or to the family.
     */
    unit?: CodeableConcept | undefined;
    /**
     * Amount may be expressed as a percentage of the service/product cost or a fixed amount of currency.
     */
    valueQuantity?: Quantity | undefined;
    /**
     * Amount may be expressed as a percentage of the service/product cost or a fixed amount of currency.
     */
    valueMoney?: Money | undefined;
}
/**
 * Financial instrument which may be used to reimburse or pay for health care products and services. Includes both insurance and self-payment.
 */
export interface Coverage extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'Coverage';
    /**
     * The party who benefits from the insurance coverage; the patient when products and/or services are provided.
     */
    beneficiary: Reference;
    /**
     * For example, class may be used to identify a class of coverage or employer group, policy, or plan.
     */
    class?: CoverageClass[] | undefined;
    /**
     * The policy(s) which constitute this insurance coverage.
     */
    contract?: Reference[] | undefined;
    /**
     * For example by knowing the patient visit co-pay, the provider can collect the amount prior to undertaking treatment.
     */
    costToBeneficiary?: CoverageCostToBeneficiary[] | undefined;
    /**
     * Sometimes the member number is constructed from the subscriberId and the dependant number.
     */
    dependent?: string | undefined;
    _dependent?: Element | undefined;
    /**
     * The main (and possibly only) identifier for the coverage - often referred to as a Member Id, Certificate number, Personal Health Number or Case ID. May be constructed as the concatenation of the Coverage.SubscriberID and the Coverage.dependant. Note that not all insurers issue unique member IDs therefore searches may result in multiple responses.
     */
    identifier?: Identifier[] | undefined;
    /**
     * The insurance plan details, benefits and costs, which constitute this insurance coverage.
     */
    insurancePlan?: Reference | undefined;
    /**
     * May provide multiple identifiers such as insurance company identifier or business identifier (BIN number).
     */
    insurer?: Reference | undefined;
    /**
     * The nature of the coverage be it insurance, or cash payment such as self-pay.
     */
    kind: 'insurance' | 'self-pay' | 'other';
    _kind?: Element | undefined;
    /**
     * The insurer-specific identifier for the insurer-defined network of providers to which the beneficiary may seek treatment which will be covered at the 'in-network' rate, otherwise 'out of network' terms and conditions apply.
     */
    network?: string | undefined;
    _network?: Element | undefined;
    /**
     * The order of applicability of this coverage relative to other coverages which are currently in force. Note, there may be gaps in the numbering and this does not imply primary, secondary etc. as the specific positioning of coverages depends upon the episode of care. For example; a patient might have (0) auto insurance (1) their own health insurance and (2) spouse's health insurance. When claiming for treatments which were not the result of an auto accident then only coverages (1) and (2) above would be applicatble and would apply in the order specified in parenthesis.
     */
    order?: number | undefined;
    /**
     * Link to the paying party and optionally what specifically they will be responsible to pay.
     */
    paymentBy?: CoveragePaymentBy[] | undefined;
    /**
     * Time period during which the coverage is in force. A missing start date indicates the start date isn't known, a missing end date means the coverage is continuing to be in force.
     */
    period?: Period | undefined;
    /**
     * For example: may be an individual, corporation or the subscriber's employer.
     */
    policyHolder?: Reference | undefined;
    /**
     * Typically, an individual uses policies which are theirs (relationship='self') before policies owned by others.
     */
    relationship?: CodeableConcept | undefined;
    /**
     * This element is labeled as a modifier because the status contains the code entered-in-error that marks the coverage as not currently valid.
     */
    status: 'active' | 'cancelled' | 'draft' | 'entered-in-error';
    _status?: Element | undefined;
    /**
     * Typically, automotive and worker's compensation policies would be flagged with 'subrogation=true' to enable healthcare payors to collect against accident claims.
     */
    subrogation?: boolean | undefined;
    _subrogation?: Element | undefined;
    /**
     * May be self or a parent in the case of dependants.
     */
    subscriber?: Reference | undefined;
    /**
     * The insurer assigned ID for the Subscriber.
     */
    subscriberId?: Identifier[] | undefined;
    /**
     * The type of coverage: social program, medical plan, accident coverage (workers compensation, auto), group health or payment by an individual or organization.
     */
    type?: CodeableConcept | undefined;
}
/**
 * Often there are multiple jurisdiction specific valuesets which are required.
 */
export interface CoverageEligibilityRequestSupportingInfo extends BackboneElement {
    /**
     * The supporting materials are applicable for all detail items, product/servce categories and specific billing codes.
     */
    appliesToAll?: boolean | undefined;
    _appliesToAll?: Element | undefined;
    /**
     * Could be used to provide references to other resources, document. For example could contain a PDF in an Attachment of the Police Report for an Accident.
     */
    information: Reference;
    /**
     * A number to uniquely identify supporting information entries.
     */
    sequence: number;
}
/**
 * All insurance coverages for the patient which may be applicable for reimbursement, of the products and services listed in the claim, are typically provided in the claim to allow insurers to confirm the ordering of the insurance coverages relative to local 'coordination of benefit' rules. One coverage (and only one) with 'focal=true' is to be used in the adjudication of this claim. Coverages appearing before the focal Coverage in the list, and where 'subrogation=false', should provide a reference to the ClaimResponse containing the adjudication results of the prior claim.
 */
export interface CoverageEligibilityRequestInsurance extends BackboneElement {
    /**
     * A business agreement number established between the provider and the insurer for special business processing purposes.
     */
    businessArrangement?: string | undefined;
    _businessArrangement?: Element | undefined;
    /**
     * Reference to the insurance card level information contained in the Coverage resource. The coverage issuing insurer will use these details to locate the patient's actual coverage within the insurer's information system.
     */
    coverage: Reference;
    /**
     * A patient may (will) have multiple insurance policies which provide reimburement for healthcare services and products. For example a person may also be covered by their spouse's policy and both appear in the list (and may be from the same insurer). This flag will be set to true for only one of the listed policies and that policy will be used for evaluating this request. Other requests would be created to request evaluation against the other listed policies.
     */
    focal?: boolean | undefined;
    _focal?: Element | undefined;
}
/**
 * Patient diagnosis for which care is sought.
 */
export interface CoverageEligibilityRequestItemDiagnosis extends BackboneElement {
    /**
     * The nature of illness or problem in a coded form or as a reference to an external defined Condition.
     */
    diagnosisCodeableConcept?: CodeableConcept | undefined;
    /**
     * The nature of illness or problem in a coded form or as a reference to an external defined Condition.
     */
    diagnosisReference?: Reference | undefined;
}
/**
 * Service categories or billable services for which benefit details and/or an authorization prior to service delivery may be required by the payor.
 */
export interface CoverageEligibilityRequestItem extends BackboneElement {
    /**
     * Examples include Medical Care, Periodontics, Renal Dialysis, Vision Coverage.
     */
    category?: CodeableConcept | undefined;
    /**
     * The plan/proposal/order describing the proposed service in detail.
     */
    detail?: Reference[] | undefined;
    /**
     * Patient diagnosis for which care is sought.
     */
    diagnosis?: CoverageEligibilityRequestItemDiagnosis[] | undefined;
    /**
     * Facility where the services will be provided.
     */
    facility?: Reference | undefined;
    /**
     * For example in Oral whether the treatment is cosmetic or associated with TMJ, or for Medical whether the treatment was outside the clinic or out of office hours.
     */
    modifier?: CodeableConcept[] | undefined;
    /**
     * Code to indicate the Professional Service or Product supplied (e.g. CTP, HCPCS, USCLS, ICD10, NCPDP, DIN, RxNorm, ACHI, CCI).
     */
    productOrService?: CodeableConcept | undefined;
    /**
     * The practitioner who is responsible for the product or service to be rendered to the patient.
     */
    provider?: Reference | undefined;
    /**
     * The number of repetitions of a service or product.
     */
    quantity?: Quantity | undefined;
    /**
     * Exceptions, special conditions and supporting information applicable for this service or product line.
     */
    supportingInfoSequence?: number[] | undefined;
    /**
     * The amount charged to the patient by the provider for a single unit.
     */
    unitPrice?: Money | undefined;
}
/**
 * The CoverageEligibilityRequest provides patient and insurance coverage information to an insurer for them to respond, in the form of an CoverageEligibilityResponse, with information regarding whether the stated coverage is valid and in-force and optionally to provide the insurance details of the policy.
 */
export interface CoverageEligibilityRequest extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'CoverageEligibilityRequest';
    /**
     * The date when this resource was created.
     */
    created: string;
    _created?: Element | undefined;
    /**
     * Person who created the request.
     */
    enterer?: Reference | undefined;
    /**
     * Facility where the services are intended to be provided.
     */
    facility?: Reference | undefined;
    /**
     * A unique identifier assigned to this coverage eligiblity request.
     */
    identifier?: Identifier[] | undefined;
    /**
     * All insurance coverages for the patient which may be applicable for reimbursement, of the products and services listed in the claim, are typically provided in the claim to allow insurers to confirm the ordering of the insurance coverages relative to local 'coordination of benefit' rules. One coverage (and only one) with 'focal=true' is to be used in the adjudication of this claim. Coverages appearing before the focal Coverage in the list, and where 'subrogation=false', should provide a reference to the ClaimResponse containing the adjudication results of the prior claim.
     */
    insurance?: CoverageEligibilityRequestInsurance[] | undefined;
    /**
     * The Insurer who issued the coverage in question and is the recipient of the request.
     */
    insurer: Reference;
    /**
     * Service categories or billable services for which benefit details and/or an authorization prior to service delivery may be required by the payor.
     */
    item?: CoverageEligibilityRequestItem[] | undefined;
    /**
     * 1..1.
     */
    patient: Reference;
    /**
     * When the requestor expects the processor to complete processing.
     */
    priority?: CodeableConcept | undefined;
    /**
     * This party is responsible for the eligibility request but not necessarily professionally responsible for the provision of the individual products and services listed below. This field is the Billing Provider, for example, a facility, provider group, lab or practitioner.
     */
    provider?: Reference | undefined;
    /**
     * Code to specify whether requesting: prior authorization requirements for some service categories or billing codes; benefits for coverages specified or discovered; discovery and return of coverages for the patient; and/or validation that the specified coverage is in-force at the date/period specified or 'now' if not specified.
     */
    purpose: ('auth-requirements' | 'benefits' | 'discovery' | 'validation')[];
    _purpose?: Element[] | undefined;
    /**
     * The date or dates when the enclosed suite of services were performed or completed.
     */
    servicedDate?: string | undefined;
    _servicedDate?: Element | undefined;
    /**
     * The date or dates when the enclosed suite of services were performed or completed.
     */
    servicedPeriod?: Period | undefined;
    /**
     * This element is labeled as a modifier because the status contains codes that mark the resource as not currently valid.
     */
    status: 'active' | 'cancelled' | 'draft' | 'entered-in-error';
    _status?: Element | undefined;
    /**
     * Often there are multiple jurisdiction specific valuesets which are required.
     */
    supportingInfo?: CoverageEligibilityRequestSupportingInfo[] | undefined;
}
/**
 * Benefits used to date.
 */
export interface CoverageEligibilityResponseInsuranceItemBenefit extends BackboneElement {
    /**
     * The quantity of the benefit which is permitted under the coverage.
     */
    allowedUnsignedInt?: number | undefined;
    /**
     * The quantity of the benefit which is permitted under the coverage.
     */
    allowedString?: string | undefined;
    _allowedString?: Element | undefined;
    /**
     * The quantity of the benefit which is permitted under the coverage.
     */
    allowedMoney?: Money | undefined;
    /**
     * For example: deductible, visits, benefit amount.
     */
    type: CodeableConcept;
    /**
     * The quantity of the benefit which have been consumed to date.
     */
    usedUnsignedInt?: number | undefined;
    /**
     * The quantity of the benefit which have been consumed to date.
     */
    usedString?: string | undefined;
    _usedString?: Element | undefined;
    /**
     * The quantity of the benefit which have been consumed to date.
     */
    usedMoney?: Money | undefined;
}
/**
 * Benefits and optionally current balances, and authorization details by category or service.
 */
export interface CoverageEligibilityResponseInsuranceItem extends BackboneElement {
    /**
     * A boolean flag indicating whether a preauthorization is required prior to actual service delivery.
     */
    authorizationRequired?: boolean | undefined;
    _authorizationRequired?: Element | undefined;
    /**
     * Codes or comments regarding information or actions associated with the preauthorization.
     */
    authorizationSupporting?: CodeableConcept[] | undefined;
    /**
     * A web location for obtaining requirements or descriptive information regarding the preauthorization.
     */
    authorizationUrl?: string | undefined;
    _authorizationUrl?: Element | undefined;
    /**
     * Benefits used to date.
     */
    benefit?: CoverageEligibilityResponseInsuranceItemBenefit[] | undefined;
    /**
     * Examples include Medical Care, Periodontics, Renal Dialysis, Vision Coverage.
     */
    category?: CodeableConcept | undefined;
    /**
     * For example 'DENT2 covers 100% of basic, 50% of major but excludes Ortho, Implants and Cosmetic services'.
     */
    description?: string | undefined;
    _description?: Element | undefined;
    /**
     * True if the indicated class of service is excluded from the plan, missing or False indicates the product or service is included in the coverage.
     */
    excluded?: boolean | undefined;
    _excluded?: Element | undefined;
    /**
     * For example in Oral whether the treatment is cosmetic or associated with TMJ, or for Medical whether the treatment was outside the clinic or out of office hours.
     */
    modifier?: CodeableConcept[] | undefined;
    /**
     * For example: MED01, or DENT2.
     */
    name?: string | undefined;
    _name?: Element | undefined;
    /**
     * Is a flag to indicate whether the benefits refer to in-network providers or out-of-network providers.
     */
    network?: CodeableConcept | undefined;
    /**
     * Code to indicate the Professional Service or Product supplied (e.g. CTP, HCPCS, USCLS, ICD10, NCPDP, DIN, RxNorm, ACHI, CCI).
     */
    productOrService?: CodeableConcept | undefined;
    /**
     * The practitioner who is eligible for the provision of the product or service.
     */
    provider?: Reference | undefined;
    /**
     * The term or period of the values such as 'maximum lifetime benefit' or 'maximum annual visits'.
     */
    term?: CodeableConcept | undefined;
    /**
     * Indicates if the benefits apply to an individual or to the family.
     */
    unit?: CodeableConcept | undefined;
}
/**
 * All insurance coverages for the patient which may be applicable for reimbursement, of the products and services listed in the claim, are typically provided in the claim to allow insurers to confirm the ordering of the insurance coverages relative to local 'coordination of benefit' rules. One coverage (and only one) with 'focal=true' is to be used in the adjudication of this claim. Coverages appearing before the focal Coverage in the list, and where 'subrogation=false', should provide a reference to the ClaimResponse containing the adjudication results of the prior claim.
 */
export interface CoverageEligibilityResponseInsurance extends BackboneElement {
    /**
     * The term of the benefits documented in this response.
     */
    benefitPeriod?: Period | undefined;
    /**
     * Reference to the insurance card level information contained in the Coverage resource. The coverage issuing insurer will use these details to locate the patient's actual coverage within the insurer's information system.
     */
    coverage: Reference;
    /**
     * Flag indicating if the coverage provided is inforce currently if no service date(s) specified or for the whole duration of the service dates.
     */
    inforce?: boolean | undefined;
    _inforce?: Element | undefined;
    /**
     * Benefits and optionally current balances, and authorization details by category or service.
     */
    item?: CoverageEligibilityResponseInsuranceItem[] | undefined;
}
/**
 * Errors encountered during the processing of the request.
 */
export interface CoverageEligibilityResponseError extends BackboneElement {
    /**
     * An error code,from a specified code system, which details why the eligibility check could not be performed.
     */
    code: CodeableConcept;
}
/**
 * This resource provides eligibility and plan details from the processing of an CoverageEligibilityRequest resource.
 */
export interface CoverageEligibilityResponse extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'CoverageEligibilityResponse';
    /**
     * The date this resource was created.
     */
    created: string;
    _created?: Element | undefined;
    /**
     * A human readable description of the status of the adjudication.
     */
    disposition?: string | undefined;
    _disposition?: Element | undefined;
    /**
     * Errors encountered during the processing of the request.
     */
    error?: CoverageEligibilityResponseError[] | undefined;
    /**
     * May be needed to identify specific jurisdictional forms.
     */
    form?: CodeableConcept | undefined;
    /**
     * A unique identifier assigned to this coverage eligiblity request.
     */
    identifier?: Identifier[] | undefined;
    /**
     * All insurance coverages for the patient which may be applicable for reimbursement, of the products and services listed in the claim, are typically provided in the claim to allow insurers to confirm the ordering of the insurance coverages relative to local 'coordination of benefit' rules. One coverage (and only one) with 'focal=true' is to be used in the adjudication of this claim. Coverages appearing before the focal Coverage in the list, and where 'subrogation=false', should provide a reference to the ClaimResponse containing the adjudication results of the prior claim.
     */
    insurance?: CoverageEligibilityResponseInsurance[] | undefined;
    /**
     * The Insurer who issued the coverage in question and is the author of the response.
     */
    insurer: Reference;
    /**
     * The resource may be used to indicate that: the request has been held (queued) for processing; that it has been processed and errors found (error); that no errors were found and that some of the adjudication has been undertaken (partial) or that all of the adjudication has been undertaken (complete).
     */
    outcome: 'queued' | 'complete' | 'error' | 'partial';
    _outcome?: Element | undefined;
    /**
     * The party who is the beneficiary of the supplied coverage and for whom eligibility is sought.
     */
    patient: Reference;
    /**
     * A reference from the Insurer to which these services pertain to be used on further communication and as proof that the request occurred.
     */
    preAuthRef?: string | undefined;
    _preAuthRef?: Element | undefined;
    /**
     * Code to specify whether requesting: prior authorization requirements for some service categories or billing codes; benefits for coverages specified or discovered; discovery and return of coverages for the patient; and/or validation that the specified coverage is in-force at the date/period specified or 'now' if not specified.
     */
    purpose: ('auth-requirements' | 'benefits' | 'discovery' | 'validation')[];
    _purpose?: Element[] | undefined;
    /**
     * Reference to the original request resource.
     */
    request: Reference;
    /**
     * This party is responsible for the claim but not necessarily professionally responsible for the provision of the individual products and services listed below. This field is the Billing Provider, for example, a facility, provider group, lab or practitioner.
     */
    requestor?: Reference | undefined;
    /**
     * The date or dates when the enclosed suite of services were performed or completed.
     */
    servicedDate?: string | undefined;
    _servicedDate?: Element | undefined;
    /**
     * The date or dates when the enclosed suite of services were performed or completed.
     */
    servicedPeriod?: Period | undefined;
    /**
     * This element is labeled as a modifier because the status contains codes that mark the resource as not currently valid.
     */
    status: 'active' | 'cancelled' | 'draft' | 'entered-in-error';
    _status?: Element | undefined;
}
/**
 * Supporting evidence or manifestations that provide the basis for identifying the detected issue such as a GuidanceResponse or MeasureReport.
 */
export interface DetectedIssueEvidence extends BackboneElement {
    /**
     * A manifestation that led to the recording of this detected issue.
     */
    code?: CodeableConcept[] | undefined;
    /**
     * Links to resources that constitute evidence for the detected issue such as a GuidanceResponse or MeasureReport.
     */
    detail?: Reference[] | undefined;
}
/**
 * Indicates an action that has been taken or is committed to reduce or eliminate the likelihood of the risk identified by the detected issue from manifesting.  Can also reflect an observation of known mitigating factors that may reduce/eliminate the need for any action.
 */
export interface DetectedIssueMitigation extends BackboneElement {
    /**
     * The "text" component can be used for detail or when no appropriate code exists.
     */
    action: CodeableConcept;
    /**
     * Identifies the practitioner who determined the mitigation and takes responsibility for the mitigation step occurring.
     */
    author?: Reference | undefined;
    /**
     * This might not be the same as when the mitigating step was actually taken.
     */
    date?: string | undefined;
    _date?: Element | undefined;
}
/**
 * Indicates an actual or potential clinical issue with or between one or more active or proposed clinical actions for a patient; e.g. Drug-drug interaction, Ineffective treatment frequency, Procedure-condition conflict, etc.
 */
export interface DetectedIssue extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'DetectedIssue';
    /**
     * Individual or device responsible for the issue being raised.  For example, a decision support application or a pharmacist conducting a medication review.
     */
    author?: Reference | undefined;
    /**
     * In addition to the required category valueset, this element allows various categorization schemes based on the owner’s definition of the category and effectively multiple categories can be used at once.  The level of granularity is defined by the category concepts in the value set.
     */
    category?: CodeableConcept[] | undefined;
    /**
     * Identifies the specific type of issue identified.
     */
    code?: CodeableConcept | undefined;
    /**
     * Should focus on information not covered elsewhere as discrete data - no need to duplicate the narrative.
     */
    detail?: string | undefined;
    _detail?: Element | undefined;
    /**
     * Supporting evidence or manifestations that provide the basis for identifying the detected issue such as a GuidanceResponse or MeasureReport.
     */
    evidence?: DetectedIssueEvidence[] | undefined;
    /**
     * The date or period when the detected issue was initially identified.
     */
    identifiedDateTime?: string | undefined;
    _identifiedDateTime?: Element | undefined;
    /**
     * The date or period when the detected issue was initially identified.
     */
    identifiedPeriod?: Period | undefined;
    /**
     * Business identifier associated with the detected issue record.
     */
    identifier?: Identifier[] | undefined;
    /**
     * There's an implicit constraint on the number of implicated resources based on DetectedIssue.type; e.g. For drug-drug, there would be more than one.  For timing, there would typically only be one.
     */
    implicated?: Reference[] | undefined;
    /**
     * Indicates an action that has been taken or is committed to reduce or eliminate the likelihood of the risk identified by the detected issue from manifesting.  Can also reflect an observation of known mitigating factors that may reduce/eliminate the need for any action.
     */
    mitigation?: DetectedIssueMitigation[] | undefined;
    /**
     * The literature, knowledge-base or similar reference that describes the propensity for the detected issue identified.
     */
    reference?: string | undefined;
    _reference?: Element | undefined;
    /**
     * Indicates the degree of importance associated with the identified issue based on the potential impact on the patient.
     */
    severity?: ('high' | 'moderate' | 'low') | undefined;
    _severity?: Element | undefined;
    /**
     * This element is labeled as a modifier because the status contains the code entered-in-error that marks the issue as not currently valid.
     */
    status: 'preliminary' | 'final' | 'entered-in-error' | 'mitigated';
    _status?: Element | undefined;
    /**
     * Indicates the subject whose record the detected issue is associated with.
     */
    subject?: Reference | undefined;
}
/**
 * UDI may identify an unique instance of a device, or it may only identify the type of the device.  See [UDI mappings](device-mappings.html#udi) for a complete mapping of UDI parts to Device.
 */
export interface DeviceUdiCarrier extends BackboneElement {
    /**
     * The AIDC form of UDIs should be scanned or otherwise used for the identification of the device whenever possible to minimize errors in records resulting from manual transcriptions. If separate barcodes for DI and PI are present, concatenate the string with DI first and in order of human readable expression on label.
     */
    carrierAIDC?: string | undefined;
    _carrierAIDC?: Element | undefined;
    /**
     * If separate barcodes for DI and PI are present, concatenate the string with DI first and in order of human readable expression on label.
     */
    carrierHRF?: string | undefined;
    _carrierHRF?: Element | undefined;
    /**
     * The device identifier (DI) is a mandatory, fixed portion of a UDI that identifies the labeler and the specific version or model of a device.
     */
    deviceIdentifier: string;
    _deviceIdentifier?: Element | undefined;
    /**
     * A coded entry to indicate how the data was entered.
     */
    entryType?:
        | ('barcode' | 'rfid' | 'manual' | 'card' | 'self-reported' | 'electronic-transmission' | 'unknown')
        | undefined;
    _entryType?: Element | undefined;
    /**
     * Organization that is charged with issuing UDIs for devices. For example, the US FDA issuers include:
     * 1) GS1: http://hl7.org/fhir/NamingSystem/gs1-di,
     * 2) HIBCC: http://hl7.org/fhir/NamingSystem/hibcc-diI,
     * 3) ICCBBA for blood containers: http://hl7.org/fhir/NamingSystem/iccbba-blood-di,
     * 4) ICCBA for other devices: http://hl7.org/fhir/NamingSystem/iccbba-other-di # Informationsstelle für Arzneispezialitäten (IFA GmbH) (EU only): http://hl7.org/fhir/NamingSystem/ifa-gmbh-di.
     */
    issuer: string;
    _issuer?: Element | undefined;
    /**
     * The identity of the authoritative source for UDI generation within a jurisdiction. All UDIs are globally unique within a single namespace with the appropriate repository uri as the system. For example, UDIs of devices managed in the U.S. by the FDA, the value is http://hl7.org/fhir/NamingSystem/us-fda-udi or in the European Union by the European Commission http://hl7.org/fhir/NamingSystem/eu-ec-udi.
     */
    jurisdiction?: string | undefined;
    _jurisdiction?: Element | undefined;
}
/**
 * This represents the manufacturer's name of the device as provided by the device, from a UDI label, or by a person describing the Device.  This typically would be used when a person provides the name(s) or when the device represents one of the names available from DeviceDefinition.
 */
export interface DeviceDeviceName extends BackboneElement {
    /**
     * The name that identifies the device.
     */
    name: string;
    _name?: Element | undefined;
    /**
     * The type of deviceName. Note that ManufactureDeviceName means that the name is the name as given by the manufacturer, not the name of the manufacturer.
     * RegisteredName | UserFriendlyName | PatientReportedName.
     */
    type: 'registered-name' | 'user-friendly-name' | 'patient-reported-name';
    _type?: Element | undefined;
}
/**
 * The actual design of the device or software version running on the device.
 */
export interface DeviceVersion extends BackboneElement {
    /**
     * Note that the module of the device would not need to be tracked as a separate device, e.g., using different UDI, thus would typically involve one or more software modules. For example, a device may involve two software modules each on a different version.
     */
    component?: Identifier | undefined;
    /**
     * The date the version was installed on the device.
     */
    installDate?: string | undefined;
    _installDate?: Element | undefined;
    /**
     * The type of the device version, e.g. manufacturer, approved, internal.
     */
    type?: CodeableConcept | undefined;
    /**
     * The version text.
     */
    value: string;
    _value?: Element | undefined;
}
/**
 * The standards to which the device adheres and may be certified to in support of its capabilities, e.g., communication, performance, process, or measurement standards.
 */
export interface DeviceSpecialization extends BackboneElement {
    /**
     * Kind of standards that the device adheres to, e.g., communication, performance or communication.
     */
    category?: Coding | undefined;
    /**
     * Code that specifies the system that identifies the specific standard that the device adheres to.
     */
    systemType: CodeableConcept;
    /**
     * The version of the standard that is used to operate and communicate.
     */
    version?: string | undefined;
    _version?: Element | undefined;
}
/**
 * Characteristics or features of the device that are otherwise not captured in available attributes, e.g., actual configuration settings, time or timing attributes, resolution, accuracy, and physical attributes.  The focus is on properties of the device actually in use while DeviceDefinition focuses on properties that are available to be used.
 */
export interface DeviceProperty extends BackboneElement {
    /**
     * Code that specifies the property being represented. No codes are specified but the MDC codes are an example: https://terminology.hl7.org/MDC.html.
     */
    type: CodeableConcept;
    /**
     * The string is used for properties that are intrinsically text, such as warning text, or assemlby instructions. CodeableConcept.text also supports unencoded text but should be used where the characteristic is a concept that might be coded but there is no assigned code for the present value. For example, a security class can be a code like 'Class 1' but in some cases may contain a string like 'Depending on the settings' - in this case the property is normally coded but this is an exception
     */
    valueQuantity?: Quantity | undefined;
    /**
     * The string is used for properties that are intrinsically text, such as warning text, or assemlby instructions. CodeableConcept.text also supports unencoded text but should be used where the characteristic is a concept that might be coded but there is no assigned code for the present value. For example, a security class can be a code like 'Class 1' but in some cases may contain a string like 'Depending on the settings' - in this case the property is normally coded but this is an exception
     */
    valueCodeableConcept?: CodeableConcept | undefined;
    /**
     * The string is used for properties that are intrinsically text, such as warning text, or assemlby instructions. CodeableConcept.text also supports unencoded text but should be used where the characteristic is a concept that might be coded but there is no assigned code for the present value. For example, a security class can be a code like 'Class 1' but in some cases may contain a string like 'Depending on the settings' - in this case the property is normally coded but this is an exception
     */
    valueString?: string | undefined;
    _valueString?: Element | undefined;
    /**
     * The string is used for properties that are intrinsically text, such as warning text, or assemlby instructions. CodeableConcept.text also supports unencoded text but should be used where the characteristic is a concept that might be coded but there is no assigned code for the present value. For example, a security class can be a code like 'Class 1' but in some cases may contain a string like 'Depending on the settings' - in this case the property is normally coded but this is an exception
     */
    valueBoolean?: boolean | undefined;
    _valueBoolean?: Element | undefined;
    /**
     * The string is used for properties that are intrinsically text, such as warning text, or assemlby instructions. CodeableConcept.text also supports unencoded text but should be used where the characteristic is a concept that might be coded but there is no assigned code for the present value. For example, a security class can be a code like 'Class 1' but in some cases may contain a string like 'Depending on the settings' - in this case the property is normally coded but this is an exception
     */
    valueInteger?: number | undefined;
    /**
     * The string is used for properties that are intrinsically text, such as warning text, or assemlby instructions. CodeableConcept.text also supports unencoded text but should be used where the characteristic is a concept that might be coded but there is no assigned code for the present value. For example, a security class can be a code like 'Class 1' but in some cases may contain a string like 'Depending on the settings' - in this case the property is normally coded but this is an exception
     */
    valueRange?: Range | undefined;
    /**
     * The string is used for properties that are intrinsically text, such as warning text, or assemlby instructions. CodeableConcept.text also supports unencoded text but should be used where the characteristic is a concept that might be coded but there is no assigned code for the present value. For example, a security class can be a code like 'Class 1' but in some cases may contain a string like 'Depending on the settings' - in this case the property is normally coded but this is an exception
     */
    valueAttachment?: Attachment | undefined;
}
/**
 * The status of the device itself - whether it is switched on, or activated, etc.
 */
export interface DeviceOperation extends BackboneElement {
    /**
     * The series of occurrences that repeats during the operation of the device.
     */
    cycle?: Count | undefined;
    /**
     * A measurement of time during the device's operation (e.g., days, hours, mins, etc).
     */
    duration?: Duration | undefined;
    /**
     * The designated condition for performing a task with the device.
     */
    mode?: CodeableConcept | undefined;
    /**
     * The individual performing the action enabled by the device.
     */
    operator?: Reference[] | undefined;
    /**
     * The state or condition of the device's operation.
     */
    status: CodeableConcept;
    /**
     * The reasons given for the current operational status - i.e. why is the device switched on etc.
     */
    statusReason?: CodeableConcept[] | undefined;
}
/**
 * The details about the device when it is affixed or inside of a patient.
 */
export interface DeviceAssociation extends BackboneElement {
    /**
     * The current anatomical location of the device in/on the humanSubject where it is attached or placed.
     */
    bodyStructure?: CodeableReference | undefined;
    /**
     * The individual to whom the device is affixed or inserted in their body.
     */
    humanSubject?: Reference | undefined;
    /**
     * The state of the usage or application of the device.
     */
    status: CodeableConcept;
    /**
     * The reasons given for the current association status - i.e. why is the device explanted, or attached to the patient, etc.
     */
    statusReason?: CodeableConcept[] | undefined;
}
/**
 * A type of a manufactured item that is used in the provision of healthcare without being substantially changed through that activity. The device may be a medical or non-medical device.
 */
export interface Device extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'Device';
    /**
     * The details about the device when it is affixed or inside of a patient.
     */
    association?: DeviceAssociation[] | undefined;
    /**
     * The availability of the device.
     */
    availabilityStatus?: CodeableConcept | undefined;
    /**
     * Necessary to support mandatory requirements for traceability from donor/source to recipient and vice versa, while also satisfying donor anonymity requirements. The element is defined consistently across BiologicallyDerivedProduct, NutritionProduct, and Device.  The identifier references an event that links to a single biological entity such as a blood donor, or to multiple biological entities (e.g. when the product is an embryo or a pooled platelet product).  A single biologicalSourceEvent identifier may appear on multiple products of many types derived from a single donation event or source extraction.  As an example, a single donation event may provide 2 kidneys and a liver for organ transplantation, 2 corneas for eye surgery, heart valves and arterial tissue for cardiovascular surgery, multiple skin grafts, tendons, multiple shaped bone grafts and a large number of bone putty/paste products; and each of them may be assigned to the same biological source event identifier.
     */
    biologicalSourceEvent?: Identifier | undefined;
    /**
     * Devices may be associated with one or more categories.
     */
    category?: CodeableConcept[] | undefined;
    /**
     * used for troubleshooting etc.
     */
    contact?: ContactPoint[] | undefined;
    /**
     * The reference to the definition for the device.
     */
    definition?: CodeableReference | undefined;
    /**
     * This represents the manufacturer's name of the device as provided by the device, from a UDI label, or by a person describing the Device.  This typically would be used when a person provides the name(s) or when the device represents one of the names available from DeviceDefinition.
     */
    deviceName?: DeviceDeviceName[] | undefined;
    /**
     * The name used to display by default when the device is referenced. Based on intent of use by the resource creator, this may reflect one of the names in Device.deviceName, or may be another simple name.
     */
    displayName?: string | undefined;
    _displayName?: Element | undefined;
    /**
     * Technical endpoints providing access to services provided by the device defined at this resource.
     */
    endpoint?: Reference[] | undefined;
    /**
     * The date and time beyond which this device is no longer valid or should not be used (if applicable).
     */
    expirationDate?: string | undefined;
    _expirationDate?: Element | undefined;
    /**
     * The linked device acting as a communication/data collector, translator or controller for the current device (e.g., mobile phone application that relays a blood pressure device's data).
     */
    gateway?: CodeableReference[] | undefined;
    /**
     * Certain attributes, like serial number and UDI Carrier (the HRF or AIDC barcode string) are not device instance identifiers as they are not consistently able to uniquely identify the instance of a device, thus are not appropriate to be used to value Device.identifier. The barcode string from a barcode present on a device label or package may identify the instance, include names given to the device in local usage, or may identify the type of device. If the identifier identifies the type of device, Device.type element should be used. The identifier is typically valued if the serialNumber or lotNumber is not valued and represents a different type of identifier.  However, it is permissible to still include those identifiers in DeviceDefinition.identifier with the appropriate identifier.type.
     */
    identifier?: Identifier[] | undefined;
    /**
     * The place where the device can be found.
     */
    location?: Reference | undefined;
    /**
     * Lot number assigned by the manufacturer.
     */
    lotNumber?: string | undefined;
    _lotNumber?: Element | undefined;
    /**
     * The date and time when the device was manufactured.
     */
    manufactureDate?: string | undefined;
    _manufactureDate?: Element | undefined;
    /**
     * A name of the manufacturer or entity legally responsible for the device.
     */
    manufacturer?: string | undefined;
    _manufacturer?: Element | undefined;
    /**
     * The manufacturer's model number for the device.
     */
    modelNumber?: string | undefined;
    _modelNumber?: Element | undefined;
    /**
     * Descriptive information, usage information or implantation information that is not captured in an existing element.
     */
    note?: Annotation[] | undefined;
    /**
     * The status of the device itself - whether it is switched on, or activated, etc.
     */
    operation?: DeviceOperation[] | undefined;
    /**
     * An organization that is responsible for the provision and ongoing maintenance of the device.
     */
    owner?: Reference | undefined;
    /**
     * For example a vital signs monitor (parent) where three separate modules can be plugged into such as interchangeable blood pressure, oximeter, temperature modules.  These modules are represented as devices with the .parent valued to the vital signs monitor when plugged in.
     */
    parent?: Reference | undefined;
    /**
     * Alphanumeric Maximum 20.
     */
    partNumber?: string | undefined;
    _partNumber?: Element | undefined;
    /**
     * Characteristics or features of the device that are otherwise not captured in available attributes, e.g., actual configuration settings, time or timing attributes, resolution, accuracy, and physical attributes.  The focus is on properties of the device actually in use while DeviceDefinition focuses on properties that are available to be used.
     */
    property?: DeviceProperty[] | undefined;
    /**
     * Provides additional safety characteristics about a medical device.  For example devices containing latex.
     */
    safety?: CodeableConcept[] | undefined;
    /**
     * Alphanumeric Maximum 20. While a serial number is a type of identifier, in the context of devices for which a UDI is required, thus a serial number may be part of the production identifier, it is more informative to have the serial number as a dedicated attribute together with the other UDI production identifier.  When the device is not subject to including the UDI (e.g., its presence and exchange is not mandated by local regulatory requirements or specific use case at hand), thus the concept of production identifiers is not relevant as a set, it remains helpful for consistency to still use Device.serialNumber rather than using Device.identifier with an appropriate type.  Systems that do not realize an identifier is a serial number may place it in Device.identifier or if the identifier.system is known for the serial number, it may also be populated in the Device.identifier as a globally unique id_.
     */
    serialNumber?: string | undefined;
    _serialNumber?: Element | undefined;
    /**
     * The standards to which the device adheres and may be certified to in support of its capabilities, e.g., communication, performance, process, or measurement standards.
     */
    specialization?: DeviceSpecialization[] | undefined;
    /**
     * This element is labeled as a modifier because the status contains the codes inactive and entered-in-error that mark the device (record)as not currently valid.
     */
    status?: ('active' | 'inactive' | 'entered-in-error') | undefined;
    _status?: Element | undefined;
    /**
     * Multiple device types are use when a device is categorized in more than one context – for example, hybrid devices in which the device is both of type gateway and sensor.
     */
    type?: CodeableConcept[] | undefined;
    /**
     * UDI may identify an unique instance of a device, or it may only identify the type of the device.  See [UDI mappings](device-mappings.html#udi) for a complete mapping of UDI parts to Device.
     */
    udiCarrier?: DeviceUdiCarrier[] | undefined;
    /**
     * If the device is running a FHIR server, the network address should  be the Base URL from which a conformance statement may be retrieved.
     */
    url?: string | undefined;
    _url?: Element | undefined;
    /**
     * The actual design of the device or software version running on the device.
     */
    version?: DeviceVersion[] | undefined;
}
/**
 * Indicates where and when the device is available on the market.
 */
export interface DeviceDefinitionUdiDeviceIdentifierMarketDistribution extends BackboneElement {
    /**
     * Begin and end dates for the commercial distribution of the device.
     */
    marketPeriod: Period;
    /**
     * National state or territory to which the marketDistribution recers, typically where the device is commercialized.
     */
    subJurisdiction: string;
    _subJurisdiction?: Element | undefined;
}
/**
 * Unique device identifier (UDI) assigned to device label or package.  Note that the Device may include multiple udiCarriers as it either may include just the udiCarrier for the jurisdiction it is sold, or for multiple jurisdictions it could have been sold.
 */
export interface DeviceDefinitionUdiDeviceIdentifier extends BackboneElement {
    /**
     * The identifier that is to be associated with every Device that references this DeviceDefintiion for the issuer and jurisdiction provided in the DeviceDefinition.udiDeviceIdentifier.
     */
    deviceIdentifier: string;
    _deviceIdentifier?: Element | undefined;
    /**
     * The organization that assigns the identifier algorithm.
     */
    issuer: string;
    _issuer?: Element | undefined;
    /**
     * The jurisdiction to which the deviceIdentifier applies.
     */
    jurisdiction: string;
    _jurisdiction?: Element | undefined;
    /**
     * Indicates where and when the device is available on the market.
     */
    marketDistribution?: DeviceDefinitionUdiDeviceIdentifierMarketDistribution[] | undefined;
}
/**
 * This should not be used for regulatory authorization numbers which are to be captured elsewhere.
 */
export interface DeviceDefinitionRegulatoryIdentifier extends BackboneElement {
    /**
     * The identifier itself.
     */
    deviceIdentifier: string;
    _deviceIdentifier?: Element | undefined;
    /**
     * The organization that issued this identifier.
     */
    issuer: string;
    _issuer?: Element | undefined;
    /**
     * The jurisdiction to which the deviceIdentifier applies.
     */
    jurisdiction: string;
    _jurisdiction?: Element | undefined;
    /**
     * The type of identifier itself.
     */
    type: 'basic' | 'master' | 'license';
    _type?: Element | undefined;
}
/**
 * The name or names of the device as given by the manufacturer.
 */
export interface DeviceDefinitionDeviceName extends BackboneElement {
    /**
     * A human-friendly name that is used to refer to the device - depending on the type, it can be the brand name, the common name or alias, or other.
     */
    name: string;
    _name?: Element | undefined;
    /**
     * The type of deviceName.
     * RegisteredName | UserFriendlyName | PatientReportedName.
     */
    type: 'registered-name' | 'user-friendly-name' | 'patient-reported-name';
    _type?: Element | undefined;
}
/**
 * In this element various classifications can be used, such as GMDN, EMDN, SNOMED CT, risk classes, national product codes.
 */
export interface DeviceDefinitionClassification extends BackboneElement {
    /**
     * Further information qualifying this classification of the device model.
     */
    justification?: RelatedArtifact[] | undefined;
    /**
     * A classification or risk class of the device model.
     */
    type: CodeableConcept;
}
/**
 * A device that is part (for example a component) of the present device.
 */
export interface DeviceDefinitionHasPart extends BackboneElement {
    /**
     * Number of instances of the component device in the current device.
     */
    count?: number | undefined;
    /**
     * Reference to the device that is part of the current device.
     */
    reference: Reference;
}
/**
 * An organization that distributes the packaged device.
 */
export interface DeviceDefinitionPackagingDistributor extends BackboneElement {
    /**
     * Distributor's human-readable name.
     */
    name?: string | undefined;
    _name?: Element | undefined;
    /**
     * Distributor as an Organization resource.
     */
    organizationReference?: Reference[] | undefined;
}
/**
 * Information about the packaging of the device, i.e. how the device is packaged.
 */
export interface DeviceDefinitionPackaging extends BackboneElement {
    /**
     * The number of items contained in the package (devices or sub-packages).
     */
    count?: number | undefined;
    /**
     * An organization that distributes the packaged device.
     */
    distributor?: DeviceDefinitionPackagingDistributor[] | undefined;
    /**
     * The business identifier of the packaged medication.
     */
    identifier?: Identifier | undefined;
    /**
     * Allows packages within packages.
     */
    packaging?: DeviceDefinitionPackaging[] | undefined;
    /**
     * A code that defines the specific type of packaging.
     */
    type?: CodeableConcept | undefined;
    /**
     * Unique Device Identifier (UDI) Barcode string on the packaging.
     */
    udiDeviceIdentifier?: DeviceDefinitionUdiDeviceIdentifier[] | undefined;
}
/**
 * The version of the device or software.
 */
export interface DeviceDefinitionVersion extends BackboneElement {
    /**
     * Note that the module of the device would not need to be tracked as a separate device, e.g., using different UDI, thus would typically involve one or more software modules. For example, a device may involve two software modules each on a different version.
     */
    component?: Identifier | undefined;
    /**
     * The type of the device version, e.g. manufacturer, approved, internal.
     */
    type?: CodeableConcept | undefined;
    /**
     * The version text.
     */
    value: string;
    _value?: Element | undefined;
}
/**
 * The potential, valid configuration settings of a device, e.g., regulation status, time properties.
 */
export interface DeviceDefinitionProperty extends BackboneElement {
    /**
     * Code that specifies the property.
     */
    type: CodeableConcept;
    /**
     * The usage of CodeableConcept is for concepts that are normally coded but for which a code is not available - for example a device implant size that is not standardized. The property string is for text characteristics, e.g. long descriptions, warnings, etc.
     */
    valueQuantity?: Quantity | undefined;
    /**
     * The usage of CodeableConcept is for concepts that are normally coded but for which a code is not available - for example a device implant size that is not standardized. The property string is for text characteristics, e.g. long descriptions, warnings, etc.
     */
    valueCodeableConcept?: CodeableConcept | undefined;
    /**
     * The usage of CodeableConcept is for concepts that are normally coded but for which a code is not available - for example a device implant size that is not standardized. The property string is for text characteristics, e.g. long descriptions, warnings, etc.
     */
    valueString?: string | undefined;
    _valueString?: Element | undefined;
    /**
     * The usage of CodeableConcept is for concepts that are normally coded but for which a code is not available - for example a device implant size that is not standardized. The property string is for text characteristics, e.g. long descriptions, warnings, etc.
     */
    valueBoolean?: boolean | undefined;
    _valueBoolean?: Element | undefined;
    /**
     * The usage of CodeableConcept is for concepts that are normally coded but for which a code is not available - for example a device implant size that is not standardized. The property string is for text characteristics, e.g. long descriptions, warnings, etc.
     */
    valueInteger?: number | undefined;
    /**
     * The usage of CodeableConcept is for concepts that are normally coded but for which a code is not available - for example a device implant size that is not standardized. The property string is for text characteristics, e.g. long descriptions, warnings, etc.
     */
    valueRange?: Range | undefined;
    /**
     * The usage of CodeableConcept is for concepts that are normally coded but for which a code is not available - for example a device implant size that is not standardized. The property string is for text characteristics, e.g. long descriptions, warnings, etc.
     */
    valueAttachment?: Attachment | undefined;
}
/**
 * An associated device, attached to, used with, communicating with or linking a previous or new device model to the focal device.
 */
export interface DeviceDefinitionLink extends BackboneElement {
    /**
     * A reference to the linked device.
     */
    relatedDevice: CodeableReference;
    /**
     * The type indicates the relationship of the related device to the device instance.
     */
    relation: Coding;
}
/**
 * A substance used to create the material(s) of which the device is made.
 */
export interface DeviceDefinitionMaterial extends BackboneElement {
    /**
     * Whether the substance is a known or suspected allergen.
     */
    allergenicIndicator?: boolean | undefined;
    _allergenicIndicator?: Element | undefined;
    /**
     * Indicates an alternative material of the device.
     */
    alternate?: boolean | undefined;
    _alternate?: Element | undefined;
    /**
     * A substance that the device contains, may contain, or is made of - for example latex - to be used to determine patient compatibility. This is not intended to represent the composition of the device, only the clinically relevant materials.
     */
    substance: CodeableConcept;
}
/**
 * Information aimed at providing directions for the usage of this model of device.
 */
export interface DeviceDefinitionGuideline extends BackboneElement {
    /**
     * A specific situation when a device should not be used because it may cause harm.
     */
    contraindication?: CodeableReference[] | undefined;
    /**
     * A clinical condition for which the device was designed to be used.
     */
    indication?: CodeableReference[] | undefined;
    /**
     * A description of the general purpose or medical use of the device or its function.
     */
    intendedUse?: string | undefined;
    _intendedUse?: Element | undefined;
    /**
     * A source of information or reference for this guideline.
     */
    relatedArtifact?: RelatedArtifact[] | undefined;
    /**
     * Detailed written and visual directions for the user on how to use the device.
     */
    usageInstruction?: string | undefined;
    _usageInstruction?: Element | undefined;
    /**
     * The circumstances that form the setting for using the device.
     */
    useContext?: UsageContext[] | undefined;
    /**
     * Specific hazard alert information that a user needs to know before using the device.
     */
    warning?: CodeableReference[] | undefined;
}
/**
 * Tracking of latest field safety corrective action.
 */
export interface DeviceDefinitionCorrectiveAction extends BackboneElement {
    /**
     * Start and end dates of the  corrective action.
     */
    period: Period;
    /**
     * When a field safety action has been initiated but not yet resulted in a recall, this element will be set to 'false'.
     */
    recall: boolean;
    _recall?: Element | undefined;
    /**
     * The scope of the corrective action - whether the action targeted all units of a given device model, or only a specific set of batches identified by lot numbers, or individually identified devices identified by the serial name.
     */
    scope?: ('model' | 'lot-numbers' | 'serial-numbers') | undefined;
    _scope?: Element | undefined;
}
/**
 * Billing code or reference associated with the device.
 */
export interface DeviceDefinitionChargeItem extends BackboneElement {
    /**
     * The code or reference for the charge item.
     */
    chargeItemCode: CodeableReference;
    /**
     * Coefficient applicable to the billing code.
     */
    count: Quantity;
    /**
     * A specific time period in which this charge item applies.
     */
    effectivePeriod?: Period | undefined;
    /**
     * The context to which this charge item applies.
     */
    useContext?: UsageContext[] | undefined;
}
/**
 * The characteristics, operational status and capabilities of a medical-related component of a medical device.
 */
export interface DeviceDefinition extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'DeviceDefinition';
    /**
     * Billing code or reference associated with the device.
     */
    chargeItem?: DeviceDefinitionChargeItem[] | undefined;
    /**
     * In this element various classifications can be used, such as GMDN, EMDN, SNOMED CT, risk classes, national product codes.
     */
    classification?: DeviceDefinitionClassification[] | undefined;
    /**
     * used for troubleshooting etc.
     */
    contact?: ContactPoint[] | undefined;
    /**
     * Tracking of latest field safety corrective action.
     */
    correctiveAction?: DeviceDefinitionCorrectiveAction | undefined;
    /**
     * Additional information to describe the device.
     */
    description?: string | undefined;
    _description?: Element | undefined;
    /**
     * The name or names of the device as given by the manufacturer.
     */
    deviceName?: DeviceDefinitionDeviceName[] | undefined;
    /**
     * Information aimed at providing directions for the usage of this model of device.
     */
    guideline?: DeviceDefinitionGuideline | undefined;
    /**
     * A device that is part (for example a component) of the present device.
     */
    hasPart?: DeviceDefinitionHasPart[] | undefined;
    /**
     * Unique instance identifiers assigned to a device by the software, manufacturers, other organizations or owners. For example: handle ID. The identifier is typically valued if the udiDeviceIdentifier, partNumber or modelNumber is not valued and represents a different type of identifier.  However, it is permissible to still include those identifiers in DeviceDefinition.identifier with the appropriate identifier.type.
     */
    identifier?: Identifier[] | undefined;
    /**
     * Language code for the human-readable text strings produced by the device (all supported).
     */
    languageCode?: CodeableConcept[] | undefined;
    /**
     * An associated device, attached to, used with, communicating with or linking a previous or new device model to the focal device.
     */
    link?: DeviceDefinitionLink[] | undefined;
    /**
     * A name of the manufacturer  or legal representative e.g. labeler. Whether this is the actual manufacturer or the labeler or responsible depends on implementation and jurisdiction.
     */
    manufacturer?: Reference | undefined;
    /**
     * A substance used to create the material(s) of which the device is made.
     */
    material?: DeviceDefinitionMaterial[] | undefined;
    /**
     * The model number for the device for example as defined by the manufacturer or labeler, or other agency.
     */
    modelNumber?: string | undefined;
    _modelNumber?: Element | undefined;
    /**
     * Descriptive information, usage information or implantation information that is not captured in an existing element.
     */
    note?: Annotation[] | undefined;
    /**
     * An organization that is responsible for the provision and ongoing maintenance of the device.
     */
    owner?: Reference | undefined;
    /**
     * Information about the packaging of the device, i.e. how the device is packaged.
     */
    packaging?: DeviceDefinitionPackaging[] | undefined;
    /**
     * The parent device it can be part of.
     */
    parentDevice?: Reference | undefined;
    /**
     * Alphanumeric Maximum 20.
     */
    partNumber?: string | undefined;
    _partNumber?: Element | undefined;
    /**
     * Indicates the production identifier(s) that are expected to appear in the UDI carrier on the device label.
     */
    productionIdentifierInUDI?:
        | (
              | 'lot-number'
              | 'manufactured-date'
              | 'serial-number'
              | 'expiration-date'
              | 'biological-source'
              | 'software-version'
          )[]
        | undefined;
    _productionIdentifierInUDI?: Element[] | undefined;
    /**
     * The potential, valid configuration settings of a device, e.g., regulation status, time properties.
     */
    property?: DeviceDefinitionProperty[] | undefined;
    /**
     * This should not be used for regulatory authorization numbers which are to be captured elsewhere.
     */
    regulatoryIdentifier?: DeviceDefinitionRegulatoryIdentifier[] | undefined;
    /**
     * Safety characteristics of the device.
     */
    safety?: CodeableConcept[] | undefined;
    /**
     * Shelf Life and storage information.
     */
    shelfLifeStorage?: ProductShelfLife[] | undefined;
    /**
     * The capabilities supported on a  device, the standards to which the device conforms for a particular purpose, and used for the communication.
     */
    specialization?: RelatedArtifact[] | undefined;
    /**
     * Unique device identifier (UDI) assigned to device label or package.  Note that the Device may include multiple udiCarriers as it either may include just the udiCarrier for the jurisdiction it is sold, or for multiple jurisdictions it could have been sold.
     */
    udiDeviceIdentifier?: DeviceDefinitionUdiDeviceIdentifier[] | undefined;
    /**
     * The version of the device or software.
     */
    version?: DeviceDefinitionVersion[] | undefined;
}
/**
 * Indicates who or what performed the event.
 */
export interface DeviceDispensePerformer extends BackboneElement {
    /**
     * The device, practitioner, etc. who performed the action.  It should be assumed that the actor is the dispenser of the device.
     */
    actor: Reference;
    /**
     * Distinguishes the type of performer in the dispense.  For example, date enterer, packager, final checker.
     */
    function?: CodeableConcept | undefined;
}
/**
 * A record of dispensation of a device.
 */
export interface DeviceDispense extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'DeviceDispense';
    /**
     * The order or request that this dispense is fulfilling.
     */
    basedOn?: Reference[] | undefined;
    /**
     * The category can be used to include where the device is expected to be consumed or other types of dispenses.  Invariants can be used to bind to different value sets when profiling to bind.
     */
    category?: CodeableConcept[] | undefined;
    /**
     * Identification of the facility/location where the device was /should be shipped to, as part of the dispense process.
     */
    destination?: Reference | undefined;
    /**
     * Identifies the device being dispensed. This is either a link to a resource representing the details of the device or a simple attribute carrying a code that identifies the device from a known list of devices.
     */
    device: CodeableReference;
    /**
     * The encounter that establishes the context for this event.
     */
    encounter?: Reference | undefined;
    /**
     * This might not include provenances for all versions of the request – only those deemed “relevant” or important. This SHALL NOT include the Provenance associated with this current version of the resource. (If that provenance is deemed to be a “relevant” change, it will need to be added as part of a later update. Until then, it can be queried directly as the Provenance that points to this version using _revinclude All Provenances should have some historical version of this Request as their subject.).
     */
    eventHistory?: Reference[] | undefined;
    /**
     * This is a business identifier, not a resource identifier.
     */
    identifier?: Identifier[] | undefined;
    /**
     * The principal physical location where the dispense was performed.
     */
    location?: Reference | undefined;
    /**
     * Extra information about the dispense that could not be conveyed in the other attributes.
     */
    note?: Annotation[] | undefined;
    /**
     * The bigger event that this dispense is a part of.
     */
    partOf?: Reference[] | undefined;
    /**
     * Indicates who or what performed the event.
     */
    performer?: DeviceDispensePerformer[] | undefined;
    /**
     * The time when the dispensed product was packaged and reviewed.
     */
    preparedDate?: string | undefined;
    _preparedDate?: Element | undefined;
    /**
     * The number of devices that have been dispensed.
     */
    quantity?: Quantity | undefined;
    /**
     * This element is labeled as a modifier because the status contains codes that mark the resource as not currently valid.
     */
    status:
        | 'preparation'
        | 'in-progress'
        | 'cancelled'
        | 'on-hold'
        | 'completed'
        | 'entered-in-error'
        | 'stopped'
        | 'declined'
        | 'unknown';
    _status?: Element | undefined;
    /**
     * Indicates the reason why a dispense was or was not performed.
     */
    statusReason?: CodeableReference | undefined;
    /**
     * What about 'this measuring Device is now asigned to Dr X who needs it now'?
     */
    subject: Reference;
    /**
     * Additional information that supports the device being dispensed.
     */
    supportingInformation?: Reference[] | undefined;
    /**
     * Indicates the type of dispensing event that is performed.
     */
    type?: CodeableConcept | undefined;
    /**
     * The content of the renderedDosageInstructions must not be different than the dose represented in the dosageInstruction content.
     */
    usageInstruction?: string | undefined;
    _usageInstruction?: Element | undefined;
    /**
     * The time the dispensed product was made available to the patient or their representative.
     */
    whenHandedOver?: string | undefined;
    _whenHandedOver?: Element | undefined;
}
/**
 * Describes the calibrations that have been performed or that are required to be performed.
 */
export interface DeviceMetricCalibration extends BackboneElement {
    /**
     * Describes the state of the calibration.
     */
    state?: ('not-calibrated' | 'calibration-required' | 'calibrated' | 'unspecified') | undefined;
    _state?: Element | undefined;
    /**
     * Describes the time last calibration has been performed.
     */
    time?: string | undefined;
    _time?: Element | undefined;
    /**
     * Describes the type of the calibration method.
     */
    type?: ('unspecified' | 'offset' | 'gain' | 'two-point') | undefined;
    _type?: Element | undefined;
}
/**
 * Describes a measurement, calculation or setting capability of a medical device.
 */
export interface DeviceMetric extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'DeviceMetric';
    /**
     * Describes the calibrations that have been performed or that are required to be performed.
     */
    calibration?: DeviceMetricCalibration[] | undefined;
    /**
     * Indicates the category of the observation generation process. A DeviceMetric can be for example a setting, measurement, or calculation.
     */
    category: 'measurement' | 'setting' | 'calculation' | 'unspecified';
    _category?: Element | undefined;
    /**
     * Describes the color representation for the metric. This is often used to aid clinicians to track and identify parameter types by color. In practice, consider a Patient Monitor that has ECG/HR and Pleth for example; the parameters are displayed in different characteristic colors, such as HR-blue, BP-green, and PR and SpO2- magenta.
     */
    color?: ('black' | 'red' | 'green' | 'yellow' | 'blue' | 'magenta' | 'cyan' | 'white') | undefined;
    _color?: Element | undefined;
    /**
     * For identifiers assigned to a device by the device or gateway software, the `system` element of the identifier should be set to the unique identifier of the device.
     */
    identifier?: Identifier[] | undefined;
    /**
     * Describes the measurement repetition time. This is not necessarily the same as the update period. The measurement repetition time can range from milliseconds up to hours. An example for a measurement repetition time in the range of milliseconds is the sampling rate of an ECG. An example for a measurement repetition time in the range of hours is a NIBP that is triggered automatically every hour. The update period may be different than the measurement repetition time, if the device does not update the published observed value with the same frequency as it was measured.
     */
    measurementPeriod?: Timing | undefined;
    /**
     * Indicates current operational state of the device. For example: On, Off, Standby, etc.
     */
    operationalStatus?: ('on' | 'off' | 'standby' | 'entered-in-error') | undefined;
    _operationalStatus?: Element | undefined;
    /**
     * Describes the link to the  Device that this DeviceMetric belongs to and that provide information about the location of this DeviceMetric in the containment structure of the parent Device. An example would be a Device that represents a Channel. This reference can be used by a client application to distinguish DeviceMetrics that have the same type, but should be interpreted based on their containment location.
     */
    parent?: Reference | undefined;
    /**
     * Describes the link to the  Device that this DeviceMetric belongs to and that contains administrative device information such as manufacturer, serial number, etc.
     */
    source?: Reference | undefined;
    /**
     * DeviceMetric.type can be referred to either IEEE 11073-10101 or LOINC.
     */
    type: CodeableConcept;
    /**
     * DeviceMetric.unit can refer to either UCUM or preferable a RTMMS coding system.
     */
    unit?: CodeableConcept | undefined;
}
/**
 * Specific parameters for the ordered item.  For example, the prism value for lenses.
 */
export interface DeviceRequestParameter extends BackboneElement {
    /**
     * A code or string that identifies the device detail being asserted.
     */
    code?: CodeableConcept | undefined;
    /**
     * Range means device should have a value that falls somewhere within the specified range.
     */
    valueCodeableConcept?: CodeableConcept | undefined;
    /**
     * Range means device should have a value that falls somewhere within the specified range.
     */
    valueQuantity?: Quantity | undefined;
    /**
     * Range means device should have a value that falls somewhere within the specified range.
     */
    valueRange?: Range | undefined;
    /**
     * Range means device should have a value that falls somewhere within the specified range.
     */
    valueBoolean?: boolean | undefined;
    _valueBoolean?: Element | undefined;
}
/**
 * Represents a request for a patient to employ a medical device. The device may be an implantable device, or an external assistive device, such as a walker.
 */
export interface DeviceRequest extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'DeviceRequest';
    /**
     * This status is to indicate whether the request is a PRN or may be given as needed.
     */
    asNeeded?: boolean | undefined;
    _asNeeded?: Element | undefined;
    /**
     * The reason for using the device.
     */
    asNeededFor?: CodeableConcept | undefined;
    /**
     * When the request transitioned to being actionable.
     */
    authoredOn?: string | undefined;
    _authoredOn?: Element | undefined;
    /**
     * Plan/proposal/order fulfilled by this request.
     */
    basedOn?: Reference[] | undefined;
    /**
     * The details of the device to be used.
     */
    code: CodeableReference;
    /**
     * If do not perform is not specified, the request is a positive request e.g. "do perform". DeviceRequest.reasonCode includes the reason for marking the DeviceRequest as 'do not perform'.
     */
    doNotPerform?: boolean | undefined;
    _doNotPerform?: Element | undefined;
    /**
     * An encounter that provides additional context in which this request is made.
     */
    encounter?: Reference | undefined;
    /**
     * Composite request this is part of.
     */
    groupIdentifier?: Identifier | undefined;
    /**
     * Identifiers assigned to this order by the orderer or by the receiver.
     */
    identifier?: Identifier[] | undefined;
    /**
     * Note: This is a business identifier, not a resource identifier (see [discussion](resource.html#identifiers)).  It is best practice for the identifier to only appear on a single resource instance, however business practices may occasionally dictate that multiple resource instances with the same identifier can exist - possibly even with different resource types.  For example, multiple Patient and a Person resource instance might share the same social insurance number.
     */
    instantiatesCanonical?: string[] | undefined;
    _instantiatesCanonical?: Element[] | undefined;
    /**
     * This might be an HTML page, PDF, etc. or could just be a non-resolvable URI identifier.
     */
    instantiatesUri?: string[] | undefined;
    _instantiatesUri?: Element[] | undefined;
    /**
     * Insurance plans, coverage extensions, pre-authorizations and/or pre-determinations that may be required for delivering the requested service.
     */
    insurance?: Reference[] | undefined;
    /**
     * Whether the request is a proposal, plan, an original order or a reflex order.
     */
    intent:
        | 'proposal'
        | 'plan'
        | 'directive'
        | 'order'
        | 'original-order'
        | 'reflex-order'
        | 'filler-order'
        | 'instance-order'
        | 'option';
    _intent?: Element | undefined;
    /**
     * Details about this request that were not represented at all or sufficiently in one of the attributes provided in a class. These may include for example a comment, an instruction, or a note associated with the statement.
     */
    note?: Annotation[] | undefined;
    /**
     * The timing schedule for the use of the device. The Schedule data type allows many different expressions, for example. "Every 8 hours"; "Three times a day"; "1/2 an hour before breakfast for 10 days from 23-Dec 2011:"; "15 Oct 2013, 17 Oct 2013 and 1 Nov 2013".
     */
    occurrenceDateTime?: string | undefined;
    _occurrenceDateTime?: Element | undefined;
    /**
     * The timing schedule for the use of the device. The Schedule data type allows many different expressions, for example. "Every 8 hours"; "Three times a day"; "1/2 an hour before breakfast for 10 days from 23-Dec 2011:"; "15 Oct 2013, 17 Oct 2013 and 1 Nov 2013".
     */
    occurrencePeriod?: Period | undefined;
    /**
     * The timing schedule for the use of the device. The Schedule data type allows many different expressions, for example. "Every 8 hours"; "Three times a day"; "1/2 an hour before breakfast for 10 days from 23-Dec 2011:"; "15 Oct 2013, 17 Oct 2013 and 1 Nov 2013".
     */
    occurrenceTiming?: Timing | undefined;
    /**
     * Specific parameters for the ordered item.  For example, the prism value for lenses.
     */
    parameter?: DeviceRequestParameter[] | undefined;
    /**
     * The desired individual or entity to provide the device to the subject of the request (e.g., patient, location).
     */
    performer?: Reference | undefined;
    /**
     * The desired kind of individual or entity to provide the device to the subject of the request (e.g., patient, location).
     */
    performerType?: CodeableConcept | undefined;
    /**
     * Indicates how quickly the request should be addressed with respect to other requests.
     */
    priority?: ('routine' | 'urgent' | 'asap' | 'stat') | undefined;
    _priority?: Element | undefined;
    /**
     * The number of devices to be provided.
     */
    quantity?: number | undefined;
    /**
     * Reason or justification for the use of this device.
     */
    reason?: CodeableReference[] | undefined;
    /**
     * This might not include provenances for all versions of the request - only those deemed "relevant" or important.
     * This SHALL NOT include the Provenance associated with this current version of the resource.  (If that provenance is deemed to be a "relevant" change, it will need to be added as part of a later update.  Until then, it can be queried directly as the Provenance that points to this version using _revinclude
     * All Provenances should have some historical version of this Request as their subject.
     */
    relevantHistory?: Reference[] | undefined;
    /**
     * The request takes the place of the referenced completed or terminated request(s).
     */
    replaces?: Reference[] | undefined;
    /**
     * The individual or entity who initiated the request and has responsibility for its activation.
     */
    requester?: Reference | undefined;
    /**
     * This element is labeled as a modifier because the status contains the codes cancelled and entered-in-error that mark the request as not currently valid.
     */
    status?: ('draft' | 'active' | 'on-hold' | 'revoked' | 'completed' | 'entered-in-error' | 'unknown') | undefined;
    _status?: Element | undefined;
    /**
     * The patient who will use the device.
     */
    subject: Reference;
    /**
     * Additional clinical information about the patient that may influence the request fulfilment.  For example, this may include where on the subject's body the device will be used (i.e. the target site).
     */
    supportingInfo?: Reference[] | undefined;
}
/**
 * This indicates how or if the device is being used.
 */
export interface DeviceUsageAdherence extends BackboneElement {
    /**
     * Type of adherence.
     */
    code: CodeableConcept;
    /**
     * Reason for adherence type.
     */
    reason: CodeableConcept[];
}
/**
 * A record of a device being used by a patient where the record is the result of a report from the patient or a clinician.
 */
export interface DeviceUsage extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'DeviceUsage';
    /**
     * This indicates how or if the device is being used.
     */
    adherence?: DeviceUsageAdherence | undefined;
    /**
     * A plan, proposal or order that is fulfilled in whole or in part by this DeviceUsage.
     */
    basedOn?: Reference[] | undefined;
    /**
     * Indicates the anotomic location on the subject's body where the device was used ( i.e. the target).
     */
    bodySite?: CodeableReference | undefined;
    /**
     * This attribute indicates a category for the statement - The device statement may be made in an inpatient or outpatient settting (inpatient | outpatient | community | patientspecified).
     */
    category?: CodeableConcept[] | undefined;
    /**
     * The encounter or episode of care that establishes the context for this device use statement.
     */
    context?: Reference | undefined;
    /**
     * The time at which the statement was recorded by informationSource.
     */
    dateAsserted?: string | undefined;
    _dateAsserted?: Element | undefined;
    /**
     * The most common use cases for deriving a DeviceUsage comes from creating it from a request or from an observation or a claim. it should be noted that the amount of information that is available varies from the type resource that you derive the DeviceUsage from.
     */
    derivedFrom?: Reference[] | undefined;
    /**
     * Code or Reference to device used.
     */
    device: CodeableReference;
    /**
     * An external identifier for this statement such as an IRI.
     */
    identifier?: Identifier[] | undefined;
    /**
     * Who reported the device was being used by the patient.
     */
    informationSource?: Reference | undefined;
    /**
     * Details about the device statement that were not represented at all or sufficiently in one of the attributes provided in a class. These may include for example a comment, an instruction, or a note associated with the statement.
     */
    note?: Annotation[] | undefined;
    /**
     * The patient who used the device.
     */
    patient: Reference;
    /**
     * When the status is not done, the reason code indicates why it was not done.
     */
    reason?: CodeableReference[] | undefined;
    /**
     * DeviceUseStatment is a statement at a point in time.  The status is only representative at the point when it was asserted.  The value set for contains codes that assert the status of the use  by the patient (for example, stopped or on hold) as well as codes that assert the status of the resource itself (for example, entered in error).
     * This element is labeled as a modifier because the status contains the codes that mark the statement as not currently valid.
     */
    status: 'active' | 'completed' | 'not-done' | 'entered-in-error' | 'intended' | 'stopped' | 'on-hold';
    _status?: Element | undefined;
    /**
     * How often the device was used.
     */
    timingTiming?: Timing | undefined;
    /**
     * How often the device was used.
     */
    timingPeriod?: Period | undefined;
    /**
     * How often the device was used.
     */
    timingDateTime?: string | undefined;
    _timingDateTime?: Element | undefined;
    /**
     * The reason for asserting the usage status - for example forgot, lost, stolen, broken.
     */
    usageReason?: CodeableConcept[] | undefined;
    /**
     * The status of the device usage, for example always, sometimes, never. This is not the same as the status of the statement.
     */
    usageStatus?: CodeableConcept | undefined;
}
/**
 * This backbone element contains supporting information that was used in the creation of the report not included in the results already included in the report.
 */
export interface DiagnosticReportSupportingInfo extends BackboneElement {
    /**
     * The reference for the supporting information in the diagnostic report.
     */
    reference: Reference;
    /**
     * The code value for the role of the supporting information in the diagnostic report.
     */
    type: CodeableConcept;
}
/**
 * A list of key images or data associated with this report. The images or data are generally created during the diagnostic process, and may be directly of the patient, or of treated specimens (i.e. slides of interest).
 */
export interface DiagnosticReportMedia extends BackboneElement {
    /**
     * The comment should be displayed with the image or data. It would be common for the report to include additional discussion of the image or data contents or in other sections such as the conclusion.
     */
    comment?: string | undefined;
    _comment?: Element | undefined;
    /**
     * Reference to the image or data source.
     */
    link: Reference;
}
/**
 * The findings and interpretation of diagnostic tests performed on patients, groups of patients, products, substances, devices, and locations, and/or specimens derived from these. The report includes clinical context such as requesting provider information, and some mix of atomic results, images, textual and coded interpretations, and formatted representation of diagnostic reports. The report also includes non-clinical context such as batch analysis and stability reporting of products and substances.
 */
export interface DiagnosticReport extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'DiagnosticReport';
    /**
     * Note: Usually there is one test request for each result, however in some circumstances multiple test requests may be represented using a single test result resource. Note that there are also cases where one request leads to multiple reports.
     */
    basedOn?: Reference[] | undefined;
    /**
     * Multiple categories are allowed using various categorization schemes.   The level of granularity is defined by the category concepts in the value set. More fine-grained filtering can be performed using the metadata and/or terminology hierarchy in DiagnosticReport.code.
     */
    category?: CodeableConcept[] | undefined;
    /**
     * A code or name that describes this diagnostic report.
     */
    code: CodeableConcept;
    /**
     * The Composition provides structure to the content of the DiagnosticReport (and only contains contents referenced in the DiagnosticReport) - e.g., to order the sections of a anatomic pathology structured report.
     */
    composition?: Reference | undefined;
    /**
     * Concise and clinically contextualized summary conclusion (interpretation/impression) of the diagnostic report.
     */
    conclusion?: string | undefined;
    _conclusion?: Element | undefined;
    /**
     * One or more codes that represent the summary conclusion (interpretation/impression) of the diagnostic report.
     */
    conclusionCode?: CodeableConcept[] | undefined;
    /**
     * If the diagnostic procedure was performed on the patient, this is the time it was performed. If there are specimens, the diagnostically relevant time can be derived from the specimen collection times, but the specimen information is not always available, and the exact relationship between the specimens and the diagnostically relevant time is not always automatic.
     */
    effectiveDateTime?: string | undefined;
    _effectiveDateTime?: Element | undefined;
    /**
     * If the diagnostic procedure was performed on the patient, this is the time it was performed. If there are specimens, the diagnostically relevant time can be derived from the specimen collection times, but the specimen information is not always available, and the exact relationship between the specimens and the diagnostically relevant time is not always automatic.
     */
    effectivePeriod?: Period | undefined;
    /**
     * This will typically be the encounter the event occurred within, but some events may be initiated prior to or after the official completion of an encounter  but still be tied to the context of the encounter  (e.g. pre-admission laboratory tests).
     */
    encounter?: Reference | undefined;
    /**
     * Usually assigned by the Information System of the diagnostic service provider (filler id).
     */
    identifier?: Identifier[] | undefined;
    /**
     * May be different from the update time of the resource itself, because that is the status of the record (potentially a secondary copy), not the actual release time of the report.
     */
    issued?: string | undefined;
    _issued?: Element | undefined;
    /**
     * A list of key images or data associated with this report. The images or data are generally created during the diagnostic process, and may be directly of the patient, or of treated specimens (i.e. slides of interest).
     */
    media?: DiagnosticReportMedia[] | undefined;
    /**
     * May include general statements about the diagnostic report, or statements about significant, unexpected or unreliable results values contained within the diagnostic report, or information about its source when relevant to its interpretation.
     */
    note?: Annotation[] | undefined;
    /**
     * This is not necessarily the source of the atomic data items or the entity that interpreted the results. It is the entity that takes responsibility for the clinical report.
     */
    performer?: Reference[] | undefined;
    /**
     * "application/pdf" is recommended as the most reliable and interoperable in this context.
     */
    presentedForm?: Attachment[] | undefined;
    /**
     * Observations can contain observations.
     */
    result?: Reference[] | undefined;
    /**
     * Might not be the same entity that takes responsibility for the clinical report.
     */
    resultsInterpreter?: Reference[] | undefined;
    /**
     * If the specimen is sufficiently specified with a code in the test result name, then this additional data may be redundant. If there are multiple specimens, these may be represented per observation or group.
     */
    specimen?: Reference[] | undefined;
    /**
     * The status of the diagnostic report.
     */
    status:
        | 'registered'
        | 'partial'
        | 'preliminary'
        | 'final'
        | 'amended'
        | 'corrected'
        | 'appended'
        | 'cancelled'
        | 'entered-in-error'
        | 'unknown';
    _status?: Element | undefined;
    /**
     * For laboratory-type studies like GenomeStudy, type resources will be used for tracking additional metadata and workflow aspects of complex studies. ImagingStudy and the media element are somewhat overlapping - typically, the list of image references in the media element will also be found in one of the imaging study resources. However, each caters to different types of displays for different types of purposes. Neither, either, or both may be provided.
     */
    study?: Reference[] | undefined;
    /**
     * The subject of the report. Usually, but not always, this is a patient. However, diagnostic services also perform analyses on specimens collected from a variety of other sources.
     */
    subject?: Reference | undefined;
    /**
     * This backbone element contains supporting information that was used in the creation of the report not included in the results already included in the report.
     */
    supportingInfo?: DiagnosticReportSupportingInfo[] | undefined;
}
/**
 * May be identifiers or resources that caused the DocumentManifest to be created.
 */
export interface DocumentManifestRelated extends BackboneElement {
    /**
     * If both identifier and ref elements are present they shall refer to the same thing.
     */
    identifier?: Identifier | undefined;
    /**
     * If both identifier and ref elements are present they shall refer to the same thing.
     */
    ref?: Reference | undefined;
}
/**
 * A collection of documents compiled for a purpose together with metadata that applies to the collection.
 */
export interface DocumentManifest extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'DocumentManifest';
    /**
     * Not necessarily who did the actual data entry (i.e. typist) or who was the source (informant).
     */
    author?: Reference[] | undefined;
    /**
     * When used for XDS the intended focus of the DocumentManifest is for the reference to target to be a set of DocumentReference Resources. The reference is to "Any" to support EN 13606 usage, where an extract is DocumentManifest that references  List and Composition resources.
     */
    content: Reference[];
    /**
     * Creation time is used for tracking, organizing versions and searching. This is the creation time of the document set, not the documents on which it is based.
     */
    created?: string | undefined;
    _created?: Element | undefined;
    /**
     * What the document is about, rather than a terse summary of the document. It is commonly the case that records do not have a title and are collectively referred to by the display name of Record code (e.g. a "consultation" or "progress note").
     */
    description?: string | undefined;
    _description?: Element | undefined;
    /**
     * Other identifiers associated with the document manifest, including version independent  identifiers.
     */
    identifier?: Identifier[] | undefined;
    /**
     * A single identifier that uniquely identifies this manifest. Principally used to refer to the manifest in non-FHIR contexts.
     */
    masterIdentifier?: Identifier | undefined;
    /**
     * How the recipient receives the document set or is notified of it is up to the implementation. This element is just a statement of intent. If the recipient is a person, and it is not known whether the person is a patient or a practitioner, RelatedPerson would be the default choice.
     */
    recipient?: Reference[] | undefined;
    /**
     * May be identifiers or resources that caused the DocumentManifest to be created.
     */
    related?: DocumentManifestRelated[] | undefined;
    /**
     * Identifies the source system, application, or software that produced the document manifest.
     */
    source?: string | undefined;
    _source?: Element | undefined;
    /**
     * This element is labeled as a modifier because the status contains the codes that mark the manifest as not currently valid.
     */
    status: 'current' | 'superseded' | 'entered-in-error';
    _status?: Element | undefined;
    /**
     * Who or what the set of documents is about. The documents can be about a person, (patient or healthcare practitioner), a device (i.e. machine) or even a group of subjects (such as a document about a herd of farm animals, or a set of patients that share a common exposure). If the documents cross more than one subject, then more than one subject is allowed here (unusual use case).
     */
    subject?: Reference | undefined;
    /**
     * Specifies the kind of this set of documents (e.g. Patient Summary, Discharge Summary, Prescription, etc.). The type of a set of documents may be the same as one of the documents in it - especially if there is only one - but it may be wider.
     */
    type?: CodeableConcept | undefined;
}
/**
 * Only list each attester once.
 */
export interface DocumentReferenceAttester extends BackboneElement {
    /**
     * The type of attestation the authenticator offers.
     */
    mode: CodeableConcept;
    /**
     * Who attested the document in the specified way.
     */
    party?: Reference | undefined;
    /**
     * When the document was attested by the party.
     */
    time?: string | undefined;
    _time?: Element | undefined;
}
/**
 * This element is labeled as a modifier because documents that append to other documents are incomplete on their own.
 */
export interface DocumentReferenceRelatesTo extends BackboneElement {
    /**
     * If this document appends another document, then the document cannot be fully understood without also accessing the referenced document.
     */
    code: CodeableConcept;
    /**
     * The target document of this relationship.
     */
    target: Reference;
}
/**
 * Note that IHE often issues URNs for formatCode codes, not all documents can be identified by a URI.
 * For FHIR content, .profile should indicate the structureDefinition profile canonical URI(s) that the content complies with.
 */
export interface DocumentReferenceContentProfile extends BackboneElement {
    /**
     * Code|uri|canonical.
     */
    valueCoding?: Coding | undefined;
    /**
     * Code|uri|canonical.
     */
    valueUri?: string | undefined;
    _valueUri?: Element | undefined;
    /**
     * Code|uri|canonical.
     */
    valueCanonical?: string | undefined;
    _valueCanonical?: Element | undefined;
}
/**
 * content element shall not contain different versions of the same content. For version handling use multiple DocumentReference with .relatesTo.
 */
export interface DocumentReferenceContent extends BackboneElement {
    /**
     * The document or URL of the document along with critical metadata to prove content has integrity.
     */
    attachment: Attachment;
    /**
     * Note that IHE often issues URNs for formatCode codes, not all documents can be identified by a URI.
     * For FHIR content, .profile should indicate the structureDefinition profile canonical URI(s) that the content complies with.
     */
    profile?: DocumentReferenceContentProfile[] | undefined;
}
/**
 * A reference to a document of any kind for any purpose. While the term “document” implies a more narrow focus, for this resource this "document" encompasses *any* serialized object with a mime-type, it includes formal patient-centric documents (CDA), clinical notes, scanned paper, non-patient specific documents like policy text, as well as a photo, video, or audio recording acquired or used in healthcare.  The DocumentReference resource provides metadata about the document so that the document can be discovered and managed.  The actual content may be inline base64 encoded data or provided by direct reference.
 */
export interface DocumentReference extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'DocumentReference';
    /**
     * Only list each attester once.
     */
    attester?: DocumentReferenceAttester[] | undefined;
    /**
     * Not necessarily who did the actual data entry (i.e. typist) or who was the source (informant).
     */
    author?: Reference[] | undefined;
    /**
     * A procedure that is fulfilled in whole or in part by the creation of this media.
     */
    basedOn?: Reference[] | undefined;
    /**
     * Key metadata element describing the the category or classification of the document. This is a broader perspective that groups similar documents based on how they would be used. This is a primary key used in searching.
     */
    category?: CodeableConcept[] | undefined;
    /**
     * content element shall not contain different versions of the same content. For version handling use multiple DocumentReference with .relatesTo.
     */
    content: DocumentReferenceContent[];
    /**
     * Describes the clinical encounter or type of care that the document content is associated with.
     */
    context?: Reference[] | undefined;
    /**
     * Identifies the logical organization (software system, vendor, or department) to go to find the current version, where to report issues, etc. This is different from the physical location (URL, disk drive, or server) of the document, which is the technical location of the document, which host may be delegated to the management of some other organization.
     */
    custodian?: Reference | undefined;
    /**
     * Referencing/indexing time is used for tracking, organizing versions and searching.
     */
    date?: string | undefined;
    _date?: Element | undefined;
    /**
     * What the document is about,  a terse summary of the document.
     */
    description?: string | undefined;
    _description?: Element | undefined;
    /**
     * The document that is pointed to might be in various lifecycle states.
     */
    docStatus?:
        | (
              | 'registered'
              | 'partial'
              | 'preliminary'
              | 'final'
              | 'amended'
              | 'corrected'
              | 'appended'
              | 'cancelled'
              | 'entered-in-error'
              | 'deprecated'
              | 'unknown'
          )
        | undefined;
    _docStatus?: Element | undefined;
    /**
     * An event can further specialize the act inherent in the type, such as  where it is simply "Procedure Report" and the procedure was a "colonoscopy". If one or more event codes are included, they shall not conflict with the values inherent in the class or type elements as such a conflict would create an ambiguous situation.
     */
    event?: CodeableReference[] | undefined;
    /**
     * The kind of facility where the patient was seen.
     */
    facilityType?: CodeableConcept | undefined;
    /**
     * The structure and format of this identifier would be consistent with the specification corresponding to the format of the document. (e.g. for a DICOM standard document, a 64-character numeric UID; for an HL7 CDA format, the CDA Document Id root and extension).
     */
    identifier?: Identifier[] | undefined;
    /**
     * The time period over which the service that is described by the document was provided.
     */
    period?: Period | undefined;
    /**
     * This element should be based on a coarse classification system for the class of specialty practice. Recommend the use of the classification system for Practice Setting, such as that described by the Subject Matter Domain in LOINC.
     */
    practiceSetting?: CodeableConcept | undefined;
    /**
     * This element is labeled as a modifier because documents that append to other documents are incomplete on their own.
     */
    relatesTo?: DocumentReferenceRelatesTo[] | undefined;
    /**
     * The confidentiality codes can carry multiple vocabulary items. HL7 has developed an understanding of security and privacy tags that might be desirable in a Document Sharing environment, called HL7 Healthcare Privacy and Security Classification System (HCS). The following specification is recommended but not mandated, as the vocabulary bindings are an administrative domain responsibility. The use of this method is up to the policy domain such as the XDS Affinity Domain or other Trust Domain where all parties including sender and recipients are trusted to appropriately tag and enforce.
     * In the HL7 Healthcare Privacy and Security Classification (HCS) there are code systems specific to Confidentiality, Sensitivity, Integrity, and Handling Caveats. Some values would come from a local vocabulary as they are related to workflow roles and special projects.
     */
    securityLabel?: CodeableConcept[] | undefined;
    /**
     * This is the status of the DocumentReference object, which might be independent from the docStatus element.
     * This element is labeled as a modifier because the status contains the codes that mark the document or reference as not currently valid.
     */
    status: 'current' | 'superseded' | 'entered-in-error';
    _status?: Element | undefined;
    /**
     * Who or what the document is about. The document can be about a person, (patient or healthcare practitioner), a device (e.g. a machine) or even a group of subjects (such as a document about a herd of farm animals, or a set of patients that share a common exposure).
     */
    subject?: Reference | undefined;
    /**
     * Key metadata element describing the document that describes he exact type of document. Helps humans to assess whether the document is of interest when viewing a list of documents.
     */
    type?: CodeableConcept | undefined;
}
/**
 * A resource that includes narrative, extensions, and contained resources.
 */
export interface DomainResource extends Resource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: string;
    /**
     * This should never be done when the content can be identified properly, as once identification is lost, it is extremely difficult (and context dependent) to restore it again. Contained resources may have profiles and tags in their meta elements, but SHALL NOT have security labels.
     */
    contained?: FhirResource[] | undefined;
    /**
     * There can be no stigma associated with the use of extensions by any application, project, or standard - regardless of the institution or jurisdiction that uses or defines the extensions.  The use of extensions is what allows the FHIR specification to retain a core level of simplicity for everyone.
     */
    extension?: Extension[] | undefined;
    /**
     * There can be no stigma associated with the use of extensions by any application, project, or standard - regardless of the institution or jurisdiction that uses or defines the extensions.  The use of extensions is what allows the FHIR specification to retain a core level of simplicity for everyone.
     */
    modifierExtension?: Extension[] | undefined;
    /**
     * Contained resources do not have a narrative. Resources that are not contained SHOULD have a narrative. In some cases, a resource may only have text with little or no additional discrete data (as long as all minOccurs=1 elements are satisfied).  This may be necessary for data from legacy systems where information is captured as a "text blob" or where text is additionally entered raw or narrated and encoded information is added later.
     */
    text?: Narrative | undefined;
}
/**
 * The current status is always found in the current version of the resource, not the status history.
 */
export interface EncounterStatusHistory extends BackboneElement {
    /**
     * The time that the episode was in the specified status.
     */
    period: Period;
    /**
     * planned | in-progress | onhold | discharged | completed | cancelled | discontinued | entered-in-error | unknown.
     */
    status:
        | 'planned'
        | 'in-progress'
        | 'onhold'
        | 'discharged'
        | 'completed'
        | 'cancelled'
        | 'discontinued'
        | 'entered-in-error'
        | 'unknown';
    _status?: Element | undefined;
}
/**
 * The class history permits the tracking of the encounters transitions without needing to go  through the resource history.  This would be used for a case where an admission starts of as an emergency encounter, then transitions into an inpatient scenario. Doing this and not restarting a new encounter ensures that any lab/diagnostic results can more easily follow the patient and not require re-processing and not get lost or cancelled during a kind of discharge from emergency to inpatient.
 */
export interface EncounterClassHistory extends BackboneElement {
    /**
     * inpatient | outpatient | ambulatory | emergency +.
     */
    class: Coding;
    /**
     * The time that the episode was in the specified class.
     */
    period: Period;
}
/**
 * Any Patient or Group present in the participation.actor must also be the subject, though the subject may be absent from the participation.actor for cases where the patient (or group) is not present, such as during a case review conference.
 */
export interface EncounterParticipant extends BackboneElement {
    /**
     * For planning purposes, Appointments may include a CareTeam participant to indicate that one specific person from the CareTeam will be assigned, but that assignment might not happen until the Encounter begins. Hence CareTeam is not included in Encounter.participant, as the specific individual should be assigned and represented as a Practitioner or other person resource.
     * Similarly, Location can be included in Appointment.participant to assist with planning.  However, the patient location is tracked on the Encounter in the Encounter.location property to allow for additional metadata and history to be recorded.
     * The role of the participant can be used to declare what the actor will be doing in the scope of this encounter participation.
     * If the individual is not specified during planning, then it is expected that the individual will be filled in at a later stage prior to the encounter commencing.
     */
    actor?: Reference | undefined;
    /**
     * The period of time that the specified participant participated in the encounter. These can overlap or be sub-sets of the overall encounter's period.
     */
    period?: Period | undefined;
    /**
     * The participant type indicates how an individual actor participates in an encounter. It includes non-practitioner participants, and for practitioners this is to describe the action type in the context of this encounter (e.g. Admitting Dr, Attending Dr, Translator, Consulting Dr). This is different to the practitioner roles which are functional roles, derived from terms of employment, education, licensing, etc.
     */
    type?: CodeableConcept[] | undefined;
}
/**
 * The list of diagnosis relevant to this encounter.
 */
export interface EncounterDiagnosis extends BackboneElement {
    /**
     * Reason the encounter takes place, as specified using information from another resource. For admissions, this is the admission diagnosis. The indication will typically be a Condition (with other resources referenced in the evidence.detail), or a Procedure.
     */
    condition: Reference;
    /**
     * Ranking of the diagnosis (for each role type).
     */
    rank?: number | undefined;
    /**
     * Role that this diagnosis has within the encounter (e.g. admission, billing, discharge …).
     */
    use?: CodeableConcept | undefined;
}
/**
 * An Encounter may cover more than just the inpatient stay. Contexts such as outpatients, community clinics, and aged care facilities are also included.
 * The duration recorded in the period of this encounter covers the entire scope of this admission record.
 */
export interface EncounterAdmission extends BackboneElement {
    /**
     * From where patient was admitted (physician referral, transfer).
     */
    admitSource?: CodeableConcept | undefined;
    /**
     * Location/organization to which the patient is discharged.
     */
    destination?: Reference | undefined;
    /**
     * For example, a patient may request both a dairy-free and nut-free diet preference (not mutually exclusive).
     */
    dietPreference?: CodeableConcept[] | undefined;
    /**
     * Category or kind of location after discharge.
     */
    dischargeDisposition?: CodeableConcept | undefined;
    /**
     * The location/organization from which the patient came before admission.
     */
    origin?: Reference | undefined;
    /**
     * Pre-admission identifier.
     */
    preAdmissionIdentifier?: Identifier | undefined;
    /**
     * Whether this admission is a readmission and why if known.
     */
    reAdmission?: CodeableConcept | undefined;
    /**
     * Any special requests that have been made for this admission encounter, such as the provision of specific equipment or other things.
     */
    specialArrangement?: CodeableConcept[] | undefined;
    /**
     * Special courtesies (VIP, board member).
     */
    specialCourtesy?: CodeableConcept[] | undefined;
}
/**
 * Virtual encounters can be recorded in the Encounter by specifying a location reference to a location of type "kind" such as "client's home" and an encounter.class = "virtual".
 */
export interface EncounterLocation extends BackboneElement {
    /**
     * This information is de-normalized from the Location resource to support the easier understanding of the encounter resource and processing in messaging or query.
     * There may be many levels in the hierachy, and this may only pic specific levels that are required for a specific usage scenario.
     */
    form?: CodeableConcept | undefined;
    /**
     * The location where the encounter takes place.
     */
    location: Reference;
    /**
     * Time period during which the patient was present at the location.
     */
    period?: Period | undefined;
    /**
     * When the patient is no longer active at a location, then the period end date is entered, and the status may be changed to completed.
     */
    status?: ('planned' | 'active' | 'reserved' | 'completed') | undefined;
    _status?: Element | undefined;
}
/**
 * An interaction between a patient and healthcare provider(s) for the purpose of providing healthcare service(s) or assessing the health status of a patient.  Encounter is primarily used to record information about the actual activities that occurred, where Appointment is used to record planned activities.
 */
export interface Encounter extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'Encounter';
    /**
     * The billing system may choose to allocate billable items associated with the Encounter to different referenced Accounts based on internal business rules.
     */
    account?: Reference[] | undefined;
    /**
     * If not (yet) known, the end of the Period may be omitted.
     */
    actualPeriod?: Period | undefined;
    /**
     * An Encounter may cover more than just the inpatient stay. Contexts such as outpatients, community clinics, and aged care facilities are also included.
     * The duration recorded in the period of this encounter covers the entire scope of this admission record.
     */
    admission?: EncounterAdmission | undefined;
    /**
     * The appointment that scheduled this encounter.
     */
    appointment?: Reference[] | undefined;
    /**
     * The request this encounter satisfies (e.g. incoming referral or procedure request).
     */
    basedOn?: Reference[] | undefined;
    /**
     * The group(s) of individuals, organizations that are allocated to participate in this encounter. The participants backbone will record the actuals of when these individuals participated during the encounter.
     */
    careTeam?: Reference[] | undefined;
    /**
     * Concepts representing classification of patient encounter such as ambulatory (outpatient), inpatient, emergency, home health or others due to local variations.
     */
    class?: CodeableConcept[] | undefined;
    /**
     * The class history permits the tracking of the encounters transitions without needing to go  through the resource history.  This would be used for a case where an admission starts of as an emergency encounter, then transitions into an inpatient scenario. Doing this and not restarting a new encounter ensures that any lab/diagnostic results can more easily follow the patient and not require re-processing and not get lost or cancelled during a kind of discharge from emergency to inpatient.
     */
    classHistory?: EncounterClassHistory[] | undefined;
    /**
     * The list of diagnosis relevant to this encounter.
     */
    diagnosis?: EncounterDiagnosis[] | undefined;
    /**
     * Where a specific encounter should be classified as a part of a specific episode(s) of care this field should be used. This association can facilitate grouping of related encounters together for a specific purpose, such as government reporting, issue tracking, association via a common problem.  The association is recorded on the encounter as these are typically created after the episode of care and grouped on entry rather than editing the episode of care to append another encounter to it (the episode of care could span years).
     */
    episodeOfCare?: Reference[] | undefined;
    /**
     * Identifier(s) by which this encounter is known.
     */
    identifier?: Identifier[] | undefined;
    /**
     * If the precision on these values is low (e.g. to the day only) then this may be considered was an all day (or multi-day) encounter, unless the duration is included, where that amount of time occurred sometime during the interval.
     * May differ from the time in `Encounter.period` due to leave of absence(s).
     */
    length?: Duration | undefined;
    /**
     * Virtual encounters can be recorded in the Encounter by specifying a location reference to a location of type "kind" such as "client's home" and an encounter.class = "virtual".
     */
    location?: EncounterLocation[] | undefined;
    /**
     * Any Patient or Group present in the participation.actor must also be the subject, though the subject may be absent from the participation.actor for cases where the patient (or group) is not present, such as during a case review conference.
     */
    participant?: EncounterParticipant[] | undefined;
    /**
     * This is also used for associating a child's encounter back to the mother's encounter.
     * Refer to the Notes section in the Patient resource for further details.
     */
    partOf?: Reference | undefined;
    /**
     * The planned end date/time (or discharge date) of the encounter.
     */
    plannedEndDate?: string | undefined;
    _plannedEndDate?: Element | undefined;
    /**
     * The planned start date/time (or admission date) of the encounter.
     */
    plannedStartDate?: string | undefined;
    _plannedStartDate?: Element | undefined;
    /**
     * Indicates the urgency of the encounter.
     */
    priority?: CodeableConcept | undefined;
    /**
     * Reason the encounter takes place, expressed as a code or a reference to another resource. For admissions, this can be used for a coded admission diagnosis.
     */
    reason?: CodeableReference[] | undefined;
    /**
     * The organization that is primarily responsible for this Encounter's services. This MAY be the same as the organization on the Patient record, however it could be different, such as if the actor performing the services was from an external organization (which may be billed seperately) for an external consultation.  Refer to the example bundle showing an abbreviated set of Encounters for a colonoscopy.
     */
    serviceProvider?: Reference | undefined;
    /**
     * Broad categorization of the service that is to be provided (e.g. cardiology).
     */
    serviceType?: CodeableReference[] | undefined;
    /**
     * Note that internal business rules will determine the appropriate transitions that may occur between statuses (and also classes).
     */
    status:
        | 'planned'
        | 'in-progress'
        | 'onhold'
        | 'discharged'
        | 'completed'
        | 'cancelled'
        | 'discontinued'
        | 'entered-in-error'
        | 'unknown';
    _status?: Element | undefined;
    /**
     * The current status is always found in the current version of the resource, not the status history.
     */
    statusHistory?: EncounterStatusHistory[] | undefined;
    /**
     * While the encounter is always about the patient, the patient might not actually be known in all contexts of use, and there may be a group of patients that could be anonymous (such as in a group therapy for Alcoholics Anonymous - where the recording of the encounter could be used for billing on the number of people/staff and not important to the context of the specific patients) or alternately in veterinary care a herd of sheep receiving treatment (where the animals are not individually tracked).
     */
    subject?: Reference | undefined;
    /**
     * The subjectStatus value can be used to track the patient's status within the encounter. It details whether the patient has arrived or departed, has been triaged or is currently in a waiting status.
     */
    subjectStatus?: CodeableConcept | undefined;
    /**
     * Since there are many ways to further classify encounters, this element is 0..*.
     */
    type?: CodeableConcept[] | undefined;
    /**
     * There are two types of virtual meetings that often exist:
     * * a persistent, virtual meeting room that can only be used for a single purpose at a time,
     * * and a dynamic virtual meeting room that is generated on demand for a specific purpose.
     * Implementers may consider using Location.virtualService for persistent meeting rooms.
     * If each participant would have a different meeting link, an extension using the VirtualServiceContactDetail  can be applied to the Encounter.participant BackboneElement.
     */
    virtualService?: VirtualServiceDetail[] | undefined;
}
/**
 * The technical details of an endpoint that can be used for electronic services, such as for web services providing XDS.b, a REST endpoint for another FHIR server, or a s/Mime email address. This may include any security context information.
 */
export interface Endpoint extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'Endpoint';
    /**
     * For rest-hook, and websocket, the end-point must be an http: or https: URL; for email, a mailto: url, for sms, a tel: url, and for message the endpoint can be in any form of url the server understands (usually, http: or mllp:). The URI is allowed to be relative; in which case, it is relative to the server end-point (since there may be more than one, clients should avoid using relative URIs)
     * This address will be to the service base, without any parameters, or sub-services or resources tacked on.
     * E.g. for a WADO-RS endpoint, the url should be "https://pacs.hospital.org/wado-rs"
     * and not "https://pacs.hospital.org/wado-rs/studies/1.2.250.1.59.40211.12345678.678910/series/1.2.250.1.59.40211.789001276.14556172.67789/instances/...".
     */
    address: string;
    _address?: Element | undefined;
    /**
     * For additional connectivity details for the protocol, extensions will be used at this point, as in the XDS example. If there are multiple payload types or mimetypes they are all applicable for all connection types, and all have the same status.
     */
    connectionType: CodeableConcept[];
    /**
     * Contact details for a human to contact about the endpoint. The primary use of this for system administrator troubleshooting.
     */
    contact?: ContactPoint[] | undefined;
    /**
     * The description of the endpoint and what it is for (typically used as supplemental information in an endpoint directory describing it's usage/purpose).
     */
    description?: string | undefined;
    _description?: Element | undefined;
    /**
     * The type of environment(s) exposed at this endpoint (dev, prod, test, etc).
     */
    environmentType?: CodeableConcept[] | undefined;
    /**
     * Exactly what these mean depends on the channel type. The can convey additional information to the recipient and/or meet security requirements.
     */
    header?: string[] | undefined;
    _header?: Element[] | undefined;
    /**
     * Identifier for the organization that is used to identify the endpoint across multiple disparate systems.
     */
    identifier?: Identifier[] | undefined;
    /**
     * This property is not typically used when searching for Endpoint resources for usage. The typical usage is via the reference from an applicable Organization/Location/Practitioner resource, which is where the context is provided. Multiple Locations may reference a single endpoint, and don't have to be within the same organization resource, but most likely within the same organizational hierarchy.
     */
    managingOrganization?: Reference | undefined;
    /**
     * A friendly name that this endpoint can be referred to with.
     */
    name?: string | undefined;
    _name?: Element | undefined;
    /**
     * Sending the payload has obvious security consequences. The server is responsible for ensuring that the content is appropriately secured.
     */
    payloadMimeType?: string[] | undefined;
    _payloadMimeType?: Element[] | undefined;
    /**
     * The payloadFormat describes the serialization format of the data, where the payloadType indicates the specific document/schema that is being transferred; e.g. DischargeSummary or CarePlan.
     */
    payloadType?: CodeableConcept[] | undefined;
    /**
     * Temporary downtimes or other short-term changes in availability would not be represented in this property.
     */
    period?: Period | undefined;
    /**
     * This element is labeled as a modifier because the status contains codes that mark the endpoint as not currently valid. Temporary downtimes or other unexpected short-term changes in availability would not be represented in this property.
     */
    status: 'active' | 'suspended' | 'error' | 'off' | 'entered-in-error';
    _status?: Element | undefined;
}
/**
 * This resource provides the insurance enrollment details to the insurer regarding a specified coverage.
 */
export interface EnrollmentRequest extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'EnrollmentRequest';
    /**
     * Patient Resource.
     */
    candidate?: Reference | undefined;
    /**
     * Reference to the program or plan identification, underwriter or payor.
     */
    coverage?: Reference | undefined;
    /**
     * The date when this resource was created.
     */
    created?: string | undefined;
    _created?: Element | undefined;
    /**
     * The Response business identifier.
     */
    identifier?: Identifier[] | undefined;
    /**
     * The Insurer who is target  of the request.
     */
    insurer?: Reference | undefined;
    /**
     * The practitioner who is responsible for the services rendered to the patient.
     */
    provider?: Reference | undefined;
    /**
     * This element is labeled as a modifier because the status contains codes that mark the request as not currently valid.
     */
    status?: ('active' | 'cancelled' | 'draft' | 'entered-in-error') | undefined;
    _status?: Element | undefined;
}
/**
 * This resource provides enrollment and plan details from the processing of an EnrollmentRequest resource.
 */
export interface EnrollmentResponse extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'EnrollmentResponse';
    /**
     * The date when the enclosed suite of services were performed or completed.
     */
    created?: string | undefined;
    _created?: Element | undefined;
    /**
     * A description of the status of the adjudication.
     */
    disposition?: string | undefined;
    _disposition?: Element | undefined;
    /**
     * The Response business identifier.
     */
    identifier?: Identifier[] | undefined;
    /**
     * The Insurer who produced this adjudicated response.
     */
    organization?: Reference | undefined;
    /**
     * Processing status: error, complete.
     */
    outcome?: ('queued' | 'complete' | 'error' | 'partial') | undefined;
    _outcome?: Element | undefined;
    /**
     * Original request resource reference.
     */
    request?: Reference | undefined;
    /**
     * The practitioner who is responsible for the services rendered to the patient.
     */
    requestProvider?: Reference | undefined;
    /**
     * This element is labeled as a modifier because the status contains codes that mark the response as not currently valid.
     */
    status?: ('active' | 'cancelled' | 'draft' | 'entered-in-error') | undefined;
    _status?: Element | undefined;
}
/**
 * The history of statuses that the EpisodeOfCare has been through (without requiring processing the history of the resource).
 */
export interface EpisodeOfCareStatusHistory extends BackboneElement {
    /**
     * The period during this EpisodeOfCare that the specific status applied.
     */
    period: Period;
    /**
     * planned | waitlist | active | onhold | finished | cancelled.
     */
    status: 'planned' | 'waitlist' | 'active' | 'onhold' | 'finished' | 'cancelled' | 'entered-in-error';
    _status?: Element | undefined;
}
/**
 * The list of diagnosis relevant to this episode of care.
 */
export interface EpisodeOfCareDiagnosis extends BackboneElement {
    /**
     * A list of conditions/problems/diagnoses that this episode of care is intended to be providing care for.
     */
    condition: CodeableReference;
    /**
     * Ranking of the diagnosis (for each role type).
     */
    rank?: number | undefined;
    /**
     * Role that this diagnosis has within the episode of care (e.g. admission, billing, discharge …).
     */
    role?: CodeableConcept | undefined;
}
/**
 * An association between a patient and an organization / healthcare provider(s) during which time encounters may occur. The managing organization assumes a level of responsibility for the patient during this time.
 */
export interface EpisodeOfCare extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'EpisodeOfCare';
    /**
     * The billing system may choose to allocate billable items associated with the EpisodeOfCare to different referenced Accounts based on internal business rules.
     */
    account?: Reference[] | undefined;
    /**
     * The practitioner that is the care manager/care coordinator for this patient.
     */
    careManager?: Reference | undefined;
    /**
     * The list of practitioners that may be facilitating this episode of care for specific purposes.
     */
    careTeam?: Reference[] | undefined;
    /**
     * The list of diagnosis relevant to this episode of care.
     */
    diagnosis?: EpisodeOfCareDiagnosis[] | undefined;
    /**
     * The EpisodeOfCare may be known by different identifiers for different contexts of use, such as when an external agency is tracking the Episode for funding purposes.
     */
    identifier?: Identifier[] | undefined;
    /**
     * If multiple organizations are involved in care delivery, the care teams from those organizations would be represented in EpisodeOfCare.team, while the primary organization managing the care would be listed in EpisodeOfCare.managingOrganization.   Other organizations may have their own EpisodeOfCare for tracking their activities.
     */
    managingOrganization?: Reference | undefined;
    /**
     * The patient who is the focus of this episode of care.
     */
    patient: Reference;
    /**
     * The interval during which the managing organization assumes the defined responsibility.
     */
    period?: Period | undefined;
    /**
     * Referral Request(s) that are fulfilled by this EpisodeOfCare, incoming referrals.
     */
    referralRequest?: Reference[] | undefined;
    /**
     * This element is labeled as a modifier because the status contains codes that mark the episode as not currently valid.
     */
    status: 'planned' | 'waitlist' | 'active' | 'onhold' | 'finished' | 'cancelled' | 'entered-in-error';
    _status?: Element | undefined;
    /**
     * The history of statuses that the EpisodeOfCare has been through (without requiring processing the history of the resource).
     */
    statusHistory?: EpisodeOfCareStatusHistory[] | undefined;
    /**
     * The type can be very important in processing as this could be used in determining if the EpisodeOfCare is relevant to specific government reporting, or other types of classifications.
     */
    type?: CodeableConcept[] | undefined;
}
/**
 * The EventDefinition resource provides a reusable description of when a particular event can occur.
 */
export interface EventDefinition extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'EventDefinition';
    /**
     * The 'date' element may be more recent than the approval date because of minor changes or editorial corrections.
     */
    approvalDate?: string | undefined;
    _approvalDate?: Element | undefined;
    /**
     * An individiual or organization primarily involved in the creation and maintenance of the content.
     */
    author?: ContactDetail[] | undefined;
    /**
     * May be a web site, an email address, a telephone number, etc.
     */
    contact?: ContactDetail[] | undefined;
    /**
     * A copyright statement relating to the event definition and/or its contents. Copyright statements are generally legal restrictions on the use and publishing of the event definition.
     */
    copyright?: string | undefined;
    _copyright?: Element | undefined;
    /**
     * Note that this is not the same as the resource last-modified-date, since the resource may be a secondary representation of the event definition. Additional specific dates may be added as extensions or be found by consulting Provenances associated with past versions of the resource.
     */
    date?: string | undefined;
    _date?: Element | undefined;
    /**
     * This description can be used to capture details such as comments about misuse, instructions for clinical use and interpretation, literature references, examples from the paper world, etc. It is not a rendering of the event definition as conveyed in the 'text' field of the resource itself. This item SHOULD be populated unless the information is available from context (e.g. the language of the event definition is presumed to be the predominant language in the place the event definition was created).
     */
    description?: string | undefined;
    _description?: Element | undefined;
    /**
     * An individual or organization primarily responsible for internal coherence of the content.
     */
    editor?: ContactDetail[] | undefined;
    /**
     * The effective period for a event definition  determines when the content is applicable for usage and is independent of publication and review dates. For example, a measure intended to be used for the year 2016 might be published in 2015.
     */
    effectivePeriod?: Period | undefined;
    /**
     * An individual or organization responsible for officially endorsing the content for use in some setting.
     */
    endorser?: ContactDetail[] | undefined;
    /**
     * Allows filtering of event definitions that are appropriate for use versus not.
     */
    experimental?: boolean | undefined;
    _experimental?: Element | undefined;
    /**
     * Typically, this is used for identifiers that can go in an HL7 V3 II (instance identifier) data type, and can then identify this event definition outside of FHIR, where it is not possible to use the logical URI.
     */
    identifier?: Identifier[] | undefined;
    /**
     * It may be possible for the event definition to be used in jurisdictions other than those for which it was originally designed or intended.
     */
    jurisdiction?: CodeableConcept[] | undefined;
    /**
     * If specified, this date follows the original approval date.
     */
    lastReviewDate?: string | undefined;
    _lastReviewDate?: Element | undefined;
    /**
     * The name is not expected to be globally unique. The name should be a simple alphanumeric type name to ensure that it is machine-processing friendly.
     */
    name?: string | undefined;
    _name?: Element | undefined;
    /**
     * Usually an organization but may be an individual. The publisher (or steward) of the event definition is the organization or individual primarily responsible for the maintenance and upkeep of the event definition. This is not necessarily the same individual or organization that developed and initially authored the content. The publisher is the primary point of contact for questions or issues with the event definition. This item SHOULD be populated unless the information is available from context.
     */
    publisher?: string | undefined;
    _publisher?: Element | undefined;
    /**
     * This element does not describe the usage of the event definition. Instead, it provides traceability of ''why'' the resource is either needed or ''why'' it is defined as it is.  This may be used to point to source materials or specifications that drove the structure of this event definition.
     */
    purpose?: string | undefined;
    _purpose?: Element | undefined;
    /**
     * Each related resource is either an attachment, or a reference to another resource, but not both.
     */
    relatedArtifact?: RelatedArtifact[] | undefined;
    /**
     * An individual or organization primarily responsible for review of some aspect of the content.
     */
    reviewer?: ContactDetail[] | undefined;
    /**
     * Allows filtering of event definitions that are appropriate for use versus not.
     */
    status: 'draft' | 'active' | 'retired' | 'unknown';
    _status?: Element | undefined;
    /**
     * A code or group definition that describes the intended subject of the event definition.
     */
    subjectCodeableConcept?: CodeableConcept | undefined;
    /**
     * A code or group definition that describes the intended subject of the event definition.
     */
    subjectReference?: Reference | undefined;
    /**
     * An explanatory or alternate title for the event definition giving additional information about its content.
     */
    subtitle?: string | undefined;
    _subtitle?: Element | undefined;
    /**
     * This name does not need to be machine-processing friendly and may contain punctuation, white-space, etc.
     */
    title?: string | undefined;
    _title?: Element | undefined;
    /**
     * Descriptive topics related to the module. Topics provide a high-level categorization of the module that can be useful for filtering and searching.
     */
    topic?: CodeableConcept[] | undefined;
    /**
     * The trigger element defines when the event occurs. If more than one trigger condition is specified, the event fires whenever any one of the trigger conditions is met.
     */
    trigger: TriggerDefinition[];
    /**
     * Can be a urn:uuid: or a urn:oid: but real http: addresses are preferred.  Multiple instances may share the same URL if they have a distinct version.
     * The determination of when to create a new version of a resource (same url, new version) vs. defining a new artifact is up to the author.  Considerations for making this decision are found in [Technical and Business Versions](resource.html#versions).
     * In some cases, the resource can no longer be found at the stated url, but the url itself cannot change. Implementations can use the [meta.source](resource.html#meta) element to indicate where the current master source of the resource can be found.
     */
    url?: string | undefined;
    _url?: Element | undefined;
    /**
     * A detailed description of how the event definition is used from a clinical perspective.
     */
    usage?: string | undefined;
    _usage?: Element | undefined;
    /**
     * When multiple useContexts are specified, there is no expectation that all or any of the contexts apply.
     */
    useContext?: UsageContext[] | undefined;
    /**
     * There may be different event definition instances that have the same identifier but different versions.  The version can be appended to the url in a reference to allow a reference to a particular business version of the event definition with the format [url]|[version]. The version SHOULD NOT contain a '#' - see [Business Version](resource.html#bv-format).
     */
    version?: string | undefined;
    _version?: Element | undefined;
}
/**
 * Evidence variable such as population, exposure, or outcome.
 */
export interface EvidenceVariableDefinition extends BackboneElement {
    /**
     * A text description or summary of the variable.
     */
    description?: string | undefined;
    _description?: Element | undefined;
    /**
     * Indication of quality of match between intended variable to actual variable.
     */
    directnessMatch?: CodeableConcept | undefined;
    /**
     * Definition of the intended variable related to the Evidence.
     */
    intended?: Reference | undefined;
    /**
     * Footnotes and/or explanatory notes.
     */
    note?: Annotation[] | undefined;
    /**
     * Definition of the actual variable related to the statistic(s).
     */
    observed?: Reference | undefined;
    /**
     * population | subpopulation | exposure | referenceExposure | measuredVariable | confounder.
     */
    variableRole: CodeableConcept;
}
/**
 * Number of samples in the statistic.
 */
export interface EvidenceStatisticSampleSize extends BackboneElement {
    /**
     * Human-readable summary of population sample size.
     */
    description?: string | undefined;
    _description?: Element | undefined;
    /**
     * Number of participants with known results for measured variables.
     */
    knownDataCount?: number | undefined;
    /**
     * Footnote or explanatory note about the sample size.
     */
    note?: Annotation[] | undefined;
    /**
     * A human-readable string to clarify or explain concepts about the sample size.
     */
    numberOfParticipants?: number | undefined;
    /**
     * Number of participants in the population.
     */
    numberOfStudies?: number | undefined;
}
/**
 * A statistical attribute of the statistic such as a measure of heterogeneity.
 */
export interface EvidenceStatisticAttributeEstimate extends BackboneElement {
    /**
     * A nested attribute estimate; which is the attribute estimate of an attribute estimate
     */
    attributeEstimate?: EvidenceStatisticAttributeEstimate[] | undefined;
    /**
     * Human-readable summary of the estimate.
     */
    description?: string | undefined;
    _description?: Element | undefined;
    /**
     * Use 95 for a 95% confidence interval.
     */
    level?: number | undefined;
    /**
     * Footnote or explanatory note about the estimate.
     */
    note?: Annotation[] | undefined;
    /**
     * Often the p value
     */
    quantity?: Quantity | undefined;
    /**
     * Lower bound of confidence interval.
     */
    range?: Range | undefined;
    /**
     * The type of attribute estimate, e.g., confidence interval or p value.
     */
    type?: CodeableConcept | undefined;
}
/**
 * A variable adjusted for in the adjusted analysis.
 */
export interface EvidenceStatisticModelCharacteristicVariable extends BackboneElement {
    /**
     * How the variable is classified for use in adjusted analysis.
     */
    handling?: ('continuous' | 'dichotomous' | 'ordinal' | 'polychotomous') | undefined;
    _handling?: Element | undefined;
    /**
     * Description for grouping of ordinal or polychotomous variables.
     */
    valueCategory?: CodeableConcept[] | undefined;
    /**
     * Discrete value for grouping of ordinal or polychotomous variables.
     */
    valueQuantity?: Quantity[] | undefined;
    /**
     * Range of values for grouping of ordinal or polychotomous variables.
     */
    valueRange?: Range[] | undefined;
    /**
     * Description of the variable.
     */
    variableDefinition: Reference;
}
/**
 * A component of the method to generate the statistic.
 */
export interface EvidenceStatisticModelCharacteristic extends BackboneElement {
    /**
     * An attribute of the statistic used as a model characteristic.
     */
    attributeEstimate?: EvidenceStatisticAttributeEstimate[] | undefined;
    /**
     * Description of a component of the method to generate the statistic.
     */
    code: CodeableConcept;
    /**
     * Further specification of the quantified value of the component of the method to generate the statistic.
     */
    value?: Quantity | undefined;
    /**
     * A variable adjusted for in the adjusted analysis.
     */
    variable?: EvidenceStatisticModelCharacteristicVariable[] | undefined;
}
/**
 * Values and parameters for a single statistic.
 */
export interface EvidenceStatistic extends BackboneElement {
    /**
     * A statistical attribute of the statistic such as a measure of heterogeneity.
     */
    attributeEstimate?: EvidenceStatisticAttributeEstimate[] | undefined;
    /**
     * Simple strings can be used for descriptive purposes. Exact matching to EvidenceVariable.category.name for the Evidence.variableDefinition[variableRole=measuredVariable].observed=Reference(EvidenceVariable) could facilitate validation within datasets.
     */
    category?: CodeableConcept | undefined;
    /**
     * A description of the content value of the statistic.
     */
    description?: string | undefined;
    _description?: Element | undefined;
    /**
     * A component of the method to generate the statistic.
     */
    modelCharacteristic?: EvidenceStatisticModelCharacteristic[] | undefined;
    /**
     * Footnotes and/or explanatory notes.
     */
    note?: Annotation[] | undefined;
    /**
     * When the number affected is the statistic, use Evidence.statistic.quantity and set Evidence.statistic.type.coding.code=C25463 and Evidence.statistic.type.coding.display=Count. When the statistic is a Proportion, use Evidence.statistic.numberAffected and enter an integer as the value. When the statistic is an Event Rate (where individual participants may have 2 or more events), use Evidence.statistic.numberAffected to record the number of participants with events rather than the total number of events.
     */
    numberAffected?: number | undefined;
    /**
     * When the number of events is the statistic, use Evidence.statistic.quantity and set Evidence.statistic.type.coding.code=C25463 and Evidence.statistic.type.coding.display=Count. When the statistic is an Event Rate (where individual participants may have 2 or more events), use Evidence.statistic.numberOfEvents to record the total number of events rather than the number of participants with events.
     */
    numberOfEvents?: number | undefined;
    /**
     * Statistic value.
     */
    quantity?: Quantity | undefined;
    /**
     * Number of samples in the statistic.
     */
    sampleSize?: EvidenceStatisticSampleSize | undefined;
    /**
     * Type of statistic, e.g., relative risk.
     */
    statisticType?: CodeableConcept | undefined;
}
/**
 * Assessment of certainty, confidence in the estimates, or quality of the evidence.
 */
export interface EvidenceCertainty extends BackboneElement {
    /**
     * Textual description of certainty.
     */
    description?: string | undefined;
    _description?: Element | undefined;
    /**
     * Footnotes and/or explanatory notes.
     */
    note?: Annotation[] | undefined;
    /**
     * Individual or group who did the rating.
     */
    rater?: string | undefined;
    _rater?: Element | undefined;
    /**
     * Assessment or judgement of the aspect.
     */
    rating?: CodeableConcept | undefined;
    /**
     * A domain or subdomain of certainty.
     */
    subcomponent?: EvidenceCertainty[] | undefined;
    /**
     * Aspect of certainty being rated.
     */
    type?: CodeableConcept | undefined;
}
/**
 * The Evidence Resource provides a machine-interpretable expression of an evidence concept including the evidence variables (e.g., population, exposures/interventions, comparators, outcomes, measured variables, confounding variables), the statistics, and the certainty of this evidence.
 */
export interface Evidence extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'Evidence';
    /**
     * The 'date' element may be more recent than the approval date because of minor changes or editorial corrections.
     */
    approvalDate?: string | undefined;
    _approvalDate?: Element | undefined;
    /**
     * Declarative description of the Evidence.
     */
    assertion?: string | undefined;
    _assertion?: Element | undefined;
    /**
     * Extensions to ContactDetail include: contactReference, contactAddress, and contributionTime (Details at: http://build.fhir.org/clinicalreasoning-module.html).
     */
    author?: ContactDetail[] | undefined;
    /**
     * Assessment of certainty, confidence in the estimates, or quality of the evidence.
     */
    certainty?: EvidenceCertainty[] | undefined;
    /**
     * Citation Resource or display of suggested citation for this evidence.
     */
    citeAsReference?: Reference | undefined;
    /**
     * Citation Resource or display of suggested citation for this evidence.
     */
    citeAsMarkdown?: string | undefined;
    _citeAsMarkdown?: Element | undefined;
    /**
     * Extensions to ContactDetail include: contactReference, contactAddress, and contributionTime (Details at: http://build.fhir.org/clinicalreasoning-module.html).
     */
    contact?: ContactDetail[] | undefined;
    /**
     * Note that this is not the same as the resource last-modified-date, since the resource may be a secondary representation of the summary. Additional specific dates may be added as extensions or be found by consulting Provenances associated with past versions of the resource.
     */
    date?: string | undefined;
    _date?: Element | undefined;
    /**
     * This description can be used to capture details such as comments about misuse, instructions for clinical use and interpretation, literature references, examples from the paper world, etc. It is not a rendering of the evidence as conveyed in the 'text' field of the resource itself. This item SHOULD be populated unless the information is available from context (e.g. the language of the evidence is presumed to be the predominant language in the place the evidence was created).
     */
    description?: string | undefined;
    _description?: Element | undefined;
    /**
     * Extensions to ContactDetail include: contactReference, contactAddress, and contributionTime (Details at: http://build.fhir.org/clinicalreasoning-module.html).
     */
    editor?: ContactDetail[] | undefined;
    /**
     * Extensions to ContactDetail include: contactReference, contactAddress, and contributionTime (Details at: http://build.fhir.org/clinicalreasoning-module.html).
     */
    endorser?: ContactDetail[] | undefined;
    /**
     * A Boolean value to indicate that this resource is authored for testing purposes (or education/evaluation/marketing) and is not intended to be used for genuine usage.
     */
    experimental?: boolean | undefined;
    _experimental?: Element | undefined;
    /**
     * Typically, this is used for identifiers that can go in an HL7 V3 II (instance identifier) data type, and can then identify this summary outside of FHIR, where it is not possible to use the logical URI.
     */
    identifier?: Identifier[] | undefined;
    /**
     * If specified, this date follows the original approval date.
     */
    lastReviewDate?: string | undefined;
    _lastReviewDate?: Element | undefined;
    /**
     * The name is not expected to be globally unique. The name should be a simple alphanumeric type name to ensure that it is machine-processing friendly.
     */
    name?: string | undefined;
    _name?: Element | undefined;
    /**
     * Footnotes and/or explanatory notes.
     */
    note?: Annotation[] | undefined;
    /**
     * Usually an organization but may be an individual. The publisher (or steward) of the evidence is the organization or individual primarily responsible for the maintenance and upkeep of the evidence. This is not necessarily the same individual or organization that developed and initially authored the content. The publisher is the primary point of contact for questions or issues with the evidence. This item SHOULD be populated unless the information is available from context.
     */
    publisher?: string | undefined;
    _publisher?: Element | undefined;
    /**
     * Link or citation to artifact associated with the summary.
     */
    relatedArtifact?: RelatedArtifact[] | undefined;
    /**
     * Extensions to ContactDetail include: contactReference, contactAddress, and contributionTime (Details at: http://build.fhir.org/clinicalreasoning-module.html).
     */
    reviewer?: ContactDetail[] | undefined;
    /**
     * Values and parameters for a single statistic.
     */
    statistic?: EvidenceStatistic[] | undefined;
    /**
     * Allows filtering of summarys s that are appropriate for use versus not.
     */
    status: 'draft' | 'active' | 'retired' | 'unknown';
    _status?: Element | undefined;
    /**
     * The design of the study that produced this evidence. The design is described with any number of study design characteristics.
     */
    studyDesign?: CodeableConcept[] | undefined;
    /**
     * The method to combine studies.
     */
    synthesisType?: CodeableConcept | undefined;
    /**
     * In some cases, the resource can no longer be found at the stated url, but the url itself cannot change. Implementations can use the meta.source element to indicate where the current master source of the resource can be found.
     */
    title?: string | undefined;
    _title?: Element | undefined;
    /**
     * Can be a urn:uuid: or a urn:oid: but real http: addresses are preferred. Multiple instances may share the same URL if they have a distinct version.
     */
    url?: string | undefined;
    _url?: Element | undefined;
    /**
     * When multiple useContexts are specified, there is no expectation that all or any of the contexts apply.
     */
    useContext?: UsageContext[] | undefined;
    /**
     * Evidence variable such as population, exposure, or outcome.
     */
    variableDefinition: EvidenceVariableDefinition[];
    /**
     * The determination of when to create a new version of a resource (same url, new version) vs. defining a new artifact is up to the author. Considerations for making this decision are found in Technical and Business Versions.
     */
    version?: string | undefined;
    _version?: Element | undefined;
}
/**
 * Characteristic.
 */
export interface EvidenceReportSubjectCharacteristic extends BackboneElement {
    /**
     * Example 1 is a Citation. Example 2 is a type of outcome. Example 3 is a specific outcome.
     */
    code: CodeableConcept;
    /**
     * Is used to express not the characteristic.
     */
    exclude?: boolean | undefined;
    _exclude?: Element | undefined;
    /**
     * Timeframe for the characteristic.
     */
    period?: Period | undefined;
    /**
     * Example 1 is Citation #37. Example 2 is selecting clinical outcomes. Example 3 is 1-year mortality.
     */
    valueReference?: Reference | undefined;
    /**
     * Example 1 is Citation #37. Example 2 is selecting clinical outcomes. Example 3 is 1-year mortality.
     */
    valueCodeableConcept?: CodeableConcept | undefined;
    /**
     * Example 1 is Citation #37. Example 2 is selecting clinical outcomes. Example 3 is 1-year mortality.
     */
    valueBoolean?: boolean | undefined;
    _valueBoolean?: Element | undefined;
    /**
     * Example 1 is Citation #37. Example 2 is selecting clinical outcomes. Example 3 is 1-year mortality.
     */
    valueQuantity?: Quantity | undefined;
    /**
     * Example 1 is Citation #37. Example 2 is selecting clinical outcomes. Example 3 is 1-year mortality.
     */
    valueRange?: Range | undefined;
}
/**
 * May be used as an expression for search queries and search results
 */
export interface EvidenceReportSubject extends BackboneElement {
    /**
     * Characteristic.
     */
    characteristic?: EvidenceReportSubjectCharacteristic[] | undefined;
    /**
     * Used for general notes and annotations not coded elsewhere.
     */
    note?: Annotation[] | undefined;
}
/**
 * The target composition/document of this relationship.
 */
export interface EvidenceReportRelatesToTarget extends BackboneElement {
    /**
     * Target of the relationship Display.
     */
    display?: string | undefined;
    _display?: Element | undefined;
    /**
     * Target of the relationship Identifier.
     */
    identifier?: Identifier | undefined;
    /**
     * Target of the relationship Resource reference.
     */
    resource?: Reference | undefined;
    /**
     * Target of the relationship URL.
     */
    url?: string | undefined;
    _url?: Element | undefined;
}
/**
 * A document is a version specific composition.
 */
export interface EvidenceReportRelatesTo extends BackboneElement {
    /**
     * If this document appends another document, then the document cannot be fully understood without also accessing the referenced document.
     */
    code:
        | 'replaces'
        | 'amends'
        | 'appends'
        | 'transforms'
        | 'replacedWith'
        | 'amendedWith'
        | 'appendedWith'
        | 'transformedWith';
    _code?: Element | undefined;
    /**
     * The target composition/document of this relationship.
     */
    target: EvidenceReportRelatesToTarget;
}
/**
 * The root of the sections that make up the composition.
 */
export interface EvidenceReportSection extends BackboneElement {
    /**
     * Identifies who is responsible for the information in this section, not necessarily who typed it in.
     */
    author?: Reference[] | undefined;
    /**
     * The various reasons for an empty section make a significant interpretation to its interpretation. Note that this code is for use when the entire section content has been suppressed, and not for when individual items are omitted - implementers may consider using a text note or a flag on an entry in these cases.
     */
    emptyReason?: CodeableConcept | undefined;
    /**
     * Specifies any type of classification of the evidence report.
     */
    entryClassifier?: CodeableConcept[] | undefined;
    /**
     * Quantity as content.
     */
    entryQuantity?: Quantity[] | undefined;
    /**
     * If there are no entries in the list, an emptyReason SHOULD be provided.
     */
    entryReference?: Reference[] | undefined;
    /**
     * The code identifies the section for an automated processor of the document. This is particularly relevant when using profiles to control the structure of the document.
     * If the section has content (instead of sub-sections), the section.code does not change the meaning or interpretation of the resource that is the content of the section in the comments for the section.code.
     */
    focus?: CodeableConcept | undefined;
    /**
     * A definitional Resource identifying the kind of content contained within the section. This should be consistent with the section title.
     */
    focusReference?: Reference | undefined;
    /**
     * This element is labeled as a modifier because a change list must not be misunderstood as a complete list.
     */
    mode?: ('working' | 'snapshot' | 'changes') | undefined;
    _mode?: Element | undefined;
    /**
     * Applications SHOULD render ordered lists in the order provided, but MAY allow users to re-order based on their own preferences as well. If there is no order specified, the order is unknown, though there may still be some order.
     */
    orderedBy?: CodeableConcept | undefined;
    /**
     * Nested sections are primarily used to help human readers navigate to particular portions of the document.
     */
    section?: EvidenceReportSection[] | undefined;
    /**
     * Document profiles may define what content should be represented in the narrative.
     */
    text?: Narrative | undefined;
    /**
     * The title identifies the section for a human reader. The title must be consistent with the narrative of the resource that is the target of the section.content reference. Generally, sections SHOULD have titles, but in some documents, it is unnecessary or inappropriate. Typically, this is where a section has subsections that have their own adequately distinguishing title,  or documents that only have a single section.
     */
    title?: string | undefined;
    _title?: Element | undefined;
}
/**
 * The EvidenceReport Resource is a specialized container for a collection of resources and codeable concepts, adapted to support compositions of Evidence, EvidenceVariable, and Citation resources and related concepts.
 */
export interface EvidenceReport extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'EvidenceReport';
    /**
     * Extensions to ContactDetail include: contactReference, contactAddress, and contributionTime (Details at: http://build.fhir.org/clinicalreasoning-module.html).
     */
    author?: ContactDetail[] | undefined;
    /**
     * used for reports for which external citation is expected, such as use in support of scholarly publications.
     */
    citeAsReference?: Reference | undefined;
    /**
     * used for reports for which external citation is expected, such as use in support of scholarly publications.
     */
    citeAsMarkdown?: string | undefined;
    _citeAsMarkdown?: Element | undefined;
    /**
     * Extensions to ContactDetail include: contactReference, contactAddress, and contributionTime (Details at: http://build.fhir.org/clinicalreasoning-module.html).
     */
    contact?: ContactDetail[] | undefined;
    /**
     * Extensions to ContactDetail include: contactReference, contactAddress, and contributionTime (Details at: http://build.fhir.org/clinicalreasoning-module.html).
     */
    editor?: ContactDetail[] | undefined;
    /**
     * Extensions to ContactDetail include: contactReference, contactAddress, and contributionTime (Details at: http://build.fhir.org/clinicalreasoning-module.html).
     */
    endorser?: ContactDetail[] | undefined;
    /**
     * This element will contain unique identifiers that support de-duplication of EvidenceReports. This identifier can be valid for only one EvidenceReport resource.
     */
    identifier?: Identifier[] | undefined;
    /**
     * Used for footnotes and annotations.
     */
    note?: Annotation[] | undefined;
    /**
     * Usually an organization but may be an individual. The publisher (or steward) of the evidence report is the organization or individual primarily responsible for the maintenance and upkeep of the evidence report. This is not necessarily the same individual or organization that developed and initially authored the content. The publisher is the primary point of contact for questions or issues with the evidence report. This item SHOULD be populated unless the information is available from context.
     */
    publisher?: string | undefined;
    _publisher?: Element | undefined;
    /**
     * Link, description or reference to artifact associated with the report.
     */
    relatedArtifact?: RelatedArtifact[] | undefined;
    /**
     * May include trial registry identifiers, e.g. NCT04372602 from clinicaltrials.gov. This identifier can be valid for multiple EvidenceReport resources.
     */
    relatedIdentifier?: Identifier[] | undefined;
    /**
     * A document is a version specific composition.
     */
    relatesTo?: EvidenceReportRelatesTo[] | undefined;
    /**
     * Extensions to ContactDetail include: contactReference, contactAddress, and contributionTime (Details at: http://build.fhir.org/clinicalreasoning-module.html).
     */
    reviewer?: ContactDetail[] | undefined;
    /**
     * The root of the sections that make up the composition.
     */
    section?: EvidenceReportSection[] | undefined;
    /**
     * Allows filtering of summaries that are appropriate for use versus not.
     */
    status: 'draft' | 'active' | 'retired' | 'unknown';
    _status?: Element | undefined;
    /**
     * May be used as an expression for search queries and search results
     */
    subject: EvidenceReportSubject;
    /**
     * Specifies the kind of report, such as grouping of classifiers, search results, or human-compiled expression.
     */
    type?: CodeableConcept | undefined;
    /**
     * Can be a urn:uuid: or a urn:oid: but real http: addresses are preferred. Multiple instances may share the same URL if they have a distinct version.
     */
    url?: string | undefined;
    _url?: Element | undefined;
    /**
     * When multiple useContexts are specified, there is no expectation that all or any of the contexts apply.
     */
    useContext?: UsageContext[] | undefined;
}
/**
 * Defines the characteristic using both a type and value[x] elements.
 */
export interface EvidenceVariableCharacteristicDefinitionByTypeAndValue extends BackboneElement {
    /**
     * Device used for determining characteristic.
     */
    device?: Reference | undefined;
    /**
     * Method for how the characteristic value was determined.
     */
    method?: CodeableConcept[] | undefined;
    /**
     * Defines the reference point for comparison when valueQuantity is not compared to zero.
     */
    offset?: CodeableConcept | undefined;
    /**
     * Used to express the type of characteristic.
     */
    type: CodeableConcept;
    /**
     * Defines the characteristic when paired with characteristic.type.
     */
    valueCodeableConcept?: CodeableConcept | undefined;
    /**
     * Defines the characteristic when paired with characteristic.type.
     */
    valueBoolean?: boolean | undefined;
    _valueBoolean?: Element | undefined;
    /**
     * Defines the characteristic when paired with characteristic.type.
     */
    valueQuantity?: Quantity | undefined;
    /**
     * Defines the characteristic when paired with characteristic.type.
     */
    valueRange?: Range | undefined;
    /**
     * Defines the characteristic when paired with characteristic.type.
     */
    valueReference?: Reference | undefined;
    /**
     * Defines the characteristic when paired with characteristic.type.
     */
    valueId?: string | undefined;
    _valueId?: Element | undefined;
}
/**
 * Defines the characteristic as a combination of two or more characteristics.
 */
export interface EvidenceVariableCharacteristicDefinitionByCombination extends BackboneElement {
    /**
     * A defining factor of the characteristic.
     */
    characteristic: EvidenceVariableCharacteristic[];
    /**
     * Used to specify if two or more characteristics are combined with OR or AND.
     */
    code: 'all-of' | 'any-of' | 'at-least' | 'at-most' | 'statistical' | 'net-effect' | 'dataset';
    _code?: Element | undefined;
    /**
     * Provides the value of "n" when "at-least" or "at-most" codes are used.
     */
    threshold?: number | undefined;
}
/**
 * Observation time from study specified event.
 */
export interface EvidenceVariableCharacteristicTimeFromEvent extends BackboneElement {
    /**
     * Human readable description.
     */
    description?: string | undefined;
    _description?: Element | undefined;
    /**
     * The event used as a base point (reference point) in time.
     */
    eventCodeableConcept?: CodeableConcept | undefined;
    /**
     * The event used as a base point (reference point) in time.
     */
    eventReference?: Reference | undefined;
    /**
     * The event used as a base point (reference point) in time.
     */
    eventDateTime?: string | undefined;
    _eventDateTime?: Element | undefined;
    /**
     * The event used as a base point (reference point) in time.
     */
    eventId?: string | undefined;
    _eventId?: Element | undefined;
    /**
     * A human-readable string to clarify or explain concepts about the timeFromEvent.
     */
    note?: Annotation[] | undefined;
    /**
     * Used to express the observation at a defined amount of time after the study start.
     */
    quantity?: Quantity | undefined;
    /**
     * Used to express the observation within a period after the study start.
     */
    range?: Range | undefined;
}
/**
 * Characteristics can be defined flexibly to accommodate different use cases for membership criteria, ranging from simple codes, all the way to using an expression language to express the criteria.
 */
export interface EvidenceVariableCharacteristic extends BackboneElement {
    /**
     * Defines the characteristic as a combination of two or more characteristics.
     */
    definitionByCombination?: EvidenceVariableCharacteristicDefinitionByCombination | undefined;
    /**
     * Defines the characteristic using both a type and value[x] elements.
     */
    definitionByTypeAndValue?: EvidenceVariableCharacteristicDefinitionByTypeAndValue | undefined;
    /**
     * Defines the characteristic using Canonical.
     */
    definitionCanonical?: string | undefined;
    _definitionCanonical?: Element | undefined;
    /**
     * Defines the characteristic using CodeableConcept.
     */
    definitionCodeableConcept?: CodeableConcept | undefined;
    /**
     * Defines the characteristic using Expression.
     */
    definitionExpression?: Expression | undefined;
    /**
     * Defines the characteristic using id.
     */
    definitionId?: string | undefined;
    _definitionId?: Element | undefined;
    /**
     * Defines the characteristic using a Reference.
     */
    definitionReference?: Reference | undefined;
    /**
     * A short, natural language description of the characteristic that could be used to communicate the criteria to an end-user.
     */
    description?: string | undefined;
    _description?: Element | undefined;
    /**
     * When true, this characteristic is an exclusion criterion. In other words, not matching this characteristic definition is equivalent to meeting this criterion.
     */
    exclude?: boolean | undefined;
    _exclude?: Element | undefined;
    /**
     * Label used for when a characteristic refers to another characteristic.
     */
    linkId?: string | undefined;
    _linkId?: Element | undefined;
    /**
     * A human-readable string to clarify or explain concepts about the characteristic.
     */
    note?: Annotation[] | undefined;
    /**
     * Observation time from study specified event.
     */
    timeFromEvent?: EvidenceVariableCharacteristicTimeFromEvent[] | undefined;
}
/**
 * A grouping for ordinal or polychotomous variables.
 */
export interface EvidenceVariableCategory extends BackboneElement {
    /**
     * Description of the grouping.
     */
    name?: string | undefined;
    _name?: Element | undefined;
    /**
     * Definition of the grouping.
     */
    valueCodeableConcept?: CodeableConcept | undefined;
    /**
     * Definition of the grouping.
     */
    valueQuantity?: Quantity | undefined;
    /**
     * Definition of the grouping.
     */
    valueRange?: Range | undefined;
}
/**
 * The EvidenceVariable resource describes an element that knowledge (Evidence) is about.
 */
export interface EvidenceVariable extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'EvidenceVariable';
    /**
     * True if the actual variable measured, false if a conceptual representation of the intended variable.
     */
    actual?: boolean | undefined;
    _actual?: Element | undefined;
    /**
     * The date on which the resource content was approved by the publisher. Approval happens once when the content is officially approved for usage.
     */
    approvalDate?: string | undefined;
    _approvalDate?: Element | undefined;
    /**
     * Extensions to ContactDetail include: contactReference, contactAddress, and contributionTime (Details at: http://build.fhir.org/clinicalreasoning-module.html).
     */
    author?: ContactDetail[] | undefined;
    /**
     * A grouping for ordinal or polychotomous variables.
     */
    category?: EvidenceVariableCategory[] | undefined;
    /**
     * Characteristics can be defined flexibly to accommodate different use cases for membership criteria, ranging from simple codes, all the way to using an expression language to express the criteria.
     */
    characteristic?: EvidenceVariableCharacteristic[] | undefined;
    /**
     * Extensions to ContactDetail include: contactReference, contactAddress, and contributionTime (Details at: http://build.fhir.org/clinicalreasoning-module.html).
     */
    contact?: ContactDetail[] | undefined;
    /**
     * A copyright statement relating to the resource and/or its contents. Copyright statements are generally legal restrictions on the use and publishing of the resource.
     */
    copyright?: string | undefined;
    _copyright?: Element | undefined;
    /**
     * Note that this is not the same as the resource last-modified-date, since the resource may be a secondary representation of the evidence variable. Additional specific dates may be added as extensions or be found by consulting Provenances associated with past versions of the resource.
     */
    date?: string | undefined;
    _date?: Element | undefined;
    /**
     * This description can be used to capture details such as comments about misuse, instructions for clinical use and interpretation, literature references, examples from the paper world, etc. It is not a rendering of the evidence variable as conveyed in the 'text' field of the resource itself. This item SHOULD be populated unless the information is available from context (e.g. the language of the evidence variable is presumed to be the predominant language in the place the evidence variable was created).
     */
    description?: string | undefined;
    _description?: Element | undefined;
    /**
     * Extensions to ContactDetail include: contactReference, contactAddress, and contributionTime (Details at: http://build.fhir.org/clinicalreasoning-module.html).
     */
    editor?: ContactDetail[] | undefined;
    /**
     * The period during which the resource content was or is planned to be in active use.
     */
    effectivePeriod?: Period | undefined;
    /**
     * Extensions to ContactDetail include: contactReference, contactAddress, and contributionTime (Details at: http://build.fhir.org/clinicalreasoning-module.html).
     */
    endorser?: ContactDetail[] | undefined;
    /**
     * A Boolean value to indicate that this resource is authored for testing purposes (or education/evaluation/marketing) and is not intended to be used for genuine usage.
     */
    experimental?: boolean | undefined;
    _experimental?: Element | undefined;
    /**
     * The method of handling in statistical analysis.
     */
    handling?: ('continuous' | 'dichotomous' | 'ordinal' | 'polychotomous') | undefined;
    _handling?: Element | undefined;
    /**
     * Typically, this is used for identifiers that can go in an HL7 V3 II (instance identifier) data type, and can then identify this evidence variable outside of FHIR, where it is not possible to use the logical URI.
     */
    identifier?: Identifier[] | undefined;
    /**
     * The date on which the resource content was last reviewed. Review happens periodically after approval but does not change the original approval date.
     */
    lastReviewDate?: string | undefined;
    _lastReviewDate?: Element | undefined;
    /**
     * The name is not expected to be globally unique. The name should be a simple alphanumeric type name to ensure that it is machine-processing friendly.
     */
    name?: string | undefined;
    _name?: Element | undefined;
    /**
     * A human-readable string to clarify or explain concepts about the resource.
     */
    note?: Annotation[] | undefined;
    /**
     * Usually an organization but may be an individual. The publisher (or steward) of the evidence variable is the organization or individual primarily responsible for the maintenance and upkeep of the evidence variable. This is not necessarily the same individual or organization that developed and initially authored the content. The publisher is the primary point of contact for questions or issues with the evidence variable. This item SHOULD be populated unless the information is available from context.
     */
    publisher?: string | undefined;
    _publisher?: Element | undefined;
    /**
     * Each related artifact is either an attachment, or a reference to another resource, but not both.
     */
    relatedArtifact?: RelatedArtifact[] | undefined;
    /**
     * Extensions to ContactDetail include: contactReference, contactAddress, and contributionTime (Details at: http://build.fhir.org/clinicalreasoning-module.html).
     */
    reviewer?: ContactDetail[] | undefined;
    /**
     * The short title provides an alternate title for use in informal descriptive contexts where the full, formal title is not necessary.
     */
    shortTitle?: string | undefined;
    _shortTitle?: Element | undefined;
    /**
     * Allows filtering of evidence variables that are appropriate for use versus not.
     */
    status: 'draft' | 'active' | 'retired' | 'unknown';
    _status?: Element | undefined;
    /**
     * An explanatory or alternate title for the EvidenceVariable giving additional information about its content.
     */
    subtitle?: string | undefined;
    _subtitle?: Element | undefined;
    /**
     * This name does not need to be machine-processing friendly and may contain punctuation, white-space, etc.
     */
    title?: string | undefined;
    _title?: Element | undefined;
    /**
     * Can be a urn:uuid: or a urn:oid: but real http: addresses are preferred.  Multiple instances may share the same URL if they have a distinct version.
     * The determination of when to create a new version of a resource (same url, new version) vs. defining a new artifact is up to the author.  Considerations for making this decision are found in [Technical and Business Versions](resource.html#versions).
     * In some cases, the resource can no longer be found at the stated url, but the url itself cannot change. Implementations can use the [meta.source](resource.html#meta) element to indicate where the current master source of the resource can be found.
     */
    url?: string | undefined;
    _url?: Element | undefined;
    /**
     * When multiple useContexts are specified, there is no expectation that all or any of the contexts apply.
     */
    useContext?: UsageContext[] | undefined;
    /**
     * There may be different evidence variable instances that have the same identifier but different versions.  The version can be appended to the url in a reference to allow a reference to a particular business version of the evidence variable with the format [url]|[version]. The version SHOULD NOT contain a '#' - see [Business Version](resource.html#bv-format).
     */
    version?: string | undefined;
    _version?: Element | undefined;
}
/**
 * A system or person who shares or receives an instance within the scenario.
 */
export interface ExampleScenarioActor extends BackboneElement {
    /**
     * An explanation of who/what the actor is and its role in the scenario.
     */
    description?: string | undefined;
    _description?: Element | undefined;
    /**
     * A unique string within the scenario that is used to reference the actor.
     */
    key: string;
    _key?: Element | undefined;
    /**
     * The human-readable name for the actor used when rendering the scenario.
     */
    title: string;
    _title?: Element | undefined;
    /**
     * The category of actor - person or system.
     */
    type: 'person' | 'system';
    _type?: Element | undefined;
}
/**
 * Not used if an instance doesn't change
 */
export interface ExampleScenarioInstanceVersion extends BackboneElement {
    /**
     * If not conveying FHIR data or not using the same version of FHIR as this ExampleScenario instance, the reference must be to a Binary.
     */
    content?: Reference | undefined;
    /**
     * An explanation of what this specific version of the instance contains and represents.
     */
    description?: string | undefined;
    _description?: Element | undefined;
    /**
     * A unique string within the instance that is used to reference the version of the instance.
     */
    key: string;
    _key?: Element | undefined;
}
/**
 * References to other instances that can be found within this instance (e.g. the observations contained in a bundle).
 */
export interface ExampleScenarioInstanceContainedInstance extends BackboneElement {
    /**
     * A reference to the key of an instance found within this one.
     */
    instanceReference: string;
    _instanceReference?: Element | undefined;
    /**
     * Required if the referenced instance has versions
     */
    versionReference?: string | undefined;
    _versionReference?: Element | undefined;
}
/**
 * A single data collection that is shared as part of the scenario.
 */
export interface ExampleScenarioInstance extends BackboneElement {
    /**
     * References to other instances that can be found within this instance (e.g. the observations contained in a bundle).
     */
    containedInstance?: ExampleScenarioInstanceContainedInstance[] | undefined;
    /**
     * If not conveying FHIR data or not using the same version of FHIR as this ExampleScenario instance, the reference must be to a Binary.
     */
    content?: Reference | undefined;
    /**
     * An explanation of what the instance contains and what it's for.
     */
    description?: string | undefined;
    _description?: Element | undefined;
    /**
     * A unique string within the scenario that is used to reference the instance.
     */
    key: string;
    _key?: Element | undefined;
    /**
     * Refers to a profile, template or other ruleset the instance adheres to.
     */
    structureProfileCanonical?: string | undefined;
    _structureProfileCanonical?: Element | undefined;
    /**
     * Refers to a profile, template or other ruleset the instance adheres to.
     */
    structureProfileUri?: string | undefined;
    _structureProfileUri?: Element | undefined;
    /**
     * A code indicating the kind of data structure (FHIR resource or some other standard) this is an instance of.
     */
    structureType: Coding;
    /**
     * Conveys the version of the data structure instantiated.  I.e. what release of FHIR, X12, OpenEHR, etc. is instance compliant with.
     */
    structureVersion?: string | undefined;
    _structureVersion?: Element | undefined;
    /**
     * A short descriptive label the instance to be used in tables or diagrams.
     */
    title: string;
    _title?: Element | undefined;
    /**
     * Not used if an instance doesn't change
     */
    version?: ExampleScenarioInstanceVersion[] | undefined;
}
/**
 * The step represents a single operation invoked on receiver by sender.
 */
export interface ExampleScenarioProcessStepOperation extends BackboneElement {
    /**
     * This should contain information not already present in the process step.  It's more of a description of what the operation in general does - if not already evident from the operation.type
     */
    description?: string | undefined;
    _description?: Element | undefined;
    /**
     * This must either be the 'key' of one of the actors defined in this scenario or the special keyword 'OTHER' if the initiator is not one of the actors defined for the scenario.  (Multiple references to 'OTHER' don't necessarily indicate the same actor.)
     */
    initiator?: string | undefined;
    _initiator?: Element | undefined;
    /**
     * De-activation of an actor means they have no further role until such time as they are the recipient of an operation.
     */
    initiatorActive?: boolean | undefined;
    _initiatorActive?: Element | undefined;
    /**
     * This must either be the 'key' of one of the actors defined in this scenario or the special keyword 'OTHER' if the receiver is not one of the actors defined for the scenario.  (Multiple references to 'OTHER' don't necessarily indicate the same actor.)  In some cases, receiver could be same as sender if representing a local operation
     */
    receiver?: string | undefined;
    _receiver?: Element | undefined;
    /**
     * De-activation of an actor means they have no further role until such time as they are the recipient of an operation.
     */
    receiverActive?: boolean | undefined;
    _receiverActive?: Element | undefined;
    /**
     * A reference to the instance that is transmitted from requester to receiver as part of the invocation of the operation.
     */
    request?: ExampleScenarioInstanceContainedInstance | undefined;
    /**
     * A reference to the instance that is transmitted from receiver to requester as part of the operation's synchronous response (if any).
     */
    response?: ExampleScenarioInstanceContainedInstance | undefined;
    /**
     * A short descriptive label the step to be used in tables or diagrams.
     */
    title: string;
    _title?: Element | undefined;
    /**
     * The standardized type of action (FHIR or otherwise).
     */
    type?: Coding | undefined;
}
/**
 * Indicates an alternative step that can be taken instead of the sub-process, scenario or operation.  E.g. to represent non-happy-path/exceptional/atypical circumstances.
 */
export interface ExampleScenarioProcessStepAlternative extends BackboneElement {
    /**
     * A human-readable description of the alternative explaining when the alternative should occur rather than the base step.
     */
    description?: string | undefined;
    _description?: Element | undefined;
    /**
     * Indicates the operation, sub-process or scenario that happens if the alternative option is selected.
     */
    step?: ExampleScenarioProcessStep[] | undefined;
    /**
     * The label to display for the alternative that gives a sense of the circumstance in which the alternative should be invoked.
     */
    title: string;
    _title?: Element | undefined;
}
/**
 * A significant action that occurs as part of the process.
 */
export interface ExampleScenarioProcessStep extends BackboneElement {
    /**
     * Indicates an alternative step that can be taken instead of the sub-process, scenario or operation.  E.g. to represent non-happy-path/exceptional/atypical circumstances.
     */
    alternative?: ExampleScenarioProcessStepAlternative[] | undefined;
    /**
     * If step numbers are simultaneous, they will be the same.  Numbers don't have to be numeric.  E.g. '2c)' is a valid step number
     */
    number?: string | undefined;
    _number?: Element | undefined;
    /**
     * The step represents a single operation invoked on receiver by sender.
     */
    operation?: ExampleScenarioProcessStepOperation | undefined;
    /**
     * If true, indicates that, following this step, there is a pause in the flow and the subsequent step will occur at some later time (triggered by some event).
     */
    pause?: boolean | undefined;
    _pause?: Element | undefined;
    /**
     * Indicates that the step is a complex sub-process with its own steps.
     */
    process?: ExampleScenarioProcess | undefined;
    /**
     * Indicates that the step is defined by a seaparate scenario instance.
     */
    workflow?: string | undefined;
    _workflow?: Element | undefined;
}
/**
 * Some scenarios might describe only one process.
 */
export interface ExampleScenarioProcess extends BackboneElement {
    /**
     * An explanation of what the process represents and what it does.
     */
    description?: string | undefined;
    _description?: Element | undefined;
    /**
     * Alternate steps might not result in all post conditions holding
     */
    postConditions?: string | undefined;
    _postConditions?: Element | undefined;
    /**
     * Description of the initial state of the actors, environment and data before the process starts.
     */
    preConditions?: string | undefined;
    _preConditions?: Element | undefined;
    /**
     * A significant action that occurs as part of the process.
     */
    step?: ExampleScenarioProcessStep[] | undefined;
    /**
     * A short descriptive label the process to be used in tables or diagrams.
     */
    title: string;
    _title?: Element | undefined;
}
/**
 * Example of workflow instance.
 */
export interface ExampleScenario extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'ExampleScenario';
    /**
     * A system or person who shares or receives an instance within the scenario.
     */
    actor?: ExampleScenarioActor[] | undefined;
    /**
     * May be a web site, an email address, a telephone number, etc.
     */
    contact?: ContactDetail[] | undefined;
    /**
     * ...
     */
    copyright?: string | undefined;
    _copyright?: Element | undefined;
    /**
     * The (c) symbol should NOT be included in this string. It will be added by software when rendering the notation. Full details about licensing, restrictions, warrantees, etc. goes in the more general 'copyright' element.
     */
    copyrightLabel?: string | undefined;
    _copyrightLabel?: Element | undefined;
    /**
     * Note that this is not the same as the resource last-modified-date, since the resource may be a secondary representation of the example scenario. Additional specific dates may be added as extensions or be found by consulting Provenances associated with past versions of the resource.
     */
    date?: string | undefined;
    _date?: Element | undefined;
    /**
     * This description can be used to capture details such as comments about misuse. It is not a rendering of the ExampleScenario as conveyed in the 'text' field of the resource itself.
     */
    description?: string | undefined;
    _description?: Element | undefined;
    /**
     * Allows filtering of example scenarios that are appropriate for use versus not.
     */
    experimental?: boolean | undefined;
    _experimental?: Element | undefined;
    /**
     * Typically, this is used for identifiers that can go in an HL7 V3 II (instance identifier) data type, and can then identify this example scenario outside of FHIR, where it is not possible to use the logical URI.
     */
    identifier?: Identifier[] | undefined;
    /**
     * A single data collection that is shared as part of the scenario.
     */
    instance?: ExampleScenarioInstance[] | undefined;
    /**
     * It may be possible for the example scenario to be used in jurisdictions other than those for which it was originally designed or intended.
     */
    jurisdiction?: CodeableConcept[] | undefined;
    /**
     * Not needed as code will never be generated from one of these
     */
    name?: string | undefined;
    _name?: Element | undefined;
    /**
     * Some scenarios might describe only one process.
     */
    process?: ExampleScenarioProcess[] | undefined;
    /**
     * Usually an organization but may be an individual. The publisher (or steward) of the example scenario is the organization or individual primarily responsible for the maintenance and upkeep of the example scenario. This is not necessarily the same individual or organization that developed and initially authored the content. The publisher is the primary point of contact for questions or issues with the example scenario. This item SHOULD be populated unless the information is available from context.
     */
    publisher?: string | undefined;
    _publisher?: Element | undefined;
    /**
     * This element does not describe the usage of the example scenario. Instead, it provides traceability of ''why'' the resource is either needed or ''why'' it is defined as it is.  This may be used to point to source materials or specifications that drove the structure of this example scenario.
     */
    purpose?: string | undefined;
    _purpose?: Element | undefined;
    /**
     * Allows filtering of example scenarios that are appropriate for use versus not.
     */
    status: 'draft' | 'active' | 'retired' | 'unknown';
    _status?: Element | undefined;
    /**
     * This name does not need to be machine-processing friendly and may contain punctuation, white-space, etc.
     */
    title?: string | undefined;
    _title?: Element | undefined;
    /**
     * Can be a urn:uuid: or a urn:oid: but real http: addresses are preferred.  Multiple instances may share the same URL if they have a distinct version.
     * The determination of when to create a new version of a resource (same url, new version) vs. defining a new artifact is up to the author.  Considerations for making this decision are found in [Technical and Business Versions](resource.html#versions).
     * In some cases, the resource can no longer be found at the stated url, but the url itself cannot change. Implementations can use the [meta.source](resource.html#meta) element to indicate where the current master source of the resource can be found.
     */
    url?: string | undefined;
    _url?: Element | undefined;
    /**
     * When multiple useContexts are specified, there is no expectation that all or any of the contexts apply.
     */
    useContext?: UsageContext[] | undefined;
    /**
     * There may be different example scenario instances that have the same identifier but different versions.  The version can be appended to the url in a reference to allow a reference to a particular business version of the example scenario with the format [url]|[version]. The version SHOULD NOT contain a '#' - see [Business Version](resource.html#bv-format).
     */
    version?: string | undefined;
    _version?: Element | undefined;
    /**
     * If set as a string, this is a FHIRPath expression that has two additional context variables passed in - %version1 and %version2 and will return a negative number if version1 is newer, a positive number if version2 and a 0 if the version ordering can't be successfully be determined.
     */
    versionAlgorithmString?: string | undefined;
    _versionAlgorithmString?: Element | undefined;
    /**
     * If set as a string, this is a FHIRPath expression that has two additional context variables passed in - %version1 and %version2 and will return a negative number if version1 is newer, a positive number if version2 and a 0 if the version ordering can't be successfully be determined.
     */
    versionAlgorithmCoding?: Coding | undefined;
}
/**
 * For example,  for the original treatment and follow-up exams.
 */
export interface ExplanationOfBenefitRelated extends BackboneElement {
    /**
     * Reference to a related claim.
     */
    claim?: Reference | undefined;
    /**
     * For example, Property/Casualty insurer claim number or Workers Compensation case number.
     */
    reference?: Identifier | undefined;
    /**
     * For example, prior claim or umbrella.
     */
    relationship?: CodeableConcept | undefined;
}
/**
 * Often providers agree to receive the benefits payable to reduce the near-term costs to the patient. The insurer may decline to pay the provider and may choose to pay the subscriber instead.
 */
export interface ExplanationOfBenefitPayee extends BackboneElement {
    /**
     * Not required if the payee is 'subscriber' or 'provider'.
     */
    party?: Reference | undefined;
    /**
     * Type of Party to be reimbursed: Subscriber, provider, other.
     */
    type?: CodeableConcept | undefined;
}
/**
 * The members of the team who provided the products and services.
 */
export interface ExplanationOfBenefitCareTeam extends BackboneElement {
    /**
     * Member of the team who provided the product or service.
     */
    provider: Reference;
    /**
     * Responsible might not be required when there is only a single provider listed.
     */
    responsible?: boolean | undefined;
    _responsible?: Element | undefined;
    /**
     * Role might not be required when there is only a single provider listed.
     */
    role?: CodeableConcept | undefined;
    /**
     * A number to uniquely identify care team entries.
     */
    sequence: number;
    /**
     * The specialization of the practitioner or provider which is applicable for this service.
     */
    specialty?: CodeableConcept | undefined;
}
/**
 * Often there are multiple jurisdiction specific valuesets which are required.
 */
export interface ExplanationOfBenefitSupportingInfo extends BackboneElement {
    /**
     * This may contain a category for the local bill type codes.
     */
    category: CodeableConcept;
    /**
     * This may contain the local bill type codes such as the US UB-04 bill type code.
     */
    code?: CodeableConcept | undefined;
    /**
     * For example: the reason for the additional stay, or why a tooth is  missing.
     */
    reason?: Coding | undefined;
    /**
     * A number to uniquely identify supporting information entries.
     */
    sequence: number;
    /**
     * The date when or period to which this information refers.
     */
    timingDate?: string | undefined;
    _timingDate?: Element | undefined;
    /**
     * The date when or period to which this information refers.
     */
    timingPeriod?: Period | undefined;
    /**
     * Could be used to provide references to other resources, document. For example, could contain a PDF in an Attachment of the Police Report for an Accident.
     */
    valueBoolean?: boolean | undefined;
    _valueBoolean?: Element | undefined;
    /**
     * Could be used to provide references to other resources, document. For example, could contain a PDF in an Attachment of the Police Report for an Accident.
     */
    valueString?: string | undefined;
    _valueString?: Element | undefined;
    /**
     * Could be used to provide references to other resources, document. For example, could contain a PDF in an Attachment of the Police Report for an Accident.
     */
    valueQuantity?: Quantity | undefined;
    /**
     * Could be used to provide references to other resources, document. For example, could contain a PDF in an Attachment of the Police Report for an Accident.
     */
    valueAttachment?: Attachment | undefined;
    /**
     * Could be used to provide references to other resources, document. For example, could contain a PDF in an Attachment of the Police Report for an Accident.
     */
    valueReference?: Reference | undefined;
    /**
     * Could be used to provide references to other resources, document. For example, could contain a PDF in an Attachment of the Police Report for an Accident.
     */
    valueIdentifier?: Identifier | undefined;
}
/**
 * Information about diagnoses relevant to the claim items.
 */
export interface ExplanationOfBenefitDiagnosis extends BackboneElement {
    /**
     * The nature of illness or problem in a coded form or as a reference to an external defined Condition.
     */
    diagnosisCodeableConcept?: CodeableConcept | undefined;
    /**
     * The nature of illness or problem in a coded form or as a reference to an external defined Condition.
     */
    diagnosisReference?: Reference | undefined;
    /**
     * Indication of whether the diagnosis was present on admission to a facility.
     */
    onAdmission?: CodeableConcept | undefined;
    /**
     * Diagnosis are presented in list order to their expected importance: primary, secondary, etc.
     */
    sequence: number;
    /**
     * For example: admitting, primary, secondary, discharge.
     */
    type?: CodeableConcept[] | undefined;
}
/**
 * Procedures performed on the patient relevant to the billing items with the claim.
 */
export interface ExplanationOfBenefitProcedure extends BackboneElement {
    /**
     * Date and optionally time the procedure was performed.
     */
    date?: string | undefined;
    _date?: Element | undefined;
    /**
     * The code or reference to a Procedure resource which identifies the clinical intervention performed.
     */
    procedureCodeableConcept?: CodeableConcept | undefined;
    /**
     * The code or reference to a Procedure resource which identifies the clinical intervention performed.
     */
    procedureReference?: Reference | undefined;
    /**
     * A number to uniquely identify procedure entries.
     */
    sequence: number;
    /**
     * When the condition was observed or the relative ranking.
     */
    type?: CodeableConcept[] | undefined;
    /**
     * Unique Device Identifiers associated with this line item.
     */
    udi?: Reference[] | undefined;
}
/**
 * All insurance coverages for the patient which may be applicable for reimbursement, of the products and services listed in the claim, are typically provided in the claim to allow insurers to confirm the ordering of the insurance coverages relative to local 'coordination of benefit' rules. One coverage (and only one) with 'focal=true' is to be used in the adjudication of this claim. Coverages appearing before the focal Coverage in the list, and where 'Coverage.subrogation=false', should provide a reference to the ClaimResponse containing the adjudication results of the prior claim.
 */
export interface ExplanationOfBenefitInsurance extends BackboneElement {
    /**
     * Reference to the insurance card level information contained in the Coverage resource. The coverage issuing insurer will use these details to locate the patient's actual coverage within the insurer's information system.
     */
    coverage: Reference;
    /**
     * A patient may (will) have multiple insurance policies which provide reimbursement for healthcare services and products. For example, a person may also be covered by their spouse's policy and both appear in the list (and may be from the same insurer). This flag will be set to true for only one of the listed policies and that policy will be used for adjudicating this claim. Other claims would be created to request adjudication against the other listed policies.
     */
    focal: boolean;
    _focal?: Element | undefined;
    /**
     * This value is an alphanumeric string that may be provided over the phone, via text, via paper, or within a ClaimResponse resource and is not a FHIR Identifier.
     */
    preAuthRef?: string[] | undefined;
    _preAuthRef?: Element[] | undefined;
}
/**
 * Details of a accident which resulted in injuries which required the products and services listed in the claim.
 */
export interface ExplanationOfBenefitAccident extends BackboneElement {
    /**
     * The date of the accident has to precede the dates of the products and services but within a reasonable timeframe.
     */
    date?: string | undefined;
    _date?: Element | undefined;
    /**
     * The physical location of the accident event.
     */
    locationAddress?: Address | undefined;
    /**
     * The physical location of the accident event.
     */
    locationReference?: Reference | undefined;
    /**
     * The type or context of the accident event for the purposes of selection of potential insurance coverages and determination of coordination between insurers.
     */
    type?: CodeableConcept | undefined;
}
/**
 * Physical location where the service is performed or applies.
 */
export interface ExplanationOfBenefitItemBodySite extends BackboneElement {
    /**
     * For example: Providing a tooth code, allows an insurer to identify a provider performing a filling on a tooth that was previously removed.
     */
    site: CodeableReference[];
    /**
     * A region or surface of the bodySite, e.g. limb region or tooth surface(s).
     */
    subSite?: CodeableConcept[] | undefined;
}
/**
 * If this item is a group then the values here are a summary of the adjudication of the detail items. If this item is a simple product or service then this is the result of the adjudication of this item.
 */
export interface ExplanationOfBenefitItemAdjudication extends BackboneElement {
    /**
     * For example, amount submitted, eligible amount, co-payment, and benefit payable.
     */
    amount?: Money | undefined;
    /**
     * For example, codes indicating: Co-Pay, deductible, eligible, benefit, tax, etc.
     */
    category: CodeableConcept;
    /**
     * For example, may indicate that the funds for this benefit type have been exhausted.
     */
    reason?: CodeableConcept | undefined;
    /**
     * For example: eligible percentage or co-payment percentage.
     */
    value?: number | undefined;
}
/**
 * Third-tier of goods and services.
 */
export interface ExplanationOfBenefitItemDetailSubDetail extends BackboneElement {
    /**
     * The adjudication results.
     */
    adjudication?: ExplanationOfBenefitItemAdjudication[] | undefined;
    /**
     * Examples include Medical Care, Periodontics, Renal Dialysis, Vision Coverage.
     */
    category?: CodeableConcept | undefined;
    /**
     * The element is used to indicate the current state of the adjudication overall for the claim resource, for example: the request has been held (pended) for adjudication processing, for manual review or other reasons; that it has been processed and will be paid, or the outstanding paid, as submitted (approved); that no amount will be paid (denied); or that some amount between zero and the submitted amoutn will be paid (partial).
     */
    decision?: CodeableConcept | undefined;
    /**
     * To show a 10% senior's discount, the value entered is: 0.90 (1.00 - 0.10).
     */
    factor?: number | undefined;
    /**
     * For example, in Oral whether the treatment is cosmetic or associated with TMJ, or for Medical whether the treatment was outside the clinic or outside of office hours.
     */
    modifier?: CodeableConcept[] | undefined;
    /**
     * For example, the formula: quantity * unitPrice * factor  = net. Quantity and factor are assumed to be 1 if not supplied.
     */
    net?: Money | undefined;
    /**
     * The numbers associated with notes below which apply to the adjudication of this item.
     */
    noteNumber?: number[] | undefined;
    /**
     * The amount paid by the patient, in total at the claim claim level or specifically for the item and detail level, to the provider for goods and services.
     */
    patientPaid?: Money | undefined;
    /**
     * If this is an actual service or product line, i.e. not a Group, then use code to indicate the Professional Service or Product supplied (e.g. CTP, HCPCS, USCLS, ICD10, NCPDP, DIN, RxNorm, ACHI, CCI). If a grouping item then use a group code to indicate the type of thing being grouped e.g. 'glasses' or 'compound'.
     */
    productOrService?: CodeableConcept | undefined;
    /**
     * This contains the end of a range of product, service, drug or other billing codes for the item. This element is not used when the .productOrService is a group code. This value may only be present when a .productOfService code has been provided to convey the start of the range. Typically this value may be used only with preauthorizations and not with claims.
     */
    productOrServiceEnd?: CodeableConcept | undefined;
    /**
     * For example: Neonatal program, child dental program or drug users recovery program.
     */
    programCode?: CodeableConcept[] | undefined;
    /**
     * The number of repetitions of a service or product.
     */
    quantity?: Quantity | undefined;
    /**
     * The type of revenue or cost center providing the product and/or service.
     */
    revenue?: CodeableConcept | undefined;
    /**
     * A claim detail line. Either a simple (a product or service) or a 'group' of sub-details which are simple items.
     */
    sequence: number;
    /**
     * The total of taxes applicable for this product or service.
     */
    tax?: Money | undefined;
    /**
     * Unique Device Identifiers associated with this line item.
     */
    udi?: Reference[] | undefined;
    /**
     * If the item is not a group then this is the fee for the product or service, otherwise this is the total of the fees for the details of the group.
     */
    unitPrice?: Money | undefined;
}
/**
 * Second-tier of goods and services.
 */
export interface ExplanationOfBenefitItemDetail extends BackboneElement {
    /**
     * The adjudication results.
     */
    adjudication?: ExplanationOfBenefitItemAdjudication[] | undefined;
    /**
     * Examples include: Medical Care, Periodontics, Renal Dialysis, Vision Coverage.
     */
    category?: CodeableConcept | undefined;
    /**
     * The element is used to indicate the current state of the adjudication overall for the claim resource, for example: the request has been held (pended) for adjudication processing, for manual review or other reasons; that it has been processed and will be paid, or the outstanding paid, as submitted (approved); that no amount will be paid (denied); or that some amount between zero and the submitted amoutn will be paid (partial).
     */
    decision?: CodeableConcept | undefined;
    /**
     * To show a 10% senior's discount, the value entered is: 0.90 (1.00 - 0.10).
     */
    factor?: number | undefined;
    /**
     * For example, in Oral whether the treatment is cosmetic or associated with TMJ, or for Medical whether the treatment was outside the clinic or out of office hours.
     */
    modifier?: CodeableConcept[] | undefined;
    /**
     * For example, the formula: quantity * unitPrice * factor  = net. Quantity and factor are assumed to be 1 if not supplied.
     */
    net?: Money | undefined;
    /**
     * The numbers associated with notes below which apply to the adjudication of this item.
     */
    noteNumber?: number[] | undefined;
    /**
     * The amount paid by the patient, in total at the claim claim level or specifically for the item and detail level, to the provider for goods and services.
     */
    patientPaid?: Money | undefined;
    /**
     * If this is an actual service or product line, i.e. not a Group, then use code to indicate the Professional Service or Product supplied (e.g. CTP, HCPCS, USCLS, ICD10, NCPDP, DIN, RxNorm, ACHI, CCI). If a grouping item then use a group code to indicate the type of thing being grouped e.g. 'glasses' or 'compound'.
     */
    productOrService?: CodeableConcept | undefined;
    /**
     * This contains the end of a range of product, service, drug or other billing codes for the item. This element is not used when the .productOrService is a group code. This value may only be present when a .productOfService code has been provided to convey the start of the range. Typically this value may be used only with preauthorizations and not with claims.
     */
    productOrServiceEnd?: CodeableConcept | undefined;
    /**
     * For example: Neonatal program, child dental program or drug users recovery program.
     */
    programCode?: CodeableConcept[] | undefined;
    /**
     * The number of repetitions of a service or product.
     */
    quantity?: Quantity | undefined;
    /**
     * The type of revenue or cost center providing the product and/or service.
     */
    revenue?: CodeableConcept | undefined;
    /**
     * A claim detail line. Either a simple (a product or service) or a 'group' of sub-details which are simple items.
     */
    sequence: number;
    /**
     * Third-tier of goods and services.
     */
    subDetail?: ExplanationOfBenefitItemDetailSubDetail[] | undefined;
    /**
     * The total of taxes applicable for this product or service.
     */
    tax?: Money | undefined;
    /**
     * Unique Device Identifiers associated with this line item.
     */
    udi?: Reference[] | undefined;
    /**
     * If the item is not a group then this is the fee for the product or service, otherwise this is the total of the fees for the details of the group.
     */
    unitPrice?: Money | undefined;
}
/**
 * A claim line. Either a simple (a product or service) or a 'group' of details which can also be a simple items or groups of sub-details.
 */
export interface ExplanationOfBenefitItem extends BackboneElement {
    /**
     * If this item is a group then the values here are a summary of the adjudication of the detail items. If this item is a simple product or service then this is the result of the adjudication of this item.
     */
    adjudication?: ExplanationOfBenefitItemAdjudication[] | undefined;
    /**
     * Physical location where the service is performed or applies.
     */
    bodySite?: ExplanationOfBenefitItemBodySite[] | undefined;
    /**
     * Care team members related to this service or product.
     */
    careTeamSequence?: number[] | undefined;
    /**
     * Examples include Medical Care, Periodontics, Renal Dialysis, Vision Coverage.
     */
    category?: CodeableConcept | undefined;
    /**
     * The element is used to indicate the current state of the adjudication overall for the claim resource, for example: the request has been held (pended) for adjudication processing, for manual review or other reasons; that it has been processed and will be paid, or the outstanding paid, as submitted (approved); that no amount will be paid (denied); or that some amount between zero and the submitted amoutn will be paid (partial).
     */
    decision?: CodeableConcept | undefined;
    /**
     * Second-tier of goods and services.
     */
    detail?: ExplanationOfBenefitItemDetail[] | undefined;
    /**
     * Diagnoses applicable for this service or product.
     */
    diagnosisSequence?: number[] | undefined;
    /**
     * A billed item may include goods or services provided in multiple encounters.
     */
    encounter?: Reference[] | undefined;
    /**
     * To show a 10% senior's discount, the value entered is: 0.90 (1.00 - 0.10).
     */
    factor?: number | undefined;
    /**
     * Exceptions, special conditions and supporting information applicable for this service or product.
     */
    informationSequence?: number[] | undefined;
    /**
     * Where the product or service was provided.
     */
    locationCodeableConcept?: CodeableConcept | undefined;
    /**
     * Where the product or service was provided.
     */
    locationAddress?: Address | undefined;
    /**
     * Where the product or service was provided.
     */
    locationReference?: Reference | undefined;
    /**
     * For example, in Oral whether the treatment is cosmetic or associated with TMJ, or for Medical whether the treatment was outside the clinic or out of office hours.
     */
    modifier?: CodeableConcept[] | undefined;
    /**
     * For example, the formula: quantity * unitPrice * factor  = net. Quantity and factor are assumed to be 1 if not supplied.
     */
    net?: Money | undefined;
    /**
     * The numbers associated with notes below which apply to the adjudication of this item.
     */
    noteNumber?: number[] | undefined;
    /**
     * The amount paid by the patient, in total at the claim claim level or specifically for the item and detail level, to the provider for goods and services.
     */
    patientPaid?: Money | undefined;
    /**
     * Procedures applicable for this service or product.
     */
    procedureSequence?: number[] | undefined;
    /**
     * If this is an actual service or product line, i.e. not a Group, then use code to indicate the Professional Service or Product supplied (e.g. CTP, HCPCS, USCLS, ICD10, NCPDP, DIN, RxNorm, ACHI, CCI). If a grouping item then use a group code to indicate the type of thing being grouped e.g. 'glasses' or 'compound'.
     */
    productOrService?: CodeableConcept | undefined;
    /**
     * This contains the end of a range of product, service, drug or other billing codes for the item. This element is not used when the .productOrService is a group code. This value may only be present when a .productOfService code has been provided to convey the start of the range. Typically this value may be used only with preauthorizations and not with claims.
     */
    productOrServiceEnd?: CodeableConcept | undefined;
    /**
     * For example: Neonatal program, child dental program or drug users recovery program.
     */
    programCode?: CodeableConcept[] | undefined;
    /**
     * The number of repetitions of a service or product.
     */
    quantity?: Quantity | undefined;
    /**
     * The type of revenue or cost center providing the product and/or service.
     */
    revenue?: CodeableConcept | undefined;
    /**
     * A number to uniquely identify item entries.
     */
    sequence: number;
    /**
     * The date or dates when the service or product was supplied, performed or completed.
     */
    servicedDate?: string | undefined;
    _servicedDate?: Element | undefined;
    /**
     * The date or dates when the service or product was supplied, performed or completed.
     */
    servicedPeriod?: Period | undefined;
    /**
     * The total of taxes applicable for this product or service.
     */
    tax?: Money | undefined;
    /**
     * Unique Device Identifiers associated with this line item.
     */
    udi?: Reference[] | undefined;
    /**
     * If the item is not a group then this is the fee for the product or service, otherwise this is the total of the fees for the details of the group.
     */
    unitPrice?: Money | undefined;
}
/**
 * Physical location where the service is performed or applies.
 */
export interface ExplanationOfBenefitAddItemBodySite extends BackboneElement {
    /**
     * For example, providing a tooth code allows an insurer to identify a provider performing a filling on a tooth that was previously removed.
     */
    site: CodeableReference[];
    /**
     * A region or surface of the bodySite, e.g. limb region or tooth surface(s).
     */
    subSite?: CodeableConcept[] | undefined;
}
/**
 * The third-tier service adjudications for payor added services.
 */
export interface ExplanationOfBenefitAddItemDetailSubDetail extends BackboneElement {
    /**
     * The adjudication results.
     */
    adjudication?: ExplanationOfBenefitItemAdjudication[] | undefined;
    /**
     * The element is used to indicate the current state of the adjudication overall for the claim resource, for example: the request has been held (pended) for adjudication processing, for manual review or other reasons; that it has been processed and will be paid, or the outstanding paid, as submitted (approved); that no amount will be paid (denied); or that some amount between zero and the submitted amoutn will be paid (partial).
     */
    decision?: CodeableConcept | undefined;
    /**
     * To show a 10% senior's discount, the value entered is: 0.90 (1.00 - 0.10).
     */
    factor?: number | undefined;
    /**
     * For example, in Oral whether the treatment is cosmetic or associated with TMJ, or for Medical whether the treatment was outside the clinic or out of office hours.
     */
    modifier?: CodeableConcept[] | undefined;
    /**
     * For example, the formula: quantity * unitPrice * factor  = net. Quantity and factor are assumed to be 1 if not supplied.
     */
    net?: Money | undefined;
    /**
     * The numbers associated with notes below which apply to the adjudication of this item.
     */
    noteNumber?: number[] | undefined;
    /**
     * The amount paid by the patient, in total at the claim claim level or specifically for the item and detail level, to the provider for goods and services.
     */
    patientPaid?: Money | undefined;
    /**
     * If this is an actual service or product line, i.e. not a Group, then use code to indicate the Professional Service or Product supplied (e.g. CTP, HCPCS, USCLS, ICD10, NCPDP, DIN, RxNorm, ACHI, CCI). If a grouping item then use a group code to indicate the type of thing being grouped e.g. 'glasses' or 'compound'.
     */
    productOrService?: CodeableConcept | undefined;
    /**
     * This contains the end of a range of product, service, drug or other billing codes for the item. This element is not used when the .productOrService is a group code. This value may only be present when a .productOfService code has been provided to convey the start of the range. Typically this value may be used only with preauthorizations and not with claims.
     */
    productOrServiceEnd?: CodeableConcept | undefined;
    /**
     * The number of repetitions of a service or product.
     */
    quantity?: Quantity | undefined;
    /**
     * The type of revenue or cost center providing the product and/or service.
     */
    revenue?: CodeableConcept | undefined;
    /**
     * The total of taxes applicable for this product or service.
     */
    tax?: Money | undefined;
    /**
     * If the item is not a group then this is the fee for the product or service, otherwise this is the total of the fees for the details of the group.
     */
    unitPrice?: Money | undefined;
}
/**
 * The second-tier service adjudications for payor added services.
 */
export interface ExplanationOfBenefitAddItemDetail extends BackboneElement {
    /**
     * The adjudication results.
     */
    adjudication?: ExplanationOfBenefitItemAdjudication[] | undefined;
    /**
     * The element is used to indicate the current state of the adjudication overall for the claim resource, for example: the request has been held (pended) for adjudication processing, for manual review or other reasons; that it has been processed and will be paid, or the outstanding paid, as submitted (approved); that no amount will be paid (denied); or that some amount between zero and the submitted amoutn will be paid (partial).
     */
    decision?: CodeableConcept | undefined;
    /**
     * To show a 10% senior's discount, the value entered is: 0.90 (1.00 - 0.10).
     */
    factor?: number | undefined;
    /**
     * For example, in Oral whether the treatment is cosmetic or associated with TMJ, or for Medical whether the treatment was outside the clinic or out of office hours.
     */
    modifier?: CodeableConcept[] | undefined;
    /**
     * For example, the formula: quantity * unitPrice * factor  = net. Quantity and factor are assumed to be 1 if not supplied.
     */
    net?: Money | undefined;
    /**
     * The numbers associated with notes below which apply to the adjudication of this item.
     */
    noteNumber?: number[] | undefined;
    /**
     * The amount paid by the patient, in total at the claim claim level or specifically for the item and detail level, to the provider for goods and services.
     */
    patientPaid?: Money | undefined;
    /**
     * If this is an actual service or product line, i.e. not a Group, then use code to indicate the Professional Service or Product supplied (e.g. CTP, HCPCS, USCLS, ICD10, NCPDP, DIN, RxNorm, ACHI, CCI). If a grouping item then use a group code to indicate the type of thing being grouped e.g. 'glasses' or 'compound'.
     */
    productOrService?: CodeableConcept | undefined;
    /**
     * This contains the end of a range of product, service, drug or other billing codes for the item. This element is not used when the .productOrService is a group code. This value may only be present when a .productOfService code has been provided to convey the start of the range. Typically this value may be used only with preauthorizations and not with claims.
     */
    productOrServiceEnd?: CodeableConcept | undefined;
    /**
     * The number of repetitions of a service or product.
     */
    quantity?: Quantity | undefined;
    /**
     * The type of revenue or cost center providing the product and/or service.
     */
    revenue?: CodeableConcept | undefined;
    /**
     * The third-tier service adjudications for payor added services.
     */
    subDetail?: ExplanationOfBenefitAddItemDetailSubDetail[] | undefined;
    /**
     * The total of taxes applicable for this product or service.
     */
    tax?: Money | undefined;
    /**
     * If the item is not a group then this is the fee for the product or service, otherwise this is the total of the fees for the details of the group.
     */
    unitPrice?: Money | undefined;
}
/**
 * The first-tier service adjudications for payor added product or service lines.
 */
export interface ExplanationOfBenefitAddItem extends BackboneElement {
    /**
     * The adjudication results.
     */
    adjudication?: ExplanationOfBenefitItemAdjudication[] | undefined;
    /**
     * Physical location where the service is performed or applies.
     */
    bodySite?: ExplanationOfBenefitAddItemBodySite[] | undefined;
    /**
     * The element is used to indicate the current state of the adjudication overall for the claim resource, for example: the request has been held (pended) for adjudication processing, for manual review or other reasons; that it has been processed and will be paid, or the outstanding paid, as submitted (approved); that no amount will be paid (denied); or that some amount between zero and the submitted amoutn will be paid (partial).
     */
    decision?: CodeableConcept | undefined;
    /**
     * The second-tier service adjudications for payor added services.
     */
    detail?: ExplanationOfBenefitAddItemDetail[] | undefined;
    /**
     * The sequence number of the details within the claim item which this line is intended to replace.
     */
    detailSequence?: number[] | undefined;
    /**
     * To show a 10% senior's discount, the value entered is: 0.90 (1.00 - 0.10).
     */
    factor?: number | undefined;
    /**
     * Claim items which this service line is intended to replace.
     */
    itemSequence?: number[] | undefined;
    /**
     * Where the product or service was provided.
     */
    locationCodeableConcept?: CodeableConcept | undefined;
    /**
     * Where the product or service was provided.
     */
    locationAddress?: Address | undefined;
    /**
     * Where the product or service was provided.
     */
    locationReference?: Reference | undefined;
    /**
     * For example, in Oral whether the treatment is cosmetic or associated with TMJ, or for Medical whether the treatment was outside the clinic or out of office hours.
     */
    modifier?: CodeableConcept[] | undefined;
    /**
     * For example, the formula: quantity * unitPrice * factor  = net. Quantity and factor are assumed to be 1 if not supplied.
     */
    net?: Money | undefined;
    /**
     * The numbers associated with notes below which apply to the adjudication of this item.
     */
    noteNumber?: number[] | undefined;
    /**
     * The amount paid by the patient, in total at the claim claim level or specifically for the item and detail level, to the provider for goods and services.
     */
    patientPaid?: Money | undefined;
    /**
     * If this is an actual service or product line, i.e. not a Group, then use code to indicate the Professional Service or Product supplied (e.g. CTP, HCPCS, USCLS, ICD10, NCPDP, DIN, RxNorm, ACHI, CCI). If a grouping item then use a group code to indicate the type of thing being grouped e.g. 'glasses' or 'compound'.
     */
    productOrService?: CodeableConcept | undefined;
    /**
     * This contains the end of a range of product, service, drug or other billing codes for the item. This element is not used when the .productOrService is a group code. This value may only be present when a .productOfService code has been provided to convey the start of the range. Typically this value may be used only with preauthorizations and not with claims.
     */
    productOrServiceEnd?: CodeableConcept | undefined;
    /**
     * For example: Neonatal program, child dental program or drug users recovery program.
     */
    programCode?: CodeableConcept[] | undefined;
    /**
     * The providers who are authorized for the services rendered to the patient.
     */
    provider?: Reference[] | undefined;
    /**
     * The number of repetitions of a service or product.
     */
    quantity?: Quantity | undefined;
    /**
     * The type of revenue or cost center providing the product and/or service.
     */
    revenue?: CodeableConcept | undefined;
    /**
     * The date or dates when the service or product was supplied, performed or completed.
     */
    servicedDate?: string | undefined;
    _servicedDate?: Element | undefined;
    /**
     * The date or dates when the service or product was supplied, performed or completed.
     */
    servicedPeriod?: Period | undefined;
    /**
     * The sequence number of the sub-details woithin the details within the claim item which this line is intended to replace.
     */
    subDetailSequence?: number[] | undefined;
    /**
     * The total of taxes applicable for this product or service.
     */
    tax?: Money | undefined;
    /**
     * If the item is not a group then this is the fee for the product or service, otherwise this is the total of the fees for the details of the group.
     */
    unitPrice?: Money | undefined;
}
/**
 * Totals for amounts submitted, co-pays, benefits payable etc.
 */
export interface ExplanationOfBenefitTotal extends BackboneElement {
    /**
     * Monetary total amount associated with the category.
     */
    amount: Money;
    /**
     * For example, codes indicating: Co-Pay, deductible, eligible, benefit, tax, etc.
     */
    category: CodeableConcept;
}
/**
 * Payment details for the adjudication of the claim.
 */
export interface ExplanationOfBenefitPayment extends BackboneElement {
    /**
     * Insurers will deduct amounts owing from the provider (adjustment), such as a prior overpayment, from the amount owing to the provider (benefits payable) when payment is made to the provider.
     */
    adjustment?: Money | undefined;
    /**
     * Reason for the payment adjustment.
     */
    adjustmentReason?: CodeableConcept | undefined;
    /**
     * Benefits payable less any payment adjustment.
     */
    amount?: Money | undefined;
    /**
     * Estimated date the payment will be issued or the actual issue date of payment.
     */
    date?: string | undefined;
    _date?: Element | undefined;
    /**
     * For example: EFT number or check number.
     */
    identifier?: Identifier | undefined;
    /**
     * Whether this represents partial or complete payment of the benefits payable.
     */
    type?: CodeableConcept | undefined;
}
/**
 * A note that describes or explains adjudication results in a human readable form.
 */
export interface ExplanationOfBenefitProcessNote extends BackboneElement {
    /**
     * Only required if the language is different from the resource language.
     */
    language?: CodeableConcept | undefined;
    /**
     * A number to uniquely identify a note entry.
     */
    number?: number | undefined;
    /**
     * The explanation or description associated with the processing.
     */
    text?: string | undefined;
    _text?: Element | undefined;
    /**
     * The business purpose of the note text.
     */
    type?: ('display' | 'print' | 'printoper') | undefined;
    _type?: Element | undefined;
}
/**
 * Benefits Used to date.
 */
export interface ExplanationOfBenefitBenefitBalanceFinancial extends BackboneElement {
    /**
     * The quantity of the benefit which is permitted under the coverage.
     */
    allowedUnsignedInt?: number | undefined;
    /**
     * The quantity of the benefit which is permitted under the coverage.
     */
    allowedString?: string | undefined;
    _allowedString?: Element | undefined;
    /**
     * The quantity of the benefit which is permitted under the coverage.
     */
    allowedMoney?: Money | undefined;
    /**
     * For example: deductible, visits, benefit amount.
     */
    type: CodeableConcept;
    /**
     * The quantity of the benefit which have been consumed to date.
     */
    usedUnsignedInt?: number | undefined;
    /**
     * The quantity of the benefit which have been consumed to date.
     */
    usedMoney?: Money | undefined;
}
/**
 * Balance by Benefit Category.
 */
export interface ExplanationOfBenefitBenefitBalance extends BackboneElement {
    /**
     * Examples include Medical Care, Periodontics, Renal Dialysis, Vision Coverage.
     */
    category: CodeableConcept;
    /**
     * For example, 'DENT2 covers 100% of basic, 50% of major but excludes Ortho, Implants and Cosmetic services'.
     */
    description?: string | undefined;
    _description?: Element | undefined;
    /**
     * True if the indicated class of service is excluded from the plan, missing or False indicates the product or service is included in the coverage.
     */
    excluded?: boolean | undefined;
    _excluded?: Element | undefined;
    /**
     * Benefits Used to date.
     */
    financial?: ExplanationOfBenefitBenefitBalanceFinancial[] | undefined;
    /**
     * For example: MED01, or DENT2.
     */
    name?: string | undefined;
    _name?: Element | undefined;
    /**
     * Is a flag to indicate whether the benefits refer to in-network providers or out-of-network providers.
     */
    network?: CodeableConcept | undefined;
    /**
     * The term or period of the values such as 'maximum lifetime benefit' or 'maximum annual visits'.
     */
    term?: CodeableConcept | undefined;
    /**
     * Indicates if the benefits apply to an individual or to the family.
     */
    unit?: CodeableConcept | undefined;
}
/**
 * This resource provides: the claim details; adjudication details from the processing of a Claim; and optionally account balance information, for informing the subscriber of the benefits provided.
 */
export interface ExplanationOfBenefit extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'ExplanationOfBenefit';
    /**
     * Details of a accident which resulted in injuries which required the products and services listed in the claim.
     */
    accident?: ExplanationOfBenefitAccident | undefined;
    /**
     * The first-tier service adjudications for payor added product or service lines.
     */
    addItem?: ExplanationOfBenefitAddItem[] | undefined;
    /**
     * The adjudication results which are presented at the header level rather than at the line-item or add-item levels.
     */
    adjudication?: ExplanationOfBenefitItemAdjudication[] | undefined;
    /**
     * Balance by Benefit Category.
     */
    benefitBalance?: ExplanationOfBenefitBenefitBalance[] | undefined;
    /**
     * Not applicable when use=claim.
     */
    benefitPeriod?: Period | undefined;
    /**
     * Typically this would be today or in the past for a claim, and today or in the future for preauthorizations and prodeterminations. Typically line item dates of service should fall within the billing period if one is specified.
     */
    billablePeriod?: Period | undefined;
    /**
     * The members of the team who provided the products and services.
     */
    careTeam?: ExplanationOfBenefitCareTeam[] | undefined;
    /**
     * The business identifier for the instance of the adjudication request: claim predetermination or preauthorization.
     */
    claim?: Reference | undefined;
    /**
     * The business identifier for the instance of the adjudication response: claim, predetermination or preauthorization response.
     */
    claimResponse?: Reference | undefined;
    /**
     * This field is independent of the date of creation of the resource as it may reflect the creation date of a source document prior to digitization. Typically for claims all services must be completed as of this date.
     */
    created: string;
    _created?: Element | undefined;
    /**
     * The element is used to indicate the current state of the adjudication overall for the claim resource, for example: the request has been held (pended) for adjudication processing, for manual review or other reasons; that it has been processed and will be paid, or the outstanding paid, as submitted (approved); that no amount will be paid (denied); or that some amount between zero and the submitted amoutn will be paid (partial).
     */
    decision?: CodeableConcept | undefined;
    /**
     * Information about diagnoses relevant to the claim items.
     */
    diagnosis?: ExplanationOfBenefitDiagnosis[] | undefined;
    /**
     * For example DRG (Diagnosis Related Group) or a bundled billing code. A patient may have a diagnosis of a Myocardial Infarction and a DRG for HeartAttack would be assigned. The Claim item (and possible subsequent claims) would refer to the DRG for those line items that were for services related to the heart attack event.
     */
    diagnosisRelatedGroup?: CodeableConcept | undefined;
    /**
     * A human readable description of the status of the adjudication.
     */
    disposition?: string | undefined;
    _disposition?: Element | undefined;
    /**
     * A billed item may include goods or services provided in multiple encounters.
     */
    encounter?: Reference[] | undefined;
    /**
     * Individual who created the claim, predetermination or preauthorization.
     */
    enterer?: Reference | undefined;
    /**
     * Facility where the services were provided.
     */
    facility?: Reference | undefined;
    /**
     * Needed to permit insurers to include the actual form.
     */
    form?: Attachment | undefined;
    /**
     * May be needed to identify specific jurisdictional forms.
     */
    formCode?: CodeableConcept | undefined;
    /**
     * Fund would be release by a future claim quoting the preAuthRef of this response. Examples of values include: provider, patient, none.
     */
    fundsReserve?: CodeableConcept | undefined;
    /**
     * This field is only used for preauthorizations.
     */
    fundsReserveRequested?: CodeableConcept | undefined;
    /**
     * A unique identifier assigned to this explanation of benefit.
     */
    identifier?: Identifier[] | undefined;
    /**
     * All insurance coverages for the patient which may be applicable for reimbursement, of the products and services listed in the claim, are typically provided in the claim to allow insurers to confirm the ordering of the insurance coverages relative to local 'coordination of benefit' rules. One coverage (and only one) with 'focal=true' is to be used in the adjudication of this claim. Coverages appearing before the focal Coverage in the list, and where 'Coverage.subrogation=false', should provide a reference to the ClaimResponse containing the adjudication results of the prior claim.
     */
    insurance?: ExplanationOfBenefitInsurance[] | undefined;
    /**
     * The party responsible for authorization, adjudication and reimbursement.
     */
    insurer?: Reference | undefined;
    /**
     * A claim line. Either a simple (a product or service) or a 'group' of details which can also be a simple items or groups of sub-details.
     */
    item?: ExplanationOfBenefitItem[] | undefined;
    /**
     * For example, a physician may prescribe a medication which the pharmacy determines is contraindicated, or for which the patient has an intolerance, and therefor issues a new prescription for an alternate medication which has the same therapeutic intent. The prescription from the pharmacy becomes the 'prescription' and that from the physician becomes the 'original prescription'.
     */
    originalPrescription?: Reference | undefined;
    /**
     * The resource may be used to indicate that the Claim/Preauthorization/Pre-determination has been received but processing has not begun (queued); that it has been processed and one or more errors have been detected (error); no errors were detected and some of the adjudication processing has been performed (partial); or all of the adjudication processing has completed without errors (complete).
     */
    outcome: 'queued' | 'complete' | 'error' | 'partial';
    _outcome?: Element | undefined;
    /**
     * The party to whom the professional services and/or products have been supplied or are being considered and for whom actual for forecast reimbursement is sought.
     */
    patient: Reference;
    /**
     * The amount paid by the patient, in total at the claim claim level or specifically for the item and detail level, to the provider for goods and services.
     */
    patientPaid?: Money | undefined;
    /**
     * Often providers agree to receive the benefits payable to reduce the near-term costs to the patient. The insurer may decline to pay the provider and may choose to pay the subscriber instead.
     */
    payee?: ExplanationOfBenefitPayee | undefined;
    /**
     * Payment details for the adjudication of the claim.
     */
    payment?: ExplanationOfBenefitPayment | undefined;
    /**
     * This value is only present on preauthorization adjudications.
     */
    preAuthRef?: string[] | undefined;
    _preAuthRef?: Element[] | undefined;
    /**
     * This value is only present on preauthorization adjudications.
     */
    preAuthRefPeriod?: Period[] | undefined;
    /**
     * This indicates the relative order of a series of EOBs related to different coverages for the same suite of services.
     */
    precedence?: number | undefined;
    /**
     * Prescription is the document/authorization given to the claim author for them to provide products and services for which consideration (reimbursement) is sought. Could be a RX for medications, an 'order' for oxygen or wheelchair or physiotherapy treatments.
     */
    prescription?: Reference | undefined;
    /**
     * If a claim processor is unable to complete the processing as per the priority then they should generate an error and not process the request.
     */
    priority?: CodeableConcept | undefined;
    /**
     * Procedures performed on the patient relevant to the billing items with the claim.
     */
    procedure?: ExplanationOfBenefitProcedure[] | undefined;
    /**
     * A note that describes or explains adjudication results in a human readable form.
     */
    processNote?: ExplanationOfBenefitProcessNote[] | undefined;
    /**
     * This party is responsible for the claim but not necessarily professionally responsible for the provision of the individual products and services listed below. This field is the Billing Provider, for example, a facility, provider group, lab or practitioner.
     */
    provider?: Reference | undefined;
    /**
     * The referral resource which lists the date, practitioner, reason and other supporting information.
     */
    referral?: Reference | undefined;
    /**
     * For example,  for the original treatment and follow-up exams.
     */
    related?: ExplanationOfBenefitRelated[] | undefined;
    /**
     * This element is labeled as a modifier because the status contains codes that mark the resource as not currently valid.
     */
    status: 'active' | 'cancelled' | 'draft' | 'entered-in-error';
    _status?: Element | undefined;
    /**
     * This may contain the local bill type codes such as the US UB-04 bill type code.
     */
    subType?: CodeableConcept | undefined;
    /**
     * Often there are multiple jurisdiction specific valuesets which are required.
     */
    supportingInfo?: ExplanationOfBenefitSupportingInfo[] | undefined;
    /**
     * Totals for amounts submitted, co-pays, benefits payable etc.
     */
    total?: ExplanationOfBenefitTotal[] | undefined;
    /**
     * The majority of jurisdictions use: oral, pharmacy, vision, professional and institutional, or variants on those terms, as the general styles of claims. The valueset is extensible to accommodate other jurisdictional requirements.
     */
    type: CodeableConcept;
    /**
     * A code to indicate whether the nature of the request is: Claim - A request to an Insurer to adjudicate the supplied charges for health care goods and services under the identified policy and to pay the determined Benefit amount, if any; Preauthorization - A request to an Insurer to adjudicate the supplied proposed future charges for health care goods and services under the identified policy and to approve the services and provide the expected benefit amounts and potentially to reserve funds to pay the benefits when Claims for the indicated services are later submitted; or, Pre-determination - A request to an Insurer to adjudicate the supplied 'what if' charges for health care goods and services under the identified policy and report back what the Benefit payable would be had the services actually been provided.
     */
    use: 'claim' | 'preauthorization' | 'predetermination';
    _use?: Element | undefined;
}
/**
 * The significant Conditions (or condition) that the family member had. This is a repeating section to allow a system to represent more than one condition per resource, though there is nothing stopping multiple resources - one per condition.
 */
export interface FamilyMemberHistoryCondition extends BackboneElement {
    /**
     * The actual condition specified. Could be a coded condition (like MI or Diabetes) or a less specific string like 'cancer' depending on how much is known about the condition and the capabilities of the creating system.
     */
    code: CodeableConcept;
    /**
     * This condition contributed to the cause of death of the related person. If contributedToDeath is not populated, then it is unknown.
     */
    contributedToDeath?: boolean | undefined;
    _contributedToDeath?: Element | undefined;
    /**
     * An area where general notes can be placed about this specific condition.
     */
    note?: Annotation[] | undefined;
    /**
     * Either the age of onset, range of approximate age or descriptive string can be recorded.  For conditions with multiple occurrences, this describes the first known occurrence.
     */
    onsetAge?: Age | undefined;
    /**
     * Either the age of onset, range of approximate age or descriptive string can be recorded.  For conditions with multiple occurrences, this describes the first known occurrence.
     */
    onsetRange?: Range | undefined;
    /**
     * Either the age of onset, range of approximate age or descriptive string can be recorded.  For conditions with multiple occurrences, this describes the first known occurrence.
     */
    onsetPeriod?: Period | undefined;
    /**
     * Either the age of onset, range of approximate age or descriptive string can be recorded.  For conditions with multiple occurrences, this describes the first known occurrence.
     */
    onsetString?: string | undefined;
    _onsetString?: Element | undefined;
    /**
     * Indicates what happened following the condition.  If the condition resulted in death, deceased date is captured on the relation.
     */
    outcome?: CodeableConcept | undefined;
}
/**
 * The significant Procedures (or procedure) that the family member had. This is a repeating section to allow a system to represent more than one procedure per resource, though there is nothing stopping multiple resources - one per procedure.
 */
export interface FamilyMemberHistoryProcedure extends BackboneElement {
    /**
     * The actual procedure specified. Could be a coded procedure or a less specific string depending on how much is known about the procedure and the capabilities of the creating system.
     */
    code: CodeableConcept;
    /**
     * This procedure contributed to the cause of death of the related person. If contributedToDeath is not populated, then it is unknown.
     */
    contributedToDeath?: boolean | undefined;
    _contributedToDeath?: Element | undefined;
    /**
     * An area where general notes can be placed about this specific procedure.
     */
    note?: Annotation[] | undefined;
    /**
     * Indicates what happened following the procedure. If the procedure resulted in death, deceased date is captured on the relation.
     */
    outcome?: CodeableConcept | undefined;
    /**
     * Estimated or actual date, date-time, period, or age when the procedure was performed. Allows a period to support complex procedures that span more than one date, and also allows for the length of the procedure to be captured.
     */
    performedAge?: Age | undefined;
    /**
     * Estimated or actual date, date-time, period, or age when the procedure was performed. Allows a period to support complex procedures that span more than one date, and also allows for the length of the procedure to be captured.
     */
    performedRange?: Range | undefined;
    /**
     * Estimated or actual date, date-time, period, or age when the procedure was performed. Allows a period to support complex procedures that span more than one date, and also allows for the length of the procedure to be captured.
     */
    performedPeriod?: Period | undefined;
    /**
     * Estimated or actual date, date-time, period, or age when the procedure was performed. Allows a period to support complex procedures that span more than one date, and also allows for the length of the procedure to be captured.
     */
    performedString?: string | undefined;
    _performedString?: Element | undefined;
    /**
     * Estimated or actual date, date-time, period, or age when the procedure was performed. Allows a period to support complex procedures that span more than one date, and also allows for the length of the procedure to be captured.
     */
    performedDateTime?: string | undefined;
    _performedDateTime?: Element | undefined;
}
/**
 * Significant health conditions for a person related to the patient relevant in the context of care for the patient.
 */
export interface FamilyMemberHistory extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'FamilyMemberHistory';
    /**
     * use estimatedAge to indicate whether the age is actual or not.
     */
    ageAge?: Age | undefined;
    /**
     * use estimatedAge to indicate whether the age is actual or not.
     */
    ageRange?: Range | undefined;
    /**
     * use estimatedAge to indicate whether the age is actual or not.
     */
    ageString?: string | undefined;
    _ageString?: Element | undefined;
    /**
     * The actual or approximate date of birth of the relative.
     */
    bornPeriod?: Period | undefined;
    /**
     * The actual or approximate date of birth of the relative.
     */
    bornDate?: string | undefined;
    _bornDate?: Element | undefined;
    /**
     * The actual or approximate date of birth of the relative.
     */
    bornString?: string | undefined;
    _bornString?: Element | undefined;
    /**
     * The significant Conditions (or condition) that the family member had. This is a repeating section to allow a system to represent more than one condition per resource, though there is nothing stopping multiple resources - one per condition.
     */
    condition?: FamilyMemberHistoryCondition[] | undefined;
    /**
     * Describes why the family member's history is not available.
     */
    dataAbsentReason?: CodeableConcept | undefined;
    /**
     * This should be captured even if the same as the date on the List aggregating the full family history.
     */
    date?: string | undefined;
    _date?: Element | undefined;
    /**
     * Deceased flag or the actual or approximate age of the relative at the time of death for the family member history record.
     */
    deceasedBoolean?: boolean | undefined;
    _deceasedBoolean?: Element | undefined;
    /**
     * Deceased flag or the actual or approximate age of the relative at the time of death for the family member history record.
     */
    deceasedAge?: Age | undefined;
    /**
     * Deceased flag or the actual or approximate age of the relative at the time of death for the family member history record.
     */
    deceasedRange?: Range | undefined;
    /**
     * Deceased flag or the actual or approximate age of the relative at the time of death for the family member history record.
     */
    deceasedDate?: string | undefined;
    _deceasedDate?: Element | undefined;
    /**
     * Deceased flag or the actual or approximate age of the relative at the time of death for the family member history record.
     */
    deceasedString?: string | undefined;
    _deceasedString?: Element | undefined;
    /**
     * This element is labeled as a modifier because the fact that age is estimated can/should change the results of any algorithm that calculates based on the specified age.
     */
    estimatedAge?: boolean | undefined;
    _estimatedAge?: Element | undefined;
    /**
     * This is a business identifier, not a resource identifier (see [discussion](resource.html#identifiers)).  It is best practice for the identifier to only appear on a single resource instance, however business practices may occasionally dictate that multiple resource instances with the same identifier can exist - possibly even with different resource types.  For example, multiple Patient and a Person resource instance might share the same social insurance number.
     */
    identifier?: Identifier[] | undefined;
    /**
     * The URL pointing to a FHIR-defined protocol, guideline, orderset or other definition that is adhered to in whole or in part by this FamilyMemberHistory.
     */
    instantiatesCanonical?: string[] | undefined;
    _instantiatesCanonical?: Element[] | undefined;
    /**
     * This might be an HTML page, PDF, etc. or could just be a non-resolvable URI identifier.
     */
    instantiatesUri?: string[] | undefined;
    _instantiatesUri?: Element[] | undefined;
    /**
     * This will either be a name or a description; e.g. "Aunt Susan", "my cousin with the red hair".
     */
    name?: string | undefined;
    _name?: Element | undefined;
    /**
     * This property allows a non condition-specific note to the made about the related person. Ideally, the note would be in the condition property, but this is not always possible.
     */
    note?: Annotation[] | undefined;
    /**
     * This is not the family member.
     */
    patient: Reference;
    /**
     * The significant Procedures (or procedure) that the family member had. This is a repeating section to allow a system to represent more than one procedure per resource, though there is nothing stopping multiple resources - one per procedure.
     */
    procedure?: FamilyMemberHistoryProcedure[] | undefined;
    /**
     * Textual reasons can be captured using reasonCode.text.
     */
    reason?: CodeableReference[] | undefined;
    /**
     * The type of relationship this person has to the patient (father, mother, brother etc.).
     */
    relationship: CodeableConcept;
    /**
     * This element should ideally reflect whether the individual is genetically male or female.  However, as reported information based on the knowledge of the patient or reporting friend/relative, there may be situations where the reported sex might not be totally accurate.  E.g. 'Aunt Sue' might be XY rather than XX.  Questions soliciting this information should be phrased to encourage capture of genetic sex where known.  However, systems performing analysis should also allow for the possibility of imprecision with this element.
     */
    sex?: CodeableConcept | undefined;
    /**
     * This element is labeled as a modifier because the status contains codes that mark the resource as not currently valid.
     */
    status: 'partial' | 'completed' | 'entered-in-error' | 'health-unknown';
    _status?: Element | undefined;
}
/**
 * Prospective warnings of potential issues when providing care to the patient.
 */
export interface Flag extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'Flag';
    /**
     * The person, organization or device that created the flag.
     */
    author?: Reference | undefined;
    /**
     * The value set will often need to be adjusted based on local business rules and usage context.
     */
    category?: CodeableConcept[] | undefined;
    /**
     * If non-coded, use CodeableConcept.text.  This element should always be included in the narrative.
     */
    code: CodeableConcept;
    /**
     * If both Flag.encounter and Flag.period are valued, then Flag.period.start shall not be before Encounter.period.start and Flag.period.end shall not be after Encounter.period.end.
     */
    encounter?: Reference | undefined;
    /**
     * This is a business identifier, not a resource identifier (see [discussion](resource.html#identifiers)).  It is best practice for the identifier to only appear on a single resource instance, however business practices may occasionally dictate that multiple resource instances with the same identifier can exist - possibly even with different resource types.  For example, multiple Patient and a Person resource instance might share the same social insurance number.
     */
    identifier?: Identifier[] | undefined;
    /**
     * The period of time from the activation of the flag to inactivation of the flag. If the flag is active, the end of the period should be unspecified.
     */
    period?: Period | undefined;
    /**
     * This element is labeled as a modifier because the status contains codes that mark the resource as not currently valid.
     */
    status: 'active' | 'inactive' | 'entered-in-error';
    _status?: Element | undefined;
    /**
     * The patient, location, group, organization, or practitioner etc. this is about record this flag is associated with.
     */
    subject: Reference;
}
/**
 * This resource describes a product or service that is available through a program and includes the conditions and constraints of availability.  All of the information in this resource is specific to the inclusion of the item in the formulary and is not inherent to the item itself.
 */
export interface FormularyItem extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'FormularyItem';
    /**
     * A code (or set of codes) that specify the product or service that is identified by this formulary item.
     */
    code?: CodeableConcept | undefined;
    /**
     * Business identifier for this formulary item.
     */
    identifier?: Identifier[] | undefined;
    /**
     * This status is intended to identify if the formulary item in a local system is in active use within the formulary.  For example, a formulary might include a medicinal product and the status relates to its inclusion.  At some point in the future, if the status of the formulary item is changed to "inactive" that means that it is no longer valid in the formulary.  But the product itself might still be in active use and might be included in other formularies.
     */
    status?: ('active' | 'entered-in-error' | 'inactive') | undefined;
    _status?: Element | undefined;
}
/**
 * Inputs for the analysis event.
 */
export interface GenomicStudyAnalysisInput extends BackboneElement {
    /**
     * File containing input data.
     */
    file?: Reference | undefined;
    /**
     * The analysis event or other GenomicStudy that generated this input file.
     */
    generatedByIdentifier?: Identifier | undefined;
    /**
     * The analysis event or other GenomicStudy that generated this input file.
     */
    generatedByReference?: Reference | undefined;
    /**
     * Type of input data, e.g., BAM, CRAM, or FASTA.
     */
    type?: CodeableConcept | undefined;
}
/**
 * Outputs for the analysis event.
 */
export interface GenomicStudyAnalysisOutput extends BackboneElement {
    /**
     * File containing output data.
     */
    file?: Reference | undefined;
    /**
     * Type of output data, e.g., VCF, MAF, or BAM.
     */
    type?: CodeableConcept | undefined;
}
/**
 * Performer for the analysis event.
 */
export interface GenomicStudyAnalysisPerformer extends BackboneElement {
    /**
     * The organization, healthcare professional, or others who participated in performing this analysis.
     */
    actor?: Reference | undefined;
    /**
     * Role of the actor for this analysis.
     */
    role?: CodeableConcept | undefined;
}
/**
 * Devices used for the analysis (e.g., instruments, software), with settings and parameters.
 */
export interface GenomicStudyAnalysisDevice extends BackboneElement {
    /**
     * Device used for the analysis.
     */
    device?: Reference | undefined;
    /**
     * Specific function for the device used for the analysis.
     */
    function?: CodeableConcept | undefined;
}
/**
 * The details about a specific analysis that was performed in this GenomicStudy.
 */
export interface GenomicStudyAnalysis extends BackboneElement {
    /**
     * Type of the genomic changes studied in the analysis, e.g., DNA, RNA, or amino acid change.
     */
    changeType?: CodeableConcept[] | undefined;
    /**
     * The date of the analysis event.
     */
    date?: string | undefined;
    _date?: Element | undefined;
    /**
     * Devices used for the analysis (e.g., instruments, software), with settings and parameters.
     */
    device?: GenomicStudyAnalysisDevice[] | undefined;
    /**
     * The reference genome build that is used in this analysis.
     */
    genomeBuild?: CodeableConcept | undefined;
    /**
     * Identifiers for the analysis event.
     */
    identifier?: Identifier[] | undefined;
    /**
     * Inputs for the analysis event.
     */
    input?: GenomicStudyAnalysisInput[] | undefined;
    /**
     * The defined protocol that describes the analysis.
     */
    instantiatesCanonical?: string | undefined;
    _instantiatesCanonical?: Element | undefined;
    /**
     * The URL pointing to an externally maintained protocol that describes the analysis.
     */
    instantiatesUri?: string | undefined;
    _instantiatesUri?: Element | undefined;
    /**
     * Type of the methods used in the analysis, e.g., Fluorescence in situ hybridization (FISH), Karyotyping, or Microsatellite instability testing (MSI).
     */
    methodType?: CodeableConcept[] | undefined;
    /**
     * Any notes capture with the analysis event.
     */
    note?: Annotation[] | undefined;
    /**
     * Outputs for the analysis event.
     */
    output?: GenomicStudyAnalysisOutput[] | undefined;
    /**
     * Performer for the analysis event.
     */
    performer?: GenomicStudyAnalysisPerformer[] | undefined;
    /**
     * The protocol that was performed for the analysis event.
     */
    protocolPerformed?: Reference | undefined;
    /**
     * Genomic regions actually called in the analysis event (BED file).
     */
    regionsCalled?: Reference[] | undefined;
    /**
     * The genomic regions to be studied in the analysis (BED file).
     */
    regionsStudied?: Reference[] | undefined;
    /**
     * The specimen used in the analysis event.
     */
    specimen?: Reference[] | undefined;
    /**
     * The subject of the analysis event.
     */
    subject?: Reference | undefined;
    /**
     * Name of the analysis event (human friendly).
     */
    title?: string | undefined;
    _title?: Element | undefined;
}
/**
 * A Genomic Study is a set of analysis performed to analyze and generate genomic data.
 */
export interface GenomicStudy extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'GenomicStudy';
    /**
     * The details about a specific analysis that was performed in this GenomicStudy.
     */
    analysis?: GenomicStudyAnalysis[] | undefined;
    /**
     * Event resources that the genomic study is based on.
     */
    basedOn?: Reference[] | undefined;
    /**
     * Description of the genomic study.
     */
    description?: string | undefined;
    _description?: Element | undefined;
    /**
     * The healthcare event with which this genomics study is associated.
     */
    encounter?: Reference | undefined;
    /**
     * Identifiers for this genomic study.
     */
    identifier?: Identifier[] | undefined;
    /**
     * The defined protocol that describes the study.
     */
    instantiatesCanonical?: string | undefined;
    _instantiatesCanonical?: Element | undefined;
    /**
     * The URL pointing to an externally maintained protocol that describes the study.
     */
    instantiatesUri?: string | undefined;
    _instantiatesUri?: Element | undefined;
    /**
     * Healthcare professionals who interpreted the genomic study.
     */
    interpreter?: Reference[] | undefined;
    /**
     * Comments related to the genomic study.
     */
    note?: Annotation[] | undefined;
    /**
     * Why the genomic study was performed.
     */
    reason?: CodeableReference[] | undefined;
    /**
     * Healthcare professional who requested or referred the genomic study.
     */
    referrer?: Reference | undefined;
    /**
     * When the genomic study was started.
     */
    startDate?: string | undefined;
    _startDate?: Element | undefined;
    /**
     * The status of the genomic study.
     */
    status: CodeableConcept;
    /**
     * The primary subject of the genomic study.
     */
    subject: Reference;
    /**
     * The type of the study, e.g., Familial variant segregation, Functional variation detection, or Gene expression profiling.
     */
    type?: CodeableConcept[] | undefined;
}
/**
 * When multiple targets are present for a single goal instance, all targets must be met for the overall goal to be met.
 */
export interface GoalTarget extends BackboneElement {
    /**
     * A CodeableConcept with just a text would be used instead of a string if the field was usually coded, or if the type associated with the Goal.target.measure defines a coded value.
     */
    detailQuantity?: Quantity | undefined;
    /**
     * A CodeableConcept with just a text would be used instead of a string if the field was usually coded, or if the type associated with the Goal.target.measure defines a coded value.
     */
    detailRange?: Range | undefined;
    /**
     * A CodeableConcept with just a text would be used instead of a string if the field was usually coded, or if the type associated with the Goal.target.measure defines a coded value.
     */
    detailCodeableConcept?: CodeableConcept | undefined;
    /**
     * A CodeableConcept with just a text would be used instead of a string if the field was usually coded, or if the type associated with the Goal.target.measure defines a coded value.
     */
    detailString?: string | undefined;
    _detailString?: Element | undefined;
    /**
     * A CodeableConcept with just a text would be used instead of a string if the field was usually coded, or if the type associated with the Goal.target.measure defines a coded value.
     */
    detailBoolean?: boolean | undefined;
    _detailBoolean?: Element | undefined;
    /**
     * A CodeableConcept with just a text would be used instead of a string if the field was usually coded, or if the type associated with the Goal.target.measure defines a coded value.
     */
    detailInteger?: number | undefined;
    /**
     * A CodeableConcept with just a text would be used instead of a string if the field was usually coded, or if the type associated with the Goal.target.measure defines a coded value.
     */
    detailRatio?: Ratio | undefined;
    /**
     * Indicates either the date or the duration after start by which the goal should be met.
     */
    dueDate?: string | undefined;
    _dueDate?: Element | undefined;
    /**
     * Indicates either the date or the duration after start by which the goal should be met.
     */
    dueDuration?: Duration | undefined;
    /**
     * The parameter whose value is being tracked, e.g. body weight, blood pressure, or hemoglobin A1c level.
     */
    measure?: CodeableConcept | undefined;
}
/**
 * Describes the intended objective(s) for a patient, group or organization care, for example, weight loss, restoring an activity of daily living, obtaining herd immunity via immunization, meeting a process improvement objective, etc.
 */
export interface Goal extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'Goal';
    /**
     * Describes the progression, or lack thereof, towards the goal against the target.
     */
    achievementStatus?: CodeableConcept | undefined;
    /**
     * The identified conditions and other health record elements that are intended to be addressed by the goal.
     */
    addresses?: Reference[] | undefined;
    /**
     * Indicates a category the goal falls within.
     */
    category?: CodeableConcept[] | undefined;
    /**
     * For example, getting a yellow fever vaccination for a planned trip is a goal that is designed to be completed (continuous = false).  A goal to sustain HbA1c levels would not be a one-time goal (continuous = true).
     */
    continuous?: boolean | undefined;
    _continuous?: Element | undefined;
    /**
     * If no code is available, use CodeableConcept.text.
     */
    description: CodeableConcept;
    /**
     * This is a business identifier, not a resource identifier (see [discussion](resource.html#identifiers)).  It is best practice for the identifier to only appear on a single resource instance, however business practices may occasionally dictate that multiple resource instances with the same identifier can exist - possibly even with different resource types.  For example, multiple Patient and a Person resource instance might share the same social insurance number.
     */
    identifier?: Identifier[] | undefined;
    /**
     * This element is labeled as a modifier because the lifecycleStatus contains codes that mark the resource as not currently valid.
     */
    lifecycleStatus:
        | 'proposed'
        | 'planned'
        | 'accepted'
        | 'active'
        | 'on-hold'
        | 'completed'
        | 'cancelled'
        | 'entered-in-error'
        | 'rejected';
    _lifecycleStatus?: Element | undefined;
    /**
     * May be used for progress notes, concerns or other related information that doesn't actually describe the goal itself.
     */
    note?: Annotation[] | undefined;
    /**
     * Note that this should not duplicate the goal status; The goal outcome is independent of the outcome of the related activities.  For example, if the Goal is to achieve a target body weight of 150 lb and a care plan activity is defined to diet, then the care plan’s activity outcome could be calories consumed whereas goal outcome is an observation for the actual body weight measured.
     */
    outcome?: CodeableReference[] | undefined;
    /**
     * Extensions are available to track priorities as established by each participant (i.e. Priority from the patient's perspective, different practitioners' perspectives, family member's perspectives)
     * The ordinal extension on Coding can be used to convey a numerically comparable ranking to priority.  (Keep in mind that different coding systems may use a "low value=important".
     */
    priority?: CodeableConcept | undefined;
    /**
     * This is the individual or team responsible for establishing the goal, not necessarily who recorded it.  (For that, use the Provenance resource.).
     */
    source?: Reference | undefined;
    /**
     * The date or event after which the goal should begin being pursued.
     */
    startDate?: string | undefined;
    _startDate?: Element | undefined;
    /**
     * The date or event after which the goal should begin being pursued.
     */
    startCodeableConcept?: CodeableConcept | undefined;
    /**
     * To see the date for past statuses, query history.
     */
    statusDate?: string | undefined;
    _statusDate?: Element | undefined;
    /**
     * This will typically be captured for statuses such as rejected, on-hold or cancelled, but could be present for others.
     */
    statusReason?: string | undefined;
    _statusReason?: Element | undefined;
    /**
     * Identifies the patient, group or organization for whom the goal is being established.
     */
    subject: Reference;
    /**
     * When multiple targets are present for a single goal instance, all targets must be met for the overall goal to be met.
     */
    target?: GoalTarget[] | undefined;
}
/**
 * Compartment Consistency Rules.
 */
export interface GraphDefinitionLinkTargetCompartment extends BackboneElement {
    /**
     * Identifies the compartment.
     */
    code: 'Patient' | 'Encounter' | 'RelatedPerson' | 'Practitioner' | 'Device';
    _code?: Element | undefined;
    /**
     * Documentation for FHIRPath expression.
     */
    description?: string | undefined;
    _description?: Element | undefined;
    /**
     * Custom rule, as a FHIRPath expression.
     */
    expression?: string | undefined;
    _expression?: Element | undefined;
    /**
     * identical | matching | different | no-rule | custom.
     */
    rule: 'identical' | 'matching' | 'different' | 'custom';
    _rule?: Element | undefined;
    /**
     * All conditional rules are evaluated; if they are true, then the rules are evaluated.
     */
    use: 'condition' | 'requirement';
    _use?: Element | undefined;
}
/**
 * Potential target for the link.
 */
export interface GraphDefinitionLinkTarget extends BackboneElement {
    /**
     * Compartment Consistency Rules.
     */
    compartment?: GraphDefinitionLinkTargetCompartment[] | undefined;
    /**
     * Additional links from target resource.
     */
    link?: GraphDefinitionLink[] | undefined;
    /**
     * At least one of the parameters must have the value {ref} which identifies the focus resource.
     */
    params?: string | undefined;
    _params?: Element | undefined;
    /**
     * Profile for the target resource.
     */
    profile?: string | undefined;
    _profile?: Element | undefined;
    /**
     * Type of resource this link refers to.
     */
    type: string;
    _type?: Element | undefined;
}
/**
 * Links this graph makes rules about.
 */
export interface GraphDefinitionLink extends BackboneElement {
    /**
     * Information about why this link is of interest in this graph definition.
     */
    description?: string | undefined;
    _description?: Element | undefined;
    /**
     * Maximum occurrences for this link.
     */
    max?: string | undefined;
    _max?: Element | undefined;
    /**
     * Minimum occurrences for this link.
     */
    min?: number | undefined;
    /**
     * The path expression cannot contain a resolve() function. If there is no path, the link is a reverse lookup, using target.params. If the path is "*" then this means all references in the resource.
     */
    path?: string | undefined;
    _path?: Element | undefined;
    /**
     * Which slice (if profiled).
     */
    sliceName?: string | undefined;
    _sliceName?: Element | undefined;
    /**
     * Potential target for the link.
     */
    target?: GraphDefinitionLinkTarget[] | undefined;
}
/**
 * A formal computable definition of a graph of resources - that is, a coherent set of resources that form a graph by following references. The Graph Definition resource defines a set and makes rules about the set.
 */
export interface GraphDefinition extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'GraphDefinition';
    /**
     * May be a web site, an email address, a telephone number, etc.
     */
    contact?: ContactDetail[] | undefined;
    /**
     * Note that this is not the same as the resource last-modified-date, since the resource may be a secondary representation of the graph definition. Additional specific dates may be added as extensions or be found by consulting Provenances associated with past versions of the resource.
     */
    date?: string | undefined;
    _date?: Element | undefined;
    /**
     * This description can be used to capture details such as comments about misuse, instructions for clinical use and interpretation, literature references, examples from the paper world, etc. It is not a rendering of the graph definition as conveyed in the 'text' field of the resource itself. This item SHOULD be populated unless the information is available from context (e.g. the language of the graph definition is presumed to be the predominant language in the place the graph definition was created).
     */
    description?: string | undefined;
    _description?: Element | undefined;
    /**
     * Allows filtering of graph definitions that are appropriate for use versus not.
     */
    experimental?: boolean | undefined;
    _experimental?: Element | undefined;
    /**
     * It may be possible for the graph definition to be used in jurisdictions other than those for which it was originally designed or intended.
     */
    jurisdiction?: CodeableConcept[] | undefined;
    /**
     * Links this graph makes rules about.
     */
    link?: GraphDefinitionLink[] | undefined;
    /**
     * The name is not expected to be globally unique. The name should be a simple alphanumeric type name to ensure that it is machine-processing friendly.
     */
    name: string;
    _name?: Element | undefined;
    /**
     * The code does not include the '$' prefix that is always included in the URL when the operation is invoked.
     */
    profile?: string | undefined;
    _profile?: Element | undefined;
    /**
     * Usually an organization but may be an individual. The publisher (or steward) of the graph definition is the organization or individual primarily responsible for the maintenance and upkeep of the graph definition. This is not necessarily the same individual or organization that developed and initially authored the content. The publisher is the primary point of contact for questions or issues with the graph definition. This item SHOULD be populated unless the information is available from context.
     */
    publisher?: string | undefined;
    _publisher?: Element | undefined;
    /**
     * This element does not describe the usage of the graph definition. Instead, it provides traceability of ''why'' the resource is either needed or ''why'' it is defined as it is.  This may be used to point to source materials or specifications that drove the structure of this graph definition.
     */
    purpose?: string | undefined;
    _purpose?: Element | undefined;
    /**
     * The type of FHIR resource at which instances of this graph start.
     */
    start: string;
    _start?: Element | undefined;
    /**
     * Allows filtering of graph definitions that are appropriate for use versus not.
     */
    status: 'draft' | 'active' | 'retired' | 'unknown';
    _status?: Element | undefined;
    /**
     * This name does not need to be machine-processing friendly and may contain punctuation, white-space, etc.
     */
    title?: string | undefined;
    _title?: Element | undefined;
    /**
     * Can be a urn:uuid: or a urn:oid: but real http: addresses are preferred.  Multiple instances may share the same URL if they have a distinct version.
     * The determination of when to create a new version of a resource (same url, new version) vs. defining a new artifact is up to the author.  Considerations for making this decision are found in [Technical and Business Versions](resource.html#versions).
     * In some cases, the resource can no longer be found at the stated url, but the url itself cannot change. Implementations can use the [meta.source](resource.html#meta) element to indicate where the current master source of the resource can be found.
     */
    url?: string | undefined;
    _url?: Element | undefined;
    /**
     * When multiple useContexts are specified, there is no expectation that all or any of the contexts apply.
     */
    useContext?: UsageContext[] | undefined;
    /**
     * There may be different graph definition instances that have the same identifier but different versions.  The version can be appended to the url in a reference to allow a reference to a particular business version of the graph definition with the format [url]|[version]. The version SHOULD NOT contain a '#' - see [Business Version](resource.html#bv-format).
     */
    version?: string | undefined;
    _version?: Element | undefined;
    /**
     * If set as a string, this is a FHIRPath expression that has two additional context variables passed in - %version1 and %version2 and will return a negative number if version1 is newer, a positive number if version2 and a 0 if the version ordering can't be successfully be determined.
     */
    versionAlgorithmString?: string | undefined;
    _versionAlgorithmString?: Element | undefined;
    /**
     * If set as a string, this is a FHIRPath expression that has two additional context variables passed in - %version1 and %version2 and will return a negative number if version1 is newer, a positive number if version2 and a 0 if the version ordering can't be successfully be determined.
     */
    versionAlgorithmCoding?: Coding | undefined;
}
/**
 * All the identified characteristics must be true for an entity to a member of the group.
 */
export interface GroupCharacteristic extends BackboneElement {
    /**
     * A code that identifies the kind of trait being asserted.
     */
    code: CodeableConcept;
    /**
     * This is labeled as "Is Modifier" because applications cannot wrongly include excluded members as included or vice versa.
     */
    exclude: boolean;
    _exclude?: Element | undefined;
    /**
     * The period over which the characteristic is tested; e.g. the patient had an operation during the month of June.
     */
    period?: Period | undefined;
    /**
     * For Range, it means members of the group have a value that falls somewhere within the specified range.
     */
    valueCodeableConcept?: CodeableConcept | undefined;
    /**
     * For Range, it means members of the group have a value that falls somewhere within the specified range.
     */
    valueBoolean?: boolean | undefined;
    _valueBoolean?: Element | undefined;
    /**
     * For Range, it means members of the group have a value that falls somewhere within the specified range.
     */
    valueQuantity?: Quantity | undefined;
    /**
     * For Range, it means members of the group have a value that falls somewhere within the specified range.
     */
    valueRange?: Range | undefined;
    /**
     * For Range, it means members of the group have a value that falls somewhere within the specified range.
     */
    valueReference?: Reference | undefined;
}
/**
 * Identifies the resource instances that are members of the group.
 */
export interface GroupMember extends BackboneElement {
    /**
     * A reference to the entity that is a member of the group. Must be consistent with Group.type. If the entity is another group, then the type must be the same.
     */
    entity: Reference;
    /**
     * A flag to indicate that the member is no longer in the group, but previously may have been a member.
     */
    inactive?: boolean | undefined;
    _inactive?: Element | undefined;
    /**
     * The period that the member was in the group, if known.
     */
    period?: Period | undefined;
}
/**
 * Represents a defined collection of entities that may be discussed or acted upon collectively but which are not expected to act collectively, and are not formally or legally recognized; i.e. a collection of entities that isn't an Organization.
 */
export interface Group extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'Group';
    /**
     * Indicates whether the record for the group is available for use or is merely being retained for historical purposes.
     */
    active?: boolean | undefined;
    _active?: Element | undefined;
    /**
     * All the identified characteristics must be true for an entity to a member of the group.
     */
    characteristic?: GroupCharacteristic[] | undefined;
    /**
     * This would generally be omitted for Person resources.
     */
    code?: CodeableConcept | undefined;
    /**
     * Explanation of what the group represents and how it is intended to be used.
     */
    description?: string | undefined;
    _description?: Element | undefined;
    /**
     * A unique business identifier for this group.
     */
    identifier?: Identifier[] | undefined;
    /**
     * This does not strictly align with ownership of a herd or flock, but may suffice to represent that relationship in simple cases. More complex cases will require an extension.
     */
    managingEntity?: Reference | undefined;
    /**
     * Identifies the resource instances that are members of the group.
     */
    member?: GroupMember[] | undefined;
    /**
     * Basis for membership in the Group:
     * * 'definitional': The Group.characteristics specified are both necessary and sufficient to determine membership. All entities that meet the criteria are considered to be members of the group, whether referenced by the group or not. If members are present, they are individuals that happen to be known as meeting the Group.characteristics. The list cannot be presumed to be complete.
     * * 'enumerated': The Group.characteristics are necessary but not sufficient to determine membership. Membership is determined by being listed as one of the Group.member.
     */
    membership: 'definitional' | 'enumerated';
    _membership?: Element | undefined;
    /**
     * A label assigned to the group for human identification and communication.
     */
    name?: string | undefined;
    _name?: Element | undefined;
    /**
     * Note that the quantity may be less than the number of members if some of the members are not active.
     */
    quantity?: number | undefined;
    /**
     * Group members SHALL be of the appropriate resource type (Patient for person or animal; or Practitioner, PractitionerRole, Device, CareTeam, HealthcareService, Location, Organization, RelatedPerson, or Specimen for the other types.), or a Group of the resources of the appropriate type.
     */
    type:
        | 'person'
        | 'animal'
        | 'practitioner'
        | 'device'
        | 'careteam'
        | 'healthcareservice'
        | 'location'
        | 'organization'
        | 'relatedperson'
        | 'specimen';
    _type?: Element | undefined;
}
/**
 * A guidance response is the formal response to a guidance request, including any output parameters returned by the evaluation, as well as the description of any proposed actions to be taken.
 */
export interface GuidanceResponse extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'GuidanceResponse';
    /**
     * If the evaluation could not be completed due to lack of information, or additional information would potentially result in a more accurate response, this element will a description of the data required in order to proceed with the evaluation. A subsequent request to the service should include this data.
     */
    dataRequirement?: DataRequirement[] | undefined;
    /**
     * This will typically be the encounter the event occurred within, but some activities may be initiated prior to or after the official copmletion of an encounter but still be tied to the context of the encounter.
     */
    encounter?: Reference | undefined;
    /**
     * Messages resulting from the evaluation of the artifact or artifacts. As part of evaluating the request, the engine may produce informational or warning messages. These messages will be provided by this element.
     */
    evaluationMessage?: Reference[] | undefined;
    /**
     * Allows a service to provide  unique, business identifiers for the response.
     */
    identifier?: Identifier[] | undefined;
    /**
     * An identifier, CodeableConcept or canonical reference to the guidance that was requested.
     */
    moduleUri?: string | undefined;
    _moduleUri?: Element | undefined;
    /**
     * An identifier, CodeableConcept or canonical reference to the guidance that was requested.
     */
    moduleCanonical?: string | undefined;
    _moduleCanonical?: Element | undefined;
    /**
     * An identifier, CodeableConcept or canonical reference to the guidance that was requested.
     */
    moduleCodeableConcept?: CodeableConcept | undefined;
    /**
     * Provides a mechanism to communicate additional information about the response.
     */
    note?: Annotation[] | undefined;
    /**
     * Indicates when the guidance response was processed.
     */
    occurrenceDateTime?: string | undefined;
    _occurrenceDateTime?: Element | undefined;
    /**
     * The output parameters of the evaluation, if any. Many modules will result in the return of specific resources such as procedure or communication requests that are returned as part of the operation result. However, modules may define specific outputs that would be returned as the result of the evaluation, and these would be returned in this element.
     */
    outputParameters?: Reference | undefined;
    /**
     * Provides a reference to the device that performed the guidance.
     */
    performer?: Reference | undefined;
    /**
     * Although this reference can be used to point to any resource, it is typically expected to refer to subject-specific data appropriate to the subject of the guidance. For example, patient-based decision support would be expected to reference patient-level data. In addition, implementations should provide as much detail as possible by using the [targetElement](extension-targetelement.html) and [targetPath](extension-targetpath.html) extensions to indicate the specific elements relevant to providing the reason for the guidance.
     */
    reason?: CodeableReference[] | undefined;
    /**
     * The identifier of the request associated with this response. If an identifier was given as part of the request, it will be reproduced here to enable the requester to more easily identify the response in a multi-request scenario.
     */
    requestIdentifier?: Identifier | undefined;
    /**
     * The actions, if any, produced by the evaluation of the artifact.
     */
    result?: Reference | undefined;
    /**
     * This element is labeled as a modifier because the status contains codes that mark the resource as not currently valid.
     */
    status: 'success' | 'data-requested' | 'data-required' | 'in-progress' | 'failure' | 'entered-in-error';
    _status?: Element | undefined;
    /**
     * The patient for which the request was processed.
     */
    subject?: Reference | undefined;
}
/**
 * Does this service have specific eligibility requirements that need to be met in order to use the service?
 */
export interface HealthcareServiceEligibility extends BackboneElement {
    /**
     * Coded value for the eligibility.
     */
    code?: CodeableConcept | undefined;
    /**
     * The description of service eligibility should, in general, not exceed one or two paragraphs. It should be sufficient for a prospective consumer to determine if they are likely to be eligible or not. Where eligibility requirements and conditions are complex, it may simply be noted that an eligibility assessment is required. Where eligibility is determined by an outside source, such as an Act of Parliament, this should be noted, preferably with a reference to a commonly available copy of the source document such as a web page.
     */
    comment?: string | undefined;
    _comment?: Element | undefined;
}
/**
 * The details of a healthcare service available at a location.
 */
export interface HealthcareService extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'HealthcareService';
    /**
     * This element is labeled as a modifier because it may be used to mark that the resource was created in error.
     */
    active?: boolean | undefined;
    _active?: Element | undefined;
    /**
     * Indicates whether or not a prospective consumer will require an appointment for a particular service at a site to be provided by the Organization. Indicates if an appointment is required for access to this service.
     */
    appointmentRequired?: boolean | undefined;
    _appointmentRequired?: Element | undefined;
    /**
     * More detailed availability information may be provided in associated Schedule/Slot resources.
     * Systems may choose to render availability differently than it is exchanged on the interface. For example, rather than "Mon, Tue, Wed, Thur, Fri from 9am-12am; Mon, Tue, Wed, Thur, Fri from 1pm-5pm" as would be implied by two availableTime repetitions, an application could render this information as "Mon-Fri 9-12am and 1-5pm".
     * The NotAvailableTime(s) included indicate the general days/periods where the service is not available (for things such as public holidays).
     */
    availability?: Availability[] | undefined;
    /**
     * Selecting a Service Category then determines the list of relevant service types that can be selected in the primary service type.
     */
    category?: CodeableConcept[] | undefined;
    /**
     * These could be such things as is wheelchair accessible.
     */
    characteristic?: CodeableConcept[] | undefined;
    /**
     * Would expect that a user would not see this information on a search results, and it would only be available when viewing the complete details of the service.
     */
    comment?: string | undefined;
    _comment?: Element | undefined;
    /**
     * When using this property it indicates that the service is available with this language, it is not derived from the practitioners, and not all are required to use this language, just that this language is available while scheduling.
     */
    communication?: CodeableConcept[] | undefined;
    /**
     * The address/telecom use code 'home' are not to be used. Note that these contacts are not the contact details of people who provide the service (that would be through PractitionerRole), these are official contacts for the HealthcareService itself for specific purposes. E.g. Mailing Addresses, Billing Addresses, Contact numbers for Booking or Billing Enquiries, general web address, web address for online bookings etc.
     * If this is empty (or the type of interest is empty), refer to the location's contacts.
     */
    contact?: ExtendedContactDetail[] | undefined;
    /**
     * The locations referenced by the coverage area can include both specific locations, including areas, and also conceptual domains too (mode = kind), such as a physical area (tri-state area) and some other attribute (covered by Example Care Organization). These types of Locations are often not managed by any specific organization. This could also include generic locations such as "in-home".
     */
    coverageArea?: Reference[] | undefined;
    /**
     * Does this service have specific eligibility requirements that need to be met in order to use the service?
     */
    eligibility?: HealthcareServiceEligibility[] | undefined;
    /**
     * Technical endpoints providing access to services operated for the specific healthcare services defined at this resource.
     */
    endpoint?: Reference[] | undefined;
    /**
     * Extra details about the service that can't be placed in the other fields.
     */
    extraDetails?: string | undefined;
    _extraDetails?: Element | undefined;
    /**
     * External identifiers for this item.
     */
    identifier?: Identifier[] | undefined;
    /**
     * The location(s) where this healthcare service may be provided.
     */
    location?: Reference[] | undefined;
    /**
     * Further description of the service as it would be presented to a consumer while searching.
     */
    name?: string | undefined;
    _name?: Element | undefined;
    /**
     * For example, if there is a generic Radiology service that offers CT Scans, MRIs, etc.  You could have HealthcareService resources for the CT Scans and MRIs, which have an offeredIn reference to the Radiology HealthcareService.
     */
    offeredIn?: Reference[] | undefined;
    /**
     * If there is a photo/symbol associated with this HealthcareService, it may be included here to facilitate quick identification of the service in a list.
     */
    photo?: Attachment | undefined;
    /**
     * Programs are often defined externally to an Organization, commonly by governments; e.g. Home and Community Care Programs, Homeless Program, ….
     */
    program?: CodeableConcept[] | undefined;
    /**
     * This property is recommended to be the same as the Location's managingOrganization, and if not provided should be interpreted as such. If the Location does not have a managing Organization, then this property should be populated.
     */
    providedBy?: Reference | undefined;
    /**
     * Ways that the service accepts referrals, if this is not provided then it is implied that no referral is required.
     */
    referralMethod?: CodeableConcept[] | undefined;
    /**
     * The provision means being commissioned by, contractually obliged or financially sourced. Types of costings that may apply to this healthcare service, such if the service may be available for free, some discounts available, or fees apply.
     */
    serviceProvisionCode?: CodeableConcept[] | undefined;
    /**
     * Collection of specialties handled by the Healthcare service. This is more of a medical term.
     */
    specialty?: CodeableConcept[] | undefined;
    /**
     * The specific type of service that may be delivered or performed.
     */
    type?: CodeableConcept[] | undefined;
}
/**
 * Selector of the instances – human or machine.
 */
export interface ImagingSelectionPerformer extends BackboneElement {
    /**
     * Author – human or machine.
     */
    actor?: Reference | undefined;
    /**
     * Distinguishes the type of involvement of the performer.
     */
    function?: CodeableConcept | undefined;
}
/**
 * Each imaging selection instance or frame list might includes an image region, specified by a region type and a set of 2D coordinates.
 *        If the parent imagingSelection.instance contains a subset element of type frame, the image region applies to all frames in the subset list.
 */
export interface ImagingSelectionInstanceImageRegion extends BackboneElement {
    /**
     * For a description of how 2D coordinates are encoded, see [DICOM PS3.3 C.18.6](http://dicom.nema.org/medical/dicom/current/output/chtml/part03/sect_C.18.6.html).
     */
    coordinate: number[];
    /**
     * See [DICOM PS3.3 C.10.5.1.2](https://dicom.nema.org/medical/dicom/current/output/chtml/part03/sect_C.10.5.html#sect_C.10.5.1.2).
     */
    regionType: 'point' | 'polyline' | 'interpolated' | 'circle' | 'ellipse';
    _regionType?: Element | undefined;
}
/**
 * Each imaging selection includes one or more selected DICOM SOP instances.
 */
export interface ImagingSelectionInstance extends BackboneElement {
    /**
     * Each imaging selection instance or frame list might includes an image region, specified by a region type and a set of 2D coordinates.
     *        If the parent imagingSelection.instance contains a subset element of type frame, the image region applies to all frames in the subset list.
     */
    imageRegion?: ImagingSelectionInstanceImageRegion[] | undefined;
    /**
     * See [DICOM PS3.3 C.7.6.1](https://dicom.nema.org/medical/dicom/current/output/chtml/part03/sect_C.7.6.html#sect_C.7.6.1).
     *        Note: A multiframe instance has a single instance number with each frame identified by a frame number.
     */
    number?: number | undefined;
    /**
     * See [DICOM PS3.3 C.12.1](http://dicom.nema.org/medical/dicom/current/output/chtml/part03/sect_C.12.html).
     */
    sopClass?: Coding | undefined;
    /**
     * Selected subset of the SOP Instance. The content and format of the subset item is determined by the SOP Class of the selected instance.
     *        May be one of:
     *        - A list of frame numbers selected from a multiframe SOP Instance.
     *        - A list of Content Item Observation UID values selected from a DICOM SR or other structured document SOP Instance.
     *        - A list of segment numbers selected from a segmentation SOP Instance.
     *        - A list of Region of Interest (ROI) numbers selected from a radiotherapy structure set SOP Instance.
     */
    subset?: string[] | undefined;
    _subset?: Element[] | undefined;
    /**
     * See [DICOM PS3.3 C.12.1](http://dicom.nema.org/medical/dicom/current/output/chtml/part03/sect_C.12.html).
     */
    uid: string;
    _uid?: Element | undefined;
}
/**
 * Each imaging selection might includes a 3D image region, specified by a region type and a set of 3D coordinates.
 */
export interface ImagingSelectionImageRegion extends BackboneElement {
    /**
     * For a description of how 3D coordinates are encoded, see [DICOM PS3.3 C.18.9](http://dicom.nema.org/medical/dicom/current/output/chtml/part03/sect_C.18.9.html).
     */
    coordinate: number[];
    /**
     * See [DICOM PS3.3 C.18.9.1.2](http://dicom.nema.org/medical/dicom/current/output/chtml/part03/sect_C.18.9.html#sect_C.18.9.1.2).
     */
    regionType: 'point' | 'multipoint' | 'polyline' | 'polygon' | 'ellipse' | 'ellipsoid';
    _regionType?: Element | undefined;
}
/**
 * A selection of DICOM SOP instances and/or frames within a single Study and Series. This might include additional specifics such as an image region, an Observation UID or a Segmentation Number, allowing linkage to an Observation Resource or transferring this information along with the ImagingStudy Resource.
 */
export interface ImagingSelection extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'ImagingSelection';
    /**
     * A list of the diagnostic requests that resulted in this imaging selection being performed.
     */
    basedOn?: Reference[] | undefined;
    /**
     * The anatomic structures examined. See DICOM Part 16 Annex L (http://dicom.nema.org/medical/dicom/current/output/chtml/part16/chapter_L.html) for DICOM to SNOMED-CT mappings.
     */
    bodySite?: CodeableReference | undefined;
    /**
     * Classifies the imaging selection.
     */
    category?: CodeableConcept[] | undefined;
    /**
     * *All* code-value and, if present, component.code-component.value pairs need to be taken into account to correctly understand the meaning of the observation.
     */
    code: CodeableConcept;
    /**
     * The imaging study from which the imaging selection is made.
     */
    derivedFrom?: Reference[] | undefined;
    /**
     * Typical endpoint types include DICOM WADO-RS, which is used to retrieve DICOM instances in native or rendered (e.g., JPG, PNG) formats using a RESTful API; DICOM WADO-URI, which can similarly retrieve native or rendered instances, except using an HTTP query-based approach.
     */
    endpoint?: Reference[] | undefined;
    /**
     * An imaging selection may reference a DICOM resource that itself references other DICOM resources.
     *        e.g. a presentation state references a set of source images or frames.
     */
    focus?: Reference[] | undefined;
    /**
     * See [DICOM PS3.3 C.7.4.1](http://dicom.nema.org/medical/dicom/current/output/chtml/part03/sect_C.7.4.html).
     */
    frameOfReferenceUid?: string | undefined;
    _frameOfReferenceUid?: Element | undefined;
    /**
     * This is a business identifier, not a resource identifier -- see discussion under [Business Identifiers](resource.html#identifiers).
     */
    identifier?: Identifier[] | undefined;
    /**
     * Each imaging selection might includes a 3D image region, specified by a region type and a set of 3D coordinates.
     */
    imageRegion?: ImagingSelectionImageRegion[] | undefined;
    /**
     * Each imaging selection includes one or more selected DICOM SOP instances.
     */
    instance?: ImagingSelectionInstance[] | undefined;
    /**
     * The date and time this imaging selection was created.
     */
    issued?: string | undefined;
    _issued?: Element | undefined;
    /**
     * Selector of the instances – human or machine.
     */
    performer?: ImagingSelectionPerformer[] | undefined;
    /**
     * See [DICOM PS3.3 C.7.3](http://dicom.nema.org/medical/dicom/current/output/chtml/part03/sect_C.7.3.html).
     */
    seriesNumber?: number | undefined;
    /**
     * See [DICOM PS3.3 C.7.3](http://dicom.nema.org/medical/dicom/current/output/chtml/part03/sect_C.7.3.html).
     */
    seriesUid?: string | undefined;
    _seriesUid?: Element | undefined;
    /**
     * Unknown does not represent "other" - one of the defined statuses must apply.  Unknown is used when the authoring system is not sure what the current status is.
     */
    status: 'available' | 'entered-in-error' | 'unknown';
    _status?: Element | undefined;
    /**
     * See [DICOM PS3.3 C.7.2](http://dicom.nema.org/medical/dicom/current/output/chtml/part03/sect_C.7.2.html).
     */
    studyUid?: string | undefined;
    _studyUid?: Element | undefined;
    /**
     * The patient, or group of patients, location, device, organization, procedure or practitioner this imaging selection is about and into whose or what record the imaging selection is placed.
     */
    subject?: Reference | undefined;
}
/**
 * If the person who performed the series is not known, their Organization may be recorded. A patient, or related person, may be the performer, e.g. for patient-captured images.
 */
export interface ImagingStudySeriesPerformer extends BackboneElement {
    /**
     * Indicates who or what performed the series.
     */
    actor: Reference;
    /**
     * Distinguishes the type of involvement of the performer in the series.
     */
    function?: CodeableConcept | undefined;
}
/**
 * A single SOP instance within the series, e.g. an image, or presentation state.
 */
export interface ImagingStudySeriesInstance extends BackboneElement {
    /**
     * The number of instance in the series.
     */
    number?: number | undefined;
    /**
     * DICOM instance  type.
     */
    sopClass: Coding;
    /**
     * Particularly for post-acquisition analytic objects, such as SR, presentation states, value mapping, etc.
     */
    title?: string | undefined;
    _title?: Element | undefined;
    /**
     * See  [DICOM PS3.3 C.12.1](http://dicom.nema.org/medical/dicom/current/output/chtml/part03/sect_C.12.html#sect_C.12.1).
     */
    uid: string;
    _uid?: Element | undefined;
}
/**
 * Each study has one or more series of images or other content.
 */
export interface ImagingStudySeries extends BackboneElement {
    /**
     * The anatomic structures examined. See DICOM Part 16 Annex L (http://dicom.nema.org/medical/dicom/current/output/chtml/part16/chapter_L.html) for DICOM to SNOMED-CT mappings. The bodySite may indicate the laterality of body part imaged; if so, it shall be consistent with any content of ImagingStudy.series.laterality.
     */
    bodySite?: CodeableReference | undefined;
    /**
     * A description of the series.
     */
    description?: string | undefined;
    _description?: Element | undefined;
    /**
     * Typical endpoint types include DICOM WADO-RS, which is used to retrieve DICOM instances in native or rendered (e.g., JPG, PNG) formats using a RESTful API; DICOM WADO-URI, which can similarly retrieve native or rendered instances, except using an HTTP query-based approach; and DICOM QIDO-RS, which allows RESTful query for DICOM information without retrieving the actual instances.
     */
    endpoint?: Reference[] | undefined;
    /**
     * A single SOP instance within the series, e.g. an image, or presentation state.
     */
    instance?: ImagingStudySeriesInstance[] | undefined;
    /**
     * The laterality of the (possibly paired) anatomic structures examined. E.g., the left knee, both lungs, or unpaired abdomen. If present, shall be consistent with any laterality information indicated in ImagingStudy.series.bodySite.
     */
    laterality?: CodeableConcept | undefined;
    /**
     * The distinct modality for this series. This may include both acquisition and non-acquisition modalities.
     */
    modality: CodeableConcept;
    /**
     * The numeric identifier of this series in the study.
     */
    number?: number | undefined;
    /**
     * Number of SOP Instances in the Study. The value given may be larger than the number of instance elements this resource contains due to resource availability, security, or other factors. This element should be present if any instance elements are present.
     */
    numberOfInstances?: number | undefined;
    /**
     * If the person who performed the series is not known, their Organization may be recorded. A patient, or related person, may be the performer, e.g. for patient-captured images.
     */
    performer?: ImagingStudySeriesPerformer[] | undefined;
    /**
     * The specimen imaged, e.g., for whole slide imaging of a biopsy.
     */
    specimen?: Reference[] | undefined;
    /**
     * The date and time the series was started.
     */
    started?: string | undefined;
    _started?: Element | undefined;
    /**
     * See [DICOM PS3.3 C.7.3](http://dicom.nema.org/medical/dicom/current/output/chtml/part03/sect_C.7.3.html).
     */
    uid: string;
    _uid?: Element | undefined;
}
/**
 * Representation of the content produced in a DICOM imaging study. A study comprises a set of series, each of which includes a set of Service-Object Pair Instances (SOP Instances - images or other data) acquired or produced in a common context.  A series is of only one modality (e.g. X-ray, CT, MR, ultrasound), but a study may have multiple series of different modalities.
 */
export interface ImagingStudy extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'ImagingStudy';
    /**
     * A list of the diagnostic requests that resulted in this imaging study being performed.
     */
    basedOn?: Reference[] | undefined;
    /**
     * The Imaging Manager description of the study. Institution-generated description or classification of the Study (component) performed.
     */
    description?: string | undefined;
    _description?: Element | undefined;
    /**
     * This will typically be the encounter the event occurred within, but some events may be initiated prior to or after the official completion of an encounter but still be tied to the context of the encounter (e.g. pre-admission test).
     */
    encounter?: Reference | undefined;
    /**
     * Typical endpoint types include DICOM WADO-RS, which is used to retrieve DICOM instances in native or rendered (e.g., JPG, PNG), formats using a RESTful API; DICOM WADO-URI, which can similarly retrieve native or rendered instances, except using an HTTP query-based approach; DICOM QIDO-RS, which allows RESTful query for DICOM information without retrieving the actual instances; or IHE Invoke Image Display (IID), which provides standard invocation of an imaging web viewer.
     */
    endpoint?: Reference[] | undefined;
    /**
     * See discussion under [Imaging Study Implementation Notes](imagingstudy.html#notes) for encoding of DICOM Study Instance UID.
     */
    identifier?: Identifier[] | undefined;
    /**
     * Who read the study and interpreted the images or other content.
     */
    interpreter?: Reference[] | undefined;
    /**
     * The principal physical location where the ImagingStudy was performed.
     */
    location?: Reference | undefined;
    /**
     * A list of all the distinct values of series.modality. This may include both acquisition and non-acquisition modalities.
     */
    modality?: CodeableConcept[] | undefined;
    /**
     * Per the recommended DICOM mapping, this element is derived from the Study Description attribute (0008,1030). Observations or findings about the imaging study should be recorded in another resource, e.g. Observation, and not in this element.
     */
    note?: Annotation[] | undefined;
    /**
     * Number of SOP Instances in Study. This value given may be larger than the number of instance elements this resource contains due to resource availability, security, or other factors. This element should be present if any instance elements are present.
     */
    numberOfInstances?: number | undefined;
    /**
     * Number of Series in the Study. This value given may be larger than the number of series elements this Resource contains due to resource availability, security, or other factors. This element should be present if any series elements are present.
     */
    numberOfSeries?: number | undefined;
    /**
     * The procedure or code from which this ImagingStudy was part of.
     */
    procedure?: CodeableReference[] | undefined;
    /**
     * Description of clinical condition indicating why the ImagingStudy was requested, and/or Indicates another resource whose existence justifies this Study.
     */
    reason?: CodeableReference[] | undefined;
    /**
     * The requesting/referring physician.
     */
    referrer?: Reference | undefined;
    /**
     * Each study has one or more series of images or other content.
     */
    series?: ImagingStudySeries[] | undefined;
    /**
     * Date and time the study started.
     */
    started?: string | undefined;
    _started?: Element | undefined;
    /**
     * Unknown does not represent "other" - one of the defined statuses must apply.  Unknown is used when the authoring system is not sure what the current status is.
     */
    status: 'registered' | 'available' | 'cancelled' | 'entered-in-error' | 'unknown';
    _status?: Element | undefined;
    /**
     * QA phantoms can be recorded with a Device; multiple subjects (such as mice) can be recorded with a Group.
     */
    subject: Reference;
}
/**
 * Indicates who performed the immunization event.
 */
export interface ImmunizationPerformer extends BackboneElement {
    /**
     * When the individual practitioner who performed the action is known, it is best to send.
     */
    actor: Reference;
    /**
     * Describes the type of performance (e.g. ordering provider, administering provider, etc.).
     */
    function?: CodeableConcept | undefined;
}
/**
 * Indicates a patient's eligibility for a funding program.
 */
export interface ImmunizationProgramEligibility extends BackboneElement {
    /**
     * Indicates which program the patient had their eligility evaluated for.
     */
    program: CodeableConcept;
    /**
     * Indicates the patient's eligility status for for a specific payment program.
     */
    programStatus: CodeableConcept;
}
/**
 * A reaction may be an indication of an allergy or intolerance and, if this is determined to be the case, it should be recorded as a new AllergyIntolerance resource instance as most systems will not query against past Immunization.reaction elements.
 */
export interface ImmunizationReaction extends BackboneElement {
    /**
     * Date of reaction to the immunization.
     */
    date?: string | undefined;
    _date?: Element | undefined;
    /**
     * Details of the reaction.
     */
    manifestation?: CodeableReference | undefined;
    /**
     * Self-reported indicator.
     */
    reported?: boolean | undefined;
    _reported?: Element | undefined;
}
/**
 * The protocol (set of recommendations) being followed by the provider who administered the dose.
 */
export interface ImmunizationProtocolApplied extends BackboneElement {
    /**
     * Indicates the authority who published the protocol (e.g. ACIP) that is being followed.
     */
    authority?: Reference | undefined;
    /**
     * The use of an integer is preferred if known. Text should only be used in cases where an integer is not available (such as when documenting a recurring booster dose).
     */
    doseNumber: string;
    _doseNumber?: Element | undefined;
    /**
     * One possible path to achieve presumed immunity against a disease - within the context of an authority.
     */
    series?: string | undefined;
    _series?: Element | undefined;
    /**
     * The use of an integer is preferred if known.
     */
    seriesDoses?: string | undefined;
    _seriesDoses?: Element | undefined;
    /**
     * The vaccine preventable disease the dose is being administered against.
     */
    targetDisease?: CodeableConcept[] | undefined;
}
/**
 * Describes the event of a patient being administered a vaccine or a record of an immunization as reported by a patient, a clinician or another party.
 */
export interface Immunization extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'Immunization';
    /**
     * An indication of which product was administered to the patient. This is typically a more detailed representation of the concept conveyed by the vaccineCode data element. If a Medication resource is referenced, it may be to a stand-alone resource or a contained resource within the Immunization resource.
     */
    administeredProduct?: CodeableReference | undefined;
    /**
     * Allows tracing of an authorization for the Immunization.
     */
    basedOn?: Reference[] | undefined;
    /**
     * The quantity of vaccine product that was administered.
     */
    doseQuantity?: Quantity | undefined;
    /**
     * The visit or admission or other contact between patient and health care provider the immunization was performed as part of.
     */
    encounter?: Reference | undefined;
    /**
     * Date vaccine batch expires.
     */
    expirationDate?: string | undefined;
    _expirationDate?: Element | undefined;
    /**
     * Indicates the source of the vaccine actually administered. This may be different than the patient eligibility (e.g. the patient may be eligible for a publically purchased vaccine but due to inventory issues, vaccine purchased with private funds was actually administered).
     */
    fundingSource?: CodeableConcept | undefined;
    /**
     * A unique identifier assigned to this immunization record.
     */
    identifier?: Identifier[] | undefined;
    /**
     * Typically will not be populated if primarySource = True, not required even if primarySource = False.
     */
    informationSource?: CodeableReference | undefined;
    /**
     * Typically, the recognition of the dose being sub-potent is retrospective, after the administration (ex. notification of a manufacturer recall after administration). However, in the case of a partial administration (the patient moves unexpectedly and only some of the dose is actually administered), subpotency may be recognized immediately, but it is still important to record the event.
     */
    isSubpotent?: boolean | undefined;
    _isSubpotent?: Element | undefined;
    /**
     * The service delivery location where the vaccine administration occurred.
     */
    location?: Reference | undefined;
    /**
     * Lot number of the  vaccine product.
     */
    lotNumber?: string | undefined;
    _lotNumber?: Element | undefined;
    /**
     * Name of vaccine manufacturer.
     */
    manufacturer?: CodeableReference | undefined;
    /**
     * Extra information about the immunization that is not conveyed by the other attributes.
     */
    note?: Annotation[] | undefined;
    /**
     * When immunizations are given a specific date and time should always be known.   When immunizations are patient reported, a specific date might not be known.  Although partial dates are allowed, an adult patient might not be able to recall the year a childhood immunization was given. An exact date is always preferable, but the use of the String data type is acceptable when an exact date is not known. A small number of vaccines (e.g. live oral typhoid vaccine) are given as a series of patient self-administered dose over a span of time. In cases like this, often, only the first dose (typically a provider supervised dose) is recorded with the occurrence indicating the date/time of the first dose.
     */
    occurrenceDateTime?: string | undefined;
    _occurrenceDateTime?: Element | undefined;
    /**
     * When immunizations are given a specific date and time should always be known.   When immunizations are patient reported, a specific date might not be known.  Although partial dates are allowed, an adult patient might not be able to recall the year a childhood immunization was given. An exact date is always preferable, but the use of the String data type is acceptable when an exact date is not known. A small number of vaccines (e.g. live oral typhoid vaccine) are given as a series of patient self-administered dose over a span of time. In cases like this, often, only the first dose (typically a provider supervised dose) is recorded with the occurrence indicating the date/time of the first dose.
     */
    occurrenceString?: string | undefined;
    _occurrenceString?: Element | undefined;
    /**
     * The patient who either received or did not receive the immunization.
     */
    patient: Reference;
    /**
     * Indicates who performed the immunization event.
     */
    performer?: ImmunizationPerformer[] | undefined;
    /**
     * Reflects the “reliability” of the content.
     */
    primarySource?: boolean | undefined;
    _primarySource?: Element | undefined;
    /**
     * Indicates a patient's eligibility for a funding program.
     */
    programEligibility?: ImmunizationProgramEligibility[] | undefined;
    /**
     * The protocol (set of recommendations) being followed by the provider who administered the dose.
     */
    protocolApplied?: ImmunizationProtocolApplied[] | undefined;
    /**
     * A reaction may be an indication of an allergy or intolerance and, if this is determined to be the case, it should be recorded as a new AllergyIntolerance resource instance as most systems will not query against past Immunization.reaction elements.
     */
    reaction?: ImmunizationReaction[] | undefined;
    /**
     * Describes why the immunization occurred in coded or textual form, or Indicates another resource (Condition, Observation or DiagnosticReport) whose existence justifies this immunization.
     */
    reason?: CodeableReference[] | undefined;
    /**
     * The path by which the vaccine product is taken into the body.
     */
    route?: CodeableConcept | undefined;
    /**
     * Body site where vaccine was administered.
     */
    site?: CodeableConcept | undefined;
    /**
     * Will generally be set to show that the immunization has been completed or not done.  This element is labeled as a modifier because the status contains codes that mark the resource as not currently valid.
     */
    status: 'completed' | 'entered-in-error' | 'not-done';
    _status?: Element | undefined;
    /**
     * This is generally only used for the status of "not-done". The reason for performing the immunization event is captured in reasonCode, not here.
     */
    statusReason?: CodeableConcept | undefined;
    /**
     * Reason why a dose is considered to be subpotent.
     */
    subpotentReason?: CodeableConcept[] | undefined;
    /**
     * Additional information that is relevant to the immunization (e.g. for a vaccine recipient who is pregnant, the gestational age of the fetus). The reason why a vaccine was given (e.g. occupation, underlying medical condition) should be conveyed in Immunization.reason, not as supporting information. The reason why a vaccine was not given (e.g. contraindication) should be conveyed in Immunization.statusReason, not as supporting information.
     */
    supportingInformation?: Reference[] | undefined;
    /**
     * The code for the administered vaccine may be reported at various levels of granularity from a very generic code for a general type of vaccine (e.g. the SNOMED or CVX code for the flu vaccine) to a very discrete code for a specific product including the form and packaging (e.g. an NDC code). Because vaccineCode uses the CodeableConcept data type, multiple codes at different levels of granularity may be reported for a single immunization event provided that all codes are appropriate for the same vaccine. For example, both NDC and CVX codes may be provided for a given immunization event by repeating the coding element within vaccineCode.
     */
    vaccineCode: CodeableConcept;
}
/**
 * Describes a comparison of an immunization event against published recommendations to determine if the administration is "valid" in relation to those  recommendations.
 */
export interface ImmunizationEvaluation extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'ImmunizationEvaluation';
    /**
     * Indicates the authority who published the protocol (e.g. ACIP).
     */
    authority?: Reference | undefined;
    /**
     * The date the evaluation of the vaccine administration event was performed.
     */
    date?: string | undefined;
    _date?: Element | undefined;
    /**
     * Additional information about the evaluation.
     */
    description?: string | undefined;
    _description?: Element | undefined;
    /**
     * The use of an integer is preferred if known. A string should only be used in cases where an integer is not available (such as when documenting a recurring booster dose).
     */
    doseNumber?: string | undefined;
    _doseNumber?: Element | undefined;
    /**
     * Indicates if the dose is valid or not valid with respect to the published recommendations.
     */
    doseStatus: CodeableConcept;
    /**
     * Provides an explanation as to why the vaccine administration event is valid or not relative to the published recommendations.
     */
    doseStatusReason?: CodeableConcept[] | undefined;
    /**
     * A unique identifier assigned to this immunization evaluation record.
     */
    identifier?: Identifier[] | undefined;
    /**
     * The vaccine administration event being evaluated.
     */
    immunizationEvent: Reference;
    /**
     * The individual for whom the evaluation is being done.
     */
    patient: Reference;
    /**
     * One possible path to achieve presumed immunity against a disease - within the context of an authority.
     */
    series?: string | undefined;
    _series?: Element | undefined;
    /**
     * The use of an integer is preferred if known.
     */
    seriesDoses?: string | undefined;
    _seriesDoses?: Element | undefined;
    /**
     * Indicates the current status of the evaluation of the vaccination administration event.
     */
    status: 'completed' | 'entered-in-error';
    _status?: Element | undefined;
    /**
     * The vaccine preventable disease the dose is being evaluated against.
     */
    targetDisease: CodeableConcept;
}
/**
 * Vaccine date recommendations.  For example, earliest date to administer, latest date to administer, etc.
 */
export interface ImmunizationRecommendationRecommendationDateCriterion extends BackboneElement {
    /**
     * Date classification of recommendation.  For example, earliest date to give, latest date to give, etc.
     */
    code: CodeableConcept;
    /**
     * The date whose meaning is specified by dateCriterion.code.
     */
    value: string;
    _value?: Element | undefined;
}
/**
 * A given instance of the .recommendation backbone element should correspond to a single recommended administration.
 */
export interface ImmunizationRecommendationRecommendation extends BackboneElement {
    /**
     * Vaccine(s) which should not be used to fulfill the recommendation.
     */
    contraindicatedVaccineCode?: CodeableConcept[] | undefined;
    /**
     * Vaccine date recommendations.  For example, earliest date to administer, latest date to administer, etc.
     */
    dateCriterion?: ImmunizationRecommendationRecommendationDateCriterion[] | undefined;
    /**
     * Contains the description about the protocol under which the vaccine was administered.
     */
    description?: string | undefined;
    _description?: Element | undefined;
    /**
     * The use of an integer is preferred if known. A string should only be used in cases where an integer is not available (such as when documenting a recurring booster dose).
     */
    doseNumber?: string | undefined;
    _doseNumber?: Element | undefined;
    /**
     * The reason for the assigned forecast status.
     */
    forecastReason?: CodeableConcept[] | undefined;
    /**
     * Indicates the patient status with respect to the path to immunity for the target disease.
     */
    forecastStatus: CodeableConcept;
    /**
     * One possible path to achieve presumed immunity against a disease - within the context of an authority.
     */
    series?: string | undefined;
    _series?: Element | undefined;
    /**
     * The use of an integer is preferred if known.
     */
    seriesDoses?: string | undefined;
    _seriesDoses?: Element | undefined;
    /**
     * Immunization event history and/or evaluation that supports the status and recommendation.
     */
    supportingImmunization?: Reference[] | undefined;
    /**
     * Patient Information that supports the status and recommendation.  This includes patient observations, adverse reactions and allergy/intolerance information.
     */
    supportingPatientInformation?: Reference[] | undefined;
    /**
     * A given instance of the .recommendation backbone element should correspond to a single recommended administration but sometimes that administration (eg. MMR) will encompass multiple diseases, thus the targetDisease element is allowed to repeat.
     */
    targetDisease?: CodeableConcept[] | undefined;
    /**
     * Vaccine(s) or vaccine group that pertain to the recommendation.
     */
    vaccineCode?: CodeableConcept[] | undefined;
}
/**
 * A patient's point-in-time set of recommendations (i.e. forecasting) according to a published schedule with optional supporting justification.
 */
export interface ImmunizationRecommendation extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'ImmunizationRecommendation';
    /**
     * Indicates the authority who published the protocol (e.g. ACIP).
     */
    authority?: Reference | undefined;
    /**
     * The date the immunization recommendation(s) were created.
     */
    date: string;
    _date?: Element | undefined;
    /**
     * A unique identifier assigned to this particular recommendation record.
     */
    identifier?: Identifier[] | undefined;
    /**
     * The patient the recommendation(s) are for.
     */
    patient: Reference;
    /**
     * A given instance of the .recommendation backbone element should correspond to a single recommended administration.
     */
    recommendation: ImmunizationRecommendationRecommendation[];
}
/**
 * Another implementation guide that this implementation depends on. Typically, an implementation guide uses value sets, profiles etc.defined in other implementation guides.
 */
export interface ImplementationGuideDependsOn extends BackboneElement {
    /**
     * The NPM package name for the Implementation Guide that this IG depends on.
     */
    packageId?: string | undefined;
    _packageId?: Element | undefined;
    /**
     * This doesn't need to enumerate every resource used, but should give some sense of why the dependency exists.  It will be used in the rendered list of dependencies
     */
    reason?: string | undefined;
    _reason?: Element | undefined;
    /**
     * Usually, A canonical reference to the implementation guide is the same as the master location at which the implementation guide is published.
     */
    uri: string;
    _uri?: Element | undefined;
    /**
     * This follows the syntax of the NPM packaging version field - see [[reference]].
     */
    version?: string | undefined;
    _version?: Element | undefined;
}
/**
 * See [Default Profiles](implementationguide.html#default) for a discussion of which resources are 'covered' by an implementation guide.
 */
export interface ImplementationGuideGlobal extends BackboneElement {
    /**
     * A reference to the profile that all instances must conform to.
     */
    profile: string;
    _profile?: Element | undefined;
    /**
     * The type must match that of the profile that is referred to but is made explicit here as a denormalization so that a system processing the implementation guide resource knows which resources the profile applies to even if the profile itself is not available.
     */
    type: string;
    _type?: Element | undefined;
}
/**
 * Groupings are arbitrary sub-divisions of content. Typically, they are used to help build Table of Contents automatically.
 */
export interface ImplementationGuideDefinitionGrouping extends BackboneElement {
    /**
     * Human readable text describing the package.
     */
    description?: string | undefined;
    _description?: Element | undefined;
    /**
     * The human-readable title to display for the package of resources when rendering the implementation guide.
     */
    name: string;
    _name?: Element | undefined;
}
/**
 * A resource that is part of the implementation guide. Conformance resources (value set, structure definition, capability statements etc.) are obvious candidates for inclusion, but any kind of resource can be included as an example resource.
 */
export interface ImplementationGuideDefinitionResource extends BackboneElement {
    /**
     * This is mostly used with examples to explain why it is present (though they can have extensive comments in the examples).
     */
    description?: string | undefined;
    _description?: Element | undefined;
    /**
     * The resource SHALL be valid against all the versions it is specified to apply to. If the resource referred to is a StructureDefinition, the fhirVersion stated in the StructureDefinition cannot disagree with the version specified here; the specified versions SHALL include the version specified by the StructureDefinition, and may include additional versions using the [applicable-version](extension-structuredefinition-applicable-version.html) extension.
     */
    fhirVersion?: string[] | undefined;
    _fhirVersion?: Element[] | undefined;
    /**
     * This must correspond to a group.id element within this implementation guide.
     */
    groupingId?: string | undefined;
    _groupingId?: Element | undefined;
    /**
     * If true, indicates the resource is an example instance.
     */
    isExample?: boolean | undefined;
    _isExample?: Element | undefined;
    /**
     * A human assigned name for the resource. All resources SHOULD have a name, but the name may be extracted from the resource (e.g. ValueSet.name).
     */
    name?: string | undefined;
    _name?: Element | undefined;
    /**
     * Examples:
     * * StructureDefinition -> Any
     * * ValueSet -> expansion
     * * OperationDefinition -> Parameters
     * * Questionnaire -> QuestionnaireResponse.
     */
    profile?: string[] | undefined;
    _profile?: Element[] | undefined;
    /**
     * Usually this is a relative URL that locates the resource within the implementation guide. If you authoring an implementation guide, and will publish it using the FHIR publication tooling, use a URI that may point to a resource, or to one of various alternative representations (e.g. spreadsheet). The tooling will convert this when it publishes it.
     */
    reference: Reference;
}
/**
 * Pages automatically become sections if they have sub-pages. By convention, the home page is called index.html.
 */
export interface ImplementationGuideDefinitionPage extends BackboneElement {
    /**
     * A code that indicates how the page is generated.
     */
    generation: 'html' | 'markdown' | 'xml' | 'generated';
    _generation?: Element | undefined;
    /**
     * This SHALL be a local reference, expressed with respect to the root of the IG output folder.  No suffix is required.  If no suffix is specified, .html will be appended.
     */
    name: string;
    _name?: Element | undefined;
    /**
     * The implementation guide breadcrumbs may be generated from this structure.
     */
    page?: ImplementationGuideDefinitionPage[] | undefined;
    /**
     * If absent and the page isn't a generated page, this may be inferred from the page name by checking input locations.  String is used for XHTML content - sent as an escaped string.  FHIR tooling can't support 'direct' XHTML anywhere other than in narrative.
     */
    sourceUrl?: string | undefined;
    _sourceUrl?: Element | undefined;
    /**
     * If absent and the page isn't a generated page, this may be inferred from the page name by checking input locations.  String is used for XHTML content - sent as an escaped string.  FHIR tooling can't support 'direct' XHTML anywhere other than in narrative.
     */
    sourceString?: string | undefined;
    _sourceString?: Element | undefined;
    /**
     * If absent and the page isn't a generated page, this may be inferred from the page name by checking input locations.  String is used for XHTML content - sent as an escaped string.  FHIR tooling can't support 'direct' XHTML anywhere other than in narrative.
     */
    sourceMarkdown?: string | undefined;
    _sourceMarkdown?: Element | undefined;
    /**
     * A short title used to represent this page in navigational structures such as table of contents, bread crumbs, etc.
     */
    title: string;
    _title?: Element | undefined;
}
/**
 * see [confluence](https://confluence.hl7.org/display/FHIR/Implementation+Guide+Parameters) for the parameters defined by the HL7 IG publisher.
 */
export interface ImplementationGuideDefinitionParameter extends BackboneElement {
    /**
     * A tool-specific code that defines the parameter.
     */
    code: Coding;
    /**
     * Value for named type.
     */
    value: string;
    _value?: Element | undefined;
}
/**
 * A template for building resources.
 */
export interface ImplementationGuideDefinitionTemplate extends BackboneElement {
    /**
     * Type of template specified.
     */
    code: string;
    _code?: Element | undefined;
    /**
     * The scope in which the template applies.
     */
    scope?: string | undefined;
    _scope?: Element | undefined;
    /**
     * The source location for the template.
     */
    source: string;
    _source?: Element | undefined;
}
/**
 * Principally, this consists of information abuot source resource and file locations, and build parameters and templates.
 */
export interface ImplementationGuideDefinition extends BackboneElement {
    /**
     * Groupings are arbitrary sub-divisions of content. Typically, they are used to help build Table of Contents automatically.
     */
    grouping?: ImplementationGuideDefinitionGrouping[] | undefined;
    /**
     * Pages automatically become sections if they have sub-pages. By convention, the home page is called index.html.
     */
    page?: ImplementationGuideDefinitionPage | undefined;
    /**
     * see [confluence](https://confluence.hl7.org/display/FHIR/Implementation+Guide+Parameters) for the parameters defined by the HL7 IG publisher.
     */
    parameter?: ImplementationGuideDefinitionParameter[] | undefined;
    /**
     * A resource that is part of the implementation guide. Conformance resources (value set, structure definition, capability statements etc.) are obvious candidates for inclusion, but any kind of resource can be included as an example resource.
     */
    resource?: ImplementationGuideDefinitionResource[] | undefined;
    /**
     * A template for building resources.
     */
    template?: ImplementationGuideDefinitionTemplate[] | undefined;
}
/**
 * A resource that is part of the implementation guide. Conformance resources (value set, structure definition, capability statements etc.) are obvious candidates for inclusion, but any kind of resource can be included as an example resource.
 */
export interface ImplementationGuideManifestResource extends BackboneElement {
    /**
     * If true, indicates the resource is an example instance.
     */
    isExample?: boolean | undefined;
    _isExample?: Element | undefined;
    /**
     * Examples:
     * * StructureDefinition -> Any
     * * ValueSet -> expansion
     * * OperationDefinition -> Parameters
     * * Questionnaire -> QuestionnaireResponse.
     */
    profile?: string[] | undefined;
    _profile?: Element[] | undefined;
    /**
     * Usually this is a relative URL that locates the resource within the implementation guide. If you authoring an implementation guide, and will publish it using the FHIR publication tooling, use a URI that may point to a resource, or to one of various alternative representations (e.g. spreadsheet). The tooling will convert this when it publishes it.
     */
    reference: Reference;
    /**
     * Appending 'rendering' + "/" + this should resolve to the resource page.
     */
    relativePath?: string | undefined;
    _relativePath?: Element | undefined;
}
/**
 * Information about a page within the IG.
 */
export interface ImplementationGuideManifestPage extends BackboneElement {
    /**
     * Appending 'rendering' + "/" + page.name + "#" + page.anchor should resolve to the anchor.
     */
    anchor?: string[] | undefined;
    _anchor?: Element[] | undefined;
    /**
     * Appending 'rendering' + "/" + this should resolve to the page.
     */
    name: string;
    _name?: Element | undefined;
    /**
     * Label for the page intended for human display.
     */
    title?: string | undefined;
    _title?: Element | undefined;
}
/**
 * Information about an assembled implementation guide, created by the publication tooling.
 */
export interface ImplementationGuideManifest extends BackboneElement {
    /**
     * Indicates a relative path to an image that exists within the IG.
     */
    image?: string[] | undefined;
    _image?: Element[] | undefined;
    /**
     * Indicates the relative path of an additional non-page, non-image file that is part of the IG - e.g. zip, jar and similar files that could be the target of a hyperlink in a derived IG.
     */
    other?: string[] | undefined;
    _other?: Element[] | undefined;
    /**
     * Information about a page within the IG.
     */
    page?: ImplementationGuideManifestPage[] | undefined;
    /**
     * A pointer to official web page, PDF or other rendering of the implementation guide.
     */
    rendering?: string | undefined;
    _rendering?: Element | undefined;
    /**
     * A resource that is part of the implementation guide. Conformance resources (value set, structure definition, capability statements etc.) are obvious candidates for inclusion, but any kind of resource can be included as an example resource.
     */
    resource: ImplementationGuideManifestResource[];
}
/**
 * A set of rules of how a particular interoperability or standards problem is solved - typically through the use of FHIR resources. This resource is used to gather all the parts of an implementation guide into a logical whole and to publish a computable definition of all the parts.
 */
export interface ImplementationGuide extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'ImplementationGuide';
    /**
     * May be a web site, an email address, a telephone number, etc.
     */
    contact?: ContactDetail[] | undefined;
    /**
     * ...
     */
    copyright?: string | undefined;
    _copyright?: Element | undefined;
    /**
     * The (c) symbol should NOT be included in this string. It will be added by software when rendering the notation. Full details about licensing, restrictions, warrantees, etc. goes in the more general 'copyright' element.
     */
    copyrightLabel?: string | undefined;
    _copyrightLabel?: Element | undefined;
    /**
     * Note that this is not the same as the resource last-modified-date, since the resource may be a secondary representation of the implementation guide. Additional specific dates may be added as extensions or be found by consulting Provenances associated with past versions of the resource.
     */
    date?: string | undefined;
    _date?: Element | undefined;
    /**
     * Principally, this consists of information abuot source resource and file locations, and build parameters and templates.
     */
    definition?: ImplementationGuideDefinition | undefined;
    /**
     * Another implementation guide that this implementation depends on. Typically, an implementation guide uses value sets, profiles etc.defined in other implementation guides.
     */
    dependsOn?: ImplementationGuideDependsOn[] | undefined;
    /**
     * This description can be used to capture details such as why the implementation guide was built, comments about misuse, instructions for clinical use and interpretation, literature references, examples from the paper world, etc. It is not a rendering of the implementation guide as conveyed in the 'text' field of the resource itself. This item SHOULD be populated unless the information is available from context (e.g. the language of the implementation guide is presumed to be the predominant language in the place the implementation guide was created).
     */
    description?: string | undefined;
    _description?: Element | undefined;
    /**
     * Allows filtering of implementation guides that are appropriate for use versus not.
     */
    experimental?: boolean | undefined;
    _experimental?: Element | undefined;
    /**
     * Most implementation guides target a single version - e.g. they describe how to use a particular version, and the profiles and examples etc are valid for that version. But some implementation guides describe how to use multiple different versions of FHIR to solve the same problem, or in concert with each other. Typically, the requirement to support multiple versions arises as implementation matures and different implementation communities are stuck at different versions by regulation or market dynamics.
     */
    fhirVersion: string[];
    _fhirVersion?: Element[] | undefined;
    /**
     * See [Default Profiles](implementationguide.html#default) for a discussion of which resources are 'covered' by an implementation guide.
     */
    global?: ImplementationGuideGlobal[] | undefined;
    /**
     * It may be possible for the implementation guide to be used in jurisdictions other than those for which it was originally designed or intended.
     */
    jurisdiction?: CodeableConcept[] | undefined;
    /**
     * The license that applies to this Implementation Guide, using an SPDX license code, or 'not-open-source'.
     */
    license?: string | undefined;
    _license?: Element | undefined;
    /**
     * Information about an assembled implementation guide, created by the publication tooling.
     */
    manifest?: ImplementationGuideManifest | undefined;
    /**
     * The name is not expected to be globally unique. The name should be a simple alphanumeric type name to ensure that it is machine-processing friendly.
     */
    name: string;
    _name?: Element | undefined;
    /**
     * Many (if not all) IG publishing tools will require that this element be present. For implementation guides published through HL7 or the FHIR foundation, the FHIR product director assigns package IDs.
     */
    packageId: string;
    _packageId?: Element | undefined;
    /**
     * Usually an organization but may be an individual. The publisher (or steward) of the implementation guide is the organization or individual primarily responsible for the maintenance and upkeep of the implementation guide. This is not necessarily the same individual or organization that developed and initially authored the content. The publisher is the primary point of contact for questions or issues with the implementation guide. This item SHOULD be populated unless the information is available from context.
     */
    publisher?: string | undefined;
    _publisher?: Element | undefined;
    /**
     * Allows filtering of implementation guides that are appropriate for use versus not.
     */
    status: 'draft' | 'active' | 'retired' | 'unknown';
    _status?: Element | undefined;
    /**
     * This name does not need to be machine-processing friendly and may contain punctuation, white-space, etc.
     */
    title?: string | undefined;
    _title?: Element | undefined;
    /**
     * Can be a urn:uuid: or a urn:oid: but real http: addresses are preferred.  Multiple instances may share the same URL if they have a distinct version.
     * The determination of when to create a new version of a resource (same url, new version) vs. defining a new artifact is up to the author.  Considerations for making this decision are found in [Technical and Business Versions](resource.html#versions).
     * In some cases, the resource can no longer be found at the stated url, but the url itself cannot change. Implementations can use the [meta.source](resource.html#meta) element to indicate where the current master source of the resource can be found.
     */
    url: string;
    _url?: Element | undefined;
    /**
     * When multiple useContexts are specified, there is no expectation that all or any of the contexts apply.
     */
    useContext?: UsageContext[] | undefined;
    /**
     * There may be different implementation guide instances that have the same identifier but different versions.  The version can be appended to the url in a reference to allow a reference to a particular business version of the implementation guide with the format [url]|[version]. The version SHOULD NOT contain a '#' - see [Business Version](resource.html#bv-format).
     */
    version?: string | undefined;
    _version?: Element | undefined;
    /**
     * If set as a string, this is a FHIRPath expression that has two additional context variables passed in - %version1 and %version2 and will return a negative number if version1 is newer, a positive number if version2 and a 0 if the version ordering can't be successfully be determined.
     */
    versionAlgorithmString?: string | undefined;
    _versionAlgorithmString?: Element | undefined;
    /**
     * If set as a string, this is a FHIRPath expression that has two additional context variables passed in - %version1 and %version2 and will return a negative number if version1 is newer, a positive number if version2 and a 0 if the version ordering can't be successfully be determined.
     */
    versionAlgorithmCoding?: Coding | undefined;
}
/**
 * The organization(s) that manufacture this ingredient. Can be used to indicate:         1) Organizations we are aware of that manufacture this ingredient         2) Specific Manufacturer(s) currently being used         3) Set of organisations allowed to manufacture this ingredient for this product         Users must be clear on the application of context relevant to their use case.
 */
export interface IngredientManufacturer extends BackboneElement {
    /**
     * An organization that manufactures this ingredient.
     */
    manufacturer: Reference;
    /**
     * The way in which this manufacturer is associated with the ingredient. For example whether it is a possible one (others allowed), or an exclusive authorized one for this ingredient. Note that this is not the manufacturing process role.
     */
    role?: ('allowed' | 'possible' | 'actual') | undefined;
    _role?: Element | undefined;
}
/**
 * Strength expressed in terms of a reference substance. For when the ingredient strength is additionally expressed as equivalent to the strength of some other closely related substance (e.g. salt vs. base). Reference strength represents the strength (quantitative composition) of the active moiety of the active substance. There are situations when the active substance and active moiety are different, therefore both a strength and a reference strength are needed.
 */
export interface IngredientSubstanceStrengthReferenceStrength extends BackboneElement {
    /**
     * The country or countries for which the strength range applies.
     */
    country?: CodeableConcept[] | undefined;
    /**
     * For when strength is measured at a particular point or distance.
     */
    measurementPoint?: string | undefined;
    _measurementPoint?: Element | undefined;
    /**
     * Strength expressed in terms of a reference substance.
     */
    strengthRatio?: Ratio | undefined;
    /**
     * Strength expressed in terms of a reference substance.
     */
    strengthRatioRange?: RatioRange | undefined;
    /**
     * Strength expressed in terms of a reference substance.
     */
    strengthQuantity?: Quantity | undefined;
    /**
     * Relevant reference substance.
     */
    substance?: CodeableReference | undefined;
}
/**
 * The quantity of substance in the unit of presentation, or in the volume (or mass) of the single pharmaceutical product or manufactured item. The allowed repetitions do not represent different strengths, but are different representations - mathematically equivalent - of a single strength.
 */
export interface IngredientSubstanceStrength extends BackboneElement {
    /**
     * A code that indicates if the strength is, for example, based on the ingredient substance as stated or on the substance base (when the ingredient is a salt).
     */
    basis?: CodeableConcept | undefined;
    /**
     * The strength per unitary volume (or mass).
     */
    concentrationRatio?: Ratio | undefined;
    /**
     * The strength per unitary volume (or mass).
     */
    concentrationRatioRange?: RatioRange | undefined;
    /**
     * The strength per unitary volume (or mass).
     */
    concentrationCodeableConcept?: CodeableConcept | undefined;
    /**
     * The strength per unitary volume (or mass).
     */
    concentrationQuantity?: Quantity | undefined;
    /**
     * The country or countries for which the strength range applies.
     */
    country?: CodeableConcept[] | undefined;
    /**
     * For when strength is measured at a particular point or distance. There are products where strength is measured at a particular point. For example, the strength of the ingredient in some inhalers is measured at a particular position relative to the point of aerosolization.
     */
    measurementPoint?: string | undefined;
    _measurementPoint?: Element | undefined;
    /**
     * The quantity of substance in the unit of presentation, or in the volume (or mass) of the single pharmaceutical product or manufactured item. Unit of presentation refers to the quantity that the item occurs in e.g. a strength per tablet size, perhaps 'per 20mg' (the size of the tablet). It is not generally normalized as a unitary unit, which would be 'per mg').
     */
    presentationRatio?: Ratio | undefined;
    /**
     * The quantity of substance in the unit of presentation, or in the volume (or mass) of the single pharmaceutical product or manufactured item. Unit of presentation refers to the quantity that the item occurs in e.g. a strength per tablet size, perhaps 'per 20mg' (the size of the tablet). It is not generally normalized as a unitary unit, which would be 'per mg').
     */
    presentationRatioRange?: RatioRange | undefined;
    /**
     * The quantity of substance in the unit of presentation, or in the volume (or mass) of the single pharmaceutical product or manufactured item. Unit of presentation refers to the quantity that the item occurs in e.g. a strength per tablet size, perhaps 'per 20mg' (the size of the tablet). It is not generally normalized as a unitary unit, which would be 'per mg').
     */
    presentationCodeableConcept?: CodeableConcept | undefined;
    /**
     * The quantity of substance in the unit of presentation, or in the volume (or mass) of the single pharmaceutical product or manufactured item. Unit of presentation refers to the quantity that the item occurs in e.g. a strength per tablet size, perhaps 'per 20mg' (the size of the tablet). It is not generally normalized as a unitary unit, which would be 'per mg').
     */
    presentationQuantity?: Quantity | undefined;
    /**
     * Strength expressed in terms of a reference substance. For when the ingredient strength is additionally expressed as equivalent to the strength of some other closely related substance (e.g. salt vs. base). Reference strength represents the strength (quantitative composition) of the active moiety of the active substance. There are situations when the active substance and active moiety are different, therefore both a strength and a reference strength are needed.
     */
    referenceStrength?: IngredientSubstanceStrengthReferenceStrength[] | undefined;
    /**
     * A textual represention of either the whole of the concentration strength or a part of it - with the rest being in Strength.concentration as a ratio.
     */
    textConcentration?: string | undefined;
    _textConcentration?: Element | undefined;
    /**
     * A textual represention of either the whole of the presentation strength or a part of it - with the rest being in Strength.presentation as a ratio.
     */
    textPresentation?: string | undefined;
    _textPresentation?: Element | undefined;
}
/**
 * The substance that comprises this ingredient.
 */
export interface IngredientSubstance extends BackboneElement {
    /**
     * A code or full resource that represents the ingredient's substance.
     */
    code: CodeableReference;
    /**
     * The quantity of substance in the unit of presentation, or in the volume (or mass) of the single pharmaceutical product or manufactured item. The allowed repetitions do not represent different strengths, but are different representations - mathematically equivalent - of a single strength.
     */
    strength?: IngredientSubstanceStrength[] | undefined;
}
/**
 * An ingredient of a manufactured item or pharmaceutical product.
 */
export interface Ingredient extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'Ingredient';
    /**
     * If the ingredient is a known or suspected allergen. Note that this is a property of the substance, so if a reference to a SubstanceDefinition is used to decribe that (rather than just a code), the allergen information should go there, not here.
     */
    allergenicIndicator?: boolean | undefined;
    _allergenicIndicator?: Element | undefined;
    /**
     * The product which this ingredient is a constituent part of.
     */
    for?: Reference[] | undefined;
    /**
     * A classification of the ingredient identifying its precise purpose(s) in the drug product. This extends the Ingredient.role to add more detail. Example: antioxidant, alkalizing agent.
     */
    function?: CodeableConcept[] | undefined;
    /**
     * A classification of the ingredient according to where in the physical item it tends to be used, such the outer shell of a tablet, inner body or ink.
     */
    group?: CodeableConcept | undefined;
    /**
     * The identifier(s) of this Ingredient that are assigned by business processes and/or used to refer to it when a direct URL reference to the resource itself is not appropriate.
     */
    identifier?: Identifier | undefined;
    /**
     * The organization(s) that manufacture this ingredient. Can be used to indicate:         1) Organizations we are aware of that manufacture this ingredient         2) Specific Manufacturer(s) currently being used         3) Set of organisations allowed to manufacture this ingredient for this product         Users must be clear on the application of context relevant to their use case.
     */
    manufacturer?: IngredientManufacturer[] | undefined;
    /**
     * A classification of the ingredient identifying its purpose within the product, e.g. active, inactive.
     */
    role: CodeableConcept;
    /**
     * Allows filtering of ingredient that are appropriate for use versus not.
     */
    status: 'draft' | 'active' | 'retired' | 'unknown';
    _status?: Element | undefined;
    /**
     * The substance that comprises this ingredient.
     */
    substance: IngredientSubstance;
}
/**
 * The specific limits on the benefit.
 */
export interface InsurancePlanCoverageBenefitLimit extends BackboneElement {
    /**
     * Use `CodeableConcept.text` element if the data is free (uncoded) text.
     */
    code?: CodeableConcept | undefined;
    /**
     * May also be called “eligible expense,” “payment allowance,” or “negotiated rate.”.
     */
    value?: Quantity | undefined;
}
/**
 * Specific benefits under this type of coverage.
 */
export interface InsurancePlanCoverageBenefit extends BackboneElement {
    /**
     * The specific limits on the benefit.
     */
    limit?: InsurancePlanCoverageBenefitLimit[] | undefined;
    /**
     * The referral requirements to have access/coverage for this benefit.
     */
    requirement?: string | undefined;
    _requirement?: Element | undefined;
    /**
     * Type of benefit (primary care; speciality care; inpatient; outpatient).
     */
    type: CodeableConcept;
}
/**
 * Details about the coverage offered by the insurance product.
 */
export interface InsurancePlanCoverage extends BackboneElement {
    /**
     * Specific benefits under this type of coverage.
     */
    benefit: InsurancePlanCoverageBenefit[];
    /**
     * Networks are represented as a hierarchy of organization resources.
     */
    network?: Reference[] | undefined;
    /**
     * Type of coverage  (Medical; Dental; Mental Health; Substance Abuse; Vision; Drug; Short Term; Long Term Care; Hospice; Home Health).
     */
    type: CodeableConcept;
}
/**
 * Overall costs associated with the plan.
 */
export interface InsurancePlanPlanGeneralCost extends BackboneElement {
    /**
     * Additional information about the general costs associated with this plan.
     */
    comment?: string | undefined;
    _comment?: Element | undefined;
    /**
     * Value of the cost.
     */
    cost?: Money | undefined;
    /**
     * Number of participants enrolled in the plan.
     */
    groupSize?: number | undefined;
    /**
     * Type of cost.
     */
    type?: CodeableConcept | undefined;
}
/**
 * List of the costs associated with a specific benefit.
 */
export interface InsurancePlanPlanSpecificCostBenefitCost extends BackboneElement {
    /**
     * Whether the cost applies to in-network or out-of-network providers (in-network; out-of-network; other).
     */
    applicability?: CodeableConcept | undefined;
    /**
     * Additional information about the cost, such as information about funding sources (e.g. HSA, HRA, FSA, RRA).
     */
    qualifiers?: CodeableConcept[] | undefined;
    /**
     * Type of cost (copay; individual cap; family cap; coinsurance; deductible).
     */
    type: CodeableConcept;
    /**
     * The actual cost value. (some of the costs may be represented as percentages rather than currency, e.g. 10% coinsurance).
     */
    value?: Quantity | undefined;
}
/**
 * List of the specific benefits under this category of benefit.
 */
export interface InsurancePlanPlanSpecificCostBenefit extends BackboneElement {
    /**
     * List of the costs associated with a specific benefit.
     */
    cost?: InsurancePlanPlanSpecificCostBenefitCost[] | undefined;
    /**
     * Type of specific benefit (preventative; primary care office visit; speciality office visit; hospitalization; emergency room; urgent care).
     */
    type: CodeableConcept;
}
/**
 * Costs associated with the coverage provided by the product.
 */
export interface InsurancePlanPlanSpecificCost extends BackboneElement {
    /**
     * List of the specific benefits under this category of benefit.
     */
    benefit?: InsurancePlanPlanSpecificCostBenefit[] | undefined;
    /**
     * General category of benefit (Medical; Dental; Vision; Drug; Mental Health; Substance Abuse; Hospice, Home Health).
     */
    category: CodeableConcept;
}
/**
 * Details about an insurance plan.
 */
export interface InsurancePlanPlan extends BackboneElement {
    /**
     * The geographic region in which a health insurance plan's benefits apply.
     */
    coverageArea?: Reference[] | undefined;
    /**
     * Overall costs associated with the plan.
     */
    generalCost?: InsurancePlanPlanGeneralCost[] | undefined;
    /**
     * Business identifiers assigned to this health insurance plan which remain constant as the resource is updated and propagates from server to server.
     */
    identifier?: Identifier[] | undefined;
    /**
     * Networks are represented as a hierarchy of organization resources.
     */
    network?: Reference[] | undefined;
    /**
     * Costs associated with the coverage provided by the product.
     */
    specificCost?: InsurancePlanPlanSpecificCost[] | undefined;
    /**
     * Type of plan. For example, "Platinum" or "High Deductable".
     */
    type?: CodeableConcept | undefined;
}
/**
 * Details of a Health Insurance product/plan provided by an organization.
 */
export interface InsurancePlan extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'InsurancePlan';
    /**
     * An organization which administer other services such as underwriting, customer service and/or claims processing on behalf of the health insurance product owner.
     */
    administeredBy?: Reference | undefined;
    /**
     * There are no dates associated with the alias/historic names, as this is not intended to track when names were used, but to assist in searching so that older names can still result in identifying the product/plan.
     */
    alias?: string[] | undefined;
    _alias?: Element[] | undefined;
    /**
     * The address/telecom use code 'home' are not to be used.
     */
    contact?: ExtendedContactDetail[] | undefined;
    /**
     * Details about the coverage offered by the insurance product.
     */
    coverage?: InsurancePlanCoverage[] | undefined;
    /**
     * The geographic region in which a health insurance product's benefits apply.
     */
    coverageArea?: Reference[] | undefined;
    /**
     * The technical endpoints providing access to services operated for the health insurance product.
     */
    endpoint?: Reference[] | undefined;
    /**
     * Business identifiers assigned to this health insurance product which remain constant as the resource is updated and propagates from server to server.
     */
    identifier?: Identifier[] | undefined;
    /**
     * If the name of the product/plan changes, consider putting the old name in the alias column so that it can still be located through searches.
     */
    name?: string | undefined;
    _name?: Element | undefined;
    /**
     * Networks are represented as a hierarchy of organization resources.
     */
    network?: Reference[] | undefined;
    /**
     * The entity that is providing  the health insurance product and underwriting the risk.  This is typically an insurance carriers, other third-party payers, or health plan sponsors comonly referred to as 'payers'.
     */
    ownedBy?: Reference | undefined;
    /**
     * The period of time that the health insurance product is available.
     */
    period?: Period | undefined;
    /**
     * Details about an insurance plan.
     */
    plan?: InsurancePlanPlan[] | undefined;
    /**
     * The current state of the health insurance product.
     */
    status?: ('draft' | 'active' | 'retired' | 'unknown') | undefined;
    _status?: Element | undefined;
    /**
     * The kind of health insurance product.
     */
    type?: CodeableConcept[] | undefined;
}
/**
 * The item or items in this listing.
 */
export interface InventoryReportInventoryListingItems extends BackboneElement {
    /**
     * The category of the item or items.
     */
    category?: CodeableConcept | undefined;
    /**
     * The expiry date of the item or items.
     */
    expiry?: string | undefined;
    _expiry?: Element | undefined;
    /**
     * The code or reference to the item type.
     */
    item: CodeableReference;
    /**
     * The lot number of the item or items.
     */
    lot?: string | undefined;
    _lot?: Element | undefined;
    /**
     * The manufacturingDate of the item or items.
     */
    manufacturingDate?: string | undefined;
    _manufacturingDate?: Element | undefined;
    /**
     * The quantity of the item or items.
     */
    quantity: Quantity;
    /**
     * The serial number of the item.
     */
    serial?: string | undefined;
    _serial?: Element | undefined;
}
/**
 * An inventory listing section (grouped by any of the attributes).
 */
export interface InventoryReportInventoryListing extends BackboneElement {
    /**
     * The date and time when the items were counted.
     */
    countingDateTime?: string | undefined;
    _countingDateTime?: Element | undefined;
    /**
     * The item or items in this listing.
     */
    items?: InventoryReportInventoryListingItems[] | undefined;
    /**
     * The status of the items.
     */
    itemStatus?: CodeableConcept | undefined;
    /**
     * Location of the inventory items.
     */
    location?: Reference | undefined;
}
/**
 * A report of inventory or stock items.
 */
export interface InventoryReport extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'InventoryReport';
    /**
     * Whether the report is about the current inventory count (snapshot) or a differential change in inventory (change).
     */
    countType: 'snapshot' | 'difference';
    _countType?: Element | undefined;
    /**
     * Identifiers for the InventoryReport.
     */
    identifier?: Identifier[] | undefined;
    /**
     * An inventory listing section (grouped by any of the attributes).
     */
    inventoryListing?: InventoryReportInventoryListing[] | undefined;
    /**
     * A note associated with the InventoryReport.
     */
    note?: Annotation | undefined;
    /**
     * What type of operation is being performed - addition or subtraction.
     */
    operationType?: CodeableConcept | undefined;
    /**
     * The reason for this count - regular count, ad-hoc count, new arrivals, etc.
     */
    operationTypeReason?: CodeableConcept | undefined;
    /**
     * When the report has been submitted.
     */
    reportedDateTime: string;
    _reportedDateTime?: Element | undefined;
    /**
     * Who submits the report.
     */
    reporter?: Reference | undefined;
    /**
     * The period the report refers to.
     */
    reportingPeriod?: Period | undefined;
    /**
     * The status of the inventory check or notification - whether this is draft (e.g. the report is still pending some updates) or active.
     */
    status: 'draft' | 'requested' | 'active' | 'entered-in-error';
    _status?: Element | undefined;
}
/**
 * Indicates who or what performed or participated in the charged service.
 */
export interface InvoiceParticipant extends BackboneElement {
    /**
     * The device, practitioner, etc. who performed or participated in the service.
     */
    actor: Reference;
    /**
     * Describes the type of involvement (e.g. transcriptionist, creator etc.). If the invoice has been created automatically, the Participant may be a billing engine or another kind of device.
     */
    role?: CodeableConcept | undefined;
}
/**
 * Each line item represents one charge for goods and services rendered. Details such as date, code and amount are found in the referenced ChargeItem resource.
 */
export interface InvoiceLineItem extends BackboneElement {
    /**
     * The ChargeItem contains information such as the billing code, date, amount etc. If no further details are required for the lineItem, inline billing codes can be added using the CodeableConcept data type instead of the Reference.
     */
    chargeItemReference?: Reference | undefined;
    /**
     * The ChargeItem contains information such as the billing code, date, amount etc. If no further details are required for the lineItem, inline billing codes can be added using the CodeableConcept data type instead of the Reference.
     */
    chargeItemCodeableConcept?: CodeableConcept | undefined;
    /**
     * The price for a ChargeItem may be calculated as a base price with surcharges/deductions that apply in certain conditions. A ChargeItemDefinition resource that defines the prices, factors and conditions that apply to a billing code is currently under development. The priceComponent element can be used to offer transparency to the recipient of the Invoice as to how the prices have been calculated.
     */
    priceComponent?: MonetaryComponent[] | undefined;
    /**
     * Sequence in which the items appear on the invoice.
     */
    sequence?: number | undefined;
    /**
     * Date/time(s) range when this service was delivered or completed.
     */
    servicedDate?: string | undefined;
    _servicedDate?: Element | undefined;
    /**
     * Date/time(s) range when this service was delivered or completed.
     */
    servicedPeriod?: Period | undefined;
}
/**
 * Invoice containing collected ChargeItems from an Account with calculated individual and total price for Billing purpose.
 */
export interface Invoice extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'Invoice';
    /**
     * Systems posting the ChargeItems might not always be able to determine, which accounts the Items need to be places into. It is up to the potprocessing Financial System to apply internal rules to decide based on the Encounter/EpisodeOfCare/Patient/Coverage context and the type of ChargeItem, which Account is appropriate.
     */
    account?: Reference | undefined;
    /**
     * Derived Profiles may choose to add invariants requiring this field to be populated if either priceOverride or factorOverride have been filled.
     */
    cancelledReason?: string | undefined;
    _cancelledReason?: Element | undefined;
    /**
     * The list of types may be constrained as appropriate for the type of charge item.
     */
    creation?: string | undefined;
    _creation?: Element | undefined;
    /**
     * Depricared by the element below.
     */
    date?: string | undefined;
    _date?: Element | undefined;
    /**
     * Identifier of this Invoice, often used for reference in correspondence about this invoice or for tracking of payments.
     */
    identifier?: Identifier[] | undefined;
    /**
     * Practitioners and Devices can be associated with multiple organizations. It has to be made clear, on behalf of which Organization the services have been rendered.
     */
    issuer?: Reference | undefined;
    /**
     * Each line item represents one charge for goods and services rendered. Details such as date, code and amount are found in the referenced ChargeItem resource.
     */
    lineItem?: InvoiceLineItem[] | undefined;
    /**
     * Comments made about the invoice by the issuer, subject, or other participants.
     */
    note?: Annotation[] | undefined;
    /**
     * Indicates who or what performed or participated in the charged service.
     */
    participant?: InvoiceParticipant[] | undefined;
    /**
     * Derived Profiles may chose to add invariants requiring this field to be populated if either priceOverride or factorOverride have been filled.
     */
    paymentTerms?: string | undefined;
    _paymentTerms?: Element | undefined;
    /**
     * Date/time(s) range of services included in this invoice.
     */
    periodDate?: string | undefined;
    _periodDate?: Element | undefined;
    /**
     * Date/time(s) range of services included in this invoice.
     */
    periodPeriod?: Period | undefined;
    /**
     * The individual or Organization responsible for balancing of this invoice.
     */
    recipient?: Reference | undefined;
    /**
     * The current state of the Invoice.
     */
    status: 'draft' | 'issued' | 'balanced' | 'cancelled' | 'entered-in-error';
    _status?: Element | undefined;
    /**
     * The individual or set of individuals receiving the goods and services billed in this invoice.
     */
    subject?: Reference | undefined;
    /**
     * There is no reason to carry the price in the instance of a ChargeItem unless circumstances require a manual override. The list prices or are usually defined in a back catalogue of the billing codes  (see ChargeItem.definition). Derived profiles may require a ChargeItem.overrideReason to be provided if either factor or price are manually overridden.
     */
    totalGross?: Money | undefined;
    /**
     * There is no reason to carry the price in the instance of a ChargeItem unless circumstances require a manual override. The list prices or are usually defined in a back catalogue of the billing codes  (see ChargeItem.definition). Derived profiles may require a ChargeItem.overrideReason to be provided if either factor or price are manually overridden.
     */
    totalNet?: Money | undefined;
    /**
     * The total amount for the Invoice may be calculated as the sum of the line items with surcharges/deductions that apply in certain conditions.  The priceComponent element can be used to offer transparency to the recipient of the Invoice of how the total price was calculated.
     */
    totalPriceComponent?: MonetaryComponent[] | undefined;
    /**
     * Type of Invoice depending on domain, realm an usage (e.g. internal/external, dental, preliminary).
     */
    type?: CodeableConcept | undefined;
}
/**
 * The Library resource is a general-purpose container for knowledge asset definitions. It can be used to describe and expose existing knowledge assets such as logic libraries and information model descriptions, as well as to describe a collection of knowledge assets.
 */
export interface Library extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'Library';
    /**
     * The 'date' element may be more recent than the approval date because of minor changes or editorial corrections.
     */
    approvalDate?: string | undefined;
    _approvalDate?: Element | undefined;
    /**
     * An individiual or organization primarily involved in the creation and maintenance of the content.
     */
    author?: ContactDetail[] | undefined;
    /**
     * May be a web site, an email address, a telephone number, etc.
     */
    contact?: ContactDetail[] | undefined;
    /**
     * The content of the library as an Attachment. The content may be a reference to a url, or may be directly embedded as a base-64 string. Either way, the contentType of the attachment determines how to interpret the content.
     */
    content?: Attachment[] | undefined;
    /**
     * A copyright statement relating to the library and/or its contents. Copyright statements are generally legal restrictions on the use and publishing of the library.
     */
    copyright?: string | undefined;
    _copyright?: Element | undefined;
    /**
     * Describes a set of data that must be provided in order to be able to successfully perform the computations defined by the library.
     */
    dataRequirement?: DataRequirement[] | undefined;
    /**
     * Note that this is not the same as the resource last-modified-date, since the resource may be a secondary representation of the library. Additional specific dates may be added as extensions or be found by consulting Provenances associated with past versions of the resource.
     */
    date?: string | undefined;
    _date?: Element | undefined;
    /**
     * This description can be used to capture details such as comments about misuse, instructions for clinical use and interpretation, literature references, examples from the paper world, etc. It is not a rendering of the library as conveyed in the 'text' field of the resource itself. This item SHOULD be populated unless the information is available from context (e.g. the language of the library is presumed to be the predominant language in the place the library was created).
     */
    description?: string | undefined;
    _description?: Element | undefined;
    /**
     * An individual or organization primarily responsible for internal coherence of the content.
     */
    editor?: ContactDetail[] | undefined;
    /**
     * The effective period for a library  determines when the content is applicable for usage and is independent of publication and review dates. For example, a measure intended to be used for the year 2016 might be published in 2015.
     */
    effectivePeriod?: Period | undefined;
    /**
     * An individual or organization responsible for officially endorsing the content for use in some setting.
     */
    endorser?: ContactDetail[] | undefined;
    /**
     * Allows filtering of librarys that are appropriate for use versus not.
     */
    experimental?: boolean | undefined;
    _experimental?: Element | undefined;
    /**
     * Typically, this is used for identifiers that can go in an HL7 V3 II (instance identifier) data type, and can then identify this library outside of FHIR, where it is not possible to use the logical URI.
     */
    identifier?: Identifier[] | undefined;
    /**
     * It may be possible for the library to be used in jurisdictions other than those for which it was originally designed or intended.
     */
    jurisdiction?: CodeableConcept[] | undefined;
    /**
     * If specified, this date follows the original approval date.
     */
    lastReviewDate?: string | undefined;
    _lastReviewDate?: Element | undefined;
    /**
     * The name is not expected to be globally unique. The name should be a simple alphanumeric type name to ensure that it is machine-processing friendly.
     */
    name?: string | undefined;
    _name?: Element | undefined;
    /**
     * The parameter element defines parameters used by the library.
     */
    parameter?: ParameterDefinition[] | undefined;
    /**
     * Usually an organization but may be an individual. The publisher (or steward) of the library is the organization or individual primarily responsible for the maintenance and upkeep of the library. This is not necessarily the same individual or organization that developed and initially authored the content. The publisher is the primary point of contact for questions or issues with the library. This item SHOULD be populated unless the information is available from context.
     */
    publisher?: string | undefined;
    _publisher?: Element | undefined;
    /**
     * This element does not describe the usage of the library. Instead, it provides traceability of ''why'' the resource is either needed or ''why'' it is defined as it is.  This may be used to point to source materials or specifications that drove the structure of this library.
     */
    purpose?: string | undefined;
    _purpose?: Element | undefined;
    /**
     * Each related artifact is either an attachment, or a reference to another resource, but not both.
     */
    relatedArtifact?: RelatedArtifact[] | undefined;
    /**
     * An individual or organization primarily responsible for review of some aspect of the content.
     */
    reviewer?: ContactDetail[] | undefined;
    /**
     * Allows filtering of libraries that are appropriate for use vs. not.
     */
    status: 'draft' | 'active' | 'retired' | 'unknown';
    _status?: Element | undefined;
    /**
     * A code or group definition that describes the intended subject of the contents of the library.
     */
    subjectCodeableConcept?: CodeableConcept | undefined;
    /**
     * A code or group definition that describes the intended subject of the contents of the library.
     */
    subjectReference?: Reference | undefined;
    /**
     * An explanatory or alternate title for the library giving additional information about its content.
     */
    subtitle?: string | undefined;
    _subtitle?: Element | undefined;
    /**
     * This name does not need to be machine-processing friendly and may contain punctuation, white-space, etc.
     */
    title?: string | undefined;
    _title?: Element | undefined;
    /**
     * Descriptive topics related to the content of the library. Topics provide a high-level categorization of the library that can be useful for filtering and searching.
     */
    topic?: CodeableConcept[] | undefined;
    /**
     * Identifies the type of library such as a Logic Library, Model Definition, Asset Collection, or Module Definition.
     */
    type: CodeableConcept;
    /**
     * Can be a urn:uuid: or a urn:oid: but real http: addresses are preferred.  Multiple instances may share the same URL if they have a distinct version.
     * The determination of when to create a new version of a resource (same url, new version) vs. defining a new artifact is up to the author.  Considerations for making this decision are found in [Technical and Business Versions](resource.html#versions).
     * In some cases, the resource can no longer be found at the stated url, but the url itself cannot change. Implementations can use the [meta.source](resource.html#meta) element to indicate where the current master source of the resource can be found.
     */
    url?: string | undefined;
    _url?: Element | undefined;
    /**
     * A detailed description of how the library is used from a clinical perspective.
     */
    usage?: string | undefined;
    _usage?: Element | undefined;
    /**
     * When multiple useContexts are specified, there is no expectation that all or any of the contexts apply.
     */
    useContext?: UsageContext[] | undefined;
    /**
     * There may be different library instances that have the same identifier but different versions.  The version can be appended to the url in a reference to allow a reference to a particular business version of the library with the format [url]|[version]. The version SHOULD NOT contain a '#' - see [Business Version](resource.html#bv-format).
     */
    version?: string | undefined;
    _version?: Element | undefined;
}
/**
 * Identifies which record considered as the reference to the same real-world occurrence as well as how the items should be evaluated within the collection of linked items.
 */
export interface LinkageItem extends BackboneElement {
    /**
     * The resource instance being linked as part of the group.
     */
    resource: Reference;
    /**
     * Distinguishes which item is "source of truth" (if any) and which items are no longer considered to be current representations.
     */
    type: 'source' | 'alternate' | 'historical';
    _type?: Element | undefined;
}
/**
 * Identifies two or more records (resource instances) that refer to the same real-world "occurrence".
 */
export interface Linkage extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'Linkage';
    /**
     * If false, any asserted linkages should not be considered current/relevant/applicable.
     */
    active?: boolean | undefined;
    _active?: Element | undefined;
    /**
     * Identifies the user or organization responsible for asserting the linkages as well as the user or organization who establishes the context in which the nature of each linkage is evaluated.
     */
    author?: Reference | undefined;
    /**
     * Identifies which record considered as the reference to the same real-world occurrence as well as how the items should be evaluated within the collection of linked items.
     */
    item: LinkageItem[];
}
/**
 * If there are no entries in the list, an emptyReason SHOULD be provided.
 */
export interface ListEntry extends BackboneElement {
    /**
     * When this item was added to the list.
     */
    date?: string | undefined;
    _date?: Element | undefined;
    /**
     * If the flag means that the entry has actually been deleted from the list, the deleted element SHALL be true. A deleted entry should be displayed in narrative as deleted.  This element is labeled as a modifier because it indicates that an item is (to be) no longer in the list.
     */
    deleted?: boolean | undefined;
    _deleted?: Element | undefined;
    /**
     * The flag can only be understood in the context of the List.code. If the flag means that the entry has actually been deleted from the list, the deleted element SHALL be true
     */
    flag?: CodeableConcept | undefined;
    /**
     * A reference to the actual resource from which data was derived.
     */
    item: Reference;
}
/**
 * A List is a curated collection of resources, for things such as problem lists, allergy lists, facility list, organization list, etc.
 */
export interface List extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'List';
    /**
     * If there is no code, the purpose of the list is implied where it is used, such as in a document section using Document.section.code.
     */
    code?: CodeableConcept | undefined;
    /**
     * The actual important date is the date of currency of the resources that were summarized, but it is usually assumed that these are current when the preparation occurs.
     */
    date?: string | undefined;
    _date?: Element | undefined;
    /**
     * The various reasons for an empty list make a significant interpretation to its interpretation. Note that this code is for use when the entire list has been suppressed, and not for when individual items are omitted - implementers may consider using a text note or a flag on an entry in these cases.
     */
    emptyReason?: CodeableConcept | undefined;
    /**
     * The encounter that is the context in which this list was created.
     */
    encounter?: Reference | undefined;
    /**
     * If there are no entries in the list, an emptyReason SHOULD be provided.
     */
    entry?: ListEntry[] | undefined;
    /**
     * Identifier for the List assigned for business purposes outside the context of FHIR.
     */
    identifier?: Identifier[] | undefined;
    /**
     * This element is labeled as a modifier because a change list must not be misunderstood as a complete list.
     */
    mode: 'working' | 'snapshot' | 'changes';
    _mode?: Element | undefined;
    /**
     * Comments that apply to the overall list.
     */
    note?: Annotation[] | undefined;
    /**
     * Applications SHOULD render ordered lists in the order provided, but MAY allow users to re-order based on their own preferences as well. If there is no order specified, the order is unknown, though there may still be some order.
     */
    orderedBy?: CodeableConcept | undefined;
    /**
     * The primary source is the entity that made the decisions what items are in the list. This may be software or user.
     */
    source?: Reference | undefined;
    /**
     * This element is labeled as a modifier because the status contains codes that mark the resource as not currently valid.
     */
    status: 'current' | 'retired' | 'entered-in-error';
    _status?: Element | undefined;
    /**
     * Some purely arbitrary lists do not have a common subject, so this is optional.
     */
    subject?: Reference | undefined;
    /**
     * A label for the list assigned by the author.
     */
    title?: string | undefined;
    _title?: Element | undefined;
}
/**
 * To define a boundary shape for this location use the standard extension [`location-boundary-geojson`](extension-location-boundary-geojson.html), and search using the `contains` special search parameter.
 */
export interface LocationPosition extends BackboneElement {
    /**
     * Altitude. The value domain and the interpretation are the same as for the text of the altitude element in KML (see notes on Location main page).
     */
    altitude?: number | undefined;
    /**
     * Latitude. The value domain and the interpretation are the same as for the text of the latitude element in KML (see notes on Location main page).
     */
    latitude: number;
    /**
     * Longitude. The value domain and the interpretation are the same as for the text of the longitude element in KML (see notes on Location main page).
     */
    longitude: number;
}
/**
 * Details and position information for a physical place where services are provided and resources and participants may be stored, found, contained, or accommodated.
 */
export interface Location extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'Location';
    /**
     * Additional addresses should be recorded using another instance of the Location resource, or via the Organization.
     */
    address?: Address | undefined;
    /**
     * There are no dates associated with the alias/historic names, as this is not intended to track when names were used, but to assist in searching so that older names can still result in identifying the location.
     */
    alias?: string[] | undefined;
    _alias?: Element[] | undefined;
    /**
     * These could be such things as is wheelchair accessible.
     */
    characteristic?: CodeableConcept[] | undefined;
    /**
     * The address/telecom use code 'home' are not to be used. Note that these contacts are not the contact details of people who provide the service (that would be through PractitionerRole), these are official contacts for the Healthcare itself for specific purposes. E.g. Mailing Addresses, Billing Addresses, Contact numbers for Booking or Billing Enquiries, general web address, web address for online bookings etc.
     * If this is empty (or the type of interest is empty), refer to the organization's contacts.
     */
    contact?: ExtendedContactDetail[] | undefined;
    /**
     * Description of the Location, which helps in finding or referencing the place.
     */
    description?: string | undefined;
    _description?: Element | undefined;
    /**
     * Technical endpoints providing access to services operated for the location.
     */
    endpoint?: Reference[] | undefined;
    /**
     * Physical form of the location, e.g. building, room, vehicle, road, virtual.
     */
    form?: CodeableConcept | undefined;
    /**
     * This type of information is commonly found published in directories and on websites informing customers when the facility is available.
     * Specific services within the location may have their own hours which could be shorter (or longer) than the locations hours.
     * Systems may choose to render availability differently than it is exchanged on the interface. For example, rather than "Mon, Tue, Wed, Thur, Fri from 9am-12am; Mon, Tue, Wed, Thur, Fri from 1pm-5pm" as would be implied by two availableTime repetitions, an application could render this information as "Mon-Fri 9-12am and 1-5pm".
     * The availableStartTime is the opening time, and the availableEndTime is the closing time.
     */
    hoursOfOperation?: Availability[] | undefined;
    /**
     * Unique code or number identifying the location to its users.
     */
    identifier?: Identifier[] | undefined;
    /**
     * This can also be used as the part of the organization hierarchy where this location provides services. These services can be defined through the HealthcareService resource.
     */
    managingOrganization?: Reference | undefined;
    /**
     * This is labeled as a modifier because whether or not the location is a class of locations changes how it can be used and understood.
     */
    mode?: ('instance' | 'kind') | undefined;
    _mode?: Element | undefined;
    /**
     * If the name of a location changes, consider putting the old name in the alias column so that it can still be located through searches.
     */
    name?: string | undefined;
    _name?: Element | undefined;
    /**
     * The operational status covers operation values most relevant to beds (but can also apply to rooms/units/chairs/etc. such as an isolation unit/dialysis chair). This typically covers concepts such as contamination, housekeeping, and other activities like maintenance.
     */
    operationalStatus?: Coding | undefined;
    /**
     * Another Location of which this Location is physically a part of.
     */
    partOf?: Reference | undefined;
    /**
     * To define a boundary shape for this location use the standard extension [`location-boundary-geojson`](extension-location-boundary-geojson.html), and search using the `contains` special search parameter.
     */
    position?: LocationPosition | undefined;
    /**
     * The status property covers the general availability of the resource, not the current value which may be covered by the operationStatus, or by a schedule/slots if they are configured for the location.
     */
    status?: ('active' | 'suspended' | 'inactive') | undefined;
    _status?: Element | undefined;
    /**
     * Indicates the type of function performed at the location.
     */
    type?: CodeableConcept[] | undefined;
    /**
     * There are two types of virtual meetings that often exist:
     * * a persistent, virtual meeting room that can only be used for a single purpose at a time,
     *  * and a dynamic virtual meeting room that is generated on demand for a specific purpose.
     *  Implementers may consider using Appointment.virtualService for virtual meeting rooms that are generated on-demand.
     */
    virtualService?: VirtualServiceDetail[] | undefined;
}
/**
 * General characteristics of this item.
 */
export interface ManufacturedItemDefinitionProperty extends BackboneElement {
    /**
     * A code expressing the type of characteristic.
     */
    type: CodeableConcept;
    /**
     * A value for the characteristic.
     */
    valueCodeableConcept?: CodeableConcept | undefined;
    /**
     * A value for the characteristic.
     */
    valueQuantity?: Quantity | undefined;
    /**
     * A value for the characteristic.
     */
    valueDate?: string | undefined;
    _valueDate?: Element | undefined;
    /**
     * A value for the characteristic.
     */
    valueBoolean?: boolean | undefined;
    _valueBoolean?: Element | undefined;
    /**
     * A value for the characteristic.
     */
    valueAttachment?: Attachment | undefined;
}
/**
 * A reference to an constituent of the manufactured item as a whole, linked here so that its component location within the item can be indicated. This not where the item's ingredient are primarily stated (for which see Ingredient.for or ManufacturedItemDefinition.ingredient).
 */
export interface ManufacturedItemDefinitionComponentConstituent extends BackboneElement {
    /**
     * The measurable amount of this constituent in this component, expressable in different ways (e.g. by mass or volume).
     */
    amount?: Quantity[] | undefined;
    /**
     * The function of this constituent within the component e.g. binder.
     */
    function?: CodeableConcept[] | undefined;
    /**
     * The type of location of the constituent within this component e.g. intragranular, blend.
     */
    location?: CodeableConcept[] | undefined;
    /**
     * An ingredient that this component is the location for in this manufactured item. The component is physically made of this ingredient (and possibly others), rather than just being a container for it.
     */
    locationForIngredient?: CodeableReference[] | undefined;
}
/**
 * Physical parts of the manufactured item, that it is intrisically made from. This is distinct from the ingredients that are part of its chemical makeup.
 */
export interface ManufacturedItemDefinitionComponent extends BackboneElement {
    /**
     * The measurable amount of substance in this component, expressable in different ways (e.g. by mass or volume).
     */
    amount?: Quantity[] | undefined;
    /**
     * A component that this component contains or is made from.
     */
    component?: ManufacturedItemDefinitionComponent[] | undefined;
    /**
     * A reference to an constituent of the manufactured item as a whole, linked here so that its component location within the item can be indicated. This not where the item's ingredient are primarily stated (for which see Ingredient.for or ManufacturedItemDefinition.ingredient).
     */
    constituent?: ManufacturedItemDefinitionComponentConstituent[] | undefined;
    /**
     * The function of this component within the item e.g. delivers active ingredient, masks taste.
     */
    function?: CodeableConcept[] | undefined;
    /**
     * General characteristics of this component.
     */
    property?: ManufacturedItemDefinitionProperty[] | undefined;
    /**
     * Defining type of the component e.g. shell, layer, ink.
     */
    type: CodeableConcept;
}
/**
 * The definition and characteristics of a medicinal manufactured item, such as a tablet or capsule, as contained in a packaged medicinal product.
 */
export interface ManufacturedItemDefinition extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'ManufacturedItemDefinition';
    /**
     * Physical parts of the manufactured item, that it is intrisically made from. This is distinct from the ingredients that are part of its chemical makeup.
     */
    component?: ManufacturedItemDefinitionComponent[] | undefined;
    /**
     * Unique identifier.
     */
    identifier?: Identifier[] | undefined;
    /**
     * The ingredients of this manufactured item. This is only needed if the ingredients are not specified by incoming references from the Ingredient resource.
     */
    ingredient?: CodeableConcept[] | undefined;
    /**
     * Dose form as manufactured and before any transformation into the pharmaceutical product.
     */
    manufacturedDoseForm: CodeableConcept;
    /**
     * Manufacturer of the item, one of several possible.
     */
    manufacturer?: Reference[] | undefined;
    /**
     * Allows specifying that an item is on the market for sale, or that it is not available, and the dates and locations associated.
     */
    marketingStatus?: MarketingStatus[] | undefined;
    /**
     * A descriptive name applied to this item.
     */
    name?: string | undefined;
    _name?: Element | undefined;
    /**
     * General characteristics of this item.
     */
    property?: ManufacturedItemDefinitionProperty[] | undefined;
    /**
     * Allows filtering of manufactured items that are appropriate for use versus not.
     */
    status: 'draft' | 'active' | 'retired' | 'unknown';
    _status?: Element | undefined;
    /**
     * The “real world” units in which the quantity of the manufactured item is described.
     */
    unitOfPresentation?: CodeableConcept | undefined;
}
/**
 * Provides a description of an individual term used within the measure.
 */
export interface MeasureTerm extends BackboneElement {
    /**
     * A codeable representation of the defined term.
     */
    code?: CodeableConcept | undefined;
    /**
     * Provides a definition for the term as used within the measure.
     */
    definition?: string | undefined;
    _definition?: Element | undefined;
}
/**
 * A population criteria for the measure.
 */
export interface MeasureGroupPopulation extends BackboneElement {
    /**
     * Specifies which method should be used to aggregate measure observation values. For most scoring types, this is implied by scoring (e.g. a proportion measure counts members of the populations). For continuous variables, however, this information must be specified to ensure correct calculation.
     */
    aggregateMethod?: CodeableConcept | undefined;
    /**
     * The type of population criteria.
     */
    code?: CodeableConcept | undefined;
    /**
     * In the case of a continuous-variable or ratio measure, this may be the name of a function that calculates the value of the individual observation for each patient or event in the population. For these types of measures, individual observations are reported as observation resources included in the evaluatedResources bundle for each patient. See the MeasureReport resource or the Quality Reporting topic for more information.
     */
    criteria: Expression;
    /**
     * The human readable description of this population criteria.
     */
    description?: string | undefined;
    _description?: Element | undefined;
    /**
     * The id of a population element in this measure that provides the input for this population criteria. In most cases, the scoring structure of the measure implies specific relationships (e.g. the Numerator uses the Denominator as the source in a proportion scoring). In some cases, however, multiple possible choices exist and must be resolved explicitly. For example in a ratio measure with multiple initial populations, the denominator must specify which population should be used as the starting point.
     */
    inputPopulationId?: string | undefined;
    _inputPopulationId?: Element | undefined;
}
/**
 * Stratifiers are defined either as a single criteria, or as a set of component criteria.
 */
export interface MeasureGroupStratifierComponent extends BackboneElement {
    /**
     * Indicates a meaning for the stratifier component. This can be as simple as a unique identifier, or it can establish meaning in a broader context by drawing from a terminology, allowing stratifiers to be correlated across measures.
     */
    code?: CodeableConcept | undefined;
    /**
     * An expression that specifies the criteria for this component of the stratifier. This is typically the name of an expression defined within a referenced library, but it may also be a path to a stratifier element.
     */
    criteria: Expression;
    /**
     * The human readable description of this stratifier criteria component.
     */
    description?: string | undefined;
    _description?: Element | undefined;
}
/**
 * The stratifier criteria for the measure report, specified as either the name of a valid CQL expression defined within a referenced library or a valid FHIR Resource Path.
 */
export interface MeasureGroupStratifier extends BackboneElement {
    /**
     * Indicates a meaning for the stratifier. This can be as simple as a unique identifier, or it can establish meaning in a broader context by drawing from a terminology, allowing stratifiers to be correlated across measures.
     */
    code?: CodeableConcept | undefined;
    /**
     * Stratifiers are defined either as a single criteria, or as a set of component criteria.
     */
    component?: MeasureGroupStratifierComponent[] | undefined;
    /**
     * An expression that specifies the criteria for the stratifier. This is typically the name of an expression defined within a referenced library, but it may also be a path to a stratifier element.
     */
    criteria?: Expression | undefined;
    /**
     * The human readable description of this stratifier criteria.
     */
    description?: string | undefined;
    _description?: Element | undefined;
}
/**
 * A group of population criteria for the measure.
 */
export interface MeasureGroup extends BackboneElement {
    /**
     * When specified at the group level, defines the population basis for this specific group. If not specified, the basis for the group is determined by the root basis element
     */
    basis?: string | undefined;
    _basis?: Element | undefined;
    /**
     * Indicates a meaning for the group. This can be as simple as a unique identifier, or it can establish meaning in a broader context by drawing from a terminology, allowing groups to be correlated across measures.
     */
    code?: CodeableConcept | undefined;
    /**
     * The human readable description of this population group.
     */
    description?: string | undefined;
    _description?: Element | undefined;
    /**
     * When specified at the group level, this element defines the improvementNotation for this specific group. If not specified, improvementNotation for this group is determined by the root improvementNotation element
     */
    improvementNotation?: CodeableConcept | undefined;
    /**
     * A population criteria for the measure.
     */
    population?: MeasureGroupPopulation[] | undefined;
    /**
     * When specified at the group level, defines the scoring for this specific group. If not specified, scoring for this group is determined by the root scoring element
     */
    scoring?: CodeableConcept | undefined;
    /**
     * When specified at the group level, this defines the scoringUnit for this specific group. If not specified, the scoringUnit for this group is determined by the root scoringUnit element. As with the root element, when this element is specified, implementations are expected to report measure scores in the specified units. Note that this may involve unit conversion if the expected units are different than the units of the resulting score calculation. In this case, unit conversions SHALL be performed according to the unit conversion semantics specified by UCUM.
     */
    scoringUnit?: CodeableConcept | undefined;
    /**
     * The stratifier criteria for the measure report, specified as either the name of a valid CQL expression defined within a referenced library or a valid FHIR Resource Path.
     */
    stratifier?: MeasureGroupStratifier[] | undefined;
    /**
     * When specified at the group level, defines the measure type for this specific group. If not specified, the measureType of the group is determined by the root type element
     */
    type?: CodeableConcept[] | undefined;
}
/**
 * Note that supplemental data are reported as observations for each patient and included in the evaluatedResources bundle. See the MeasureReport resource or the Quality Reporting topic for more information.
 */
export interface MeasureSupplementalData extends BackboneElement {
    /**
     * Indicates a meaning for the supplemental data. This can be as simple as a unique identifier, or it can establish meaning in a broader context by drawing from a terminology, allowing supplemental data to be correlated across measures.
     */
    code?: CodeableConcept | undefined;
    /**
     * The criteria for the supplemental data. This is typically the name of a valid expression defined within a referenced library, but it may also be a path to a specific data element. The criteria defines the data to be returned for this element.
     */
    criteria: Expression;
    /**
     * The human readable description of this supplemental data.
     */
    description?: string | undefined;
    _description?: Element | undefined;
    /**
     * An indicator of the intended usage for the supplemental data element. Supplemental data indicates the data is additional information requested to augment the measure information. Risk adjustment factor indicates the data is additional information used to calculate risk adjustment factors when applying a risk model to the measure calculation.
     */
    usage?: CodeableConcept[] | undefined;
}
/**
 * The Measure resource provides the definition of a quality measure.
 */
export interface Measure extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'Measure';
    /**
     * The 'date' element may be more recent than the approval date because of minor changes or editorial corrections.
     */
    approvalDate?: string | undefined;
    _approvalDate?: Element | undefined;
    /**
     * An individiual or organization primarily involved in the creation and maintenance of the content.
     */
    author?: ContactDetail[] | undefined;
    /**
     * For a subject-based measure, the population basis is simply boolean; all the criteria are expressed as true/false conditions that determine membership of an individual case in the population. For non-subject-based measures, the population basis can be any resource type, and the criteria are queries that return the subject's contribution to the population as a list of that resource type. For example, for a procedure-based measure, the population criteria would return lists of procedures that should be included in each population.
     */
    basis?: string | undefined;
    _basis?: Element | undefined;
    /**
     * Provides a summary of relevant clinical guidelines or other clinical recommendations supporting the measure.
     */
    clinicalRecommendationStatement?: string | undefined;
    _clinicalRecommendationStatement?: Element | undefined;
    /**
     * If this is a composite measure, the scoring method used to combine the component measures to determine the composite score.
     */
    compositeScoring?: CodeableConcept | undefined;
    /**
     * May be a web site, an email address, a telephone number, etc.
     */
    contact?: ContactDetail[] | undefined;
    /**
     * A copyright statement relating to the measure and/or its contents. Copyright statements are generally legal restrictions on the use and publishing of the measure.
     */
    copyright?: string | undefined;
    _copyright?: Element | undefined;
    /**
     * Note that this is not the same as the resource last-modified-date, since the resource may be a secondary representation of the measure. Additional specific dates may be added as extensions or be found by consulting Provenances associated with past versions of the resource.
     */
    date?: string | undefined;
    _date?: Element | undefined;
    /**
     * This description can be used to capture details such as comments about misuse, instructions for clinical use and interpretation, literature references, examples from the paper world, etc. It is not a rendering of the measure as conveyed in the 'text' field of the resource itself. This item SHOULD be populated unless the information is available from context (e.g. the language of the measure is presumed to be the predominant language in the place the measure was created).
     */
    description?: string | undefined;
    _description?: Element | undefined;
    /**
     * Notices and disclaimers regarding the use of the measure or related to intellectual property (such as code systems) referenced by the measure.
     */
    disclaimer?: string | undefined;
    _disclaimer?: Element | undefined;
    /**
     * An individual or organization primarily responsible for internal coherence of the content.
     */
    editor?: ContactDetail[] | undefined;
    /**
     * The effective period for a measure  determines when the content is applicable for usage and is independent of publication and review dates. For example, a measure intended to be used for the year 2016 might be published in 2015.
     */
    effectivePeriod?: Period | undefined;
    /**
     * An individual or organization responsible for officially endorsing the content for use in some setting.
     */
    endorser?: ContactDetail[] | undefined;
    /**
     * Allows filtering of measures that are appropriate for use versus not.
     */
    experimental?: boolean | undefined;
    _experimental?: Element | undefined;
    /**
     * A group of population criteria for the measure.
     */
    group?: MeasureGroup[] | undefined;
    /**
     * Additional guidance for the measure including how it can be used in a clinical context, and the intent of the measure.
     */
    guidance?: string | undefined;
    _guidance?: Element | undefined;
    /**
     * Typically, this is used for identifiers that can go in an HL7 V3 II (instance identifier) data type, and can then identify this measure outside of FHIR, where it is not possible to use the logical URI.
     */
    identifier?: Identifier[] | undefined;
    /**
     * Information on whether an increase or decrease in score is the preferred result (e.g., a higher score indicates better quality OR a lower score indicates better quality OR quality is within a range).
     */
    improvementNotation?: CodeableConcept | undefined;
    /**
     * It may be possible for the measure to be used in jurisdictions other than those for which it was originally designed or intended.
     */
    jurisdiction?: CodeableConcept[] | undefined;
    /**
     * If specified, this date follows the original approval date.
     */
    lastReviewDate?: string | undefined;
    _lastReviewDate?: Element | undefined;
    /**
     * A reference to a Library resource containing the formal logic used by the measure.
     */
    library?: string[] | undefined;
    _library?: Element[] | undefined;
    /**
     * The name is not expected to be globally unique. The name should be a simple alphanumeric type name to ensure that it is machine-processing friendly.
     */
    name?: string | undefined;
    _name?: Element | undefined;
    /**
     * Usually an organization but may be an individual. The publisher (or steward) of the measure is the organization or individual primarily responsible for the maintenance and upkeep of the measure. This is not necessarily the same individual or organization that developed and initially authored the content. The publisher is the primary point of contact for questions or issues with the measure. This item SHOULD be populated unless the information is available from context.
     */
    publisher?: string | undefined;
    _publisher?: Element | undefined;
    /**
     * This element does not describe the usage of the measure. Instead, it provides traceability of ''why'' the resource is either needed or ''why'' it is defined as it is.  This may be used to point to source materials or specifications that drove the structure of this measure.
     */
    purpose?: string | undefined;
    _purpose?: Element | undefined;
    /**
     * The measure rate for an organization or clinician is based upon the entities’ aggregate data and summarizes the performance of the entity over a given time period (e.g., monthly, quarterly, yearly). The aggregated data are derived from the results of a specific measure algorithm and, if appropriate, the application of specific risk adjustment models.  Can also be used to describe how to risk adjust the data based on supplemental data elements described in the eMeasure (e.g., pneumonia hospital measures antibiotic selection in the ICU versus non-ICU and then the roll-up of the two). This could be applied to aggregated cohort measure definitions (e.g., CDC's aggregate reporting for TB at the state level).
     */
    rateAggregation?: string | undefined;
    _rateAggregation?: Element | undefined;
    /**
     * Provides a succinct statement of the need for the measure. Usually includes statements pertaining to importance criterion: impact, gap in care, and evidence.
     */
    rationale?: string | undefined;
    _rationale?: Element | undefined;
    /**
     * Each related artifact is either an attachment, or a reference to another resource, but not both.
     */
    relatedArtifact?: RelatedArtifact[] | undefined;
    /**
     * An individual or organization primarily responsible for review of some aspect of the content.
     */
    reviewer?: ContactDetail[] | undefined;
    /**
     * Describes the method of adjusting for clinical severity and conditions present at the start of care that can influence patient outcomes for making valid comparisons of outcome measures across providers. Indicates whether a measure is subject to the statistical process for reducing, removing, or clarifying the influences of confounding factors to allow for more useful comparisons.
     */
    riskAdjustment?: string | undefined;
    _riskAdjustment?: Element | undefined;
    /**
     * Indicates how the calculation is performed for the measure, including proportion, ratio, continuous-variable, and cohort. The value set is extensible, allowing additional measure scoring types to be represented.
     */
    scoring?: CodeableConcept | undefined;
    /**
     * When this element is specified, implementations are expected to report measure scores in the specified units. Note that this may involve unit conversion if the expected units are different than the units of the resulting score calculation. In this case, unit conversions SHALL be performed according to the unit conversion semantics specified by UCUM.
     */
    scoringUnit?: CodeableConcept | undefined;
    /**
     * Allows filtering of measures that are appropriate for use versus not.
     */
    status: 'draft' | 'active' | 'retired' | 'unknown';
    _status?: Element | undefined;
    /**
     * The subject of the measure is critical in interpreting the criteria definitions, as the logic in the measures is evaluated with respect to a particular subject. This corresponds roughly to the notion of a Compartment in that it limits what content is available based on its relationship to the subject. In CQL, this corresponds to the context declaration.
     */
    subjectCodeableConcept?: CodeableConcept | undefined;
    /**
     * The subject of the measure is critical in interpreting the criteria definitions, as the logic in the measures is evaluated with respect to a particular subject. This corresponds roughly to the notion of a Compartment in that it limits what content is available based on its relationship to the subject. In CQL, this corresponds to the context declaration.
     */
    subjectReference?: Reference | undefined;
    /**
     * An explanatory or alternate title for the measure giving additional information about its content.
     */
    subtitle?: string | undefined;
    _subtitle?: Element | undefined;
    /**
     * Note that supplemental data are reported as observations for each patient and included in the evaluatedResources bundle. See the MeasureReport resource or the Quality Reporting topic for more information.
     */
    supplementalData?: MeasureSupplementalData[] | undefined;
    /**
     * Provides a description of an individual term used within the measure.
     */
    term?: MeasureTerm[] | undefined;
    /**
     * This name does not need to be machine-processing friendly and may contain punctuation, white-space, etc.
     */
    title?: string | undefined;
    _title?: Element | undefined;
    /**
     * Descriptive topics related to the content of the measure. Topics provide a high-level categorization grouping types of measures that can be useful for filtering and searching.
     */
    topic?: CodeableConcept[] | undefined;
    /**
     * Indicates whether the measure is used to examine a process, an outcome over time, a patient-reported outcome, or a structure measure such as utilization.
     */
    type?: CodeableConcept[] | undefined;
    /**
     * Can be a urn:uuid: or a urn:oid: but real http: addresses are preferred.  Multiple instances may share the same URL if they have a distinct version.
     * The determination of when to create a new version of a resource (same url, new version) vs. defining a new artifact is up to the author.  Considerations for making this decision are found in [Technical and Business Versions](resource.html#versions).
     * In some cases, the resource can no longer be found at the stated url, but the url itself cannot change. Implementations can use the [meta.source](resource.html#meta) element to indicate where the current master source of the resource can be found.
     */
    url?: string | undefined;
    _url?: Element | undefined;
    /**
     * A detailed description, from a clinical perspective, of how the measure is used.
     */
    usage?: string | undefined;
    _usage?: Element | undefined;
    /**
     * When multiple useContexts are specified, there is no expectation that all or any of the contexts apply.
     */
    useContext?: UsageContext[] | undefined;
    /**
     * There may be different measure instances that have the same identifier but different versions.  The version can be appended to the url in a reference to allow a reference to a particular business version of the measure with the format [url]|[version]. The version SHOULD NOT contain a '#' - see [Business Version](resource.html#bv-format).
     */
    version?: string | undefined;
    _version?: Element | undefined;
}
/**
 * The populations that make up the population group, one for each type of population appropriate for the measure.
 */
export interface MeasureReportGroupPopulation extends BackboneElement {
    /**
     * The type of the population.
     */
    code?: CodeableConcept | undefined;
    /**
     * The number of members of the population.
     */
    count?: number | undefined;
    /**
     * This element refers to a List of subject level MeasureReport resources, one for each subject in this population.
     */
    subjectResults?: Reference | undefined;
}
/**
 * A stratifier component value.
 */
export interface MeasureReportGroupStratifierStratumComponent extends BackboneElement {
    /**
     * The code for the stratum component value.
     */
    code: CodeableConcept;
    /**
     * The stratum component value.
     */
    valueCodeableConcept?: CodeableConcept | undefined;
    /**
     * The stratum component value.
     */
    valueBoolean?: boolean | undefined;
    _valueBoolean?: Element | undefined;
    /**
     * The stratum component value.
     */
    valueQuantity?: Quantity | undefined;
    /**
     * The stratum component value.
     */
    valueRange?: Range | undefined;
    /**
     * The stratum component value.
     */
    valueReference?: Reference | undefined;
}
/**
 * The populations that make up the stratum, one for each type of population appropriate to the measure.
 */
export interface MeasureReportGroupStratifierStratumPopulation extends BackboneElement {
    /**
     * The type of the population.
     */
    code?: CodeableConcept | undefined;
    /**
     * The number of members of the population in this stratum.
     */
    count?: number | undefined;
    /**
     * This element refers to a List of subject level MeasureReport resources, one for each subject in this population in this stratum.
     */
    subjectResults?: Reference | undefined;
}
/**
 * This element contains the results for a single stratum within the stratifier. For example, when stratifying on administrative gender, there will be four strata, one for each possible gender value.
 */
export interface MeasureReportGroupStratifierStratum extends BackboneElement {
    /**
     * A stratifier component value.
     */
    component?: MeasureReportGroupStratifierStratumComponent[] | undefined;
    /**
     * The measure score for this stratum, calculated as appropriate for the measure type and scoring method, and based on only the members of this stratum.
     */
    measureScoreQuantity?: Quantity | undefined;
    /**
     * The measure score for this stratum, calculated as appropriate for the measure type and scoring method, and based on only the members of this stratum.
     */
    measureScoreDateTime?: string | undefined;
    _measureScoreDateTime?: Element | undefined;
    /**
     * The measure score for this stratum, calculated as appropriate for the measure type and scoring method, and based on only the members of this stratum.
     */
    measureScoreCodeableConcept?: CodeableConcept | undefined;
    /**
     * The measure score for this stratum, calculated as appropriate for the measure type and scoring method, and based on only the members of this stratum.
     */
    measureScorePeriod?: Period | undefined;
    /**
     * The measure score for this stratum, calculated as appropriate for the measure type and scoring method, and based on only the members of this stratum.
     */
    measureScoreRange?: Range | undefined;
    /**
     * The measure score for this stratum, calculated as appropriate for the measure type and scoring method, and based on only the members of this stratum.
     */
    measureScoreDuration?: Duration | undefined;
    /**
     * The populations that make up the stratum, one for each type of population appropriate to the measure.
     */
    population?: MeasureReportGroupStratifierStratumPopulation[] | undefined;
    /**
     * The value for this stratum, expressed as a CodeableConcept. When defining stratifiers on complex values, the value must be rendered such that the value for each stratum within the stratifier is unique.
     */
    valueCodeableConcept?: CodeableConcept | undefined;
    /**
     * The value for this stratum, expressed as a CodeableConcept. When defining stratifiers on complex values, the value must be rendered such that the value for each stratum within the stratifier is unique.
     */
    valueBoolean?: boolean | undefined;
    _valueBoolean?: Element | undefined;
    /**
     * The value for this stratum, expressed as a CodeableConcept. When defining stratifiers on complex values, the value must be rendered such that the value for each stratum within the stratifier is unique.
     */
    valueQuantity?: Quantity | undefined;
    /**
     * The value for this stratum, expressed as a CodeableConcept. When defining stratifiers on complex values, the value must be rendered such that the value for each stratum within the stratifier is unique.
     */
    valueRange?: Range | undefined;
    /**
     * The value for this stratum, expressed as a CodeableConcept. When defining stratifiers on complex values, the value must be rendered such that the value for each stratum within the stratifier is unique.
     */
    valueReference?: Reference | undefined;
}
/**
 * When a measure includes multiple stratifiers, there will be a stratifier group for each stratifier defined by the measure.
 */
export interface MeasureReportGroupStratifier extends BackboneElement {
    /**
     * The meaning of this stratifier, as defined in the measure definition.
     */
    code?: CodeableConcept | undefined;
    /**
     * This element contains the results for a single stratum within the stratifier. For example, when stratifying on administrative gender, there will be four strata, one for each possible gender value.
     */
    stratum?: MeasureReportGroupStratifierStratum[] | undefined;
}
/**
 * The results of the calculation, one for each population group in the measure.
 */
export interface MeasureReportGroup extends BackboneElement {
    /**
     * The meaning of the population group as defined in the measure definition.
     */
    code?: CodeableConcept | undefined;
    /**
     * The measure score for this population group, calculated as appropriate for the measure type and scoring method, and based on the contents of the populations defined in the group.
     */
    measureScoreQuantity?: Quantity | undefined;
    /**
     * The measure score for this population group, calculated as appropriate for the measure type and scoring method, and based on the contents of the populations defined in the group.
     */
    measureScoreDateTime?: string | undefined;
    _measureScoreDateTime?: Element | undefined;
    /**
     * The measure score for this population group, calculated as appropriate for the measure type and scoring method, and based on the contents of the populations defined in the group.
     */
    measureScoreCodeableConcept?: CodeableConcept | undefined;
    /**
     * The measure score for this population group, calculated as appropriate for the measure type and scoring method, and based on the contents of the populations defined in the group.
     */
    measureScorePeriod?: Period | undefined;
    /**
     * The measure score for this population group, calculated as appropriate for the measure type and scoring method, and based on the contents of the populations defined in the group.
     */
    measureScoreRange?: Range | undefined;
    /**
     * The measure score for this population group, calculated as appropriate for the measure type and scoring method, and based on the contents of the populations defined in the group.
     */
    measureScoreDuration?: Duration | undefined;
    /**
     * The populations that make up the population group, one for each type of population appropriate for the measure.
     */
    population?: MeasureReportGroupPopulation[] | undefined;
    /**
     * When a measure includes multiple stratifiers, there will be a stratifier group for each stratifier defined by the measure.
     */
    stratifier?: MeasureReportGroupStratifier[] | undefined;
}
/**
 * The MeasureReport resource contains the results of the calculation of a measure; and optionally a reference to the resources involved in that calculation.
 */
export interface MeasureReport extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'MeasureReport';
    /**
     * This element only applies to Data-collection reports used to communicate the data-of-interest for a measure
     */
    dataUpdateType?: ('incremental' | 'snapshot') | undefined;
    _dataUpdateType?: Element | undefined;
    /**
     * The date this measure report was generated.
     */
    date?: string | undefined;
    _date?: Element | undefined;
    /**
     * A reference to a Resource that was used in the calculation of this measure.
     */
    evaluatedResource?: Reference[] | undefined;
    /**
     * The results of the calculation, one for each population group in the measure.
     */
    group?: MeasureReportGroup[] | undefined;
    /**
     * Typically, this is used for identifiers that can go in an HL7 V3 II data type - e.g. to identify this {{title}} outside of FHIR, where the logical URL is not possible to use.
     */
    identifier?: Identifier[] | undefined;
    /**
     * This element is typically defined by the measure, but reproduced here to ensure the measure score can be interpreted. The element is labeled as a modifier because it changes the interpretation of the reported measure score.
     */
    improvementNotation?: CodeableConcept | undefined;
    /**
     * Whenever an operation returns a MeasureReport, the IN parameters from that operation can be returned to the caller of the operation using MeasureReport.inputParameters. Operations include but are not limited to $evaluate-measure, $care-gaps, $data-requirements, and $collect-data. The inputParameters can also be used in a MeasureReport used to define a test case (i.e., not as a result of an operation).
     */
    inputParameters?: Reference | undefined;
    /**
     * A reference to the location for which the data is being reported.
     */
    location?: Reference | undefined;
    /**
     * A reference to the Measure that was calculated to produce this report.
     */
    measure?: string | undefined;
    _measure?: Element | undefined;
    /**
     * The reporting period for which the report was calculated.
     */
    period: Period;
    /**
     * The individual or organization that is reporting the data.
     */
    reporter?: Reference | undefined;
    /**
     * A reference to the vendor who queried the data, calculated results and/or generated the report. The ‘reporting vendor’ is intended to represent the submitting entity when it is not the same as the reporting entity. This extension is used when the Receiver is interested in getting vendor information in the report.
     */
    reportingVendor?: Reference | undefined;
    /**
     * Indicates how the calculation is performed for the measure, including proportion, ratio, continuous-variable, and cohort. The value set is extensible, allowing additional measure scoring types to be represented. It is expected to be the same as the scoring element on the referenced Measure.
     */
    scoring?: CodeableConcept | undefined;
    /**
     * This element is labeled as a modifier because the status contains codes that mark the resource as not currently valid.
     */
    status: 'complete' | 'pending' | 'error';
    _status?: Element | undefined;
    /**
     * Optional subject identifying the individual or individuals the report is for.
     */
    subject?: Reference | undefined;
    /**
     * Data-exchange reports are used only to communicate data-of-interest for a measure. They do not necessarily include all the data for a particular subject or population, but they may.
     */
    type: 'individual' | 'subject-list' | 'summary' | 'data-exchange';
    _type?: Element | undefined;
}
/**
 * The ingredients need not be a complete list.  If an ingredient is not specified, this does not indicate whether an ingredient is present or absent.  If an ingredient is specified it does not mean that all ingredients are specified.  It is possible to specify both inactive and active ingredients.
 */
export interface MedicationIngredient extends BackboneElement {
    /**
     * Indication of whether this ingredient affects the therapeutic action of the drug.
     */
    isActive?: boolean | undefined;
    _isActive?: Element | undefined;
    /**
     * The ingredient (substance or medication) that the ingredient.strength relates to.  This is represented as a concept from a code system or described in another resource (Substance or Medication).
     */
    item: CodeableReference;
    /**
     * Specifies how many (or how much) of the items there are in this Medication.  For example, 250 mg per tablet.  This is expressed as a ratio where the numerator is 250mg and the denominator is 1 tablet but can also be expressed a quantity when the denominator is assumed to be 1 tablet.
     */
    strengthRatio?: Ratio | undefined;
    /**
     * Specifies how many (or how much) of the items there are in this Medication.  For example, 250 mg per tablet.  This is expressed as a ratio where the numerator is 250mg and the denominator is 1 tablet but can also be expressed a quantity when the denominator is assumed to be 1 tablet.
     */
    strengthCodeableConcept?: CodeableConcept | undefined;
    /**
     * Specifies how many (or how much) of the items there are in this Medication.  For example, 250 mg per tablet.  This is expressed as a ratio where the numerator is 250mg and the denominator is 1 tablet but can also be expressed a quantity when the denominator is assumed to be 1 tablet.
     */
    strengthQuantity?: Quantity | undefined;
}
/**
 * Information that only applies to packages (not products).
 */
export interface MedicationBatch extends BackboneElement {
    /**
     * When this specific batch of product will expire.
     */
    expirationDate?: string | undefined;
    _expirationDate?: Element | undefined;
    /**
     * The assigned lot number of a batch of the specified product.
     */
    lotNumber?: string | undefined;
    _lotNumber?: Element | undefined;
}
/**
 * This resource is primarily used for the identification and definition of a medication, including ingredients, for the purposes of prescribing, dispensing, and administering a medication as well as for making statements about medication use.
 */
export interface Medication extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'Medication';
    /**
     * Information that only applies to packages (not products).
     */
    batch?: MedicationBatch | undefined;
    /**
     * Depending on the context of use, the code that was actually selected by the user (prescriber, dispenser, etc.) will have the coding.userSelected set to true.  As described in the coding datatype: "A coding may be marked as a "userSelected" if a user selected the particular coded value in a user interface (e.g. the user selects an item in a pick-list). If a user selected coding exists, it is the preferred choice for performing translations etc. Other codes can only be literal translations to alternative code systems, or codes at a lower level of granularity (e.g. a generic code for a vendor-specific primary one).
     */
    code?: CodeableConcept | undefined;
    /**
     * When Medication is referenced from MedicationRequest, this is the ordered form.  When Medication is referenced within MedicationDispense, this is the dispensed form.  When Medication is referenced within MedicationAdministration, this is administered form.
     */
    doseForm?: CodeableConcept | undefined;
    /**
     * The serial number could be included as an identifier.
     */
    identifier?: Identifier[] | undefined;
    /**
     * The ingredients need not be a complete list.  If an ingredient is not specified, this does not indicate whether an ingredient is present or absent.  If an ingredient is specified it does not mean that all ingredients are specified.  It is possible to specify both inactive and active ingredients.
     */
    ingredient?: MedicationIngredient[] | undefined;
    /**
     * The company or other legal entity that has authorization, from the appropriate drug regulatory authority,  to market a medicine in one or more jurisdictions.  Typically abbreviated MAH.Note:  The MAH may manufacture the product and may also contract the manufacturing of the product to one or more companies (organizations).
     */
    marketingAuthorizationHolder?: Reference | undefined;
    /**
     * This status is intended to identify if the medication in a local system is in active use within a drug database or inventory.  For example, a pharmacy system may create a new drug file record for a compounded product "ABC Hospital Special Cream" with an active status.  At some point in the future, it may be determined that the drug record was created with an error and the status is changed to "entered in error".   This status is not intended to specify if a medication is part of a particular formulary.  It is possible that the drug record may be referenced by multiple formularies or catalogues and each of those entries would have a separate status.
     */
    status?: ('active' | 'inactive' | 'entered-in-error') | undefined;
    _status?: Element | undefined;
    /**
     * When the specified product code does not infer a package size, this is the specific amount of drug in the product.  For example, when specifying a product that has the same strength (For example, Insulin glargine 100 unit per mL solution for injection), this attribute provides additional clarification of the package amount (For example, 3 mL, 10mL, etc.).
     */
    totalVolume?: Ratio | undefined;
}
/**
 * Indicates who or what performed the medication administration and how they were involved. For devices, this is the device that is actually performing the administration of the medication.  An IV Pump would be an example of a device that is performing the administration.  Both the IV Pump and the practitioner that set the rate or bolus on the pump can be listed as performers.
 */
export interface MedicationAdministrationPerformer extends BackboneElement {
    /**
     * Indicates who or what performed the medication administration.
     */
    actor: Reference;
    /**
     * Distinguishes the type of involvement of the performer in the medication administration.
     */
    function?: CodeableConcept | undefined;
}
/**
 * Describes the medication dosage information details e.g. dose, rate, site, route, etc.
 */
export interface MedicationAdministrationDosage extends BackboneElement {
    /**
     * If the administration is not instantaneous (rate is present), this can be specified to convey the total amount administered over period of time of a single administration.
     */
    dose?: Quantity | undefined;
    /**
     * One of the reasons this attribute is not used often, is that the method is often pre-coordinated with the route and/or form of administration.  This means the codes used in route or form may pre-coordinate the method in the route code or the form code.  The implementation decision about what coding system to use for route or form code will determine how frequently the method code will be populated e.g. if route or form code pre-coordinate method code, then this attribute will not be populated often; if there is no pre-coordination then method code may  be used frequently.
     */
    method?: CodeableConcept | undefined;
    /**
     * If the rate changes over time, and you want to capture this in MedicationAdministration, then each change should be captured as a distinct MedicationAdministration, with a specific MedicationAdministration.dosage.rate, and the date time when the rate change occurred. Typically, the MedicationAdministration.dosage.rate element is not used to convey an average rate.
     */
    rateRatio?: Ratio | undefined;
    /**
     * If the rate changes over time, and you want to capture this in MedicationAdministration, then each change should be captured as a distinct MedicationAdministration, with a specific MedicationAdministration.dosage.rate, and the date time when the rate change occurred. Typically, the MedicationAdministration.dosage.rate element is not used to convey an average rate.
     */
    rateQuantity?: Quantity | undefined;
    /**
     * A code specifying the route or physiological path of administration of a therapeutic agent into or onto the patient.  For example, topical, intravenous, etc.
     */
    route?: CodeableConcept | undefined;
    /**
     * If the use case requires attributes from the BodySite resource (e.g. to identify and track separately) then use the standard extension [bodySite](extension-bodysite.html).  May be a summary code, or a reference to a very precise definition of the location, or both.
     */
    site?: CodeableConcept | undefined;
    /**
     * Free text dosage can be used for cases where the dosage administered is too complex to code. When coded dosage is present, the free text dosage may still be present for display to humans.
     * The dosage instructions should reflect the dosage of the medication that was administered.
     */
    text?: string | undefined;
    _text?: Element | undefined;
}
/**
 * Describes the event of a patient consuming or otherwise being administered a medication.  This may be as simple as swallowing a tablet or it may be a long running infusion.  Related resources tie this event to the authorizing prescription, and the specific encounter between patient and health care practitioner.
 */
export interface MedicationAdministration extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'MedicationAdministration';
    /**
     * A plan that is fulfilled in whole or in part by this MedicationAdministration.
     */
    basedOn?: Reference[] | undefined;
    /**
     * The type of medication administration (for example, drug classification like ATC, where meds would be administered, legal category of the medication).
     */
    category?: CodeableConcept[] | undefined;
    /**
     * The device used in administering the medication to the patient.  For example, a particular infusion pump.
     */
    device?: Reference[] | undefined;
    /**
     * Describes the medication dosage information details e.g. dose, rate, site, route, etc.
     */
    dosage?: MedicationAdministrationDosage | undefined;
    /**
     * The visit, admission, or other contact between patient and health care provider during which the medication administration was performed.
     */
    encounter?: Reference | undefined;
    /**
     * This might not include provenances for all versions of the request – only those deemed “relevant” or important. This SHALL NOT include the Provenance associated with this current version of the resource. (If that provenance is deemed to be a “relevant” change, it will need to be added as part of a later update. Until then, it can be queried directly as the Provenance that points to this version using _revinclude All Provenances should have some historical version of this Request as their subject.
     */
    eventHistory?: Reference[] | undefined;
    /**
     * This is a business identifier, not a resource identifier.
     */
    identifier?: Identifier[] | undefined;
    /**
     * An indication that the full dose was not administered.
     */
    isSubPotent?: boolean | undefined;
    _isSubPotent?: Element | undefined;
    /**
     * If only a code is specified, then it needs to be a code for a specific product. If more information is required, then the use of the medication resource is recommended.  For example, if you require form or lot number, then you must reference the Medication resource.
     */
    medication: CodeableReference;
    /**
     * Extra information about the medication administration that is not conveyed by the other attributes.
     */
    note?: Annotation[] | undefined;
    /**
     * A specific date/time or interval of time during which the administration took place (or did not take place). For many administrations, such as swallowing a tablet the use of dateTime is more appropriate.
     */
    occurenceDateTime?: string | undefined;
    _occurenceDateTime?: Element | undefined;
    /**
     * A specific date/time or interval of time during which the administration took place (or did not take place). For many administrations, such as swallowing a tablet the use of dateTime is more appropriate.
     */
    occurencePeriod?: Period | undefined;
    /**
     * A larger event of which this particular event is a component or step.
     */
    partOf?: Reference[] | undefined;
    /**
     * Indicates who or what performed the medication administration and how they were involved. For devices, this is the device that is actually performing the administration of the medication.  An IV Pump would be an example of a device that is performing the administration.  Both the IV Pump and the practitioner that set the rate or bolus on the pump can be listed as performers.
     */
    performer?: MedicationAdministrationPerformer[] | undefined;
    /**
     * A code, Condition or observation that supports why the medication was administered.
     */
    reason?: CodeableReference[] | undefined;
    /**
     * The date the occurrence of the  MedicationAdministration was first captured in the record - potentially significantly after the occurrence of the event.
     */
    recorded?: string | undefined;
    _recorded?: Element | undefined;
    /**
     * This is a reference to the MedicationRequest  where the intent is either order or instance-order.  It should not reference MedicationRequests where the intent is any other value.
     */
    request?: Reference | undefined;
    /**
     * This element is labeled as a modifier because the status contains codes that mark the resource as not currently valid.
     */
    status: 'in-progress' | 'not-done' | 'on-hold' | 'completed' | 'entered-in-error' | 'stopped' | 'unknown';
    _status?: Element | undefined;
    /**
     * A code indicating why the administration was not performed.
     */
    statusReason?: CodeableConcept[] | undefined;
    /**
     * The person or animal or group receiving the medication.
     */
    subject: Reference;
    /**
     * The reason or reasons why the full dose was not administered.
     */
    subPotentReason?: CodeableConcept[] | undefined;
    /**
     * Additional information (for example, patient height and weight) that supports the administration of the medication.  This attribute can be used to provide documentation of specific characteristics of the patient present at the time of administration.  For example, if the dose says "give "x" if the heartrate exceeds "y"", then the heart rate can be included using this attribute.
     */
    supportingInformation?: Reference[] | undefined;
}
/**
 * Indicates who or what performed the event.
 */
export interface MedicationDispensePerformer extends BackboneElement {
    /**
     * The device, practitioner, etc. who performed the action.  It should be assumed that the actor is the dispenser of the medication.
     */
    actor: Reference;
    /**
     * Distinguishes the type of performer in the dispense.  For example, date enterer, packager, final checker.
     */
    function?: CodeableConcept | undefined;
}
/**
 * Indicates whether or not substitution was made as part of the dispense.  In some cases, substitution will be expected but does not happen, in other cases substitution is not expected but does happen.  This block explains what substitution did or did not happen and why.  If nothing is specified, substitution was not done.
 */
export interface MedicationDispenseSubstitution extends BackboneElement {
    /**
     * Indicates the reason for the substitution (or lack of substitution) from what was prescribed.
     */
    reason?: CodeableConcept[] | undefined;
    /**
     * The person or organization that has primary responsibility for the substitution.
     */
    responsibleParty?: Reference | undefined;
    /**
     * A code signifying whether a different drug was dispensed from what was prescribed.
     */
    type?: CodeableConcept | undefined;
    /**
     * True if the dispenser dispensed a different drug or product from what was prescribed.
     */
    wasSubstituted: boolean;
    _wasSubstituted?: Element | undefined;
}
/**
 * Indicates that a medication product is to be or has been dispensed for a named person/patient.  This includes a description of the medication product (supply) provided and the instructions for administering the medication.  The medication dispense is the result of a pharmacy system responding to a medication order.
 */
export interface MedicationDispense extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'MedicationDispense';
    /**
     * Maps to basedOn in Event logical model.
     */
    authorizingPrescription?: Reference[] | undefined;
    /**
     * A plan that is fulfilled in whole or in part by this MedicationDispense.
     */
    basedOn?: Reference[] | undefined;
    /**
     * The category can be used to include where the medication is expected to be consumed or other types of dispenses.  Invariants can be used to bind to different value sets when profiling to bind.
     */
    category?: CodeableConcept[] | undefined;
    /**
     * The amount of medication expressed as a timing amount.
     */
    daysSupply?: Quantity | undefined;
    /**
     * Identification of the facility/location where the medication was shipped to, as part of the dispense event.
     */
    destination?: Reference | undefined;
    /**
     * When the dose or rate is intended to change over the entire administration period (e.g. Tapering dose prescriptions), multiple instances of dosage instructions will need to be supplied to convey the different doses/rates.
     * The pharmacist reviews the medication order prior to dispense and updates the dosageInstruction based on the actual product being dispensed.
     */
    dosageInstruction?: Dosage[] | undefined;
    /**
     * The encounter that establishes the context for this event.
     */
    encounter?: Reference | undefined;
    /**
     * This might not include provenances for all versions of the request – only those deemed “relevant” or important. This SHALL NOT include the Provenance associated with this current version of the resource. (If that provenance is deemed to be a “relevant” change, it will need to be added as part of a later update. Until then, it can be queried directly as the Provenance that points to this version using _revinclude All Provenances should have some historical version of this Request as their subject.).
     */
    eventHistory?: Reference[] | undefined;
    /**
     * This is a business identifier, not a resource identifier.
     */
    identifier?: Identifier[] | undefined;
    /**
     * The principal physical location where the dispense was performed.
     */
    location?: Reference | undefined;
    /**
     * If only a code is specified, then it needs to be a code for a specific product. If more information is required, then the use of the medication resource is recommended.  For example, if you require form or lot number, then you must reference the Medication resource.
     */
    medication: CodeableReference;
    /**
     * Extra information about the dispense that could not be conveyed in the other attributes.
     */
    note?: Annotation[] | undefined;
    /**
     * Indicates the reason why a dispense was not performed.
     */
    notPerformedReason?: CodeableReference | undefined;
    /**
     * While both a procedure and a medication administration may have triggered the dispense, but it is not expected that multiple procedures and/or multiple medication administrations would be triggers.
     */
    partOf?: Reference[] | undefined;
    /**
     * Indicates who or what performed the event.
     */
    performer?: MedicationDispensePerformer[] | undefined;
    /**
     * The amount of medication that has been dispensed. Includes unit of measure.
     */
    quantity?: Quantity | undefined;
    /**
     * Identifies the person who picked up the medication or the location of where the medication was delivered.  This will usually be a patient or their caregiver, but some cases exist where it can be a healthcare professional or a location.
     */
    receiver?: Reference[] | undefined;
    /**
     * The date (and maybe time) when the dispense activity started if whenPrepared or whenHandedOver is not populated.
     */
    recorded?: string | undefined;
    _recorded?: Element | undefined;
    /**
     * The content of the renderedDosageInstructions must not be different than the dose represented in the dosageInstruction content.
     */
    renderedDosageInstruction?: string | undefined;
    _renderedDosageInstruction?: Element | undefined;
    /**
     * This element is labeled as a modifier because the status contains codes that mark the resource as not currently valid.
     */
    status:
        | 'preparation'
        | 'in-progress'
        | 'cancelled'
        | 'on-hold'
        | 'completed'
        | 'entered-in-error'
        | 'stopped'
        | 'declined'
        | 'unknown';
    _status?: Element | undefined;
    /**
     * The date (and maybe time) when the status of the dispense record changed.
     */
    statusChanged?: string | undefined;
    _statusChanged?: Element | undefined;
    /**
     * SubstanceAdministration->subject->Patient.
     */
    subject: Reference;
    /**
     * Indicates whether or not substitution was made as part of the dispense.  In some cases, substitution will be expected but does not happen, in other cases substitution is not expected but does happen.  This block explains what substitution did or did not happen and why.  If nothing is specified, substitution was not done.
     */
    substitution?: MedicationDispenseSubstitution | undefined;
    /**
     * Additional information that supports the medication being dispensed.  For example, there may be requirements that a specific lab test has been completed prior to dispensing or the patient's weight at the time of dispensing is documented.
     */
    supportingInformation?: Reference[] | undefined;
    /**
     * Indicates the type of dispensing event that is performed. For example, Trial Fill, Completion of Trial, Partial Fill, Emergency Fill, Samples, etc.
     */
    type?: CodeableConcept | undefined;
    /**
     * The time the dispensed product was provided to the patient or their representative.
     */
    whenHandedOver?: string | undefined;
    _whenHandedOver?: Element | undefined;
    /**
     * The time when the dispensed product was packaged and reviewed.
     */
    whenPrepared?: string | undefined;
    _whenPrepared?: Element | undefined;
}
/**
 * Associated or related medications. For example, if the medication is a branded product (e.g. Crestor), this is the Therapeutic Moeity (e.g. Rosuvastatin) or if this is a generic medication (e.g. Rosuvastatin), this would link to a branded product (e.g. Crestor.
 */
export interface MedicationKnowledgeRelatedMedicationKnowledge extends BackboneElement {
    /**
     * Associated documentation about the associated medication knowledge.
     */
    reference: Reference[];
    /**
     * The category of the associated medication knowledge reference.
     */
    type: CodeableConcept;
}
/**
 * Associated documentation about the medication.
 */
export interface MedicationKnowledgeMonograph extends BackboneElement {
    /**
     * Associated documentation about the medication.
     */
    source?: Reference | undefined;
    /**
     * The category of documentation about the medication. (e.g. professional monograph, patient education monograph).
     */
    type?: CodeableConcept | undefined;
}
/**
 * The price of the medication.
 */
export interface MedicationKnowledgeCost extends BackboneElement {
    /**
     * The price or representation of the cost (for example, Band A, Band B or $, $$) of the medication.
     */
    costMoney?: Money | undefined;
    /**
     * The price or representation of the cost (for example, Band A, Band B or $, $$) of the medication.
     */
    costCodeableConcept?: CodeableConcept | undefined;
    /**
     * The date range for which the cost information of the medication is effective.
     */
    effectiveDate?: Period[] | undefined;
    /**
     * The source or owner that assigns the price to the medication.
     */
    source?: string | undefined;
    _source?: Element | undefined;
    /**
     * The category of the cost information.  For example, manufacturers' cost, patient cost, claim reimbursement cost, actual acquisition cost.
     */
    type: CodeableConcept;
}
/**
 * The program under which the medication is reviewed.
 */
export interface MedicationKnowledgeMonitoringProgram extends BackboneElement {
    /**
     * Name of the reviewing program.
     */
    name?: string | undefined;
    _name?: Element | undefined;
    /**
     * Type of program under which the medication is monitored.
     */
    type?: CodeableConcept | undefined;
}
/**
 * Dosage for the medication for the specific guidelines.
 */
export interface MedicationKnowledgeIndicationGuidelineDosingGuidelineDosage extends BackboneElement {
    /**
     * Dosage for the medication for the specific guidelines.
     */
    dosage: Dosage[];
    /**
     * The type or category of dosage for a given medication (for example, prophylaxis, maintenance, therapeutic, etc.).
     */
    type: CodeableConcept;
}
/**
 * Characteristics of the patient that are relevant to the administration guidelines (for example, height, weight, gender, etc.).
 */
export interface MedicationKnowledgeIndicationGuidelineDosingGuidelinePatientCharacteristic extends BackboneElement {
    /**
     * The categorization of the specific characteristic that is relevant to the administration guideline (e.g. height, weight, gender).
     */
    type: CodeableConcept;
    /**
     * The specific characteristic (e.g. height, weight, gender, etc.).
     */
    valueCodeableConcept?: CodeableConcept | undefined;
    /**
     * The specific characteristic (e.g. height, weight, gender, etc.).
     */
    valueQuantity?: Quantity | undefined;
    /**
     * The specific characteristic (e.g. height, weight, gender, etc.).
     */
    valueRange?: Range | undefined;
}
/**
 * The guidelines for the dosage of the medication for the indication.
 */
export interface MedicationKnowledgeIndicationGuidelineDosingGuideline extends BackboneElement {
    /**
     * The type of the treatment that the guideline applies to, for example, long term therapy, first line treatment, etc.
     */
    administrationTreatment?: CodeableConcept | undefined;
    /**
     * Dosage for the medication for the specific guidelines.
     */
    dosage?: MedicationKnowledgeIndicationGuidelineDosingGuidelineDosage[] | undefined;
    /**
     * Characteristics of the patient that are relevant to the administration guidelines (for example, height, weight, gender, etc.).
     */
    patientCharacteristic?: MedicationKnowledgeIndicationGuidelineDosingGuidelinePatientCharacteristic[] | undefined;
    /**
     * The overall intention of the treatment, for example, prophylactic, supporative, curative, etc.
     */
    treatmentIntent?: CodeableConcept | undefined;
}
/**
 * Guidelines or protocols that are applicable for the administration of the medication based on indication.
 */
export interface MedicationKnowledgeIndicationGuideline extends BackboneElement {
    /**
     * The guidelines for the dosage of the medication for the indication.
     */
    dosingGuideline?: MedicationKnowledgeIndicationGuidelineDosingGuideline[] | undefined;
    /**
     * Indication or reason for use of the medication that applies to the specific administration guideline.
     */
    indication?: CodeableReference[] | undefined;
}
/**
 * Categorization of the medication within a formulary or classification system.
 */
export interface MedicationKnowledgeMedicineClassification extends BackboneElement {
    /**
     * Specific category assigned to the medication (e.g. anti-infective, anti-hypertensive, antibiotic, etc.).
     */
    classification?: CodeableConcept[] | undefined;
    /**
     * Either a textual source of the classification or a reference to an online source.
     */
    sourceString?: string | undefined;
    _sourceString?: Element | undefined;
    /**
     * Either a textual source of the classification or a reference to an online source.
     */
    sourceUri?: string | undefined;
    _sourceUri?: Element | undefined;
    /**
     * The type of category for the medication (for example, therapeutic classification, therapeutic sub-classification).
     */
    type: CodeableConcept;
}
/**
 * Information that only applies to packages (not products).
 */
export interface MedicationKnowledgePackaging extends BackboneElement {
    /**
     * The cost of the packaged medication.
     */
    cost?: MedicationKnowledgeCost[] | undefined;
    /**
     * A reference to a PackagedProductDefinition that provides the details of the product that is in the packaging and is being priced.
     */
    packagedProduct?: Reference | undefined;
}
/**
 * Describes a setting/value on the environment for the adequate storage of the medication and other substances.  Environment settings may involve temperature, humidity, or exposure to light.
 */
export interface MedicationKnowledgeStorageGuidelineEnvironmentalSetting extends BackboneElement {
    /**
     * Identifies the category or type of setting (e.g., type of location, temperature, humidity).
     */
    type: CodeableConcept;
    /**
     * Value associated to the setting. E.g., 40° – 50°F for temperature.
     */
    valueQuantity?: Quantity | undefined;
    /**
     * Value associated to the setting. E.g., 40° – 50°F for temperature.
     */
    valueRange?: Range | undefined;
    /**
     * Value associated to the setting. E.g., 40° – 50°F for temperature.
     */
    valueCodeableConcept?: CodeableConcept | undefined;
}
/**
 * Information on how the medication should be stored, for example, refrigeration temperatures and length of stability at a given temperature.
 */
export interface MedicationKnowledgeStorageGuideline extends BackboneElement {
    /**
     * Describes a setting/value on the environment for the adequate storage of the medication and other substances.  Environment settings may involve temperature, humidity, or exposure to light.
     */
    environmentalSetting?: MedicationKnowledgeStorageGuidelineEnvironmentalSetting[] | undefined;
    /**
     * Additional notes about the storage.
     */
    note?: Annotation[] | undefined;
    /**
     * Reference to additional information about the storage guidelines.
     */
    reference?: string | undefined;
    _reference?: Element | undefined;
    /**
     * Duration that the medication remains stable if the environmentalSetting is respected.
     */
    stabilityDuration?: Duration | undefined;
}
/**
 * Specifies if changes are allowed when dispensing a medication from a regulatory perspective.
 */
export interface MedicationKnowledgeRegulatorySubstitution extends BackboneElement {
    /**
     * Specifies if regulation allows for changes in the medication when dispensing.
     */
    allowed: boolean;
    _allowed?: Element | undefined;
    /**
     * Specifies the type of substitution allowed.
     */
    type: CodeableConcept;
}
/**
 * The maximum number of units of the medication that can be dispensed in a period.
 */
export interface MedicationKnowledgeRegulatoryMaxDispense extends BackboneElement {
    /**
     * The period that applies to the maximum number of units.
     */
    period?: Duration | undefined;
    /**
     * The maximum number of units of the medication that can be dispensed.
     */
    quantity: Quantity;
}
/**
 * Regulatory information about a medication.
 */
export interface MedicationKnowledgeRegulatory extends BackboneElement {
    /**
     * The maximum number of units of the medication that can be dispensed in a period.
     */
    maxDispense?: MedicationKnowledgeRegulatoryMaxDispense | undefined;
    /**
     * The authority that is specifying the regulations.
     */
    regulatoryAuthority: Reference;
    /**
     * Specifies the schedule of a medication in jurisdiction.
     */
    schedule?: CodeableConcept[] | undefined;
    /**
     * Specifies if changes are allowed when dispensing a medication from a regulatory perspective.
     */
    substitution?: MedicationKnowledgeRegulatorySubstitution[] | undefined;
}
/**
 * Identifies a particular constituent of interest in the product.
 */
export interface MedicationKnowledgeDefinitionalIngredient extends BackboneElement {
    /**
     * A reference to the resource that provides information about the ingredient.
     */
    item: CodeableReference;
    /**
     * Specifies how many (or how much) of the items there are in this Medication.  For example, 250 mg per tablet.  This is expressed as a ratio where the numerator is 250mg and the denominator is 1 tablet but can also be expressed a quantity when the denominator is assumed to be 1 tablet.
     */
    strengthRatio?: Ratio | undefined;
    /**
     * Specifies how many (or how much) of the items there are in this Medication.  For example, 250 mg per tablet.  This is expressed as a ratio where the numerator is 250mg and the denominator is 1 tablet but can also be expressed a quantity when the denominator is assumed to be 1 tablet.
     */
    strengthCodeableConcept?: CodeableConcept | undefined;
    /**
     * Specifies how many (or how much) of the items there are in this Medication.  For example, 250 mg per tablet.  This is expressed as a ratio where the numerator is 250mg and the denominator is 1 tablet but can also be expressed a quantity when the denominator is assumed to be 1 tablet.
     */
    strengthQuantity?: Quantity | undefined;
    /**
     * Indication of whether this ingredient affects the therapeutic action of the drug.
     */
    type?: CodeableConcept | undefined;
}
/**
 * Specifies descriptive properties of the medicine, such as color, shape, imprints, etc.
 */
export interface MedicationKnowledgeDefinitionalDrugCharacteristic extends BackboneElement {
    /**
     * A code specifying which characteristic of the medicine is being described (for example, colour, shape, imprint).
     */
    type?: CodeableConcept | undefined;
    /**
     * The description should be provided as a CodeableConcept, SimpleQuantity or an image.  The description can be a string only when these others are not available.
     */
    valueCodeableConcept?: CodeableConcept | undefined;
    /**
     * The description should be provided as a CodeableConcept, SimpleQuantity or an image.  The description can be a string only when these others are not available.
     */
    valueString?: string | undefined;
    _valueString?: Element | undefined;
    /**
     * The description should be provided as a CodeableConcept, SimpleQuantity or an image.  The description can be a string only when these others are not available.
     */
    valueQuantity?: Quantity | undefined;
    /**
     * The description should be provided as a CodeableConcept, SimpleQuantity or an image.  The description can be a string only when these others are not available.
     */
    valueBase64Binary?: string | undefined;
    _valueBase64Binary?: Element | undefined;
    /**
     * The description should be provided as a CodeableConcept, SimpleQuantity or an image.  The description can be a string only when these others are not available.
     */
    valueAttachment?: Attachment | undefined;
}
/**
 * Along with the link to a Medicinal Product Definition resource, this information provides common definitional elements that are needed to understand the specific medication that is being described.
 */
export interface MedicationKnowledgeDefinitional extends BackboneElement {
    /**
     * Associated definitions for this medication.
     */
    definition?: Reference[] | undefined;
    /**
     * When Medication is referenced from MedicationRequest, this is the ordered form.  When Medication is referenced within MedicationDispense, this is the dispensed form.  When Medication is referenced within MedicationAdministration, this is administered form.
     */
    doseForm?: CodeableConcept | undefined;
    /**
     * Specifies descriptive properties of the medicine, such as color, shape, imprints, etc.
     */
    drugCharacteristic?: MedicationKnowledgeDefinitionalDrugCharacteristic[] | undefined;
    /**
     * Identifies a particular constituent of interest in the product.
     */
    ingredient?: MedicationKnowledgeDefinitionalIngredient[] | undefined;
    /**
     * The intended or approved route of administration.
     */
    intendedRoute?: CodeableConcept[] | undefined;
}
/**
 * Information about a medication that is used to support knowledge.
 */
export interface MedicationKnowledge extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'MedicationKnowledge';
    /**
     * Links to associated medications that could be prescribed, dispensed or administered.
     */
    associatedMedication?: Reference[] | undefined;
    /**
     * The creator or owner of the knowledge or information about the medication.
     */
    author?: Reference | undefined;
    /**
     * Potential clinical issue with or between medication(s) (for example, drug-drug interaction, drug-disease contraindication, drug-allergy interaction, etc.).
     */
    clinicalUseIssue?: Reference[] | undefined;
    /**
     * Depending on the context of use, the code that was actually selected by the user (prescriber, dispenser, etc.) will have the coding.userSelected set to true.  As described in the coding datatype: "A coding may be marked as a "userSelected" if a user selected the particular coded value in a user interface (e.g. the user selects an item in a pick-list). If a user selected coding exists, it is the preferred choice for performing translations etc. Other codes can only be literal translations to alternative code systems, or codes at a lower level of granularity (e.g. a generic code for a vendor-specific primary one).
     */
    code?: CodeableConcept | undefined;
    /**
     * The price of the medication.
     */
    cost?: MedicationKnowledgeCost[] | undefined;
    /**
     * Along with the link to a Medicinal Product Definition resource, this information provides common definitional elements that are needed to understand the specific medication that is being described.
     */
    definitional?: MedicationKnowledgeDefinitional | undefined;
    /**
     * This is a business identifier, not a resource identifier (see discussion).
     */
    identifier?: Identifier[] | undefined;
    /**
     * Guidelines or protocols that are applicable for the administration of the medication based on indication.
     */
    indicationGuideline?: MedicationKnowledgeIndicationGuideline[] | undefined;
    /**
     * The codes could have varying granularity from a country or group of countries down to a specific district.
     */
    intendedJurisdiction?: CodeableConcept[] | undefined;
    /**
     * Categorization of the medication within a formulary or classification system.
     */
    medicineClassification?: MedicationKnowledgeMedicineClassification[] | undefined;
    /**
     * The program under which the medication is reviewed.
     */
    monitoringProgram?: MedicationKnowledgeMonitoringProgram[] | undefined;
    /**
     * Associated documentation about the medication.
     */
    monograph?: MedicationKnowledgeMonograph[] | undefined;
    /**
     * All of the names for a medication, for example, the name(s) given to a medication in different countries.  For example, acetaminophen and paracetamol or salbutamol and albuterol.
     */
    name?: string[] | undefined;
    _name?: Element[] | undefined;
    /**
     * Information that only applies to packages (not products).
     */
    packaging?: MedicationKnowledgePackaging[] | undefined;
    /**
     * The instructions for preparing the medication.
     */
    preparationInstruction?: string | undefined;
    _preparationInstruction?: Element | undefined;
    /**
     * Category of the medication or product (e.g. branded product, therapeutic moeity, generic product, innovator product, etc.).
     */
    productType?: CodeableConcept[] | undefined;
    /**
     * Regulatory information about a medication.
     */
    regulatory?: MedicationKnowledgeRegulatory[] | undefined;
    /**
     * Associated or related medications. For example, if the medication is a branded product (e.g. Crestor), this is the Therapeutic Moeity (e.g. Rosuvastatin) or if this is a generic medication (e.g. Rosuvastatin), this would link to a branded product (e.g. Crestor.
     */
    relatedMedicationKnowledge?: MedicationKnowledgeRelatedMedicationKnowledge[] | undefined;
    /**
     * This status is intended to identify if the medication in a local system is in active use within a drug database or inventory.  For example, a pharmacy system may create a new drug file record for a compounded product "ABC Hospital Special Cream" with an active status.  At some point in the future, it may be determined that the drug record was created with an error and the status is changed to "entered in error".   This status is not intended to specify if a medication is part of a particular formulary.  It is possible that the drug record may be referenced by multiple formularies or catalogues and each of those entries would have a separate status.
     */
    status?: ('active' | 'entered-in-error' | 'inactive') | undefined;
    _status?: Element | undefined;
    /**
     * Information on how the medication should be stored, for example, refrigeration temperatures and length of stability at a given temperature.
     */
    storageGuideline?: MedicationKnowledgeStorageGuideline[] | undefined;
}
/**
 * Indicates how the medication is to be used by the patient.
 */
export interface MedicationRequestDose extends BackboneElement {
    /**
     * There are examples where a medication request may include the option of an oral dose or an Intravenous or Intramuscular dose.  For example, "Ondansetron 8mg orally or IV twice a day as needed for nausea" or "Compazine® (prochlorperazine) 5-10mg PO or 25mg PR bid prn nausea or vomiting".  In these cases, two medication requests would be created that could be grouped together.  The decision on which dose and route of administration to use is based on the patient's condition at the time the dose is needed. In general, each prescribed drug will be a separate Medication Request.
     * When drug orders are grouped together at the time of order entry,  but each of the drugs can be manipulated independently e.g. changing the status of one order to "completed" or "cancelled", changing another order status to "on-hold", the method to "group" all of the medication requests together is to use  MedicationRequest.groupIdentifier element.  All of the orders grouped together in this manner will have the same groupIdentifier, and separately, each order in the group may have a unique identifier.
     * There are cases that require grouping of Medication orders together when it is necessary to specify optionality e.g. order two drugs at one time, but stating either of these drugs may be used to treat the patient.  The use of a RequestOrchestration should be used as a parent for the Medication orders that require this type of grouping. An example when it may be necessary to group medication orders together is when you specify timing relationships e.g. order drug "xyz" with dose 123, then taper the same drug to a different dose after some interval of time
     * precedence:
     * e.g. give drug "abc" followed by drug "def"
     * e.g. give drug 30 minutes before some procedure was performed
     * more generically this supports - hierarchical groups of actions, where each specific action references the action to be performed (in terms of a Request resource), and each group describes additional behavior, relationships, and applicable conditions between the actions in the overall group.
     * Note that one should NOT use the List or Composition resource to accomplish the above requirements.  You may use List or Composition for other business requirements, but not to address the specific requirements of grouping medication orders.
     */
    dosageInstruction?: Dosage[] | undefined;
    /**
     * The period over which the medication is to be taken.  Where there are multiple dosageInstruction lines (for example, tapering doses), this is the earliest date and the latest end date of the dosageInstructions.
     */
    effectiveDosePeriod?: Period | undefined;
    /**
     * The full representation of the dose of the medication included in all dosage instructions.  To be used when multiple dosage instructions are included to represent complex dosing such as increasing or tapering doses.
     */
    renderedDosageInstruction?: string | undefined;
    _renderedDosageInstruction?: Element | undefined;
}
/**
 * If populating this element, either the quantity or the duration must be included.
 */
export interface MedicationRequestDispenseRequestInitialFill extends BackboneElement {
    /**
     * The length of time that the first dispense is expected to last.
     */
    duration?: Duration | undefined;
    /**
     * The amount or quantity to provide as part of the first dispense.
     */
    quantity?: Quantity | undefined;
}
/**
 * Indicates the specific details for the dispense or medication supply part of a medication request (also known as a Medication Prescription or Medication Order).  Note that this information is not always sent with the order.  There may be in some settings (e.g. hospitals) institutional or system support for completing the dispense details in the pharmacy department.
 */
export interface MedicationRequestDispenseRequest extends BackboneElement {
    /**
     * The minimum period of time that must occur between dispenses of the medication.
     */
    dispenseInterval?: Duration | undefined;
    /**
     * Indicates the intended performing Organization that will dispense the medication as specified by the prescriber.
     */
    dispenser?: Reference | undefined;
    /**
     * Provides additional information to the dispenser, for example, counselling to be provided to the patient.
     */
    dispenserInstruction?: Annotation[] | undefined;
    /**
     * Provides information about the type of adherence packaging to be supplied for the medication dispense.
     */
    doseAdministrationAid?: CodeableConcept | undefined;
    /**
     * In some situations, this attribute may be used instead of quantity to identify the amount supplied by how long it is expected to last, rather than the physical quantity issued, e.g. 90 days supply of medication (based on an ordered dosage). When possible, it is always better to specify quantity, as this tends to be more precise. expectedSupplyDuration will always be an estimate that can be influenced by external factors.
     */
    expectedSupplyDuration?: Duration | undefined;
    /**
     * If populating this element, either the quantity or the duration must be included.
     */
    initialFill?: MedicationRequestDispenseRequestInitialFill | undefined;
    /**
     * If displaying "number of authorized fills", add 1 to this number.
     */
    numberOfRepeatsAllowed?: number | undefined;
    /**
     * The amount that is to be dispensed for one fill.
     */
    quantity?: Quantity | undefined;
    /**
     * It reflects the prescribers' perspective for the validity of the prescription. Dispenses must not be made against the prescription outside of this period. The lower-bound of the Dispensing Window signifies the earliest date that the prescription can be filled for the first time. If an upper-bound is not specified then the Prescription is open-ended or will default to a stale-date based on regulations.
     */
    validityPeriod?: Period | undefined;
}
/**
 * Indicates whether or not substitution can or should be part of the dispense. In some cases, substitution must happen, in other cases substitution must not happen. This block explains the prescriber's intent. If nothing is specified substitution may be done.
 */
export interface MedicationRequestSubstitution extends BackboneElement {
    /**
     * This element is labeled as a modifier because whether substitution is allow or not, it cannot be ignored.
     */
    allowedBoolean?: boolean | undefined;
    _allowedBoolean?: Element | undefined;
    /**
     * This element is labeled as a modifier because whether substitution is allow or not, it cannot be ignored.
     */
    allowedCodeableConcept?: CodeableConcept | undefined;
    /**
     * Indicates the reason for the substitution, or why substitution must or must not be performed.
     */
    reason?: CodeableConcept | undefined;
}
/**
 * An order or request for both supply of the medication and the instructions for administration of the medication to a patient. The resource is called "MedicationRequest" rather than "MedicationPrescription" or "MedicationOrder" to generalize the use across inpatient and outpatient settings, including care plans, etc., and to harmonize with workflow patterns.
 */
export interface MedicationRequest extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'MedicationRequest';
    /**
     * The date (and perhaps time) when the prescription was initially written or authored on.
     */
    authoredOn?: string | undefined;
    _authoredOn?: Element | undefined;
    /**
     * A plan or request that is fulfilled in whole or in part by this medication request.
     */
    basedOn?: Reference[] | undefined;
    /**
     * The category can be used to include where the medication is expected to be consumed or other types of requests.
     */
    category?: CodeableConcept[] | undefined;
    /**
     * This attribute should not be confused with the protocol of the medication.
     */
    courseOfTherapyType?: CodeableConcept | undefined;
    /**
     * The intended type of device that is to be used for the administration of the medication (for example, PCA Pump).
     */
    device?: CodeableReference | undefined;
    /**
     * Indicates the specific details for the dispense or medication supply part of a medication request (also known as a Medication Prescription or Medication Order).  Note that this information is not always sent with the order.  There may be in some settings (e.g. hospitals) institutional or system support for completing the dispense details in the pharmacy department.
     */
    dispenseRequest?: MedicationRequestDispenseRequest | undefined;
    /**
     * If do not perform is not specified, the request is a positive request e.g. "do perform".
     */
    doNotPerform?: boolean | undefined;
    _doNotPerform?: Element | undefined;
    /**
     * Indicates how the medication is to be used by the patient.
     */
    dose?: MedicationRequestDose | undefined;
    /**
     * This will typically be the encounter the event occurred within, but some activities may be initiated prior to or after the official completion of an encounter but still be tied to the context of the encounter."    If there is a need to link to episodes of care they will be handled with an extension.
     */
    encounter?: Reference | undefined;
    /**
     * This might not include provenances for all versions of the request – only those deemed “relevant” or important. This SHALL NOT include the provenance associated with this current version of the resource. (If that provenance is deemed to be a “relevant” change, it will need to be added as part of a later update. Until then, it can be queried directly as the provenance that points to this version using _revinclude All Provenances should have some historical version of this Request as their subject.).
     */
    eventHistory?: Reference[] | undefined;
    /**
     * A shared identifier common to all requests that were authorized more or less simultaneously by a single author, representing the identifier of the requisition or prescription.
     */
    groupIdentifier?: Identifier | undefined;
    /**
     * This is a business identifier, not a resource identifier.
     */
    identifier?: Identifier[] | undefined;
    /**
     * The person or organization who provided the information about this request, if the source is someone other than the requestor.  This is often used when the MedicationRequest is reported by another person.
     */
    informationSource?: Reference[] | undefined;
    /**
     * Insurance plans, coverage extensions, pre-authorizations and/or pre-determinations that may be required for delivering the requested service.
     */
    insurance?: Reference[] | undefined;
    /**
     * It is expected that the type of requester will be restricted for different stages of a MedicationRequest.  For example, Proposals can be created by a patient, relatedPerson, Practitioner or Device.  Plans can be created by Practitioners, Patients, RelatedPersons and Devices.  Original orders can be created by a Practitioner only.
     * An instance-order is an instantiation of a request or order and may be used to populate Medication Administration Record.
     * This element is labeled as a modifier because the intent alters when and how the resource is actually applicable.
     */
    intent:
        | 'proposal'
        | 'plan'
        | 'order'
        | 'original-order'
        | 'reflex-order'
        | 'filler-order'
        | 'instance-order'
        | 'option';
    _intent?: Element | undefined;
    /**
     * If only a code is specified, then it needs to be a code for a specific product. If more information is required, then the use of the Medication resource is recommended.  For example, if you require form or lot number or if the medication is compounded or extemporaneously prepared, then you must reference the Medication resource.
     */
    medication: CodeableReference;
    /**
     * Extra information about the prescription that could not be conveyed by the other attributes.
     */
    note?: Annotation[] | undefined;
    /**
     * The specified desired performer of the medication treatment (e.g. the performer of the medication administration).  For devices, this is the device that is intended to perform the administration of the medication.  An IV Pump would be an example of a device that is performing the administration.  Both the IV Pump and the practitioner that set the rate or bolus on the pump can be listed as performers.
     */
    performer?: Reference[] | undefined;
    /**
     * If specified without indicating a performer, this indicates that the performer must be of the specified type. If specified with a performer then it indicates the requirements of the performer if the designated performer is not available.
     */
    performerType?: CodeableConcept | undefined;
    /**
     * Indicates how quickly the Medication Request should be addressed with respect to other requests.
     */
    priority?: ('routine' | 'urgent' | 'asap' | 'stat') | undefined;
    _priority?: Element | undefined;
    /**
     * A link to a resource representing an earlier order related order or prescription.
     */
    priorPrescription?: Reference | undefined;
    /**
     * This could be a diagnosis code. If a full condition record exists or additional detail is needed, use reasonReference.
     */
    reason?: CodeableReference[] | undefined;
    /**
     * The person who entered the order on behalf of another individual for example in the case of a verbal or a telephone order.
     */
    recorder?: Reference | undefined;
    /**
     * If not populated, then assume that this is the original record and not reported
     */
    reported?: boolean | undefined;
    _reported?: Element | undefined;
    /**
     * The individual, organization, or device that initiated the request and has responsibility for its activation.
     */
    requester?: Reference | undefined;
    /**
     * This element is labeled as a modifier because the status contains codes that mark the resource as not currently valid.
     * Clinical decision support systems should take the status into account when determining which medications to include in their algorithms.
     */
    status:
        | 'active'
        | 'on-hold'
        | 'ended'
        | 'stopped'
        | 'completed'
        | 'cancelled'
        | 'entered-in-error'
        | 'draft'
        | 'unknown';
    _status?: Element | undefined;
    /**
     * The date (and perhaps time) when the status was changed.
     */
    statusChanged?: string | undefined;
    _statusChanged?: Element | undefined;
    /**
     * This is generally only used for "exception" statuses such as "suspended" or "cancelled". The reason why the MedicationRequest was created at all is captured in reasonCode, not here.
     */
    statusReason?: CodeableConcept | undefined;
    /**
     * The subject on a medication request is mandatory.  For the secondary use case where the actual subject is not provided, there still must be an anonymized subject specified.
     */
    subject: Reference;
    /**
     * Indicates whether or not substitution can or should be part of the dispense. In some cases, substitution must happen, in other cases substitution must not happen. This block explains the prescriber's intent. If nothing is specified substitution may be done.
     */
    substitution?: MedicationRequestSubstitution | undefined;
    /**
     * This attribute can be used to reference a MedicationUsage about the patients' medication use.
     */
    supportingInformation?: Reference[] | undefined;
}
/**
 * Indicates if the medication is being consumed or administered as instructed.
 */
export interface MedicationUsageAdherence extends BackboneElement {
    /**
     * Type of the adherence for the medication.
     */
    code: CodeableConcept;
    /**
     * This is generally only used for "exception" statuses such as "entered-in-error". The reason for performing the event at all is captured in reasonCode, not here.
     */
    reason?: CodeableConcept | undefined;
}
/**
 * A record of a medication that is being consumed by a patient.   A MedicationUsage may indicate that the patient may be taking the medication now or has taken the medication in the past or will be taking the medication in the future.  The source of this information can be the patient, significant other (such as a family member or spouse), or a clinician.  A common scenario where this information is captured is during the history taking process during a patient visit or stay.   The medication information may come from sources such as the patient's memory, from a prescription bottle,  or from a list of medications the patient, clinician or other party maintains.
 * The primary difference between a medicationusage and a medicationadministration is that the medication administration has complete administration information and is based on actual administration information from the person who administered the medication.  A medicationusage is often, if not always, less specific.  There is no required date/time when the medication was administered, in fact we only know that a source has reported the patient is taking this medication, where details such as time, quantity, or rate or even medication product may be incomplete or missing or less precise.  As stated earlier, the Medication Usage information may come from the patient's memory, from a prescription bottle or from a list of medications the patient, clinician or other party maintains.  Medication administration is more formal and is not missing detailed information.
 * The MedicationUsage resource was previously called MedicationStatement.
 */
export interface MedicationUsage extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'MedicationUsage';
    /**
     * Indicates if the medication is being consumed or administered as instructed.
     */
    adherence?: MedicationUsageAdherence | undefined;
    /**
     * Type of medication usage (for example, drug classification like ATC, where meds would be administered, legal category of the medication.).
     */
    category?: CodeableConcept[] | undefined;
    /**
     * The date when the Medication Usage was asserted by the information source.
     */
    dateAsserted?: string | undefined;
    _dateAsserted?: Element | undefined;
    /**
     * Likely references would be to MedicationRequest, MedicationDispense, Claim, Observation or QuestionnaireAnswers.  The most common use cases for deriving a MedicationUsage comes from creating a MedicationUsage from a MedicationRequest or from a lab observation or a claim.  it should be noted that the amount of information that is available varies from the type resource that you derive the MedicationUsage from.
     */
    derivedFrom?: Reference[] | undefined;
    /**
     * The dates included in the dosage on a Medication Usage reflect the dates for a given dose.  For example, "from November 1, 2016 to November 3, 2016, take one tablet daily and from November 4, 2016 to November 7, 2016, take two tablets daily."  It is expected that this specificity may only be populated where the patient brings in their labeled container or where the Medication Usage is derived from a MedicationRequest.
     */
    dosage?: Dosage[] | undefined;
    /**
     * This attribute reflects the period over which the patient consumed the medication and is expected to be populated on the majority of Medication Usages. If the medication is still being taken and is expected to continue indefinitely at the time the usage is recorded, the "end" date will be omitted.  If the end date is known, then it is included as the "end date".  The date/time attribute supports a variety of dates - year, year/month and exact date.  If something more than this is required, this should be conveyed as text.
     */
    effectiveDateTime?: string | undefined;
    _effectiveDateTime?: Element | undefined;
    /**
     * This attribute reflects the period over which the patient consumed the medication and is expected to be populated on the majority of Medication Usages. If the medication is still being taken and is expected to continue indefinitely at the time the usage is recorded, the "end" date will be omitted.  If the end date is known, then it is included as the "end date".  The date/time attribute supports a variety of dates - year, year/month and exact date.  If something more than this is required, this should be conveyed as text.
     */
    effectivePeriod?: Period | undefined;
    /**
     * The encounter that establishes the context for this MedicationUsage.
     */
    encounter?: Reference | undefined;
    /**
     * This is a business identifier, not a resource identifier.
     */
    identifier?: Identifier[] | undefined;
    /**
     * The person or organization that provided the information about the taking of this medication. Note: Use derivedFrom when a MedicationUsage is derived from other resources, e.g. Claim or MedicationRequest.
     */
    informationSource?: Reference[] | undefined;
    /**
     * If only a code is specified, then it needs to be a code for a specific product. If more information is required, then the use of the medication resource is recommended.  For example, if you require form or lot number, then you must reference the Medication resource.
     */
    medication: CodeableReference;
    /**
     * Provides extra information about the Medication Usage that is not conveyed by the other attributes.
     */
    note?: Annotation[] | undefined;
    /**
     * A larger event of which this particular MedicationUsage is a component or step.
     */
    partOf?: Reference[] | undefined;
    /**
     * This could be a diagnosis code. If a full condition record exists or additional detail is needed, use reasonForUseReference.
     */
    reason?: CodeableReference[] | undefined;
    /**
     * Link to information that is relevant to a medication usage, for example, illicit drug use, gestational age, etc.
     */
    relatedClinicalInformation?: Reference[] | undefined;
    /**
     * The full representation of the dose of the medication included in all dosage instructions.  To be used when multiple dosage instructions are included to represent complex dosing such as increasing or tapering doses.
     */
    renderedDosageInstruction?: string | undefined;
    _renderedDosageInstruction?: Element | undefined;
    /**
     * This status concerns just the recording of the medication usage.  MedicationUsage.adherence should be used for indicating a patient's adherence to the information in this resource.  Note, the statuses are different than in previous releases for MedicationUsage or MedicationStatement.
     * This element is labeled as a modifier because the status contains codes that mark the resource as not currently valid.
     */
    status: 'recorded' | 'entered-in-error' | 'draft';
    _status?: Element | undefined;
    /**
     * The person, animal or group who is/was taking the medication.
     */
    subject: Reference;
}
/**
 * A product specific contact, person (in a role), or an organization.
 */
export interface MedicinalProductDefinitionContact extends BackboneElement {
    /**
     * A product specific contact, person (in a role), or an organization.
     */
    contact: Reference;
    /**
     * Allows the contact to be classified, for example QPPV, Pharmacovigilance Enquiry Information.
     */
    type?: CodeableConcept | undefined;
}
/**
 * Coding words or phrases of the name.
 */
export interface MedicinalProductDefinitionNamePart extends BackboneElement {
    /**
     * A fragment of a product name.
     */
    part: string;
    _part?: Element | undefined;
    /**
     * Identifying type for this part of the name (e.g. strength part).
     */
    type: CodeableConcept;
}
/**
 * Country and jurisdiction where the name applies, and associated language.
 */
export interface MedicinalProductDefinitionNameUsage extends BackboneElement {
    /**
     * Country code for where this name applies.
     */
    country: CodeableConcept;
    /**
     * Jurisdiction code for where this name applies. A jurisdiction may be a sub- or supra-national entity (e.g. a state or a geographic region).
     */
    jurisdiction?: CodeableConcept | undefined;
    /**
     * Language code for this name.
     */
    language: CodeableConcept;
}
/**
 * The product's name, including full name and possibly coded parts.
 */
export interface MedicinalProductDefinitionName extends BackboneElement {
    /**
     * Coding words or phrases of the name.
     */
    part?: MedicinalProductDefinitionNamePart[] | undefined;
    /**
     * The full product name.
     */
    productName: string;
    _productName?: Element | undefined;
    /**
     * Type of product name, such as rINN, BAN, Proprietary, Non-Proprietary.
     */
    type?: CodeableConcept | undefined;
    /**
     * Country and jurisdiction where the name applies, and associated language.
     */
    usage?: MedicinalProductDefinitionNameUsage[] | undefined;
}
/**
 * Reference to another product, e.g. for linking authorised to investigational product, or a virtual product.
 */
export interface MedicinalProductDefinitionCrossReference extends BackboneElement {
    /**
     * Reference to another product, e.g. for linking authorised to investigational product.
     */
    product: CodeableReference;
    /**
     * The type of relationship, for instance branded to generic, virtual to actual product, product to development product (investigational), parallel import version.
     */
    type?: CodeableConcept | undefined;
}
/**
 * A manufacturing or administrative process or step associated with (or performed on) the medicinal product.
 */
export interface MedicinalProductDefinitionOperation extends BackboneElement {
    /**
     * Specifies whether this particular business or manufacturing process is considered proprietary or confidential.
     */
    confidentialityIndicator?: CodeableConcept | undefined;
    /**
     * Date range of applicability.
     */
    effectiveDate?: Period | undefined;
    /**
     * The organization or establishment responsible for (or associated with) the particular process or step, examples include the manufacturer, importer, agent.
     */
    organization?: Reference[] | undefined;
    /**
     * The type of manufacturing operation e.g. manufacturing itself, re-packaging. For the authorization of this, a RegulatedAuthorization would point to the same plan or activity referenced here.
     */
    type?: CodeableReference | undefined;
}
/**
 * Allows the key product features to be recorded, such as "sugar free", "modified release", "parallel import".
 */
export interface MedicinalProductDefinitionCharacteristic extends BackboneElement {
    /**
     * A code expressing the type of characteristic.
     */
    type: CodeableConcept;
    /**
     * String is intended to be used only when the item cannot ever reasonably be coded. Otherwise, for cases where a code is not available, but is feasible, use CodeableConcept. (temporary text to make build work: The usage of CodeableConcept is for concepts that are normally coded but for which a code is not available - for example a device implant size that is not standardized. The property string is for text characteristics, e.g. long descriptions, warnings, etc.).
     */
    valueCodeableConcept?: CodeableConcept | undefined;
    /**
     * String is intended to be used only when the item cannot ever reasonably be coded. Otherwise, for cases where a code is not available, but is feasible, use CodeableConcept. (temporary text to make build work: The usage of CodeableConcept is for concepts that are normally coded but for which a code is not available - for example a device implant size that is not standardized. The property string is for text characteristics, e.g. long descriptions, warnings, etc.).
     */
    valueString?: string | undefined;
    _valueString?: Element | undefined;
    /**
     * String is intended to be used only when the item cannot ever reasonably be coded. Otherwise, for cases where a code is not available, but is feasible, use CodeableConcept. (temporary text to make build work: The usage of CodeableConcept is for concepts that are normally coded but for which a code is not available - for example a device implant size that is not standardized. The property string is for text characteristics, e.g. long descriptions, warnings, etc.).
     */
    valueQuantity?: Quantity | undefined;
    /**
     * String is intended to be used only when the item cannot ever reasonably be coded. Otherwise, for cases where a code is not available, but is feasible, use CodeableConcept. (temporary text to make build work: The usage of CodeableConcept is for concepts that are normally coded but for which a code is not available - for example a device implant size that is not standardized. The property string is for text characteristics, e.g. long descriptions, warnings, etc.).
     */
    valueInteger?: number | undefined;
    /**
     * String is intended to be used only when the item cannot ever reasonably be coded. Otherwise, for cases where a code is not available, but is feasible, use CodeableConcept. (temporary text to make build work: The usage of CodeableConcept is for concepts that are normally coded but for which a code is not available - for example a device implant size that is not standardized. The property string is for text characteristics, e.g. long descriptions, warnings, etc.).
     */
    valueDate?: string | undefined;
    _valueDate?: Element | undefined;
    /**
     * String is intended to be used only when the item cannot ever reasonably be coded. Otherwise, for cases where a code is not available, but is feasible, use CodeableConcept. (temporary text to make build work: The usage of CodeableConcept is for concepts that are normally coded but for which a code is not available - for example a device implant size that is not standardized. The property string is for text characteristics, e.g. long descriptions, warnings, etc.).
     */
    valueBoolean?: boolean | undefined;
    _valueBoolean?: Element | undefined;
    /**
     * String is intended to be used only when the item cannot ever reasonably be coded. Otherwise, for cases where a code is not available, but is feasible, use CodeableConcept. (temporary text to make build work: The usage of CodeableConcept is for concepts that are normally coded but for which a code is not available - for example a device implant size that is not standardized. The property string is for text characteristics, e.g. long descriptions, warnings, etc.).
     */
    valueAttachment?: Attachment | undefined;
}
/**
 * A medicinal product, being a substance or combination of substances that is intended to treat, prevent or diagnose a disease, or to restore, correct or modify physiological functions by exerting a pharmacological, immunological or metabolic action. This resource is intended to define and detail such products and their properties, for uses other than direct patient care (e.g. regulatory use, or drug catalogs).
 */
export interface MedicinalProductDefinition extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'MedicinalProductDefinition';
    /**
     * Whether the Medicinal Product is subject to additional monitoring for regulatory reasons, such as heightened reporting requirements.
     */
    additionalMonitoringIndicator?: CodeableConcept | undefined;
    /**
     * Additional information or supporting documentation about the medicinal product.
     */
    attachedDocument?: Reference[] | undefined;
    /**
     * Allows the key product features to be recorded, such as "sugar free", "modified release", "parallel import".
     */
    characteristic?: MedicinalProductDefinitionCharacteristic[] | undefined;
    /**
     * Allows the product to be classified by various systems, commonly WHO ATC.
     */
    classification?: CodeableConcept[] | undefined;
    /**
     * Clinical trials or studies that this product is involved in.
     */
    clinicalTrial?: Reference[] | undefined;
    /**
     * A code that this product is known by, usually within some formal terminology, perhaps assigned by a third party (i.e. not the manufacturer or regulator). Products (types of medications) tend to be known by identifiers during development and within regulatory process. However when they are prescribed they tend to be identified by codes. The same product may be have multiple codes, applied to it by multiple organizations.
     */
    code?: Coding[] | undefined;
    /**
     * The dose form for a single part product, or combined form of a multiple part product. This is one concept that describes all the components. It does not represent the form with components physically mixed, if that might be necessary, for which see (AdministrableProductDefinition.administrableDoseForm).
     */
    combinedPharmaceuticalDoseForm?: CodeableConcept | undefined;
    /**
     * Types of medicinal manufactured items and/or devices that this product consists of, such as tablets, capsule, or syringes. Used as a direct link when the item's packaging is not being recorded (see also PackagedProductDefinition.package.containedItem.item).
     */
    comprisedOf?: Reference[] | undefined;
    /**
     * A product specific contact, person (in a role), or an organization.
     */
    contact?: MedicinalProductDefinitionContact[] | undefined;
    /**
     * Reference to another product, e.g. for linking authorised to investigational product, or a virtual product.
     */
    crossReference?: MedicinalProductDefinitionCrossReference[] | undefined;
    /**
     * General description of this product.
     */
    description?: string | undefined;
    _description?: Element | undefined;
    /**
     * If this medicine applies to human or veterinary uses.
     */
    domain?: CodeableConcept | undefined;
    /**
     * Business identifier for this product. Could be an MPID. When in development or being regulated, products are typically referenced by official identifiers, assigned by a manufacturer or regulator, and unique to a product (which, when compared to a product instance being prescribed, is actually a product type). See also MedicinalProductDefinition.code.
     */
    identifier?: Identifier[] | undefined;
    /**
     * Any component of the drug product which is not the chemical entity defined as the drug substance, or an excipient in the drug product. This includes process-related impurities and contaminants, product-related impurities including degradation products.
     */
    impurity?: CodeableReference[] | undefined;
    /**
     * Description of indication(s) for this product, used when structured indications are not required. In cases where structured indications are required, they are captured using the ClinicalUseDefinition resource. An indication is a medical situation for which using the product is appropriate.
     */
    indication?: string | undefined;
    _indication?: Element | undefined;
    /**
     * The ingredients of this medicinal product - when not detailed in other resources. This is only needed if the ingredients are not specified by incoming references from the Ingredient resource, or indirectly via incoming AdministrableProductDefinition, PackagedProductDefinition or ManufacturedItemDefinition references. In cases where those levels of detail are not used, the ingredients may be specified directly here as codes.
     */
    ingredient?: CodeableConcept[] | undefined;
    /**
     * The legal status of supply of the medicinal product as classified by the regulator.
     */
    legalStatusOfSupply?: CodeableConcept | undefined;
    /**
     * Marketing status of the medicinal product, in contrast to marketing authorization. This refers to the product being actually 'on the market' as opposed to being allowed to be on the market (which is an authorization).
     */
    marketingStatus?: MarketingStatus[] | undefined;
    /**
     * A master file for the medicinal product (e.g. Pharmacovigilance System Master File). Drug master files (DMFs) are documents submitted to regulatory agencies to provide confidential detailed information about facilities, processes or articles used in the manufacturing, processing, packaging and storing of drug products.
     */
    masterFile?: Reference[] | undefined;
    /**
     * The product's name, including full name and possibly coded parts.
     */
    name: MedicinalProductDefinitionName[];
    /**
     * A manufacturing or administrative process or step associated with (or performed on) the medicinal product.
     */
    operation?: MedicinalProductDefinitionOperation[] | undefined;
    /**
     * Package type for the product. See also the PackagedProductDefinition resource.
     */
    packagedMedicinalProduct?: CodeableConcept[] | undefined;
    /**
     * If authorised for use in children, or infants, neonates etc.
     */
    pediatricUseIndicator?: CodeableConcept | undefined;
    /**
     * The path by which the product is taken into or makes contact with the body. In some regions this is referred to as the licenced or approved route. See also AdministrableProductDefinition resource. MedicinalProductDefinition.route is the same concept as AdministrableProductDefinition.routeOfAdministration.code, and they cannot be used together.
     */
    route?: CodeableConcept[] | undefined;
    /**
     * Whether the Medicinal Product is subject to special measures for regulatory reasons, such as a requirement to conduct post-authorisation studies.
     */
    specialMeasures?: CodeableConcept[] | undefined;
    /**
     * The status within the lifecycle of this product record. A high-level status, this is not intended to duplicate details carried elsewhere such as legal status, or authorization status.
     */
    status?: CodeableConcept | undefined;
    /**
     * The date at which the given status became applicable.
     */
    statusDate?: string | undefined;
    _statusDate?: Element | undefined;
    /**
     * Regulatory type, e.g. Investigational or Authorized.
     */
    type?: CodeableConcept | undefined;
    /**
     * A business identifier relating to a specific version of the product, this is commonly used to support revisions to an existing product.
     */
    version?: string | undefined;
    _version?: Element | undefined;
}
/**
 * Identifies the resource (or resources) that are being addressed by the event.  For example, the Encounter for an admit message or two Account records for a merge.
 */
export interface MessageDefinitionFocus extends BackboneElement {
    /**
     * Multiple focuses addressing different resources may occasionally occur.  E.g. to link or unlink a resource from a particular account or encounter, etc.
     */
    code: string;
    _code?: Element | undefined;
    /**
     * Identifies the maximum number of resources of this type that must be pointed to by a message in order for it to be valid against this MessageDefinition.
     */
    max?: string | undefined;
    _max?: Element | undefined;
    /**
     * Identifies the minimum number of resources of this type that must be pointed to by a message in order for it to be valid against this MessageDefinition.
     */
    min: number;
    /**
     * This should be present for most message definitions.  However, if the message focus is only a single resource and there is no need to include referenced resources or otherwise enforce the presence of particular elements, it can be omitted.
     */
    profile?: string | undefined;
    _profile?: Element | undefined;
}
/**
 * This indicates an application level response to "close" a transaction implicit in a particular request message.  To define a complete workflow scenario, look to the [[PlanDefinition]] resource which allows the definition of complex orchestrations, conditionality, etc.
 */
export interface MessageDefinitionAllowedResponse extends BackboneElement {
    /**
     * A reference to the message definition that must be adhered to by this supported response.
     */
    message: string;
    _message?: Element | undefined;
    /**
     * Provides a description of the circumstances in which this response should be used (as opposed to one of the alternative responses).
     */
    situation?: string | undefined;
    _situation?: Element | undefined;
}
/**
 * Defines the characteristics of a message that can be shared between systems, including the type of event that initiates the message, the content to be transmitted and what response(s), if any, are permitted.
 */
export interface MessageDefinition extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'MessageDefinition';
    /**
     * This indicates an application level response to "close" a transaction implicit in a particular request message.  To define a complete workflow scenario, look to the [[PlanDefinition]] resource which allows the definition of complex orchestrations, conditionality, etc.
     */
    allowedResponse?: MessageDefinitionAllowedResponse[] | undefined;
    /**
     * The MessageDefinition that is the basis for the contents of this resource.
     */
    base?: string | undefined;
    _base?: Element | undefined;
    /**
     * The impact of the content of the message.
     */
    category?: ('consequence' | 'currency' | 'notification') | undefined;
    _category?: Element | undefined;
    /**
     * May be a web site, an email address, a telephone number, etc.
     */
    contact?: ContactDetail[] | undefined;
    /**
     * A copyright statement relating to the message definition and/or its contents. Copyright statements are generally legal restrictions on the use and publishing of the message definition.
     */
    copyright?: string | undefined;
    _copyright?: Element | undefined;
    /**
     * Note that this is not the same as the resource last-modified-date, since the resource may be a secondary representation of the message definition. Additional specific dates may be added as extensions or be found by consulting Provenances associated with past versions of the resource.
     */
    date: string;
    _date?: Element | undefined;
    /**
     * This description can be used to capture details such as comments about misuse, instructions for clinical use and interpretation, literature references, examples from the paper world, etc. It is not a rendering of the message definition as conveyed in the 'text' field of the resource itself. This item SHOULD be populated unless the information is available from context (e.g. the language of the message definition is presumed to be the predominant language in the place the message definition was created).
     */
    description?: string | undefined;
    _description?: Element | undefined;
    /**
     * Event code or link to the EventDefinition.
     */
    eventCoding?: Coding | undefined;
    /**
     * Event code or link to the EventDefinition.
     */
    eventUri?: string | undefined;
    _eventUri?: Element | undefined;
    /**
     * Allows filtering of message definitions that are appropriate for use versus not.
     */
    experimental?: boolean | undefined;
    _experimental?: Element | undefined;
    /**
     * Identifies the resource (or resources) that are being addressed by the event.  For example, the Encounter for an admit message or two Account records for a merge.
     */
    focus?: MessageDefinitionFocus[] | undefined;
    /**
     * Canonical reference to a GraphDefinition. If a URL is provided, it is the canonical reference to a [GraphDefinition](graphdefinition.html) that it controls what resources are to be added to the bundle when building the document. The GraphDefinition can also specify profiles that apply to the various resources.
     */
    graph?: string | undefined;
    _graph?: Element | undefined;
    /**
     * Typically, this is used for identifiers that can go in an HL7 V3 II (instance identifier) data type, and can then identify this message definition outside of FHIR, where it is not possible to use the logical URI.
     */
    identifier?: Identifier[] | undefined;
    /**
     * It may be possible for the message definition to be used in jurisdictions other than those for which it was originally designed or intended.
     */
    jurisdiction?: CodeableConcept[] | undefined;
    /**
     * The name is not expected to be globally unique. The name should be a simple alphanumeric type name to ensure that it is machine-processing friendly.
     */
    name?: string | undefined;
    _name?: Element | undefined;
    /**
     * It should be possible to use MessageDefinition to describe a message to be used by certain steps in a particular protocol as part of a PlanDefinition or ActivityDefinition.
     */
    parent?: string[] | undefined;
    _parent?: Element[] | undefined;
    /**
     * Usually an organization but may be an individual. The publisher (or steward) of the message definition is the organization or individual primarily responsible for the maintenance and upkeep of the message definition. This is not necessarily the same individual or organization that developed and initially authored the content. The publisher is the primary point of contact for questions or issues with the message definition. This item SHOULD be populated unless the information is available from context.
     */
    publisher?: string | undefined;
    _publisher?: Element | undefined;
    /**
     * This element does not describe the usage of the message definition. Instead, it provides traceability of ''why'' the resource is either needed or ''why'' it is defined as it is.  This may be used to point to source materials or specifications that drove the structure of this message definition.
     */
    purpose?: string | undefined;
    _purpose?: Element | undefined;
    /**
     * A MessageDefinition that is superseded by this definition.
     */
    replaces?: string[] | undefined;
    _replaces?: Element[] | undefined;
    /**
     * This enables the capability currently available through MSH-16 (Application Level acknowledgement) in HL7 Version 2 to declare at a message instance level whether a response is required or only upon error or success, or never.
     */
    responseRequired?: ('always' | 'on-error' | 'never' | 'on-success') | undefined;
    _responseRequired?: Element | undefined;
    /**
     * Allows filtering of message definitions that are appropriate for use versus not.
     */
    status: 'draft' | 'active' | 'retired' | 'unknown';
    _status?: Element | undefined;
    /**
     * This name does not need to be machine-processing friendly and may contain punctuation, white-space, etc.
     */
    title?: string | undefined;
    _title?: Element | undefined;
    /**
     * Can be a urn:uuid: or a urn:oid: but real http: addresses are preferred.  Multiple instances may share the same URL if they have a distinct version.
     * The determination of when to create a new version of a resource (same url, new version) vs. defining a new artifact is up to the author.  Considerations for making this decision are found in [Technical and Business Versions](resource.html#versions).
     * In some cases, the resource can no longer be found at the stated url, but the url itself cannot change. Implementations can use the [meta.source](resource.html#meta) element to indicate where the current master source of the resource can be found.
     */
    url?: string | undefined;
    _url?: Element | undefined;
    /**
     * When multiple useContexts are specified, there is no expectation that all or any of the contexts apply.
     */
    useContext?: UsageContext[] | undefined;
    /**
     * There may be different message definition instances that have the same identifier but different versions.  The version can be appended to the url in a reference to allow a reference to a particular business version of the message definition with the format [url]|[version]. The version SHOULD NOT contain a '#' - see [Business Version](resource.html#bv-format).
     */
    version?: string | undefined;
    _version?: Element | undefined;
}
/**
 * There SHOULD be at least one destination, but in some circumstances, the source system is unaware of any particular destination system.
 */
export interface MessageHeaderDestination extends BackboneElement {
    /**
     * The id may be a non-resolvable URI for systems that do not use standard network-based addresses.
     */
    endpoint?: string | undefined;
    _endpoint?: Element | undefined;
    /**
     * Human-readable name for the target system.
     */
    name?: string | undefined;
    _name?: Element | undefined;
    /**
     * Allows data conveyed by a message to be addressed to a particular person or department when routing to a specific application isn't sufficient.
     */
    receiver?: Reference | undefined;
    /**
     * Identifies the target end system in situations where the initial message transmission is to an intermediary system.
     */
    target?: Reference | undefined;
}
/**
 * The source application from which this message originated.
 */
export interface MessageHeaderSource extends BackboneElement {
    /**
     * An e-mail, phone, website or other contact point to use to resolve issues with message communications.
     */
    contact?: ContactPoint | undefined;
    /**
     * The id may be a non-resolvable URI for systems that do not use standard network-based addresses.
     */
    endpoint?: string | undefined;
    _endpoint?: Element | undefined;
    /**
     * Human-readable name for the source system.
     */
    name?: string | undefined;
    _name?: Element | undefined;
    /**
     * May include configuration or other information useful in debugging.
     */
    software?: string | undefined;
    _software?: Element | undefined;
    /**
     * Can convey versions of multiple systems in situations where a message passes through multiple hands.
     */
    version?: string | undefined;
    _version?: Element | undefined;
}
/**
 * Information about the message that this message is a response to.  Only present if this message is a response.
 */
export interface MessageHeaderResponse extends BackboneElement {
    /**
     * This is a generic response to the request message. Specific data for the response will be found in MessageHeader.focus.
     */
    code: 'ok' | 'transient-error' | 'fatal-error';
    _code?: Element | undefined;
    /**
     * This SHALL be contained in the bundle. If any of the issues are errors, the response code SHALL be an error.
     */
    details?: Reference | undefined;
    /**
     * The MessageHeader.id of the message to which this message is a response.
     */
    identifier: Identifier;
}
/**
 * The header for a message exchange that is either requesting or responding to an action.  The reference(s) that are the subject of the action as well as other information related to the action are typically transmitted in a bundle in which the MessageHeader resource instance is the first resource in the bundle.
 */
export interface MessageHeader extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'MessageHeader';
    /**
     * Usually only for the request but can be used in a response.
     */
    author?: Reference | undefined;
    /**
     * Permanent link to the MessageDefinition for this message.
     */
    definition?: string | undefined;
    _definition?: Element | undefined;
    /**
     * There SHOULD be at least one destination, but in some circumstances, the source system is unaware of any particular destination system.
     */
    destination?: MessageHeaderDestination[] | undefined;
    /**
     * Usually only for the request but can be used in a response.
     */
    enterer?: Reference | undefined;
    /**
     * The time of the event will be found in the focus resource. The time of the message will be found in [Bundle.timestamp](bundle-definitions.html#Bundle.timestamp).
     */
    eventCoding?: Coding | undefined;
    /**
     * The time of the event will be found in the focus resource. The time of the message will be found in [Bundle.timestamp](bundle-definitions.html#Bundle.timestamp).
     */
    eventCanonical?: string | undefined;
    _eventCanonical?: Element | undefined;
    /**
     * The data is defined where the transaction type is defined. The transaction data is always included in the bundle that is the full message.  Only the root resource is specified.  The resources it references should be contained in the bundle but are not also listed here.  Multiple repetitions are allowed to cater for merges and other situations with multiple focal targets.
     */
    focus?: Reference[] | undefined;
    /**
     * Coded indication of the cause for the event - indicates  a reason for the occurrence of the event that is a focus of this message.
     */
    reason?: CodeableConcept | undefined;
    /**
     * Information about the message that this message is a response to.  Only present if this message is a response.
     */
    response?: MessageHeaderResponse | undefined;
    /**
     * Usually only for the request but can be used in a response.
     */
    responsible?: Reference | undefined;
    /**
     * Use case is for where a (trusted) sending system is responsible for multiple organizations, and therefore cannot differentiate based on source endpoint / authentication alone.
     */
    sender?: Reference | undefined;
    /**
     * The source application from which this message originated.
     */
    source: MessageHeaderSource;
}
/**
 * Common Interface declaration for conformance and knowledge artifact resources.
 */
export interface MetadataResource extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: string;
    /**
     * The 'date' element may be more recent than the approval date because of minor changes or editorial corrections.
     */
    approvalDate?: string | undefined;
    _approvalDate?: Element | undefined;
    /**
     * An individiual or organization primarily involved in the creation and maintenance of the {{title}}.
     */
    author?: ContactDetail[] | undefined;
    /**
     * An individual or organization primarily responsible for internal coherence of the {{title}}.
     */
    editor?: ContactDetail[] | undefined;
    /**
     * The effective period for a {{title}}  determines when the content is applicable for usage and is independent of publication and review dates. For example, a measure intended to be used for the year 2016 might be published in 2015.
     */
    effectivePeriod?: Period | undefined;
    /**
     * An individual or organization responsible for officially endorsing the {{title}} for use in some setting.
     */
    endorser?: ContactDetail[] | undefined;
    /**
     * If specified, this date follows the original approval date.
     */
    lastReviewDate?: string | undefined;
    _lastReviewDate?: Element | undefined;
    /**
     * Each related artifact is either an attachment, or a reference to another resource, but not both.
     */
    relatedArtifact?: RelatedArtifact[] | undefined;
    /**
     * An individual or organization primarily responsible for review of some aspect of the {{title}}.
     */
    reviewer?: ContactDetail[] | undefined;
    /**
     * This element provides topical categorization and keywords for the {{title}}, as opposed to the more structured context-of-use information provided in the useContext element
     */
    topic?: CodeableConcept[] | undefined;
}
/**
 * A sequence that is used as a starting sequence to describe variants that are present in a sequence analyzed.
 */
export interface MolecularSequenceRelativeStartingSequence extends BackboneElement {
    /**
     * Structural unit composed of a nucleic acid molecule which controls its own replication through the interaction of specific proteins at one or more origins of replication ([SO:0000340](http://www.sequenceontology.org/browser/current_svn/term/SO:0000340)).
     */
    chromosome?: CodeableConcept | undefined;
    /**
     * The genome assembly used for starting sequence, e.g. GRCh38.
     */
    genomeAssembly?: CodeableConcept | undefined;
    /**
     * A relative reference to a DNA strand based on gene orientation. The strand that contains the open reading frame of the gene is the "sense" strand, and the opposite complementary strand is the "antisense" strand.
     */
    orientation?: ('sense' | 'antisense') | undefined;
    _orientation?: Element | undefined;
    /**
     * A starting sequence may be represented in one of three ways:
     *   1.  CodeableConcept, using NCBI, LRG or other appropriate code systems
     *   1.  a simple string of IUPAC codes
     *   1.  a reference to another MolecularSequence resource.
     */
    sequenceCodeableConcept?: CodeableConcept | undefined;
    /**
     * A starting sequence may be represented in one of three ways:
     *   1.  CodeableConcept, using NCBI, LRG or other appropriate code systems
     *   1.  a simple string of IUPAC codes
     *   1.  a reference to another MolecularSequence resource.
     */
    sequenceString?: string | undefined;
    _sequenceString?: Element | undefined;
    /**
     * A starting sequence may be represented in one of three ways:
     *   1.  CodeableConcept, using NCBI, LRG or other appropriate code systems
     *   1.  a simple string of IUPAC codes
     *   1.  a reference to another MolecularSequence resource.
     */
    sequenceReference?: Reference | undefined;
    /**
     * An absolute reference to a strand. The Watson strand is the strand whose 5'-end is on the short arm of the chromosome, and the Crick strand as the one whose 5'-end is on the long arm.
     */
    strand?: ('watson' | 'crick') | undefined;
    _strand?: Element | undefined;
    /**
     * End position of the window on the starting sequence. This value should honor the rules of the  coordinateSystem.
     */
    windowEnd?: number | undefined;
    /**
     * Start position of the window on the starting sequence. This value should honor the rules of the coordinateSystem.
     */
    windowStart?: number | undefined;
}
/**
 * Changes in sequence from the starting sequence.
 */
export interface MolecularSequenceRelativeEdit extends BackboneElement {
    /**
     * End position of the edit on the starting sequence. If the coordinate system is 0-based then end is exclusive and does not include the last position. If the coordinate system is 1-base, then end is inclusive and includes the last position.
     */
    end?: number | undefined;
    /**
     * Allele in the starting sequence. Nucleotide(s)/amino acids from start position of sequence to stop position of sequence on the positive (+) strand of the starting sequence. When the sequence  type is DNA, it should be the sequence on the positive (+) strand. This will lay in the range between variant.start and variant.end.
     */
    replacedSequence?: string | undefined;
    _replacedSequence?: Element | undefined;
    /**
     * Allele that was observed. Nucleotide(s)/amino acids from start position of sequence to stop position of sequence on the positive (+) strand of the observed sequence. When the sequence type is DNA, it should be the sequence on the positive (+) strand. This will lay in the range between variant.start and variant.end.
     */
    replacementSequence?: string | undefined;
    _replacementSequence?: Element | undefined;
    /**
     * Start position of the edit on the starting sequence. If the coordinate system is either 0-based or 1-based, then start position is inclusive.
     */
    start?: number | undefined;
}
/**
 * A sequence defined relative to another sequence.
 */
export interface MolecularSequenceRelative extends BackboneElement {
    /**
     * These are different ways of identifying nucleotides or amino acids within a sequence. Different databases and file types may use different systems. For detail definitions, see https://loinc.org/92822-6/ for more detail.
     */
    coordinateSystem: CodeableConcept;
    /**
     * Changes in sequence from the starting sequence.
     */
    edit?: MolecularSequenceRelativeEdit[] | undefined;
    /**
     * Indicates the order in which the sequence should be considered when putting multiple 'relative' elements together.
     */
    ordinalPosition?: number | undefined;
    /**
     * Indicates the nucleotide range in the composed sequence when multiple 'relative' elements are used together.
     */
    sequenceRange?: Range | undefined;
    /**
     * A sequence that is used as a starting sequence to describe variants that are present in a sequence analyzed.
     */
    startingSequence?: MolecularSequenceRelativeStartingSequence | undefined;
}
/**
 * Representation of a molecular sequence.
 */
export interface MolecularSequence extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'MolecularSequence';
    /**
     * The method for sequencing, for example, chip information.
     */
    device?: Reference | undefined;
    /**
     * Sequence that was observed as file content. Can be an actual file contents, or referenced by a URL to an external system.
     */
    formatted?: Attachment[] | undefined;
    /**
     * A unique identifier for this particular sequence instance.
     */
    identifier?: Identifier[] | undefined;
    /**
     * Sequence that was observed.
     */
    literal?: string | undefined;
    _literal?: Element | undefined;
    /**
     * The organization or lab that should be responsible for this result.
     */
    performer?: Reference | undefined;
    /**
     * A sequence defined relative to another sequence.
     */
    relative?: MolecularSequenceRelative[] | undefined;
    /**
     * Specimen used for sequencing.
     */
    specimen?: Reference | undefined;
    /**
     * Indicates the subject this sequence is associated too.
     */
    subject?: Reference | undefined;
    /**
     * Amino Acid Sequence/ DNA Sequence / RNA Sequence.
     */
    type?: ('aa' | 'dna' | 'rna') | undefined;
    _type?: Element | undefined;
}
/**
 * Multiple identifiers may exist, either due to duplicate registration, regional rules, needs of different communication technologies, etc.
 */
export interface NamingSystemUniqueId extends BackboneElement {
    /**
     * Indicates whether this identifier ie endorsed by the official owner of the associated naming system.
     */
    authoritative?: boolean | undefined;
    _authoritative?: Element | undefined;
    /**
     * e.g. "must be used in Germany" or "was initially published in error with this value".
     */
    comment?: string | undefined;
    _comment?: Element | undefined;
    /**
     * Within a registry, a given identifier should only be "active" for a single namespace at a time.  (Ideally, an identifier should only ever be associated with a single namespace across all time).
     * The nuance of identifier deprecation is not represented with a separate element. When a determination is made that an identifier should not be used, the validity period end date is updated with the date the identifier is deemed to be deprecated. The identifier should no longer be used once the period end date is reached.
     */
    period?: Period | undefined;
    /**
     * Indicates whether this identifier is the "preferred" identifier of this type.
     */
    preferred?: boolean | undefined;
    _preferred?: Element | undefined;
    /**
     * Different identifier types may be used in different types of communications (OIDs for v3, URIs for FHIR, etc.).  Other includes RUIDs from v3, standard v2 code name strings, etc. Preferred identifiers for the same identifier type SHOULD NOT overlap by period.
     */
    type: 'oid' | 'uuid' | 'uri' | 'v2csmnemonic' | 'other';
    _type?: Element | undefined;
    /**
     * If the value is a URI intended for use as FHIR system identifier, the URI should not contain "\" or "?" or "," since this makes escaping very difficult.
     */
    value: string;
    _value?: Element | undefined;
}
/**
 * A curated namespace that issues unique symbols within that namespace for the identification of concepts, people, devices, etc.  Represents a "System" used within the Identifier and Coding data types.
 */
export interface NamingSystem extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'NamingSystem';
    /**
     * The 'date' element may be more recent than the approval date because of minor changes or editorial corrections.
     */
    approvalDate?: string | undefined;
    _approvalDate?: Element | undefined;
    /**
     * An individiual or organization primarily involved in the creation and maintenance of the NamingSystem.
     */
    author?: ContactDetail[] | undefined;
    /**
     * May be a web site, an email address, a telephone number, etc.
     */
    contact?: ContactDetail[] | undefined;
    /**
     * A copyright statement relating to the naming system and/or its contents. Copyright statements are generally legal restrictions on the use and publishing of the naming system.
     */
    copyright?: string | undefined;
    _copyright?: Element | undefined;
    /**
     * Note that this is not the same as the resource last-modified-date, since the resource may be a secondary representation of the naming system. Additional specific dates may be added as extensions or be found by consulting Provenances associated with past versions of the resource.
     */
    date: string;
    _date?: Element | undefined;
    /**
     * This description can be used to capture details such as comments about misuse, instructions for clinical use and interpretation, literature references, examples from the paper world, etc. It is not a rendering of the naming system as conveyed in the 'text' field of the resource itself. This item SHOULD be populated unless the information is available from context (e.g. the language of the naming system is presumed to be the predominant language in the place the naming system was created).
     */
    description?: string | undefined;
    _description?: Element | undefined;
    /**
     * An individual or organization primarily responsible for internal coherence of the NamingSystem.
     */
    editor?: ContactDetail[] | undefined;
    /**
     * The effective period for a NamingSystem determines when the content is applicable for usage and is independent of publication and review dates. For example, a measure intended to be used for the year 2016 might be published in 2015.
     */
    effectivePeriod?: Period | undefined;
    /**
     * An individual or organization responsible for officially endorsing the NamingSystem for use in some setting.
     */
    endorser?: ContactDetail[] | undefined;
    /**
     * Allows filtering of naming systems that are appropriate for use versus not.
     */
    experimental?: boolean | undefined;
    _experimental?: Element | undefined;
    /**
     * Typically, this is used for identifiers that can go in an HL7 V3 II (instance identifier) data type, and can then identify this naming system outside of FHIR, where it is not possible to use the logical URI.
     */
    identifier?: Identifier[] | undefined;
    /**
     * It may be possible for the naming system to be used in jurisdictions other than those for which it was originally designed or intended.
     */
    jurisdiction?: CodeableConcept[] | undefined;
    /**
     * Indicates the purpose for the naming system - what kinds of things does it make unique?
     */
    kind: 'codesystem' | 'identifier' | 'root';
    _kind?: Element | undefined;
    /**
     * If specified, this date follows the original approval date.
     */
    lastReviewDate?: string | undefined;
    _lastReviewDate?: Element | undefined;
    /**
     * The name is not expected to be globally unique. The name should be a simple alphanumeric type name to ensure that it is machine-processing friendly.The"symbolic name" for an OID would be captured as an extension.
     */
    name: string;
    _name?: Element | undefined;
    /**
     * Usually an organization but may be an individual. The publisher (or steward) of the naming system is the organization or individual primarily responsible for the maintenance and upkeep of the naming system. This is not necessarily the same individual or organization that developed and initially authored the content. The publisher is the primary point of contact for questions or issues with the naming system. This item SHOULD be populated unless the information is available from context.
     */
    publisher?: string | undefined;
    _publisher?: Element | undefined;
    /**
     * This element does not describe the usage of the code system. Instead, it provides traceability of ''why'' the resource is either needed or ''why'' it is defined as it is.  This may be used to point to source materials or specifications that drove the structure of this code system.
     */
    purpose?: string | undefined;
    _purpose?: Element | undefined;
    /**
     * Each related artifact is either an attachment, or a reference to another resource, but not both.
     */
    relatedArtifact?: RelatedArtifact[] | undefined;
    /**
     * This is the primary organization.  Responsibility for some aspects of a namespace may be delegated.
     */
    responsible?: string | undefined;
    _responsible?: Element | undefined;
    /**
     * An individual or organization primarily responsible for review of some aspect of the NamingSystem.
     */
    reviewer?: ContactDetail[] | undefined;
    /**
     * Allows filtering of naming systems that are appropriate for use versus not.
     */
    status: 'draft' | 'active' | 'retired' | 'unknown';
    _status?: Element | undefined;
    /**
     * This title does not need to be machine-processing friendly and may contain punctuation, white-space, etc.
     */
    title?: string | undefined;
    _title?: Element | undefined;
    /**
     * This element provides topical categorization and keywords for the NamingSystem, as opposed to the more structured context-of-use information provided in the useContext element
     */
    topic?: CodeableConcept[] | undefined;
    /**
     * This will most commonly be used for identifier namespaces, but categories could potentially be useful for code systems and authorities as well.
     */
    type?: CodeableConcept | undefined;
    /**
     * Multiple identifiers may exist, either due to duplicate registration, regional rules, needs of different communication technologies, etc.
     */
    uniqueId: NamingSystemUniqueId[];
    /**
     * Can be a urn:uuid: or a urn:oid: but real http: addresses are preferred.  Multiple instances may share the same URL if they have a distinct version.
     * The determination of when to create a new version of a resource (same url, new version) vs. defining a new artifact is up to the author.  Considerations for making this decision are found in [Technical and Business Versions](resource.html#versions).
     * In some cases, the resource can no longer be found at the stated url, but the url itself cannot change. Implementations can use the [meta.source](resource.html#meta) element to indicate where the current master source of the resource can be found.
     */
    url?: string | undefined;
    _url?: Element | undefined;
    /**
     * Provides guidance on the use of the namespace, including the handling of formatting characters, use of upper vs. lower case, etc.
     */
    usage?: string | undefined;
    _usage?: Element | undefined;
    /**
     * When multiple useContexts are specified, there is no expectation that all or any of the contexts apply.
     */
    useContext?: UsageContext[] | undefined;
    /**
     * There may be different naming system instances that have the same identifier but different versions.  The version can be appended to the url in a reference to allow a reference to a particular business version of the naming system with the format [url]|[version]. The version SHOULD NOT contain a '#' - see [Business Version](resource.html#bv-format).
     */
    version?: string | undefined;
    _version?: Element | undefined;
}
/**
 * What food or fluid product or item was consumed.
 */
export interface NutritionIntakeConsumedItem extends BackboneElement {
    /**
     * Quantity of the specified food.
     */
    amount?: Quantity | undefined;
    /**
     * Indicator when a patient is in a setting where it is helpful to know if food was not consumed, such as it was refused, held (as in tube feedings), or otherwise not provided. If a consumption is being recorded from an app, such as MyFitnessPal, this indicator will likely not be used.
     */
    notConsumed?: boolean | undefined;
    _notConsumed?: Element | undefined;
    /**
     * Document the reason the food or fluid was not consumed, such as refused, held, etc.
     */
    notConsumedReason?: CodeableConcept | undefined;
    /**
     * If only a code is specified, then it needs to be a code for a specific product. If more information is required, then the use of a new nutritionProduct resource is recommended.
     */
    nutritionProduct: CodeableReference;
    /**
     * Rate at which enteral feeding was administered.
     */
    rate?: Quantity | undefined;
    /**
     * Scheduled frequency of consumption.
     */
    schedule?: Timing | undefined;
    /**
     * Indicates what a category of item that was consumed: eg., food, fluid, enteral, etc.
     */
    type: CodeableConcept;
}
/**
 * Individual item nutrients are not currently included in the resource and will likely end up as a reference in nutritionProduct to represent the individual items.
 */
export interface NutritionIntakeIngredientLabel extends BackboneElement {
    /**
     * Total amount of nutrient consumed.
     */
    amount: Quantity;
    /**
     * Total nutrient consumed. This could be a macronutrient (protein, fat, carbohydrate), or a vitamin and mineral.
     */
    nutrient: CodeableReference;
}
/**
 * Who performed the intake and how they were involved.
 */
export interface NutritionIntakePerformer extends BackboneElement {
    /**
     * Who performed the intake.
     */
    actor: Reference;
    /**
     * Type of performer.
     */
    function?: CodeableConcept | undefined;
}
/**
 * A record of food or fluid that is being consumed by a patient.   A NutritionIntake may indicate that the patient may be consuming the food or fluid now or has consumed the food or fluid in the past.  The source of this information can be the patient, significant other (such as a family member or spouse), or a clinician.  A common scenario where this information is captured is during the history taking process during a patient visit or stay or through an app that tracks food or fluids consumed.   The consumption information may come from sources such as the patient's memory, from a nutrition label,  or from a clinician documenting observed intake.
 */
export interface NutritionIntake extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'NutritionIntake';
    /**
     * A plan, proposal or order that is fulfilled in whole or in part by this event.
     */
    basedOn?: Reference[] | undefined;
    /**
     * Overall type of nutrition intake.
     */
    code?: CodeableConcept | undefined;
    /**
     * What food or fluid product or item was consumed.
     */
    consumedItem: NutritionIntakeConsumedItem[];
    /**
     * Likely references would be to AllergyIntolerance,  Observation or QuestionnaireAnswers.
     */
    derivedFrom?: Reference[] | undefined;
    /**
     * The encounter that establishes the context for this NutritionIntake.
     */
    encounter?: Reference | undefined;
    /**
     * This is a business identifier, not a resource identifier.
     */
    identifier?: Identifier[] | undefined;
    /**
     * Individual item nutrients are not currently included in the resource and will likely end up as a reference in nutritionProduct to represent the individual items.
     */
    ingredientLabel?: NutritionIntakeIngredientLabel[] | undefined;
    /**
     * Instantiates FHIR protocol or definition.
     */
    instantiatesCanonical?: string[] | undefined;
    _instantiatesCanonical?: Element[] | undefined;
    /**
     * Instantiates external protocol or definition.
     */
    instantiatesUri?: string[] | undefined;
    _instantiatesUri?: Element[] | undefined;
    /**
     * Where the intake occurred.
     */
    location?: Reference | undefined;
    /**
     * Provides extra information about the Nutrition Intake that is not conveyed by the other attributes.
     */
    note?: Annotation[] | undefined;
    /**
     * This attribute reflects the period over which the patient consumed the food fluid and is expected to be populated on the majority of NutritionIntake. If the food or fluid is still being taken and is expected to continue indefinitely at the time the usage is recorded, the "end" date will be omitted.  If the end date is known, then it is included as the "end date".  The date/time attribute supports a variety of dates - year, year/month and exact date.  If something more than this is required, this should be conveyed as text.
     */
    occurrenceDateTime?: string | undefined;
    _occurrenceDateTime?: Element | undefined;
    /**
     * This attribute reflects the period over which the patient consumed the food fluid and is expected to be populated on the majority of NutritionIntake. If the food or fluid is still being taken and is expected to continue indefinitely at the time the usage is recorded, the "end" date will be omitted.  If the end date is known, then it is included as the "end date".  The date/time attribute supports a variety of dates - year, year/month and exact date.  If something more than this is required, this should be conveyed as text.
     */
    occurrencePeriod?: Period | undefined;
    /**
     * A larger event of which this particular event is a component or step.
     */
    partOf?: Reference[] | undefined;
    /**
     * Who performed the intake and how they were involved.
     */
    performer?: NutritionIntakePerformer[] | undefined;
    /**
     * A reason, Condition or observation for why the food or fluid is /was consumed.
     */
    reason?: CodeableReference[] | undefined;
    /**
     * The date when the Nutrition Intake was asserted by the information source.
     */
    recorded?: string | undefined;
    _recorded?: Element | undefined;
    /**
     * The person or organization that provided the information about the consumption of this food or fluid. Note: Use derivedFrom when a NutritionIntake is derived from other resources.
     */
    reportedBoolean?: boolean | undefined;
    _reportedBoolean?: Element | undefined;
    /**
     * The person or organization that provided the information about the consumption of this food or fluid. Note: Use derivedFrom when a NutritionIntake is derived from other resources.
     */
    reportedReference?: Reference | undefined;
    /**
     * NutritionIntake is a statement at a point in time.  The status is only representative at the point when it was asserted.  The value set for NutritionIntake.status contains codes that assert the status of the consumption of the food or fluid by the patient (for example, stopped or on hold) as well as codes that assert the status of the Nutrition Intake itself (for example, entered in error).
     * This element is labeled as a modifier because the status contains codes that mark the resource as not currently valid.
     */
    status:
        | 'preparation'
        | 'in-progress'
        | 'not-done'
        | 'on-hold'
        | 'stopped'
        | 'completed'
        | 'entered-in-error'
        | 'unknown';
    _status?: Element | undefined;
    /**
     * This is generally only used for "exception" statuses such as "not-taken", "on-hold", "cancelled" or "entered-in-error". The reason for performing the event at all is captured in reasonCode, not here.
     */
    statusReason?: CodeableConcept[] | undefined;
    /**
     * The person, animal or group who is/was consuming the food or fluid.
     */
    subject: Reference;
}
/**
 * Schedule information for an oral diet.
 */
export interface NutritionOrderOralDietSchedule extends BackboneElement {
    /**
     * Indicates whether the product is only taken when needed within a specific dosing schedule.
     */
    asNeeded?: boolean | undefined;
    _asNeeded?: Element | undefined;
    /**
     * Indicates whether the product is only taken based on a precondition for taking the product.
     */
    asNeededFor?: CodeableConcept | undefined;
    /**
     * The time period and frequency at which the diet should be given.  The diet should be given for the combination of all schedules if more than one schedule is present.
     */
    timing?: Timing[] | undefined;
}
/**
 * Class that defines the quantity and type of nutrient modifications (for example carbohydrate, fiber or sodium) required for the oral diet.
 */
export interface NutritionOrderOralDietNutrient extends BackboneElement {
    /**
     * The quantity of the specified nutrient to include in diet.
     */
    amount?: Quantity | undefined;
    /**
     * The nutrient that is being modified such as carbohydrate or sodium.
     */
    modifier?: CodeableConcept | undefined;
}
/**
 * Class that describes any texture modifications required for the patient to safely consume various types of solid foods.
 */
export interface NutritionOrderOralDietTexture extends BackboneElement {
    /**
     * Coupled with the `texture.modifier`; could be (All Foods).
     */
    foodType?: CodeableConcept | undefined;
    /**
     * Coupled with the foodType (Meat).
     */
    modifier?: CodeableConcept | undefined;
}
/**
 * Diet given orally in contrast to enteral (tube) feeding.
 */
export interface NutritionOrderOralDiet extends BackboneElement {
    /**
     * The required consistency (e.g. honey-thick, nectar-thick, thin, thickened.) of liquids or fluids served to the patient.
     */
    fluidConsistencyType?: CodeableConcept[] | undefined;
    /**
     * Free text dosage instructions can be used for cases where the instructions are too complex to code.
     */
    instruction?: string | undefined;
    _instruction?: Element | undefined;
    /**
     * Class that defines the quantity and type of nutrient modifications (for example carbohydrate, fiber or sodium) required for the oral diet.
     */
    nutrient?: NutritionOrderOralDietNutrient[] | undefined;
    /**
     * Schedule information for an oral diet.
     */
    schedule?: NutritionOrderOralDietSchedule | undefined;
    /**
     * Class that describes any texture modifications required for the patient to safely consume various types of solid foods.
     */
    texture?: NutritionOrderOralDietTexture[] | undefined;
    /**
     * The kind of diet or dietary restriction such as fiber restricted diet or diabetic diet.
     */
    type?: CodeableConcept[] | undefined;
}
/**
 * Schedule information for a supplement.
 */
export interface NutritionOrderSupplementSchedule extends BackboneElement {
    /**
     * Indicates whether the supplement is only taken when needed within a specific dosing schedule.
     */
    asNeeded?: boolean | undefined;
    _asNeeded?: Element | undefined;
    /**
     * Indicates whether the supplement is only taken based on a precondition for taking the supplement.
     */
    asNeededFor?: CodeableConcept | undefined;
    /**
     * The time period and frequency at which the supplement should be given.  The supplement should be given for the combination of all schedules if more than one schedule is present.
     */
    timing?: Timing[] | undefined;
}
/**
 * Oral nutritional products given in order to add further nutritional value to the patient's diet.
 */
export interface NutritionOrderSupplement extends BackboneElement {
    /**
     * Free text dosage instructions can be used for cases where the instructions are too complex to code.
     */
    instruction?: string | undefined;
    _instruction?: Element | undefined;
    /**
     * The product or brand name of the nutritional supplement such as "Acme Protein Shake".
     */
    productName?: string | undefined;
    _productName?: Element | undefined;
    /**
     * The amount of the nutritional supplement to be given.
     */
    quantity?: Quantity | undefined;
    /**
     * Schedule information for a supplement.
     */
    schedule?: NutritionOrderSupplementSchedule | undefined;
    /**
     * The kind of nutritional supplement product required such as a high protein or pediatric clear liquid supplement.
     */
    type?: CodeableReference | undefined;
}
/**
 * Indicates modular components to be provided in addition or mixed with the base formula.
 */
export interface NutritionOrderEnteralFormulaAdditive extends BackboneElement {
    /**
     * The product or brand name of the type of modular component to be added to the formula.
     */
    productName?: string | undefined;
    _productName?: Element | undefined;
    /**
     * The amount of additive to be given in addition or to be mixed in with the base formula.
     */
    quantity?: Quantity | undefined;
    /**
     * Indicates the type of modular component such as protein, carbohydrate, fat or fiber to be provided in addition to or mixed with the base formula.
     */
    type?: CodeableReference | undefined;
}
/**
 * Schedule information for an enteral formula.
 */
export interface NutritionOrderEnteralFormulaAdministrationSchedule extends BackboneElement {
    /**
     * Indicates whether the enteral formula is only taken when needed within a specific dosing schedule.
     */
    asNeeded?: boolean | undefined;
    _asNeeded?: Element | undefined;
    /**
     * Indicates whether the enteral formula is only taken based on a precondition for taking the enteral formula.
     */
    asNeededFor?: CodeableConcept | undefined;
    /**
     * The time period and frequency at which the enteral formula should be given.  The enteral formula should be given for the combination of all schedules if more than one schedule is present.
     */
    timing?: Timing[] | undefined;
}
/**
 * See implementation notes below for further discussion on how to order continuous vs bolus enteral feeding using this resource.
 */
export interface NutritionOrderEnteralFormulaAdministration extends BackboneElement {
    /**
     * The volume of formula to provide to the patient per the specified administration schedule.
     */
    quantity?: Quantity | undefined;
    /**
     * Ratio is used when the quantity value in the denominator is not "1", otherwise use Quantity. For example, the Ratio datatype is used for "200 mL/4 hrs" versus the Quantity datatype for "50 mL/hr".
     */
    rateQuantity?: Quantity | undefined;
    /**
     * Ratio is used when the quantity value in the denominator is not "1", otherwise use Quantity. For example, the Ratio datatype is used for "200 mL/4 hrs" versus the Quantity datatype for "50 mL/hr".
     */
    rateRatio?: Ratio | undefined;
    /**
     * Schedule information for an enteral formula.
     */
    schedule?: NutritionOrderEnteralFormulaAdministrationSchedule | undefined;
}
/**
 * Feeding provided through the gastrointestinal tract via a tube, catheter, or stoma that delivers nutrition distal to the oral cavity.
 */
export interface NutritionOrderEnteralFormula extends BackboneElement {
    /**
     * Indicates modular components to be provided in addition or mixed with the base formula.
     */
    additive?: NutritionOrderEnteralFormulaAdditive[] | undefined;
    /**
     * See implementation notes below for further discussion on how to order continuous vs bolus enteral feeding using this resource.
     */
    administration?: NutritionOrderEnteralFormulaAdministration[] | undefined;
    /**
     * Free text dosage instructions can be used for cases where the instructions are too complex to code.
     */
    administrationInstruction?: string | undefined;
    _administrationInstruction?: Element | undefined;
    /**
     * The product or brand name of the enteral or infant formula product such as "ACME Adult Standard Formula".
     */
    baseFormulaProductName?: string | undefined;
    _baseFormulaProductName?: Element | undefined;
    /**
     * The type of enteral or infant formula such as an adult standard formula with fiber or a soy-based infant formula.
     */
    baseFormulaType?: CodeableReference | undefined;
    /**
     * The amount of energy (calories) that the formula should provide per specified volume, typically per mL or fluid oz.  For example, an infant may require a formula that provides 24 calories per fluid ounce or an adult may require an enteral formula that provides 1.5 calorie/mL.
     */
    caloricDensity?: Quantity | undefined;
    /**
     * The intended type of device that is to be used for the administration of the enteral formula.
     */
    deliveryDevice?: CodeableReference[] | undefined;
    /**
     * The maximum total quantity of formula that may be administered to a subject over the period of time, e.g. 1440 mL over 24 hours.
     */
    maxVolumeToDeliver?: Quantity | undefined;
    /**
     * The route or physiological path of administration into the patient's gastrointestinal  tract for purposes of providing the formula feeding, e.g. nasogastric tube.
     */
    routeOfAdministration?: CodeableConcept | undefined;
}
/**
 * A request to supply a diet, formula feeding (enteral) or oral nutritional supplement to a patient/resident.
 */
export interface NutritionOrder extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'NutritionOrder';
    /**
     * Information on a patient's food allergies and intolerances to inform healthcare personnel about the type of foods that the patient shouldn't receive or consume.
     */
    allergyIntolerance?: Reference[] | undefined;
    /**
     * A plan or request that is fulfilled in whole or in part by this nutrition order.
     */
    basedOn?: Reference[] | undefined;
    /**
     * The date and time that this nutrition order was requested.
     */
    dateTime: string;
    _dateTime?: Element | undefined;
    /**
     * An encounter that provides additional information about the healthcare context in which this request is made.
     */
    encounter?: Reference | undefined;
    /**
     * Feeding provided through the gastrointestinal tract via a tube, catheter, or stoma that delivers nutrition distal to the oral cavity.
     */
    enteralFormula?: NutritionOrderEnteralFormula | undefined;
    /**
     * Information on a patient's food allergies, intolerances and preferences to inform healthcare personnel about the type  of foods that the patient shouldn't receive or consume.
     */
    excludeFoodModifier?: CodeableConcept[] | undefined;
    /**
     * Information on a patient's food preferences that inform healthcare personnel about the food that the patient should receive or consume.
     */
    foodPreferenceModifier?: CodeableConcept[] | undefined;
    /**
     * The Identifier.type element can be to indicate filler vs. placer if needed.  This is explained in further detail [here](servicerequest.html#notes).
     */
    identifier?: Identifier[] | undefined;
    /**
     * The URL pointing to a protocol, guideline, orderset or other definition that is adhered to in whole or in part by this NutritionOrder.
     */
    instantiates?: string[] | undefined;
    _instantiates?: Element[] | undefined;
    /**
     * Note: This is a business identifier, not a resource identifier (see [discussion](resource.html#identifiers)).  It is best practice for the identifier to only appear on a single resource instance, however business practices may occasionally dictate that multiple resource instances with the same identifier can exist - possibly even with different resource types.  For example, multiple Patient and a Person resource instance might share the same social insurance number.
     */
    instantiatesCanonical?: string[] | undefined;
    _instantiatesCanonical?: Element[] | undefined;
    /**
     * This might be an HTML page, PDF, etc. or could just be a non-resolvable URI identifier.
     */
    instantiatesUri?: string[] | undefined;
    _instantiatesUri?: Element[] | undefined;
    /**
     * When resources map to this element, they are free to define as many codes as necessary to cover their space and will map to "proposal, plan or order".  Can have multiple codes that map to one of these.  E.g. "original order", "encoded order", "reflex order" would all map to "order".  Expectation is that the set of codes is mutually exclusive or a strict all-encompassing hierarchy.
     */
    intent:
        | 'proposal'
        | 'plan'
        | 'directive'
        | 'order'
        | 'original-order'
        | 'reflex-order'
        | 'filler-order'
        | 'instance-order'
        | 'option';
    _intent?: Element | undefined;
    /**
     * This element SHALL NOT be used to supply free text instructions for the diet which are represented in the `.oralDiet.instruction`, `supplement.instruction`, or `enteralFormula.administrationInstruction` elements.
     */
    note?: Annotation[] | undefined;
    /**
     * Diet given orally in contrast to enteral (tube) feeding.
     */
    oralDiet?: NutritionOrderOralDiet | undefined;
    /**
     * The practitioner that holds legal responsibility for ordering the diet, nutritional supplement, or formula feedings.
     */
    orderer?: Reference | undefined;
    /**
     * This modifier is used to convey whether a food item is allowed to be brought in by the patient and/or family.  If set to true, indicates that the receiving system does not need to supply the food item.
     */
    outsideFoodAllowed?: boolean | undefined;
    _outsideFoodAllowed?: Element | undefined;
    /**
     * The specified desired performer of the nutrition order.
     */
    performer?: CodeableReference[] | undefined;
    /**
     * Indicates how quickly the Nutrition Order should be addressed with respect to other        requests.
     */
    priority?: ('routine' | 'urgent' | 'asap' | 'stat') | undefined;
    _priority?: Element | undefined;
    /**
     * Typically the system placing the order sets the status to "requested". Thereafter, the order is maintained by the receiver that updates the status as the request is handled.  This element is labeled as a modifier because the status contains codes that mark the resource as not currently valid.
     */
    status: 'draft' | 'active' | 'on-hold' | 'revoked' | 'completed' | 'entered-in-error' | 'unknown';
    _status?: Element | undefined;
    /**
     * The person or set of individuals who needs the nutrition order for an oral diet, nutritional supplement and/or enteral or formula feeding.
     */
    subject: Reference;
    /**
     * Oral nutritional products given in order to add further nutritional value to the patient's diet.
     */
    supplement?: NutritionOrderSupplement[] | undefined;
    /**
     * Information to support fulfilling (i.e. dispensing or administering) of the nutrition,        for example, patient height and weight).
     */
    supportingInformation?: Reference[] | undefined;
}
/**
 * Note: This is a business identifier, not a resource identifier (see [discussion](resource.html#identifiers)).  It is best practice for the identifier to only appear on a single resource instance, however business practices may occasionally dictate that multiple resource instances with the same identifier can exist - possibly even with different resource types.  For example, multiple Patient and a Person resource instance might share the same social insurance number.
 */
export interface NutritionProductNutrient extends BackboneElement {
    /**
     * The amount of nutrient expressed in one or more units: X per pack / per serving / per dose.
     */
    amount?: Ratio[] | undefined;
    /**
     * The (relevant) nutrients in the product.
     */
    item?: CodeableReference | undefined;
}
/**
 * Ingredients contained in this product.
 */
export interface NutritionProductIngredient extends BackboneElement {
    /**
     * The amount of ingredient that is in the product.
     */
    amount?: Ratio[] | undefined;
    /**
     * The ingredient contained in the product.
     */
    item: CodeableReference;
}
/**
 * Specifies descriptive properties of the nutrition product.
 */
export interface NutritionProductCharacteristic extends BackboneElement {
    /**
     * A code specifying which characteristic of the product is being described (for example, colour, shape).
     */
    type: CodeableConcept;
    /**
     * The description should be provided as a CodeableConcept, SimpleQuantity or an image.  The description can be a string only when these others are not available.
     */
    valueCodeableConcept?: CodeableConcept | undefined;
    /**
     * The description should be provided as a CodeableConcept, SimpleQuantity or an image.  The description can be a string only when these others are not available.
     */
    valueString?: string | undefined;
    _valueString?: Element | undefined;
    /**
     * The description should be provided as a CodeableConcept, SimpleQuantity or an image.  The description can be a string only when these others are not available.
     */
    valueQuantity?: Quantity | undefined;
    /**
     * The description should be provided as a CodeableConcept, SimpleQuantity or an image.  The description can be a string only when these others are not available.
     */
    valueBase64Binary?: string | undefined;
    _valueBase64Binary?: Element | undefined;
    /**
     * The description should be provided as a CodeableConcept, SimpleQuantity or an image.  The description can be a string only when these others are not available.
     */
    valueAttachment?: Attachment | undefined;
    /**
     * The description should be provided as a CodeableConcept, SimpleQuantity or an image.  The description can be a string only when these others are not available.
     */
    valueBoolean?: boolean | undefined;
    _valueBoolean?: Element | undefined;
}
/**
 * Conveys instance-level information about this product item. One or several physical, countable instances or occurrences of the product.
 */
export interface NutritionProductInstance extends BackboneElement {
    /**
     * Necessary to support mandatory requirements for traceability from donor/source to recipient and vice versa, while also satisfying donor anonymity requirements. The element is defined consistently across BiologicallyDerivedProduct, NutritionProduct, and Device.  The identifier references an event that links to a single biological entity such as a blood donor, or to multiple biological entities (e.g. when the product is an embryo or a pooled platelet product).  A single biologicalSourceEvent identifier may appear on multiple products of many types derived from a single donation event or source extraction.  As an example, a single donation event may provide 2 kidneys and a liver for organ transplantation, 2 corneas for eye surgery, heart valves and arterial tissue for cardiovascular surgery, multiple skin grafts, tendons, multiple shaped bone grafts and a large number of bone putty/paste products; and each of them may be assigned to the same biological source event identifier.
     */
    biologicalSourceEvent?: Identifier | undefined;
    /**
     * The time after which the product is no longer expected to be in proper condition, or its use is not advised or not allowed.
     */
    expiry?: string | undefined;
    _expiry?: Element | undefined;
    /**
     * The identifier for the physical instance, typically a serial number or manufacturer number.
     */
    identifier?: Identifier[] | undefined;
    /**
     * The identification of the batch or lot of the product.
     */
    lotNumber?: string | undefined;
    _lotNumber?: Element | undefined;
    /**
     * The name for the specific product.
     */
    name?: string | undefined;
    _name?: Element | undefined;
    /**
     * The amount of items or instances that the resource considers, for instance when referring to 2 identical units together.
     */
    quantity?: Quantity | undefined;
    /**
     * The time after which the product is no longer expected to be in proper condition, or its use is not advised or not allowed.
     */
    useBy?: string | undefined;
    _useBy?: Element | undefined;
}
/**
 * A food or supplement that is consumed by patients.
 */
export interface NutritionProduct extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'NutritionProduct';
    /**
     * Nutrition products can have different classifications - according to its nutritional properties, preparation methods, etc.
     */
    category?: CodeableConcept[] | undefined;
    /**
     * Specifies descriptive properties of the nutrition product.
     */
    characteristic?: NutritionProductCharacteristic[] | undefined;
    /**
     * The code assigned to the product, for example a USDA NDB number, a USDA FDC ID number, or a Langual code.
     */
    code?: CodeableConcept | undefined;
    /**
     * Ingredients contained in this product.
     */
    ingredient?: NutritionProductIngredient[] | undefined;
    /**
     * Conveys instance-level information about this product item. One or several physical, countable instances or occurrences of the product.
     */
    instance?: NutritionProductInstance[] | undefined;
    /**
     * Allergens that are known or suspected to be a part of this nutrition product.
     */
    knownAllergen?: CodeableReference[] | undefined;
    /**
     * The organisation (manufacturer, representative or legal authorisation holder) that is responsible for the device.
     */
    manufacturer?: Reference[] | undefined;
    /**
     * Comments made about the product.
     */
    note?: Annotation[] | undefined;
    /**
     * Note: This is a business identifier, not a resource identifier (see [discussion](resource.html#identifiers)).  It is best practice for the identifier to only appear on a single resource instance, however business practices may occasionally dictate that multiple resource instances with the same identifier can exist - possibly even with different resource types.  For example, multiple Patient and a Person resource instance might share the same social insurance number.
     */
    nutrient?: NutritionProductNutrient[] | undefined;
    /**
     * Unknown does not represent "other" - one of the defined statuses must apply.  Unknown is used when the authoring system is not sure what the current status is.
     */
    status: 'active' | 'inactive' | 'entered-in-error';
    _status?: Element | undefined;
}
/**
 * Identifies the observation(s) that triggered the performance of this observation.
 */
export interface ObservationTriggeredBy extends BackboneElement {
    /**
     * Reference to the triggering observation.
     */
    observation: Reference;
    /**
     * Provides the reason why this observation was performed as a result of the observation(s) referenced.
     */
    reason?: string | undefined;
    _reason?: Element | undefined;
    /**
     * The type of trigger.
     * Reflex | Repeat | Re-run.
     */
    type: 'reflex' | 'repeat' | 're-run';
    _type?: Element | undefined;
}
/**
 * Most observations only have one generic reference range. Systems MAY choose to restrict to only supplying the relevant reference range based on knowledge about the patient (e.g., specific to the patient's age, gender, weight and other factors), but this might not be possible or appropriate. Whenever more than one reference range is supplied, the differences between them SHOULD be provided in the reference range and/or age properties.
 */
export interface ObservationReferenceRange extends BackboneElement {
    /**
     * The age at which this reference range is applicable. This is a neonatal age (e.g. number of weeks at term) if the meaning says so.
     */
    age?: Range | undefined;
    /**
     * This SHOULD be populated if there is more than one range.  If this element is not present then the normal population is assumed.
     */
    appliesTo?: CodeableConcept[] | undefined;
    /**
     * The value of the high bound of the reference range.  The high bound of the reference range endpoint is inclusive of the value (e.g.  reference range is >=5 - <=9). If the high bound is omitted,  it is assumed to be meaningless (e.g. reference range is >= 2.3).
     */
    high?: Quantity | undefined;
    /**
     * The value of the low bound of the reference range.  The low bound of the reference range endpoint is inclusive of the value (e.g.  reference range is >=5 - <=9). If the low bound is omitted,  it is assumed to be meaningless (e.g. reference range is <=2.3).
     */
    low?: Quantity | undefined;
    /**
     * The value of the normal value of the reference range.
     */
    normalValue?: CodeableConcept | undefined;
    /**
     * Text based reference range in an observation which may be used when a quantitative range is not appropriate for an observation.  An example would be a reference value of "Negative" or a list or table of "normals".
     */
    text?: string | undefined;
    _text?: Element | undefined;
    /**
     * This SHOULD be populated if there is more than one range.  If this element is not present then the normal range is assumed.
     */
    type?: CodeableConcept | undefined;
}
/**
 * For a discussion on the ways Observations can be assembled in groups together see [Notes](observation.html#notes) below.
 */
export interface ObservationComponent extends BackboneElement {
    /**
     * *All* code-value and  component.code-component.value pairs need to be taken into account to correctly understand the meaning of the observation.
     */
    code: CodeableConcept;
    /**
     * "Null" or exceptional values can be represented two ways in FHIR Observations.  One way is to simply include them in the value set and represent the exceptions in the value.  For example, measurement values for a serology test could be  "detected", "not detected", "inconclusive", or  "test not done".
     * The alternate way is to use the value element for actual observations and use the explicit dataAbsentReason element to record exceptional values.  For example, the dataAbsentReason code "error" could be used when the measurement was not completed.  Because of these options, use-case agreements are required to interpret general observations for exceptional values.
     */
    dataAbsentReason?: CodeableConcept | undefined;
    /**
     * Historically used for laboratory results (known as 'abnormal flag' ),  its use extends to other use cases where coded interpretations  are relevant.  Often reported as one or more simple compact codes this element is often placed adjacent to the result value in reports and flow sheets to signal the meaning/normalcy status of the result.
     */
    interpretation?: CodeableConcept[] | undefined;
    /**
     * Most observations only have one generic reference range. Systems MAY choose to restrict to only supplying the relevant reference range based on knowledge about the patient (e.g., specific to the patient's age, gender, weight and other factors), but this might not be possible or appropriate. Whenever more than one reference range is supplied, the differences between them SHOULD be provided in the reference range and/or age properties.
     */
    referenceRange?: ObservationReferenceRange[] | undefined;
    /**
     * Used when observation has a set of component observations:
     * -  An observation may have both a value (e.g. an  Apgar score) and component observations (the observations from which the Apgar score was derived).
     * -  If a value is present, the datatype for this element should be determined by the `code`.
     * -  *CodeableConcept* with just a text would be used instead of a string if the field was usually coded, or if the type associated with the `code` defines a coded value.
     * -  *Attachment* is used if the observation result value is a binary file such as an image.  If the observation result value is derived from the binary file (for example 'X' detected and here is the the proof in this image), the binary file may be directly represented using *DocumentReference* and referenced by `derivedFrom`.
     * - For additional guidance, see the [Notes section](observation.html#notes) below.
     */
    valueQuantity?: Quantity | undefined;
    /**
     * Used when observation has a set of component observations:
     * -  An observation may have both a value (e.g. an  Apgar score) and component observations (the observations from which the Apgar score was derived).
     * -  If a value is present, the datatype for this element should be determined by the `code`.
     * -  *CodeableConcept* with just a text would be used instead of a string if the field was usually coded, or if the type associated with the `code` defines a coded value.
     * -  *Attachment* is used if the observation result value is a binary file such as an image.  If the observation result value is derived from the binary file (for example 'X' detected and here is the the proof in this image), the binary file may be directly represented using *DocumentReference* and referenced by `derivedFrom`.
     * - For additional guidance, see the [Notes section](observation.html#notes) below.
     */
    valueCodeableConcept?: CodeableConcept | undefined;
    /**
     * Used when observation has a set of component observations:
     * -  An observation may have both a value (e.g. an  Apgar score) and component observations (the observations from which the Apgar score was derived).
     * -  If a value is present, the datatype for this element should be determined by the `code`.
     * -  *CodeableConcept* with just a text would be used instead of a string if the field was usually coded, or if the type associated with the `code` defines a coded value.
     * -  *Attachment* is used if the observation result value is a binary file such as an image.  If the observation result value is derived from the binary file (for example 'X' detected and here is the the proof in this image), the binary file may be directly represented using *DocumentReference* and referenced by `derivedFrom`.
     * - For additional guidance, see the [Notes section](observation.html#notes) below.
     */
    valueString?: string | undefined;
    _valueString?: Element | undefined;
    /**
     * Used when observation has a set of component observations:
     * -  An observation may have both a value (e.g. an  Apgar score) and component observations (the observations from which the Apgar score was derived).
     * -  If a value is present, the datatype for this element should be determined by the `code`.
     * -  *CodeableConcept* with just a text would be used instead of a string if the field was usually coded, or if the type associated with the `code` defines a coded value.
     * -  *Attachment* is used if the observation result value is a binary file such as an image.  If the observation result value is derived from the binary file (for example 'X' detected and here is the the proof in this image), the binary file may be directly represented using *DocumentReference* and referenced by `derivedFrom`.
     * - For additional guidance, see the [Notes section](observation.html#notes) below.
     */
    valueBoolean?: boolean | undefined;
    _valueBoolean?: Element | undefined;
    /**
     * Used when observation has a set of component observations:
     * -  An observation may have both a value (e.g. an  Apgar score) and component observations (the observations from which the Apgar score was derived).
     * -  If a value is present, the datatype for this element should be determined by the `code`.
     * -  *CodeableConcept* with just a text would be used instead of a string if the field was usually coded, or if the type associated with the `code` defines a coded value.
     * -  *Attachment* is used if the observation result value is a binary file such as an image.  If the observation result value is derived from the binary file (for example 'X' detected and here is the the proof in this image), the binary file may be directly represented using *DocumentReference* and referenced by `derivedFrom`.
     * - For additional guidance, see the [Notes section](observation.html#notes) below.
     */
    valueInteger?: number | undefined;
    /**
     * Used when observation has a set of component observations:
     * -  An observation may have both a value (e.g. an  Apgar score) and component observations (the observations from which the Apgar score was derived).
     * -  If a value is present, the datatype for this element should be determined by the `code`.
     * -  *CodeableConcept* with just a text would be used instead of a string if the field was usually coded, or if the type associated with the `code` defines a coded value.
     * -  *Attachment* is used if the observation result value is a binary file such as an image.  If the observation result value is derived from the binary file (for example 'X' detected and here is the the proof in this image), the binary file may be directly represented using *DocumentReference* and referenced by `derivedFrom`.
     * - For additional guidance, see the [Notes section](observation.html#notes) below.
     */
    valueRange?: Range | undefined;
    /**
     * Used when observation has a set of component observations:
     * -  An observation may have both a value (e.g. an  Apgar score) and component observations (the observations from which the Apgar score was derived).
     * -  If a value is present, the datatype for this element should be determined by the `code`.
     * -  *CodeableConcept* with just a text would be used instead of a string if the field was usually coded, or if the type associated with the `code` defines a coded value.
     * -  *Attachment* is used if the observation result value is a binary file such as an image.  If the observation result value is derived from the binary file (for example 'X' detected and here is the the proof in this image), the binary file may be directly represented using *DocumentReference* and referenced by `derivedFrom`.
     * - For additional guidance, see the [Notes section](observation.html#notes) below.
     */
    valueRatio?: Ratio | undefined;
    /**
     * Used when observation has a set of component observations:
     * -  An observation may have both a value (e.g. an  Apgar score) and component observations (the observations from which the Apgar score was derived).
     * -  If a value is present, the datatype for this element should be determined by the `code`.
     * -  *CodeableConcept* with just a text would be used instead of a string if the field was usually coded, or if the type associated with the `code` defines a coded value.
     * -  *Attachment* is used if the observation result value is a binary file such as an image.  If the observation result value is derived from the binary file (for example 'X' detected and here is the the proof in this image), the binary file may be directly represented using *DocumentReference* and referenced by `derivedFrom`.
     * - For additional guidance, see the [Notes section](observation.html#notes) below.
     */
    valueSampledData?: SampledData | undefined;
    /**
     * Used when observation has a set of component observations:
     * -  An observation may have both a value (e.g. an  Apgar score) and component observations (the observations from which the Apgar score was derived).
     * -  If a value is present, the datatype for this element should be determined by the `code`.
     * -  *CodeableConcept* with just a text would be used instead of a string if the field was usually coded, or if the type associated with the `code` defines a coded value.
     * -  *Attachment* is used if the observation result value is a binary file such as an image.  If the observation result value is derived from the binary file (for example 'X' detected and here is the the proof in this image), the binary file may be directly represented using *DocumentReference* and referenced by `derivedFrom`.
     * - For additional guidance, see the [Notes section](observation.html#notes) below.
     */
    valueTime?: string | undefined;
    _valueTime?: Element | undefined;
    /**
     * Used when observation has a set of component observations:
     * -  An observation may have both a value (e.g. an  Apgar score) and component observations (the observations from which the Apgar score was derived).
     * -  If a value is present, the datatype for this element should be determined by the `code`.
     * -  *CodeableConcept* with just a text would be used instead of a string if the field was usually coded, or if the type associated with the `code` defines a coded value.
     * -  *Attachment* is used if the observation result value is a binary file such as an image.  If the observation result value is derived from the binary file (for example 'X' detected and here is the the proof in this image), the binary file may be directly represented using *DocumentReference* and referenced by `derivedFrom`.
     * - For additional guidance, see the [Notes section](observation.html#notes) below.
     */
    valueDateTime?: string | undefined;
    _valueDateTime?: Element | undefined;
    /**
     * Used when observation has a set of component observations:
     * -  An observation may have both a value (e.g. an  Apgar score) and component observations (the observations from which the Apgar score was derived).
     * -  If a value is present, the datatype for this element should be determined by the `code`.
     * -  *CodeableConcept* with just a text would be used instead of a string if the field was usually coded, or if the type associated with the `code` defines a coded value.
     * -  *Attachment* is used if the observation result value is a binary file such as an image.  If the observation result value is derived from the binary file (for example 'X' detected and here is the the proof in this image), the binary file may be directly represented using *DocumentReference* and referenced by `derivedFrom`.
     * - For additional guidance, see the [Notes section](observation.html#notes) below.
     */
    valuePeriod?: Period | undefined;
    /**
     * Used when observation has a set of component observations:
     * -  An observation may have both a value (e.g. an  Apgar score) and component observations (the observations from which the Apgar score was derived).
     * -  If a value is present, the datatype for this element should be determined by the `code`.
     * -  *CodeableConcept* with just a text would be used instead of a string if the field was usually coded, or if the type associated with the `code` defines a coded value.
     * -  *Attachment* is used if the observation result value is a binary file such as an image.  If the observation result value is derived from the binary file (for example 'X' detected and here is the the proof in this image), the binary file may be directly represented using *DocumentReference* and referenced by `derivedFrom`.
     * - For additional guidance, see the [Notes section](observation.html#notes) below.
     */
    valueAttachment?: Attachment | undefined;
}
/**
 * Measurements and simple assertions made about a patient, device or other subject.
 */
export interface Observation extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'Observation';
    /**
     * A plan, proposal or order that is fulfilled in whole or in part by this event.  For example, a MedicationRequest may require a patient to have laboratory test performed before  it is dispensed.
     */
    basedOn?: Reference[] | undefined;
    /**
     * Only used if not implicit in code found in Observation.code.  In many systems, this may be represented as a related observation instead of an inline component.
     */
    bodySite?: CodeableConcept | undefined;
    /**
     * Only used if not implicit in code found in Observation.code or bodySite is used. In many systems, this may be represented as a related observation instead of an inline component.
     */
    bodyStructure?: Reference | undefined;
    /**
     * In addition to the required category valueset, this element allows various categorization schemes based on the owner’s definition of the category and effectively multiple categories can be used at once.  The level of granularity is defined by the category concepts in the value set.
     */
    category?: CodeableConcept[] | undefined;
    /**
     * *All* code-value and, if present, component.code-component.value pairs need to be taken into account to correctly understand the meaning of the observation.
     */
    code: CodeableConcept;
    /**
     * For a discussion on the ways Observations can be assembled in groups together see [Notes](observation.html#notes) below.
     */
    component?: ObservationComponent[] | undefined;
    /**
     * Null or exceptional values can be represented two ways in FHIR Observations.  One way is to simply include them in the value set and represent the exceptions in the value.  For example, measurement values for a serology test could be  "detected", "not detected", "inconclusive", or  "specimen unsatisfactory".
     * The alternate way is to use the value element for actual observations and use the explicit dataAbsentReason element to record exceptional values.  For example, the dataAbsentReason code "error" could be used when the measurement was not completed. Note that an observation may only be reported if there are values to report. For example differential cell counts values may be reported only when > 0.  Because of these options, use-case agreements are required to interpret general observations for null or exceptional values.
     */
    dataAbsentReason?: CodeableConcept | undefined;
    /**
     * All the reference choices that are listed in this element can represent clinical observations and other measurements that may be the source for a derived value.  The most common reference will be another Observation.  For a discussion on the ways Observations can assembled in groups together, see [Notes](observation.html#obsgrouping) below.
     */
    derivedFrom?: Reference[] | undefined;
    /**
     * Note that this is not meant to represent a device involved in the transmission of the result, e.g., a gateway.  Such devices may be documented using the Provenance resource where relevant.
     */
    device?: Reference | undefined;
    /**
     * At least a date should be present unless this observation is a historical report.  For recording imprecise or "fuzzy" times (For example, a blood glucose measurement taken "after breakfast") use the [Timing](datatypes.html#timing) datatype which allow the measurement to be tied to regular life events.
     */
    effectiveDateTime?: string | undefined;
    _effectiveDateTime?: Element | undefined;
    /**
     * At least a date should be present unless this observation is a historical report.  For recording imprecise or "fuzzy" times (For example, a blood glucose measurement taken "after breakfast") use the [Timing](datatypes.html#timing) datatype which allow the measurement to be tied to regular life events.
     */
    effectivePeriod?: Period | undefined;
    /**
     * At least a date should be present unless this observation is a historical report.  For recording imprecise or "fuzzy" times (For example, a blood glucose measurement taken "after breakfast") use the [Timing](datatypes.html#timing) datatype which allow the measurement to be tied to regular life events.
     */
    effectiveTiming?: Timing | undefined;
    /**
     * At least a date should be present unless this observation is a historical report.  For recording imprecise or "fuzzy" times (For example, a blood glucose measurement taken "after breakfast") use the [Timing](datatypes.html#timing) datatype which allow the measurement to be tied to regular life events.
     */
    effectiveInstant?: string | undefined;
    _effectiveInstant?: Element | undefined;
    /**
     * This will typically be the encounter the event occurred within, but some events may be initiated prior to or after the official completion of an encounter but still be tied to the context of the encounter (e.g. pre-admission laboratory tests).
     */
    encounter?: Reference | undefined;
    /**
     * Typically, an observation is made about the subject - a patient, or group of patients, location, or device - and the distinction between the subject and what is directly measured for an observation is specified in the observation code itself ( e.g., "Blood Glucose") and does not need to be represented separately using this element.  Use `specimen` if a reference to a specimen is required.  If a code is required instead of a resource use either  `bodysite` for bodysites or the standard extension [focusCode](extension-observation-focuscode.html).
     */
    focus?: Reference[] | undefined;
    /**
     * When using this element, an observation will typically have either a value or a set of related resources, although both may be present in some cases.  For a discussion on the ways Observations can assembled in groups together, see [Notes](observation.html#obsgrouping) below.  Note that a system may calculate results from [QuestionnaireResponse](questionnaireresponse.html)  into a final score and represent the score as an Observation.
     */
    hasMember?: Reference[] | undefined;
    /**
     * A unique identifier assigned to this observation.
     */
    identifier?: Identifier[] | undefined;
    /**
     * ObservationDefinition can be referenced by its canonical url using instantiatesCanonical, or by a name or an identifier using the appropriate sub-elements of instantiatesReference.
     */
    instantiatesCanonical?: string | undefined;
    _instantiatesCanonical?: Element | undefined;
    /**
     * ObservationDefinition can be referenced by its canonical url using instantiatesCanonical, or by a name or an identifier using the appropriate sub-elements of instantiatesReference.
     */
    instantiatesReference?: Reference | undefined;
    /**
     * Historically used for laboratory results (known as 'abnormal flag' ),  its use extends to other use cases where coded interpretations  are relevant.  Often reported as one or more simple compact codes this element is often placed adjacent to the result value in reports and flow sheets to signal the meaning/normalcy status of the result.
     */
    interpretation?: CodeableConcept[] | undefined;
    /**
     * For Observations that don't require review and verification, it may be the same as the [`lastUpdated` ](resource-definitions.html#Meta.lastUpdated) time of the resource itself.  For Observations that do require review and verification for certain updates, it might not be the same as the `lastUpdated` time of the resource itself due to a non-clinically significant update that doesn't require the new version to be reviewed and verified again.
     */
    issued?: string | undefined;
    _issued?: Element | undefined;
    /**
     * Only used if not implicit in code for Observation.code.
     */
    method?: CodeableConcept | undefined;
    /**
     * May include general statements about the observation, or statements about significant, unexpected or unreliable results values, or information about its source when relevant to its interpretation.
     */
    note?: Annotation[] | undefined;
    /**
     * To link an Observation to an Encounter use `encounter`.  See the  [Notes](observation.html#obsgrouping) below for guidance on referencing another Observation.
     */
    partOf?: Reference[] | undefined;
    /**
     * Who was responsible for asserting the observed value as "true".
     */
    performer?: Reference[] | undefined;
    /**
     * Most observations only have one generic reference range. Systems MAY choose to restrict to only supplying the relevant reference range based on knowledge about the patient (e.g., specific to the patient's age, gender, weight and other factors), but this might not be possible or appropriate. Whenever more than one reference range is supplied, the differences between them SHOULD be provided in the reference range and/or age properties.
     */
    referenceRange?: ObservationReferenceRange[] | undefined;
    /**
     * Should only be used if not implicit in code found in `Observation.code`.  Observations are not made on specimens themselves; they are made on a subject, but in many cases by the means of a specimen. Note that although specimens are often involved, they are not always tracked and reported explicitly. Also note that observation resources may be used in contexts that track the specimen explicitly (e.g. Diagnostic Report).
     */
    specimen?: Reference | undefined;
    /**
     * This element is labeled as a modifier because the status contains codes that mark the resource as not currently valid.
     */
    status:
        | 'registered'
        | 'preliminary'
        | 'final'
        | 'amended'
        | 'corrected'
        | 'cancelled'
        | 'entered-in-error'
        | 'unknown';
    _status?: Element | undefined;
    /**
     * One would expect this element to be a cardinality of 1..1. The only circumstance in which the subject can be missing is when the observation is made by a device that does not know the patient. In this case, the observation SHALL be matched to a patient through some context/channel matching technique, and at this point, the observation should be updated. The subject of an Observation may in some cases be a procedure.  This supports the regulatory inspection use case where observations are captured during inspections of a procedure that is being performed (independent of any particular patient or whether patient related at all).
     */
    subject?: Reference | undefined;
    /**
     * Identifies the observation(s) that triggered the performance of this observation.
     */
    triggeredBy?: ObservationTriggeredBy[] | undefined;
    /**
     * - An observation may have:
     *   1.  a single value here
     *   1.  both a value and a set of related or component values
     *   1.  only a set of related or component values.
     * -  If a value is present, the datatype for this element should be determined by the `code`.
     * -  *CodeableConcept* with just a text would be used instead of a string if the field was usually coded, or if the type associated with the `code` defines a coded value.
     * -  *Attachment* is used if the observation result value is a binary file such as an image.  If the observation result value is derived from the binary file (for example 'X' detected and here is the the proof in this image), the binary file may be directly represented using *DocumentReference* and referenced by `derivedFrom`.
     * - For additional guidance, see the [Notes section](observation.html#notes) below.
     */
    valueQuantity?: Quantity | undefined;
    /**
     * - An observation may have:
     *   1.  a single value here
     *   1.  both a value and a set of related or component values
     *   1.  only a set of related or component values.
     * -  If a value is present, the datatype for this element should be determined by the `code`.
     * -  *CodeableConcept* with just a text would be used instead of a string if the field was usually coded, or if the type associated with the `code` defines a coded value.
     * -  *Attachment* is used if the observation result value is a binary file such as an image.  If the observation result value is derived from the binary file (for example 'X' detected and here is the the proof in this image), the binary file may be directly represented using *DocumentReference* and referenced by `derivedFrom`.
     * - For additional guidance, see the [Notes section](observation.html#notes) below.
     */
    valueCodeableConcept?: CodeableConcept | undefined;
    /**
     * - An observation may have:
     *   1.  a single value here
     *   1.  both a value and a set of related or component values
     *   1.  only a set of related or component values.
     * -  If a value is present, the datatype for this element should be determined by the `code`.
     * -  *CodeableConcept* with just a text would be used instead of a string if the field was usually coded, or if the type associated with the `code` defines a coded value.
     * -  *Attachment* is used if the observation result value is a binary file such as an image.  If the observation result value is derived from the binary file (for example 'X' detected and here is the the proof in this image), the binary file may be directly represented using *DocumentReference* and referenced by `derivedFrom`.
     * - For additional guidance, see the [Notes section](observation.html#notes) below.
     */
    valueString?: string | undefined;
    _valueString?: Element | undefined;
    /**
     * - An observation may have:
     *   1.  a single value here
     *   1.  both a value and a set of related or component values
     *   1.  only a set of related or component values.
     * -  If a value is present, the datatype for this element should be determined by the `code`.
     * -  *CodeableConcept* with just a text would be used instead of a string if the field was usually coded, or if the type associated with the `code` defines a coded value.
     * -  *Attachment* is used if the observation result value is a binary file such as an image.  If the observation result value is derived from the binary file (for example 'X' detected and here is the the proof in this image), the binary file may be directly represented using *DocumentReference* and referenced by `derivedFrom`.
     * - For additional guidance, see the [Notes section](observation.html#notes) below.
     */
    valueBoolean?: boolean | undefined;
    _valueBoolean?: Element | undefined;
    /**
     * - An observation may have:
     *   1.  a single value here
     *   1.  both a value and a set of related or component values
     *   1.  only a set of related or component values.
     * -  If a value is present, the datatype for this element should be determined by the `code`.
     * -  *CodeableConcept* with just a text would be used instead of a string if the field was usually coded, or if the type associated with the `code` defines a coded value.
     * -  *Attachment* is used if the observation result value is a binary file such as an image.  If the observation result value is derived from the binary file (for example 'X' detected and here is the the proof in this image), the binary file may be directly represented using *DocumentReference* and referenced by `derivedFrom`.
     * - For additional guidance, see the [Notes section](observation.html#notes) below.
     */
    valueInteger?: number | undefined;
    /**
     * - An observation may have:
     *   1.  a single value here
     *   1.  both a value and a set of related or component values
     *   1.  only a set of related or component values.
     * -  If a value is present, the datatype for this element should be determined by the `code`.
     * -  *CodeableConcept* with just a text would be used instead of a string if the field was usually coded, or if the type associated with the `code` defines a coded value.
     * -  *Attachment* is used if the observation result value is a binary file such as an image.  If the observation result value is derived from the binary file (for example 'X' detected and here is the the proof in this image), the binary file may be directly represented using *DocumentReference* and referenced by `derivedFrom`.
     * - For additional guidance, see the [Notes section](observation.html#notes) below.
     */
    valueRange?: Range | undefined;
    /**
     * - An observation may have:
     *   1.  a single value here
     *   1.  both a value and a set of related or component values
     *   1.  only a set of related or component values.
     * -  If a value is present, the datatype for this element should be determined by the `code`.
     * -  *CodeableConcept* with just a text would be used instead of a string if the field was usually coded, or if the type associated with the `code` defines a coded value.
     * -  *Attachment* is used if the observation result value is a binary file such as an image.  If the observation result value is derived from the binary file (for example 'X' detected and here is the the proof in this image), the binary file may be directly represented using *DocumentReference* and referenced by `derivedFrom`.
     * - For additional guidance, see the [Notes section](observation.html#notes) below.
     */
    valueRatio?: Ratio | undefined;
    /**
     * - An observation may have:
     *   1.  a single value here
     *   1.  both a value and a set of related or component values
     *   1.  only a set of related or component values.
     * -  If a value is present, the datatype for this element should be determined by the `code`.
     * -  *CodeableConcept* with just a text would be used instead of a string if the field was usually coded, or if the type associated with the `code` defines a coded value.
     * -  *Attachment* is used if the observation result value is a binary file such as an image.  If the observation result value is derived from the binary file (for example 'X' detected and here is the the proof in this image), the binary file may be directly represented using *DocumentReference* and referenced by `derivedFrom`.
     * - For additional guidance, see the [Notes section](observation.html#notes) below.
     */
    valueSampledData?: SampledData | undefined;
    /**
     * - An observation may have:
     *   1.  a single value here
     *   1.  both a value and a set of related or component values
     *   1.  only a set of related or component values.
     * -  If a value is present, the datatype for this element should be determined by the `code`.
     * -  *CodeableConcept* with just a text would be used instead of a string if the field was usually coded, or if the type associated with the `code` defines a coded value.
     * -  *Attachment* is used if the observation result value is a binary file such as an image.  If the observation result value is derived from the binary file (for example 'X' detected and here is the the proof in this image), the binary file may be directly represented using *DocumentReference* and referenced by `derivedFrom`.
     * - For additional guidance, see the [Notes section](observation.html#notes) below.
     */
    valueTime?: string | undefined;
    _valueTime?: Element | undefined;
    /**
     * - An observation may have:
     *   1.  a single value here
     *   1.  both a value and a set of related or component values
     *   1.  only a set of related or component values.
     * -  If a value is present, the datatype for this element should be determined by the `code`.
     * -  *CodeableConcept* with just a text would be used instead of a string if the field was usually coded, or if the type associated with the `code` defines a coded value.
     * -  *Attachment* is used if the observation result value is a binary file such as an image.  If the observation result value is derived from the binary file (for example 'X' detected and here is the the proof in this image), the binary file may be directly represented using *DocumentReference* and referenced by `derivedFrom`.
     * - For additional guidance, see the [Notes section](observation.html#notes) below.
     */
    valueDateTime?: string | undefined;
    _valueDateTime?: Element | undefined;
    /**
     * - An observation may have:
     *   1.  a single value here
     *   1.  both a value and a set of related or component values
     *   1.  only a set of related or component values.
     * -  If a value is present, the datatype for this element should be determined by the `code`.
     * -  *CodeableConcept* with just a text would be used instead of a string if the field was usually coded, or if the type associated with the `code` defines a coded value.
     * -  *Attachment* is used if the observation result value is a binary file such as an image.  If the observation result value is derived from the binary file (for example 'X' detected and here is the the proof in this image), the binary file may be directly represented using *DocumentReference* and referenced by `derivedFrom`.
     * - For additional guidance, see the [Notes section](observation.html#notes) below.
     */
    valuePeriod?: Period | undefined;
    /**
     * - An observation may have:
     *   1.  a single value here
     *   1.  both a value and a set of related or component values
     *   1.  only a set of related or component values.
     * -  If a value is present, the datatype for this element should be determined by the `code`.
     * -  *CodeableConcept* with just a text would be used instead of a string if the field was usually coded, or if the type associated with the `code` defines a coded value.
     * -  *Attachment* is used if the observation result value is a binary file such as an image.  If the observation result value is derived from the binary file (for example 'X' detected and here is the the proof in this image), the binary file may be directly represented using *DocumentReference* and referenced by `derivedFrom`.
     * - For additional guidance, see the [Notes section](observation.html#notes) below.
     */
    valueAttachment?: Attachment | undefined;
}
/**
 * Characteristics for quantitative results of observations conforming to this ObservationDefinition.
 */
export interface ObservationDefinitionQuantitativeDetails extends BackboneElement {
    /**
     * Factor for converting value expressed with primary unit to value expressed with secondary unit.
     */
    conversionFactor?: number | undefined;
    /**
     * Secondary unit used to report quantitative results of observations conforming to this ObservationDefinition.
     */
    customaryUnit?: CodeableConcept | undefined;
    /**
     * Number of digits after decimal separator when the results of such observations are of type Quantity.
     */
    decimalPrecision?: number | undefined;
    /**
     * Primary unit used to report quantitative results of observations conforming to this ObservationDefinition.
     */
    unit?: CodeableConcept | undefined;
}
/**
 * A set of qualified values associated with a context and a set of conditions -  provides a range for quantitative and ordinal observations and a collection of value sets for qualitative observations.
 */
export interface ObservationDefinitionQualifiedValue extends BackboneElement {
    /**
     * The set of abnormal coded results for qualitative observations  that match the criteria of this set of qualified values.
     */
    abnormalCodedValueSet?: string | undefined;
    _abnormalCodedValueSet?: Element | undefined;
    /**
     * Some analytes vary greatly over age.
     */
    age?: Range | undefined;
    /**
     * If this element is not present then the global population is assumed.
     */
    appliesTo?: CodeableConcept[] | undefined;
    /**
     * Text based condition for which the the set of qualified values is valid.
     */
    condition?: string | undefined;
    _condition?: Element | undefined;
    /**
     * A concept defining the context for this set of qualified values.
     */
    context?: CodeableConcept | undefined;
    /**
     * The set of critical coded results for qualitative observations  that match the criteria of this set of qualified values.
     */
    criticalCodedValueSet?: string | undefined;
    _criticalCodedValueSet?: Element | undefined;
    /**
     * The gender this  set of qualified values applies to.
     */
    gender?: ('male' | 'female' | 'other' | 'unknown') | undefined;
    _gender?: Element | undefined;
    /**
     * The gestational age this  set of qualified values applies to.
     */
    gestationalAge?: Range | undefined;
    /**
     * The set of normal coded results for qualitative observations  that match the criteria of this set of qualified values.
     */
    normalCodedValueSet?: string | undefined;
    _normalCodedValueSet?: Element | undefined;
    /**
     * The range of values defined for continuous or ordinal observations that match the criteria of this set of qualified values.
     */
    range?: Range | undefined;
    /**
     * The category of range of values for continuous or ordinal observations that match the criteria of this set of qualified values.
     */
    rangeCategory?: ('reference' | 'critical' | 'absolute') | undefined;
    _rangeCategory?: Element | undefined;
    /**
     * The set of valid coded results for qualitative observations  that match the criteria of this set of qualified values.
     */
    validCodedValueSet?: string | undefined;
    _validCodedValueSet?: Element | undefined;
}
/**
 * Some observations have multiple component observations, expressed as separate code value pairs.
 */
export interface ObservationDefinitionComponent extends BackboneElement {
    /**
     * Describes what will be observed.
     */
    code: CodeableConcept;
    /**
     * The data types allowed for the value element of the instance of this component observations.
     */
    permittedDataType?:
        | (
              | 'Quantity'
              | 'CodeableConcept'
              | 'string'
              | 'boolean'
              | 'integer'
              | 'Range'
              | 'Ratio'
              | 'SampledData'
              | 'time'
              | 'dateTime'
              | 'Period'
          )[]
        | undefined;
    _permittedDataType?: Element[] | undefined;
    /**
     * A set of qualified values associated with a context and a set of conditions -  provides a range for quantitative and ordinal observations and a collection of value sets for qualitative observations.
     */
    qualifiedValue?: ObservationDefinitionQualifiedValue[] | undefined;
    /**
     * Characteristics for quantitative results of this observation.
     */
    quantitativeDetails?: ObservationDefinitionQuantitativeDetails | undefined;
}
/**
 * Set of definitional characteristics for a kind of observation or measurement produced or consumed by an orderable health care service.
 */
export interface ObservationDefinition extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'ObservationDefinition';
    /**
     * The date may be more recent than the approval date because of minor changes / editorial corrections.
     */
    approvalDate?: string | undefined;
    _approvalDate?: Element | undefined;
    /**
     * Only used if the defined observation is to be made directly on a body part and if this body part is not implied by code found in ObservationDefinition.code. Not used for in vitro diagnostic observations.
     * Example: 14975008 |Forearm structure|.
     */
    bodySite?: CodeableConcept | undefined;
    /**
     * This element allows various categorization schemes based on the owner’s definition of the category and effectively multiple categories can be used for one instance of ObservationDefinition. The level of granularity is defined by the category concepts in the value set.
     */
    category?: CodeableConcept[] | undefined;
    /**
     * Describes what will be observed. Sometimes this is called the observation "name".
     */
    code: CodeableConcept;
    /**
     * Some observations have multiple component observations, expressed as separate code value pairs.
     */
    component?: ObservationDefinitionComponent[] | undefined;
    /**
     * May be a web site, an email address, a telephone number, etc.
     */
    contact?: ContactDetail[] | undefined;
    /**
     * Copyright statement relating to the ObservationDefinition and/or its contents. Copyright statements are generally legal restrictions on the use and publishing of the ObservationDefinition.
     */
    copyright?: string | undefined;
    _copyright?: Element | undefined;
    /**
     * Note that this is not the same as the resource last-modified-date, since the resource may be a secondary representation of the ObservationDefinition. Additional specific dates may be added as extensions or be found by consulting Provenances associated with past versions of the resource.
     */
    date?: string | undefined;
    _date?: Element | undefined;
    /**
     * The canonical URL pointing to another FHIR-defined ObservationDefinition that is adhered to in whole or in part by this definition.
     */
    derivedFromCanonical?: string[] | undefined;
    _derivedFromCanonical?: Element[] | undefined;
    /**
     * The URL pointing to an externally-defined observation definition, guideline or other definition that is adhered to in whole or in part by this definition.
     */
    derivedFromUri?: string[] | undefined;
    _derivedFromUri?: Element[] | undefined;
    /**
     * This description can be used to capture details such as comments about misuse, instructions for clinical use and interpretation, literature references, examples from the paper world, etc. It is not a rendering of the module as conveyed in the text field of the resource itself. This item SHOULD be populated unless the information is available from context.
     */
    description?: string | undefined;
    _description?: Element | undefined;
    /**
     * When multiple occurrences of device are present, they are to be combined with a logical OR: at least one of the specified devices is used to produce the observation.
     */
    device?: Reference[] | undefined;
    /**
     * The effective period for an ObservationDefinition determines when the content is applicable for usage and is independent of publication and review dates. For example, an observation intended to be used for the year 2021 might be published in 2020.
     */
    effectivePeriod?: Period | undefined;
    /**
     * Allows filtering of ObservationDefinition that are appropriate for use vs. not.
     */
    experimental?: boolean | undefined;
    _experimental?: Element | undefined;
    /**
     * This ObservationDefinition defines a group  observation (e.g. a battery, a panel of tests, a set of vital sign measurements) that includes the target as a member of the group.
     */
    hasMember?: Reference[] | undefined;
    /**
     * This is a business identifier, not a resource identifier. It is best practice for the identifier to only appear on a single resource instance, however business practices may occasionally dictate that multiple resource instances with the same identifier can exist - possibly even with different resource types.
     */
    identifier?: Identifier | undefined;
    /**
     * It may be possible for the ObservationDefinition to be used in jurisdictions other than those for which it was originally designed or intended.
     */
    jurisdiction?: CodeableConcept[] | undefined;
    /**
     * If specified, this is usually after the approval date.
     */
    lastReviewDate?: string | undefined;
    _lastReviewDate?: Element | undefined;
    /**
     * Only used if not implicit in observation code
     * Example: 702659008 |Automated count technique|.
     */
    method?: CodeableConcept | undefined;
    /**
     * An example of observation allowing multiple results is "bacteria identified by culture". Conversely, the measurement of a potassium level allows a single result.
     */
    multipleResultsAllowed?: boolean | undefined;
    _multipleResultsAllowed?: Element | undefined;
    /**
     * The name is not expected to be globally unique. The name should be a simple alphanumeric type name to ensure that it is machine-processing friendly.
     */
    name?: string | undefined;
    _name?: Element | undefined;
    /**
     * The type of individual/organization/device that is expected to act upon instances of this definition.
     */
    performerType?: CodeableConcept | undefined;
    /**
     * The data types allowed for the value element of the instance observations conforming to this ObservationDefinition.
     */
    permittedDataType?:
        | (
              | 'Quantity'
              | 'CodeableConcept'
              | 'string'
              | 'boolean'
              | 'integer'
              | 'Range'
              | 'Ratio'
              | 'SampledData'
              | 'time'
              | 'dateTime'
              | 'Period'
          )[]
        | undefined;
    _permittedDataType?: Element[] | undefined;
    /**
     * The preferred name to be used when reporting the results of observations conforming to this ObservationDefinition.
     */
    preferredReportName?: string | undefined;
    _preferredReportName?: Element | undefined;
    /**
     * Helps establish the "authority/credibility" of the ObservationDefinition. May also allow for contact.
     */
    publisher?: Reference | undefined;
    /**
     * This element does not describe the usage of the ObservationDefinition. Rather it is for traceability of ''why'' the resource is either needed or ''why'' it is defined as it is. This may be used to point to source materials or specifications that drove the structure of this ObservationDefinition.
     */
    purpose?: string | undefined;
    _purpose?: Element | undefined;
    /**
     * A set of qualified values associated with a context and a set of conditions -  provides a range for quantitative and ordinal observations and a collection of value sets for qualitative observations.
     */
    qualifiedValue?: ObservationDefinitionQualifiedValue[] | undefined;
    /**
     * Characteristics for quantitative results of observations conforming to this ObservationDefinition.
     */
    quantitativeDetails?: ObservationDefinitionQuantitativeDetails | undefined;
    /**
     * Only used for in vitro observations. When multiple occurrences of specimen are present, they are to be combined with a logical AND: all specified specimens  are needed for the observation.
     */
    specimen?: Reference[] | undefined;
    /**
     * A nominal state-transition diagram can be found in the [[definition.html#statemachine | Definition pattern]] documentation
     * Unknown does not represent "other" - one of the defined statuses must apply. Unknown is used when the authoring system is not sure what the current status is.
     */
    status: 'draft' | 'active' | 'retired' | 'unknown';
    _status?: Element | undefined;
    /**
     * Examples: person, animal, device, air, surface ….
     */
    subject?: CodeableConcept[] | undefined;
    /**
     * A short, descriptive, user-friendly title for the ObservationDefinition.
     */
    title?: string | undefined;
    _title?: Element | undefined;
    /**
     * Can be a urn:uuid: or a urn:oid:, but real http: addresses are preferred. This is the URI that will be used when making canonical references to this resource.
     */
    url?: string | undefined;
    _url?: Element | undefined;
    /**
     * When multiple usageContexts are specified, there is no expectation for whether all or any of the contexts apply.
     */
    useContext?: UsageContext[] | undefined;
    /**
     * There may be multiple different instances of a ObservationDefinition that have the same identifier but different versions.
     */
    version?: string | undefined;
    _version?: Element | undefined;
}
/**
 * Binds to a value set if this parameter is coded (code, Coding, CodeableConcept).
 */
export interface OperationDefinitionParameterBinding extends BackboneElement {
    /**
     * For further discussion, see [Using Terminologies](terminologies.html).
     */
    strength: 'required' | 'extensible' | 'preferred' | 'example';
    _strength?: Element | undefined;
    /**
     * For value sets with a referenceResource, the display can contain the value set description.  The reference may be version-specific or not.
     */
    valueSet: string;
    _valueSet?: Element | undefined;
}
/**
 * Resolution applies if the referenced parameter exists.
 */
export interface OperationDefinitionParameterReferencedFrom extends BackboneElement {
    /**
     * The name of the parameter or dot-separated path of parameter names pointing to the resource parameter that is expected to contain a reference to this resource.
     */
    source: string;
    _source?: Element | undefined;
    /**
     * The id of the element in the referencing resource that is expected to resolve to this resource.
     */
    sourceId?: string | undefined;
    _sourceId?: Element | undefined;
}
/**
 * Query Definitions only have one output parameter, named "result". This might not be described, but can be to allow a profile to be defined.
 */
export interface OperationDefinitionParameter extends BackboneElement {
    /**
     * In previous versions of FHIR, there was an extension for this: http://hl7.org/fhir/StructureDefinition/operationdefinition-allowed-type
     */
    allowedType?: string[] | undefined;
    _allowedType?: Element[] | undefined;
    /**
     * Binds to a value set if this parameter is coded (code, Coding, CodeableConcept).
     */
    binding?: OperationDefinitionParameterBinding | undefined;
    /**
     * Describes the meaning or use of this parameter.
     */
    documentation?: string | undefined;
    _documentation?: Element | undefined;
    /**
     * The maximum number of times this element is permitted to appear in the request or response.
     */
    max: string;
    _max?: Element | undefined;
    /**
     * The minimum number of times this parameter SHALL appear in the request or response.
     */
    min: number;
    /**
     * This name must be a token (start with a letter in a..z, and only contain letters, numerals, and underscore. Note that for search parameters (type = string, with a search type), the name may be altered by the search modifiers.
     */
    name: string;
    _name?: Element | undefined;
    /**
     * Query Definitions only have one output parameter, named "result". This might not be described, but can be to allow a profile to be defined.
     */
    part?: OperationDefinitionParameter[] | undefined;
    /**
     * Resolution applies if the referenced parameter exists.
     */
    referencedFrom?: OperationDefinitionParameterReferencedFrom[] | undefined;
    /**
     * If present, indicates that the parameter applies when the operation is being invoked at the specified level.
     */
    scope?: ('instance' | 'type' | 'system')[] | undefined;
    _scope?: Element[] | undefined;
    /**
     * Parameters that are search parameter usually - but not always - are also defined for use with general searches, and function in the operation to filter the set of resources in scope for the operation just like the do when searching. The searchType describes how the server processes the value. For an example, see the [Evaluate Measure Operation](measure-operation-evaluate-measure.html)
     */
    searchType?:
        | ('number' | 'date' | 'string' | 'token' | 'reference' | 'composite' | 'quantity' | 'uri' | 'special')
        | undefined;
    _searchType?: Element | undefined;
    /**
     * Often, these profiles are the base definitions from the spec (e.g. http://hl7.org/fhir/StructureDefinition/Patient).
     */
    targetProfile?: string[] | undefined;
    _targetProfile?: Element[] | undefined;
    /**
     * if there is no stated parameter, then the parameter is a multi-part parameter type and must have at least one part defined.
     */
    type?: string | undefined;
    _type?: Element | undefined;
    /**
     * If a parameter name is used for both an input and an output parameter, the parameter should be defined twice.
     */
    use: 'in' | 'out';
    _use?: Element | undefined;
}
/**
 * The combinations are suggestions as to which sets of parameters to use together, but the combinations are not intended to be authoritative.
 */
export interface OperationDefinitionOverload extends BackboneElement {
    /**
     * Comments to go on overload.
     */
    comment?: string | undefined;
    _comment?: Element | undefined;
    /**
     * Name of parameter to include in overload.
     */
    parameterName?: string[] | undefined;
    _parameterName?: Element[] | undefined;
}
/**
 * A formal computable definition of an operation (on the RESTful interface) or a named query (using the search interaction).
 */
export interface OperationDefinition extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'OperationDefinition';
    /**
     * What http methods can be used for the operation depends on the .affectsState value and whether the input parameters are primitive or complex:
     * 1. Servers SHALL support POST method for all operations.
     * 2. Servers SHALL support the GET method when the operation has affectsState = false and all required parameters for the operation are primitive.
     */
    affectsState?: boolean | undefined;
    _affectsState?: Element | undefined;
    /**
     * A constrained profile can make optional parameters required or not used, introduce new parameters (required or optional), and clarify documentation.
     */
    base?: string | undefined;
    _base?: Element | undefined;
    /**
     * For maximum compatibility, use only lowercase ASCII characters. Note that HL7 will never define multiple operations with the same code. See [Naming Rules & Guidelines](https://confluence.hl7.org/display/FHIR/Guide+to+Designing+Resources#GuidetoDesigningResources-NamingRules&Guidelines) for the internal HL7 rules around codes, which are useful additional advice for all implementers
     */
    code: string;
    _code?: Element | undefined;
    /**
     * Additional information about how to use this operation or named query.
     */
    comment?: string | undefined;
    _comment?: Element | undefined;
    /**
     * May be a web site, an email address, a telephone number, etc.
     */
    contact?: ContactDetail[] | undefined;
    /**
     * ...
     */
    copyright?: string | undefined;
    _copyright?: Element | undefined;
    /**
     * The (c) symbol should NOT be included in this string. It will be added by software when rendering the notation. Full details about licensing, restrictions, warrantees, etc. goes in the more general 'copyright' element.
     */
    copyrightLabel?: string | undefined;
    _copyrightLabel?: Element | undefined;
    /**
     * Note that this is not the same as the resource last-modified-date, since the resource may be a secondary representation of the operation definition. Additional specific dates may be added as extensions or be found by consulting Provenances associated with past versions of the resource.
     */
    date?: string | undefined;
    _date?: Element | undefined;
    /**
     * This description can be used to capture details such as comments about misuse, instructions for clinical use and interpretation, literature references, examples from the paper world, etc. It is not a rendering of the operation definition as conveyed in the 'text' field of the resource itself. This item SHOULD be populated unless the information is available from context (e.g. the language of the operation definition is presumed to be the predominant language in the place the operation definition was created).
     */
    description?: string | undefined;
    _description?: Element | undefined;
    /**
     * Allows filtering of operation definitions that are appropriate for use versus not.
     */
    experimental?: boolean | undefined;
    _experimental?: Element | undefined;
    /**
     * If present the profile shall not conflict with what is specified in the parameters in the operation definition (max/min etc.), though it may provide additional constraints. The constraints expressed in the profile apply whether the operation is invoked by a POST wih parameters or not.
     */
    inputProfile?: string | undefined;
    _inputProfile?: Element | undefined;
    /**
     * Indicates whether this operation can be invoked on a particular instance of one of the given types.
     */
    instance: boolean;
    _instance?: Element | undefined;
    /**
     * It may be possible for the operation definition to be used in jurisdictions other than those for which it was originally designed or intended.
     */
    jurisdiction?: CodeableConcept[] | undefined;
    /**
     * Named queries are invoked differently, and have different capabilities.
     */
    kind: 'operation' | 'query';
    _kind?: Element | undefined;
    /**
     * The name is not expected to be globally unique. The name should be a simple alphanumeric type name to ensure that it is machine-processing friendly.
     */
    name: string;
    _name?: Element | undefined;
    /**
     * If present the profile shall not conflict with what is specified in the parameters in the operation definition (max/min etc.), though it may provide additional constraints. The constraints expressed in the profile apply whether the operation is invoked by a POST wih parameters or not.
     */
    outputProfile?: string | undefined;
    _outputProfile?: Element | undefined;
    /**
     * The combinations are suggestions as to which sets of parameters to use together, but the combinations are not intended to be authoritative.
     */
    overload?: OperationDefinitionOverload[] | undefined;
    /**
     * Query Definitions only have one output parameter, named "result". This might not be described, but can be to allow a profile to be defined.
     */
    parameter?: OperationDefinitionParameter[] | undefined;
    /**
     * Usually an organization but may be an individual. The publisher (or steward) of the operation definition is the organization or individual primarily responsible for the maintenance and upkeep of the operation definition. This is not necessarily the same individual or organization that developed and initially authored the content. The publisher is the primary point of contact for questions or issues with the operation definition. This item SHOULD be populated unless the information is available from context.
     */
    publisher?: string | undefined;
    _publisher?: Element | undefined;
    /**
     * This element does not describe the usage of the operation definition. Instead, it provides traceability of ''why'' the resource is either needed or ''why'' it is defined as it is.  This may be used to point to source materials or specifications that drove the structure of this operation definition.
     */
    purpose?: string | undefined;
    _purpose?: Element | undefined;
    /**
     * If the type is an abstract resource ("Resource" or "DomainResource") then the operation can be invoked on any concrete specialization.
     */
    resource?: string[] | undefined;
    _resource?: Element[] | undefined;
    /**
     * Allows filtering of operation definitions that are appropriate for use versus not.
     */
    status: 'draft' | 'active' | 'retired' | 'unknown';
    _status?: Element | undefined;
    /**
     * Indicates whether this operation or named query can be invoked at the system level (e.g. without needing to choose a resource type for the context).
     */
    system: boolean;
    _system?: Element | undefined;
    /**
     * This name does not need to be machine-processing friendly and may contain punctuation, white-space, etc.
     */
    title?: string | undefined;
    _title?: Element | undefined;
    /**
     * Indicates whether this operation or named query can be invoked at the resource type level for any given resource type level (e.g. without needing to choose a specific resource id for the context).
     */
    type: boolean;
    _type?: Element | undefined;
    /**
     * Can be a urn:uuid: or a urn:oid: but real http: addresses are preferred.  Multiple instances may share the same URL if they have a distinct version.
     * The determination of when to create a new version of a resource (same url, new version) vs. defining a new artifact is up to the author.  Considerations for making this decision are found in [Technical and Business Versions](resource.html#versions).
     * In some cases, the resource can no longer be found at the stated url, but the url itself cannot change. Implementations can use the [meta.source](resource.html#meta) element to indicate where the current master source of the resource can be found.
     */
    url?: string | undefined;
    _url?: Element | undefined;
    /**
     * When multiple useContexts are specified, there is no expectation that all or any of the contexts apply.
     */
    useContext?: UsageContext[] | undefined;
    /**
     * There may be different operation definition instances that have the same identifier but different versions.  The version can be appended to the url in a reference to allow a reference to a particular business version of the operation definition with the format [url]|[version]. The version SHOULD NOT contain a '#' - see [Business Version](resource.html#bv-format).
     */
    version?: string | undefined;
    _version?: Element | undefined;
    /**
     * If set as a string, this is a FHIRPath expression that has two additional context variables passed in - %version1 and %version2 and will return a negative number if version1 is newer, a positive number if version2 and a 0 if the version ordering can't be successfully be determined.
     */
    versionAlgorithmString?: string | undefined;
    _versionAlgorithmString?: Element | undefined;
    /**
     * If set as a string, this is a FHIRPath expression that has two additional context variables passed in - %version1 and %version2 and will return a negative number if version1 is newer, a positive number if version2 and a 0 if the version ordering can't be successfully be determined.
     */
    versionAlgorithmCoding?: Coding | undefined;
}
/**
 * An error, warning, or information message that results from a system action.
 */
export interface OperationOutcomeIssue extends BackboneElement {
    /**
     * Code values should align with the severity.  For example, a code of ```forbidden``` generally wouldn't make sense with a severity of ```information``` or ```warning```.  Similarly, a code of ```informational``` would generally not make sense with a severity of ```fatal``` or ```error```.  However, there are no strict rules about what severities must be used with which codes.  For example, ```code-invalid``` might be a ```warning``` or ```error```, depending on the context
     */
    code: string;
    _code?: Element | undefined;
    /**
     * A human readable description of the error issue SHOULD be placed in details.text.
     */
    details?: CodeableConcept | undefined;
    /**
     * This may be a description of how a value is erroneous, a stack dump to help trace the issue or other troubleshooting information.
     */
    diagnostics?: string | undefined;
    _diagnostics?: Element | undefined;
    /**
     * The root of the FHIRPath is the resource or bundle that generated OperationOutcome.  Each FHIRPath SHALL resolve to a single node.
     */
    expression?: string[] | undefined;
    _expression?: Element[] | undefined;
    /**
     * The root of the XPath is the resource or bundle that generated OperationOutcome.  Each XPath SHALL resolve to a single node.  This element is deprecated, and is being replaced by expression.
     */
    location?: string[] | undefined;
    _location?: Element[] | undefined;
    /**
     * Indicates whether the issue indicates a variation from successful processing.
     */
    severity: 'fatal' | 'error' | 'warning' | 'information';
    _severity?: Element | undefined;
}
/**
 * A collection of error, warning, or information messages that result from a system action.
 */
export interface OperationOutcome extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'OperationOutcome';
    /**
     * An error, warning, or information message that results from a system action.
     */
    issue: OperationOutcomeIssue[];
}
/**
 * The official certifications, accreditations, training, designations and licenses that authorize and/or otherwise endorse the provision of care by the organization.
 * For example, an approval to provide a type of services issued by a certifying body (such as the US Joint Commission) to an organization.
 */
export interface OrganizationQualification extends BackboneElement {
    /**
     * Coded representation of the qualification.
     */
    code: CodeableConcept;
    /**
     * An identifier allocated to this qualification for this organization.
     */
    identifier?: Identifier[] | undefined;
    /**
     * Organization that regulates and issues the qualification.
     */
    issuer?: Reference | undefined;
    /**
     * Period during which the qualification is valid.
     */
    period?: Period | undefined;
}
/**
 * A formally or informally recognized grouping of people or organizations formed for the purpose of achieving some form of collective action.  Includes companies, institutions, corporations, departments, community groups, healthcare practice groups, payer/insurer, etc.
 */
export interface Organization extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'Organization';
    /**
     * This active flag is not intended to be used to mark an organization as temporarily closed or under construction. Instead the Location(s) within the Organization should have the suspended status. If further details of the reason for the suspension are required, then an extension on this element should be used.
     * This element is labeled as a modifier because it may be used to mark that the resource was created in error.
     */
    active?: boolean | undefined;
    _active?: Element | undefined;
    /**
     * There are no dates associated with the alias/historic names, as this is not intended to track when names were used, but to assist in searching so that older names can still result in identifying the organization.
     */
    alias?: string[] | undefined;
    _alias?: Element[] | undefined;
    /**
     * The address/telecom use code 'home' are not to be used. Note that these contacts are not the contact details of people who provide the service (that would be through PractitionerRole), these are official contacts for the Organization itself for specific purposes. E.g. Mailing Addresses, Billing Addresses, Contact numbers for Booking or Billing Enquiries, general web address, web address for online bookings etc.
     */
    contact?: ExtendedContactDetail[] | undefined;
    /**
     * Description of the organization, which helps provide additional general context on the organization to ensure that the correct organization is selected.
     */
    description?: string | undefined;
    _description?: Element | undefined;
    /**
     * Technical endpoints providing access to services operated for the organization.
     */
    endpoint?: Reference[] | undefined;
    /**
     * Identifier for the organization that is used to identify the organization across multiple disparate systems.
     */
    identifier?: Identifier[] | undefined;
    /**
     * If the name of an organization changes, consider putting the old name in the alias column so that it can still be located through searches.
     */
    name?: string | undefined;
    _name?: Element | undefined;
    /**
     * The organization of which this organization forms a part.
     */
    partOf?: Reference | undefined;
    /**
     * The official certifications, accreditations, training, designations and licenses that authorize and/or otherwise endorse the provision of care by the organization.
     * For example, an approval to provide a type of services issued by a certifying body (such as the US Joint Commission) to an organization.
     */
    qualification?: OrganizationQualification[] | undefined;
    /**
     * Organizations can be corporations, wards, sections, clinical teams, government departments, etc. Note that code is generally a classifier of the type of organization; in many applications, codes are used to identity a particular organization (say, ward) as opposed to another of the same type - these are identifiers, not codes
     * When considering if multiple types are appropriate, you should evaluate if child organizations would be a more appropriate use of the concept, as different types likely are in different sub-areas of the organization. This is most likely to be used where type values have orthogonal values, such as a religious, academic and medical center.
     * We expect that some jurisdictions will profile this optionality to be a single cardinality.
     */
    type?: CodeableConcept[] | undefined;
}
/**
 * Defines an affiliation/assotiation/relationship between 2 distinct organizations, that is not a part-of relationship/sub-division relationship.
 */
export interface OrganizationAffiliation extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'OrganizationAffiliation';
    /**
     * If this value is false, you may refer to the period to see when the role was in active use. If there is no period specified, no inference can be made about when it was active.
     */
    active?: boolean | undefined;
    _active?: Element | undefined;
    /**
     * Definition of the role the participatingOrganization plays in the association.
     */
    code?: CodeableConcept[] | undefined;
    /**
     * The contact details of communication devices available at the participatingOrganization relevant to this Affiliation.
     */
    contact?: ExtendedContactDetail[] | undefined;
    /**
     * Technical endpoints providing access to services operated for this role.
     */
    endpoint?: Reference[] | undefined;
    /**
     * Healthcare services provided through the role.
     */
    healthcareService?: Reference[] | undefined;
    /**
     * Business identifiers that are specific to this role.
     */
    identifier?: Identifier[] | undefined;
    /**
     * The location(s) at which the role occurs.
     */
    location?: Reference[] | undefined;
    /**
     * e.g. Commonly used for Health Insurance provider networks.
     */
    network?: Reference[] | undefined;
    /**
     * Organization where the role is available (primary organization/has members).
     */
    organization?: Reference | undefined;
    /**
     * The Participating Organization provides/performs the role(s) defined by the code to the Primary Organization (e.g. providing services or is a member of).
     */
    participatingOrganization?: Reference | undefined;
    /**
     * The period during which the participatingOrganization is affiliated with the primary organization.
     */
    period?: Period | undefined;
    /**
     * Specific specialty of the participatingOrganization in the context of the role.
     */
    specialty?: CodeableConcept[] | undefined;
}
/**
 * The legal status of supply of the packaged item as classified by the regulator.
 */
export interface PackagedProductDefinitionLegalStatusOfSupply extends BackboneElement {
    /**
     * The actual status of supply. Conveys in what situation this package type may be supplied for use.
     */
    code?: CodeableConcept | undefined;
    /**
     * The place where the legal status of supply applies. When not specified, this indicates it is unknown in this context.
     */
    jurisdiction?: CodeableConcept | undefined;
}
/**
 * General characteristics of this item.
 */
export interface PackagedProductDefinitionPackagingProperty extends BackboneElement {
    /**
     * A code expressing the type of characteristic.
     */
    type: CodeableConcept;
    /**
     * A value for the characteristic.
     */
    valueCodeableConcept?: CodeableConcept | undefined;
    /**
     * A value for the characteristic.
     */
    valueQuantity?: Quantity | undefined;
    /**
     * A value for the characteristic.
     */
    valueDate?: string | undefined;
    _valueDate?: Element | undefined;
    /**
     * A value for the characteristic.
     */
    valueBoolean?: boolean | undefined;
    _valueBoolean?: Element | undefined;
    /**
     * A value for the characteristic.
     */
    valueAttachment?: Attachment | undefined;
}
/**
 * The item(s) within the packaging.
 */
export interface PackagedProductDefinitionPackagingContainedItem extends BackboneElement {
    /**
     * The number of this type of item within this packaging.
     */
    amount?: Quantity | undefined;
    /**
     * The actual item(s) of medication, as manufactured, or a device (typically, but not necessarily, a co-packaged one), or other medically related item (such as food, biologicals, raw materials, medical fluids, gases etc.), as contained in the package. This also allows another whole packaged product to be included, which is solely for the case where a package of other entire packages is wanted - such as a wholesale or distribution pack (for layers within one package, use PackagedProductDefinition.packaging.packaging).
     */
    item: CodeableReference;
}
/**
 * A packaging item, as a container for medically related items, possibly with other packaging items within, or a packaging component, such as bottle cap (which is not a device or a medication manufactured item).
 */
export interface PackagedProductDefinitionPackaging extends BackboneElement {
    /**
     * A possible alternate material for this part of the packaging, that is allowed to be used instead of the usual material (e.g. different types of plastic for a blister sleeve).
     */
    alternateMaterial?: CodeableConcept[] | undefined;
    /**
     * The item(s) within the packaging.
     */
    containedItem?: PackagedProductDefinitionPackagingContainedItem[] | undefined;
    /**
     * A business identifier that is specific to this particular part of the packaging, often assigned by the manufacturer. Including possibly Data Carrier Identifier (a GS1 barcode).
     */
    identifier?: Identifier[] | undefined;
    /**
     * Manufacturer of this packaging item. When there are multiple values each one is a potential manufacturer of this packaging item.
     */
    manufacturer?: Reference[] | undefined;
    /**
     * Material type of the package item.
     */
    material?: CodeableConcept[] | undefined;
    /**
     * Allows containers (and parts of containers) parwithin containers, still a single packaged product.  See also PackagedProductDefinition.packaging.containedItem.item(PackagedProductDefinition).
     */
    packaging?: PackagedProductDefinitionPackaging[] | undefined;
    /**
     * General characteristics of this item.
     */
    property?: PackagedProductDefinitionPackagingProperty[] | undefined;
    /**
     * The quantity of packaging items contained at this layer of the package. This does not relate to the number of contained items but relates solely to the number of packaging items. When looking at the outermost layer it is always 1. If there are two boxes within, at the next layer it would be 2.
     */
    quantity?: number | undefined;
    /**
     * Shelf Life and storage information.
     */
    shelfLifeStorage?: ProductShelfLife[] | undefined;
    /**
     * The physical type of the container of the items.
     */
    type?: CodeableConcept | undefined;
}
/**
 * A medically related item or items, in a container or package.
 */
export interface PackagedProductDefinition extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'PackagedProductDefinition';
    /**
     * Additional information or supporting documentation about the packaged product.
     */
    attachedDocument?: Reference[] | undefined;
    /**
     * Allows the key features to be recorded, such as "hospital pack", "nurse prescribable", "calendar pack".
     */
    characteristic?: CodeableConcept[] | undefined;
    /**
     * A total of the complete count of contained items of a particular type/form, independent of sub-packaging or organization. This can be considered as the pack size. This attribute differs from containedItem.amount in that it can give a single aggregated count of all tablet types in a pack, even when these are different manufactured items. For example a pill pack of 21 tablets plus 7 sugar tablets, can be denoted here as '28 tablets'. This attribute is repeatable so that the different item types in one pack type can be counted (e.g. a count of vials and count of syringes). Each repeat must have different units, so that it is clear what the different sets of counted items are, and it is not intended to allow different counts of similar items (e.g. not '2 tubes and 3 tubes'). Repeats are not to be used to represent different pack sizes (e.g. 20 pack vs. 50 pack) - which would be different instances of this resource.
     */
    containedItemQuantity?: Quantity[] | undefined;
    /**
     * Identifies if the package contains different items, such as when a drug product is supplied with another item e.g. a diluent or adjuvant.
     */
    copackagedIndicator?: boolean | undefined;
    _copackagedIndicator?: Element | undefined;
    /**
     * Textual description. Note that this is not the name of the package or product.
     */
    description?: string | undefined;
    _description?: Element | undefined;
    /**
     * A unique identifier for this package as whole - not the the content of the package. Unique instance identifiers assigned to a package by manufacturers, regulators, drug catalogue custodians or other organizations.
     */
    identifier?: Identifier[] | undefined;
    /**
     * The legal status of supply of the packaged item as classified by the regulator.
     */
    legalStatusOfSupply?: PackagedProductDefinitionLegalStatusOfSupply[] | undefined;
    /**
     * Manufacturer of this package type. When there are multiple it means these are all possible manufacturers.
     */
    manufacturer?: Reference[] | undefined;
    /**
     * Allows specifying that an item is on the market for sale, or that it is not available, and the dates and locations associated.
     */
    marketingStatus?: MarketingStatus[] | undefined;
    /**
     * A name for this package. Typically what it would be listed as in a drug formulary or catalogue, inventory etc.
     */
    name?: string | undefined;
    _name?: Element | undefined;
    /**
     * The product this package model relates to, not the contents of the package (for which see package.containedItem).
     */
    packageFor?: Reference[] | undefined;
    /**
     * A packaging item, as a container for medically related items, possibly with other packaging items within, or a packaging component, such as bottle cap (which is not a device or a medication manufactured item).
     */
    packaging?: PackagedProductDefinitionPackaging | undefined;
    /**
     * The status within the lifecycle of this item. A high level status, this is not intended to duplicate details carried elsewhere such as legal status, or authorization or marketing status.
     */
    status?: CodeableConcept | undefined;
    /**
     * The date at which the given status became applicable.
     */
    statusDate?: string | undefined;
    _statusDate?: Element | undefined;
    /**
     * A high level category e.g. medicinal product, raw material, shipping/transport container, etc.
     */
    type?: CodeableConcept | undefined;
}
/**
 * A parameter passed to or received from the operation.
 */
export interface ParametersParameter extends BackboneElement {
    /**
     * The name of the parameter (reference to the operation definition).
     */
    name: string;
    _name?: Element | undefined;
    /**
     * A named part of a multi-part parameter.
     */
    part?: ParametersParameter[] | undefined;
    /**
     * When resolving references in resources, the operation definition may specify how references may be resolved between parameters. If a reference cannot be resolved between the parameters, the application should fall back to it's general resource resolution methods.
     */
    resource?: FhirResource | undefined;
    /**
     * Conveys the content if the parameter is a data type.
     */
    valueBase64Binary?: string | undefined;
    _valueBase64Binary?: Element | undefined;
    /**
     * Conveys the content if the parameter is a data type.
     */
    valueBoolean?: boolean | undefined;
    _valueBoolean?: Element | undefined;
    /**
     * Conveys the content if the parameter is a data type.
     */
    valueCanonical?: string | undefined;
    _valueCanonical?: Element | undefined;
    /**
     * Conveys the content if the parameter is a data type.
     */
    valueCode?: string | undefined;
    _valueCode?: Element | undefined;
    /**
     * Conveys the content if the parameter is a data type.
     */
    valueDate?: string | undefined;
    _valueDate?: Element | undefined;
    /**
     * Conveys the content if the parameter is a data type.
     */
    valueDateTime?: string | undefined;
    _valueDateTime?: Element | undefined;
    /**
     * Conveys the content if the parameter is a data type.
     */
    valueDecimal?: number | undefined;
    /**
     * Conveys the content if the parameter is a data type.
     */
    valueId?: string | undefined;
    _valueId?: Element | undefined;
    /**
     * Conveys the content if the parameter is a data type.
     */
    valueInstant?: string | undefined;
    _valueInstant?: Element | undefined;
    /**
     * Conveys the content if the parameter is a data type.
     */
    valueInteger?: number | undefined;
    /**
     * Conveys the content if the parameter is a data type.
     */
    valueInteger64?: string | undefined;
    _valueInteger64?: Element | undefined;
    /**
     * Conveys the content if the parameter is a data type.
     */
    valueMarkdown?: string | undefined;
    _valueMarkdown?: Element | undefined;
    /**
     * Conveys the content if the parameter is a data type.
     */
    valueOid?: string | undefined;
    _valueOid?: Element | undefined;
    /**
     * Conveys the content if the parameter is a data type.
     */
    valuePositiveInt?: number | undefined;
    /**
     * Conveys the content if the parameter is a data type.
     */
    valueString?: string | undefined;
    _valueString?: Element | undefined;
    /**
     * Conveys the content if the parameter is a data type.
     */
    valueTime?: string | undefined;
    _valueTime?: Element | undefined;
    /**
     * Conveys the content if the parameter is a data type.
     */
    valueUnsignedInt?: number | undefined;
    /**
     * Conveys the content if the parameter is a data type.
     */
    valueUri?: string | undefined;
    _valueUri?: Element | undefined;
    /**
     * Conveys the content if the parameter is a data type.
     */
    valueUrl?: string | undefined;
    _valueUrl?: Element | undefined;
    /**
     * Conveys the content if the parameter is a data type.
     */
    valueUuid?: string | undefined;
    _valueUuid?: Element | undefined;
    /**
     * Conveys the content if the parameter is a data type.
     */
    valueAddress?: Address | undefined;
    /**
     * Conveys the content if the parameter is a data type.
     */
    valueAge?: Age | undefined;
    /**
     * Conveys the content if the parameter is a data type.
     */
    valueAnnotation?: Annotation | undefined;
    /**
     * Conveys the content if the parameter is a data type.
     */
    valueAttachment?: Attachment | undefined;
    /**
     * Conveys the content if the parameter is a data type.
     */
    valueCodeableConcept?: CodeableConcept | undefined;
    /**
     * Conveys the content if the parameter is a data type.
     */
    valueCodeableReference?: CodeableReference | undefined;
    /**
     * Conveys the content if the parameter is a data type.
     */
    valueCoding?: Coding | undefined;
    /**
     * Conveys the content if the parameter is a data type.
     */
    valueContactPoint?: ContactPoint | undefined;
    /**
     * Conveys the content if the parameter is a data type.
     */
    valueCount?: Count | undefined;
    /**
     * Conveys the content if the parameter is a data type.
     */
    valueDistance?: Distance | undefined;
    /**
     * Conveys the content if the parameter is a data type.
     */
    valueDuration?: Duration | undefined;
    /**
     * Conveys the content if the parameter is a data type.
     */
    valueHumanName?: HumanName | undefined;
    /**
     * Conveys the content if the parameter is a data type.
     */
    valueIdentifier?: Identifier | undefined;
    /**
     * Conveys the content if the parameter is a data type.
     */
    valueMoney?: Money | undefined;
    /**
     * Conveys the content if the parameter is a data type.
     */
    valuePeriod?: Period | undefined;
    /**
     * Conveys the content if the parameter is a data type.
     */
    valueQuantity?: Quantity | undefined;
    /**
     * Conveys the content if the parameter is a data type.
     */
    valueRange?: Range | undefined;
    /**
     * Conveys the content if the parameter is a data type.
     */
    valueRatio?: Ratio | undefined;
    /**
     * Conveys the content if the parameter is a data type.
     */
    valueRatioRange?: RatioRange | undefined;
    /**
     * Conveys the content if the parameter is a data type.
     */
    valueReference?: Reference | undefined;
    /**
     * Conveys the content if the parameter is a data type.
     */
    valueSampledData?: SampledData | undefined;
    /**
     * Conveys the content if the parameter is a data type.
     */
    valueSignature?: Signature | undefined;
    /**
     * Conveys the content if the parameter is a data type.
     */
    valueTiming?: Timing | undefined;
    /**
     * Conveys the content if the parameter is a data type.
     */
    valueContactDetail?: ContactDetail | undefined;
    /**
     * Conveys the content if the parameter is a data type.
     */
    valueDataRequirement?: DataRequirement | undefined;
    /**
     * Conveys the content if the parameter is a data type.
     */
    valueExpression?: Expression | undefined;
    /**
     * Conveys the content if the parameter is a data type.
     */
    valueParameterDefinition?: ParameterDefinition | undefined;
    /**
     * Conveys the content if the parameter is a data type.
     */
    valueRelatedArtifact?: RelatedArtifact | undefined;
    /**
     * Conveys the content if the parameter is a data type.
     */
    valueTriggerDefinition?: TriggerDefinition | undefined;
    /**
     * Conveys the content if the parameter is a data type.
     */
    valueUsageContext?: UsageContext | undefined;
    /**
     * Conveys the content if the parameter is a data type.
     */
    valueAvailability?: Availability | undefined;
    /**
     * Conveys the content if the parameter is a data type.
     */
    valueExtendedContactDetail?: ExtendedContactDetail | undefined;
    /**
     * Conveys the content if the parameter is a data type.
     */
    valueDosage?: Dosage | undefined;
    /**
     * Conveys the content if the parameter is a data type.
     */
    valueMeta?: Meta | undefined;
}
/**
 * This resource is used to pass information into and back from an operation (whether invoked directly from REST or within a messaging environment).  It is not persisted or allowed to be referenced by other resources.
 */
export interface Parameters extends Resource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'Parameters';
    /**
     * A parameter passed to or received from the operation.
     */
    parameter?: ParametersParameter[] | undefined;
}
/**
 * Contact covers all kinds of contact parties: family members, business contacts, guardians, caregivers. Not applicable to register pedigree and family ties beyond use of having contact.
 */
export interface PatientContact extends BackboneElement {
    /**
     * Address for the contact person.
     */
    address?: Address | undefined;
    /**
     * Administrative Gender - the gender that the contact person is considered to have for administration and record keeping purposes.
     */
    gender?: ('male' | 'female' | 'other' | 'unknown') | undefined;
    _gender?: Element | undefined;
    /**
     * A name associated with the contact person.
     */
    name?: HumanName | undefined;
    /**
     * Organization on behalf of which the contact is acting or for which the contact is working.
     */
    organization?: Reference | undefined;
    /**
     * The period during which this contact person or organization is valid to be contacted relating to this patient.
     */
    period?: Period | undefined;
    /**
     * The nature of the relationship between the patient and the contact person.
     */
    relationship?: CodeableConcept[] | undefined;
    /**
     * Contact may have multiple ways to be contacted with different uses or applicable periods.  May need to have options for contacting the person urgently, and also to help with identification.
     */
    telecom?: ContactPoint[] | undefined;
}
/**
 * If no language is specified, this *implies* that the default local language is spoken.  If you need to convey proficiency for multiple modes, then you need multiple Patient.Communication associations.   For animals, language is not a relevant field, and should be absent from the instance. If the Patient does not speak the default local language, then the Interpreter Required Standard can be used to explicitly declare that an interpreter is required.
 */
export interface PatientCommunication extends BackboneElement {
    /**
     * The structure aa-BB with this exact casing is one the most widely used notations for locale. However not all systems actually code this but instead have it as free text. Hence CodeableConcept instead of code as the data type.
     */
    language: CodeableConcept;
    /**
     * This language is specifically identified for communicating healthcare information.
     */
    preferred?: boolean | undefined;
    _preferred?: Element | undefined;
}
/**
 * There is no assumption that linked patient records have mutual links.
 */
export interface PatientLink extends BackboneElement {
    /**
     * Referencing a RelatedPerson here removes the need to use a Person record to associate a Patient and RelatedPerson as the same individual.
     */
    other: Reference;
    /**
     * The type of link between this patient resource and another patient resource.
     */
    type: 'replaced-by' | 'replaces' | 'refer' | 'seealso';
    _type?: Element | undefined;
}
/**
 * Demographics and other administrative information about an individual or animal receiving care or other health-related services.
 */
export interface Patient extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'Patient';
    /**
     * If a record is inactive, and linked to an active record, then future patient/record updates should occur on the other patient.
     */
    active?: boolean | undefined;
    _active?: Element | undefined;
    /**
     * Patient may have multiple addresses with different uses or applicable periods.
     */
    address?: Address[] | undefined;
    /**
     * Partial dates are allowed if the specific date of birth is unknown. There is a standard extension "patient-birthTime" available that should be used where Time is required (such as in maternity/infant care systems).
     */
    birthDate?: string | undefined;
    _birthDate?: Element | undefined;
    /**
     * If no language is specified, this *implies* that the default local language is spoken.  If you need to convey proficiency for multiple modes, then you need multiple Patient.Communication associations.   For animals, language is not a relevant field, and should be absent from the instance. If the Patient does not speak the default local language, then the Interpreter Required Standard can be used to explicitly declare that an interpreter is required.
     */
    communication?: PatientCommunication[] | undefined;
    /**
     * Contact covers all kinds of contact parties: family members, business contacts, guardians, caregivers. Not applicable to register pedigree and family ties beyond use of having contact.
     */
    contact?: PatientContact[] | undefined;
    /**
     * If there's no value in the instance, it means there is no statement on whether or not the individual is deceased. Most systems will interpret the absence of a value as a sign of the person being alive.
     */
    deceasedBoolean?: boolean | undefined;
    _deceasedBoolean?: Element | undefined;
    /**
     * If there's no value in the instance, it means there is no statement on whether or not the individual is deceased. Most systems will interpret the absence of a value as a sign of the person being alive.
     */
    deceasedDateTime?: string | undefined;
    _deceasedDateTime?: Element | undefined;
    /**
     * The gender might not match the biological sex as determined by genetics or the individual's preferred identification. Note that for both humans and particularly animals, there are other legitimate possibilities than male and female, though the vast majority of systems and contexts only support male and female.  Systems providing decision support or enforcing business rules should ideally do this on the basis of Observations dealing with the specific sex or gender aspect of interest (anatomical, chromosomal, social, etc.)  However, because these observations are infrequently recorded, defaulting to the administrative gender is common practice.  Where such defaulting occurs, rule enforcement should allow for the variation between administrative and biological, chromosomal and other gender aspects.  For example, an alert about a hysterectomy on a male should be handled as a warning or overridable error, not a "hard" error.  See the Patient Gender and Sex section for additional information about communicating patient gender and sex.
     */
    gender?: ('male' | 'female' | 'other' | 'unknown') | undefined;
    _gender?: Element | undefined;
    /**
     * This may be the primary care provider (in a GP context), or it may be a patient nominated care manager in a community/disability setting, or even organization that will provide people to perform the care provider roles.  It is not to be used to record Care Teams, these should be in a CareTeam resource that may be linked to the CarePlan or EpisodeOfCare resources.
     * Multiple GPs may be recorded against the patient for various reasons, such as a student that has his home GP listed along with the GP at university during the school semesters, or a "fly-in/fly-out" worker that has the onsite GP also included with his home GP to remain aware of medical issues.
     * Jurisdictions may decide that they can profile this down to 1 if desired, or 1 per type.
     */
    generalPractitioner?: Reference[] | undefined;
    /**
     * An identifier for this patient.
     */
    identifier?: Identifier[] | undefined;
    /**
     * There is no assumption that linked patient records have mutual links.
     */
    link?: PatientLink[] | undefined;
    /**
     * There is only one managing organization for a specific patient record. Other organizations will have their own Patient record, and may use the Link property to join the records together (or a Person resource which can include confidence ratings for the association).
     */
    managingOrganization?: Reference | undefined;
    /**
     * This field contains a patient's most recent marital (civil) status.
     */
    maritalStatus?: CodeableConcept | undefined;
    /**
     * Where the valueInteger is provided, the number is the birth number in the sequence. E.g. The middle birth in triplets would be valueInteger=2 and the third born would have valueInteger=3 If a boolean value was provided for this triplets example, then all 3 patient records would have valueBoolean=true (the ordering is not indicated).
     */
    multipleBirthBoolean?: boolean | undefined;
    _multipleBirthBoolean?: Element | undefined;
    /**
     * Where the valueInteger is provided, the number is the birth number in the sequence. E.g. The middle birth in triplets would be valueInteger=2 and the third born would have valueInteger=3 If a boolean value was provided for this triplets example, then all 3 patient records would have valueBoolean=true (the ordering is not indicated).
     */
    multipleBirthInteger?: number | undefined;
    /**
     * A patient may have multiple names with different uses or applicable periods. For animals, the name is a "HumanName" in the sense that is assigned and used by humans and has the same patterns. Animal names may be communicated as given names, and optionally may include a family name.
     */
    name?: HumanName[] | undefined;
    /**
     * Guidelines:
     * * Use id photos, not clinical photos.
     * * Limit dimensions to thumbnail.
     * * Keep byte count low to ease resource updates.
     */
    photo?: Attachment[] | undefined;
    /**
     * A Patient may have multiple ways to be contacted with different uses or applicable periods.  May need to have options for contacting the person urgently and also to help with identification. The address might not go directly to the individual, but may reach another party that is able to proxy for the patient (i.e. home phone, or pet owner's phone).
     */
    telecom?: ContactPoint[] | undefined;
}
/**
 * This resource provides the status of the payment for goods and services rendered, and the request and response resource references.
 */
export interface PaymentNotice extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'PaymentNotice';
    /**
     * The amount sent to the payee.
     */
    amount: Money;
    /**
     * The date when this resource was created.
     */
    created: string;
    _created?: Element | undefined;
    /**
     * A unique identifier assigned to this payment notice.
     */
    identifier?: Identifier[] | undefined;
    /**
     * The party who will receive or has received payment that is the subject of this notification.
     */
    payee?: Reference | undefined;
    /**
     * A reference to the payment which is the subject of this notice.
     */
    payment: Reference;
    /**
     * The date when the above payment action occurred.
     */
    paymentDate?: string | undefined;
    _paymentDate?: Element | undefined;
    /**
     * Typically paid: payment sent, cleared: payment received.
     */
    paymentStatus?: CodeableConcept | undefined;
    /**
     * The practitioner who is responsible for the services rendered to the patient.
     */
    provider?: Reference | undefined;
    /**
     * The party who is notified of the payment status.
     */
    recipient: Reference;
    /**
     * Reference of resource for which payment is being made.
     */
    request?: Reference | undefined;
    /**
     * Reference of response to resource for which payment is being made.
     */
    response?: Reference | undefined;
    /**
     * This element is labeled as a modifier because the status contains codes that mark the resource as not currently valid.
     */
    status: 'active' | 'cancelled' | 'draft' | 'entered-in-error';
    _status?: Element | undefined;
}
/**
 * Distribution of the payment amount for a previously acknowledged payable.
 */
export interface PaymentReconciliationAllocation extends BackboneElement {
    /**
     * The Account to which this payment applies, may be completed by the receiver, used for search.
     */
    account?: Reference | undefined;
    /**
     * The monetary amount allocated from the total payment to the payable.
     */
    amount?: Money | undefined;
    /**
     * The date from the response resource containing a commitment to pay.
     */
    date?: string | undefined;
    _date?: Element | undefined;
    /**
     * The Encounter to which this payment applies, may be completed by the receiver, used for search.
     */
    encounter?: Reference | undefined;
    /**
     * Unique identifier for the current payment item for the referenced payable.
     */
    identifier?: Identifier | undefined;
    /**
     * The party which is receiving the payment.
     */
    payee?: Reference | undefined;
    /**
     * Unique identifier for the prior payment item for the referenced payable.
     */
    predecessor?: Identifier | undefined;
    /**
     * A resource, such as a ClaimResponse, which contains a commitment to payment.
     */
    response?: Reference | undefined;
    /**
     * A reference to the individual who is responsible for inquiries regarding the response and its payment.
     */
    responsible?: Reference | undefined;
    /**
     * The party which submitted the claim or financial transaction.
     */
    submitter?: Reference | undefined;
    /**
     * Specific resource to which the payment/adjustment/advance applies.
     */
    target?: Reference | undefined;
    /**
     *  Identifies the claim line item, encounter or other sub-element being paid. Note payment may be partial, that is not match the then outstanding balance or amount incurred.
     */
    targetItemString?: string | undefined;
    _targetItemString?: Element | undefined;
    /**
     *  Identifies the claim line item, encounter or other sub-element being paid. Note payment may be partial, that is not match the then outstanding balance or amount incurred.
     */
    targetItemIdentifier?: Identifier | undefined;
    /**
     *  Identifies the claim line item, encounter or other sub-element being paid. Note payment may be partial, that is not match the then outstanding balance or amount incurred.
     */
    targetItemPositiveInt?: number | undefined;
    /**
     * For example: payment, adjustment, funds advance, etc.
     */
    type?: CodeableConcept | undefined;
}
/**
 * A note that describes or explains the processing in a human readable form.
 */
export interface PaymentReconciliationProcessNote extends BackboneElement {
    /**
     * The explanation or description associated with the processing.
     */
    text?: string | undefined;
    _text?: Element | undefined;
    /**
     * The business purpose of the note text.
     */
    type?: ('display' | 'print' | 'printoper') | undefined;
    _type?: Element | undefined;
}
/**
 * This resource provides the details including amount of a payment and allocates the payment items being paid.
 */
export interface PaymentReconciliation extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'PaymentReconciliation';
    /**
     * A portion of the account number, often the last 4 digits, used for verification not charging purposes.
     */
    accountNumber?: string | undefined;
    _accountNumber?: Element | undefined;
    /**
     * Distribution of the payment amount for a previously acknowledged payable.
     */
    allocation?: PaymentReconciliationAllocation[] | undefined;
    /**
     * Total payment amount as indicated on the financial instrument.
     */
    amount: Money;
    /**
     * An alphanumeric issued by the processor to confirm the successful issuance of payment.
     */
    authorization?: string | undefined;
    _authorization?: Element | undefined;
    /**
     * The card brand such as debit, Visa, Amex etc. used if a card is the method of payment.
     */
    cardBrand?: string | undefined;
    _cardBrand?: Element | undefined;
    /**
     * The date when the resource was created.
     */
    created: string;
    _created?: Element | undefined;
    /**
     * The date of payment as indicated on the financial instrument.
     */
    date: string;
    _date?: Element | undefined;
    /**
     * A human readable description of the status of the request for the reconciliation.
     */
    disposition?: string | undefined;
    _disposition?: Element | undefined;
    /**
     * Payment enterer if not the actual payment issuer.
     */
    enterer?: Reference | undefined;
    /**
     * The year and month (YYYY-MM) when the instrument, typically card, expires.
     */
    expirationDate?: string | undefined;
    _expirationDate?: Element | undefined;
    /**
     * May be needed to identify specific jurisdictional forms.
     */
    formCode?: CodeableConcept | undefined;
    /**
     * A unique identifier assigned to this payment reconciliation.
     */
    identifier?: Identifier[] | undefined;
    /**
     * The type of the source such as patient or insurance.
     */
    issuerType?: CodeableConcept | undefined;
    /**
     * The workflow or activity which gave rise to or during which the payment ocurred such as a kiosk, deposit on account, periodic payment etc.
     */
    kind?: CodeableConcept | undefined;
    /**
     * The location of the site or device for electronic transfers or physical location for cash payments.
     */
    location?: Reference | undefined;
    /**
     * The means of payment such as check, card cash, or electronic funds transfer.
     */
    method?: CodeableConcept | undefined;
    /**
     * The resource may be used to indicate that: the request has been held (queued) for processing; that it has been processed and errors found (error); that no errors were found and that some of the adjudication has been undertaken (partial) or that all of the adjudication has been undertaken (complete).
     */
    outcome?: ('queued' | 'complete' | 'error' | 'partial') | undefined;
    _outcome?: Element | undefined;
    /**
     * For example: EFT number or check number.
     */
    paymentIdentifier?: Identifier | undefined;
    /**
     * This party is also responsible for the reconciliation.
     */
    paymentIssuer?: Reference | undefined;
    /**
     * The period of time for which payments have been gathered into this bulk payment for settlement.
     */
    period?: Period | undefined;
    /**
     * A note that describes or explains the processing in a human readable form.
     */
    processNote?: PaymentReconciliationProcessNote[] | undefined;
    /**
     * The name of the card processor, etf processor, bank for checks.
     */
    processor?: string | undefined;
    _processor?: Element | undefined;
    /**
     * The check number, eft reference, car processor reference.
     */
    referenceNumber?: string | undefined;
    _referenceNumber?: Element | undefined;
    /**
     * Original request resource reference.
     */
    request?: Reference | undefined;
    /**
     * The practitioner who is responsible for the services rendered to the patient.
     */
    requestor?: Reference | undefined;
    /**
     * The amount returned by the receiver which is excess to the amount payable, often referred to as 'change'.
     */
    returnedAmount?: Money | undefined;
    /**
     * This element is labeled as a modifier because the status contains codes that mark the resource as not currently valid.
     */
    status: 'active' | 'cancelled' | 'draft' | 'entered-in-error';
    _status?: Element | undefined;
    /**
     * The amount offered by the issuer, typically applies to cash when the issuer provides an amount in bank note denominations equal to or excess of the amount actually being paid.
     */
    tenderedAmount?: Money | undefined;
    /**
     * Code to indicate the nature of the payment such as payment, adjustment.
     */
    type: CodeableConcept;
}
/**
 * The asserted justification for using the data.
 */
export interface PermissionJustification extends BackboneElement {
    /**
     * This would be a codeableconcept, or a coding, which can be constrained to , for example, the 6 grounds for processing in GDPR.
     */
    basis?: CodeableConcept[] | undefined;
    /**
     * While any resource may be used, DocumentReference, Consent, PlanDefinition, and Contract would be most frequent
     */
    evidence?: Reference[] | undefined;
}
/**
 * Explicit FHIR Resource references.
 */
export interface PermissionRuleDataResource extends BackboneElement {
    /**
     * How the resource reference is interpreted when testing consent restrictions.
     */
    meaning: 'instance' | 'related' | 'dependents' | 'authoredby';
    _meaning?: Element | undefined;
    /**
     * A reference to a specific resource that defines which resources are covered by this consent.
     */
    reference: Reference;
}
/**
 * A description or definition of which activities are allowed to be done on the data.
 */
export interface PermissionRuleData extends BackboneElement {
    /**
     * Used when other data selection elements are insufficient.
     */
    expression?: Expression | undefined;
    /**
     * This has a different sense to the .validity.
     */
    period?: Period[] | undefined;
    /**
     * Explicit FHIR Resource references.
     */
    resource?: PermissionRuleDataResource[] | undefined;
    /**
     * Note the ConfidentialityCode vocabulary indicates the highest value, thus a security label of "R" then it applies to all resources that are labeled "R" or lower. E.g. for Confidentiality, it's a high water mark. For other kinds of security labels, subsumption logic applies. When the purpose of use tag is on the data, access request purpose of use shall not conflict.
     */
    security?: Coding[] | undefined;
}
/**
 * A description or definition of which activities are allowed to be done on the data.
 */
export interface PermissionRuleActivity extends BackboneElement {
    /**
     * Note that this is the direct action (not the grounds for the action covered in the purpose element). At present, the only action in the understood and tested scope of this resource is 'read'.
     */
    action?: CodeableConcept[] | undefined;
    /**
     * The actor(s) authorized for the defined activity.
     */
    actor?: Reference[] | undefined;
    /**
     * The purpose for which the permission is given.
     */
    purpose?: CodeableConcept[] | undefined;
}
/**
 * A set of rules.
 */
export interface PermissionRule extends BackboneElement {
    /**
     * A description or definition of which activities are allowed to be done on the data.
     */
    activity?: PermissionRuleActivity[] | undefined;
    /**
     * A description or definition of which activities are allowed to be done on the data.
     */
    data?: PermissionRuleData[] | undefined;
    /**
     * What limits apply to the use of the data.
     */
    limit?: CodeableConcept[] | undefined;
    /**
     * deny | permit.
     */
    type?: ('deny' | 'permit') | undefined;
    _type?: Element | undefined;
}
/**
 * Permission resource holds access rules for a given data and context.
 */
export interface Permission extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'Permission';
    /**
     * The person or entity that asserts the permission.
     */
    asserter?: Reference | undefined;
    /**
     * see [XACML Combining Rules](http://docs.oasis-open.org/xacml/3.0/xacml-3.0-core-spec-cos01-en.html#_Toc325047267)
     */
    combining:
        | 'deny-overrides'
        | 'permit-overrides'
        | 'ordered-deny-overrides'
        | 'ordered-permit-overrides'
        | 'deny-unless-permit'
        | 'permit-unless-deny';
    _combining?: Element | undefined;
    /**
     * The date that permission was asserted.
     */
    date?: string[] | undefined;
    _date?: Element[] | undefined;
    /**
     * The asserted justification for using the data.
     */
    justification?: PermissionJustification | undefined;
    /**
     * A set of rules.
     */
    rule?: PermissionRule[] | undefined;
    /**
     * Status.
     */
    status: 'active' | 'entered-in-error' | 'draft' | 'rejected';
    _status?: Element | undefined;
    /**
     * The period in which the permission is active.
     */
    validity?: Period | undefined;
}
/**
 * If no language is specified, this *implies* that the default local language is spoken.  If you need to convey proficiency for multiple modes, then you need multiple Person.Communication associations.   For animals, language is not a relevant field, and should be absent from the instance. If the Patient does not speak the default local language, then the Interpreter Required Standard can be used to explicitly declare that an interpreter is required.
 * Note that this property should not be used to update any linked/logically linked practitioner resources as it serves as a language that can be used to communicate with patients - however it may be used to inform the value on practitioner, along with their role at the organization (with the practitioner's permission)
 */
export interface PersonCommunication extends BackboneElement {
    /**
     * The structure aa-BB with this exact casing is one the most widely used notations for locale. However not all systems actually code this but instead have it as free text. Hence CodeableConcept instead of code as the data type.
     */
    language: CodeableConcept;
    /**
     * This language is specifically identified for communicating healthcare information.
     */
    preferred?: boolean | undefined;
    _preferred?: Element | undefined;
}
/**
 * Link to a resource that concerns the same actual person.
 */
export interface PersonLink extends BackboneElement {
    /**
     * Level of assurance that this link is associated with the target resource.
     */
    assurance?: ('level1' | 'level2' | 'level3' | 'level4') | undefined;
    _assurance?: Element | undefined;
    /**
     * The resource to which this actual person is associated.
     */
    target: Reference;
}
/**
 * Demographics and administrative information about a person independent of a specific health-related context.
 */
export interface Person extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'Person';
    /**
     * Whether this person's record is in active use.
     */
    active?: boolean | undefined;
    _active?: Element | undefined;
    /**
     * Person may have multiple addresses with different uses or applicable periods.
     */
    address?: Address[] | undefined;
    /**
     * At least an estimated year should be provided as a guess if the real DOB is unknown.
     */
    birthDate?: string | undefined;
    _birthDate?: Element | undefined;
    /**
     * If no language is specified, this *implies* that the default local language is spoken.  If you need to convey proficiency for multiple modes, then you need multiple Person.Communication associations.   For animals, language is not a relevant field, and should be absent from the instance. If the Patient does not speak the default local language, then the Interpreter Required Standard can be used to explicitly declare that an interpreter is required.
     * Note that this property should not be used to update any linked/logically linked practitioner resources as it serves as a language that can be used to communicate with patients - however it may be used to inform the value on practitioner, along with their role at the organization (with the practitioner's permission)
     */
    communication?: PersonCommunication[] | undefined;
    /**
     * If there's no value in the instance, it means there is no statement on whether or not the individual is deceased. Most systems will interpret the absence of a value as a sign of the person being alive.
     */
    deceasedBoolean?: boolean | undefined;
    _deceasedBoolean?: Element | undefined;
    /**
     * If there's no value in the instance, it means there is no statement on whether or not the individual is deceased. Most systems will interpret the absence of a value as a sign of the person being alive.
     */
    deceasedDateTime?: string | undefined;
    _deceasedDateTime?: Element | undefined;
    /**
     * The gender might not match the biological sex as determined by genetics, or the individual's preferred identification. Note that for both humans and particularly animals, there are other legitimate possibilities than M and F, though a clear majority of systems and contexts only support M and F.
     */
    gender?: ('male' | 'female' | 'other' | 'unknown') | undefined;
    _gender?: Element | undefined;
    /**
     * Identifier for a person within a particular scope.
     */
    identifier?: Identifier[] | undefined;
    /**
     * Link to a resource that concerns the same actual person.
     */
    link?: PersonLink[] | undefined;
    /**
     * The organization that is the custodian of the person record.
     */
    managingOrganization?: Reference | undefined;
    /**
     * Many countries maintain a register of births, deaths and marriages, and this would simplify that interaction
     */
    maritalStatus?: CodeableConcept | undefined;
    /**
     * Person may have multiple names with different uses or applicable periods.
     */
    name?: HumanName[] | undefined;
    /**
     * An image that can be displayed as a thumbnail of the person to enhance the identification of the individual.
     */
    photo?: Attachment[] | undefined;
    /**
     * Person may have multiple ways to be contacted with different uses or applicable periods.  May need to have options for contacting the person urgently and also to help with identification.
     */
    telecom?: ContactPoint[] | undefined;
}
/**
 * Indicates what should be done and within what timeframe.
 */
export interface PlanDefinitionGoalTarget extends BackboneElement {
    /**
     * A CodeableConcept with just a text would be used instead of a string if the field was usually coded, or if the type associated with the goal.target.measure defines a coded value.
     */
    detailQuantity?: Quantity | undefined;
    /**
     * A CodeableConcept with just a text would be used instead of a string if the field was usually coded, or if the type associated with the goal.target.measure defines a coded value.
     */
    detailRange?: Range | undefined;
    /**
     * A CodeableConcept with just a text would be used instead of a string if the field was usually coded, or if the type associated with the goal.target.measure defines a coded value.
     */
    detailCodeableConcept?: CodeableConcept | undefined;
    /**
     * A CodeableConcept with just a text would be used instead of a string if the field was usually coded, or if the type associated with the goal.target.measure defines a coded value.
     */
    detailString?: string | undefined;
    _detailString?: Element | undefined;
    /**
     * A CodeableConcept with just a text would be used instead of a string if the field was usually coded, or if the type associated with the goal.target.measure defines a coded value.
     */
    detailBoolean?: boolean | undefined;
    _detailBoolean?: Element | undefined;
    /**
     * A CodeableConcept with just a text would be used instead of a string if the field was usually coded, or if the type associated with the goal.target.measure defines a coded value.
     */
    detailInteger?: number | undefined;
    /**
     * A CodeableConcept with just a text would be used instead of a string if the field was usually coded, or if the type associated with the goal.target.measure defines a coded value.
     */
    detailRatio?: Ratio | undefined;
    /**
     * Indicates the timeframe after the start of the goal in which the goal should be met.
     */
    due?: Duration | undefined;
    /**
     * The parameter whose value is to be tracked, e.g. body weight, blood pressure, or hemoglobin A1c level.
     */
    measure?: CodeableConcept | undefined;
}
/**
 * A goal describes an expected outcome that activities within the plan are intended to achieve. For example, weight loss, restoring an activity of daily living, obtaining herd immunity via immunization, meeting a process improvement objective, meeting the acceptance criteria for a test as specified by a quality specification, etc.
 */
export interface PlanDefinitionGoal extends BackboneElement {
    /**
     * Identifies problems, conditions, issues, or concerns the goal is intended to address.
     */
    addresses?: CodeableConcept[] | undefined;
    /**
     * Indicates a category the goal falls within.
     */
    category?: CodeableConcept | undefined;
    /**
     * If no code is available, use CodeableConcept.text.
     */
    description: CodeableConcept;
    /**
     * Didactic or other informational resources associated with the goal that provide further supporting information about the goal. Information resources can include inline text commentary and links to web resources.
     */
    documentation?: RelatedArtifact[] | undefined;
    /**
     * Identifies the expected level of importance associated with reaching/sustaining the defined goal.
     */
    priority?: CodeableConcept | undefined;
    /**
     * The event after which the goal should begin being pursued.
     */
    start?: CodeableConcept | undefined;
    /**
     * Indicates what should be done and within what timeframe.
     */
    target?: PlanDefinitionGoalTarget[] | undefined;
}
/**
 * The characteristics of the candidates that could serve as the actor.
 */
export interface PlanDefinitionActorOption extends BackboneElement {
    /**
     * The role the participant should play in performing the described action.
     */
    role?: CodeableConcept | undefined;
    /**
     * The type of participant in the action.
     */
    type?:
        | (
              | 'careteam'
              | 'device'
              | 'group'
              | 'healthcareservice'
              | 'location'
              | 'organization'
              | 'patient'
              | 'practitioner'
              | 'practitionerrole'
              | 'relatedperson'
          )
        | undefined;
    _type?: Element | undefined;
    /**
     * The type of participant in the action.
     */
    typeCanonical?: string | undefined;
    _typeCanonical?: Element | undefined;
    /**
     * When this element is a reference, it SHOULD be a reference to a definitional resource (for example, a location type, rather than a specific location).
     */
    typeReference?: Reference | undefined;
}
/**
 * Actors represent the individuals or groups involved in the execution of the defined set of activities.
 */
export interface PlanDefinitionActor extends BackboneElement {
    /**
     * A description of how the actor fits into the overall actions of the plan definition.
     */
    description?: string | undefined;
    _description?: Element | undefined;
    /**
     * The characteristics of the candidates that could serve as the actor.
     */
    option: PlanDefinitionActorOption[];
    /**
     * A descriptive label for the actor.
     */
    title?: string | undefined;
    _title?: Element | undefined;
}
/**
 * When multiple conditions of the same kind are present, the effects are combined using AND semantics, so the overall condition is true only if all the conditions are true.
 */
export interface PlanDefinitionActionCondition extends BackboneElement {
    /**
     * The expression may be inlined or may be a reference to a named expression within a logic library referenced by the library element.
     */
    expression?: Expression | undefined;
    /**
     * Applicability criteria are used to determine immediate applicability when a plan definition is applied to a given context. Start and stop criteria are carried through application and used to describe enter/exit criteria for an action.
     */
    kind: 'applicability' | 'start' | 'stop';
    _kind?: Element | undefined;
}
/**
 * Defines input data requirements for the action.
 */
export interface PlanDefinitionActionInput extends BackboneElement {
    /**
     * The relatedData element allows indicating that an input to a parent action is an input to specific child actions. It also allows the output of one action to be identified as the input to a different action
     */
    relatedData?: string | undefined;
    _relatedData?: Element | undefined;
    /**
     * Defines the data that is to be provided as input to the action.
     */
    requirement?: DataRequirement | undefined;
    /**
     * A human-readable label for the data requirement used to label data flows in BPMN or similar diagrams. Also provides a human readable label when rendering the data requirement that conveys its purpose to human readers.
     */
    title?: string | undefined;
    _title?: Element | undefined;
}
/**
 * Defines the outputs of the action, if any.
 */
export interface PlanDefinitionActionOutput extends BackboneElement {
    /**
     * The relatedData element allows indicating that the output of child action is also the output of a parent action. It also allows the input of one action to be identified as the output of a different action
     */
    relatedData?: string | undefined;
    _relatedData?: Element | undefined;
    /**
     * Defines the data that results as output from the action.
     */
    requirement?: DataRequirement | undefined;
    /**
     * A human-readable label for the data requirement used to label data flows in BPMN or similar diagrams. Also provides a human readable label when rendering the data requirement that conveys its purpose to human readers.
     */
    title?: string | undefined;
    _title?: Element | undefined;
}
/**
 * When an action depends on multiple actions, the meaning is that all actions are dependencies, rather than that any of the actions are a dependency.
 */
export interface PlanDefinitionActionRelatedAction extends BackboneElement {
    /**
     * A duration or range of durations to apply to the relationship. For example, 30-60 minutes before.
     */
    offsetDuration?: Duration | undefined;
    /**
     * A duration or range of durations to apply to the relationship. For example, 30-60 minutes before.
     */
    offsetRange?: Range | undefined;
    /**
     * The relationship of this action to the related action.
     */
    relationship:
        | 'before-start'
        | 'before'
        | 'before-end'
        | 'concurrent-with-start'
        | 'concurrent'
        | 'concurrent-with-end'
        | 'after-start'
        | 'after'
        | 'after-end';
    _relationship?: Element | undefined;
    /**
     * The element id of the target related action.
     */
    targetId: string;
    _targetId?: Element | undefined;
}
/**
 * Indicates who should participate in performing the action described.
 */
export interface PlanDefinitionActionParticipant extends BackboneElement {
    /**
     * A reference to the id element of the actor who will participate in this action.
     */
    actorId?: string | undefined;
    _actorId?: Element | undefined;
    /**
     * Indicates how the actor will be involved in the action - author, reviewer, witness, etc.
     */
    function?: CodeableConcept | undefined;
    /**
     * The role the participant should play in performing the described action.
     */
    role?: CodeableConcept | undefined;
    /**
     * The type of participant in the action.
     */
    type?:
        | (
              | 'careteam'
              | 'device'
              | 'group'
              | 'healthcareservice'
              | 'location'
              | 'organization'
              | 'patient'
              | 'practitioner'
              | 'practitionerrole'
              | 'relatedperson'
          )
        | undefined;
    _type?: Element | undefined;
    /**
     * The type of participant in the action.
     */
    typeCanonical?: string | undefined;
    _typeCanonical?: Element | undefined;
    /**
     * When this element is a reference, it SHOULD be a reference to a definitional resource (for example, a location type, rather than a specific location).
     */
    typeReference?: Reference | undefined;
}
/**
 * Dynamic values are applied in the order in which they are defined in the PlanDefinition resource. Note that when dynamic values are also specified by a referenced ActivityDefinition, the dynamicValues from the ActivityDefinition are applied first, followed by the dynamicValues specified here. In addition, if both a transform and dynamic values are specific, the dynamic values are applied to the result of the transform.
 */
export interface PlanDefinitionActionDynamicValue extends BackboneElement {
    /**
     * The expression may be inlined or may be a reference to a named expression within a logic library referenced by the library element.
     */
    expression?: Expression | undefined;
    /**
     * To specify the path to the current action being realized, the %action environment variable is available in this path. For example, to specify the description element of the target action, the path would be %action.description. The path attribute contains a [Simple FHIRPath Subset](fhirpath.html#simple) that allows path traversal, but not calculation.
     */
    path?: string | undefined;
    _path?: Element | undefined;
}
/**
 * Note that there is overlap between many of the elements defined here and the ActivityDefinition resource. When an ActivityDefinition is referenced (using the definition element), the overlapping elements in the plan override the content of the referenced ActivityDefinition unless otherwise documented in the specific elements. See the PlanDefinition resource for more detailed information.
 */
export interface PlanDefinitionAction extends BackboneElement {
    /**
     * Sub actions that are contained within the action. The behavior of this action determines the functionality of the sub-actions. For example, a selection behavior of at-most-one indicates that of the sub-actions, at most one may be chosen as part of realizing the action definition.
     */
    action?: PlanDefinitionAction[] | undefined;
    /**
     * Defines whether the action can be selected multiple times.
     */
    cardinalityBehavior?: ('single' | 'multiple') | undefined;
    _cardinalityBehavior?: Element | undefined;
    /**
     * A code that provides a meaning, grouping, or classification for the action or action group. For example, a section may have a LOINC code for the section of a documentation template. In pharmaceutical quality, an action (Test) such as pH could be classified as a physical property.
     */
    code?: CodeableConcept | undefined;
    /**
     * When multiple conditions of the same kind are present, the effects are combined using AND semantics, so the overall condition is true only if all the conditions are true.
     */
    condition?: PlanDefinitionActionCondition[] | undefined;
    /**
     * Note that the definition is optional, and if no definition is specified, a dynamicValue with a root ($this) path can be used to define the entire resource dynamically.
     */
    definitionCanonical?: string | undefined;
    _definitionCanonical?: Element | undefined;
    /**
     * Note that the definition is optional, and if no definition is specified, a dynamicValue with a root ($this) path can be used to define the entire resource dynamically.
     */
    definitionUri?: string | undefined;
    _definitionUri?: Element | undefined;
    /**
     * A brief description of the action used to provide a summary to display to the user.
     */
    description?: string | undefined;
    _description?: Element | undefined;
    /**
     * Didactic or other informational resources associated with the action that can be provided to the CDS recipient. Information resources can include inline text commentary and links to web resources.
     */
    documentation?: RelatedArtifact[] | undefined;
    /**
     * Dynamic values are applied in the order in which they are defined in the PlanDefinition resource. Note that when dynamic values are also specified by a referenced ActivityDefinition, the dynamicValues from the ActivityDefinition are applied first, followed by the dynamicValues specified here. In addition, if both a transform and dynamic values are specific, the dynamic values are applied to the result of the transform.
     */
    dynamicValue?: PlanDefinitionActionDynamicValue[] | undefined;
    /**
     * Identifies goals that this action supports. The reference must be to a goal element defined within this plan definition. In pharmaceutical quality, a goal represents acceptance criteria (Goal) for a given action (Test), so the goalId would be the unique id of a defined goal element establishing the acceptance criteria for the action.
     */
    goalId?: string[] | undefined;
    _goalId?: Element[] | undefined;
    /**
     * Defines the grouping behavior for the action and its children.
     */
    groupingBehavior?: ('visual-group' | 'logical-group' | 'sentence-group') | undefined;
    _groupingBehavior?: Element | undefined;
    /**
     * Defines input data requirements for the action.
     */
    input?: PlanDefinitionActionInput[] | undefined;
    /**
     * An identifier that is unique within the PlanDefinition to allow linkage within the realized CarePlan and/or RequestOrchestration.
     */
    linkId?: string | undefined;
    _linkId?: Element | undefined;
    /**
     * May reference a specific clinical location or may just identify a type of location.
     */
    location?: CodeableReference | undefined;
    /**
     * Defines the outputs of the action, if any.
     */
    output?: PlanDefinitionActionOutput[] | undefined;
    /**
     * Indicates who should participate in performing the action described.
     */
    participant?: PlanDefinitionActionParticipant[] | undefined;
    /**
     * Defines whether the action should usually be preselected.
     */
    precheckBehavior?: ('yes' | 'no') | undefined;
    _precheckBehavior?: Element | undefined;
    /**
     * A user-visible prefix for the action. For example a section or item numbering such as 1. or A.
     */
    prefix?: string | undefined;
    _prefix?: Element | undefined;
    /**
     * Indicates how quickly the action should be addressed with respect to other actions.
     */
    priority?: ('routine' | 'urgent' | 'asap' | 'stat') | undefined;
    _priority?: Element | undefined;
    /**
     * This is different than the clinical evidence documentation, it's an actual business description of the reason for performing the action.
     */
    reason?: CodeableConcept[] | undefined;
    /**
     * When an action depends on multiple actions, the meaning is that all actions are dependencies, rather than that any of the actions are a dependency.
     */
    relatedAction?: PlanDefinitionActionRelatedAction[] | undefined;
    /**
     * Defines the required behavior for the action.
     */
    requiredBehavior?: ('must' | 'could' | 'must-unless-documented') | undefined;
    _requiredBehavior?: Element | undefined;
    /**
     * Defines the selection behavior for the action and its children.
     */
    selectionBehavior?: ('any' | 'all' | 'all-or-none' | 'exactly-one' | 'at-most-one' | 'one-or-more') | undefined;
    _selectionBehavior?: Element | undefined;
    /**
     * The subject of an action overrides the subject at a parent action or on the root of the PlanDefinition if specified.
     * In addition, because the subject needs to be resolved during realization, use of subjects in actions (or in the ActivityDefinition referenced by the action) resolves based on the set of subjects supplied in context and by type (i.e. the patient subject would resolve to a resource of type Patient).
     */
    subjectCodeableConcept?: CodeableConcept | undefined;
    /**
     * The subject of an action overrides the subject at a parent action or on the root of the PlanDefinition if specified.
     * In addition, because the subject needs to be resolved during realization, use of subjects in actions (or in the ActivityDefinition referenced by the action) resolves based on the set of subjects supplied in context and by type (i.e. the patient subject would resolve to a resource of type Patient).
     */
    subjectReference?: Reference | undefined;
    /**
     * The subject of an action overrides the subject at a parent action or on the root of the PlanDefinition if specified.
     * In addition, because the subject needs to be resolved during realization, use of subjects in actions (or in the ActivityDefinition referenced by the action) resolves based on the set of subjects supplied in context and by type (i.e. the patient subject would resolve to a resource of type Patient).
     */
    subjectCanonical?: string | undefined;
    _subjectCanonical?: Element | undefined;
    /**
     * A text equivalent of the action to be performed. This provides a human-interpretable description of the action when the definition is consumed by a system that might not be capable of interpreting it dynamically.
     */
    textEquivalent?: string | undefined;
    _textEquivalent?: Element | undefined;
    /**
     * The intent of the timing element is to provide timing when the action should be performed. As a definitional resource, this timing is interpreted as part of an apply operation so that the timing of the result actions in a CarePlan or RequestOrchestration, for example, would be specified by evaluating the timing definition in the context of the apply and setting the resulting timing on the appropriate elements of the target resource. If the timing is an Age, the activity is expected to be performed when the subject is the given Age. When the timing is a Duration, the activity is expected to be performed within the specified duration from the apply. When the timing is a Range, it may be a range of Ages or Durations, providing a range for the expected timing of the resulting activity. When the timing is a Timing, it is establishing a schedule for the timing of the resulting activity.
     */
    timingAge?: Age | undefined;
    /**
     * The intent of the timing element is to provide timing when the action should be performed. As a definitional resource, this timing is interpreted as part of an apply operation so that the timing of the result actions in a CarePlan or RequestOrchestration, for example, would be specified by evaluating the timing definition in the context of the apply and setting the resulting timing on the appropriate elements of the target resource. If the timing is an Age, the activity is expected to be performed when the subject is the given Age. When the timing is a Duration, the activity is expected to be performed within the specified duration from the apply. When the timing is a Range, it may be a range of Ages or Durations, providing a range for the expected timing of the resulting activity. When the timing is a Timing, it is establishing a schedule for the timing of the resulting activity.
     */
    timingDuration?: Duration | undefined;
    /**
     * The intent of the timing element is to provide timing when the action should be performed. As a definitional resource, this timing is interpreted as part of an apply operation so that the timing of the result actions in a CarePlan or RequestOrchestration, for example, would be specified by evaluating the timing definition in the context of the apply and setting the resulting timing on the appropriate elements of the target resource. If the timing is an Age, the activity is expected to be performed when the subject is the given Age. When the timing is a Duration, the activity is expected to be performed within the specified duration from the apply. When the timing is a Range, it may be a range of Ages or Durations, providing a range for the expected timing of the resulting activity. When the timing is a Timing, it is establishing a schedule for the timing of the resulting activity.
     */
    timingRange?: Range | undefined;
    /**
     * The intent of the timing element is to provide timing when the action should be performed. As a definitional resource, this timing is interpreted as part of an apply operation so that the timing of the result actions in a CarePlan or RequestOrchestration, for example, would be specified by evaluating the timing definition in the context of the apply and setting the resulting timing on the appropriate elements of the target resource. If the timing is an Age, the activity is expected to be performed when the subject is the given Age. When the timing is a Duration, the activity is expected to be performed within the specified duration from the apply. When the timing is a Range, it may be a range of Ages or Durations, providing a range for the expected timing of the resulting activity. When the timing is a Timing, it is establishing a schedule for the timing of the resulting activity.
     */
    timingTiming?: Timing | undefined;
    /**
     * The textual description of the action displayed to a user. For example, when the action is a test to be performed, the title would be the title of the test such as Assay by HPLC.
     */
    title?: string | undefined;
    _title?: Element | undefined;
    /**
     * Note that when a referenced ActivityDefinition also defines a transform, the transform specified here generally takes precedence. In addition, if both a transform and dynamic values are specific, the dynamic values are applied to the result of the transform.
     */
    transform?: string | undefined;
    _transform?: Element | undefined;
    /**
     * A description of when the action should be triggered. When multiple triggers are specified on an action, any triggering event invokes the action.
     */
    trigger?: TriggerDefinition[] | undefined;
    /**
     * The type of action to perform (create, update, remove).
     */
    type?: CodeableConcept | undefined;
}
/**
 * This resource allows for the definition of various types of plans as a sharable, consumable, and executable artifact. The resource is general enough to support the description of a broad range of clinical and non-clinical artifacts such as clinical decision support rules, order sets, protocols, and drug quality specifications.
 */
export interface PlanDefinition extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'PlanDefinition';
    /**
     * Note that there is overlap between many of the elements defined here and the ActivityDefinition resource. When an ActivityDefinition is referenced (using the definition element), the overlapping elements in the plan override the content of the referenced ActivityDefinition unless otherwise documented in the specific elements. See the PlanDefinition resource for more detailed information.
     */
    action?: PlanDefinitionAction[] | undefined;
    /**
     * Actors represent the individuals or groups involved in the execution of the defined set of activities.
     */
    actor?: PlanDefinitionActor[] | undefined;
    /**
     * The 'date' element may be more recent than the approval date because of minor changes or editorial corrections.
     */
    approvalDate?: string | undefined;
    _approvalDate?: Element | undefined;
    /**
     * If a CodeableConcept is present, it indicates the pre-condition for performing the service.  For example "pain", "on flare-up", etc.
     */
    asNeededBoolean?: boolean | undefined;
    _asNeededBoolean?: Element | undefined;
    /**
     * If a CodeableConcept is present, it indicates the pre-condition for performing the service.  For example "pain", "on flare-up", etc.
     */
    asNeededCodeableConcept?: CodeableConcept | undefined;
    /**
     * An individiual or organization primarily involved in the creation and maintenance of the content.
     */
    author?: ContactDetail[] | undefined;
    /**
     * May be a web site, an email address, a telephone number, etc.
     */
    contact?: ContactDetail[] | undefined;
    /**
     * A copyright statement relating to the plan definition and/or its contents. Copyright statements are generally legal restrictions on the use and publishing of the plan definition.
     */
    copyright?: string | undefined;
    _copyright?: Element | undefined;
    /**
     * Note that this is not the same as the resource last-modified-date, since the resource may be a secondary representation of the plan definition. Additional specific dates may be added as extensions or be found by consulting Provenances associated with past versions of the resource.
     */
    date?: string | undefined;
    _date?: Element | undefined;
    /**
     * This description can be used to capture details such as comments about misuse, instructions for clinical use and interpretation, literature references, examples from the paper world, etc. It is not a rendering of the plan definition as conveyed in the 'text' field of the resource itself. This item SHOULD be populated unless the information is available from context (e.g. the language of the plan definition is presumed to be the predominant language in the place the plan definition was created).
     */
    description?: string | undefined;
    _description?: Element | undefined;
    /**
     * An individual or organization primarily responsible for internal coherence of the content.
     */
    editor?: ContactDetail[] | undefined;
    /**
     * The effective period for a plan definition  determines when the content is applicable for usage and is independent of publication and review dates. For example, a measure intended to be used for the year 2016 might be published in 2015.
     */
    effectivePeriod?: Period | undefined;
    /**
     * An individual or organization responsible for officially endorsing the content for use in some setting.
     */
    endorser?: ContactDetail[] | undefined;
    /**
     * Allows filtering of plan definitions that are appropriate for use versus not.
     */
    experimental?: boolean | undefined;
    _experimental?: Element | undefined;
    /**
     * A goal describes an expected outcome that activities within the plan are intended to achieve. For example, weight loss, restoring an activity of daily living, obtaining herd immunity via immunization, meeting a process improvement objective, meeting the acceptance criteria for a test as specified by a quality specification, etc.
     */
    goal?: PlanDefinitionGoal[] | undefined;
    /**
     * Typically, this is used for identifiers that can go in an HL7 V3 II (instance identifier) data type, and can then identify this plan definition outside of FHIR, where it is not possible to use the logical URI.
     */
    identifier?: Identifier[] | undefined;
    /**
     * It may be possible for the plan definition to be used in jurisdictions other than those for which it was originally designed or intended.
     */
    jurisdiction?: CodeableConcept[] | undefined;
    /**
     * If specified, this date follows the original approval date.
     */
    lastReviewDate?: string | undefined;
    _lastReviewDate?: Element | undefined;
    /**
     * A reference to a Library resource containing any formal logic used by the plan definition.
     */
    library?: string[] | undefined;
    _library?: Element[] | undefined;
    /**
     * The name is not expected to be globally unique. The name should be a simple alphanumeric type name to ensure that it is machine-processing friendly.
     */
    name?: string | undefined;
    _name?: Element | undefined;
    /**
     * Usually an organization but may be an individual. The publisher (or steward) of the plan definition is the organization or individual primarily responsible for the maintenance and upkeep of the plan definition. This is not necessarily the same individual or organization that developed and initially authored the content. The publisher is the primary point of contact for questions or issues with the plan definition. This item SHOULD be populated unless the information is available from context.
     */
    publisher?: string | undefined;
    _publisher?: Element | undefined;
    /**
     * This element does not describe the usage of the plan definition. Instead, it provides traceability of ''why'' the resource is either needed or ''why'' it is defined as it is.  This may be used to point to source materials or specifications that drove the structure of this plan definition.
     */
    purpose?: string | undefined;
    _purpose?: Element | undefined;
    /**
     * Each related artifact is either an attachment, or a reference to another resource, but not both.
     */
    relatedArtifact?: RelatedArtifact[] | undefined;
    /**
     * An individual or organization primarily responsible for review of some aspect of the content.
     */
    reviewer?: ContactDetail[] | undefined;
    /**
     * Allows filtering of plan definitions that are appropriate for use versus not.
     */
    status: 'draft' | 'active' | 'retired' | 'unknown';
    _status?: Element | undefined;
    /**
     * Note that the choice of canonical for the subject element was introduced in R4B to support pharmaceutical quality use cases. To ensure as much backwards-compatibility as possible, it is recommended to only use the new canonical type with these use cases.
     */
    subjectCodeableConcept?: CodeableConcept | undefined;
    /**
     * Note that the choice of canonical for the subject element was introduced in R4B to support pharmaceutical quality use cases. To ensure as much backwards-compatibility as possible, it is recommended to only use the new canonical type with these use cases.
     */
    subjectReference?: Reference | undefined;
    /**
     * Note that the choice of canonical for the subject element was introduced in R4B to support pharmaceutical quality use cases. To ensure as much backwards-compatibility as possible, it is recommended to only use the new canonical type with these use cases.
     */
    subjectCanonical?: string | undefined;
    _subjectCanonical?: Element | undefined;
    /**
     * An explanatory or alternate title for the plan definition giving additional information about its content.
     */
    subtitle?: string | undefined;
    _subtitle?: Element | undefined;
    /**
     * This name does not need to be machine-processing friendly and may contain punctuation, white-space, etc.
     */
    title?: string | undefined;
    _title?: Element | undefined;
    /**
     * Descriptive topics related to the content of the plan definition. Topics provide a high-level categorization of the definition that can be useful for filtering and searching.
     */
    topic?: CodeableConcept[] | undefined;
    /**
     * A high-level category for the plan definition that distinguishes the kinds of systems that would be interested in the plan definition.
     */
    type?: CodeableConcept | undefined;
    /**
     * Can be a urn:uuid: or a urn:oid: but real http: addresses are preferred.  Multiple instances may share the same URL if they have a distinct version.
     * The determination of when to create a new version of a resource (same url, new version) vs. defining a new artifact is up to the author.  Considerations for making this decision are found in [Technical and Business Versions](resource.html#versions).
     * In some cases, the resource can no longer be found at the stated url, but the url itself cannot change. Implementations can use the [meta.source](resource.html#meta) element to indicate where the current master source of the resource can be found.
     */
    url?: string | undefined;
    _url?: Element | undefined;
    /**
     * A detailed description of how the plan definition is used from a clinical perspective.
     */
    usage?: string | undefined;
    _usage?: Element | undefined;
    /**
     * When multiple useContexts are specified, there is no expectation that all or any of the contexts apply.
     */
    useContext?: UsageContext[] | undefined;
    /**
     * There may be different plan definition instances that have the same identifier but different versions.  The version can be appended to the url in a reference to allow a reference to a particular business version of the plan definition with the format [url]|[version]. The version SHOULD NOT contain a '#' - see [Business Version](resource.html#bv-format).
     */
    version?: string | undefined;
    _version?: Element | undefined;
}
/**
 * The PractitionerRole.specialty defines the functional role that they are practicing at a given organization or location.  Those specialties may or might not require a qualification, and are not defined on the practitioner.
 */
export interface PractitionerQualification extends BackboneElement {
    /**
     * Coded representation of the qualification.
     */
    code: CodeableConcept;
    /**
     * An identifier that applies to this person's qualification.
     */
    identifier?: Identifier[] | undefined;
    /**
     * Organization that regulates and issues the qualification.
     */
    issuer?: Reference | undefined;
    /**
     * Period during which the qualification is valid.
     */
    period?: Period | undefined;
}
/**
 * A person who is directly or indirectly involved in the provisioning of healthcare or related services.
 */
export interface Practitioner extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'Practitioner';
    /**
     * If the practitioner is not in use by one organization, then it should mark the period on the PractitonerRole with an end date (even if they are active) as they may be active in another role.
     */
    active?: boolean | undefined;
    _active?: Element | undefined;
    /**
     * The PractitionerRole does not have an address value on it, as it is expected that the location property be used for this purpose (which has an address).
     */
    address?: Address[] | undefined;
    /**
     * The date of birth for the practitioner.
     */
    birthDate?: string | undefined;
    _birthDate?: Element | undefined;
    /**
     * The structure aa-BB with this exact casing is one the most widely used notations for locale. However not all systems code this but instead have it as free text. Hence CodeableConcept instead of code as the data type.
     */
    communication?: CodeableConcept[] | undefined;
    /**
     * If there's no value in the instance, it means there is no statement on whether or not the practitioner is deceased. Most systems will interpret the absence of a value as a sign of the person being alive.
     */
    deceasedBoolean?: boolean | undefined;
    _deceasedBoolean?: Element | undefined;
    /**
     * If there's no value in the instance, it means there is no statement on whether or not the practitioner is deceased. Most systems will interpret the absence of a value as a sign of the person being alive.
     */
    deceasedDateTime?: string | undefined;
    _deceasedDateTime?: Element | undefined;
    /**
     * Administrative Gender - the gender that the person is considered to have for administration and record keeping purposes.
     */
    gender?: ('male' | 'female' | 'other' | 'unknown') | undefined;
    _gender?: Element | undefined;
    /**
     * An identifier that applies to this person in this role.
     */
    identifier?: Identifier[] | undefined;
    /**
     * The selection of the use property should ensure that there is a single usual name specified, and others use the nickname (alias), old, or other values as appropriate.
     * In general, select the value to be used in the ResourceReference.display based on this:
     * 1. There is more than 1 name
     * 2. Use = usual
     * 3. Period is current to the date of the usage
     * 4. Use = official
     * 5. Other order as decided by internal business rules.
     */
    name?: HumanName[] | undefined;
    /**
     * Image of the person.
     */
    photo?: Attachment[] | undefined;
    /**
     * The PractitionerRole.specialty defines the functional role that they are practicing at a given organization or location.  Those specialties may or might not require a qualification, and are not defined on the practitioner.
     */
    qualification?: PractitionerQualification[] | undefined;
    /**
     * Person may have multiple ways to be contacted with different uses or applicable periods.  May need to have options for contacting the person urgently and to help with identification.  These typically will have home numbers, or mobile numbers that are not role specific.
     */
    telecom?: ContactPoint[] | undefined;
}
/**
 * A specific set of Roles/Locations/specialties/services that a practitioner may perform at an organization for a period of time.
 */
export interface PractitionerRole extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'PractitionerRole';
    /**
     * If this value is false, you may refer to the period to see when the role was in active use. If there is no period specified, no inference can be made about when it was active.
     */
    active?: boolean | undefined;
    _active?: Element | undefined;
    /**
     * More detailed availability information may be provided in associated Schedule/Slot resources.
     * Systems may choose to render availability differently than it is exchanged on the interface. For example, rather than "Mon, Tue, Wed, Thur, Fri from 9am-12am; Mon, Tue, Wed, Thur, Fri from 1pm-5pm" as would be implied by two availableTime repetitions, an application could render this information as "Mon-Fri 9-12am and 1-5pm".
     * The NotAvailableTime(s) included indicate the general days/periods where the practitioner is not available (for things such as vacation time, or public holidays).
     */
    availability?: Availability[] | undefined;
    /**
     * A person may have more than one role.
     */
    code?: CodeableConcept[] | undefined;
    /**
     * The contact details of communication devices available relevant to the specific PractitionerRole. This can include addresses, phone numbers, fax numbers, mobile numbers, email addresses and web sites.
     */
    contact?: ExtendedContactDetail[] | undefined;
    /**
     * Technical endpoints providing access to services operated for the practitioner with this role.
     */
    endpoint?: Reference[] | undefined;
    /**
     * The list of healthcare services that this worker provides for this role's Organization/Location(s).
     */
    healthcareService?: Reference[] | undefined;
    /**
     * A specific identifier value (e.g. Minnesota Registration ID) may appear on multiple PractitionerRole instances which could be for different periods, or different Roles (or other reasons).  A specific identifier may be included multiple times in the same PractitionerRole instance with different identifier validity periods.
     */
    identifier?: Identifier[] | undefined;
    /**
     * The location(s) at which this practitioner provides care.
     */
    location?: Reference[] | undefined;
    /**
     * The organization where the Practitioner performs the roles associated.
     */
    organization?: Reference | undefined;
    /**
     * If a practitioner is performing a role within an organization over multiple, non-adjacent periods, there should be a distinct PractitionerRole instance for each period.  For example, if a nurse is employed at a hospital, leaves the organization for a period of time due to pandemic related stress, but returns post-pandemic to the same job role, there would be two PractitionerRole instances for the different periods of employment.
     */
    period?: Period | undefined;
    /**
     * Practitioner that is able to provide the defined services for the organization.
     */
    practitioner?: Reference | undefined;
    /**
     * The specialty represents the functional role a practitioner is playing within an organization/location. This role may require the individual have certain qualifications, which would be represented in the Practitioner.qualifications property.  Note that qualifications may or might not relate to or be required by the practicing specialty.
     */
    specialty?: CodeableConcept[] | undefined;
}
/**
 * Limited to "real" people rather than equipment.
 */
export interface ProcedurePerformer extends BackboneElement {
    /**
     * Indicates who or what performed the procedure.
     */
    actor: Reference;
    /**
     * Distinguishes the type of involvement of the performer in the procedure. For example, surgeon, anaesthetist, endoscopist.
     */
    function?: CodeableConcept | undefined;
    /**
     * The organization the device or practitioner was acting on behalf of.
     */
    onBehalfOf?: Reference | undefined;
    /**
     * Time period during which the performer performed the procedure.
     */
    period?: Period | undefined;
}
/**
 * A device that is implanted, removed or otherwise manipulated (calibration, battery replacement, fitting a prosthesis, attaching a wound-vac, etc.) as a focal portion of the Procedure.
 */
export interface ProcedureFocalDevice extends BackboneElement {
    /**
     * The kind of change that happened to the device during the procedure.
     */
    action?: CodeableConcept | undefined;
    /**
     * The device that was manipulated (changed) during the procedure.
     */
    manipulated: Reference;
}
/**
 * An action that is or was performed on or for a patient, practitioner, device, organization, or location. For example, this can be a physical intervention on a patient like an operation, or less invasive like long term services, counseling, or hypnotherapy.  This can be a quality or safety inspection for a location, organization, or device.  This can be an accreditation procedure on a practitioner for licensing.
 */
export interface Procedure extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'Procedure';
    /**
     * A reference to a resource that contains details of the request for this procedure.
     */
    basedOn?: Reference[] | undefined;
    /**
     * If the use case requires attributes from the BodySite resource (e.g. to identify and track separately) then use the standard extension [procedure-targetbodystructure](extension-procedure-targetbodystructure.html).
     */
    bodySite?: CodeableConcept[] | undefined;
    /**
     * A code that classifies the procedure for searching, sorting and display purposes (e.g. "Surgical Procedure").
     */
    category?: CodeableConcept[] | undefined;
    /**
     * The specific procedure that is performed. Use text if the exact nature of the procedure cannot be coded (e.g. "Laparoscopic Appendectomy").
     */
    code?: CodeableConcept | undefined;
    /**
     * If complications are only expressed by the narrative text, they can be captured using the CodeableConcept.text.
     */
    complication?: CodeableConcept[] | undefined;
    /**
     * Any complications that occurred during the procedure, or in the immediate post-performance period.
     */
    complicationDetail?: Reference[] | undefined;
    /**
     * This will typically be the encounter the event occurred within, but some activities may be initiated prior to or after the official completion of an encounter but still be tied to the context of the encounter.
     */
    encounter?: Reference | undefined;
    /**
     * A device that is implanted, removed or otherwise manipulated (calibration, battery replacement, fitting a prosthesis, attaching a wound-vac, etc.) as a focal portion of the Procedure.
     */
    focalDevice?: ProcedureFocalDevice[] | undefined;
    /**
     * Who is the target of the procedure when it is not the subject of record only.  If focus is not present, then subject is the focus.  If focus is present and the subject is one of the targets of the procedure, include subject as a focus as well. If focus is present and the subject is not included in focus, it implies that the procedure was only targeted on the focus. For example, when a caregiver is given education for a patient, the caregiver would be the focus and the procedure record is associated with the subject (e.g. patient).  For example, use focus when recording the target of the education, training, or counseling is the parent or relative of a patient.
     */
    focus?: Reference | undefined;
    /**
     * If the procedure required specific follow up - e.g. removal of sutures. The follow up may be represented as a simple note or could potentially be more complex, in which case the CarePlan resource can be used.
     */
    followUp?: CodeableConcept[] | undefined;
    /**
     * This is a business identifier, not a resource identifier (see [discussion](resource.html#identifiers)).  It is best practice for the identifier to only appear on a single resource instance, however business practices may occasionally dictate that multiple resource instances with the same identifier can exist - possibly even with different resource types.  For example, multiple Patient and Person resource instances might share the same social insurance number.
     */
    identifier?: Identifier[] | undefined;
    /**
     * The URL pointing to a FHIR-defined protocol, guideline, order set or other definition that is adhered to in whole or in part by this Procedure.
     */
    instantiatesCanonical?: string[] | undefined;
    _instantiatesCanonical?: Element[] | undefined;
    /**
     * This might be an HTML page, PDF, etc. or could just be a non-resolvable URI identifier.
     */
    instantiatesUri?: string[] | undefined;
    _instantiatesUri?: Element[] | undefined;
    /**
     * The location where the procedure actually happened.  E.g. a newborn at home, a tracheostomy at a restaurant.
     */
    location?: Reference | undefined;
    /**
     * Any other notes and comments about the procedure.
     */
    note?: Annotation[] | undefined;
    /**
     * This indicates when the procedure actually occurred or is occurring, not when it was asked/requested/ordered to occur. For the latter, look at the occurence element of the Request this Procedure is "basedOn". The status code allows differentiation of whether the timing reflects a historic event or an ongoing event. Ongoing events should not include an upper bound in the Period or Timing.bounds.
     * Age is generally used when the patient reports an age at which the procedure was performed. Range is generally used when the patient reports an age range when the procedure was performed, such as sometime between 20-25 years old.  dateTime supports a range of precision due to some procedures being reported as past procedures that might not have millisecond precision while other procedures performed and documented during the encounter might have more precise UTC timestamps with timezone.
     */
    occurrenceDateTime?: string | undefined;
    _occurrenceDateTime?: Element | undefined;
    /**
     * This indicates when the procedure actually occurred or is occurring, not when it was asked/requested/ordered to occur. For the latter, look at the occurence element of the Request this Procedure is "basedOn". The status code allows differentiation of whether the timing reflects a historic event or an ongoing event. Ongoing events should not include an upper bound in the Period or Timing.bounds.
     * Age is generally used when the patient reports an age at which the procedure was performed. Range is generally used when the patient reports an age range when the procedure was performed, such as sometime between 20-25 years old.  dateTime supports a range of precision due to some procedures being reported as past procedures that might not have millisecond precision while other procedures performed and documented during the encounter might have more precise UTC timestamps with timezone.
     */
    occurrencePeriod?: Period | undefined;
    /**
     * This indicates when the procedure actually occurred or is occurring, not when it was asked/requested/ordered to occur. For the latter, look at the occurence element of the Request this Procedure is "basedOn". The status code allows differentiation of whether the timing reflects a historic event or an ongoing event. Ongoing events should not include an upper bound in the Period or Timing.bounds.
     * Age is generally used when the patient reports an age at which the procedure was performed. Range is generally used when the patient reports an age range when the procedure was performed, such as sometime between 20-25 years old.  dateTime supports a range of precision due to some procedures being reported as past procedures that might not have millisecond precision while other procedures performed and documented during the encounter might have more precise UTC timestamps with timezone.
     */
    occurrenceString?: string | undefined;
    _occurrenceString?: Element | undefined;
    /**
     * This indicates when the procedure actually occurred or is occurring, not when it was asked/requested/ordered to occur. For the latter, look at the occurence element of the Request this Procedure is "basedOn". The status code allows differentiation of whether the timing reflects a historic event or an ongoing event. Ongoing events should not include an upper bound in the Period or Timing.bounds.
     * Age is generally used when the patient reports an age at which the procedure was performed. Range is generally used when the patient reports an age range when the procedure was performed, such as sometime between 20-25 years old.  dateTime supports a range of precision due to some procedures being reported as past procedures that might not have millisecond precision while other procedures performed and documented during the encounter might have more precise UTC timestamps with timezone.
     */
    occurrenceAge?: Age | undefined;
    /**
     * This indicates when the procedure actually occurred or is occurring, not when it was asked/requested/ordered to occur. For the latter, look at the occurence element of the Request this Procedure is "basedOn". The status code allows differentiation of whether the timing reflects a historic event or an ongoing event. Ongoing events should not include an upper bound in the Period or Timing.bounds.
     * Age is generally used when the patient reports an age at which the procedure was performed. Range is generally used when the patient reports an age range when the procedure was performed, such as sometime between 20-25 years old.  dateTime supports a range of precision due to some procedures being reported as past procedures that might not have millisecond precision while other procedures performed and documented during the encounter might have more precise UTC timestamps with timezone.
     */
    occurrenceRange?: Range | undefined;
    /**
     * This indicates when the procedure actually occurred or is occurring, not when it was asked/requested/ordered to occur. For the latter, look at the occurence element of the Request this Procedure is "basedOn". The status code allows differentiation of whether the timing reflects a historic event or an ongoing event. Ongoing events should not include an upper bound in the Period or Timing.bounds.
     * Age is generally used when the patient reports an age at which the procedure was performed. Range is generally used when the patient reports an age range when the procedure was performed, such as sometime between 20-25 years old.  dateTime supports a range of precision due to some procedures being reported as past procedures that might not have millisecond precision while other procedures performed and documented during the encounter might have more precise UTC timestamps with timezone.
     */
    occurrenceTiming?: Timing | undefined;
    /**
     * If outcome contains narrative text only, it can be captured using the CodeableConcept.text.
     */
    outcome?: CodeableConcept | undefined;
    /**
     * The MedicationAdministration resource has a partOf reference to Procedure, but this is not a circular reference.   For example, the anesthesia MedicationAdministration is part of the surgical Procedure (MedicationAdministration.partOf = Procedure).  For example, the procedure to insert the IV port for an IV medication administration is part of the medication administration (Procedure.partOf = MedicationAdministration).
     */
    partOf?: Reference[] | undefined;
    /**
     * Limited to "real" people rather than equipment.
     */
    performer?: ProcedurePerformer[] | undefined;
    /**
     * Use Procedure.reason.concept when a code sufficiently describes the reason. Use Procedure.reason.reference when referencing a resource, which allows more information to be conveyed, such as onset date.  For a single Procedure.reason, if both Procedure.reason.concept and Procedure.reason.reference are present, they are expected to be consistent with each other.
     */
    reason?: CodeableReference[] | undefined;
    /**
     * The date the occurrence of the procedure was first captured in the record regardless of Procedure.status (potentially after the occurrence of the event).
     */
    recorded?: string | undefined;
    _recorded?: Element | undefined;
    /**
     * Individual who recorded the record and takes responsibility for its content.
     */
    recorder?: Reference | undefined;
    /**
     * There could potentially be multiple reports - e.g. if this was a procedure which took multiple biopsies resulting in a number of anatomical pathology reports.
     */
    report?: Reference[] | undefined;
    /**
     * Indicates if this record was captured as a secondary 'reported' record rather than as an original primary source-of-truth record.  It may also indicate the source of the report.
     */
    reportedBoolean?: boolean | undefined;
    _reportedBoolean?: Element | undefined;
    /**
     * Indicates if this record was captured as a secondary 'reported' record rather than as an original primary source-of-truth record.  It may also indicate the source of the report.
     */
    reportedReference?: Reference | undefined;
    /**
     * The "unknown" code is not to be used to convey other statuses.  The "unknown" code should be used when one of the statuses applies, but the authoring system doesn't know the current state of the procedure.
     * This element is labeled as a modifier because the status contains codes that mark the resource as not currently valid.
     */
    status:
        | 'preparation'
        | 'in-progress'
        | 'not-done'
        | 'on-hold'
        | 'stopped'
        | 'completed'
        | 'entered-in-error'
        | 'unknown';
    _status?: Element | undefined;
    /**
     * This is generally only used for "exception" statuses such as "not-done", "suspended" or "aborted". The reason for performing the event at all is captured in reasonCode, not here.
     */
    statusReason?: CodeableConcept | undefined;
    /**
     * On whom or on what the procedure was performed. This is usually an individual human, but can also be performed on animals, groups of humans or animals, organizations or practitioners (for licensing), locations or devices (for safety inspections or regulatory authorizations).  If the actual focus of the procedure is different from the subject, the focus element specifies the actual focus of the procedure.
     */
    subject: Reference;
    /**
     * Other resources from the patient record that may be relevant to the procedure.  The information from these resources was either used to create the instance or is provided to help with its interpretation. This extension should not be used if more specific inline elements or extensions are available.
     */
    supportingInfo?: Reference[] | undefined;
    /**
     * For devices actually implanted or removed, use Procedure.device.
     */
    used?: CodeableReference[] | undefined;
}
/**
 * Several agents may be associated (i.e. has some responsibility for an activity) with an activity and vice-versa.
 */
export interface ProvenanceAgent extends BackboneElement {
    /**
     * The agent that delegated authority to perform the activity performed by the agent.who element.
     */
    onBehalfOf?: Reference | undefined;
    /**
     * For example: Chief-of-Radiology, Nurse, Physician, Medical-Student, etc.
     */
    role?: CodeableConcept[] | undefined;
    /**
     * For example: assembler, author, prescriber, signer, investigator, etc.
     */
    type?: CodeableConcept | undefined;
    /**
     * Indicates who or what performed in the event.
     */
    who: Reference;
}
/**
 * An entity used in this activity.
 */
export interface ProvenanceEntity extends BackboneElement {
    /**
     * A usecase where one Provenance.entity.agent is used where the Entity that was used in the creation/updating of the Target, is not in the context of the same custodianship as the Target, and thus the meaning of Provenance.entity.agent is to say that the entity referenced is managed elsewhere and that this Agent provided access to it.  This would be similar to where the Entity being referenced is managed outside FHIR, such as through HL7 v2, v3, or XDS. This might be where the Entity being referenced is managed in another FHIR resource server. Thus it explains the Provenance of that Entity's use in the context of this Provenance activity.
     */
    agent?: ProvenanceAgent[] | undefined;
    /**
     * How the entity was used during the activity.
     */
    role: 'revision' | 'quotation' | 'source' | 'instantiates' | 'removal';
    _role?: Element | undefined;
    /**
     * whatIdentity should be used for entities that are not a Resource type.
     */
    what: Reference;
}
/**
 * Provenance of a resource is a record that describes entities and processes involved in producing and delivering or otherwise influencing that resource. Provenance provides a critical foundation for assessing authenticity, enabling trust, and allowing reproducibility. Provenance assertions are a form of contextual metadata and can themselves become important records with their own provenance. Provenance statement indicates clinical significance in terms of confidence in authenticity, reliability, and trustworthiness, integrity, and stage in lifecycle (e.g. Document Completion - has the artifact been legally authenticated), all of which may impact security, privacy, and trust policies.
 */
export interface Provenance extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'Provenance';
    /**
     * An activity is something that occurs over a period of time and acts upon or with entities; it may include consuming, processing, transforming, modifying, relocating, using, or generating entities.
     */
    activity?: CodeableConcept | undefined;
    /**
     * Several agents may be associated (i.e. has some responsibility for an activity) with an activity and vice-versa.
     */
    agent: ProvenanceAgent[];
    /**
     * The authorization (e.g., PurposeOfUse) that was used during the event being recorded.
     */
    authorization?: CodeableReference[] | undefined;
    /**
     * Allows tracing of authorizatino for the events and tracking whether proposals/recommendations were acted upon.
     */
    basedOn?: Reference[] | undefined;
    /**
     * This will typically be the encounter the event occurred, but some events may be initiated prior to or after the official completion of an encounter but still be tied to the context of the encounter (e.g. pre-admission lab tests).
     */
    encounter?: Reference | undefined;
    /**
     * An entity used in this activity.
     */
    entity?: ProvenanceEntity[] | undefined;
    /**
     * Where the activity occurred, if relevant.
     */
    location?: Reference | undefined;
    /**
     * The period can be a little arbitrary; where possible, the time should correspond to human assessment of the activity time.
     */
    occurredPeriod?: Period | undefined;
    /**
     * The period can be a little arbitrary; where possible, the time should correspond to human assessment of the activity time.
     */
    occurredDateTime?: string | undefined;
    _occurredDateTime?: Element | undefined;
    /**
     * The patient element is available to enable deterministic tracking of activities that involve the patient as the subject of the data used in an activity.
     */
    patient?: Reference | undefined;
    /**
     * For example: Where an OAuth token authorizes, the unique identifier from the OAuth token is placed into the policy element Where a policy engine (e.g. XACML) holds policy logic, the unique policy identifier is placed into the policy element.
     */
    policy?: string[] | undefined;
    _policy?: Element[] | undefined;
    /**
     * This can be a little different from the time stamp on the resource if there is a delay between recording the event and updating the provenance and target resource.
     */
    recorded?: string | undefined;
    _recorded?: Element | undefined;
    /**
     * A digital signature on the target Reference(s). The signer should match a Provenance.agent. The purpose of the signature is indicated.
     */
    signature?: Signature[] | undefined;
    /**
     * Target references are usually version specific, but might not be, if a version has not been assigned or if the provenance information is part of the set of resources being maintained (i.e. a document). When using the RESTful API, the identity of the resource might not be known (especially not the version specific one); the client may either submit the resource first, and then the provenance, or it may submit both using a single transaction. See the notes on transaction for further discussion.
     */
    target: Reference[];
}
/**
 * If multiple repetitions of this extension are present, the interpretation is driven by enableBehavior (either all repetitions must evaluate to true for this item to be enabled, or only one must evaluate to true for the item to be enabled).  If the enableWhen.question has multiple answers, the condition evaluates to true if *any* of the answers for the referenced item match the enableWhen condition.  This element is a modifier because if enableWhen is present for an item, "required" is ignored unless one of the enableWhen conditions is met. When an item is disabled, all of its descendants are disabled, regardless of what their own enableWhen logic might evaluate to.  If enableWhen logic depends on an item that is disabled, the logic should proceed as though the item is not valued - even if a default value or other value might be retained in memory in the event of the item being re-enabled.  In some cases, the comparison between the indicated answer and the specified value may differ only by precision.  For example, the enableWhen might be Q1 > 1970, but the answer to Q1 is 1970-10-15.  There is not a clear answer as to whether 1970-10-15 should be considered 'greater' than 1970, given that it is an imprecise value.  In these indeterminate situations, the form filler has the option of refusing to render the form.  If the form **is** displayed, items where enableWhen is indeterminate SHOULD be treated as enabled with a warning indicating that the questionnaire logic was faulty and it is possible that the item should not be enabled.  Questionnaires SHOULD be designed to take into account challenges around varying precision to minimize non-deterministic situations by setting constraints around expected precision, etc.
 */
export interface QuestionnaireItemEnableWhen extends BackboneElement {
    /**
     * A value that the referenced question is tested using the specified operator in order for the item to be enabled.  If there are multiple answers, a match on any of the answers suffices.  If different behavior is desired (all must match, at least 2 must match, etc.), consider using the enableWhenExpression extension.
     */
    answerBoolean?: boolean | undefined;
    _answerBoolean?: Element | undefined;
    /**
     * A value that the referenced question is tested using the specified operator in order for the item to be enabled.  If there are multiple answers, a match on any of the answers suffices.  If different behavior is desired (all must match, at least 2 must match, etc.), consider using the enableWhenExpression extension.
     */
    answerDecimal?: number | undefined;
    /**
     * A value that the referenced question is tested using the specified operator in order for the item to be enabled.  If there are multiple answers, a match on any of the answers suffices.  If different behavior is desired (all must match, at least 2 must match, etc.), consider using the enableWhenExpression extension.
     */
    answerInteger?: number | undefined;
    /**
     * A value that the referenced question is tested using the specified operator in order for the item to be enabled.  If there are multiple answers, a match on any of the answers suffices.  If different behavior is desired (all must match, at least 2 must match, etc.), consider using the enableWhenExpression extension.
     */
    answerDate?: string | undefined;
    _answerDate?: Element | undefined;
    /**
     * A value that the referenced question is tested using the specified operator in order for the item to be enabled.  If there are multiple answers, a match on any of the answers suffices.  If different behavior is desired (all must match, at least 2 must match, etc.), consider using the enableWhenExpression extension.
     */
    answerDateTime?: string | undefined;
    _answerDateTime?: Element | undefined;
    /**
     * A value that the referenced question is tested using the specified operator in order for the item to be enabled.  If there are multiple answers, a match on any of the answers suffices.  If different behavior is desired (all must match, at least 2 must match, etc.), consider using the enableWhenExpression extension.
     */
    answerTime?: string | undefined;
    _answerTime?: Element | undefined;
    /**
     * A value that the referenced question is tested using the specified operator in order for the item to be enabled.  If there are multiple answers, a match on any of the answers suffices.  If different behavior is desired (all must match, at least 2 must match, etc.), consider using the enableWhenExpression extension.
     */
    answerString?: string | undefined;
    _answerString?: Element | undefined;
    /**
     * A value that the referenced question is tested using the specified operator in order for the item to be enabled.  If there are multiple answers, a match on any of the answers suffices.  If different behavior is desired (all must match, at least 2 must match, etc.), consider using the enableWhenExpression extension.
     */
    answerCoding?: Coding | undefined;
    /**
     * A value that the referenced question is tested using the specified operator in order for the item to be enabled.  If there are multiple answers, a match on any of the answers suffices.  If different behavior is desired (all must match, at least 2 must match, etc.), consider using the enableWhenExpression extension.
     */
    answerQuantity?: Quantity | undefined;
    /**
     * A value that the referenced question is tested using the specified operator in order for the item to be enabled.  If there are multiple answers, a match on any of the answers suffices.  If different behavior is desired (all must match, at least 2 must match, etc.), consider using the enableWhenExpression extension.
     */
    answerReference?: Reference | undefined;
    /**
     * Specifies the criteria by which the question is enabled.
     */
    operator: 'exists' | '=' | '!=' | '>' | '<' | '>=' | '<=';
    _operator?: Element | undefined;
    /**
     * If multiple question occurrences are present for the same question (same linkId), then this refers to the nearest question occurrence reachable by tracing first the "ancestor" axis and then the "preceding" axis and then the "following" axis.  If there are multiple items with the same linkId and all are equadistant (e.g. a question references a question that appears in a separate repeating group), that is an error.  (Consider using the enableWhenExpression extension to define logic to handle such a situation.)
     */
    question: string;
    _question?: Element | undefined;
}
/**
 * This element can be used when the value set machinery of answerValueSet is deemed too cumbersome or when there's a need to capture possible answers that are not codes.
 */
export interface QuestionnaireItemAnswerOption extends BackboneElement {
    /**
     * Indicates whether the answer value is selected when the list of possible answers is initially shown.
     */
    initialSelected?: boolean | undefined;
    _initialSelected?: Element | undefined;
    /**
     * The data type of the value must agree with the item.type.
     */
    valueInteger?: number | undefined;
    /**
     * The data type of the value must agree with the item.type.
     */
    valueDate?: string | undefined;
    _valueDate?: Element | undefined;
    /**
     * The data type of the value must agree with the item.type.
     */
    valueTime?: string | undefined;
    _valueTime?: Element | undefined;
    /**
     * The data type of the value must agree with the item.type.
     */
    valueString?: string | undefined;
    _valueString?: Element | undefined;
    /**
     * The data type of the value must agree with the item.type.
     */
    valueCoding?: Coding | undefined;
    /**
     * The data type of the value must agree with the item.type.
     */
    valueReference?: Reference | undefined;
}
/**
 * The user is allowed to change the value and override the default (unless marked as read-only). If the user doesn't change the value, then this initial value will be persisted when the QuestionnaireResponse is initially created.  Note that initial values can influence results.  The data type of initial.answer[x] must agree with the item.type, and only repeating items can have more then one initial value.
 */
export interface QuestionnaireItemInitial extends BackboneElement {
    /**
     * The type of the initial value must be consistent with the type of the item.
     */
    valueBoolean?: boolean | undefined;
    _valueBoolean?: Element | undefined;
    /**
     * The type of the initial value must be consistent with the type of the item.
     */
    valueDecimal?: number | undefined;
    /**
     * The type of the initial value must be consistent with the type of the item.
     */
    valueInteger?: number | undefined;
    /**
     * The type of the initial value must be consistent with the type of the item.
     */
    valueDate?: string | undefined;
    _valueDate?: Element | undefined;
    /**
     * The type of the initial value must be consistent with the type of the item.
     */
    valueDateTime?: string | undefined;
    _valueDateTime?: Element | undefined;
    /**
     * The type of the initial value must be consistent with the type of the item.
     */
    valueTime?: string | undefined;
    _valueTime?: Element | undefined;
    /**
     * The type of the initial value must be consistent with the type of the item.
     */
    valueString?: string | undefined;
    _valueString?: Element | undefined;
    /**
     * The type of the initial value must be consistent with the type of the item.
     */
    valueUri?: string | undefined;
    _valueUri?: Element | undefined;
    /**
     * The type of the initial value must be consistent with the type of the item.
     */
    valueAttachment?: Attachment | undefined;
    /**
     * The type of the initial value must be consistent with the type of the item.
     */
    valueCoding?: Coding | undefined;
    /**
     * The type of the initial value must be consistent with the type of the item.
     */
    valueQuantity?: Quantity | undefined;
    /**
     * The type of the initial value must be consistent with the type of the item.
     */
    valueReference?: Reference | undefined;
}
/**
 * The content of the questionnaire is constructed from an ordered, hierarchical collection of items.
 */
export interface QuestionnaireItem extends BackboneElement {
    /**
     * For items that have a defined set of allowed answers (via answerOption or answerValueset), indicates whether values *other* than those specified can be selected.
     */
    answerConstraint?: ('optionsOnly' | 'optionsOrType' | 'optionsOrString') | undefined;
    _answerConstraint?: Element | undefined;
    /**
     * This element can be used when the value set machinery of answerValueSet is deemed too cumbersome or when there's a need to capture possible answers that are not codes.
     */
    answerOption?: QuestionnaireItemAnswerOption[] | undefined;
    /**
     * LOINC defines many useful value sets for questionnaire responses. See [LOINC Answer Lists](https://terminology.hl7.org/LOINC.html). The value may come from the ElementDefinition referred to by .definition.
     */
    answerValueSet?: string | undefined;
    _answerValueSet?: Element | undefined;
    /**
     * The value may come from the ElementDefinition referred to by .definition.
     */
    code?: Coding[] | undefined;
    /**
     * The uri refers to an ElementDefinition in a [StructureDefinition](structuredefinition.html#) or to an [ObservationDefinition](observationdefinition.html) and always starts with the [canonical URL](references.html#canonical) for the target resource. When referring to a StructureDefinition, a fragment identifier is used to specify the element definition by its id [Element.id](types-definitions.html#Element.id). E.g. http://hl7.org/fhir/StructureDefinition/Observation#Observation.value[x]. In the absence of a fragment identifier, the first/root element definition in the target is the matching element definition.
     */
    definition?: string | undefined;
    _definition?: Element | undefined;
    /**
     * If not specified, rendering behavior is up to the client.  This element is only meaningful if enableWhen or an equivalent extension is present
     */
    disabledDisplay?: ('hidden' | 'protected') | undefined;
    _disabledDisplay?: Element | undefined;
    /**
     * This element must be specified if more than one enableWhen value is provided.
     */
    enableBehavior?: ('all' | 'any') | undefined;
    _enableBehavior?: Element | undefined;
    /**
     * If multiple repetitions of this extension are present, the interpretation is driven by enableBehavior (either all repetitions must evaluate to true for this item to be enabled, or only one must evaluate to true for the item to be enabled).  If the enableWhen.question has multiple answers, the condition evaluates to true if *any* of the answers for the referenced item match the enableWhen condition.  This element is a modifier because if enableWhen is present for an item, "required" is ignored unless one of the enableWhen conditions is met. When an item is disabled, all of its descendants are disabled, regardless of what their own enableWhen logic might evaluate to.  If enableWhen logic depends on an item that is disabled, the logic should proceed as though the item is not valued - even if a default value or other value might be retained in memory in the event of the item being re-enabled.  In some cases, the comparison between the indicated answer and the specified value may differ only by precision.  For example, the enableWhen might be Q1 > 1970, but the answer to Q1 is 1970-10-15.  There is not a clear answer as to whether 1970-10-15 should be considered 'greater' than 1970, given that it is an imprecise value.  In these indeterminate situations, the form filler has the option of refusing to render the form.  If the form **is** displayed, items where enableWhen is indeterminate SHOULD be treated as enabled with a warning indicating that the questionnaire logic was faulty and it is possible that the item should not be enabled.  Questionnaires SHOULD be designed to take into account challenges around varying precision to minimize non-deterministic situations by setting constraints around expected precision, etc.
     */
    enableWhen?: QuestionnaireItemEnableWhen[] | undefined;
    /**
     * The user is allowed to change the value and override the default (unless marked as read-only). If the user doesn't change the value, then this initial value will be persisted when the QuestionnaireResponse is initially created.  Note that initial values can influence results.  The data type of initial.answer[x] must agree with the item.type, and only repeating items can have more then one initial value.
     */
    initial?: QuestionnaireItemInitial[] | undefined;
    /**
     * There is no specified limit to the depth of nesting.  However, Questionnaire authors are encouraged to consider the impact on the user and user interface of overly deep nesting.
     */
    item?: QuestionnaireItem[] | undefined;
    /**
     * This ''can'' be a meaningful identifier (e.g. a LOINC code) but is not intended to have any meaning.  GUIDs or sequential numbers are appropriate here.
     */
    linkId: string;
    _linkId?: Element | undefined;
    /**
     * For base64binary, reflects the number of characters representing the encoded data, not the number of bytes of the binary data. The value may come from the ElementDefinition referred to by .definition.
     */
    maxLength?: number | undefined;
    /**
     * These are generally unique within a questionnaire, though this is not guaranteed. Some questionnaires may have multiple questions with the same label with logic to control which gets exposed.  Typically, these won't be used for "display" items, though such use is not prohibited.  Systems SHOULD NOT generate their own prefixes if prefixes are defined for any items within a Questionnaire.
     */
    prefix?: string | undefined;
    _prefix?: Element | undefined;
    /**
     * If specified on a 'group', then all items beneath the specified group are read only.  For questions, this only marks the answer associated with the specific item read only.  Descendant questions are not impacted.  The value of readOnly elements can be established by asserting extensions for defaultValues, linkages that support pre-population and/or extensions that support calculation based on other answers.
     */
    readOnly?: boolean | undefined;
    _readOnly?: Element | undefined;
    /**
     * If a question is marked as repeats=true, then multiple answers can be provided for the question in the corresponding QuestionnaireResponse.  When rendering the questionnaire, it is up to the rendering software whether to render the question text for each answer repetition (i.e. "repeat the question") or to simply allow entry/selection of multiple answers for the question (repeat the answers).  Which is most appropriate visually may depend on the type of answer as well as whether there are nested items.
     * The resulting QuestionnaireResponse will be populated the same way regardless of rendering - one 'question' item with multiple answer values.
     *  The value may come from the ElementDefinition referred to by .definition.  When repeats=true for a group, it'll be represented with multiple items with the same linkId in the QuestionnaireResponse.  For a question, it'll be represented by a single item with that linkId with multiple answers.
     */
    repeats?: boolean | undefined;
    _repeats?: Element | undefined;
    /**
     * If the required item is a question, it must have a direct answer (i.e. an answer to the question itself, not merely answers to child questions) in order for the QuestionnaireResponse to be complete.  If the required item is a group, it must have at least one descendant question which has an answer  Questionnaire.item.required only has meaning for elements that are conditionally enabled with enableWhen if the condition evaluates to true.  It also only has meaning if the parent element is present.  If a non-required 'group' item contains a 'required' question item, it's completely fine to omit the group (because it's not required) despite it having a required child.  Similarly, if an item that contains other items is marked as required, that does not automatically make the contained elements required (though required groups must contain at least one descendant item with a populated answer). The value for 'required' may come from the ElementDefinition referred to by .definition.
     */
    required?: boolean | undefined;
    _required?: Element | undefined;
    /**
     * When using this element to represent the name of a section, use group type item and also make sure to limit the text element to a short string suitable for display as a section heading.  Group item instructions should be included as a display type item within the group.
     */
    text?: string | undefined;
    _text?: Element | undefined;
    /**
     * Additional constraints on the type of answer can be conveyed by extensions. The value may come from the ElementDefinition referred to by .definition.
     */
    type:
        | 'group'
        | 'display'
        | 'question'
        | 'boolean'
        | 'decimal'
        | 'integer'
        | 'date'
        | 'dateTime'
        | 'time'
        | 'string'
        | 'text'
        | 'url'
        | 'coding'
        | 'attachment'
        | 'reference'
        | 'quantity';
    _type?: Element | undefined;
}
/**
 * A structured set of questions intended to guide the collection of answers from end-users. Questionnaires provide detailed control over order, presentation, phraseology and grouping to allow coherent, consistent data collection.
 */
export interface Questionnaire extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'Questionnaire';
    /**
     * The 'date' element may be more recent than the approval date because of minor changes or editorial corrections.
     */
    approvalDate?: string | undefined;
    _approvalDate?: Element | undefined;
    /**
     * An identifier for this question or group of questions in a particular terminology such as LOINC.
     */
    code?: Coding[] | undefined;
    /**
     * May be a web site, an email address, a telephone number, etc.
     */
    contact?: ContactDetail[] | undefined;
    /**
     * ...
     */
    copyright?: string | undefined;
    _copyright?: Element | undefined;
    /**
     * The (c) symbol should NOT be included in this string. It will be added by software when rendering the notation. Full details about licensing, restrictions, warrantees, etc. goes in the more general 'copyright' element.
     */
    copyrightLabel?: string | undefined;
    _copyrightLabel?: Element | undefined;
    /**
     * Note that this is not the same as the resource last-modified-date, since the resource may be a secondary representation of the questionnaire. Additional specific dates may be added as extensions or be found by consulting Provenances associated with past versions of the resource.
     */
    date?: string | undefined;
    _date?: Element | undefined;
    /**
     * The URL of a Questionnaire that this Questionnaire is based on.
     */
    derivedFrom?: string[] | undefined;
    _derivedFrom?: Element[] | undefined;
    /**
     * This description can be used to capture details such as comments about misuse, instructions for clinical use and interpretation, literature references, examples from the paper world, etc. It is not a rendering of the questionnaire as conveyed in the 'text' field of the resource itself. This item SHOULD be populated unless the information is available from context (e.g. the language of the questionnaire is presumed to be the predominant language in the place the questionnaire was created).
     */
    description?: string | undefined;
    _description?: Element | undefined;
    /**
     * The effective period for a questionnaire  determines when the content is applicable for usage and is independent of publication and review dates. For example, a measure intended to be used for the year 2016 might be published in 2015.
     */
    effectivePeriod?: Period | undefined;
    /**
     * Allows filtering of questionnaires that are appropriate for use versus not.
     */
    experimental?: boolean | undefined;
    _experimental?: Element | undefined;
    /**
     * Typically, this is used for identifiers that can go in an HL7 V3 II (instance identifier) data type, and can then identify this questionnaire outside of FHIR, where it is not possible to use the logical URI.
     */
    identifier?: Identifier[] | undefined;
    /**
     * The content of the questionnaire is constructed from an ordered, hierarchical collection of items.
     */
    item?: QuestionnaireItem[] | undefined;
    /**
     * It may be possible for the questionnaire to be used in jurisdictions other than those for which it was originally designed or intended.
     */
    jurisdiction?: CodeableConcept[] | undefined;
    /**
     * If specified, this date follows the original approval date.
     */
    lastReviewDate?: string | undefined;
    _lastReviewDate?: Element | undefined;
    /**
     * The name is not expected to be globally unique. The name should be a simple alphanumeric type name to ensure that it is machine-processing friendly.
     */
    name?: string | undefined;
    _name?: Element | undefined;
    /**
     * Usually an organization but may be an individual. The publisher (or steward) of the questionnaire is the organization or individual primarily responsible for the maintenance and upkeep of the questionnaire. This is not necessarily the same individual or organization that developed and initially authored the content. The publisher is the primary point of contact for questions or issues with the questionnaire. This item SHOULD be populated unless the information is available from context.
     */
    publisher?: string | undefined;
    _publisher?: Element | undefined;
    /**
     * This element does not describe the usage of the questionnaire. Instead, it provides traceability of ''why'' the resource is either needed or ''why'' it is defined as it is.  This may be used to point to source materials or specifications that drove the structure of this questionnaire.
     */
    purpose?: string | undefined;
    _purpose?: Element | undefined;
    /**
     * Allows filtering of questionnaires that are appropriate for use versus not.
     */
    status: 'draft' | 'active' | 'retired' | 'unknown';
    _status?: Element | undefined;
    /**
     * If none are specified, then the subject is unlimited.
     */
    subjectType?: string[] | undefined;
    _subjectType?: Element[] | undefined;
    /**
     * This name does not need to be machine-processing friendly and may contain punctuation, white-space, etc.
     */
    title?: string | undefined;
    _title?: Element | undefined;
    /**
     * The name of the referenced questionnaire can be conveyed using the http://hl7.org/fhir/StructureDefinition/display extension.
     */
    url?: string | undefined;
    _url?: Element | undefined;
    /**
     * When multiple useContexts are specified, there is no expectation that all or any of the contexts apply.
     */
    useContext?: UsageContext[] | undefined;
    /**
     * There may be different questionnaire instances that have the same identifier but different versions.  The version can be appended to the url in a reference to allow a reference to a particular business version of the questionnaire with the format [url]|[version]. The version SHOULD NOT contain a '#' - see [Business Version](resource.html#bv-format).
     */
    version?: string | undefined;
    _version?: Element | undefined;
    /**
     * If set as a string, this is a FHIRPath expression that has two additional context variables passed in - %version1 and %version2 and will return a negative number if version1 is newer, a positive number if version2 and a 0 if the version ordering can't be successfully be determined.
     */
    versionAlgorithmString?: string | undefined;
    _versionAlgorithmString?: Element | undefined;
    /**
     * If set as a string, this is a FHIRPath expression that has two additional context variables passed in - %version1 and %version2 and will return a negative number if version1 is newer, a positive number if version2 and a 0 if the version ordering can't be successfully be determined.
     */
    versionAlgorithmCoding?: Coding | undefined;
}
/**
 * The value is nested because we cannot have a repeating structure that has variable type.
 */
export interface QuestionnaireResponseItemAnswer extends BackboneElement {
    /**
     * Only used when nesting beneath a question - see item.item for nesting beneath groups
     */
    item?: QuestionnaireResponseItem[] | undefined;
    /**
     * More complex structures (Attachment, Resource and Quantity) will typically be limited to electronic forms that can expose an appropriate user interface to capture the components and enforce the constraints of a complex data type.  Additional complex types can be introduced through extensions. Must match the datatype specified by Questionnaire.item.type in the corresponding Questionnaire.     Note that a question is answered using one of the possible choices defined with answerOption, answerValueSet or some other means and the answer has a complex data type, all elements within the answer in the QuestionnaireResponse **SHOULD** match the elements defined corresponding choice value in the Questionnaire.  However, it is possible that not all elements will be propagated.  Also, some systems might use language translations resulting in different displays.  Comparison of value to the values defined in the Questionnaire (whether by answerOption, answerValueSet or answerExpression) **SHALL NOT** pay attention to Coding.display, Reference.display, Quantity.unit unless those are the only elements present.  As well, systems are not required to check for a match on any extensions (e.g. ordinal values, translations, etc).  Systems **MAY** enforce that if extensions such as ordinal values are present in both Questionnaire and QuestionnaireResponse, they match.
     */
    valueBoolean?: boolean | undefined;
    _valueBoolean?: Element | undefined;
    /**
     * More complex structures (Attachment, Resource and Quantity) will typically be limited to electronic forms that can expose an appropriate user interface to capture the components and enforce the constraints of a complex data type.  Additional complex types can be introduced through extensions. Must match the datatype specified by Questionnaire.item.type in the corresponding Questionnaire.     Note that a question is answered using one of the possible choices defined with answerOption, answerValueSet or some other means and the answer has a complex data type, all elements within the answer in the QuestionnaireResponse **SHOULD** match the elements defined corresponding choice value in the Questionnaire.  However, it is possible that not all elements will be propagated.  Also, some systems might use language translations resulting in different displays.  Comparison of value to the values defined in the Questionnaire (whether by answerOption, answerValueSet or answerExpression) **SHALL NOT** pay attention to Coding.display, Reference.display, Quantity.unit unless those are the only elements present.  As well, systems are not required to check for a match on any extensions (e.g. ordinal values, translations, etc).  Systems **MAY** enforce that if extensions such as ordinal values are present in both Questionnaire and QuestionnaireResponse, they match.
     */
    valueDecimal?: number | undefined;
    /**
     * More complex structures (Attachment, Resource and Quantity) will typically be limited to electronic forms that can expose an appropriate user interface to capture the components and enforce the constraints of a complex data type.  Additional complex types can be introduced through extensions. Must match the datatype specified by Questionnaire.item.type in the corresponding Questionnaire.     Note that a question is answered using one of the possible choices defined with answerOption, answerValueSet or some other means and the answer has a complex data type, all elements within the answer in the QuestionnaireResponse **SHOULD** match the elements defined corresponding choice value in the Questionnaire.  However, it is possible that not all elements will be propagated.  Also, some systems might use language translations resulting in different displays.  Comparison of value to the values defined in the Questionnaire (whether by answerOption, answerValueSet or answerExpression) **SHALL NOT** pay attention to Coding.display, Reference.display, Quantity.unit unless those are the only elements present.  As well, systems are not required to check for a match on any extensions (e.g. ordinal values, translations, etc).  Systems **MAY** enforce that if extensions such as ordinal values are present in both Questionnaire and QuestionnaireResponse, they match.
     */
    valueInteger?: number | undefined;
    /**
     * More complex structures (Attachment, Resource and Quantity) will typically be limited to electronic forms that can expose an appropriate user interface to capture the components and enforce the constraints of a complex data type.  Additional complex types can be introduced through extensions. Must match the datatype specified by Questionnaire.item.type in the corresponding Questionnaire.     Note that a question is answered using one of the possible choices defined with answerOption, answerValueSet or some other means and the answer has a complex data type, all elements within the answer in the QuestionnaireResponse **SHOULD** match the elements defined corresponding choice value in the Questionnaire.  However, it is possible that not all elements will be propagated.  Also, some systems might use language translations resulting in different displays.  Comparison of value to the values defined in the Questionnaire (whether by answerOption, answerValueSet or answerExpression) **SHALL NOT** pay attention to Coding.display, Reference.display, Quantity.unit unless those are the only elements present.  As well, systems are not required to check for a match on any extensions (e.g. ordinal values, translations, etc).  Systems **MAY** enforce that if extensions such as ordinal values are present in both Questionnaire and QuestionnaireResponse, they match.
     */
    valueDate?: string | undefined;
    _valueDate?: Element | undefined;
    /**
     * More complex structures (Attachment, Resource and Quantity) will typically be limited to electronic forms that can expose an appropriate user interface to capture the components and enforce the constraints of a complex data type.  Additional complex types can be introduced through extensions. Must match the datatype specified by Questionnaire.item.type in the corresponding Questionnaire.     Note that a question is answered using one of the possible choices defined with answerOption, answerValueSet or some other means and the answer has a complex data type, all elements within the answer in the QuestionnaireResponse **SHOULD** match the elements defined corresponding choice value in the Questionnaire.  However, it is possible that not all elements will be propagated.  Also, some systems might use language translations resulting in different displays.  Comparison of value to the values defined in the Questionnaire (whether by answerOption, answerValueSet or answerExpression) **SHALL NOT** pay attention to Coding.display, Reference.display, Quantity.unit unless those are the only elements present.  As well, systems are not required to check for a match on any extensions (e.g. ordinal values, translations, etc).  Systems **MAY** enforce that if extensions such as ordinal values are present in both Questionnaire and QuestionnaireResponse, they match.
     */
    valueDateTime?: string | undefined;
    _valueDateTime?: Element | undefined;
    /**
     * More complex structures (Attachment, Resource and Quantity) will typically be limited to electronic forms that can expose an appropriate user interface to capture the components and enforce the constraints of a complex data type.  Additional complex types can be introduced through extensions. Must match the datatype specified by Questionnaire.item.type in the corresponding Questionnaire.     Note that a question is answered using one of the possible choices defined with answerOption, answerValueSet or some other means and the answer has a complex data type, all elements within the answer in the QuestionnaireResponse **SHOULD** match the elements defined corresponding choice value in the Questionnaire.  However, it is possible that not all elements will be propagated.  Also, some systems might use language translations resulting in different displays.  Comparison of value to the values defined in the Questionnaire (whether by answerOption, answerValueSet or answerExpression) **SHALL NOT** pay attention to Coding.display, Reference.display, Quantity.unit unless those are the only elements present.  As well, systems are not required to check for a match on any extensions (e.g. ordinal values, translations, etc).  Systems **MAY** enforce that if extensions such as ordinal values are present in both Questionnaire and QuestionnaireResponse, they match.
     */
    valueTime?: string | undefined;
    _valueTime?: Element | undefined;
    /**
     * More complex structures (Attachment, Resource and Quantity) will typically be limited to electronic forms that can expose an appropriate user interface to capture the components and enforce the constraints of a complex data type.  Additional complex types can be introduced through extensions. Must match the datatype specified by Questionnaire.item.type in the corresponding Questionnaire.     Note that a question is answered using one of the possible choices defined with answerOption, answerValueSet or some other means and the answer has a complex data type, all elements within the answer in the QuestionnaireResponse **SHOULD** match the elements defined corresponding choice value in the Questionnaire.  However, it is possible that not all elements will be propagated.  Also, some systems might use language translations resulting in different displays.  Comparison of value to the values defined in the Questionnaire (whether by answerOption, answerValueSet or answerExpression) **SHALL NOT** pay attention to Coding.display, Reference.display, Quantity.unit unless those are the only elements present.  As well, systems are not required to check for a match on any extensions (e.g. ordinal values, translations, etc).  Systems **MAY** enforce that if extensions such as ordinal values are present in both Questionnaire and QuestionnaireResponse, they match.
     */
    valueString?: string | undefined;
    _valueString?: Element | undefined;
    /**
     * More complex structures (Attachment, Resource and Quantity) will typically be limited to electronic forms that can expose an appropriate user interface to capture the components and enforce the constraints of a complex data type.  Additional complex types can be introduced through extensions. Must match the datatype specified by Questionnaire.item.type in the corresponding Questionnaire.     Note that a question is answered using one of the possible choices defined with answerOption, answerValueSet or some other means and the answer has a complex data type, all elements within the answer in the QuestionnaireResponse **SHOULD** match the elements defined corresponding choice value in the Questionnaire.  However, it is possible that not all elements will be propagated.  Also, some systems might use language translations resulting in different displays.  Comparison of value to the values defined in the Questionnaire (whether by answerOption, answerValueSet or answerExpression) **SHALL NOT** pay attention to Coding.display, Reference.display, Quantity.unit unless those are the only elements present.  As well, systems are not required to check for a match on any extensions (e.g. ordinal values, translations, etc).  Systems **MAY** enforce that if extensions such as ordinal values are present in both Questionnaire and QuestionnaireResponse, they match.
     */
    valueUri?: string | undefined;
    _valueUri?: Element | undefined;
    /**
     * More complex structures (Attachment, Resource and Quantity) will typically be limited to electronic forms that can expose an appropriate user interface to capture the components and enforce the constraints of a complex data type.  Additional complex types can be introduced through extensions. Must match the datatype specified by Questionnaire.item.type in the corresponding Questionnaire.     Note that a question is answered using one of the possible choices defined with answerOption, answerValueSet or some other means and the answer has a complex data type, all elements within the answer in the QuestionnaireResponse **SHOULD** match the elements defined corresponding choice value in the Questionnaire.  However, it is possible that not all elements will be propagated.  Also, some systems might use language translations resulting in different displays.  Comparison of value to the values defined in the Questionnaire (whether by answerOption, answerValueSet or answerExpression) **SHALL NOT** pay attention to Coding.display, Reference.display, Quantity.unit unless those are the only elements present.  As well, systems are not required to check for a match on any extensions (e.g. ordinal values, translations, etc).  Systems **MAY** enforce that if extensions such as ordinal values are present in both Questionnaire and QuestionnaireResponse, they match.
     */
    valueAttachment?: Attachment | undefined;
    /**
     * More complex structures (Attachment, Resource and Quantity) will typically be limited to electronic forms that can expose an appropriate user interface to capture the components and enforce the constraints of a complex data type.  Additional complex types can be introduced through extensions. Must match the datatype specified by Questionnaire.item.type in the corresponding Questionnaire.     Note that a question is answered using one of the possible choices defined with answerOption, answerValueSet or some other means and the answer has a complex data type, all elements within the answer in the QuestionnaireResponse **SHOULD** match the elements defined corresponding choice value in the Questionnaire.  However, it is possible that not all elements will be propagated.  Also, some systems might use language translations resulting in different displays.  Comparison of value to the values defined in the Questionnaire (whether by answerOption, answerValueSet or answerExpression) **SHALL NOT** pay attention to Coding.display, Reference.display, Quantity.unit unless those are the only elements present.  As well, systems are not required to check for a match on any extensions (e.g. ordinal values, translations, etc).  Systems **MAY** enforce that if extensions such as ordinal values are present in both Questionnaire and QuestionnaireResponse, they match.
     */
    valueCoding?: Coding | undefined;
    /**
     * More complex structures (Attachment, Resource and Quantity) will typically be limited to electronic forms that can expose an appropriate user interface to capture the components and enforce the constraints of a complex data type.  Additional complex types can be introduced through extensions. Must match the datatype specified by Questionnaire.item.type in the corresponding Questionnaire.     Note that a question is answered using one of the possible choices defined with answerOption, answerValueSet or some other means and the answer has a complex data type, all elements within the answer in the QuestionnaireResponse **SHOULD** match the elements defined corresponding choice value in the Questionnaire.  However, it is possible that not all elements will be propagated.  Also, some systems might use language translations resulting in different displays.  Comparison of value to the values defined in the Questionnaire (whether by answerOption, answerValueSet or answerExpression) **SHALL NOT** pay attention to Coding.display, Reference.display, Quantity.unit unless those are the only elements present.  As well, systems are not required to check for a match on any extensions (e.g. ordinal values, translations, etc).  Systems **MAY** enforce that if extensions such as ordinal values are present in both Questionnaire and QuestionnaireResponse, they match.
     */
    valueQuantity?: Quantity | undefined;
    /**
     * More complex structures (Attachment, Resource and Quantity) will typically be limited to electronic forms that can expose an appropriate user interface to capture the components and enforce the constraints of a complex data type.  Additional complex types can be introduced through extensions. Must match the datatype specified by Questionnaire.item.type in the corresponding Questionnaire.     Note that a question is answered using one of the possible choices defined with answerOption, answerValueSet or some other means and the answer has a complex data type, all elements within the answer in the QuestionnaireResponse **SHOULD** match the elements defined corresponding choice value in the Questionnaire.  However, it is possible that not all elements will be propagated.  Also, some systems might use language translations resulting in different displays.  Comparison of value to the values defined in the Questionnaire (whether by answerOption, answerValueSet or answerExpression) **SHALL NOT** pay attention to Coding.display, Reference.display, Quantity.unit unless those are the only elements present.  As well, systems are not required to check for a match on any extensions (e.g. ordinal values, translations, etc).  Systems **MAY** enforce that if extensions such as ordinal values are present in both Questionnaire and QuestionnaireResponse, they match.
     */
    valueReference?: Reference | undefined;
}
/**
 * Groups cannot have answers and therefore must nest directly within item. When dealing with questions, nesting must occur within each answer because some questions may have multiple answers (and the nesting occurs for each answer).\nWhen dealing with repeating items, each group repetition will be handled by a separate item.  However, repeating questions are handled with a single question item and potentially multiple answers.
 */
export interface QuestionnaireResponseItem extends BackboneElement {
    /**
     * The value is nested because we cannot have a repeating structure that has variable type.
     */
    answer?: QuestionnaireResponseItemAnswer[] | undefined;
    /**
     * The ElementDefinition must be in a [StructureDefinition](structuredefinition.html#), and must have a fragment identifier that identifies the specific data element by its id (Element.id). E.g. http://hl7.org/fhir/StructureDefinition/Observation#Observation.value[x].
     * There is no need for this element if the item pointed to by the linkId has a definition listed.
     */
    definition?: string | undefined;
    _definition?: Element | undefined;
    /**
     * Only used when nesting beneath a group - see item.answer.item for nesting beneath questions
     */
    item?: QuestionnaireResponseItem[] | undefined;
    /**
     * The item from the Questionnaire that corresponds to this item in the QuestionnaireResponse resource.
     */
    linkId: string;
    _linkId?: Element | undefined;
    /**
     * The text for an item SHOULD be identical to the text from the corresponding Questionnaire.item. This can't be strictly enforced because it's possible for the Questionnaire to be updated subsequent to the QuestionnaireResponse having been created, however the intention is that the text in the QuestionnaireResponse reflects what the user saw when completing the Questionnaire.
     */
    text?: string | undefined;
    _text?: Element | undefined;
}
/**
 * A structured set of questions and their answers. The questions are ordered and grouped into coherent subsets, corresponding to the structure of the grouping of the questionnaire being responded to.
 */
export interface QuestionnaireResponse extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'QuestionnaireResponse';
    /**
     * Mapping a subject's answers to multiple choice options and determining what to put in the textual answer is a matter of interpretation. Authoring by device would indicate that some portion of the questionnaire had been auto-populated. Device should only be used if it directly determined the answers, not if it was merely used as a capture tool to record answers provided by others. In the latter case, information about the physical device, software, etc. would be captured using Provenance.
     */
    author?: Reference | undefined;
    /**
     * May be different from the lastUpdateTime of the resource itself, because that reflects when the data was known to the server, not when the data was captured.
     * This element is optional to allow for systems that might not know the value, however it SHOULD be populated if possible.
     */
    authored?: string | undefined;
    _authored?: Element | undefined;
    /**
     * The order, proposal or plan that is fulfilled in whole or in part by this QuestionnaireResponse.  For example, a ServiceRequest seeking an intake assessment or a decision support recommendation to assess for post-partum depression.
     */
    basedOn?: Reference[] | undefined;
    /**
     * This will typically be the encounter the event occurred within, but some activities may be initiated prior to or after the official completion of an encounter but still be tied to the context of the encounter. A questionnaire that was initiated during an encounter but not fully completed during the encounter would still generally be associated with the encounter.
     */
    encounter?: Reference | undefined;
    /**
     * A business identifier assigned to a particular completed (or partially completed) questionnaire.
     */
    identifier?: Identifier[] | undefined;
    /**
     * Groups cannot have answers and therefore must nest directly within item. When dealing with questions, nesting must occur within each answer because some questions may have multiple answers (and the nesting occurs for each answer).\nWhen dealing with repeating items, each group repetition will be handled by a separate item.  However, repeating questions are handled with a single question item and potentially multiple answers.
     */
    item?: QuestionnaireResponseItem[] | undefined;
    /**
     * Composition of questionnaire responses will be handled by the parent questionnaire having answers that reference the child questionnaire.  For relationships to referrals, and other types of requests, use basedOn.
     */
    partOf?: Reference[] | undefined;
    /**
     * If a QuestionnaireResponse references a Questionnaire that can be resolved, then the QuestionnaireResponse structure must be consistent with the Questionnaire (i.e. questions must be organized into the same groups, nested questions must still be nested, etc.).  It is possible to have a QuestionnaireResponse whose 'questionnaire' element does not resolve.  It is also possible for the questionnaire element to not have a value but only extensions (e.g. conveying the title or identifier for the questionnaire).  This may happen for legacy data.  If there is no formally defined Questionnaire, it is undefined what the 'correct' values for the linkId elements should be and it is possible that linkIds might be inconsistent for QuestionnaireResponses for the same form if captured by distinct systems.
     */
    questionnaire: string;
    _questionnaire?: Element | undefined;
    /**
     * If not specified, no inference can be made about who provided the data. Device should only be used if it directly determined the answers, not if it was merely used as a capture tool to record answers provided by others. In the latter case, information about the physical device, software, etc. would be captured using Provenance.
     */
    source?: Reference | undefined;
    /**
     * This element is labeled as a modifier because the status contains codes that mark the resource as not currently valid.
     */
    status: 'in-progress' | 'completed' | 'amended' | 'entered-in-error' | 'stopped';
    _status?: Element | undefined;
    /**
     * If the Questionnaire declared a subjectType, the resource pointed to by this element must be an instance of one of the listed types.
     */
    subject?: Reference | undefined;
}
/**
 * The case or regulatory procedure for granting or amending a regulated authorization. An authorization is granted in response to submissions/applications by those seeking authorization. A case is the administrative process that deals with the application(s) that relate to this and assesses them. Note: This area is subject to ongoing review and the workgroup is seeking implementer feedback on its use (see link at bottom of page).
 */
export interface RegulatedAuthorizationCase extends BackboneElement {
    /**
     * A regulatory submission from an organization to a regulator, as part of an assessing case. Multiple applications may occur over time, with more or different information to support or modify the submission or the authorization. The applications can be considered as steps within the longer running case or procedure for this authorization process.
     */
    application?: RegulatedAuthorizationCase[] | undefined;
    /**
     * Relevant date for this case.
     */
    datePeriod?: Period | undefined;
    /**
     * Relevant date for this case.
     */
    dateDateTime?: string | undefined;
    _dateDateTime?: Element | undefined;
    /**
     * Identifier by which this case can be referenced.
     */
    identifier?: Identifier | undefined;
    /**
     * The status associated with the case.
     */
    status?: CodeableConcept | undefined;
    /**
     * The defining type of case.
     */
    type?: CodeableConcept | undefined;
}
/**
 * Regulatory approval, clearance or licencing related to a regulated product, treatment, facility or activity that is cited in a guidance, regulation, rule or legislative act. An example is Market Authorization relating to a Medicinal Product.
 */
export interface RegulatedAuthorization extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'RegulatedAuthorization';
    /**
     * Additional information or supporting documentation about the authorization.
     */
    attachedDocument?: Reference[] | undefined;
    /**
     * The legal or regulatory framework against which this authorization is granted, or other reasons for it.
     */
    basis?: CodeableConcept[] | undefined;
    /**
     * The case or regulatory procedure for granting or amending a regulated authorization. An authorization is granted in response to submissions/applications by those seeking authorization. A case is the administrative process that deals with the application(s) that relate to this and assesses them. Note: This area is subject to ongoing review and the workgroup is seeking implementer feedback on its use (see link at bottom of page).
     */
    case?: RegulatedAuthorizationCase | undefined;
    /**
     * General textual supporting information.
     */
    description?: string | undefined;
    _description?: Element | undefined;
    /**
     * The organization that has been granted this authorization, by some authoritative body (the 'regulator').
     */
    holder?: Reference | undefined;
    /**
     * Business identifier for the authorization, typically assigned by the authorizing body.
     */
    identifier?: Identifier[] | undefined;
    /**
     * Condition for which the use of the regulated product applies.
     */
    indication?: CodeableReference[] | undefined;
    /**
     * The intended use of the product, e.g. prevention, treatment, diagnosis.
     */
    intendedUse?: CodeableConcept | undefined;
    /**
     * The territory (e.g., country, jurisdiction etc.) in which the authorization has been granted.
     */
    region?: CodeableConcept[] | undefined;
    /**
     * The regulatory authority or authorizing body granting the authorization. For example, European Medicines Agency (EMA), Food and Drug Administration (FDA), Health Canada (HC), etc.
     */
    regulator?: Reference | undefined;
    /**
     * The status that is authorised e.g. approved. Intermediate states and actions can be tracked with cases and applications.
     */
    status?: CodeableConcept | undefined;
    /**
     * The date at which the current status was assigned.
     */
    statusDate?: string | undefined;
    _statusDate?: Element | undefined;
    /**
     * The product type, treatment, facility or activity that is being authorized.
     */
    subject?: Reference[] | undefined;
    /**
     * Overall type of this authorization, for example drug marketing approval, orphan drug designation.
     */
    type?: CodeableConcept | undefined;
    /**
     * The time period in which the regulatory approval, clearance or licencing is in effect. As an example, a Marketing Authorization includes the date of authorization and/or an expiration date.
     */
    validityPeriod?: Period | undefined;
}
/**
 * If no language is specified, this *implies* that the default local language is spoken.  If you need to convey proficiency for multiple modes, then you need multiple RelatedPerson.Communication associations.   If the RelatedPerson does not speak the default local language, then the Interpreter Required Standard can be used to explicitly declare that an interpreter is required.
 */
export interface RelatedPersonCommunication extends BackboneElement {
    /**
     * The structure aa-BB with this exact casing is one the most widely used notations for locale. However not all systems actually code this but instead have it as free text. Hence CodeableConcept instead of code as the data type.
     */
    language: CodeableConcept;
    /**
     * This language is specifically identified for communicating healthcare information.
     */
    preferred?: boolean | undefined;
    _preferred?: Element | undefined;
}
/**
 * Information about a person that is involved in a patient's health or the care for a patient, but who is not the target of healthcare, nor has a formal responsibility in the care process.
 */
export interface RelatedPerson extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'RelatedPerson';
    /**
     * This element is labeled as a modifier because it may be used to mark that the resource was created in error.
     */
    active?: boolean | undefined;
    _active?: Element | undefined;
    /**
     * Address where the related person can be contacted or visited.
     */
    address?: Address[] | undefined;
    /**
     * The date on which the related person was born.
     */
    birthDate?: string | undefined;
    _birthDate?: Element | undefined;
    /**
     * If no language is specified, this *implies* that the default local language is spoken.  If you need to convey proficiency for multiple modes, then you need multiple RelatedPerson.Communication associations.   If the RelatedPerson does not speak the default local language, then the Interpreter Required Standard can be used to explicitly declare that an interpreter is required.
     */
    communication?: RelatedPersonCommunication[] | undefined;
    /**
     * Administrative Gender - the gender that the person is considered to have for administration and record keeping purposes.
     */
    gender?: ('male' | 'female' | 'other' | 'unknown') | undefined;
    _gender?: Element | undefined;
    /**
     * RelatedPerson identifiers might not be unique across instances within a system, as a single human individual may be represented as many different RelatedPerson resources with different roles, periods, or relationships.
     */
    identifier?: Identifier[] | undefined;
    /**
     * A name associated with the person.
     */
    name?: HumanName[] | undefined;
    /**
     * The patient this person is related to.
     */
    patient: Reference;
    /**
     * If an individual has a relationship with a patient over multiple, non-adjacent periods, there should be a distinct RelatedPerson instance for each period.  For example, if a person is a roommate for a period of time, moves out, and is later a roommate with the same person again, you would have two RelatedPerson instances.
     */
    period?: Period | undefined;
    /**
     * Image of the person.
     */
    photo?: Attachment[] | undefined;
    /**
     * The directionality of the relationship is from the RelatedPerson to the Patient. For example, if the Patient is a child, and the RelatedPerson is the mother, the relationship would be PRN (parent) or MTH (mother).
     */
    relationship?: CodeableConcept[] | undefined;
    /**
     * Person may have multiple ways to be contacted with different uses or applicable periods.  May need to have options for contacting the person urgently, and also to help with identification.
     */
    telecom?: ContactPoint[] | undefined;
}
/**
 * When multiple conditions of the same kind are present, the effects are combined using AND semantics, so the overall condition is true only if all of the conditions are true.
 */
export interface RequestGroupActionCondition extends BackboneElement {
    /**
     * The expression may be inlined, or may be a reference to a named expression within a logic library referenced by the library element.
     */
    expression?: Expression | undefined;
    /**
     * Applicability criteria are used to determine immediate applicability when a plan definition is applied to a given context. Start and stop criteria are carried through application and used to describe enter/exit criteria for an action.
     */
    kind: 'applicability' | 'start' | 'stop';
    _kind?: Element | undefined;
}
/**
 * A relationship to another action such as "before" or "30-60 minutes after start of".
 */
export interface RequestGroupActionRelatedAction extends BackboneElement {
    /**
     * A duration or range of durations to apply to the relationship. For example, 30-60 minutes before.
     */
    offsetDuration?: Duration | undefined;
    /**
     * A duration or range of durations to apply to the relationship. For example, 30-60 minutes before.
     */
    offsetRange?: Range | undefined;
    /**
     * The relationship of this action to the related action.
     */
    relationship:
        | 'before-start'
        | 'before'
        | 'before-end'
        | 'concurrent-with-start'
        | 'concurrent'
        | 'concurrent-with-end'
        | 'after-start'
        | 'after'
        | 'after-end';
    _relationship?: Element | undefined;
    /**
     * The element id of the target related action.
     */
    targetId: string;
    _targetId?: Element | undefined;
}
/**
 * Because request groups represent potential options for performing activities, some specific participants may still be unknown, so this element allows for both definitional participants (in the same way they are specified in ActivityDefinition and PlanDefinition resources) as well as identifying specific participants when they are known.
 */
export interface RequestGroupActionParticipant extends BackboneElement {
    /**
     * A reference to the actual participant.
     */
    actor?: Reference | undefined;
    /**
     * Indicates how the actor will be involved in the action - author, reviewer, witness, etc.
     */
    function?: CodeableConcept | undefined;
    /**
     * The role the participant should play in performing the described action.
     */
    role?: CodeableConcept | undefined;
    /**
     * The type of participant in the action.
     */
    type?:
        | (
              | 'careteam'
              | 'device'
              | 'group'
              | 'healthcareservice'
              | 'location'
              | 'organization'
              | 'patient'
              | 'practitioner'
              | 'practitionerrole'
              | 'relatedperson'
          )
        | undefined;
    _type?: Element | undefined;
    /**
     * When this element is a reference, it SHOULD be a reference to a definitional resource (for example, a location type, rather than a specific location).
     */
    typeReference?: Reference | undefined;
}
/**
 * The actions, if any, produced by the evaluation of the artifact.
 */
export interface RequestGroupAction extends BackboneElement {
    /**
     * Sub actions.
     */
    action?: RequestGroupAction[] | undefined;
    /**
     * Defines whether the action can be selected multiple times.
     */
    cardinalityBehavior?: ('single' | 'multiple') | undefined;
    _cardinalityBehavior?: Element | undefined;
    /**
     * A code that provides meaning for the action or action group. For example, a section may have a LOINC code for a section of a documentation template.
     */
    code?: CodeableConcept[] | undefined;
    /**
     * When multiple conditions of the same kind are present, the effects are combined using AND semantics, so the overall condition is true only if all of the conditions are true.
     */
    condition?: RequestGroupActionCondition[] | undefined;
    /**
     * A short description of the action used to provide a summary to display to the user.
     */
    description?: string | undefined;
    _description?: Element | undefined;
    /**
     * Didactic or other informational resources associated with the action that can be provided to the CDS recipient. Information resources can include inline text commentary and links to web resources.
     */
    documentation?: RelatedArtifact[] | undefined;
    /**
     * Goals that are intended to be achieved by following the requests in this action.
     */
    goal?: Reference[] | undefined;
    /**
     * Defines the grouping behavior for the action and its children.
     */
    groupingBehavior?: ('visual-group' | 'logical-group' | 'sentence-group') | undefined;
    _groupingBehavior?: Element | undefined;
    /**
     * The linkId of the action from the PlanDefinition that corresponds to this action in the RequestGroup resource.
     */
    linkId?: string | undefined;
    _linkId?: Element | undefined;
    /**
     * May reference a specific clinical location or may just identify a type of location.
     */
    location?: CodeableReference | undefined;
    /**
     * Because request groups represent potential options for performing activities, some specific participants may still be unknown, so this element allows for both definitional participants (in the same way they are specified in ActivityDefinition and PlanDefinition resources) as well as identifying specific participants when they are known.
     */
    participant?: RequestGroupActionParticipant[] | undefined;
    /**
     * Defines whether the action should usually be preselected.
     */
    precheckBehavior?: ('yes' | 'no') | undefined;
    _precheckBehavior?: Element | undefined;
    /**
     * A user-visible prefix for the action. For example a section or item numbering such as 1. or A.
     */
    prefix?: string | undefined;
    _prefix?: Element | undefined;
    /**
     * Indicates how quickly the action should be addressed with respect to other actions.
     */
    priority?: ('routine' | 'urgent' | 'asap' | 'stat') | undefined;
    _priority?: Element | undefined;
    /**
     * A relationship to another action such as "before" or "30-60 minutes after start of".
     */
    relatedAction?: RequestGroupActionRelatedAction[] | undefined;
    /**
     * Defines expectations around whether an action is required.
     */
    requiredBehavior?: ('must' | 'could' | 'must-unless-documented') | undefined;
    _requiredBehavior?: Element | undefined;
    /**
     * The target resource SHALL be a [Request](request.html) resource with a Request.intent set to "option".
     */
    resource?: Reference | undefined;
    /**
     * Defines the selection behavior for the action and its children.
     */
    selectionBehavior?: ('any' | 'all' | 'all-or-none' | 'exactly-one' | 'at-most-one' | 'one-or-more') | undefined;
    _selectionBehavior?: Element | undefined;
    /**
     * A text equivalent of the action to be performed. This provides a human-interpretable description of the action when the definition is consumed by a system that might not be capable of interpreting it dynamically.
     */
    textEquivalent?: string | undefined;
    _textEquivalent?: Element | undefined;
    /**
     * The intent of the timing element is to provide timing for when the action should be performed. The timing may be absolute (specified as a dateTime or Period) or relative (specified as an Age, Duration, or Range), or it may be a more complex, potentially repeating timing specified using Timing.
     */
    timingDateTime?: string | undefined;
    _timingDateTime?: Element | undefined;
    /**
     * The intent of the timing element is to provide timing for when the action should be performed. The timing may be absolute (specified as a dateTime or Period) or relative (specified as an Age, Duration, or Range), or it may be a more complex, potentially repeating timing specified using Timing.
     */
    timingAge?: Age | undefined;
    /**
     * The intent of the timing element is to provide timing for when the action should be performed. The timing may be absolute (specified as a dateTime or Period) or relative (specified as an Age, Duration, or Range), or it may be a more complex, potentially repeating timing specified using Timing.
     */
    timingPeriod?: Period | undefined;
    /**
     * The intent of the timing element is to provide timing for when the action should be performed. The timing may be absolute (specified as a dateTime or Period) or relative (specified as an Age, Duration, or Range), or it may be a more complex, potentially repeating timing specified using Timing.
     */
    timingDuration?: Duration | undefined;
    /**
     * The intent of the timing element is to provide timing for when the action should be performed. The timing may be absolute (specified as a dateTime or Period) or relative (specified as an Age, Duration, or Range), or it may be a more complex, potentially repeating timing specified using Timing.
     */
    timingRange?: Range | undefined;
    /**
     * The intent of the timing element is to provide timing for when the action should be performed. The timing may be absolute (specified as a dateTime or Period) or relative (specified as an Age, Duration, or Range), or it may be a more complex, potentially repeating timing specified using Timing.
     */
    timingTiming?: Timing | undefined;
    /**
     * The title of the action displayed to a user.
     */
    title?: string | undefined;
    _title?: Element | undefined;
    /**
     * The type of action to perform (create, update, remove).
     */
    type?: CodeableConcept | undefined;
}
/**
 * A group of related requests that can be used to capture intended activities that have inter-dependencies such as "give this medication after that one".
 */
export interface RequestGroup extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'RequestGroup';
    /**
     * The actions, if any, produced by the evaluation of the artifact.
     */
    action?: RequestGroupAction[] | undefined;
    /**
     * Provides a reference to the author of the request group.
     */
    author?: Reference | undefined;
    /**
     * Indicates when the request group was created.
     */
    authoredOn?: string | undefined;
    _authoredOn?: Element | undefined;
    /**
     * A plan, proposal or order that is fulfilled in whole or in part by this request.
     */
    basedOn?: Reference[] | undefined;
    /**
     * This element can be used to provide a code that captures the meaning of the request group as a whole, as opposed to the code of the action element, which captures the meaning of the individual actions within the request group.
     */
    code?: CodeableConcept | undefined;
    /**
     * Describes the context of the request group, if any.
     */
    encounter?: Reference | undefined;
    /**
     * Goals that are intended to be achieved by following the requests in this RequestGroup.
     */
    goal?: Reference[] | undefined;
    /**
     * Requests are linked either by a "basedOn" relationship (i.e. one request is fulfilling another) or by having a common requisition.  Requests that are part of the same requisition are generally treated independently from the perspective of changing their state or maintaining them after initial creation.
     */
    groupIdentifier?: Identifier | undefined;
    /**
     * Allows a service to provide a unique, business identifier for the request.
     */
    identifier?: Identifier[] | undefined;
    /**
     * A canonical URL referencing a FHIR-defined protocol, guideline, orderset or other definition that is adhered to in whole or in part by this request.
     */
    instantiatesCanonical?: string[] | undefined;
    _instantiatesCanonical?: Element[] | undefined;
    /**
     * A URL referencing an externally defined protocol, guideline, orderset or other definition that is adhered to in whole or in part by this request.
     */
    instantiatesUri?: string[] | undefined;
    _instantiatesUri?: Element[] | undefined;
    /**
     * Indicates the level of authority/intentionality associated with the request and where the request fits into the workflow chain.
     */
    intent:
        | 'proposal'
        | 'plan'
        | 'directive'
        | 'order'
        | 'original-order'
        | 'reflex-order'
        | 'filler-order'
        | 'instance-order'
        | 'option';
    _intent?: Element | undefined;
    /**
     * Provides a mechanism to communicate additional information about the response.
     */
    note?: Annotation[] | undefined;
    /**
     * Indicates how quickly the request should be addressed with respect to other requests.
     */
    priority?: ('routine' | 'urgent' | 'asap' | 'stat') | undefined;
    _priority?: Element | undefined;
    /**
     * Describes the reason for the request group in coded or textual form.
     */
    reason?: CodeableReference[] | undefined;
    /**
     * The replacement could be because the initial request was immediately rejected (due to an issue) or because the previous request was completed, but the need for the action described by the request remains ongoing.
     */
    replaces?: Reference[] | undefined;
    /**
     * The current state of the request. For request groups, the status reflects the status of all the requests in the group.
     */
    status: 'draft' | 'active' | 'on-hold' | 'revoked' | 'completed' | 'entered-in-error' | 'unknown';
    _status?: Element | undefined;
    /**
     * The subject for which the request group was created.
     */
    subject?: Reference | undefined;
}
/**
 * When multiple conditions of the same kind are present, the effects are combined using AND semantics, so the overall condition is true only if all of the conditions are true.
 */
export interface RequestOrchestrationActionCondition extends BackboneElement {
    /**
     * The expression may be inlined, or may be a reference to a named expression within a logic library referenced by the library element.
     */
    expression?: Expression | undefined;
    /**
     * Applicability criteria are used to determine immediate applicability when a plan definition is applied to a given context. Start and stop criteria are carried through application and used to describe enter/exit criteria for an action.
     */
    kind: 'applicability' | 'start' | 'stop';
    _kind?: Element | undefined;
}
/**
 * Defines input data requirements for the action.
 */
export interface RequestOrchestrationActionInput extends BackboneElement {
    /**
     * The relatedData element allows indicating that an input to a parent action is an input to specific child actions. It also allows the output of one action to be identified as the input to a different action
     */
    relatedData?: string | undefined;
    _relatedData?: Element | undefined;
    /**
     * Defines the data that is to be provided as input to the action.
     */
    requirement?: DataRequirement | undefined;
    /**
     * A human-readable label for the data requirement used to label data flows in BPMN or similar diagrams. Also provides a human readable label when rendering the data requirement that conveys its purpose to human readers.
     */
    title?: string | undefined;
    _title?: Element | undefined;
}
/**
 * Defines the outputs of the action, if any.
 */
export interface RequestOrchestrationActionOutput extends BackboneElement {
    /**
     * The relatedData element allows indicating that the output of child action is also the output of a parent action. It also allows the input of one action to be identified as the output of a different action
     */
    relatedData?: string | undefined;
    _relatedData?: Element | undefined;
    /**
     * Defines the data that results as output from the action.
     */
    requirement?: DataRequirement | undefined;
    /**
     * A human-readable label for the data requirement used to label data flows in BPMN or similar diagrams. Also provides a human readable label when rendering the data requirement that conveys its purpose to human readers.
     */
    title?: string | undefined;
    _title?: Element | undefined;
}
/**
 * A relationship to another action such as "before" or "30-60 minutes after start of".
 */
export interface RequestOrchestrationActionRelatedAction extends BackboneElement {
    /**
     * A duration or range of durations to apply to the relationship. For example, 30-60 minutes before.
     */
    offsetDuration?: Duration | undefined;
    /**
     * A duration or range of durations to apply to the relationship. For example, 30-60 minutes before.
     */
    offsetRange?: Range | undefined;
    /**
     * The relationship of this action to the related action.
     */
    relationship:
        | 'before-start'
        | 'before'
        | 'before-end'
        | 'concurrent-with-start'
        | 'concurrent'
        | 'concurrent-with-end'
        | 'after-start'
        | 'after'
        | 'after-end';
    _relationship?: Element | undefined;
    /**
     * The element id of the target related action.
     */
    targetId: string;
    _targetId?: Element | undefined;
}
/**
 * Because request orchestrations represent potential options for performing activities, some specific participants may still be unknown, so this element allows for both definitional participants (in the same way they are specified in ActivityDefinition and PlanDefinition resources) as well as identifying specific participants when they are known.
 */
export interface RequestOrchestrationActionParticipant extends BackboneElement {
    /**
     * A reference to the actual participant.
     */
    actorCanonical?: string | undefined;
    _actorCanonical?: Element | undefined;
    /**
     * A reference to the actual participant.
     */
    actorReference?: Reference | undefined;
    /**
     * Indicates how the actor will be involved in the action - author, reviewer, witness, etc.
     */
    function?: CodeableConcept | undefined;
    /**
     * The role the participant should play in performing the described action.
     */
    role?: CodeableConcept | undefined;
    /**
     * The type of participant in the action.
     */
    type?:
        | (
              | 'careteam'
              | 'device'
              | 'group'
              | 'healthcareservice'
              | 'location'
              | 'organization'
              | 'patient'
              | 'practitioner'
              | 'practitionerrole'
              | 'relatedperson'
          )
        | undefined;
    _type?: Element | undefined;
    /**
     * The type of participant in the action.
     */
    typeCanonical?: string | undefined;
    _typeCanonical?: Element | undefined;
    /**
     * When this element is a reference, it SHOULD be a reference to a definitional resource (for example, a location type, rather than a specific location).
     */
    typeReference?: Reference | undefined;
}
/**
 * Dynamic values are applied in the order in which they are defined in the RequestOrchestration resource. Note that when dynamic values are also specified by a referenced ActivityDefinition, the dynamicValues from the ActivityDefinition are applied first, followed by the dynamicValues specified here. In addition, if both a transform and dynamic values are specific, the dynamic values are applied to the result of the transform.
 */
export interface RequestOrchestrationActionDynamicValue extends BackboneElement {
    /**
     * The expression may be inlined or may be a reference to a named expression within a logic library referenced by the library element.
     */
    expression?: Expression | undefined;
    /**
     * To specify the path to the current action being realized, the %action environment variable is available in this path. For example, to specify the description element of the target action, the path would be %action.description. The path attribute contains a [Simple FHIRPath Subset](fhirpath.html#simple) that allows path traversal, but not calculation.
     */
    path?: string | undefined;
    _path?: Element | undefined;
}
/**
 * The actions, if any, produced by the evaluation of the artifact.
 */
export interface RequestOrchestrationAction extends BackboneElement {
    /**
     * Sub actions.
     */
    action?: RequestOrchestrationAction[] | undefined;
    /**
     * Defines whether the action can be selected multiple times.
     */
    cardinalityBehavior?: ('single' | 'multiple') | undefined;
    _cardinalityBehavior?: Element | undefined;
    /**
     * A code that provides meaning for the action or action group. For example, a section may have a LOINC code for a section of a documentation template.
     */
    code?: CodeableConcept[] | undefined;
    /**
     * When multiple conditions of the same kind are present, the effects are combined using AND semantics, so the overall condition is true only if all of the conditions are true.
     */
    condition?: RequestOrchestrationActionCondition[] | undefined;
    /**
     * Note that the definition is optional, and if no definition is specified, a dynamicValue with a root ($this) path can be used to define the entire resource dynamically.
     */
    definitionCanonical?: string | undefined;
    _definitionCanonical?: Element | undefined;
    /**
     * Note that the definition is optional, and if no definition is specified, a dynamicValue with a root ($this) path can be used to define the entire resource dynamically.
     */
    definitionUri?: string | undefined;
    _definitionUri?: Element | undefined;
    /**
     * A short description of the action used to provide a summary to display to the user.
     */
    description?: string | undefined;
    _description?: Element | undefined;
    /**
     * Didactic or other informational resources associated with the action that can be provided to the CDS recipient. Information resources can include inline text commentary and links to web resources.
     */
    documentation?: RelatedArtifact[] | undefined;
    /**
     * Dynamic values are applied in the order in which they are defined in the RequestOrchestration resource. Note that when dynamic values are also specified by a referenced ActivityDefinition, the dynamicValues from the ActivityDefinition are applied first, followed by the dynamicValues specified here. In addition, if both a transform and dynamic values are specific, the dynamic values are applied to the result of the transform.
     */
    dynamicValue?: RequestOrchestrationActionDynamicValue[] | undefined;
    /**
     * Goals that are intended to be achieved by following the requests in this action.
     */
    goal?: Reference[] | undefined;
    /**
     * Defines the grouping behavior for the action and its children.
     */
    groupingBehavior?: ('visual-group' | 'logical-group' | 'sentence-group') | undefined;
    _groupingBehavior?: Element | undefined;
    /**
     * Defines input data requirements for the action.
     */
    input?: RequestOrchestrationActionInput[] | undefined;
    /**
     * The linkId of the action from the PlanDefinition that corresponds to this action in the RequestOrchestration resource.
     */
    linkId?: string | undefined;
    _linkId?: Element | undefined;
    /**
     * May reference a specific clinical location or may just identify a type of location.
     */
    location?: CodeableReference | undefined;
    /**
     * Defines the outputs of the action, if any.
     */
    output?: RequestOrchestrationActionOutput[] | undefined;
    /**
     * Because request orchestrations represent potential options for performing activities, some specific participants may still be unknown, so this element allows for both definitional participants (in the same way they are specified in ActivityDefinition and PlanDefinition resources) as well as identifying specific participants when they are known.
     */
    participant?: RequestOrchestrationActionParticipant[] | undefined;
    /**
     * Defines whether the action should usually be preselected.
     */
    precheckBehavior?: ('yes' | 'no') | undefined;
    _precheckBehavior?: Element | undefined;
    /**
     * A user-visible prefix for the action. For example a section or item numbering such as 1. or A.
     */
    prefix?: string | undefined;
    _prefix?: Element | undefined;
    /**
     * Indicates how quickly the action should be addressed with respect to other actions.
     */
    priority?: ('routine' | 'urgent' | 'asap' | 'stat') | undefined;
    _priority?: Element | undefined;
    /**
     * A relationship to another action such as "before" or "30-60 minutes after start of".
     */
    relatedAction?: RequestOrchestrationActionRelatedAction[] | undefined;
    /**
     * Defines expectations around whether an action is required.
     */
    requiredBehavior?: ('must' | 'could' | 'must-unless-documented') | undefined;
    _requiredBehavior?: Element | undefined;
    /**
     * The target resource SHALL be a [Request](request.html) resource with a Request.intent set to "option".
     */
    resource?: Reference | undefined;
    /**
     * Defines the selection behavior for the action and its children.
     */
    selectionBehavior?: ('any' | 'all' | 'all-or-none' | 'exactly-one' | 'at-most-one' | 'one-or-more') | undefined;
    _selectionBehavior?: Element | undefined;
    /**
     * A text equivalent of the action to be performed. This provides a human-interpretable description of the action when the definition is consumed by a system that might not be capable of interpreting it dynamically.
     */
    textEquivalent?: string | undefined;
    _textEquivalent?: Element | undefined;
    /**
     * The intent of the timing element is to provide timing for when the action should be performed. The timing may be absolute (specified as a dateTime or Period) or relative (specified as an Age, Duration, or Range), or it may be a more complex, potentially repeating timing specified using Timing.
     */
    timingDateTime?: string | undefined;
    _timingDateTime?: Element | undefined;
    /**
     * The intent of the timing element is to provide timing for when the action should be performed. The timing may be absolute (specified as a dateTime or Period) or relative (specified as an Age, Duration, or Range), or it may be a more complex, potentially repeating timing specified using Timing.
     */
    timingAge?: Age | undefined;
    /**
     * The intent of the timing element is to provide timing for when the action should be performed. The timing may be absolute (specified as a dateTime or Period) or relative (specified as an Age, Duration, or Range), or it may be a more complex, potentially repeating timing specified using Timing.
     */
    timingPeriod?: Period | undefined;
    /**
     * The intent of the timing element is to provide timing for when the action should be performed. The timing may be absolute (specified as a dateTime or Period) or relative (specified as an Age, Duration, or Range), or it may be a more complex, potentially repeating timing specified using Timing.
     */
    timingDuration?: Duration | undefined;
    /**
     * The intent of the timing element is to provide timing for when the action should be performed. The timing may be absolute (specified as a dateTime or Period) or relative (specified as an Age, Duration, or Range), or it may be a more complex, potentially repeating timing specified using Timing.
     */
    timingRange?: Range | undefined;
    /**
     * The intent of the timing element is to provide timing for when the action should be performed. The timing may be absolute (specified as a dateTime or Period) or relative (specified as an Age, Duration, or Range), or it may be a more complex, potentially repeating timing specified using Timing.
     */
    timingTiming?: Timing | undefined;
    /**
     * The title of the action displayed to a user.
     */
    title?: string | undefined;
    _title?: Element | undefined;
    /**
     * Note that when a referenced ActivityDefinition also defines a transform, the transform specified here generally takes precedence. In addition, if both a transform and dynamic values are specific, the dynamic values are applied to the result of the transform.
     */
    transform?: string | undefined;
    _transform?: Element | undefined;
    /**
     * The type of action to perform (create, update, remove).
     */
    type?: CodeableConcept | undefined;
}
/**
 * A set of related requests that can be used to capture intended activities that have inter-dependencies such as "give this medication after that one".
 */
export interface RequestOrchestration extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'RequestOrchestration';
    /**
     * The actions, if any, produced by the evaluation of the artifact.
     */
    action?: RequestOrchestrationAction[] | undefined;
    /**
     * Provides a reference to the author of the request orchestration.
     */
    author?: Reference | undefined;
    /**
     * Indicates when the request orchestration was created.
     */
    authoredOn?: string | undefined;
    _authoredOn?: Element | undefined;
    /**
     * A plan, proposal or order that is fulfilled in whole or in part by this request.
     */
    basedOn?: Reference[] | undefined;
    /**
     * This element can be used to provide a code that captures the meaning of the request orchestration as a whole, as opposed to the code of the action element, which captures the meaning of the individual actions within the request orchestration.
     */
    code?: CodeableConcept | undefined;
    /**
     * Describes the context of the request orchestration, if any.
     */
    encounter?: Reference | undefined;
    /**
     * Goals that are intended to be achieved by following the requests in this RequestOrchestration.
     */
    goal?: Reference[] | undefined;
    /**
     * Requests are linked either by a "basedOn" relationship (i.e. one request is fulfilling another) or by having a common requisition.  Requests that are part of the same requisition are generally treated independently from the perspective of changing their state or maintaining them after initial creation.
     */
    groupIdentifier?: Identifier | undefined;
    /**
     * Allows a service to provide a unique, business identifier for the request.
     */
    identifier?: Identifier[] | undefined;
    /**
     * A canonical URL referencing a FHIR-defined protocol, guideline, orderset or other definition that is adhered to in whole or in part by this request.
     */
    instantiatesCanonical?: string[] | undefined;
    _instantiatesCanonical?: Element[] | undefined;
    /**
     * A URL referencing an externally defined protocol, guideline, orderset or other definition that is adhered to in whole or in part by this request.
     */
    instantiatesUri?: string[] | undefined;
    _instantiatesUri?: Element[] | undefined;
    /**
     * Indicates the level of authority/intentionality associated with the request and where the request fits into the workflow chain.
     */
    intent:
        | 'proposal'
        | 'plan'
        | 'directive'
        | 'order'
        | 'original-order'
        | 'reflex-order'
        | 'filler-order'
        | 'instance-order'
        | 'option';
    _intent?: Element | undefined;
    /**
     * Provides a mechanism to communicate additional information about the response.
     */
    note?: Annotation[] | undefined;
    /**
     * Indicates how quickly the request should be addressed with respect to other requests.
     */
    priority?: ('routine' | 'urgent' | 'asap' | 'stat') | undefined;
    _priority?: Element | undefined;
    /**
     * Describes the reason for the request orchestration in coded or textual form.
     */
    reason?: CodeableReference[] | undefined;
    /**
     * The replacement could be because the initial request was immediately rejected (due to an issue) or because the previous request was completed, but the need for the action described by the request remains ongoing.
     */
    replaces?: Reference[] | undefined;
    /**
     * The current state of the request. For request orchestrations, the status reflects the status of all the requests in the orchestration.
     */
    status: 'draft' | 'active' | 'on-hold' | 'revoked' | 'completed' | 'entered-in-error' | 'unknown';
    _status?: Element | undefined;
    /**
     * The subject for which the request orchestration was created.
     */
    subject?: Reference | undefined;
}
/**
 * A statement of requirements.
 */
export interface RequirementsStatement extends BackboneElement {
    /**
     * The conformance code is extracted from the requirement to make indexing and display easier. The requirement needs to express the conformance verbs directly in the markdown content. It's not unusual to mix verbs in a single sentence (e.g. System SHALL do X and SHOULD do Y)
     */
    conformance?: ('SHALL' | 'SHOULD' | 'MAY' | 'SHOULD-NOT')[] | undefined;
    _conformance?: Element[] | undefined;
    /**
     * This can be just a key, if there is only one matching statement in the requirements statements referenced in Requirements.derivedFrom. if there is more than one, it should be in the format ```url#key```
     */
    derivedFrom?: string | undefined;
    _derivedFrom?: Element | undefined;
    /**
     * Refer directly to the statement by {url}#{key}
     */
    key: string;
    _key?: Element | undefined;
    /**
     * The key is intended for computers, while this is intended for humans. In its absence, the key should be convenient for a human to say (e.g. not a UUID)
     */
    label?: string | undefined;
    _label?: Element | undefined;
    /**
     * A reference to another artifact that created this requirement. This could be a Profile, etc, or external regulation, or business requirements expressed elsewhere.
     */
    reference?: string[] | undefined;
    _reference?: Element[] | undefined;
    /**
     * The actual requirement for human consumption.
     */
    requirement: string;
    _requirement?: Element | undefined;
    /**
     * This is for use when the requirement is met be an pre-existing artifact e.g. a rule that's met by the base FHIR spec, or a national implementation guide.
     */
    satisfiedBy?: string[] | undefined;
    _satisfiedBy?: Element[] | undefined;
    /**
     * Who asked for this statement to be a requirement. By default, it's assumed that the publisher knows who it is if it matters.
     */
    source?: Reference[] | undefined;
}
/**
 * A set of requirements - a list of features or behaviors of designed systems that are necessary to achieve organizational or regulatory goals.
 */
export interface Requirements extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'Requirements';
    /**
     * If more than one actor is specified, then it's up to the statements to describe in narrative if they don't apply to all actors.
     */
    actor?: string[] | undefined;
    _actor?: Element[] | undefined;
    /**
     * May be a web site, an email address, a telephone number, etc.
     */
    contact?: ContactDetail[] | undefined;
    /**
     * ...
     */
    copyright?: string | undefined;
    _copyright?: Element | undefined;
    /**
     * The (c) symbol should NOT be included in this string. It will be added by software when rendering the notation. Full details about licensing, restrictions, warrantees, etc. goes in the more general 'copyright' element.
     */
    copyrightLabel?: string | undefined;
    _copyrightLabel?: Element | undefined;
    /**
     * Note that this is not the same as the resource last-modified-date, since the resource may be a secondary representation of the Requirements. Additional specific dates may be added as extensions or be found by consulting Provenances associated with past versions of the resource.
     */
    date?: string | undefined;
    _date?: Element | undefined;
    /**
     * Existing statements (by key) may be narrowed or clarified, and additional statements added.
     */
    derivedFrom?: string[] | undefined;
    _derivedFrom?: Element[] | undefined;
    /**
     * This description can be used for a general description of the actor, and which covers why it was defined.
     */
    description?: string | undefined;
    _description?: Element | undefined;
    /**
     * Allows filtering of Requirements that are appropriate for use versus not.
     */
    experimental?: boolean | undefined;
    _experimental?: Element | undefined;
    /**
     * A formal identifier that is used to identify this Requirements when it is represented in other formats, or referenced in a specification, model, design or an instance.
     */
    identifier?: Identifier[] | undefined;
    /**
     * It may be possible for the Requirements to be used in jurisdictions other than those for which it was originally designed or intended.
     */
    jurisdiction?: CodeableConcept[] | undefined;
    /**
     * The name is not expected to be globally unique. The name should be a simple alphanumeric type name to ensure that it is machine-processing friendly.
     */
    name?: string | undefined;
    _name?: Element | undefined;
    /**
     * Usually an organization but may be an individual. The publisher (or steward) of the Requirements is the organization or individual primarily responsible for the maintenance and upkeep of the Requirements. This is not necessarily the same individual or organization that developed and initially authored the content. The publisher is the primary point of contact for questions or issues with the Requirements. This item SHOULD be populated unless the information is available from context.
     */
    publisher?: string | undefined;
    _publisher?: Element | undefined;
    /**
     * This element does not describe the usage of the Requirements. Instead, it provides traceability of ''why'' the resource is either needed or ''why'' it is defined as it is.  This may be used to point to source materials or specifications that drove the structure of this Requirements.
     */
    purpose?: string | undefined;
    _purpose?: Element | undefined;
    /**
     * A statement of requirements.
     */
    statement?: RequirementsStatement[] | undefined;
    /**
     * Allows filtering of Requirements that are appropriate for use versus not.
     */
    status: 'draft' | 'active' | 'retired' | 'unknown';
    _status?: Element | undefined;
    /**
     * This name does not need to be machine-processing friendly and may contain punctuation, white-space, etc.
     */
    title?: string | undefined;
    _title?: Element | undefined;
    /**
     * Can be a urn:uuid: or a urn:oid: but real http: addresses are preferred.  Multiple instances may share the same URL if they have a distinct version.
     * The determination of when to create a new version of a resource (same url, new version) vs. defining a new artifact is up to the author.  Considerations for making this decision are found in [Technical and Business Versions](resource.html#versions).
     * In some cases, the resource can no longer be found at the stated url, but the url itself cannot change. Implementations can use the [meta.source](resource.html#meta) element to indicate where the current master source of the resource can be found.
     */
    url?: string | undefined;
    _url?: Element | undefined;
    /**
     * When multiple useContexts are specified, there is no expectation that all or any of the contexts apply.
     */
    useContext?: UsageContext[] | undefined;
    /**
     * There may be different Requirements instances that have the same identifier but different versions.  The version can be appended to the url in a reference to allow a reference to a particular business version of the Requirements with the format [url]|[version]. The version SHOULD NOT contain a '#' - see [Business Version](resource.html#bv-format).
     */
    version?: string | undefined;
    _version?: Element | undefined;
}
/**
 * Additional names for the study.
 */
export interface ResearchStudyLabel extends BackboneElement {
    /**
     * Kind of name.
     */
    type?: CodeableConcept | undefined;
    /**
     * The name.
     */
    value?: string | undefined;
    _value?: Element | undefined;
}
/**
 * The medication(s), food(s), therapy(ies), device(s) or other concerns or interventions that the study is seeking to gain more information about.
 */
export interface ResearchStudyFocus extends BackboneElement {
    /**
     * A factor corresponds to an independent variable manipulated by the experimentalist with the intention to affect biological systems in a way that can be measured by an assay.
     */
    factor?: string | undefined;
    _factor?: Element | undefined;
    /**
     * Indicates whether the focus is a medication, a device, a procedure, a specific factor or some other intervention or characteristic.
     */
    focusType?: CodeableConcept[] | undefined;
    /**
     * Identification of product under study.  This may be any combination of code and/or name.
     */
    productCode?: CodeableConcept | undefined;
}
/**
 * For a Sponsor or a PrincipalInvestigator use the dedicated attributes provided.
 */
export interface ResearchStudyAssociatedParty extends BackboneElement {
    /**
     * A categorization other than role for the associated party.
     */
    classifier?: CodeableConcept[] | undefined;
    /**
     * Name of associated party.
     */
    name?: string | undefined;
    _name?: Element | undefined;
    /**
     * Suggestions of a better attribute name are appreciated
     */
    party?: Reference | undefined;
    /**
     * The cardinality is 0..* due to the fact that an associated party may be intermittently active in a given role over multiple time periods.
     */
    period?: Period[] | undefined;
    /**
     * Type of association.
     */
    role: CodeableConcept;
}
/**
 * Status of study with time for that status.
 */
export interface ResearchStudyProgressStatus extends BackboneElement {
    /**
     * An indication of whether or not the date is a known date when the state changed or will change. A value of true indicates a known date. A value of false indicates an estimated date.
     */
    actual?: boolean | undefined;
    _actual?: Element | undefined;
    /**
     * Date range.
     */
    period?: Period | undefined;
    /**
     * Label for status or state (e.g. recruitment status).
     */
    state: CodeableConcept;
}
/**
 * Target or actual group of participants enrolled in study.
 */
export interface ResearchStudyRecruitment extends BackboneElement {
    /**
     * Group of participants who were enrolled in study.
     */
    actualGroup?: Reference | undefined;
    /**
     * Actual total number of participants enrolled in study.
     */
    actualNumber?: number | undefined;
    /**
     * Inclusion and exclusion criteria.
     */
    eligibility?: Reference | undefined;
    /**
     * Estimated total number of participants to be enrolled.
     */
    targetNumber?: number | undefined;
}
/**
 * In many clinical trials this is refered to as the ARM of the study, but such a term is not used in other sorts of trials even when there is a comparison between two or more groups.
 */
export interface ResearchStudyComparisonGroup extends BackboneElement {
    /**
     * A succinct description of the path through the study that would be followed by a subject adhering to this comparisonGroup.
     */
    description?: string | undefined;
    _description?: Element | undefined;
    /**
     * Allows the comparisonGroup for the study and the comparisonGroup for the subject to be linked easily.
     */
    identifierUri?: string | undefined;
    _identifierUri?: Element | undefined;
    /**
     * Allows the comparisonGroup for the study and the comparisonGroup for the subject to be linked easily.
     */
    identifierIdentifier?: Identifier | undefined;
    /**
     * Interventions or exposures in this comparisonGroup or cohort.
     */
    intendedExposure?: Reference[] | undefined;
    /**
     * Unique, human-readable label for this comparisonGroup of the study.
     */
    name: string;
    _name?: Element | undefined;
    /**
     * Group of participants who were enrolled in study comparisonGroup.
     */
    observedGroup?: Reference | undefined;
    /**
     * Categorization of study comparisonGroup, e.g. experimental, active comparator, placebo comparater.
     */
    type?: CodeableConcept | undefined;
}
/**
 * A goal that the study is aiming to achieve in terms of a scientific question to be answered by the analysis of data collected during the study.
 */
export interface ResearchStudyObjective extends BackboneElement {
    /**
     * Free text description of the objective of the study.  This is what the study is trying to achieve rather than how it is going to achieve it (see ResearchStudy.description).
     */
    description?: string | undefined;
    _description?: Element | undefined;
    /**
     * Unique, human-readable label for this objective of the study.
     */
    name?: string | undefined;
    _name?: Element | undefined;
    /**
     * The kind of study objective.
     */
    type?: CodeableConcept | undefined;
}
/**
 * An outcome or planned variable to measure during the study.
 */
export interface ResearchStudyOutcomeMeasure extends BackboneElement {
    /**
     * Description of the outcome.
     */
    description?: string | undefined;
    _description?: Element | undefined;
    /**
     * Label for the outcome.
     */
    name?: string | undefined;
    _name?: Element | undefined;
    /**
     * Structured outcome definition.
     */
    reference?: Reference | undefined;
    /**
     * The parameter or characteristic being assessed as one of the values by which the study is assessed.
     */
    type?: CodeableConcept[] | undefined;
}
/**
 * A general storage or archive location for the study.  This may contain an assortment of content which is not specified in advance.
 */
export interface ResearchStudyWebLocation extends BackboneElement {
    /**
     * Describes the nature of the location being specified.
     */
    classifier?: CodeableConcept | undefined;
    /**
     * The location address.
     */
    url: string;
    _url?: Element | undefined;
}
/**
 * A scientific study of nature that sometimes includes processes involved in health and disease. For example, clinical trials are research studies that involve people. These studies may be related to new ways to screen, prevent, diagnose, and treat disease. They may also study certain outcomes and certain groups of people by looking at data collected in the past or future.
 */
export interface ResearchStudy extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'ResearchStudy';
    /**
     * For a Sponsor or a PrincipalInvestigator use the dedicated attributes provided.
     */
    associatedParty?: ResearchStudyAssociatedParty[] | undefined;
    /**
     * Additional grouping mechanism or categorization of a research study. Example: FDA regulated device, FDA regulated drug, MPG Paragraph 23b (a German legal requirement), IRB-exempt, etc. Implementation Note: do not use the classifier element to support existing semantics that are already supported thru explicit elements in the resource.
     */
    classifier?: CodeableConcept[] | undefined;
    /**
     * In many clinical trials this is refered to as the ARM of the study, but such a term is not used in other sorts of trials even when there is a comparison between two or more groups.
     */
    comparisonGroup?: ResearchStudyComparisonGroup[] | undefined;
    /**
     * The condition that is the focus of the study.  For example, In a study to examine risk factors for Lupus, might have as an inclusion criterion "healthy volunteer", but the target condition code would be a Lupus SNOMED code.
     */
    condition?: CodeableConcept[] | undefined;
    /**
     * The date (and optionally time) when the ResearchStudy Resource was published. The date must change when the business version changes and it must change if the status code changes. In addition, it should change when the substantive content of the ResearchStudy Resource changes.
     */
    date?: string | undefined;
    _date?: Element | undefined;
    /**
     * A detailed and human-readable narrative of the study. E.g., study abstract.
     */
    description?: string | undefined;
    _description?: Element | undefined;
    /**
     * A brief text for explaining the study.
     */
    descriptionSummary?: string | undefined;
    _descriptionSummary?: Element | undefined;
    /**
     * The medication(s), food(s), therapy(ies), device(s) or other concerns or interventions that the study is seeking to gain more information about.
     */
    focus?: ResearchStudyFocus[] | undefined;
    /**
     * Identifiers assigned to this research study by the sponsor or other systems.
     */
    identifier?: Identifier[] | undefined;
    /**
     * Key terms to aid in searching for or filtering the study.
     */
    keyword?: CodeableConcept[] | undefined;
    /**
     * Additional names for the study.
     */
    label?: ResearchStudyLabel[] | undefined;
    /**
     * Name for this study (computer friendly).
     */
    name?: string | undefined;
    _name?: Element | undefined;
    /**
     * Comments made about the study by the performer, subject or other participants.
     */
    note?: Annotation[] | undefined;
    /**
     * A goal that the study is aiming to achieve in terms of a scientific question to be answered by the analysis of data collected during the study.
     */
    objective?: ResearchStudyObjective[] | undefined;
    /**
     * An outcome or planned variable to measure during the study.
     */
    outcomeMeasure?: ResearchStudyOutcomeMeasure[] | undefined;
    /**
     * A larger research study of which this particular study is a component or step.
     */
    partOf?: Reference[] | undefined;
    /**
     * Identifies the start date and the expected (or actual, depending on status) end date for the study.
     */
    period?: Period | undefined;
    /**
     * The stage in the progression of a therapy from initial experimental use in humans in clinical trials to post-market evaluation.
     */
    phase?: CodeableConcept | undefined;
    /**
     * The type of study based upon the intent of the study activities. A classification of the intent of the study.
     */
    primaryPurposeType?: CodeableConcept | undefined;
    /**
     * Status of study with time for that status.
     */
    progressStatus?: ResearchStudyProgressStatus[] | undefined;
    /**
     * The set of steps expected to be performed as part of the execution of the study.
     */
    protocol?: Reference[] | undefined;
    /**
     * Target or actual group of participants enrolled in study.
     */
    recruitment?: ResearchStudyRecruitment | undefined;
    /**
     * A country, state or other area where the study is taking place rather than its precise geographic location or address.
     */
    region?: CodeableConcept[] | undefined;
    /**
     * Citations, references and other related documents.
     */
    relatedArtifact?: RelatedArtifact[] | undefined;
    /**
     * Link to one or more sets of results generated by the study.  Could also link to a research registry holding the results such as ClinicalTrials.gov.
     */
    result?: Reference[] | undefined;
    /**
     * A facility in which study activities are conducted.
     */
    site?: Reference[] | undefined;
    /**
     * The publication state of the resource (not of the study).
     */
    status: 'draft' | 'active' | 'retired' | 'unknown';
    _status?: Element | undefined;
    /**
     * Codes categorizing the type of study such as investigational vs. observational, type of blinding, type of randomization, safety vs. efficacy, etc.
     */
    studyDesign?: CodeableConcept[] | undefined;
    /**
     * The human readable name of the research study.
     */
    title?: string | undefined;
    _title?: Element | undefined;
    /**
     * Canonical identifier for this study resource, represented as a globally unique URI.
     */
    url?: string | undefined;
    _url?: Element | undefined;
    /**
     * The business version for the study record.
     */
    version?: string | undefined;
    _version?: Element | undefined;
    /**
     * A general storage or archive location for the study.  This may contain an assortment of content which is not specified in advance.
     */
    webLocation?: ResearchStudyWebLocation[] | undefined;
    /**
     * A description and/or code explaining the premature termination of the study.
     */
    whyStopped?: CodeableConcept | undefined;
}
/**
 * This is intended to deal with the confusion routinely created by haing two conflated concepts of being in a particular state and having achieved a particular milestone.  In strict terms a milestone is a point of time event that results in a change from one state to another.  The state before the milestone is achieved is often given the same name as the milestone, and sometimes the state may have the same description.  For instance "Randomised" and "Visit 1" may be different milestones but the state remains at "on study" after each of them.
 * It is likely that more than one "state" pattern will be recorded for a subject and a type has been introduced to allow this simultaneous recording.
 */
export interface ResearchSubjectProgress extends BackboneElement {
    /**
     * The date when the state ended.
     */
    endDate?: string | undefined;
    _endDate?: Element | undefined;
    /**
     * There can be multiple entries but it is also valid to just have the most recent.  This should npt be rlied upon as the full path the subject has taken.
     */
    milestone?: CodeableConcept | undefined;
    /**
     * The reason for the state change.  If coded it should follow the formal subject state model.
     */
    reason?: CodeableConcept | undefined;
    /**
     * This is NOT the date the change in state was recorded.
     */
    startDate?: string | undefined;
    _startDate?: Element | undefined;
    /**
     * The current state of the subject.
     */
    subjectState?: CodeableConcept | undefined;
    /**
     * Identifies the aspect of the subject's journey that the state refers to.
     */
    type?: CodeableConcept | undefined;
}
/**
 * A physical entity which is the primary unit of operational and/or administrative interest in a study.
 */
export interface ResearchSubject extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'ResearchSubject';
    /**
     * The name of the arm in the study the subject actually followed as part of this study.
     */
    actualArm?: string | undefined;
    _actualArm?: Element | undefined;
    /**
     * Maybe this is really PlanDefinition.
     */
    assignedArm?: string | undefined;
    _assignedArm?: Element | undefined;
    /**
     * A record of the patient's informed agreement to participate in the study.
     */
    consent?: Reference | undefined;
    /**
     * Identifiers assigned to this research subject for a study.
     */
    identifier?: Identifier[] | undefined;
    /**
     * The dates the subject began and ended their participation in the study.
     */
    period?: Period | undefined;
    /**
     * This is intended to deal with the confusion routinely created by haing two conflated concepts of being in a particular state and having achieved a particular milestone.  In strict terms a milestone is a point of time event that results in a change from one state to another.  The state before the milestone is achieved is often given the same name as the milestone, and sometimes the state may have the same description.  For instance "Randomised" and "Visit 1" may be different milestones but the state remains at "on study" after each of them.
     * It is likely that more than one "state" pattern will be recorded for a subject and a type has been introduced to allow this simultaneous recording.
     */
    progress?: ResearchSubjectProgress[] | undefined;
    /**
     * The publication state of the resource (not of the subject).
     */
    status: 'draft' | 'active' | 'retired' | 'unknown';
    _status?: Element | undefined;
    /**
     * Reference to the study the subject is participating in.
     */
    study: Reference;
    /**
     * The record of the person, animal or other entity involved in the study.
     */
    subject: Reference;
}
/**
 * This is the base resource type for everything.
 */
export interface Resource extends Base {
    /** Resource Type Name (for serialization) */
    readonly resourceType: string;
    /**
     * Within the context of the FHIR RESTful interactions, the resource has an id except for cases like the create and conditional update. Otherwise, the use of the resouce id depends on the given use case.
     */
    id?: string | undefined;
    _id?: Element | undefined;
    /**
     * Asserting this rule set restricts the content to be only understood by a limited set of trading partners. This inherently limits the usefulness of the data in the long term. However, the existing health eco-system is highly fractured, and not yet ready to define, collect, and exchange data in a generally computable sense. Wherever possible, implementers and/or specification writers should avoid using this element. Often, when used, the URL is a reference to an implementation guide that defines these special rules as part of it's narrative along with other profiles, value sets, etc.
     */
    implicitRules?: string | undefined;
    _implicitRules?: Element | undefined;
    /**
     * Language is provided to support indexing and accessibility (typically, services such as text to speech use the language tag). The html language tag in the narrative applies  to the narrative. The language tag on the resource may be used to specify the language of other presentations generated from the data in the resource. Not all the content has to be in the base language. The Resource.language should not be assumed to apply to the narrative automatically. If a language is specified, it should it also be specified on the div element in the html (see rules in HTML5 for information about the relationship between xml:lang and the html lang attribute).
     */
    language?: string | undefined;
    _language?: Element | undefined;
    /**
     * The metadata about the resource. This is content that is maintained by the infrastructure. Changes to the content might not always be associated with version changes to the resource.
     */
    meta?: Meta | undefined;
}
/**
 * Multiple repetitions can be used to identify the same type of outcome in different timeframes as well as different types of outcomes.
 */
export interface RiskAssessmentPrediction extends BackboneElement {
    /**
     * One of the potential outcomes for the patient (e.g. remission, death,  a particular condition).
     */
    outcome?: CodeableConcept | undefined;
    /**
     * If range is used, it represents the lower and upper bounds of certainty; e.g. 40-60%  Decimal values are expressed as percentages as well (max = 100).
     */
    probabilityDecimal?: number | undefined;
    /**
     * If range is used, it represents the lower and upper bounds of certainty; e.g. 40-60%  Decimal values are expressed as percentages as well (max = 100).
     */
    probabilityRange?: Range | undefined;
    /**
     * Indicates how likely the outcome is (in the specified timeframe), expressed as a qualitative value (e.g. low, medium, or high).
     */
    qualitativeRisk?: CodeableConcept | undefined;
    /**
     * Additional information explaining the basis for the prediction.
     */
    rationale?: string | undefined;
    _rationale?: Element | undefined;
    /**
     * Indicates the risk for this particular subject (with their specific characteristics) divided by the risk of the population in general.  (Numbers greater than 1 = higher risk than the population, numbers less than 1 = lower risk.).
     */
    relativeRisk?: number | undefined;
    /**
     * If not specified, the risk applies "over the subject's lifespan".
     */
    whenPeriod?: Period | undefined;
    /**
     * If not specified, the risk applies "over the subject's lifespan".
     */
    whenRange?: Range | undefined;
}
/**
 * An assessment of the likely outcome(s) for a patient or other subject as well as the likelihood of each outcome.
 */
export interface RiskAssessment extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'RiskAssessment';
    /**
     * A reference to the request that is fulfilled by this risk assessment.
     */
    basedOn?: Reference | undefined;
    /**
     * Indicates the source data considered as part of the assessment (for example, FamilyHistory, Observations, Procedures, Conditions, etc.).
     */
    basis?: Reference[] | undefined;
    /**
     * The type of the risk assessment performed.
     */
    code?: CodeableConcept | undefined;
    /**
     * For assessments or prognosis specific to a particular condition, indicates the condition being assessed.
     */
    condition?: Reference | undefined;
    /**
     * The encounter where the assessment was performed.
     */
    encounter?: Reference | undefined;
    /**
     * Business identifier assigned to the risk assessment.
     */
    identifier?: Identifier[] | undefined;
    /**
     * The algorithm, process or mechanism used to evaluate the risk.
     */
    method?: CodeableConcept | undefined;
    /**
     * A description of the steps that might be taken to reduce the identified risk(s).
     */
    mitigation?: string | undefined;
    _mitigation?: Element | undefined;
    /**
     * Additional comments about the risk assessment.
     */
    note?: Annotation[] | undefined;
    /**
     * The date (and possibly time) the risk assessment was performed.
     */
    occurrenceDateTime?: string | undefined;
    _occurrenceDateTime?: Element | undefined;
    /**
     * The date (and possibly time) the risk assessment was performed.
     */
    occurrencePeriod?: Period | undefined;
    /**
     * A reference to a resource that this risk assessment is part of, such as a Procedure.
     */
    parent?: Reference | undefined;
    /**
     * The provider, patient, related person, or software application that performed the assessment.
     */
    performer?: Reference | undefined;
    /**
     * Multiple repetitions can be used to identify the same type of outcome in different timeframes as well as different types of outcomes.
     */
    prediction?: RiskAssessmentPrediction[] | undefined;
    /**
     * The reason the risk assessment was performed.
     */
    reason?: CodeableReference[] | undefined;
    /**
     * The status of the RiskAssessment, using the same statuses as an Observation.
     */
    status:
        | 'registered'
        | 'preliminary'
        | 'final'
        | 'amended'
        | 'corrected'
        | 'cancelled'
        | 'entered-in-error'
        | 'unknown';
    _status?: Element | undefined;
    /**
     * The patient or group the risk assessment applies to.
     */
    subject: Reference;
}
/**
 * A container for slots of time that may be available for booking appointments.
 */
export interface Schedule extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'Schedule';
    /**
     * This element is labeled as a modifier because it may be used to mark that the resource was created in error.
     */
    active?: boolean | undefined;
    _active?: Element | undefined;
    /**
     * The capacity to support multiple referenced resource types should be used in cases where the specific resources themselves cannot be scheduled without the other, and thus only make sense to the system exposing them as a group. Common examples of this are where the combination of a practitioner and a room (Location) are always required by a system.
     */
    actor: Reference[];
    /**
     * Comments on the availability to describe any extended information. Such as custom constraints on the slots that may be associated.
     */
    comment?: string | undefined;
    _comment?: Element | undefined;
    /**
     * External Ids for this item.
     */
    identifier?: Identifier[] | undefined;
    /**
     * This MAY be used to describe what the schedule is for where it is clearer than just the name of the single actor.
     */
    name?: string | undefined;
    _name?: Element | undefined;
    /**
     * The period of time that the slots that reference this Schedule resource cover (even if none exist). These  cover the amount of time that an organization's planning horizon; the interval for which they are currently accepting appointments. This does not define a "template" for planning outside these dates.
     */
    planningHorizon?: Period | undefined;
    /**
     * A broad categorization of the service that is to be performed during this appointment.
     */
    serviceCategory?: CodeableConcept[] | undefined;
    /**
     * The specific service that is to be performed during this appointment.
     */
    serviceType?: CodeableReference[] | undefined;
    /**
     * The specialty of a practitioner that would be required to perform the service requested in this appointment.
     */
    specialty?: CodeableConcept[] | undefined;
}
/**
 * Used to define the parts of a composite search parameter.
 */
export interface SearchParameterComponent extends BackboneElement {
    /**
     * The definition of the search parameter that describes this part.
     */
    definition: string;
    _definition?: Element | undefined;
    /**
     * This expression overrides the expression in the definition and extracts the index values from the outcome of the composite expression.
     */
    expression: string;
    _expression?: Element | undefined;
}
/**
 * A search parameter that defines a named search item that can be used to search/filter on a resource.
 */
export interface SearchParameter extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'SearchParameter';
    /**
     * A search parameter must always apply to at least one resource type. When search parameters apply to more than one resource type, they can be used against any of the listed resource types, or in a cross-type search (see [Cross Resource Search](http.html#xres-search)).
     */
    base: string[];
    _base?: Element[] | undefined;
    /**
     * Notes:
     * * Not all systems will declare chain values.  If no chain values are specified at all, then no presumptions can be made about the degree of chaining support available
     * * A server may use a single chain value of '*' to mean that all reference search types supported can be chained rather than enumerating all chain values
     * *  If a particular reference search parameter is supported, but not subsequent chaining, then just list that search parameter name - e.g. for Patient.practitioner, the chain value would be ```practitioner```
     * * If a particular reference search parameter is supported with unlimited chaining beneath it, then follow the search parameter name with '.'.  E.g. ```practitioner.```
     * * If there are specific (not unlimited) chaining paths allowed they can be explicitly enumerated.  E.g. ```practitioner.organization```
     */
    chain?: string[] | undefined;
    _chain?: Element[] | undefined;
    /**
     * For maximum compatibility, use only lowercase ASCII characters. Note that HL7 will never define multiple search parameters with the same code.
     */
    code: string;
    _code?: Element | undefined;
    /**
     * If no comparators are listed, clients should not expect servers to support any comparators.
     */
    comparator?: ('eq' | 'ne' | 'gt' | 'lt' | 'ge' | 'le' | 'sa' | 'eb' | 'ap')[] | undefined;
    _comparator?: Element[] | undefined;
    /**
     * Used to define the parts of a composite search parameter.
     */
    component?: SearchParameterComponent[] | undefined;
    /**
     * FHIRPath expression that defines/sets a complex constraint for when this SearchParameter is applicable.
     */
    constraint?: string | undefined;
    _constraint?: Element | undefined;
    /**
     * May be a web site, an email address, a telephone number, etc.
     */
    contact?: ContactDetail[] | undefined;
    /**
     * Note that this is not the same as the resource last-modified-date, since the resource may be a secondary representation of the search parameter. Additional specific dates may be added as extensions or be found by consulting Provenances associated with past versions of the resource.
     */
    date?: string | undefined;
    _date?: Element | undefined;
    /**
     * The intent of this is that a server can designate that it provides support for a search parameter defined in the specification itself (e.g.  [`value-quantity`](http://hl7.org/fhir/SearchParameter/Observation-value-quantity), but detail how it is supported by the server.
     */
    derivedFrom?: string | undefined;
    _derivedFrom?: Element | undefined;
    /**
     * This description can be used to capture details such as comments about misuse, instructions for clinical use and interpretation, literature references, examples from the paper world, etc. It is not a rendering of the search parameter as conveyed in the 'text' field of the resource itself. This item SHOULD be populated unless the information is available from context (e.g. the language of the search parameter is presumed to be the predominant language in the place the search parameter was created).
     */
    description: string;
    _description?: Element | undefined;
    /**
     * Allows filtering of search parameters that are appropriate for use versus not.
     */
    experimental?: boolean | undefined;
    _experimental?: Element | undefined;
    /**
     * Note that the elements returned by the expression are sometimes complex elements where logic is required to determine quite how to handle them; e.g. CodeableConcepts may contain text and/or multiple codings, where the codings themselves contain a code and a system. For composite search parameters, the outcome of the expression must a collection of base elements from which the composites are derived.
     */
    expression?: string | undefined;
    _expression?: Element | undefined;
    /**
     * It may be possible for the search parameter to be used in jurisdictions other than those for which it was originally designed or intended.
     */
    jurisdiction?: CodeableConcept[] | undefined;
    /**
     * A modifier supported for the search parameter.
     */
    modifier?:
        | (
              | 'missing'
              | 'exact'
              | 'contains'
              | 'not'
              | 'text'
              | 'in'
              | 'not-in'
              | 'below'
              | 'above'
              | 'type'
              | 'identifier'
              | 'of-type'
              | 'code-text'
              | 'text-advanced'
              | 'iterate'
          )[]
        | undefined;
    _modifier?: Element[] | undefined;
    /**
     * Whether multiple parameters are allowed - e.g. more than one parameter with the same name. The search matches if all the parameters match.
     */
    multipleAnd?: boolean | undefined;
    _multipleAnd?: Element | undefined;
    /**
     * Whether multiple values are allowed for each time the parameter exists. Values are separated by commas, and the parameter matches if any of the values match.
     */
    multipleOr?: boolean | undefined;
    _multipleOr?: Element | undefined;
    /**
     * The name is not expected to be globally unique. The name should be a simple alphanumeric type name to ensure that it is machine-processing friendly.
     */
    name: string;
    _name?: Element | undefined;
    /**
     * How the search parameter relates to the set of elements returned by evaluating the expression query.
     */
    processingMode?: ('normal' | 'phonetic' | 'other') | undefined;
    _processingMode?: Element | undefined;
    /**
     * Usually an organization but may be an individual. The publisher (or steward) of the search parameter is the organization or individual primarily responsible for the maintenance and upkeep of the search parameter. This is not necessarily the same individual or organization that developed and initially authored the content. The publisher is the primary point of contact for questions or issues with the search parameter. This item SHOULD be populated unless the information is available from context.
     */
    publisher?: string | undefined;
    _publisher?: Element | undefined;
    /**
     * This element does not describe the usage of the search parameter. Instead, it provides traceability of ''why'' the resource is either needed or ''why'' it is defined as it is.  This may be used to point to source materials or specifications that drove the structure of this search parameter.
     */
    purpose?: string | undefined;
    _purpose?: Element | undefined;
    /**
     * Allows filtering of search parameters that are appropriate for use versus not.
     */
    status: 'draft' | 'active' | 'retired' | 'unknown';
    _status?: Element | undefined;
    /**
     * In some cases, targets may exist for a search parameter that do not exist for the specified FHIRPath for all of the resources identified in SearchParameter.base.  For example, the core clinical-encounter search parameter allows both Encounter and EpisodeOfCare as targets, even though not all of the base resources have EpisodeOfCare as a valid target.
     * However, the list of targets SHOULD cover all targets that might appear that are permitted by the specified FHIRPath.
     */
    target?: string[] | undefined;
    _target?: Element[] | undefined;
    /**
     * This name does not need to be machine-processing friendly and may contain punctuation, white-space, etc.
     */
    title?: string | undefined;
    _title?: Element | undefined;
    /**
     * The type of value that a search parameter may contain, and how the content is interpreted.
     */
    type: 'number' | 'date' | 'string' | 'token' | 'reference' | 'composite' | 'quantity' | 'uri' | 'special';
    _type?: Element | undefined;
    /**
     * Can be a urn:uuid: or a urn:oid: but real http: addresses are preferred.  Multiple instances may share the same URL if they have a distinct version.
     * The determination of when to create a new version of a resource (same url, new version) vs. defining a new artifact is up to the author.  Considerations for making this decision are found in [Technical and Business Versions](resource.html#versions).
     * In some cases, the resource can no longer be found at the stated url, but the url itself cannot change. Implementations can use the [meta.source](resource.html#meta) element to indicate where the current master source of the resource can be found.
     */
    url: string;
    _url?: Element | undefined;
    /**
     * When multiple useContexts are specified, there is no expectation that all or any of the contexts apply.
     */
    useContext?: UsageContext[] | undefined;
    /**
     * There may be different search parameter instances that have the same identifier but different versions.  The version can be appended to the url in a reference to allow a reference to a particular business version of the search parameter with the format [url]|[version]. The version SHOULD NOT contain a '#' - see [Business Version](resource.html#bv-format).
     */
    version?: string | undefined;
    _version?: Element | undefined;
    /**
     * If set as a string, this is a FHIRPath expression that has two additional context variables passed in - %version1 and %version2 and will return a negative number if version1 is newer, a positive number if version2 and a 0 if the version ordering can't be successfully be determined.
     */
    versionAlgorithmString?: string | undefined;
    _versionAlgorithmString?: Element | undefined;
    /**
     * If set as a string, this is a FHIRPath expression that has two additional context variables passed in - %version1 and %version2 and will return a negative number if version1 is newer, a positive number if version2 and a 0 if the version ordering can't be successfully be determined.
     */
    versionAlgorithmCoding?: Coding | undefined;
}
/**
 * A record of a request for service such as diagnostic investigations, treatments, or operations to be performed.
 */
export interface ServiceRequest extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'ServiceRequest';
    /**
     * If a CodeableConcept is present, it indicates the pre-condition for performing the service.  For example "pain", "on flare-up", etc.
     */
    asNeededBoolean?: boolean | undefined;
    _asNeededBoolean?: Element | undefined;
    /**
     * If a CodeableConcept is present, it indicates the pre-condition for performing the service.  For example "pain", "on flare-up", etc.
     */
    asNeededCodeableConcept?: CodeableConcept | undefined;
    /**
     * When the request transitioned to being actionable.
     */
    authoredOn?: string | undefined;
    _authoredOn?: Element | undefined;
    /**
     * Plan/proposal/order fulfilled by this request.
     */
    basedOn?: Reference[] | undefined;
    /**
     * Only used if not implicit in the code found in ServiceRequest.code.  If the use case requires BodySite to be handled as a separate resource instead of an inline coded element (e.g. to identify and track separately)  then use the standard extension [procedure-targetBodyStructure](extension-procedure-targetbodystructure.html).
     */
    bodySite?: CodeableConcept[] | undefined;
    /**
     * Anatomic location where the procedure should be performed. This is the target site.
     */
    bodyStructure?: Reference | undefined;
    /**
     * There may be multiple axis of categorization depending on the context or use case for retrieving or displaying the resource.  The level of granularity is defined by the category concepts in the value set.
     */
    category?: CodeableConcept[] | undefined;
    /**
     * Many laboratory and radiology procedure codes embed the specimen/organ system in the test order name, for example,  serum or serum/plasma glucose, or a chest x-ray. The specimen might not be recorded separately from the test code.
     */
    code?: CodeableReference | undefined;
    /**
     * In general, only the code and timeframe will be present, though occasional additional qualifiers such as body site or even performer could be included to narrow the scope of the prohibition.  If the ServiceRequest.code and ServiceRequest.doNotPerform both contain negation, that will reinforce prohibition and should not have a double negative interpretation.
     */
    doNotPerform?: boolean | undefined;
    _doNotPerform?: Element | undefined;
    /**
     * An encounter that provides additional information about the healthcare context in which this request is made.
     */
    encounter?: Reference | undefined;
    /**
     * The actual focus of a service request when it is not the subject of record representing something or someone associated with the subject such as a spouse, parent, fetus, or donor. The focus of a service request could also be an existing condition,  an intervention, the subject's diet,  another service request on the subject,  or a body structure such as tumor or implanted device.
     */
    focus?: Reference[] | undefined;
    /**
     * The identifier.type element is used to distinguish between the identifiers assigned by the orderer (known as the 'Placer' in HL7 v2) and the producer of the observations in response to the order (known as the 'Filler' in HL7 v2).  For further discussion and examples see the resource notes section below.
     */
    identifier?: Identifier[] | undefined;
    /**
     * Note: This is a business identifier, not a resource identifier (see [discussion](resource.html#identifiers)).  It is best practice for the identifier to only appear on a single resource instance, however business practices may occasionally dictate that multiple resource instances with the same identifier can exist - possibly even with different resource types.  For example, multiple Patient and a Person resource instance might share the same social insurance number.
     */
    instantiatesCanonical?: string[] | undefined;
    _instantiatesCanonical?: Element[] | undefined;
    /**
     * This might be an HTML page, PDF, etc. or could just be a non-resolvable URI identifier.
     */
    instantiatesUri?: string[] | undefined;
    _instantiatesUri?: Element[] | undefined;
    /**
     * Insurance plans, coverage extensions, pre-authorizations and/or pre-determinations that may be needed for delivering the requested service.
     */
    insurance?: Reference[] | undefined;
    /**
     * This element is labeled as a modifier because the intent alters when and how the resource is actually applicable.
     */
    intent:
        | 'proposal'
        | 'plan'
        | 'directive'
        | 'order'
        | 'original-order'
        | 'reflex-order'
        | 'filler-order'
        | 'instance-order'
        | 'option';
    _intent?: Element | undefined;
    /**
     * The preferred location(s) where the procedure should actually happen in coded or free text form. E.g. at home or nursing day care center.
     */
    location?: CodeableReference[] | undefined;
    /**
     * Any other notes and comments made about the service request. For example, internal billing notes.
     */
    note?: Annotation[] | undefined;
    /**
     * The date/time at which the requested service should occur.
     */
    occurrenceDateTime?: string | undefined;
    _occurrenceDateTime?: Element | undefined;
    /**
     * The date/time at which the requested service should occur.
     */
    occurrencePeriod?: Period | undefined;
    /**
     * The date/time at which the requested service should occur.
     */
    occurrenceTiming?: Timing | undefined;
    /**
     * For information from the medical record intended to support the delivery of the requested services, use the `supportingInformation` element.
     */
    orderDetail?: CodeableConcept[] | undefined;
    /**
     * Instructions in terms that are understood by the patient or consumer.
     */
    patientInstruction?: string | undefined;
    _patientInstruction?: Element | undefined;
    /**
     * If multiple performers are present, it is interpreted as a list of *alternative* performers without any preference regardless of order.  If order of preference is needed use the [request-performerOrder extension](extension-request-performerorder.html).  Use CareTeam to represent a group of performers (for example, Practitioner A *and* Practitioner B).
     */
    performer?: Reference[] | undefined;
    /**
     * This is a  role, not  a participation type.  In other words, does not describe the task but describes the capacity.  For example, “compounding pharmacy”, “psychiatrist” or “internal referral”.
     */
    performerType?: CodeableConcept | undefined;
    /**
     * Indicates how quickly the ServiceRequest should be addressed with respect to other requests.
     */
    priority?: ('routine' | 'urgent' | 'asap' | 'stat') | undefined;
    _priority?: Element | undefined;
    /**
     * An amount of service being requested which can be a quantity ( for example $1,500 home modification), a ratio ( for example, 20 half day visits per month), or a range (2.0 to 1.8 Gy per fraction).
     */
    quantityQuantity?: Quantity | undefined;
    /**
     * An amount of service being requested which can be a quantity ( for example $1,500 home modification), a ratio ( for example, 20 half day visits per month), or a range (2.0 to 1.8 Gy per fraction).
     */
    quantityRatio?: Ratio | undefined;
    /**
     * An amount of service being requested which can be a quantity ( for example $1,500 home modification), a ratio ( for example, 20 half day visits per month), or a range (2.0 to 1.8 Gy per fraction).
     */
    quantityRange?: Range | undefined;
    /**
     * This element represents why the referral is being made and may be used to decide how the service will be performed, or even if it will be performed at all. To be as specific as possible,  a reference to  *Observation* or *Condition* should be used if available.  Otherwise, use `concept.text` element if the data is free (uncoded) text as shown in the [CT Scan example](servicerequest-example-di.html).
     */
    reason?: CodeableReference[] | undefined;
    /**
     * This might not include provenances for all versions of the request – only those deemed “relevant” or important.
     * This SHALL NOT include the Provenance associated with this current version of the resource.  (If that provenance is deemed to be a “relevant” change, it will need to be added as part of a later update.  Until then, it can be queried directly as the Provenance that points to this version using _revinclude
     * All Provenances should have some historical version of this Request as their subject.
     */
    relevantHistory?: Reference[] | undefined;
    /**
     * The request takes the place of the referenced completed or terminated request(s).
     */
    replaces?: Reference[] | undefined;
    /**
     * This not the dispatcher, but rather who is the authorizer.  This element is not intended to handle delegation which would generally be managed through the Provenance resource.
     */
    requester?: Reference | undefined;
    /**
     * Requests are linked either by a "basedOn" relationship (i.e. one request is fulfilling another) or by having a common requisition. Requests that are part of the same requisition are generally treated independently from the perspective of changing their state or maintaining them after initial creation.
     */
    requisition?: Identifier | undefined;
    /**
     * The purpose of the ServiceRequest.specimen is to reflect the actual specimen that the requested test/procedure is asked to be performed on, whether the lab already has it or not.  References to specimens for purposes other than to perform a test/procedure on should be made using the ServiceRequest.supportingInfo or the Specimen.parent where the Specimen.parent would enable descendency and ServiceRequest.supportingInfo a general reference for context.
     */
    specimen?: Reference[] | undefined;
    /**
     * The status is generally fully in the control of the requester - they determine whether the order is draft or active and, after it has been activated, competed, cancelled or suspended. States relating to the activities of the performer are reflected on either the corresponding event (see [Event Pattern](event.html) for general discussion) or using the [Task](task.html) resource.
     */
    status: 'draft' | 'active' | 'on-hold' | 'revoked' | 'completed' | 'entered-in-error' | 'unknown';
    _status?: Element | undefined;
    /**
     * On whom or what the service is to be performed. This is usually a human patient, but can also be requested on animals, groups of humans or animals, devices such as dialysis machines, or even locations (typically for environmental scans).
     */
    subject: Reference;
    /**
     * To represent information about how the services are to be delivered use the `instructions` element.
     */
    supportingInfo?: Reference[] | undefined;
}
/**
 * A slot of time on a schedule that may be available for booking appointments.
 */
export interface Slot extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'Slot';
    /**
     * A slot may be allow multiple appointment types, but when booked, would only be used for one of the given appointment types.
     */
    appointmentType?: CodeableConcept[] | undefined;
    /**
     * Comments on the slot to describe any extended information. Such as custom constraints on the slot.
     */
    comment?: string | undefined;
    _comment?: Element | undefined;
    /**
     * Date/Time that the slot is to conclude.
     */
    end: string;
    _end?: Element | undefined;
    /**
     * External Ids for this item.
     */
    identifier?: Identifier[] | undefined;
    /**
     * This slot has already been overbooked, appointments are unlikely to be accepted for this time.
     */
    overbooked?: boolean | undefined;
    _overbooked?: Element | undefined;
    /**
     * The schedule resource that this slot defines an interval of status information.
     */
    schedule: Reference;
    /**
     * A broad categorization of the service that is to be performed during this appointment.
     */
    serviceCategory?: CodeableConcept[] | undefined;
    /**
     * The type of appointments that can be booked into this slot (ideally this would be an identifiable service - which is at a location, rather than the location itself). If provided then this overrides the value provided on the Schedule resource.
     */
    serviceType?: CodeableReference[] | undefined;
    /**
     * The specialty of a practitioner that would be required to perform the service requested in this appointment.
     */
    specialty?: CodeableConcept[] | undefined;
    /**
     * Date/Time that the slot is to begin.
     */
    start: string;
    _start?: Element | undefined;
    /**
     * busy | free | busy-unavailable | busy-tentative | entered-in-error.
     */
    status: 'busy' | 'free' | 'busy-unavailable' | 'busy-tentative' | 'entered-in-error';
    _status?: Element | undefined;
}
/**
 * A physical feature or landmark on a specimen, highlighted for context by the collector of the specimen (e.g. surgeon), that identifies the type of feature as well as its meaning (e.g. the red ink indicating the resection margin of the right lobe of the excised prostate tissue or wire loop at radiologically suspected tumor location).
 */
export interface SpecimenFeature extends BackboneElement {
    /**
     * Description of the feature of the specimen.
     */
    description: string;
    _description?: Element | undefined;
    /**
     * The landmark or feature being highlighted.
     */
    type: CodeableConcept;
}
/**
 * Details concerning the specimen collection.
 */
export interface SpecimenCollection extends BackboneElement {
    /**
     * Reasons for using BodyStructure reference include: 1.) Need to identify a specific site instance (e.g. a specific mole), 2.) a single pre-coordinated coded concept for the body site does not exist, 3.) a desire to represent bodySite using multiple attributes (e.g. modifiers).
     */
    bodySite?: CodeableReference | undefined;
    /**
     * Time when specimen was collected from subject - the physiologically relevant time.
     */
    collectedDateTime?: string | undefined;
    _collectedDateTime?: Element | undefined;
    /**
     * Time when specimen was collected from subject - the physiologically relevant time.
     */
    collectedPeriod?: Period | undefined;
    /**
     * Person who collected the specimen.
     */
    collector?: Reference | undefined;
    /**
     * A coded value specifying the technique that is used to perform the procedure.
     */
    device?: CodeableReference | undefined;
    /**
     * The span of time over which the collection of a specimen occurred.
     */
    duration?: Duration | undefined;
    /**
     * Representing fasting status using this element is preferred to representing it with an observation using a 'pre-coordinated code'  such as  LOINC 2005-7 (Calcium [Moles/​time] in 2 hour Urine --12 hours fasting), or  using  a component observation ` such as `Observation.component code`  = LOINC 49541-6 (Fasting status - Reported).
     */
    fastingStatusCodeableConcept?: CodeableConcept | undefined;
    /**
     * Representing fasting status using this element is preferred to representing it with an observation using a 'pre-coordinated code'  such as  LOINC 2005-7 (Calcium [Moles/​time] in 2 hour Urine --12 hours fasting), or  using  a component observation ` such as `Observation.component code`  = LOINC 49541-6 (Fasting status - Reported).
     */
    fastingStatusDuration?: Duration | undefined;
    /**
     * A coded value specifying the technique that is used to perform the procedure.
     */
    method?: CodeableConcept | undefined;
    /**
     * The procedure event during which the specimen was collected (e.g. the surgery leading to the collection of a pathology sample).
     */
    procedure?: Reference | undefined;
    /**
     * The quantity of specimen collected; for instance the volume of a blood sample, or the physical measurement of an anatomic pathology sample.
     */
    quantity?: Quantity | undefined;
}
/**
 * Details concerning processing and processing steps for the specimen.
 */
export interface SpecimenProcessing extends BackboneElement {
    /**
     * Material used in the processing step.
     */
    additive?: Reference[] | undefined;
    /**
     * Textual description of procedure.
     */
    description?: string | undefined;
    _description?: Element | undefined;
    /**
     * A coded value specifying the method used to process the specimen.
     */
    method?: CodeableConcept | undefined;
    /**
     * A record of the time or period when the specimen processing occurred.  For example the time of sample fixation or the period of time the sample was in formalin.
     */
    timeDateTime?: string | undefined;
    _timeDateTime?: Element | undefined;
    /**
     * A record of the time or period when the specimen processing occurred.  For example the time of sample fixation or the period of time the sample was in formalin.
     */
    timePeriod?: Period | undefined;
}
/**
 * The container holding the specimen.  The recursive nature of containers; i.e. blood in tube in tray in rack is not addressed here.
 */
export interface SpecimenContainer extends BackboneElement {
    /**
     * The device resource for the the container holding the specimen. If the container is in a holder then the referenced device will point to a parent device.
     */
    device: Reference;
    /**
     * The location of the container holding the specimen.
     */
    location?: Reference | undefined;
    /**
     * The quantity of specimen in the container; may be volume, dimensions, or other appropriate measurements, depending on the specimen type.
     */
    specimenQuantity?: Quantity | undefined;
}
/**
 * A sample to be used for analysis.
 */
export interface Specimen extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'Specimen';
    /**
     * The identifier assigned by the lab when accessioning specimen(s). This is not necessarily the same as the specimen identifier, depending on local lab procedures.
     */
    accessionIdentifier?: Identifier | undefined;
    /**
     * Details concerning the specimen collection.
     */
    collection?: SpecimenCollection | undefined;
    /**
     * This element signifies if the specimen is part of a group or pooled.
     */
    combined?: ('grouped' | 'pooled') | undefined;
    _combined?: Element | undefined;
    /**
     * Specimen condition is an observation made about the specimen.  It's a point-in-time assessment.  It can be used to assess its quality or appropriateness for a specific test.
     */
    condition?: CodeableConcept[] | undefined;
    /**
     * The container holding the specimen.  The recursive nature of containers; i.e. blood in tube in tray in rack is not addressed here.
     */
    container?: SpecimenContainer[] | undefined;
    /**
     * A physical feature or landmark on a specimen, highlighted for context by the collector of the specimen (e.g. surgeon), that identifies the type of feature as well as its meaning (e.g. the red ink indicating the resection margin of the right lobe of the excised prostate tissue or wire loop at radiologically suspected tumor location).
     */
    feature?: SpecimenFeature[] | undefined;
    /**
     * Id for specimen.
     */
    identifier?: Identifier[] | undefined;
    /**
     * To communicate any details or issues about the specimen or during the specimen collection. (for example: broken vial, sent with patient, frozen).
     */
    note?: Annotation[] | undefined;
    /**
     * The parent specimen could be the source from which the current specimen is derived by some processing step (e.g. an aliquot or isolate or extracted nucleic acids from clinical samples) or one of many specimens that were combined to create a pooled sample.
     */
    parent?: Reference[] | undefined;
    /**
     * Details concerning processing and processing steps for the specimen.
     */
    processing?: SpecimenProcessing[] | undefined;
    /**
     * Time when specimen is received by the testing laboratory for processing or testing.
     */
    receivedTime?: string | undefined;
    _receivedTime?: Element | undefined;
    /**
     * The request may be explicit or implied such with a ServiceRequest that requires a blood draw.
     */
    request?: Reference[] | undefined;
    /**
     * The role or reason for the specimen in the testing workflow.
     */
    role?: CodeableConcept[] | undefined;
    /**
     * This element is labeled as a modifier because the status contains codes that mark the resource as not currently valid.
     */
    status?: ('available' | 'unavailable' | 'unsatisfactory' | 'entered-in-error') | undefined;
    _status?: Element | undefined;
    /**
     * Where the specimen came from. This may be from patient(s), from a location (e.g., the source of an environmental sample), or a sampling of a substance, a biologically-derived product, or a device.
     */
    subject?: Reference | undefined;
    /**
     * The type can change the way that a specimen is handled and drives what kind of analyses can properly be performed on the specimen. It is frequently used in diagnostic work flow decision making systems.
     */
    type?: CodeableConcept | undefined;
}
/**
 * Substance introduced in the kind of container to preserve, maintain or enhance the specimen. Examples: Formalin, Citrate, EDTA.
 */
export interface SpecimenDefinitionTypeTestedContainerAdditive extends BackboneElement {
    /**
     * Substance introduced in the kind of container to preserve, maintain or enhance the specimen. Examples: Formalin, Citrate, EDTA.
     */
    additiveCodeableConcept?: CodeableConcept | undefined;
    /**
     * Substance introduced in the kind of container to preserve, maintain or enhance the specimen. Examples: Formalin, Citrate, EDTA.
     */
    additiveReference?: Reference | undefined;
}
/**
 * The specimen's container.
 */
export interface SpecimenDefinitionTypeTestedContainer extends BackboneElement {
    /**
     * Substance introduced in the kind of container to preserve, maintain or enhance the specimen. Examples: Formalin, Citrate, EDTA.
     */
    additive?: SpecimenDefinitionTypeTestedContainerAdditive[] | undefined;
    /**
     * Color of container cap.
     */
    cap?: CodeableConcept | undefined;
    /**
     * The capacity (volume or other measure) of this kind of container.
     */
    capacity?: Quantity | undefined;
    /**
     * The textual description of the kind of container.
     */
    description?: string | undefined;
    _description?: Element | undefined;
    /**
     * Examples: glass, plastic, metal.
     */
    material?: CodeableConcept | undefined;
    /**
     * The minimum volume to be conditioned in the container.
     */
    minimumVolumeQuantity?: Quantity | undefined;
    /**
     * The minimum volume to be conditioned in the container.
     */
    minimumVolumeString?: string | undefined;
    _minimumVolumeString?: Element | undefined;
    /**
     * Special processing that should be applied to the container for this kind of specimen.
     */
    preparation?: string | undefined;
    _preparation?: Element | undefined;
    /**
     * The type of container used to contain this kind of specimen.
     */
    type?: CodeableConcept | undefined;
}
/**
 * Set of instructions for preservation/transport of the specimen at a defined temperature interval, prior the testing process.
 */
export interface SpecimenDefinitionTypeTestedHandling extends BackboneElement {
    /**
     * Additional textual instructions for the preservation or transport of the specimen. For instance, 'Protect from light exposure'.
     */
    instruction?: string | undefined;
    _instruction?: Element | undefined;
    /**
     * The maximum time interval of preservation of the specimen with these conditions.
     */
    maxDuration?: Duration | undefined;
    /**
     * It qualifies the interval of temperature, which characterizes an occurrence of handling. Conditions that are not related to temperature may be handled in the instruction element.
     */
    temperatureQualifier?: CodeableConcept | undefined;
    /**
     * The temperature interval for this set of handling instructions.
     */
    temperatureRange?: Range | undefined;
}
/**
 * Specimen conditioned in a container as expected by the testing laboratory.
 */
export interface SpecimenDefinitionTypeTested extends BackboneElement {
    /**
     * The specimen's container.
     */
    container?: SpecimenDefinitionTypeTestedContainer | undefined;
    /**
     * Set of instructions for preservation/transport of the specimen at a defined temperature interval, prior the testing process.
     */
    handling?: SpecimenDefinitionTypeTestedHandling[] | undefined;
    /**
     * Primary of secondary specimen.
     */
    isDerived?: boolean | undefined;
    _isDerived?: Element | undefined;
    /**
     * The preference for this type of conditioned specimen.
     */
    preference: 'preferred' | 'alternate';
    _preference?: Element | undefined;
    /**
     * Criterion for rejection of the specimen in its container by the laboratory.
     */
    rejectionCriterion?: CodeableConcept[] | undefined;
    /**
     * Requirements for delivery and special handling of this kind of conditioned specimen.
     */
    requirement?: string | undefined;
    _requirement?: Element | undefined;
    /**
     * The usual time that a specimen of this kind is retained after the ordered tests are completed, for the purpose of additional testing.
     */
    retentionTime?: Duration | undefined;
    /**
     * Specimen can be used by only one test or panel if the value is "true".
     */
    singleUse?: boolean | undefined;
    _singleUse?: Element | undefined;
    /**
     * Where the specimen will be tested: e.g., lab, sector, device or any combination of these.
     */
    testingDestination?: CodeableConcept[] | undefined;
    /**
     * The kind of specimen conditioned for testing expected by lab.
     */
    type?: CodeableConcept | undefined;
}
/**
 * A kind of specimen with associated set of requirements.
 */
export interface SpecimenDefinition extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'SpecimenDefinition';
    /**
     * The date may be more recent than the approval date because of minor changes / editorial corrections.
     */
    approvalDate?: string | undefined;
    _approvalDate?: Element | undefined;
    /**
     * The action to be performed for collecting the specimen.
     */
    collection?: CodeableConcept[] | undefined;
    /**
     * May be a web site, an email address, a telephone number, etc.
     */
    contact?: ContactDetail[] | undefined;
    /**
     * Copyright statement relating to the SpecimenDefinition and/or its contents. Copyright statements are generally legal restrictions on the use and publishing of the SpecimenDefinition.
     */
    copyright?: string | undefined;
    _copyright?: Element | undefined;
    /**
     * For draft definitions, indicates the date of initial creation. For active definitions, represents the date of activation. For withdrawn definitions, indicates the date of withdrawal.
     */
    date?: string | undefined;
    _date?: Element | undefined;
    /**
     * The canonical URL pointing to another FHIR-defined SpecimenDefinition that is adhered to in whole or in part by this definition.
     */
    derivedFromCanonical?: string[] | undefined;
    _derivedFromCanonical?: Element[] | undefined;
    /**
     * The URL pointing to an externally-defined type of specimen, guideline or other definition that is adhered to in whole or in part by this definition.
     */
    derivedFromUri?: string[] | undefined;
    _derivedFromUri?: Element[] | undefined;
    /**
     * This description can be used to capture details such as comments about misuse, instructions for clinical use and interpretation, literature references, examples from the paper world, etc. It is not a rendering of the module as conveyed in the text field of the resource itself. This item SHOULD be populated unless the information is available from context.
     */
    description?: string | undefined;
    _description?: Element | undefined;
    /**
     * The effective period for a SpecimenDefinition determines when the content is applicable for usage and is independent of publication and review dates.
     */
    effectivePeriod?: Period | undefined;
    /**
     * Allows filtering of SpecimenDefinition that are appropriate for use vs. not.
     */
    experimental?: boolean | undefined;
    _experimental?: Element | undefined;
    /**
     * A business identifier assigned to this SpecimenDefinition.
     */
    identifier?: Identifier | undefined;
    /**
     * A jurisdiction in which the SpecimenDefinition is intended to be used.
     */
    jurisdiction?: CodeableConcept[] | undefined;
    /**
     * If specified, this is usually after the approval date.
     */
    lastReviewDate?: string | undefined;
    _lastReviewDate?: Element | undefined;
    /**
     * Preparation of the patient for specimen collection.
     */
    patientPreparation?: CodeableConcept[] | undefined;
    /**
     * Helps establish the "authority/credibility" of the SpecimenDefinition. May also allow for contact.
     */
    publisher?: Reference | undefined;
    /**
     * This element does not describe the usage of the SpecimenDefinition. Rather it is for traceability of ''why'' the resource is either needed or ''why'' it is defined as it is. This may be used to point to source materials or specifications that drove the structure of this SpecimenDefinition.
     */
    purpose?: string | undefined;
    _purpose?: Element | undefined;
    /**
     * A nominal state-transition diagram can be found in the [[definition.html#statemachine | Definition pattern]] documentation
     * Unknown does not represent "other" - one of the defined statuses must apply. Unknown is used when the authoring system is not sure what the current status is.
     */
    status: 'draft' | 'active' | 'retired' | 'unknown';
    _status?: Element | undefined;
    /**
     * Examples: person, animal, device, water ….
     */
    subjectCodeableConcept?: CodeableConcept | undefined;
    /**
     * Examples: person, animal, device, water ….
     */
    subjectReference?: Reference | undefined;
    /**
     * Time aspect of specimen collection (duration or offset).
     */
    timeAspect?: string | undefined;
    _timeAspect?: Element | undefined;
    /**
     * A short, descriptive, user-friendly title for the SpecimenDefinition.
     */
    title?: string | undefined;
    _title?: Element | undefined;
    /**
     * The kind of material to be collected.
     */
    typeCollected?: CodeableConcept | undefined;
    /**
     * Specimen conditioned in a container as expected by the testing laboratory.
     */
    typeTested?: SpecimenDefinitionTypeTested[] | undefined;
    /**
     * Can be a urn:uuid: or a urn:oid:, but real http: addresses are preferred. This is the URI that will be used when making canonical references to this resource.
     */
    url?: string | undefined;
    _url?: Element | undefined;
    /**
     * When multiple usageContexts are specified, there is no expectation for whether all or any of the contexts apply.
     */
    useContext?: UsageContext[] | undefined;
    /**
     * There may be multiple different instances of a SpecimenDefinition that have the same identifier but different versions.
     */
    version?: string | undefined;
    _version?: Element | undefined;
}
/**
 * An external specification that the content is mapped to.
 */
export interface StructureDefinitionMapping extends BackboneElement {
    /**
     * Comments about this mapping, including version notes, issues, scope limitations, and other important notes for usage.
     */
    comment?: string | undefined;
    _comment?: Element | undefined;
    /**
     * The specification is described once, with general comments, and then specific mappings are made that reference this declaration.
     */
    identity: string;
    _identity?: Element | undefined;
    /**
     * A name for the specification that is being mapped to.
     */
    name?: string | undefined;
    _name?: Element | undefined;
    /**
     * A formal identity for the specification being mapped to helps with identifying maps consistently.
     */
    uri?: string | undefined;
    _uri?: Element | undefined;
}
/**
 * Identifies the types of resource or data type elements to which the extension can be applied.
 */
export interface StructureDefinitionContext extends BackboneElement {
    /**
     * An expression that defines where an extension can be used in resources.
     */
    expression: string;
    _expression?: Element | undefined;
    /**
     * Defines how to interpret the expression that defines what the context of the extension is.
     */
    type: 'fhirpath' | 'element' | 'extension';
    _type?: Element | undefined;
}
/**
 * A snapshot view is expressed in a standalone form that can be used and interpreted without considering the base StructureDefinition.
 */
export interface StructureDefinitionSnapshot extends BackboneElement {
    /**
     * Captures constraints on each element within the resource.
     */
    element: ElementDefinition[];
}
/**
 * A differential view is expressed relative to the base StructureDefinition - a statement of differences that it applies.
 */
export interface StructureDefinitionDifferential extends BackboneElement {
    /**
     * Captures constraints on each element within the resource.
     */
    element: ElementDefinition[];
}
/**
 * A definition of a FHIR structure. This resource is used to describe the underlying resources, data types defined in FHIR, and also for describing extensions and constraints on resources and data types.
 */
export interface StructureDefinition extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'StructureDefinition';
    /**
     * Abstract Resources cannot be instantiated - a concrete sub-type must be used. Abstract datatypes and extensions cannot be used in an instance. For logical models, the exact implication of "abstract" will rest with the author, depending how the logical model is used. Flagging a constraint structure as abstract conveys design intent but makes no difference to how the structure definition is handled. Note that inline declared elements that are given the type "Element" in the StructureDefinition, but have children described, are anonymous concrete types that specialize Element.
     */
    abstract: boolean;
    _abstract?: Element | undefined;
    /**
     * If differential constraints are specified in this structure, they are applied to the base in a "differential" fashion. If there is no base, then the differential constraints cannot be provided (snapshot only). Differential structures are useful for the editing perspective, and snapshot structures are suitable for operational use. The FHIR Project provides a number of tools/services to populate snapshots from differential constraints. Logical Models have a base of "Element" or another logical model.
     */
    baseDefinition?: string | undefined;
    _baseDefinition?: Element | undefined;
    /**
     * May be a web site, an email address, a telephone number, etc.
     */
    contact?: ContactDetail[] | undefined;
    /**
     * Identifies the types of resource or data type elements to which the extension can be applied.
     */
    context?: StructureDefinitionContext[] | undefined;
    /**
     * The rules are only evaluated when the extension is present. When evaluating the invariant, the FHIRPath focus is the element that holds the extension, and %extension refers to the extension itself.
     */
    contextInvariant?: string[] | undefined;
    _contextInvariant?: Element[] | undefined;
    /**
     * ...
     */
    copyright?: string | undefined;
    _copyright?: Element | undefined;
    /**
     * The (c) symbol should NOT be included in this string. It will be added by software when rendering the notation. Full details about licensing, restrictions, warrantees, etc. goes in the more general 'copyright' element.
     */
    copyrightLabel?: string | undefined;
    _copyrightLabel?: Element | undefined;
    /**
     * Note that this is not the same as the resource last-modified-date, since the resource may be a secondary representation of the structure definition. Additional specific dates may be added as extensions or be found by consulting Provenances associated with past versions of the resource.
     */
    date?: string | undefined;
    _date?: Element | undefined;
    /**
     * If the definition is a specialization, then it adds to the differential new elements and optionally additional rules to an existing concrete type, and the snapshot includes the inherited elements and rules.  If the definition is a constraint, then it cannot define new elements, it can only make new rules about existing content (see [Profiling Resources](profiling.html#resources)).
     */
    derivation?: ('specialization' | 'constraint') | undefined;
    _derivation?: Element | undefined;
    /**
     * This description can be used to capture details such as comments about misuse, instructions for clinical use and interpretation, literature references, examples from the paper world, etc. It is not a rendering of the structure definition as conveyed in the 'text' field of the resource itself. This item SHOULD be populated unless the information is available from context (e.g. the language of the structure definition is presumed to be the predominant language in the place the structure definition was created).
     */
    description?: string | undefined;
    _description?: Element | undefined;
    /**
     * A differential view is expressed relative to the base StructureDefinition - a statement of differences that it applies.
     */
    differential?: StructureDefinitionDifferential | undefined;
    /**
     * Allows filtering of structure definitions that are appropriate for use versus not.
     */
    experimental?: boolean | undefined;
    _experimental?: Element | undefined;
    /**
     * A StructureDefinition does not need to specify the target it applies to as StructureDefinitions will often be valid across multiple versions of FHIR. FHIR tooling can determine whether a StructureDefinition is consistent with a particular StructureDefinition if desired.
     */
    fhirVersion?: string | undefined;
    _fhirVersion?: Element | undefined;
    /**
     * Typically, this is used for identifiers that can go in an HL7 V3 II (instance identifier) data type, and can then identify this structure definition outside of FHIR, where it is not possible to use the logical URI.
     */
    identifier?: Identifier[] | undefined;
    /**
     * It may be possible for the structure definition to be used in jurisdictions other than those for which it was originally designed or intended.
     */
    jurisdiction?: CodeableConcept[] | undefined;
    /**
     * Note that this element is deprecated. Use the artifact-keyword extension instead.
     */
    keyword?: Coding[] | undefined;
    /**
     * Defines the kind of structure that this definition is describing.
     */
    kind: 'primitive-type' | 'complex-type' | 'resource' | 'logical';
    _kind?: Element | undefined;
    /**
     * An external specification that the content is mapped to.
     */
    mapping?: StructureDefinitionMapping[] | undefined;
    /**
     * The name is not expected to be globally unique. The name should be a simple alphanumeric type name to ensure that it is machine-processing friendly.Some Examples:
     * * O2SatObservation
     * * PresentationReport
     * * Immunization2
     * * AcmeAdmissionRecordOld.
     */
    name: string;
    _name?: Element | undefined;
    /**
     * Usually an organization but may be an individual. The publisher (or steward) of the structure definition is the organization or individual primarily responsible for the maintenance and upkeep of the structure definition. This is not necessarily the same individual or organization that developed and initially authored the content. The publisher is the primary point of contact for questions or issues with the structure definition. This item SHOULD be populated unless the information is available from context.
     */
    publisher?: string | undefined;
    _publisher?: Element | undefined;
    /**
     * This element does not describe the usage of the structure definition. Instead, it provides traceability of ''why'' the resource is either needed or ''why'' it is defined as it is.  This may be used to point to source materials or specifications that drove the structure of this structure definition.
     */
    purpose?: string | undefined;
    _purpose?: Element | undefined;
    /**
     * A snapshot view is expressed in a standalone form that can be used and interpreted without considering the base StructureDefinition.
     */
    snapshot?: StructureDefinitionSnapshot | undefined;
    /**
     * Allows filtering of structure definitions that are appropriate for use versus not.
     */
    status: 'draft' | 'active' | 'retired' | 'unknown';
    _status?: Element | undefined;
    /**
     * This name does not need to be machine-processing friendly and may contain punctuation, white-space, etc.Applications don't have to use this name but can always fall back to it. The title also corresponds to the label for the root element.
     */
    title?: string | undefined;
    _title?: Element | undefined;
    /**
     * Note that in the case of constraints, the type could be determined by chasing through the baseDefinition references until a concrete structure (derivation = specialization) is reached, or by looking at the path of the first element in the snapshot - if present - but providing the type directly makes for simpler tooling and indexing.
     * The type must match the elements defined in the differential and the snapshot. For all FHIR defined types, the path name of the element will start with the type name. For logical models, where the type SHALL be a fully specified URL, the type name SHOULD start with the final path segment of the type URL where required. E.g. If the type was "http://example.org/fhir/MyLogicalModelType", the type name should start with 'MyLogicalModelType'.
     */
    type: string;
    _type?: Element | undefined;
    /**
     * Can be a urn:uuid: or a urn:oid: but real http: addresses are preferred.  Multiple instances may share the same URL if they have a distinct version.
     * The determination of when to create a new version of a resource (same url, new version) vs. defining a new artifact is up to the author.  Considerations for making this decision are found in [Technical and Business Versions](resource.html#versions).
     * In some cases, the resource can no longer be found at the stated url, but the url itself cannot change. Implementations can use the [meta.source](resource.html#meta) element to indicate where the current master source of the resource can be found.
     */
    url: string;
    _url?: Element | undefined;
    /**
     * When multiple useContexts are specified, there is no expectation that all or any of the contexts apply.
     */
    useContext?: UsageContext[] | undefined;
    /**
     * There may be different structure definition instances that have the same identifier but different versions.  The version can be appended to the url in a reference to allow a reference to a particular business version of the structure definition with the format [url]|[version]. The version SHOULD NOT contain a '#' - see [Business Version](resource.html#bv-format).
     */
    version?: string | undefined;
    _version?: Element | undefined;
    /**
     * If set as a string, this is a FHIRPath expression that has two additional context variables passed in - %version1 and %version2 and will return a negative number if version1 is newer, a positive number if version2 and a 0 if the version ordering can't be successfully be determined.
     */
    versionAlgorithmString?: string | undefined;
    _versionAlgorithmString?: Element | undefined;
    /**
     * If set as a string, this is a FHIRPath expression that has two additional context variables passed in - %version1 and %version2 and will return a negative number if version1 is newer, a positive number if version2 and a 0 if the version ordering can't be successfully be determined.
     */
    versionAlgorithmCoding?: Coding | undefined;
}
/**
 * It is not necessary for a structure map to identify any dependent structures, though not listing them may restrict its usefulness.
 */
export interface StructureMapStructure extends BackboneElement {
    /**
     * This is needed if both types have the same name (e.g. version conversion).
     */
    alias?: string | undefined;
    _alias?: Element | undefined;
    /**
     * Documentation that describes how the structure is used in the mapping.
     */
    documentation?: string | undefined;
    _documentation?: Element | undefined;
    /**
     * How the referenced structure is used in this mapping.
     */
    mode: 'source' | 'queried' | 'target' | 'produced';
    _mode?: Element | undefined;
    /**
     * The canonical reference to the structure.
     */
    url: string;
    _url?: Element | undefined;
}
/**
 * If no inputs are named, then the entry mappings are type based.
 */
export interface StructureMapGroupInput extends BackboneElement {
    /**
     * Documentation for this instance of data.
     */
    documentation?: string | undefined;
    _documentation?: Element | undefined;
    /**
     * Mode for this instance of data.
     */
    mode: 'source' | 'target';
    _mode?: Element | undefined;
    /**
     * Name for this instance of data.
     */
    name: string;
    _name?: Element | undefined;
    /**
     * Type for this instance of data.
     */
    type?: string | undefined;
    _type?: Element | undefined;
}
/**
 * Source inputs to the mapping.
 */
export interface StructureMapGroupRuleSource extends BackboneElement {
    /**
     * FHIRPath expression  - must be true or the mapping engine throws an error instead of completing.
     */
    check?: string | undefined;
    _check?: Element | undefined;
    /**
     * FHIRPath expression  - must be true or the rule does not apply.
     */
    condition?: string | undefined;
    _condition?: Element | undefined;
    /**
     * Type or variable this rule applies to.
     */
    context: string;
    _context?: Element | undefined;
    /**
     * If there's a default value on an item that can repeat, it will only be used once.
     */
    defaultValue?: string | undefined;
    _defaultValue?: Element | undefined;
    /**
     * Optional field for this source.
     */
    element?: string | undefined;
    _element?: Element | undefined;
    /**
     * How to handle the list mode for this element.
     */
    listMode?: ('first' | 'not_first' | 'last' | 'not_last' | 'only_one') | undefined;
    _listMode?: Element | undefined;
    /**
     * This is typically used for recording that something Is not transformed to the target for some reason.
     */
    logMessage?: string | undefined;
    _logMessage?: Element | undefined;
    /**
     * Specified maximum cardinality for the element - a number or a "*". This is optional; if present, it acts an implicit check on the input content (* just serves as documentation; it's the default value).
     */
    max?: string | undefined;
    _max?: Element | undefined;
    /**
     * Specified minimum cardinality for the element. This is optional; if present, it acts an implicit check on the input content.
     */
    min?: number | undefined;
    /**
     * Specified type for the element. This works as a condition on the mapping - use for polymorphic elements.
     */
    type?: string | undefined;
    _type?: Element | undefined;
    /**
     * Named context for field, if a field is specified.
     */
    variable?: string | undefined;
    _variable?: Element | undefined;
}
/**
 * Parameters to the transform.
 */
export interface StructureMapGroupRuleTargetParameter extends BackboneElement {
    /**
     * Parameter value - variable or literal.
     */
    valueId?: string | undefined;
    _valueId?: Element | undefined;
    /**
     * Parameter value - variable or literal.
     */
    valueString?: string | undefined;
    _valueString?: Element | undefined;
    /**
     * Parameter value - variable or literal.
     */
    valueBoolean?: boolean | undefined;
    _valueBoolean?: Element | undefined;
    /**
     * Parameter value - variable or literal.
     */
    valueInteger?: number | undefined;
    /**
     * Parameter value - variable or literal.
     */
    valueDecimal?: number | undefined;
    /**
     * Parameter value - variable or literal.
     */
    valueDate?: string | undefined;
    _valueDate?: Element | undefined;
    /**
     * Parameter value - variable or literal.
     */
    valueTime?: string | undefined;
    _valueTime?: Element | undefined;
    /**
     * Parameter value - variable or literal.
     */
    valueDateTime?: string | undefined;
    _valueDateTime?: Element | undefined;
}
/**
 * Content to create because of this mapping rule.
 */
export interface StructureMapGroupRuleTarget extends BackboneElement {
    /**
     * Variable this rule applies to.
     */
    context?: string | undefined;
    _context?: Element | undefined;
    /**
     * Field to create in the context.
     */
    element?: string | undefined;
    _element?: Element | undefined;
    /**
     * If field is a list, how to manage the list.
     */
    listMode?: ('first' | 'share' | 'last' | 'collate')[] | undefined;
    _listMode?: Element[] | undefined;
    /**
     * Internal rule reference for shared list items.
     */
    listRuleId?: string | undefined;
    _listRuleId?: Element | undefined;
    /**
     * Parameters to the transform.
     */
    parameter?: StructureMapGroupRuleTargetParameter[] | undefined;
    /**
     * How the data is copied / created.
     */
    transform?:
        | (
              | 'create'
              | 'copy'
              | 'truncate'
              | 'escape'
              | 'cast'
              | 'append'
              | 'translate'
              | 'reference'
              | 'dateOp'
              | 'uuid'
              | 'pointer'
              | 'evaluate'
              | 'cc'
              | 'c'
              | 'qty'
              | 'id'
              | 'cp'
          )
        | undefined;
    _transform?: Element | undefined;
    /**
     * Named context for field, if desired, and a field is specified.
     */
    variable?: string | undefined;
    _variable?: Element | undefined;
}
/**
 * Which other rules to apply in the context of this rule.
 */
export interface StructureMapGroupRuleDependent extends BackboneElement {
    /**
     * Name of a rule or group to apply.
     */
    name: string;
    _name?: Element | undefined;
    /**
     * Parameter to pass to the rule or group.
     */
    parameter: StructureMapGroupRuleTargetParameter[];
}
/**
 * Transform Rule from source to target.
 */
export interface StructureMapGroupRule extends BackboneElement {
    /**
     * Which other rules to apply in the context of this rule.
     */
    dependent?: StructureMapGroupRuleDependent[] | undefined;
    /**
     * Documentation for this instance of data.
     */
    documentation?: string | undefined;
    _documentation?: Element | undefined;
    /**
     * Name of the rule for internal references.
     */
    name?: string | undefined;
    _name?: Element | undefined;
    /**
     * Rules contained in this rule.
     */
    rule?: StructureMapGroupRule[] | undefined;
    /**
     * Source inputs to the mapping.
     */
    source: StructureMapGroupRuleSource[];
    /**
     * Content to create because of this mapping rule.
     */
    target?: StructureMapGroupRuleTarget[] | undefined;
}
/**
 * Organizes the mapping into manageable chunks for human review/ease of maintenance.
 */
export interface StructureMapGroup extends BackboneElement {
    /**
     * Additional supporting documentation that explains the purpose of the group and the types of mappings within it.
     */
    documentation?: string | undefined;
    _documentation?: Element | undefined;
    /**
     * Another group that this group adds rules to.
     */
    extends?: string | undefined;
    _extends?: Element | undefined;
    /**
     * If no inputs are named, then the entry mappings are type based.
     */
    input: StructureMapGroupInput[];
    /**
     * A unique name for the group for the convenience of human readers.
     */
    name: string;
    _name?: Element | undefined;
    /**
     * Transform Rule from source to target.
     */
    rule?: StructureMapGroupRule[] | undefined;
    /**
     * Not applicable if the underlying model is untyped. There can only be one default mapping for any particular type combination.
     */
    typeMode?: ('types' | 'type-and-types') | undefined;
    _typeMode?: Element | undefined;
}
/**
 * A Map of relationships between 2 structures that can be used to transform data.
 */
export interface StructureMap extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'StructureMap';
    /**
     * May be a web site, an email address, a telephone number, etc.
     */
    contact?: ContactDetail[] | undefined;
    /**
     * ...
     */
    copyright?: string | undefined;
    _copyright?: Element | undefined;
    /**
     * The (c) symbol should NOT be included in this string. It will be added by software when rendering the notation. Full details about licensing, restrictions, warrantees, etc. goes in the more general 'copyright' element.
     */
    copyrightLabel?: string | undefined;
    _copyrightLabel?: Element | undefined;
    /**
     * Note that this is not the same as the resource last-modified-date, since the resource may be a secondary representation of the structure map. Additional specific dates may be added as extensions or be found by consulting Provenances associated with past versions of the resource.
     */
    date?: string | undefined;
    _date?: Element | undefined;
    /**
     * This description can be used to capture details such as comments about misuse, instructions for clinical use and interpretation, literature references, examples from the paper world, etc. It is not a rendering of the structure map as conveyed in the 'text' field of the resource itself. This item SHOULD be populated unless the information is available from context (e.g. the language of the structure map is presumed to be the predominant language in the place the structure map was created).
     */
    description?: string | undefined;
    _description?: Element | undefined;
    /**
     * Allows filtering of structure maps that are appropriate for use versus not.
     */
    experimental?: boolean | undefined;
    _experimental?: Element | undefined;
    /**
     * Organizes the mapping into manageable chunks for human review/ease of maintenance.
     */
    group: StructureMapGroup[];
    /**
     * Typically, this is used for identifiers that can go in an HL7 V3 II (instance identifier) data type, and can then identify this structure map outside of FHIR, where it is not possible to use the logical URI.
     */
    identifier?: Identifier[] | undefined;
    /**
     * Other maps used by this map (canonical URLs).
     */
    import?: string[] | undefined;
    _import?: Element[] | undefined;
    /**
     * It may be possible for the structure map to be used in jurisdictions other than those for which it was originally designed or intended.
     */
    jurisdiction?: CodeableConcept[] | undefined;
    /**
     * The name is not expected to be globally unique. The name should be a simple alphanumeric type name to ensure that it is machine-processing friendly.
     */
    name: string;
    _name?: Element | undefined;
    /**
     * Usually an organization but may be an individual. The publisher (or steward) of the structure map is the organization or individual primarily responsible for the maintenance and upkeep of the structure map. This is not necessarily the same individual or organization that developed and initially authored the content. The publisher is the primary point of contact for questions or issues with the structure map. This item SHOULD be populated unless the information is available from context.
     */
    publisher?: string | undefined;
    _publisher?: Element | undefined;
    /**
     * This element does not describe the usage of the structure map. Instead, it provides traceability of ''why'' the resource is either needed or ''why'' it is defined as it is.  This may be used to point to source materials or specifications that drove the structure of this structure map.
     */
    purpose?: string | undefined;
    _purpose?: Element | undefined;
    /**
     * Allows filtering of structure maps that are appropriate for use versus not.
     */
    status: 'draft' | 'active' | 'retired' | 'unknown';
    _status?: Element | undefined;
    /**
     * It is not necessary for a structure map to identify any dependent structures, though not listing them may restrict its usefulness.
     */
    structure?: StructureMapStructure[] | undefined;
    /**
     * This name does not need to be machine-processing friendly and may contain punctuation, white-space, etc.
     */
    title?: string | undefined;
    _title?: Element | undefined;
    /**
     * Can be a urn:uuid: or a urn:oid: but real http: addresses are preferred.  Multiple instances may share the same URL if they have a distinct version.
     * The determination of when to create a new version of a resource (same url, new version) vs. defining a new artifact is up to the author.  Considerations for making this decision are found in [Technical and Business Versions](resource.html#versions).
     * In some cases, the resource can no longer be found at the stated url, but the url itself cannot change. Implementations can use the [meta.source](resource.html#meta) element to indicate where the current master source of the resource can be found.
     */
    url: string;
    _url?: Element | undefined;
    /**
     * When multiple useContexts are specified, there is no expectation that all or any of the contexts apply.
     */
    useContext?: UsageContext[] | undefined;
    /**
     * There may be different structure map instances that have the same identifier but different versions.  The version can be appended to the url in a reference to allow a reference to a particular business version of the structure map with the format [url]|[version]. The version SHOULD NOT contain a '#' - see [Business Version](resource.html#bv-format).
     */
    version?: string | undefined;
    _version?: Element | undefined;
    /**
     * If set as a string, this is a FHIRPath expression that has two additional context variables passed in - %version1 and %version2 and will return a negative number if version1 is newer, a positive number if version2 and a 0 if the version ordering can't be successfully be determined.
     */
    versionAlgorithmString?: string | undefined;
    _versionAlgorithmString?: Element | undefined;
    /**
     * If set as a string, this is a FHIRPath expression that has two additional context variables passed in - %version1 and %version2 and will return a negative number if version1 is newer, a positive number if version2 and a 0 if the version ordering can't be successfully be determined.
     */
    versionAlgorithmCoding?: Coding | undefined;
}
/**
 * The filter properties to be applied to narrow the subscription topic stream.  When multiple filters are applied, evaluates to true if all the conditions are met; otherwise it returns false.   (i.e., logical AND).
 */
export interface SubscriptionFilterBy extends BackboneElement {
    /**
     * The filter as defined in the `SubscriptionTopic.canfilterBy.filterParameter` element.
     */
    filterParameter: string;
    _filterParameter?: Element | undefined;
    /**
     * If filterDefinition refers to a SearchParameter, then the modifiers indicated must be allowed by the SearchParameter.
     */
    modifier?:
        | (
              | '='
              | 'eq'
              | 'ne'
              | 'gt'
              | 'lt'
              | 'ge'
              | 'le'
              | 'sa'
              | 'eb'
              | 'ap'
              | 'above'
              | 'below'
              | 'in'
              | 'not-in'
              | 'of-type'
          )
        | undefined;
    _modifier?: Element | undefined;
    /**
     * If the element is a reference to another resource, this element contains "Reference", and the targetProfile element defines what resources can be referenced. The targetProfile may be a reference to the general definition of a resource (e.g. http://hl7.org/fhir/StructureDefinition/Patient).
     */
    resourceType?: string | undefined;
    _resourceType?: Element | undefined;
    /**
     * The literal value or resource path as is legal in search - for example, "Patient/123" or "le1950".
     */
    value: string;
    _value?: Element | undefined;
}
/**
 * The subscription resource describes a particular client's request to be notified about a SubscriptionTopic.
 */
export interface Subscription extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'Subscription';
    /**
     * The type of channel to send notifications on.
     */
    channelType: Coding;
    /**
     * Contact details for a human to contact about the subscription. The primary use of this for system administrator troubleshooting.
     */
    contact?: ContactPoint[] | undefined;
    /**
     * Sending the payload has obvious security implications. The server is responsible for ensuring that the content is appropriately secured.
     */
    content?: ('empty' | 'id-only' | 'full-resource') | undefined;
    _content?: Element | undefined;
    /**
     * The mime type to send the payload in - either application/fhir+xml, or application/fhir+json. The MIME types "text/plain" and "text/html" may also be used for Email subscriptions.
     */
    contentType?: string | undefined;
    _contentType?: Element | undefined;
    /**
     * The server is permitted to deviate from this time but should observe it.
     */
    end?: string | undefined;
    _end?: Element | undefined;
    /**
     * For rest-hook the end-point must be an http: or https: URL; for websocket ws: or wss:; for email, a mailto: url; and for message the endpoint can be in any form of url the server understands (usually, http/s: or mllp:). The URI is allowed to be relative; in which case, it is relative to the server end-point (since there may be more than one, clients should avoid using relative URIs).
     */
    endpoint?: string | undefined;
    _endpoint?: Element | undefined;
    /**
     * The filter properties to be applied to narrow the subscription topic stream.  When multiple filters are applied, evaluates to true if all the conditions are met; otherwise it returns false.   (i.e., logical AND).
     */
    filterBy?: SubscriptionFilterBy[] | undefined;
    /**
     * Exactly what these mean depend on the channel type. They can convey additional information to the recipient and/or meet security requirements; for example, support of multiple headers in the outgoing notifications for rest-hook type subscriptions.
     */
    header?: string[] | undefined;
    _header?: Element[] | undefined;
    /**
     * If present,  a 'hearbeat" notification (keepalive) is sent via this channel with an the interval period equal to this elements integer value in seconds.    If not present, a heartbeat notification is not sent.
     */
    heartbeatPeriod?: number | undefined;
    /**
     * A formal identifier that is used to identify this code system when it is represented in other formats, or referenced in a specification, model, design or an instance.
     */
    identifier?: Identifier[] | undefined;
    /**
     * Entity with authorization to make subsequent revisions to the Subscription and also determines what data the subscription is authorized to disclose.
     */
    managingEntity?: Reference | undefined;
    /**
     * If present, the maximum number of triggering resources that will be included in a notification bundle (e.g., a server will not include more than this number of trigger resources in a single notification).  Note that this is not a strict limit on the number of entries in a bundle, as dependent resources can be included.
     */
    maxCount?: number | undefined;
    /**
     * A natural language name identifying the subscription.
     */
    name?: string | undefined;
    _name?: Element | undefined;
    /**
     * A description of why this subscription is defined.
     */
    reason?: string | undefined;
    _reason?: Element | undefined;
    /**
     * A client can only submit subscription resources in the requested or off state. Only the server can  move a subscription from requested to active, and then to error. Either the server or the client can turn a subscription off.
     * This element is labeled as a modifier because the status contains codes that mark the resource as not currently valid.
     */
    status: 'requested' | 'active' | 'error' | 'off' | 'entered-in-error';
    _status?: Element | undefined;
    /**
     * If present, the maximum amount of time a server will allow before failing a notification attempt.
     */
    timeout?: number | undefined;
    /**
     * The reference to the subscription topic to be notified about.
     */
    topic: string;
    _topic?: Element | undefined;
}
/**
 * Detailed information about events relevant to this subscription notification.
 */
export interface SubscriptionStatusNotificationEvent extends BackboneElement {
    /**
     * Additional context information for this event. Generally, this will contain references to additional resources included with the event (e.g., the Patient relevant to an Encounter), however it MAY refer to non-FHIR objects.
     */
    additionalContext?: Reference[] | undefined;
    /**
     * The sequential number of this event in this subscription context.
     */
    eventNumber: string;
    _eventNumber?: Element | undefined;
    /**
     * The focus of this event. While this will usually be a reference to the focus resource of the event, it MAY contain a reference to a non-FHIR object.
     */
    focus?: Reference | undefined;
    /**
     * The actual time this event occured on the server.
     */
    timestamp?: string | undefined;
    _timestamp?: Element | undefined;
}
/**
 * The SubscriptionStatus resource describes the state of a Subscription during notifications.
 */
export interface SubscriptionStatus extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'SubscriptionStatus';
    /**
     * Recommended practice: clear errors when status is updated.
     */
    error?: CodeableConcept[] | undefined;
    /**
     * The total number of actual events which have been generated since the Subscription was created (inclusive of this notification) - regardless of how many have been successfully communicated.  This number is NOT incremented for handshake and heartbeat notifications.
     */
    eventsSinceSubscriptionStart?: string | undefined;
    _eventsSinceSubscriptionStart?: Element | undefined;
    /**
     * Detailed information about events relevant to this subscription notification.
     */
    notificationEvent?: SubscriptionStatusNotificationEvent[] | undefined;
    /**
     * The status of the subscription, which marks the server state for managing the subscription.
     */
    status?: ('requested' | 'active' | 'error' | 'off' | 'entered-in-error') | undefined;
    _status?: Element | undefined;
    /**
     * The reference to the Subscription which generated this notification.
     */
    subscription: Reference;
    /**
     * This value SHOULD NOT be present when using `empty` payloads, MAY be present when using `id-only` payloads, and SHOULD be present when using `full-resource` payloads.
     */
    topic?: string | undefined;
    _topic?: Element | undefined;
    /**
     * The type of event being conveyed with this notificaiton.
     */
    type: 'handshake' | 'heartbeat' | 'event-notification' | 'query-status' | 'query-event';
    _type?: Element | undefined;
}
/**
 * The FHIR query based rules that the server should use to determine when to trigger a notification for this subscription topic.
 */
export interface SubscriptionTopicResourceTriggerQueryCriteria extends BackboneElement {
    /**
     * The rules are search criteria (without the [base] part). Like Bundle.entry.request.url, it has no leading "/".
     */
    current?: string | undefined;
    _current?: Element | undefined;
    /**
     * The rules are search criteria (without the [base] part). Like Bundle.entry.request.url, it has no leading "/".
     */
    previous?: string | undefined;
    _previous?: Element | undefined;
    /**
     * If set to true, both current and previous criteria must evaluate true to  trigger a notification for this topic.  Otherwise a notification for this topic will be triggered if either one evaluates to true.
     */
    requireBoth?: boolean | undefined;
    _requireBoth?: Element | undefined;
    /**
     * For "create" interactions, should the "previous" criteria count as an automatic pass or an automatic fail.
     */
    resultForCreate?: ('test-passes' | 'test-fails') | undefined;
    _resultForCreate?: Element | undefined;
    /**
     * For "delete" interactions, should the "current" criteria count as an automatic pass or an automatic fail.
     */
    resultForDelete?: ('test-passes' | 'test-fails') | undefined;
    _resultForDelete?: Element | undefined;
}
/**
 * A definition of a resource-based event that triggers a notification based on the SubscriptionTopic. The criteria may be just a human readable description and/or a full FHIR search string or FHIRPath expression. Multiple triggers are considered OR joined (e.g., a resource update matching ANY of the definitions will trigger a notification).
 */
export interface SubscriptionTopicResourceTrigger extends BackboneElement {
    /**
     * Implementation of particular subscription topics might not use a computable definition and instead base their design on the definition.
     */
    description?: string | undefined;
    _description?: Element | undefined;
    /**
     * FHIRPath expression with %previous and %current vars.
     */
    fhirPathCriteria?: string | undefined;
    _fhirPathCriteria?: Element | undefined;
    /**
     * The FHIR query based rules that the server should use to determine when to trigger a notification for this subscription topic.
     */
    queryCriteria?: SubscriptionTopicResourceTriggerQueryCriteria | undefined;
    /**
     * URL of the Resource that is the type used in this resource trigger.  Relative URLs are relative to the StructureDefinition root of the implemented FHIR version (e.g., http://hl7.org/fhir/StructureDefinition). For example, "Patient" maps to http://hl7.org/fhir/StructureDefinition/Patient.  For more information, see <a href="elementdefinition-definitions.html#ElementDefinition.type.code">ElementDefinition.type.code</a>.
     */
    resource: string;
    _resource?: Element | undefined;
    /**
     * The FHIR RESTful interaction which can be used to trigger a notification for the SubscriptionTopic. Multiple values are considered OR joined (e.g., CREATE or UPDATE).
     */
    supportedInteraction?: ('create' | 'update' | 'delete')[] | undefined;
    _supportedInteraction?: Element[] | undefined;
}
/**
 * Event definition which can be used to trigger the SubscriptionTopic.
 */
export interface SubscriptionTopicEventTrigger extends BackboneElement {
    /**
     * Implementation of particular subscription topics might not use a computable definition and instead base their design on the definition.
     */
    description?: string | undefined;
    _description?: Element | undefined;
    /**
     * A well-defined event which can be used to trigger notifications from the SubscriptionTopic.
     */
    event: CodeableConcept;
    /**
     * URL of the Resource that is the focus type used in this event trigger.  Relative URLs are relative to the StructureDefinition root of the implemented FHIR version (e.g., http://hl7.org/fhir/StructureDefinition). For example, "Patient" maps to http://hl7.org/fhir/StructureDefinition/Patient.  For more information, see <a href="elementdefinition-definitions.html#ElementDefinition.type.code">ElementDefinition.type.code</a>.
     */
    resource: string;
    _resource?: Element | undefined;
}
/**
 * List of properties by which Subscriptions on the SubscriptionTopic can be filtered. May be defined Search Parameters (e.g., Encounter.patient) or parameters defined within this SubscriptionTopic context (e.g., hub.event).
 */
export interface SubscriptionTopicCanFilterBy extends BackboneElement {
    /**
     * Description of how this filtering parameter is intended to be used.
     */
    description?: string | undefined;
    _description?: Element | undefined;
    /**
     * Chained parameters are allowed (like "patient.gender") - but can not use colons or modifiers.
     */
    filterDefinition?: string | undefined;
    _filterDefinition?: Element | undefined;
    /**
     * Chained parameters are allowed (like "patient.gender") - but can not use colons or modifiers.
     */
    filterParameter: string;
    _filterParameter?: Element | undefined;
    /**
     * Allowable operators to apply when determining matches (Search Modifiers).  If the filterParameter is a SearchParameter, this list of modifiers SHALL be a strict subset of the modifiers defined on that SearchParameter.
     */
    modifier?:
        | (
              | '='
              | 'eq'
              | 'ne'
              | 'gt'
              | 'lt'
              | 'ge'
              | 'le'
              | 'sa'
              | 'eb'
              | 'ap'
              | 'above'
              | 'below'
              | 'in'
              | 'not-in'
              | 'of-type'
          )[]
        | undefined;
    _modifier?: Element[] | undefined;
    /**
     * URL of the Resource that is the type used in this filter. Relative URLs are relative to the StructureDefinition root of the implemented FHIR version (e.g., http://hl7.org/fhir/StructureDefinition). For example, "Patient" maps to http://hl7.org/fhir/StructureDefinition/Patient. For more information, see <a href="elementdefinition-definitions.html#ElementDefinition.type.code">ElementDefinition.type.code</a>.
     */
    resource?: string | undefined;
    _resource?: Element | undefined;
}
/**
 * List of properties to describe the shape (e.g., resources) included in notifications from this Subscription Topic.
 */
export interface SubscriptionTopicNotificationShape extends BackboneElement {
    /**
     * Search-style _include directives, rooted in the resource for this shape. Servers SHOULD include resources listed here, if they exist and the user is authorized to receive them.  Clients SHOULD be prepared to receive these additional resources, but SHALL function properly without them.
     */
    include?: string[] | undefined;
    _include?: Element[] | undefined;
    /**
     * URL of the Resource that is the type used in this shape. Relative URLs are relative to the StructureDefinition root of the implemented FHIR version (e.g., http://hl7.org/fhir/StructureDefinition). For example, "Patient" maps to http://hl7.org/fhir/StructureDefinition/Patient. For more information, see <a href="elementdefinition-definitions.html#ElementDefinition.type.code">ElementDefinition.type.code</a>.
     */
    resource: string;
    _resource?: Element | undefined;
    /**
     * Search-style _revinclude directives, rooted in the resource for this shape. Servers SHOULD include resources listed here, if they exist and the user is authorized to receive them.  Clients SHOULD be prepared to receive these additional resources, but SHALL function properly without them.
     */
    revInclude?: string[] | undefined;
    _revInclude?: Element[] | undefined;
}
/**
 * Describes a stream of resource state changes or events and annotated with labels useful to filter projections from this topic.
 */
export interface SubscriptionTopic extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'SubscriptionTopic';
    /**
     * The date may be more recent than the approval date because of minor changes / editorial corrections.
     */
    approvalDate?: string | undefined;
    _approvalDate?: Element | undefined;
    /**
     * List of properties by which Subscriptions on the SubscriptionTopic can be filtered. May be defined Search Parameters (e.g., Encounter.patient) or parameters defined within this SubscriptionTopic context (e.g., hub.event).
     */
    canFilterBy?: SubscriptionTopicCanFilterBy[] | undefined;
    /**
     * May be a web site, an email address, a telephone number, etc.
     */
    contact?: ContactDetail[] | undefined;
    /**
     * ...
     */
    copyright?: string | undefined;
    _copyright?: Element | undefined;
    /**
     * The (c) symbol should NOT be included in this string. It will be added by software when rendering the notation. Full details about licensing, restrictions, warrantees, etc. goes in the more general 'copyright' element.
     */
    copyrightLabel?: string | undefined;
    _copyrightLabel?: Element | undefined;
    /**
     * For draft definitions, indicates the date of initial creation.  For active definitions, represents the date of activation.  For withdrawn definitions, indicates the date of withdrawal.
     */
    date?: string | undefined;
    _date?: Element | undefined;
    /**
     * The canonical URL pointing to another FHIR-defined SubscriptionTopic that is adhered to in whole or in part by this SubscriptionTopic.
     */
    derivedFrom?: string[] | undefined;
    _derivedFrom?: Element[] | undefined;
    /**
     * This description can be used to capture details such as comments about misuse, instructions for clinical use and interpretation, literature references, examples from the paper world, etc. It is not a rendering of the module as conveyed in the text field of the resource itself. This item SHOULD be populated unless the information is available from context.
     */
    description?: string | undefined;
    _description?: Element | undefined;
    /**
     * The effective period for a SubscriptionTopic  determines when the content is applicable for usage and is independent of publication and review dates. For example, a measure intended to be used for the year 2016 would be published in 2015.
     */
    effectivePeriod?: Period | undefined;
    /**
     * Event definition which can be used to trigger the SubscriptionTopic.
     */
    eventTrigger?: SubscriptionTopicEventTrigger[] | undefined;
    /**
     * Allows filtering of SubscriptionTopic that are appropriate for use vs. not.
     */
    experimental?: boolean | undefined;
    _experimental?: Element | undefined;
    /**
     * Note: This is a business identifier, not a resource identifier (see [discussion](resource.html#identifiers)).  It is best practice for the identifier to only appear on a single resource instance, however business practices may occasionally dictate that multiple resource instances with the same identifier can exist - possibly even with different resource types.  For example, multiple Patient and a Person resource instance might share the same social insurance number.
     */
    identifier?: Identifier[] | undefined;
    /**
     * A jurisdiction in which the Topic is intended to be used.
     */
    jurisdiction?: CodeableConcept[] | undefined;
    /**
     * If specified, this is usually after the approval date.
     */
    lastReviewDate?: string | undefined;
    _lastReviewDate?: Element | undefined;
    /**
     * List of properties to describe the shape (e.g., resources) included in notifications from this Subscription Topic.
     */
    notificationShape?: SubscriptionTopicNotificationShape[] | undefined;
    /**
     * Helps establish the "authority/credibility" of the SubscriptionTopic.  May also allow for contact.
     */
    publisher?: string | undefined;
    _publisher?: Element | undefined;
    /**
     * This element does not describe the usage of the Topic.  Rather it is for traceability of ''why'' the resource is either needed or ''why'' it is defined as it is.  This may be used to point to source materials or specifications that drove the structure of this Topic.
     */
    purpose?: string | undefined;
    _purpose?: Element | undefined;
    /**
     * A definition of a resource-based event that triggers a notification based on the SubscriptionTopic. The criteria may be just a human readable description and/or a full FHIR search string or FHIRPath expression. Multiple triggers are considered OR joined (e.g., a resource update matching ANY of the definitions will trigger a notification).
     */
    resourceTrigger?: SubscriptionTopicResourceTrigger[] | undefined;
    /**
     * A nominal state-transition diagram can be found in the [[definition.html#statemachine | Definition pattern]] documentation
     * Unknown does not represent "other" - one of the defined statuses must apply.  Unknown is used when the authoring system is not sure what the current status is.
     */
    status: 'draft' | 'active' | 'retired' | 'unknown';
    _status?: Element | undefined;
    /**
     * A short, descriptive, user-friendly title for the SubscriptionTopic, for example, "admission".
     */
    title?: string | undefined;
    _title?: Element | undefined;
    /**
     * Can be a urn:uuid: or a urn:oid: but real http: addresses are preferred.  Multiple instances may share the same URL if they have a distinct version.
     * The determination of when to create a new version of a resource (same url, new version) vs. defining a new artifact is up to the author.  Considerations for making this decision are found in [Technical and Business Versions](resource.html#versions).
     * In some cases, the resource can no longer be found at the stated url, but the url itself cannot change. Implementations can use the [meta.source](resource.html#meta) element to indicate where the current master source of the resource can be found.
     */
    url: string;
    _url?: Element | undefined;
    /**
     * When multiple usageContexts are specified, there is no expectation for whether all or any of the contexts apply.
     */
    useContext?: UsageContext[] | undefined;
    /**
     * There may be multiple different instances of a subscription topic that have the same identifier but different versions.
     */
    version?: string | undefined;
    _version?: Element | undefined;
}
/**
 * A substance can be composed of other substances.
 */
export interface SubstanceIngredient extends BackboneElement {
    /**
     * The amount of the ingredient in the substance - a concentration ratio.
     */
    quantity?: Ratio | undefined;
    /**
     * Another substance that is a component of this substance.
     */
    substanceCodeableConcept?: CodeableConcept | undefined;
    /**
     * Another substance that is a component of this substance.
     */
    substanceReference?: Reference | undefined;
}
/**
 * A homogeneous material with a definite composition.
 */
export interface Substance extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'Substance';
    /**
     * The level of granularity is defined by the category concepts in the value set.   More fine-grained filtering can be performed using the metadata and/or terminology hierarchy in Substance.code.
     */
    category?: CodeableConcept[] | undefined;
    /**
     * This could be a reference to an externally defined code.  It could also be a locally assigned code (e.g. a formulary),  optionally with translations to the standard drug codes.
     */
    code: CodeableReference;
    /**
     * A description of the substance - its appearance, handling requirements, and other usage notes.
     */
    description?: string | undefined;
    _description?: Element | undefined;
    /**
     * When the substance is no longer valid to use. For some substances, a single arbitrary date is used for expiry.
     */
    expiry?: string | undefined;
    _expiry?: Element | undefined;
    /**
     * This identifier is associated with the kind of substance in contrast to the  Substance.instance.identifier which is associated with the package/container.
     */
    identifier?: Identifier[] | undefined;
    /**
     * A substance can be composed of other substances.
     */
    ingredient?: SubstanceIngredient[] | undefined;
    /**
     * A boolean to indicate if this an instance of a substance or a kind of one (a definition).
     */
    instance: boolean;
    _instance?: Element | undefined;
    /**
     * The amount of the substance.
     */
    quantity?: Quantity | undefined;
    /**
     * A code to indicate if the substance is actively used.
     */
    status?: ('active' | 'inactive' | 'entered-in-error') | undefined;
    _status?: Element | undefined;
}
/**
 * Moiety, for structural modifications.
 */
export interface SubstanceDefinitionMoiety extends BackboneElement {
    /**
     * Quantitative value for this moiety.
     */
    amountQuantity?: Quantity | undefined;
    /**
     * Quantitative value for this moiety.
     */
    amountString?: string | undefined;
    _amountString?: Element | undefined;
    /**
     * Identifier by which this moiety substance is known.
     */
    identifier?: Identifier | undefined;
    /**
     * The measurement type of the quantitative value. In capturing the actual relative amounts of substances or molecular fragments it may be necessary to indicate whether the amount refers to, for example, a mole ratio or weight ratio.
     */
    measurementType?: CodeableConcept | undefined;
    /**
     * Molecular formula for this moiety of this substance, typically using the Hill system.
     */
    molecularFormula?: string | undefined;
    _molecularFormula?: Element | undefined;
    /**
     * Textual name for this moiety substance.
     */
    name?: string | undefined;
    _name?: Element | undefined;
    /**
     * Optical activity type.
     */
    opticalActivity?: CodeableConcept | undefined;
    /**
     * Role that the moiety is playing.
     */
    role?: CodeableConcept | undefined;
    /**
     * Stereochemistry type.
     */
    stereochemistry?: CodeableConcept | undefined;
}
/**
 * General specifications for this substance.
 */
export interface SubstanceDefinitionProperty extends BackboneElement {
    /**
     * A code expressing the type of property.
     */
    type: CodeableConcept;
    /**
     * A value for the property.
     */
    valueCodeableConcept?: CodeableConcept | undefined;
    /**
     * A value for the property.
     */
    valueQuantity?: Quantity | undefined;
    /**
     * A value for the property.
     */
    valueDate?: string | undefined;
    _valueDate?: Element | undefined;
    /**
     * A value for the property.
     */
    valueBoolean?: boolean | undefined;
    _valueBoolean?: Element | undefined;
    /**
     * A value for the property.
     */
    valueAttachment?: Attachment | undefined;
}
/**
 * The molecular weight or weight range (for proteins, polymers or nucleic acids).
 */
export interface SubstanceDefinitionMolecularWeight extends BackboneElement {
    /**
     * Used to capture quantitative values for a variety of elements. If only limits are given, the arithmetic mean would be the average. If only a single definite value for a given element is given, it would be captured in this field.
     */
    amount: Quantity;
    /**
     * The method by which the molecular weight was determined.
     */
    method?: CodeableConcept | undefined;
    /**
     * Type of molecular weight such as exact, average (also known as. number average), weight average.
     */
    type?: CodeableConcept | undefined;
}
/**
 * A depiction of the structure or characterization of the substance.
 */
export interface SubstanceDefinitionStructureRepresentation extends BackboneElement {
    /**
     * An attached file with the structural representation or characterization e.g. a molecular structure graphic of the substance, a JCAMP or AnIML file.
     */
    document?: Reference | undefined;
    /**
     * The format of the representation e.g. InChI, SMILES, MOLFILE, CDX, SDF, PDB, mmCIF. The logical content type rather than the physical file format of a document.
     */
    format?: CodeableConcept | undefined;
    /**
     * The structural representation or characterization as a text string in a standard format.
     */
    representation?: string | undefined;
    _representation?: Element | undefined;
    /**
     * The kind of structural representation (e.g. full, partial).
     */
    type?: CodeableConcept | undefined;
}
/**
 * Structural information.
 */
export interface SubstanceDefinitionStructure extends BackboneElement {
    /**
     * Molecular formula of this substance, typically using the Hill system.
     */
    molecularFormula?: string | undefined;
    _molecularFormula?: Element | undefined;
    /**
     * Specified per moiety according to the Hill system, i.e. first C, then H, then alphabetical, each moiety separated by a dot.
     */
    molecularFormulaByMoiety?: string | undefined;
    _molecularFormulaByMoiety?: Element | undefined;
    /**
     * The molecular weight or weight range (for proteins, polymers or nucleic acids).
     */
    molecularWeight?: SubstanceDefinitionMolecularWeight | undefined;
    /**
     * Optical activity type.
     */
    opticalActivity?: CodeableConcept | undefined;
    /**
     * A depiction of the structure or characterization of the substance.
     */
    representation?: SubstanceDefinitionStructureRepresentation[] | undefined;
    /**
     * The source of information about the structure.
     */
    sourceDocument?: Reference[] | undefined;
    /**
     * Stereochemistry type.
     */
    stereochemistry?: CodeableConcept | undefined;
    /**
     * The method used to elucidate the structure or characterization of the drug substance. Examples: X-ray, HPLC, NMR, Peptide mapping, Ligand binding assay.
     */
    technique?: CodeableConcept[] | undefined;
}
/**
 * Codes associated with the substance.
 */
export interface SubstanceDefinitionCode extends BackboneElement {
    /**
     * The specific code.
     */
    code?: CodeableConcept | undefined;
    /**
     * Any comment can be provided in this field, if necessary.
     */
    note?: Annotation[] | undefined;
    /**
     * Supporting literature.
     */
    source?: Reference[] | undefined;
    /**
     * Status of the code assignment, for example 'provisional', 'approved'.
     */
    status?: CodeableConcept | undefined;
    /**
     * The date at which the code status was changed as part of the terminology maintenance.
     */
    statusDate?: string | undefined;
    _statusDate?: Element | undefined;
}
/**
 * Details of the official nature of this name.
 */
export interface SubstanceDefinitionNameOfficial extends BackboneElement {
    /**
     * Which authority uses this official name.
     */
    authority?: CodeableConcept | undefined;
    /**
     * Date of the official name change.
     */
    date?: string | undefined;
    _date?: Element | undefined;
    /**
     * The status of the official name, for example 'draft', 'active', 'retired'.
     */
    status?: CodeableConcept | undefined;
}
/**
 * Names applicable to this substance.
 */
export interface SubstanceDefinitionName extends BackboneElement {
    /**
     * The use context of this name for example if there is a different name a drug active ingredient as opposed to a food colour additive.
     */
    domain?: CodeableConcept[] | undefined;
    /**
     * The jurisdiction where this name applies.
     */
    jurisdiction?: CodeableConcept[] | undefined;
    /**
     * Human language that the name is written in.
     */
    language?: CodeableConcept[] | undefined;
    /**
     * The actual name.
     */
    name: string;
    _name?: Element | undefined;
    /**
     * Details of the official nature of this name.
     */
    official?: SubstanceDefinitionNameOfficial[] | undefined;
    /**
     * If this is the preferred name for this substance.
     */
    preferred?: boolean | undefined;
    _preferred?: Element | undefined;
    /**
     * Supporting literature.
     */
    source?: Reference[] | undefined;
    /**
     * The status of the name, for example 'current', 'proposed'.
     */
    status?: CodeableConcept | undefined;
    /**
     * A synonym of this particular name, by which the substance is also known.
     */
    synonym?: SubstanceDefinitionName[] | undefined;
    /**
     * A translation for this name into another human language.
     */
    translation?: SubstanceDefinitionName[] | undefined;
    /**
     * Name type, for example 'systematic',  'scientific, 'brand'.
     */
    type?: CodeableConcept | undefined;
}
/**
 * A link between this substance and another, with details of the relationship.
 */
export interface SubstanceDefinitionRelationship extends BackboneElement {
    /**
     * A numeric factor for the relationship, for instance to express that the salt of a substance has some percentage of the active substance in relation to some other.
     */
    amountQuantity?: Quantity | undefined;
    /**
     * A numeric factor for the relationship, for instance to express that the salt of a substance has some percentage of the active substance in relation to some other.
     */
    amountRatio?: Ratio | undefined;
    /**
     * A numeric factor for the relationship, for instance to express that the salt of a substance has some percentage of the active substance in relation to some other.
     */
    amountString?: string | undefined;
    _amountString?: Element | undefined;
    /**
     * An operator for the amount, for example "average", "approximately", "less than".
     */
    comparator?: CodeableConcept | undefined;
    /**
     * For example where an enzyme strongly bonds with a particular substance, this is a defining relationship for that enzyme, out of several possible substance relationships.
     */
    isDefining?: boolean | undefined;
    _isDefining?: Element | undefined;
    /**
     * For use when the numeric has an uncertain range.
     */
    ratioHighLimitAmount?: Ratio | undefined;
    /**
     * Supporting literature.
     */
    source?: Reference[] | undefined;
    /**
     * A pointer to another substance, as a resource or just a representational code.
     */
    substanceDefinitionReference?: Reference | undefined;
    /**
     * A pointer to another substance, as a resource or just a representational code.
     */
    substanceDefinitionCodeableConcept?: CodeableConcept | undefined;
    /**
     * For example "salt to parent", "active moiety", "starting material", "polymorph", "impurity of".
     */
    type: CodeableConcept;
}
/**
 * Material or taxonomic/anatomical source for the substance.
 */
export interface SubstanceDefinitionSourceMaterial extends BackboneElement {
    /**
     * The country or countries where the material is harvested.
     */
    countryOfOrigin?: CodeableConcept[] | undefined;
    /**
     * The genus of an organism, typically referring to the Latin epithet of the genus element of the plant/animal scientific name.
     */
    genus?: CodeableConcept | undefined;
    /**
     * An anatomical origin of the source material within an organism.
     */
    part?: CodeableConcept | undefined;
    /**
     * The species of an organism, typically referring to the Latin epithet of the species of the plant/animal.
     */
    species?: CodeableConcept | undefined;
    /**
     * A classification that provides the origin of the raw material. Example: cat hair would be an Animal source type.
     */
    type?: CodeableConcept | undefined;
}
/**
 * The detailed description of a substance, typically at a level beyond what is used for prescribing.
 */
export interface SubstanceDefinition extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'SubstanceDefinition';
    /**
     * A high level categorization, e.g. polymer or nucleic acid, or food, chemical, biological, or a lower level such as the general types of polymer (linear or branch chain) or type of impurity (process related or contaminant).
     */
    classification?: CodeableConcept[] | undefined;
    /**
     * Codes associated with the substance.
     */
    code?: SubstanceDefinitionCode[] | undefined;
    /**
     * Textual description of the substance.
     */
    description?: string | undefined;
    _description?: Element | undefined;
    /**
     * If the substance applies to human or veterinary use.
     */
    domain?: CodeableConcept | undefined;
    /**
     * The quality standard, established benchmark, to which substance complies (e.g. USP/NF, Ph. Eur, JP, BP, Company Standard).
     */
    grade?: CodeableConcept[] | undefined;
    /**
     * Identifier by which this substance is known.
     */
    identifier?: Identifier[] | undefined;
    /**
     * Supporting literature.
     */
    informationSource?: Reference[] | undefined;
    /**
     * The entity that creates, makes, produces or fabricates the substance. This is a set of potential manufacturers but is not necessarily comprehensive.
     */
    manufacturer?: Reference[] | undefined;
    /**
     * Moiety, for structural modifications.
     */
    moiety?: SubstanceDefinitionMoiety[] | undefined;
    /**
     * The molecular weight or weight range (for proteins, polymers or nucleic acids).
     */
    molecularWeight?: SubstanceDefinitionMolecularWeight[] | undefined;
    /**
     * Names applicable to this substance.
     */
    name?: SubstanceDefinitionName[] | undefined;
    /**
     * Textual comment about the substance's catalogue or registry record.
     */
    note?: Annotation[] | undefined;
    /**
     * Data items specific to nucleic acids.
     */
    nucleicAcid?: Reference | undefined;
    /**
     * Data items specific to polymers.
     */
    polymer?: Reference | undefined;
    /**
     * General specifications for this substance.
     */
    property?: SubstanceDefinitionProperty[] | undefined;
    /**
     * Data items specific to proteins.
     */
    protein?: Reference | undefined;
    /**
     * General information detailing this substance.
     */
    referenceInformation?: Reference | undefined;
    /**
     * A link between this substance and another, with details of the relationship.
     */
    relationship?: SubstanceDefinitionRelationship[] | undefined;
    /**
     * Material or taxonomic/anatomical source for the substance.
     */
    sourceMaterial?: SubstanceDefinitionSourceMaterial | undefined;
    /**
     * Status of substance within the catalogue e.g. active, retired.
     */
    status?: CodeableConcept | undefined;
    /**
     * Structural information.
     */
    structure?: SubstanceDefinitionStructure | undefined;
    /**
     * An entity that is the source for the substance. It may be different from the manufacturer. Supplier is synonymous to a distributor.
     */
    supplier?: Reference[] | undefined;
    /**
     * A business level version identifier of the substance.
     */
    version?: string | undefined;
    _version?: Element | undefined;
}
/**
 * The linkages between sugar residues will also be captured.
 */
export interface SubstanceNucleicAcidSubunitLinkage extends BackboneElement {
    /**
     * The entity that links the sugar residues together should also be captured for nearly all naturally occurring nucleic acid the linkage is a phosphate group. For many synthetic oligonucleotides phosphorothioate linkages are often seen. Linkage connectivity is assumed to be 3’-5’. If the linkage is either 3’-3’ or 5’-5’ this should be specified.
     */
    connectivity?: string | undefined;
    _connectivity?: Element | undefined;
    /**
     * Each linkage will be registered as a fragment and have an ID.
     */
    identifier?: Identifier | undefined;
    /**
     * Each linkage will be registered as a fragment and have at least one name. A single name shall be assigned to each linkage.
     */
    name?: string | undefined;
    _name?: Element | undefined;
    /**
     * Residues shall be captured as described in 5.3.6.8.3.
     */
    residueSite?: string | undefined;
    _residueSite?: Element | undefined;
}
/**
 * 5.3.6.8.1 Sugar ID (Mandatory).
 */
export interface SubstanceNucleicAcidSubunitSugar extends BackboneElement {
    /**
     * The Substance ID of the sugar or sugar-like component that make up the nucleotide.
     */
    identifier?: Identifier | undefined;
    /**
     * The name of the sugar or sugar-like component that make up the nucleotide.
     */
    name?: string | undefined;
    _name?: Element | undefined;
    /**
     * The residues that contain a given sugar will be captured. The order of given residues will be captured in the 5‘-3‘direction consistent with the base sequences listed above.
     */
    residueSite?: string | undefined;
    _residueSite?: Element | undefined;
}
/**
 * Subunits are listed in order of decreasing length; sequences of the same length will be ordered by molecular weight; subunits that have identical sequences will be repeated multiple times.
 */
export interface SubstanceNucleicAcidSubunit extends BackboneElement {
    /**
     * The nucleotide present at the 5’ terminal shall be specified based on a controlled vocabulary. Since the sequence is represented from the 5' to the 3' end, the 5’ prime nucleotide is the letter at the first position in the sequence. A separate representation would be redundant.
     */
    fivePrime?: CodeableConcept | undefined;
    /**
     * The length of the sequence shall be captured.
     */
    length?: number | undefined;
    /**
     * The linkages between sugar residues will also be captured.
     */
    linkage?: SubstanceNucleicAcidSubunitLinkage[] | undefined;
    /**
     * Actual nucleotide sequence notation from 5' to 3' end using standard single letter codes. In addition to the base sequence, sugar and type of phosphate or non-phosphate linkage should also be captured.
     */
    sequence?: string | undefined;
    _sequence?: Element | undefined;
    /**
     * (TBC).
     */
    sequenceAttachment?: Attachment | undefined;
    /**
     * Index of linear sequences of nucleic acids in order of decreasing length. Sequences of the same length will be ordered by molecular weight. Subunits that have identical sequences will be repeated and have sequential subscripts.
     */
    subunit?: number | undefined;
    /**
     * 5.3.6.8.1 Sugar ID (Mandatory).
     */
    sugar?: SubstanceNucleicAcidSubunitSugar[] | undefined;
    /**
     * The nucleotide present at the 3’ terminal shall be specified based on a controlled vocabulary. Since the sequence is represented from the 5' to the 3' end, the 5’ prime nucleotide is the letter at the last position in the sequence. A separate representation would be redundant.
     */
    threePrime?: CodeableConcept | undefined;
}
/**
 * Nucleic acids are defined by three distinct elements: the base, sugar and linkage. Individual substance/moiety IDs will be created for each of these elements. The nucleotide sequence will be always entered in the 5’-3’ direction.
 */
export interface SubstanceNucleicAcid extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'SubstanceNucleicAcid';
    /**
     * The area of hybridisation shall be described if applicable for double stranded RNA or DNA. The number associated with the subunit followed by the number associated to the residue shall be specified in increasing order. The underscore “” shall be used as separator as follows: “Subunitnumber Residue”.
     */
    areaOfHybridisation?: string | undefined;
    _areaOfHybridisation?: Element | undefined;
    /**
     * The number of linear sequences of nucleotides linked through phosphodiester bonds shall be described. Subunits would be strands of nucleic acids that are tightly associated typically through Watson-Crick base pairing. NOTE: If not specified in the reference source, the assumption is that there is 1 subunit.
     */
    numberOfSubunits?: number | undefined;
    /**
     * (TBC).
     */
    oligoNucleotideType?: CodeableConcept | undefined;
    /**
     * The type of the sequence shall be specified based on a controlled vocabulary.
     */
    sequenceType?: CodeableConcept | undefined;
    /**
     * Subunits are listed in order of decreasing length; sequences of the same length will be ordered by molecular weight; subunits that have identical sequences will be repeated multiple times.
     */
    subunit?: SubstanceNucleicAcidSubunit[] | undefined;
}
/**
 * The starting materials - monomer(s) used in the synthesis of the polymer.
 */
export interface SubstancePolymerMonomerSetStartingMaterial extends BackboneElement {
    /**
     * A percentage.
     */
    amount?: Quantity | undefined;
    /**
     * Substance high level category, e.g. chemical substance.
     */
    category?: CodeableConcept | undefined;
    /**
     * The type of substance for this starting material.
     */
    code?: CodeableConcept | undefined;
    /**
     * Used to specify whether the attribute described is a defining element for the unique identification of the polymer.
     */
    isDefining?: boolean | undefined;
    _isDefining?: Element | undefined;
}
/**
 * Todo.
 */
export interface SubstancePolymerMonomerSet extends BackboneElement {
    /**
     * Captures the type of ratio to the entire polymer, e.g. Monomer/Polymer ratio, SRU/Polymer Ratio.
     */
    ratioType?: CodeableConcept | undefined;
    /**
     * The starting materials - monomer(s) used in the synthesis of the polymer.
     */
    startingMaterial?: SubstancePolymerMonomerSetStartingMaterial[] | undefined;
}
/**
 * Applies to homopolymer and block co-polymers where the degree of polymerisation within a block can be described.
 */
export interface SubstancePolymerRepeatRepeatUnitDegreeOfPolymerisation extends BackboneElement {
    /**
     * An average amount of polymerisation.
     */
    average?: number | undefined;
    /**
     * A high expected limit of the amount.
     */
    high?: number | undefined;
    /**
     * A low expected limit of the amount.
     */
    low?: number | undefined;
    /**
     * The type of the degree of polymerisation shall be described, e.g. SRU/Polymer Ratio.
     */
    type?: CodeableConcept | undefined;
}
/**
 * A graphical structure for this SRU.
 */
export interface SubstancePolymerRepeatRepeatUnitStructuralRepresentation extends BackboneElement {
    /**
     * An attached file with the structural representation.
     */
    attachment?: Attachment | undefined;
    /**
     * The format of the representation e.g. InChI, SMILES, MOLFILE, CDX, SDF, PDB, mmCIF.
     */
    format?: CodeableConcept | undefined;
    /**
     * The structural representation as text string in a standard format e.g. InChI, SMILES, MOLFILE, CDX, SDF, PDB, mmCIF.
     */
    representation?: string | undefined;
    _representation?: Element | undefined;
    /**
     * The type of structure (e.g. Full, Partial, Representative).
     */
    type?: CodeableConcept | undefined;
}
/**
 * An SRU - Structural Repeat Unit.
 */
export interface SubstancePolymerRepeatRepeatUnit extends BackboneElement {
    /**
     * Number of repeats of this unit.
     */
    amount?: number | undefined;
    /**
     * Applies to homopolymer and block co-polymers where the degree of polymerisation within a block can be described.
     */
    degreeOfPolymerisation?: SubstancePolymerRepeatRepeatUnitDegreeOfPolymerisation[] | undefined;
    /**
     * The orientation of the polymerisation, e.g. head-tail, head-head, random.
     */
    orientation?: CodeableConcept | undefined;
    /**
     * A graphical structure for this SRU.
     */
    structuralRepresentation?: SubstancePolymerRepeatRepeatUnitStructuralRepresentation[] | undefined;
    /**
     * Structural repeat units are essential elements for defining polymers.
     */
    unit?: string | undefined;
    _unit?: Element | undefined;
}
/**
 * Specifies and quantifies the repeated units and their configuration.
 */
export interface SubstancePolymerRepeat extends BackboneElement {
    /**
     * A representation of an (average) molecular formula from a polymer.
     */
    averageMolecularFormula?: string | undefined;
    _averageMolecularFormula?: Element | undefined;
    /**
     * An SRU - Structural Repeat Unit.
     */
    repeatUnit?: SubstancePolymerRepeatRepeatUnit[] | undefined;
    /**
     * How the quantitative amount of Structural Repeat Units is captured (e.g. Exact, Numeric, Average).
     */
    repeatUnitAmountType?: CodeableConcept | undefined;
}
/**
 * Properties of a substance specific to it being a polymer.
 */
export interface SubstancePolymer extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'SubstancePolymer';
    /**
     * Overall type of the polymer.
     */
    class?: CodeableConcept | undefined;
    /**
     * Descrtibes the copolymer sequence type (polymer connectivity).
     */
    copolymerConnectivity?: CodeableConcept[] | undefined;
    /**
     * Polymer geometry, e.g. linear, branched, cross-linked, network or dendritic.
     */
    geometry?: CodeableConcept | undefined;
    /**
     * A business idenfier for this polymer, but typically this is handled by a SubstanceDefinition identifier.
     */
    identifier?: Identifier | undefined;
    /**
     * Todo - this is intended to connect to a repeating full modification structure, also used by Protein and Nucleic Acid . String is just a placeholder.
     */
    modification?: string | undefined;
    _modification?: Element | undefined;
    /**
     * Todo.
     */
    monomerSet?: SubstancePolymerMonomerSet[] | undefined;
    /**
     * Specifies and quantifies the repeated units and their configuration.
     */
    repeat?: SubstancePolymerRepeat[] | undefined;
}
/**
 * This subclause refers to the description of each subunit constituting the SubstanceProtein. A subunit is a linear sequence of amino acids linked through peptide bonds. The Subunit information shall be provided when the finished SubstanceProtein is a complex of multiple sequences; subunits are not used to delineate domains within a single sequence. Subunits are listed in order of decreasing length; sequences of the same length will be ordered by decreasing molecular weight; subunits that have identical sequences will be repeated multiple times.
 */
export interface SubstanceProteinSubunit extends BackboneElement {
    /**
     * The modification at the C-terminal shall be specified.
     */
    cTerminalModification?: string | undefined;
    _cTerminalModification?: Element | undefined;
    /**
     * Unique identifier for molecular fragment modification based on the ISO 11238 Substance ID.
     */
    cTerminalModificationId?: Identifier | undefined;
    /**
     * Length of linear sequences of amino acids contained in the subunit.
     */
    length?: number | undefined;
    /**
     * The name of the fragment modified at the N-terminal of the SubstanceProtein shall be specified.
     */
    nTerminalModification?: string | undefined;
    _nTerminalModification?: Element | undefined;
    /**
     * Unique identifier for molecular fragment modification based on the ISO 11238 Substance ID.
     */
    nTerminalModificationId?: Identifier | undefined;
    /**
     * The sequence information shall be provided enumerating the amino acids from N- to C-terminal end using standard single-letter amino acid codes. Uppercase shall be used for L-amino acids and lowercase for D-amino acids. Transcribed SubstanceProteins will always be described using the translated sequence; for synthetic peptide containing amino acids that are not represented with a single letter code an X should be used within the sequence. The modified amino acids will be distinguished by their position in the sequence.
     */
    sequence?: string | undefined;
    _sequence?: Element | undefined;
    /**
     * The sequence information shall be provided enumerating the amino acids from N- to C-terminal end using standard single-letter amino acid codes. Uppercase shall be used for L-amino acids and lowercase for D-amino acids. Transcribed SubstanceProteins will always be described using the translated sequence; for synthetic peptide containing amino acids that are not represented with a single letter code an X should be used within the sequence. The modified amino acids will be distinguished by their position in the sequence.
     */
    sequenceAttachment?: Attachment | undefined;
    /**
     * Index of primary sequences of amino acids linked through peptide bonds in order of decreasing length. Sequences of the same length will be ordered by molecular weight. Subunits that have identical sequences will be repeated and have sequential subscripts.
     */
    subunit?: number | undefined;
}
/**
 * A SubstanceProtein is defined as a single unit of a linear amino acid sequence, or a combination of subunits that are either covalently linked or have a defined invariant stoichiometric relationship. This includes all synthetic, recombinant and purified SubstanceProteins of defined sequence, whether the use is therapeutic or prophylactic. This set of elements will be used to describe albumins, coagulation factors, cytokines, growth factors, peptide/SubstanceProtein hormones, enzymes, toxins, toxoids, recombinant vaccines, and immunomodulators.
 */
export interface SubstanceProtein extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'SubstanceProtein';
    /**
     * The disulphide bond between two cysteine residues either on the same subunit or on two different subunits shall be described. The position of the disulfide bonds in the SubstanceProtein shall be listed in increasing order of subunit number and position within subunit followed by the abbreviation of the amino acids involved. The disulfide linkage positions shall actually contain the amino acid Cysteine at the respective positions.
     */
    disulfideLinkage?: string[] | undefined;
    _disulfideLinkage?: Element[] | undefined;
    /**
     * Number of linear sequences of amino acids linked through peptide bonds. The number of subunits constituting the SubstanceProtein shall be described. It is possible that the number of subunits can be variable.
     */
    numberOfSubunits?: number | undefined;
    /**
     * The SubstanceProtein descriptive elements will only be used when a complete or partial amino acid sequence is available or derivable from a nucleic acid sequence.
     */
    sequenceType?: CodeableConcept | undefined;
    /**
     * This subclause refers to the description of each subunit constituting the SubstanceProtein. A subunit is a linear sequence of amino acids linked through peptide bonds. The Subunit information shall be provided when the finished SubstanceProtein is a complex of multiple sequences; subunits are not used to delineate domains within a single sequence. Subunits are listed in order of decreasing length; sequences of the same length will be ordered by decreasing molecular weight; subunits that have identical sequences will be repeated multiple times.
     */
    subunit?: SubstanceProteinSubunit[] | undefined;
}
/**
 * Todo.
 */
export interface SubstanceReferenceInformationGene extends BackboneElement {
    /**
     * Todo.
     */
    gene?: CodeableConcept | undefined;
    /**
     * Todo.
     */
    geneSequenceOrigin?: CodeableConcept | undefined;
    /**
     * Todo.
     */
    source?: Reference[] | undefined;
}
/**
 * Todo.
 */
export interface SubstanceReferenceInformationGeneElement extends BackboneElement {
    /**
     * Todo.
     */
    element?: Identifier | undefined;
    /**
     * Todo.
     */
    source?: Reference[] | undefined;
    /**
     * Todo.
     */
    type?: CodeableConcept | undefined;
}
/**
 * Todo.
 */
export interface SubstanceReferenceInformationTarget extends BackboneElement {
    /**
     * Todo.
     */
    amountQuantity?: Quantity | undefined;
    /**
     * Todo.
     */
    amountRange?: Range | undefined;
    /**
     * Todo.
     */
    amountString?: string | undefined;
    _amountString?: Element | undefined;
    /**
     * Todo.
     */
    amountType?: CodeableConcept | undefined;
    /**
     * Todo.
     */
    interaction?: CodeableConcept | undefined;
    /**
     * Todo.
     */
    organism?: CodeableConcept | undefined;
    /**
     * Todo.
     */
    organismType?: CodeableConcept | undefined;
    /**
     * Todo.
     */
    source?: Reference[] | undefined;
    /**
     * Todo.
     */
    target?: Identifier | undefined;
    /**
     * Todo.
     */
    type?: CodeableConcept | undefined;
}
/**
 * Todo.
 */
export interface SubstanceReferenceInformation extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'SubstanceReferenceInformation';
    /**
     * Todo.
     */
    comment?: string | undefined;
    _comment?: Element | undefined;
    /**
     * Todo.
     */
    gene?: SubstanceReferenceInformationGene[] | undefined;
    /**
     * Todo.
     */
    geneElement?: SubstanceReferenceInformationGeneElement[] | undefined;
    /**
     * Todo.
     */
    target?: SubstanceReferenceInformationTarget[] | undefined;
}
/**
 * Many complex materials are fractions of parts of plants, animals, or minerals. Fraction elements are often necessary to define both Substances and Specified Group 1 Substances. For substances derived from Plants, fraction information will be captured at the Substance information level ( . Oils, Juices and Exudates). Additional information for Extracts, such as extraction solvent composition, will be captured at the Specified Substance Group 1 information level. For plasma-derived products fraction information will be captured at the Substance and the Specified Substance Group 1 levels.
 */
export interface SubstanceSourceMaterialFractionDescription extends BackboneElement {
    /**
     * This element is capturing information about the fraction of a plant part, or human plasma for fractionation.
     */
    fraction?: string | undefined;
    _fraction?: Element | undefined;
    /**
     * The specific type of the material constituting the component. For Herbal preparations the particulars of the extracts (liquid/dry) is described in Specified Substance Group 1.
     */
    materialType?: CodeableConcept | undefined;
}
/**
 * 4.9.13.6.1 Author type (Conditional).
 */
export interface SubstanceSourceMaterialOrganismAuthor extends BackboneElement {
    /**
     * The author of an organism species shall be specified. The author year of an organism shall also be specified when applicable; refers to the year in which the first author(s) published the infraspecific plant/animal name (of any rank).
     */
    authorDescription?: string | undefined;
    _authorDescription?: Element | undefined;
    /**
     * The type of author of an organism species shall be specified. The parenthetical author of an organism species refers to the first author who published the plant/animal name (of any rank). The primary author of an organism species refers to the first author(s), who validly published the plant/animal name.
     */
    authorType?: CodeableConcept | undefined;
}
/**
 * 4.9.13.8.1 Hybrid species maternal organism ID (Optional).
 */
export interface SubstanceSourceMaterialOrganismHybrid extends BackboneElement {
    /**
     * The hybrid type of an organism shall be specified.
     */
    hybridType?: CodeableConcept | undefined;
    /**
     * The identifier of the maternal species constituting the hybrid organism shall be specified based on a controlled vocabulary. For plants, the parents aren’t always known, and it is unlikely that it will be known which is maternal and which is paternal.
     */
    maternalOrganismId?: string | undefined;
    _maternalOrganismId?: Element | undefined;
    /**
     * The name of the maternal species constituting the hybrid organism shall be specified. For plants, the parents aren’t always known, and it is unlikely that it will be known which is maternal and which is paternal.
     */
    maternalOrganismName?: string | undefined;
    _maternalOrganismName?: Element | undefined;
    /**
     * The identifier of the paternal species constituting the hybrid organism shall be specified based on a controlled vocabulary.
     */
    paternalOrganismId?: string | undefined;
    _paternalOrganismId?: Element | undefined;
    /**
     * The name of the paternal species constituting the hybrid organism shall be specified.
     */
    paternalOrganismName?: string | undefined;
    _paternalOrganismName?: Element | undefined;
}
/**
 * 4.9.13.7.1 Kingdom (Conditional).
 */
export interface SubstanceSourceMaterialOrganismOrganismGeneral extends BackboneElement {
    /**
     * The class of an organism shall be specified.
     */
    class?: CodeableConcept | undefined;
    /**
     * The kingdom of an organism shall be specified.
     */
    kingdom?: CodeableConcept | undefined;
    /**
     * The order of an organism shall be specified,.
     */
    order?: CodeableConcept | undefined;
    /**
     * The phylum of an organism shall be specified.
     */
    phylum?: CodeableConcept | undefined;
}
/**
 * This subclause describes the organism which the substance is derived from. For vaccines, the parent organism shall be specified based on these subclause elements. As an example, full taxonomy will be described for the Substance Name: ., Leaf.
 */
export interface SubstanceSourceMaterialOrganism extends BackboneElement {
    /**
     * 4.9.13.6.1 Author type (Conditional).
     */
    author?: SubstanceSourceMaterialOrganismAuthor[] | undefined;
    /**
     * The family of an organism shall be specified.
     */
    family?: CodeableConcept | undefined;
    /**
     * The genus of an organism shall be specified; refers to the Latin epithet of the genus element of the plant/animal scientific name; it is present in names for genera, species and infraspecies.
     */
    genus?: CodeableConcept | undefined;
    /**
     * 4.9.13.8.1 Hybrid species maternal organism ID (Optional).
     */
    hybrid?: SubstanceSourceMaterialOrganismHybrid | undefined;
    /**
     * The intraspecific description of an organism shall be specified based on a controlled vocabulary. For Influenza Vaccine, the intraspecific description shall contain the syntax of the antigen in line with the WHO convention.
     */
    intraspecificDescription?: string | undefined;
    _intraspecificDescription?: Element | undefined;
    /**
     * The Intraspecific type of an organism shall be specified.
     */
    intraspecificType?: CodeableConcept | undefined;
    /**
     * 4.9.13.7.1 Kingdom (Conditional).
     */
    organismGeneral?: SubstanceSourceMaterialOrganismOrganismGeneral | undefined;
    /**
     * The species of an organism shall be specified; refers to the Latin epithet of the species of the plant/animal; it is present in names for species and infraspecies.
     */
    species?: CodeableConcept | undefined;
}
/**
 * To do.
 */
export interface SubstanceSourceMaterialPartDescription extends BackboneElement {
    /**
     * Entity of anatomical origin of source material within an organism.
     */
    part?: CodeableConcept | undefined;
    /**
     * The detailed anatomic location when the part can be extracted from different anatomical locations of the organism. Multiple alternative locations may apply.
     */
    partLocation?: CodeableConcept | undefined;
}
/**
 * Source material shall capture information on the taxonomic and anatomical origins as well as the fraction of a material that can result in or can be modified to form a substance. This set of data elements shall be used to define polymer substances isolated from biological matrices. Taxonomic and anatomical origins shall be described using a controlled vocabulary as required. This information is captured for naturally derived polymers ( . starch) and structurally diverse substances. For Organisms belonging to the Kingdom Plantae the Substance level defines the fresh material of a single species or infraspecies, the Herbal Drug and the Herbal preparation. For Herbal preparations, the fraction information will be captured at the Substance information level and additional information for herbal extracts will be captured at the Specified Substance Group 1 information level. See for further explanation the Substance Class: Structurally Diverse and the herbal annex.
 */
export interface SubstanceSourceMaterial extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'SubstanceSourceMaterial';
    /**
     * The country where the plant material is harvested or the countries where the plasma is sourced from as laid down in accordance with the Plasma Master File. For “Plasma-derived substances” the attribute country of origin provides information about the countries used for the manufacturing of the Cryopoor plama or Crioprecipitate.
     */
    countryOfOrigin?: CodeableConcept[] | undefined;
    /**
     * Stage of life for animals, plants, insects and microorganisms. This information shall be provided only when the substance is significantly different in these stages (e.g. foetal bovine serum).
     */
    developmentStage?: CodeableConcept | undefined;
    /**
     * Many complex materials are fractions of parts of plants, animals, or minerals. Fraction elements are often necessary to define both Substances and Specified Group 1 Substances. For substances derived from Plants, fraction information will be captured at the Substance information level ( . Oils, Juices and Exudates). Additional information for Extracts, such as extraction solvent composition, will be captured at the Specified Substance Group 1 information level. For plasma-derived products fraction information will be captured at the Substance and the Specified Substance Group 1 levels.
     */
    fractionDescription?: SubstanceSourceMaterialFractionDescription[] | undefined;
    /**
     * The place/region where the plant is harvested or the places/regions where the animal source material has its habitat.
     */
    geographicalLocation?: string[] | undefined;
    _geographicalLocation?: Element[] | undefined;
    /**
     * This subclause describes the organism which the substance is derived from. For vaccines, the parent organism shall be specified based on these subclause elements. As an example, full taxonomy will be described for the Substance Name: ., Leaf.
     */
    organism?: SubstanceSourceMaterialOrganism | undefined;
    /**
     * The unique identifier associated with the source material parent organism shall be specified.
     */
    organismId?: Identifier | undefined;
    /**
     * The organism accepted Scientific name shall be provided based on the organism taxonomy.
     */
    organismName?: string | undefined;
    _organismName?: Element | undefined;
    /**
     * The parent of the herbal drug Ginkgo biloba, Leaf is the substance ID of the substance (fresh) of Ginkgo biloba L. or Ginkgo biloba L. (Whole plant).
     */
    parentSubstanceId?: Identifier[] | undefined;
    /**
     * The parent substance of the Herbal Drug, or Herbal preparation.
     */
    parentSubstanceName?: string[] | undefined;
    _parentSubstanceName?: Element[] | undefined;
    /**
     * To do.
     */
    partDescription?: SubstanceSourceMaterialPartDescription[] | undefined;
    /**
     * General high level classification of the source material specific to the origin of the material.
     */
    sourceMaterialClass?: CodeableConcept | undefined;
    /**
     * The state of the source material when extracted.
     */
    sourceMaterialState?: CodeableConcept | undefined;
    /**
     * The type of the source material shall be specified based on a controlled vocabulary. For vaccines, this subclause refers to the class of infectious agent.
     */
    sourceMaterialType?: CodeableConcept | undefined;
}
/**
 * The item that is being delivered or has been supplied.
 */
export interface SupplyDeliverySuppliedItem extends BackboneElement {
    /**
     * Identifies the medication, substance, device or biologically derived product being supplied. This is either a link to a resource representing the details of the item or a code that identifies the item from a known list.
     */
    itemCodeableConcept?: CodeableConcept | undefined;
    /**
     * Identifies the medication, substance, device or biologically derived product being supplied. This is either a link to a resource representing the details of the item or a code that identifies the item from a known list.
     */
    itemReference?: Reference | undefined;
    /**
     * The amount of the item that has been supplied.  Unit of measure may be included.
     */
    quantity?: Quantity | undefined;
}
/**
 * Record of delivery of what is supplied.
 */
export interface SupplyDelivery extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'SupplyDelivery';
    /**
     * A plan, proposal or order that is fulfilled in whole or in part by this event.
     */
    basedOn?: Reference[] | undefined;
    /**
     * Identification of the facility/location where the delivery was shipped to.
     */
    destination?: Reference | undefined;
    /**
     * This identifier is typically assigned by the supplier, and may be used to reference the delivery when exchanging information about it with other systems.
     */
    identifier?: Identifier[] | undefined;
    /**
     * [The list of types may be constrained as appropriate for the type of event].
     */
    occurrenceDateTime?: string | undefined;
    _occurrenceDateTime?: Element | undefined;
    /**
     * [The list of types may be constrained as appropriate for the type of event].
     */
    occurrencePeriod?: Period | undefined;
    /**
     * [The list of types may be constrained as appropriate for the type of event].
     */
    occurrenceTiming?: Timing | undefined;
    /**
     * Not to be used to link an event to an Encounter - use Event.context for that.
     * [The allowed reference resources may be adjusted as appropriate for the event resource].
     */
    partOf?: Reference[] | undefined;
    /**
     * A link to a resource representing the person whom the delivered item is for.
     */
    patient?: Reference | undefined;
    /**
     * Identifies the person or organization that received the delivery.
     */
    receiver?: Reference[] | undefined;
    /**
     * This element is labeled as a modifier because the status contains codes that mark the resource as not currently valid.
     */
    status?: ('in-progress' | 'completed' | 'abandoned' | 'entered-in-error') | undefined;
    _status?: Element | undefined;
    /**
     * The item that is being delivered or has been supplied.
     */
    suppliedItem?: SupplyDeliverySuppliedItem[] | undefined;
    /**
     * The individual or organization responsible for supplying the delivery.
     */
    supplier?: Reference | undefined;
    /**
     * Indicates the type of supply being provided.  Examples include: Medication, Device, Biologically Derived Product.
     */
    type?: CodeableConcept | undefined;
}
/**
 * Specific parameters for the ordered item.  For example, the size of the indicated item.
 */
export interface SupplyRequestParameter extends BackboneElement {
    /**
     * A code or string that identifies the device detail being asserted.
     */
    code?: CodeableConcept | undefined;
    /**
     * Range means device should have a value that falls somewhere within the specified range.
     */
    valueCodeableConcept?: CodeableConcept | undefined;
    /**
     * Range means device should have a value that falls somewhere within the specified range.
     */
    valueQuantity?: Quantity | undefined;
    /**
     * Range means device should have a value that falls somewhere within the specified range.
     */
    valueRange?: Range | undefined;
    /**
     * Range means device should have a value that falls somewhere within the specified range.
     */
    valueBoolean?: boolean | undefined;
    _valueBoolean?: Element | undefined;
}
/**
 * A record of a request for a medication, substance or device used in the healthcare setting.
 */
export interface SupplyRequest extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'SupplyRequest';
    /**
     * When the request was made.
     */
    authoredOn?: string | undefined;
    _authoredOn?: Element | undefined;
    /**
     * Plan/proposal/order fulfilled by this request.
     */
    basedOn?: Reference[] | undefined;
    /**
     * Category of supply, e.g.  central, non-stock, etc. This is used to support work flows associated with the supply process.
     */
    category?: CodeableConcept | undefined;
    /**
     * Where the supply is expected to come from.
     */
    deliverFrom?: Reference | undefined;
    /**
     * Where the supply is destined to go.
     */
    deliverTo?: Reference | undefined;
    /**
     * The identifier.type element is used to distinguish between the identifiers assigned by the requester/placer and the performer/filler.
     */
    identifier?: Identifier[] | undefined;
    /**
     * Note that there's a difference between a prescription - an instruction to take a medication, along with a (sometimes) implicit supply, and an explicit request to supply, with no explicit instructions.
     */
    item: CodeableReference;
    /**
     * When the request should be fulfilled.
     */
    occurrenceDateTime?: string | undefined;
    _occurrenceDateTime?: Element | undefined;
    /**
     * When the request should be fulfilled.
     */
    occurrencePeriod?: Period | undefined;
    /**
     * When the request should be fulfilled.
     */
    occurrenceTiming?: Timing | undefined;
    /**
     * Specific parameters for the ordered item.  For example, the size of the indicated item.
     */
    parameter?: SupplyRequestParameter[] | undefined;
    /**
     * Indicates how quickly this SupplyRequest should be addressed with respect to other requests.
     */
    priority?: ('routine' | 'urgent' | 'asap' | 'stat') | undefined;
    _priority?: Element | undefined;
    /**
     * The amount that is being ordered of the indicated item.
     */
    quantity: Quantity;
    /**
     * The reason why the supply item was requested.
     */
    reason?: CodeableReference[] | undefined;
    /**
     * The device, practitioner, etc. who initiated the request.
     */
    requester?: Reference | undefined;
    /**
     * Status of the supply request.
     */
    status?:
        | ('draft' | 'active' | 'suspended' | 'cancelled' | 'completed' | 'entered-in-error' | 'unknown')
        | undefined;
    _status?: Element | undefined;
    /**
     * Who is intended to fulfill the request.
     */
    supplier?: Reference[] | undefined;
}
/**
 * Task.restriction can only be present if the Task is seeking fulfillment of another Request resource, and the restriction identifies what subset of the authorization conveyed by the request is supposed to be fulfilled by this Task. A possible example could be a standing order (the request) covering a significant time period and/or individuals, while the Task seeks fulfillment for only a subset of that time-period and a single individual.
 */
export interface TaskRestriction extends BackboneElement {
    /**
     * This is distinct from Task.executionPeriod. ExecutionPeriod indicates when the task needs to be initiated, while Task.restriction.period specifies the subset of the overall authorization that this period covers. For example, a MedicationRequest with an overall effective period of 1 year might have a Task whose restriction.period is 2 months (i.e. satisfy 2 months of medication therapy), while the execution period might be 'between now and 5 days from now' - i.e. If you say yes to this, then you're agreeing to supply medication for that 2 month period within the next 5 days.
     * Note that period.high is the due date representing the time by which the task should be completed.
     */
    period?: Period | undefined;
    /**
     * For requests that are targeted to more than one potential recipient/target, to identify who is fulfillment is sought for.
     */
    recipient?: Reference[] | undefined;
    /**
     * Indicates the number of times the requested action should occur.
     */
    repetitions?: number | undefined;
}
/**
 * Additional information that may be needed in the execution of the task.
 */
export interface TaskInput extends BackboneElement {
    /**
     * If referencing a BPMN workflow or Protocol, the "system" is the URL for the workflow definition and the code is the "name" of the required input.
     */
    type: CodeableConcept;
    /**
     * The value of the input parameter as a basic type.
     */
    valueBase64Binary?: string | undefined;
    _valueBase64Binary?: Element | undefined;
    /**
     * The value of the input parameter as a basic type.
     */
    valueBoolean?: boolean | undefined;
    _valueBoolean?: Element | undefined;
    /**
     * The value of the input parameter as a basic type.
     */
    valueCanonical?: string | undefined;
    _valueCanonical?: Element | undefined;
    /**
     * The value of the input parameter as a basic type.
     */
    valueCode?: string | undefined;
    _valueCode?: Element | undefined;
    /**
     * The value of the input parameter as a basic type.
     */
    valueDate?: string | undefined;
    _valueDate?: Element | undefined;
    /**
     * The value of the input parameter as a basic type.
     */
    valueDateTime?: string | undefined;
    _valueDateTime?: Element | undefined;
    /**
     * The value of the input parameter as a basic type.
     */
    valueDecimal?: number | undefined;
    /**
     * The value of the input parameter as a basic type.
     */
    valueId?: string | undefined;
    _valueId?: Element | undefined;
    /**
     * The value of the input parameter as a basic type.
     */
    valueInstant?: string | undefined;
    _valueInstant?: Element | undefined;
    /**
     * The value of the input parameter as a basic type.
     */
    valueInteger?: number | undefined;
    /**
     * The value of the input parameter as a basic type.
     */
    valueInteger64?: string | undefined;
    _valueInteger64?: Element | undefined;
    /**
     * The value of the input parameter as a basic type.
     */
    valueMarkdown?: string | undefined;
    _valueMarkdown?: Element | undefined;
    /**
     * The value of the input parameter as a basic type.
     */
    valueOid?: string | undefined;
    _valueOid?: Element | undefined;
    /**
     * The value of the input parameter as a basic type.
     */
    valuePositiveInt?: number | undefined;
    /**
     * The value of the input parameter as a basic type.
     */
    valueString?: string | undefined;
    _valueString?: Element | undefined;
    /**
     * The value of the input parameter as a basic type.
     */
    valueTime?: string | undefined;
    _valueTime?: Element | undefined;
    /**
     * The value of the input parameter as a basic type.
     */
    valueUnsignedInt?: number | undefined;
    /**
     * The value of the input parameter as a basic type.
     */
    valueUri?: string | undefined;
    _valueUri?: Element | undefined;
    /**
     * The value of the input parameter as a basic type.
     */
    valueUrl?: string | undefined;
    _valueUrl?: Element | undefined;
    /**
     * The value of the input parameter as a basic type.
     */
    valueUuid?: string | undefined;
    _valueUuid?: Element | undefined;
    /**
     * The value of the input parameter as a basic type.
     */
    valueAddress?: Address | undefined;
    /**
     * The value of the input parameter as a basic type.
     */
    valueAge?: Age | undefined;
    /**
     * The value of the input parameter as a basic type.
     */
    valueAnnotation?: Annotation | undefined;
    /**
     * The value of the input parameter as a basic type.
     */
    valueAttachment?: Attachment | undefined;
    /**
     * The value of the input parameter as a basic type.
     */
    valueCodeableConcept?: CodeableConcept | undefined;
    /**
     * The value of the input parameter as a basic type.
     */
    valueCodeableReference?: CodeableReference | undefined;
    /**
     * The value of the input parameter as a basic type.
     */
    valueCoding?: Coding | undefined;
    /**
     * The value of the input parameter as a basic type.
     */
    valueContactPoint?: ContactPoint | undefined;
    /**
     * The value of the input parameter as a basic type.
     */
    valueCount?: Count | undefined;
    /**
     * The value of the input parameter as a basic type.
     */
    valueDistance?: Distance | undefined;
    /**
     * The value of the input parameter as a basic type.
     */
    valueDuration?: Duration | undefined;
    /**
     * The value of the input parameter as a basic type.
     */
    valueHumanName?: HumanName | undefined;
    /**
     * The value of the input parameter as a basic type.
     */
    valueIdentifier?: Identifier | undefined;
    /**
     * The value of the input parameter as a basic type.
     */
    valueMoney?: Money | undefined;
    /**
     * The value of the input parameter as a basic type.
     */
    valuePeriod?: Period | undefined;
    /**
     * The value of the input parameter as a basic type.
     */
    valueQuantity?: Quantity | undefined;
    /**
     * The value of the input parameter as a basic type.
     */
    valueRange?: Range | undefined;
    /**
     * The value of the input parameter as a basic type.
     */
    valueRatio?: Ratio | undefined;
    /**
     * The value of the input parameter as a basic type.
     */
    valueRatioRange?: RatioRange | undefined;
    /**
     * The value of the input parameter as a basic type.
     */
    valueReference?: Reference | undefined;
    /**
     * The value of the input parameter as a basic type.
     */
    valueSampledData?: SampledData | undefined;
    /**
     * The value of the input parameter as a basic type.
     */
    valueSignature?: Signature | undefined;
    /**
     * The value of the input parameter as a basic type.
     */
    valueTiming?: Timing | undefined;
    /**
     * The value of the input parameter as a basic type.
     */
    valueContactDetail?: ContactDetail | undefined;
    /**
     * The value of the input parameter as a basic type.
     */
    valueDataRequirement?: DataRequirement | undefined;
    /**
     * The value of the input parameter as a basic type.
     */
    valueExpression?: Expression | undefined;
    /**
     * The value of the input parameter as a basic type.
     */
    valueParameterDefinition?: ParameterDefinition | undefined;
    /**
     * The value of the input parameter as a basic type.
     */
    valueRelatedArtifact?: RelatedArtifact | undefined;
    /**
     * The value of the input parameter as a basic type.
     */
    valueTriggerDefinition?: TriggerDefinition | undefined;
    /**
     * The value of the input parameter as a basic type.
     */
    valueUsageContext?: UsageContext | undefined;
    /**
     * The value of the input parameter as a basic type.
     */
    valueAvailability?: Availability | undefined;
    /**
     * The value of the input parameter as a basic type.
     */
    valueExtendedContactDetail?: ExtendedContactDetail | undefined;
    /**
     * The value of the input parameter as a basic type.
     */
    valueDosage?: Dosage | undefined;
    /**
     * The value of the input parameter as a basic type.
     */
    valueMeta?: Meta | undefined;
}
/**
 * Outputs produced by the Task.
 */
export interface TaskOutput extends BackboneElement {
    /**
     * The name of the Output parameter.
     */
    type: CodeableConcept;
    /**
     * The value of the Output parameter as a basic type.
     */
    valueBase64Binary?: string | undefined;
    _valueBase64Binary?: Element | undefined;
    /**
     * The value of the Output parameter as a basic type.
     */
    valueBoolean?: boolean | undefined;
    _valueBoolean?: Element | undefined;
    /**
     * The value of the Output parameter as a basic type.
     */
    valueCanonical?: string | undefined;
    _valueCanonical?: Element | undefined;
    /**
     * The value of the Output parameter as a basic type.
     */
    valueCode?: string | undefined;
    _valueCode?: Element | undefined;
    /**
     * The value of the Output parameter as a basic type.
     */
    valueDate?: string | undefined;
    _valueDate?: Element | undefined;
    /**
     * The value of the Output parameter as a basic type.
     */
    valueDateTime?: string | undefined;
    _valueDateTime?: Element | undefined;
    /**
     * The value of the Output parameter as a basic type.
     */
    valueDecimal?: number | undefined;
    /**
     * The value of the Output parameter as a basic type.
     */
    valueId?: string | undefined;
    _valueId?: Element | undefined;
    /**
     * The value of the Output parameter as a basic type.
     */
    valueInstant?: string | undefined;
    _valueInstant?: Element | undefined;
    /**
     * The value of the Output parameter as a basic type.
     */
    valueInteger?: number | undefined;
    /**
     * The value of the Output parameter as a basic type.
     */
    valueInteger64?: string | undefined;
    _valueInteger64?: Element | undefined;
    /**
     * The value of the Output parameter as a basic type.
     */
    valueMarkdown?: string | undefined;
    _valueMarkdown?: Element | undefined;
    /**
     * The value of the Output parameter as a basic type.
     */
    valueOid?: string | undefined;
    _valueOid?: Element | undefined;
    /**
     * The value of the Output parameter as a basic type.
     */
    valuePositiveInt?: number | undefined;
    /**
     * The value of the Output parameter as a basic type.
     */
    valueString?: string | undefined;
    _valueString?: Element | undefined;
    /**
     * The value of the Output parameter as a basic type.
     */
    valueTime?: string | undefined;
    _valueTime?: Element | undefined;
    /**
     * The value of the Output parameter as a basic type.
     */
    valueUnsignedInt?: number | undefined;
    /**
     * The value of the Output parameter as a basic type.
     */
    valueUri?: string | undefined;
    _valueUri?: Element | undefined;
    /**
     * The value of the Output parameter as a basic type.
     */
    valueUrl?: string | undefined;
    _valueUrl?: Element | undefined;
    /**
     * The value of the Output parameter as a basic type.
     */
    valueUuid?: string | undefined;
    _valueUuid?: Element | undefined;
    /**
     * The value of the Output parameter as a basic type.
     */
    valueAddress?: Address | undefined;
    /**
     * The value of the Output parameter as a basic type.
     */
    valueAge?: Age | undefined;
    /**
     * The value of the Output parameter as a basic type.
     */
    valueAnnotation?: Annotation | undefined;
    /**
     * The value of the Output parameter as a basic type.
     */
    valueAttachment?: Attachment | undefined;
    /**
     * The value of the Output parameter as a basic type.
     */
    valueCodeableConcept?: CodeableConcept | undefined;
    /**
     * The value of the Output parameter as a basic type.
     */
    valueCodeableReference?: CodeableReference | undefined;
    /**
     * The value of the Output parameter as a basic type.
     */
    valueCoding?: Coding | undefined;
    /**
     * The value of the Output parameter as a basic type.
     */
    valueContactPoint?: ContactPoint | undefined;
    /**
     * The value of the Output parameter as a basic type.
     */
    valueCount?: Count | undefined;
    /**
     * The value of the Output parameter as a basic type.
     */
    valueDistance?: Distance | undefined;
    /**
     * The value of the Output parameter as a basic type.
     */
    valueDuration?: Duration | undefined;
    /**
     * The value of the Output parameter as a basic type.
     */
    valueHumanName?: HumanName | undefined;
    /**
     * The value of the Output parameter as a basic type.
     */
    valueIdentifier?: Identifier | undefined;
    /**
     * The value of the Output parameter as a basic type.
     */
    valueMoney?: Money | undefined;
    /**
     * The value of the Output parameter as a basic type.
     */
    valuePeriod?: Period | undefined;
    /**
     * The value of the Output parameter as a basic type.
     */
    valueQuantity?: Quantity | undefined;
    /**
     * The value of the Output parameter as a basic type.
     */
    valueRange?: Range | undefined;
    /**
     * The value of the Output parameter as a basic type.
     */
    valueRatio?: Ratio | undefined;
    /**
     * The value of the Output parameter as a basic type.
     */
    valueRatioRange?: RatioRange | undefined;
    /**
     * The value of the Output parameter as a basic type.
     */
    valueReference?: Reference | undefined;
    /**
     * The value of the Output parameter as a basic type.
     */
    valueSampledData?: SampledData | undefined;
    /**
     * The value of the Output parameter as a basic type.
     */
    valueSignature?: Signature | undefined;
    /**
     * The value of the Output parameter as a basic type.
     */
    valueTiming?: Timing | undefined;
    /**
     * The value of the Output parameter as a basic type.
     */
    valueContactDetail?: ContactDetail | undefined;
    /**
     * The value of the Output parameter as a basic type.
     */
    valueDataRequirement?: DataRequirement | undefined;
    /**
     * The value of the Output parameter as a basic type.
     */
    valueExpression?: Expression | undefined;
    /**
     * The value of the Output parameter as a basic type.
     */
    valueParameterDefinition?: ParameterDefinition | undefined;
    /**
     * The value of the Output parameter as a basic type.
     */
    valueRelatedArtifact?: RelatedArtifact | undefined;
    /**
     * The value of the Output parameter as a basic type.
     */
    valueTriggerDefinition?: TriggerDefinition | undefined;
    /**
     * The value of the Output parameter as a basic type.
     */
    valueUsageContext?: UsageContext | undefined;
    /**
     * The value of the Output parameter as a basic type.
     */
    valueAvailability?: Availability | undefined;
    /**
     * The value of the Output parameter as a basic type.
     */
    valueExtendedContactDetail?: ExtendedContactDetail | undefined;
    /**
     * The value of the Output parameter as a basic type.
     */
    valueDosage?: Dosage | undefined;
    /**
     * The value of the Output parameter as a basic type.
     */
    valueMeta?: Meta | undefined;
}
/**
 * A task to be performed.
 */
export interface Task extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'Task';
    /**
     * The date and time this task was created.
     */
    authoredOn?: string | undefined;
    _authoredOn?: Element | undefined;
    /**
     * Task.basedOn is never the same as Task.focus.  Task.basedOn will typically not be present for 'please fulfill' Tasks as a distinct authorization is rarely needed to request fulfillment.  If the Task is seeking fulfillment of an order, the order to be fulfilled is always communicated using `focus`, never basedOn.  However, authorization may be needed to perform other types of Task actions.  As an example of when both would be present, a Task seeking suspension of a prescription might have a Task.basedOn pointing to the ServiceRequest ordering surgery (which is the driver for suspending the MedicationRequest - which would be the Task.focus).
     */
    basedOn?: Reference[] | undefined;
    /**
     * Contains business-specific nuances of the business state.
     */
    businessStatus?: CodeableConcept | undefined;
    /**
     * The title (eg "My Tasks", "Outstanding Tasks for Patient X") should go into the code.
     */
    code?: CodeableConcept | undefined;
    /**
     * A free-text description of what is to be performed.
     */
    description?: string | undefined;
    _description?: Element | undefined;
    /**
     * The attributes provided with the Task qualify what is not to be done. For example, if a requestedPeriod is provided, the 'do not' request only applies within the specified time. If a requestedPerformer is specified then the 'do not' request only applies to performers of that type. Qualifiers include: code, subject, occurrence, requestedPerformer and performer.
     * In some cases, the Request.code may pre-coordinate prohibition into the requested action. E.g. 'NPO' (nothing by mouth), 'DNR' (do not recussitate). If this happens, doNotPerform SHALL NOT be set to true. I.e. The resource shall not have double negation. (E.g. 'Do not DNR').
     * doNotPerform should ONLY be used with Tasks that are tightly bounded in time or process phase.  E.g. 'Do not fulfill the midnight dose of medication X tonight due to the early morning scheduled procedure, where the nurse could reasonably check off 'Med X not given at midnight as instructed'.  Similarly, a decision support proposal that a patient should not be given a standard intake questionnaire (because the patient is cognitively impaired) would be marked as 'complete' or 'rejected' when the clinician preps the CarePlan or order set after reviewing the decision support results.  If there is a need to create a standing order to not do something that can't be satisfied by a single 'non-action', but rather an ongoing refusal to perform the function, MedicationRequest, ServiceRequest or some other form of authorization should be used.
     */
    doNotPerform?: boolean | undefined;
    _doNotPerform?: Element | undefined;
    /**
     * The healthcare event  (e.g. a patient and healthcare provider interaction) during which this task was created.
     */
    encounter?: Reference | undefined;
    /**
     * Identifies the time action was first taken against the task (start) and/or the time final action was taken against the task prior to marking it as completed (end).
     */
    executionPeriod?: Period | undefined;
    /**
     * If multiple resources need to be manipulated, use sub-tasks.  (This ensures that status can be tracked independently for each referenced resource.).
     */
    focus?: Reference | undefined;
    /**
     * The entity who benefits from the performance of the service specified in the task (e.g., the patient).
     */
    for?: Reference | undefined;
    /**
     * An identifier that links together multiple tasks and other requests that were created in the same context.
     */
    groupIdentifier?: Identifier | undefined;
    /**
     * The business identifier for this task.
     */
    identifier?: Identifier[] | undefined;
    /**
     * Additional information that may be needed in the execution of the task.
     */
    input?: TaskInput[] | undefined;
    /**
     * The URL pointing to a *FHIR*-defined protocol, guideline, orderset or other definition that is adhered to in whole or in part by this Task.
     */
    instantiatesCanonical?: string | undefined;
    _instantiatesCanonical?: Element | undefined;
    /**
     * The URL pointing to an *externally* maintained  protocol, guideline, orderset or other definition that is adhered to in whole or in part by this Task.
     */
    instantiatesUri?: string | undefined;
    _instantiatesUri?: Element | undefined;
    /**
     * Insurance plans, coverage extensions, pre-authorizations and/or pre-determinations that may be relevant to the Task.
     */
    insurance?: Reference[] | undefined;
    /**
     * This element is immutable.  Proposed tasks, planned tasks, etc. must be distinct instances.
     * In most cases, Tasks will have an intent of "order".
     */
    intent:
        | 'unknown'
        | 'proposal'
        | 'plan'
        | 'order'
        | 'original-order'
        | 'reflex-order'
        | 'filler-order'
        | 'instance-order'
        | 'option';
    _intent?: Element | undefined;
    /**
     * The date and time of last modification to this task.
     */
    lastModified?: string | undefined;
    _lastModified?: Element | undefined;
    /**
     * This should only be specified when the Task to be/being performed happens or is expected to happen primarily within the bounds of a single Location.  Other locations (e.g. source, destination, etc.) would either be reflected on the 'basedOn' Request or be conveyed as distinct Task.input values.
     */
    location?: Reference | undefined;
    /**
     * Free-text information captured about the task as it progresses.
     */
    note?: Annotation[] | undefined;
    /**
     * Outputs produced by the Task.
     */
    output?: TaskOutput[] | undefined;
    /**
     * Tasks may be created with an owner not yet identified.
     */
    owner?: Reference | undefined;
    /**
     * This should usually be 0..1.
     */
    partOf?: Reference[] | undefined;
    /**
     * Indicates how quickly the Task should be addressed with respect to other requests.
     */
    priority?: ('routine' | 'urgent' | 'asap' | 'stat') | undefined;
    _priority?: Element | undefined;
    /**
     * This will typically not be present for Tasks with a code of 'please fulfill' as, for those, the reason for action is conveyed on the Request pointed to by Task.focus.  Some types of tasks will not need a 'reason'.  E.g. a request to discharge a patient can be inferred to be 'because the patient is ready' and this would not need a reason to be stated on the Task.
     */
    reason?: CodeableReference[] | undefined;
    /**
     * This element does not point to the Provenance associated with the *current* version of the resource - as it would be created after this version existed.  The Provenance for the current version can be retrieved with a _revinclude.
     */
    relevantHistory?: Reference[] | undefined;
    /**
     * The kind of participant or specific participant that should perform the task.
     */
    requestedPerformer?: CodeableReference[] | undefined;
    /**
     * This is typically used when the Task is *not* seeking fulfillment of a focus Request, as in that case the period would be specified on the Request and/or in the Task.restriction.period.  Instead, it is used for stand-alone tasks.
     */
    requestedPeriod?: Period | undefined;
    /**
     * The creator of the task.
     */
    requester?: Reference | undefined;
    /**
     * Task.restriction can only be present if the Task is seeking fulfillment of another Request resource, and the restriction identifies what subset of the authorization conveyed by the request is supposed to be fulfilled by this Task. A possible example could be a standing order (the request) covering a significant time period and/or individuals, while the Task seeks fulfillment for only a subset of that time-period and a single individual.
     */
    restriction?: TaskRestriction | undefined;
    /**
     * The current status of the task.
     */
    status:
        | 'draft'
        | 'requested'
        | 'received'
        | 'accepted'
        | 'rejected'
        | 'ready'
        | 'cancelled'
        | 'in-progress'
        | 'on-hold'
        | 'failed'
        | 'completed'
        | 'entered-in-error';
    _status?: Element | undefined;
    /**
     * This applies to the current status.  Look at the history of the task to see reasons for past statuses.
     */
    statusReason?: CodeableReference | undefined;
}
/**
 * Software that is covered by this terminology capability statement.  It is used when the statement describes the capabilities of a particular software version, independent of an installation.
 */
export interface TerminologyCapabilitiesSoftware extends BackboneElement {
    /**
     * Name the software is known by.
     */
    name: string;
    _name?: Element | undefined;
    /**
     * If possible, a version should be specified, as statements are likely to be different for different versions of software.
     */
    version?: string | undefined;
    _version?: Element | undefined;
}
/**
 * Identifies a specific implementation instance that is described by the terminology capability statement - i.e. a particular installation, rather than the capabilities of a software program.
 */
export interface TerminologyCapabilitiesImplementation extends BackboneElement {
    /**
     * Information about the specific installation that this terminology capability statement relates to.
     */
    description: string;
    _description?: Element | undefined;
    /**
     * An absolute base URL for the implementation.
     */
    url?: string | undefined;
    _url?: Element | undefined;
}
/**
 * Filter Properties supported.
 */
export interface TerminologyCapabilitiesCodeSystemVersionFilter extends BackboneElement {
    /**
     * Code of the property supported.
     */
    code: string;
    _code?: Element | undefined;
    /**
     * Operations supported for the property.
     */
    op: string[];
    _op?: Element[] | undefined;
}
/**
 * Language translations might not be available for all codes.
 */
export interface TerminologyCapabilitiesCodeSystemVersion extends BackboneElement {
    /**
     * For version-less code systems, there should be a single version with no identifier.
     */
    code?: string | undefined;
    _code?: Element | undefined;
    /**
     * If the compositional grammar defined by the code system is supported.
     */
    compositional?: boolean | undefined;
    _compositional?: Element | undefined;
    /**
     * Filter Properties supported.
     */
    filter?: TerminologyCapabilitiesCodeSystemVersionFilter[] | undefined;
    /**
     * If this is the default version for this code system.
     */
    isDefault?: boolean | undefined;
    _isDefault?: Element | undefined;
    /**
     * Language Displays supported.
     */
    language?: string[] | undefined;
    _language?: Element[] | undefined;
    /**
     * Properties supported for $lookup.
     */
    property?: string[] | undefined;
    _property?: Element[] | undefined;
}
/**
 * The code system - identified by its system URL - may also be declared explicitly as a Code System Resource at /CodeSystem, but it might not be.
 */
export interface TerminologyCapabilitiesCodeSystem extends BackboneElement {
    /**
     * The extent of the content of the code system (the concepts and codes it defines) are represented in this resource instance.
     */
    content: 'not-present' | 'example' | 'fragment' | 'complete' | 'supplement';
    _content?: Element | undefined;
    /**
     * True if subsumption is supported for this version of the code system.
     */
    subsumption?: boolean | undefined;
    _subsumption?: Element | undefined;
    /**
     * Canonical identifier for the code system, represented as a URI.
     */
    uri?: string | undefined;
    _uri?: Element | undefined;
    /**
     * Language translations might not be available for all codes.
     */
    version?: TerminologyCapabilitiesCodeSystemVersion[] | undefined;
}
/**
 * Supported expansion parameter.
 */
export interface TerminologyCapabilitiesExpansionParameter extends BackboneElement {
    /**
     * Description of support for parameter.
     */
    documentation?: string | undefined;
    _documentation?: Element | undefined;
    /**
     * Name of the supported expansion parameter.
     */
    name: string;
    _name?: Element | undefined;
}
/**
 * Information about the [ValueSet/$expand](valueset-operation-expand.html) operation.
 */
export interface TerminologyCapabilitiesExpansion extends BackboneElement {
    /**
     * Whether the server can return nested value sets.
     */
    hierarchical?: boolean | undefined;
    _hierarchical?: Element | undefined;
    /**
     * True if requests for incomplete expansions are allowed.
     */
    incomplete?: boolean | undefined;
    _incomplete?: Element | undefined;
    /**
     * Whether the server supports paging on expansion.
     */
    paging?: boolean | undefined;
    _paging?: Element | undefined;
    /**
     * Supported expansion parameter.
     */
    parameter?: TerminologyCapabilitiesExpansionParameter[] | undefined;
    /**
     * This documentation should cover things like case sensitivity,  use of punctuation if not ignored, what wild cards are supported (if any), whether text is starts with or contains, and whether word order matters.
     */
    textFilter?: string | undefined;
    _textFilter?: Element | undefined;
}
/**
 * Information about the [ValueSet/$validate-code](valueset-operation-validate-code.html) operation.
 */
export interface TerminologyCapabilitiesValidateCode extends BackboneElement {
    /**
     * Whether translations are validated.
     */
    translations: boolean;
    _translations?: Element | undefined;
}
/**
 * Information about the [ConceptMap/$translate](conceptmap-operation-translate.html) operation.
 */
export interface TerminologyCapabilitiesTranslation extends BackboneElement {
    /**
     * Whether the client must identify the map.
     */
    needsMap: boolean;
    _needsMap?: Element | undefined;
}
/**
 * Whether the $closure operation is supported.
 */
export interface TerminologyCapabilitiesClosure extends BackboneElement {
    /**
     * If cross-system closure is supported.
     */
    translation?: boolean | undefined;
    _translation?: Element | undefined;
}
/**
 * A TerminologyCapabilities resource documents a set of capabilities (behaviors) of a FHIR Terminology Server that may be used as a statement of actual server functionality or a statement of required or desired server implementation.
 */
export interface TerminologyCapabilities extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'TerminologyCapabilities';
    /**
     * Whether the $closure operation is supported.
     */
    closure?: TerminologyCapabilitiesClosure | undefined;
    /**
     * See notes on the [ValueSet](valueset.html#) resource.
     */
    codeSearch?: ('in-compose' | 'in-expansion' | 'in-compose-or-expansion') | undefined;
    _codeSearch?: Element | undefined;
    /**
     * The code system - identified by its system URL - may also be declared explicitly as a Code System Resource at /CodeSystem, but it might not be.
     */
    codeSystem?: TerminologyCapabilitiesCodeSystem[] | undefined;
    /**
     * May be a web site, an email address, a telephone number, etc.
     */
    contact?: ContactDetail[] | undefined;
    /**
     * A copyright statement relating to the terminology capabilities and/or its contents. Copyright statements are generally legal restrictions on the use and publishing of the terminology capabilities.
     */
    copyright?: string | undefined;
    _copyright?: Element | undefined;
    /**
     * Note that this is not the same as the resource last-modified-date, since the resource may be a secondary representation of the terminology capabilities. Additional specific dates may be added as extensions or be found by consulting Provenances associated with past versions of the resource.
     */
    date: string;
    _date?: Element | undefined;
    /**
     * This description can be used to capture details such as comments about misuse, instructions for clinical use and interpretation, literature references, examples from the paper world, etc. It is not a rendering of the terminology capabilities as conveyed in the 'text' field of the resource itself. This item SHOULD be populated unless the information is available from context (e.g. the language of the terminology capabilities is presumed to be the predominant language in the place the terminology capabilities was created).This does not need to be populated if the description is adequately implied by the software or implementation details.
     */
    description?: string | undefined;
    _description?: Element | undefined;
    /**
     * Information about the [ValueSet/$expand](valueset-operation-expand.html) operation.
     */
    expansion?: TerminologyCapabilitiesExpansion | undefined;
    /**
     * Allows filtering of terminology capabilitiess that are appropriate for use versus not.
     */
    experimental?: boolean | undefined;
    _experimental?: Element | undefined;
    /**
     * Typically, this is used for identifiers that can go in an HL7 V3 II (instance identifier) data type, and can then identify this terminology capabilities outside of FHIR, where it is not possible to use the logical URI.
     */
    identifier?: Identifier[] | undefined;
    /**
     * Identifies a specific implementation instance that is described by the terminology capability statement - i.e. a particular installation, rather than the capabilities of a software program.
     */
    implementation?: TerminologyCapabilitiesImplementation | undefined;
    /**
     * It may be possible for the terminology capabilities to be used in jurisdictions other than those for which it was originally designed or intended.
     */
    jurisdiction?: CodeableConcept[] | undefined;
    /**
     * The way that this statement is intended to be used, to describe an actual running instance of software, a particular product (kind, not instance of software) or a class of implementation (e.g. a desired purchase).
     */
    kind: 'instance' | 'capability' | 'requirements';
    _kind?: Element | undefined;
    /**
     * Whether the server supports lockedDate.
     */
    lockedDate?: boolean | undefined;
    _lockedDate?: Element | undefined;
    /**
     * The name is not expected to be globally unique. The name should be a simple alphanumeric type name to ensure that it is machine-processing friendly.
     */
    name?: string | undefined;
    _name?: Element | undefined;
    /**
     * Usually an organization but may be an individual. The publisher (or steward) of the terminology capabilities is the organization or individual primarily responsible for the maintenance and upkeep of the terminology capabilities. This is not necessarily the same individual or organization that developed and initially authored the content. The publisher is the primary point of contact for questions or issues with the terminology capabilities. This item SHOULD be populated unless the information is available from context.
     */
    publisher?: string | undefined;
    _publisher?: Element | undefined;
    /**
     * This element does not describe the usage of the terminology capabilities. Instead, it provides traceability of ''why'' the resource is either needed or ''why'' it is defined as it is.  This may be used to point to source materials or specifications that drove the structure of this terminology capabilities.
     */
    purpose?: string | undefined;
    _purpose?: Element | undefined;
    /**
     * Software that is covered by this terminology capability statement.  It is used when the statement describes the capabilities of a particular software version, independent of an installation.
     */
    software?: TerminologyCapabilitiesSoftware | undefined;
    /**
     * Allows filtering of terminology capabilitiess that are appropriate for use versus not.This is not intended for use with actual capability statements, but where capability statements are used to describe possible or desired systems.
     */
    status: 'draft' | 'active' | 'retired' | 'unknown';
    _status?: Element | undefined;
    /**
     * This name does not need to be machine-processing friendly and may contain punctuation, white-space, etc.
     */
    title?: string | undefined;
    _title?: Element | undefined;
    /**
     * Information about the [ConceptMap/$translate](conceptmap-operation-translate.html) operation.
     */
    translation?: TerminologyCapabilitiesTranslation | undefined;
    /**
     * Can be a urn:uuid: or a urn:oid: but real http: addresses are preferred.  Multiple instances may share the same URL if they have a distinct version.
     * The determination of when to create a new version of a resource (same url, new version) vs. defining a new artifact is up to the author.  Considerations for making this decision are found in [Technical and Business Versions](resource.html#versions).
     * In some cases, the resource can no longer be found at the stated url, but the url itself cannot change. Implementations can use the [meta.source](resource.html#meta) element to indicate where the current master source of the resource can be found.
     */
    url?: string | undefined;
    _url?: Element | undefined;
    /**
     * When multiple useContexts are specified, there is no expectation that all or any of the contexts apply.
     */
    useContext?: UsageContext[] | undefined;
    /**
     * Information about the [ValueSet/$validate-code](valueset-operation-validate-code.html) operation.
     */
    validateCode?: TerminologyCapabilitiesValidateCode | undefined;
    /**
     * There may be different terminology capabilities instances that have the same identifier but different versions.  The version can be appended to the url in a reference to allow a reference to a particular business version of the terminology capabilities with the format [url]|[version]. The version SHOULD NOT contain a '#' - see [Business Version](resource.html#bv-format).
     */
    version?: string | undefined;
    _version?: Element | undefined;
}
/**
 * A participant in the test execution, either the execution engine, a client, or a server.
 */
export interface TestReportParticipant extends BackboneElement {
    /**
     * The display name of the participant.
     */
    display?: string | undefined;
    _display?: Element | undefined;
    /**
     * The type of participant.
     */
    type: 'test-engine' | 'client' | 'server';
    _type?: Element | undefined;
    /**
     * The uri of the participant. An absolute URL is preferred.
     */
    uri: string;
    _uri?: Element | undefined;
}
/**
 * The operation performed.
 */
export interface TestReportSetupActionOperation extends BackboneElement {
    /**
     * A link to further details on the result.
     */
    detail?: string | undefined;
    _detail?: Element | undefined;
    /**
     * An explanatory message associated with the result.
     */
    message?: string | undefined;
    _message?: Element | undefined;
    /**
     * The result of this operation.
     */
    result: 'pass' | 'skip' | 'fail' | 'warning' | 'error';
    _result?: Element | undefined;
}
/**
 * The results of the assertion performed on the previous operations.
 */
export interface TestReportSetupActionAssert extends BackboneElement {
    /**
     * A link to further details on the result.
     */
    detail?: string | undefined;
    _detail?: Element | undefined;
    /**
     * An explanatory message associated with the result.
     */
    message?: string | undefined;
    _message?: Element | undefined;
    /**
     * The result of this assertion.
     */
    result: 'pass' | 'skip' | 'fail' | 'warning' | 'error';
    _result?: Element | undefined;
}
/**
 * An action should contain either an operation or an assertion but not both.  It can contain any number of variables.
 */
export interface TestReportSetupAction extends BackboneElement {
    /**
     * The results of the assertion performed on the previous operations.
     */
    assert?: TestReportSetupActionAssert | undefined;
    /**
     * The operation performed.
     */
    operation?: TestReportSetupActionOperation | undefined;
}
/**
 * The results of the series of required setup operations before the tests were executed.
 */
export interface TestReportSetup extends BackboneElement {
    /**
     * An action should contain either an operation or an assertion but not both.  It can contain any number of variables.
     */
    action: TestReportSetupAction[];
}
/**
 * An action should contain either an operation or an assertion but not both.  It can contain any number of variables.
 */
export interface TestReportTestAction extends BackboneElement {
    /**
     * The results of the assertion performed on the previous operations.
     */
    assert?: TestReportSetupActionAssert | undefined;
    /**
     * An operation would involve a REST request to a server.
     */
    operation?: TestReportSetupActionOperation | undefined;
}
/**
 * A test executed from the test script.
 */
export interface TestReportTest extends BackboneElement {
    /**
     * An action should contain either an operation or an assertion but not both.  It can contain any number of variables.
     */
    action: TestReportTestAction[];
    /**
     * A short description of the test used by test engines for tracking and reporting purposes.
     */
    description?: string | undefined;
    _description?: Element | undefined;
    /**
     * The name of this test used for tracking/logging purposes by test engines.
     */
    name?: string | undefined;
    _name?: Element | undefined;
}
/**
 * An action should contain either an operation or an assertion but not both.  It can contain any number of variables.
 */
export interface TestReportTeardownAction extends BackboneElement {
    /**
     * An operation would involve a REST request to a server.
     */
    operation: TestReportSetupActionOperation;
}
/**
 * The results of the series of operations required to clean up after all the tests were executed (successfully or otherwise).
 */
export interface TestReportTeardown extends BackboneElement {
    /**
     * An action should contain either an operation or an assertion but not both.  It can contain any number of variables.
     */
    action: TestReportTeardownAction[];
}
/**
 * A summary of information based on the results of executing a TestScript.
 */
export interface TestReport extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'TestReport';
    /**
     * Identifier for the TestReport assigned for external purposes outside the context of FHIR.
     */
    identifier?: Identifier | undefined;
    /**
     * Additional specific dates may be added as extensions.
     */
    issued?: string | undefined;
    _issued?: Element | undefined;
    /**
     * Not expected to be globally unique.
     */
    name?: string | undefined;
    _name?: Element | undefined;
    /**
     * A participant in the test execution, either the execution engine, a client, or a server.
     */
    participant?: TestReportParticipant[] | undefined;
    /**
     * The pass and fail result represents a completed test script execution. The pending result represents a test script execution that has not yet started or is currently in progress.
     */
    result: 'pass' | 'fail' | 'pending';
    _result?: Element | undefined;
    /**
     * The final score (percentage of tests passed) resulting from the execution of the TestScript.
     */
    score?: number | undefined;
    /**
     * The results of the series of required setup operations before the tests were executed.
     */
    setup?: TestReportSetup | undefined;
    /**
     * The status represents where the execution is currently within the test script execution life cycle.
     * This element is labeled as a modifier because the status contains codes that mark the resource as not currently valid.
     */
    status: 'completed' | 'in-progress' | 'waiting' | 'stopped' | 'entered-in-error';
    _status?: Element | undefined;
    /**
     * The results of the series of operations required to clean up after all the tests were executed (successfully or otherwise).
     */
    teardown?: TestReportTeardown | undefined;
    /**
     * A test executed from the test script.
     */
    test?: TestReportTest[] | undefined;
    /**
     * Usually an organization, but may be an individual. This item SHOULD be populated unless the information is available from context.
     */
    tester?: string | undefined;
    _tester?: Element | undefined;
    /**
     * Ideally this is an absolute URL that is used to identify the version-specific TestScript that was executed, matching the `TestScript.url`.
     */
    testScript: string;
    _testScript?: Element | undefined;
}
/**
 * The purpose of this element is to define the profile of an origin element used elsewhere in the script.  Test engines could then use the origin-profile mapping to offer a filtered list of test systems that can serve as the sender for the interaction.
 */
export interface TestScriptOrigin extends BackboneElement {
    /**
     * A given origin index (e.g. 1) can appear only once in the list (e.g. Origin 1 cannot be specified twice ... once as FormFiller and again as FormProcessor within the same script as that could get confusing during test configuration).
     * Different origin indices could play the same actor in the same test script (e.g. You could have two different test systems acting as Form-Filler).
     * The origin indices provided elsewhere in the test script must be one of these origin indices.
     */
    index: number;
    /**
     * Must be a "sender"/"client" profile.
     */
    profile: Coding;
}
/**
 * The purpose of this element is to define the profile of a destination element used elsewhere in the script.  Test engines could then use the destination-profile mapping to offer a filtered list of test systems that can serve as the receiver for the interaction.
 */
export interface TestScriptDestination extends BackboneElement {
    /**
     * A given destination index (e.g. 1) can appear only once in the list (e.g. Destination 1 cannot be specified twice ... once as Form-Manager and again as Form-Processor within the same script as that could get confusing during test configuration).
     * Different destination indices could play the same actor in the same test script (e.g. You could have two different test systems acting as Form-Manager).
     * The destination indices provided elsewhere in the test script must be one of these destination indices.
     */
    index: number;
    /**
     * Must be a "receiver"/"server" profile.
     */
    profile: Coding;
}
/**
 * A link to the FHIR specification that this test is covering.
 */
export interface TestScriptMetadataLink extends BackboneElement {
    /**
     * Short description of the link.
     */
    description?: string | undefined;
    _description?: Element | undefined;
    /**
     * URL to a particular requirement or feature within the FHIR specification.
     */
    url: string;
    _url?: Element | undefined;
}
/**
 * When the metadata capabilities section is defined at TestScript.metadata or at TestScript.setup.metadata, and the server's conformance statement does not contain the elements defined in the minimal conformance statement, then all the tests in the TestScript are skipped.  When the metadata capabilities section is defined at TestScript.test.metadata and the server's conformance statement does not contain the elements defined in the minimal conformance statement, then only that test is skipped.  The "metadata.capabilities.required" and "metadata.capabilities.validated" elements only indicate whether the capabilities are the primary focus of the test script or not.  They do not impact the skipping logic.  Capabilities whose "metadata.capabilities.validated" flag is true are the primary focus of the test script.
 */
export interface TestScriptMetadataCapability extends BackboneElement {
    /**
     * The conformance statement of the server has to contain at a minimum the contents of the reference pointed to by this element.
     */
    capabilities: string;
    _capabilities?: Element | undefined;
    /**
     * Description of the capabilities that this test script is requiring the server to support.
     */
    description?: string | undefined;
    _description?: Element | undefined;
    /**
     * Which server these requirements apply to.
     */
    destination?: number | undefined;
    /**
     * Links to the FHIR specification that describes this interaction and the resources involved in more detail.
     */
    link?: string[] | undefined;
    _link?: Element[] | undefined;
    /**
     * Which origin server these requirements apply to.
     */
    origin?: number[] | undefined;
    /**
     * Whether or not the test execution will require the given capabilities of the server in order for this test script to execute.
     */
    required: boolean;
    _required?: Element | undefined;
    /**
     * Whether or not the test execution will validate the given capabilities of the server in order for this test script to execute.
     */
    validated: boolean;
    _validated?: Element | undefined;
}
/**
 * The required capability must exist and are assumed to function correctly on the FHIR server being tested.
 */
export interface TestScriptMetadata extends BackboneElement {
    /**
     * When the metadata capabilities section is defined at TestScript.metadata or at TestScript.setup.metadata, and the server's conformance statement does not contain the elements defined in the minimal conformance statement, then all the tests in the TestScript are skipped.  When the metadata capabilities section is defined at TestScript.test.metadata and the server's conformance statement does not contain the elements defined in the minimal conformance statement, then only that test is skipped.  The "metadata.capabilities.required" and "metadata.capabilities.validated" elements only indicate whether the capabilities are the primary focus of the test script or not.  They do not impact the skipping logic.  Capabilities whose "metadata.capabilities.validated" flag is true are the primary focus of the test script.
     */
    capability: TestScriptMetadataCapability[];
    /**
     * A link to the FHIR specification that this test is covering.
     */
    link?: TestScriptMetadataLink[] | undefined;
}
/**
 * The scope indicates a conformance artifact that is tested by the test(s) within this test case and the expectation of the test outcome(s) as well as the intended test phase inclusion.
 */
export interface TestScriptScope extends BackboneElement {
    /**
     * The specific conformance artifact being tested. The canonical reference can be version-specific.
     */
    artifact: string;
    _artifact?: Element | undefined;
    /**
     * The expectation of whether the test must pass for the system to be considered conformant with the artifact: required - all tests are expected to pass, optional - all test are expected to pass but non-pass status may be allowed, strict - all tests are expected to pass and warnings are treated as a failure.
     */
    conformance?: CodeableConcept | undefined;
    /**
     * The phase of testing for this artifact: unit - development / implementation phase, integration - internal system to system phase, production - live system to system phase (Note, this may involve pii/phi data).
     */
    phase?: CodeableConcept | undefined;
}
/**
 * Fixture in the test script - by reference (uri). All fixtures are required for the test script to execute.
 */
export interface TestScriptFixture extends BackboneElement {
    /**
     * Whether or not to implicitly create the fixture during setup. If true, the fixture is automatically created on each server being tested during setup, therefore no create operation is required for this fixture in the TestScript.setup section.
     */
    autocreate: boolean;
    _autocreate?: Element | undefined;
    /**
     * Whether or not to implicitly delete the fixture during teardown. If true, the fixture is automatically deleted on each server being tested during teardown, therefore no delete operation is required for this fixture in the TestScript.teardown section.
     */
    autodelete: boolean;
    _autodelete?: Element | undefined;
    /**
     * See http://build.fhir.org/resourcelist.html for complete list of resource types.
     */
    resource?: Reference | undefined;
}
/**
 * Variables would be set based either on XPath/JSONPath expressions against fixtures (static and response), or headerField evaluations against response headers. If variable evaluates to nodelist or anything other than a primitive value, then test engine would report an error.  Variables would be used to perform clean replacements in "operation.params", "operation.requestHeader.value", and "operation.url" element values during operation calls and in "assert.value" during assertion evaluations. This limits the places that test engines would need to look for placeholders "${}".  Variables are scoped to the whole script. They are NOT evaluated at declaration. They are evaluated by test engine when used for substitutions in "operation.params", "operation.requestHeader.value", and "operation.url" element values during operation calls and in "assert.value" during assertion evaluations.  See example testscript-search.xml.
 */
export interface TestScriptVariable extends BackboneElement {
    /**
     * The purpose of this element is to allow for a pre-defined value that can be used as a default or as an override value. Test engines can optionally use this as a placeholder for user-defined execution time values.
     */
    defaultValue?: string | undefined;
    _defaultValue?: Element | undefined;
    /**
     * A free text natural language description of the variable and its purpose.
     */
    description?: string | undefined;
    _description?: Element | undefined;
    /**
     * If headerField is defined, then the variable will be evaluated against the headers that sourceId is pointing to.  If expression or path is defined, then the variable will be evaluated against the fixture body that sourceId is pointing to.  It is an error to define any combination of expression, headerField and path.
     */
    expression?: string | undefined;
    _expression?: Element | undefined;
    /**
     * If headerField is defined, then the variable will be evaluated against the headers that sourceId is pointing to.  If path is defined, then the variable will be evaluated against the fixture body that sourceId is pointing to.  It is an error to define both headerField and path.
     */
    headerField?: string | undefined;
    _headerField?: Element | undefined;
    /**
     * Displayable text string with hint help information to the user when entering a default value.
     */
    hint?: string | undefined;
    _hint?: Element | undefined;
    /**
     * Placeholders would contain the variable name wrapped in ${} in "operation.params", "operation.requestHeader.value", and "operation.url" elements.  These placeholders would need to be replaced by the variable value before the operation is executed.
     */
    name: string;
    _name?: Element | undefined;
    /**
     * If headerField is defined, then the variable will be evaluated against the headers that sourceId is pointing to.  If expression or path is defined, then the variable will be evaluated against the fixture body that sourceId is pointing to.  It is an error to define any combination of expression, headerField and path.
     */
    path?: string | undefined;
    _path?: Element | undefined;
    /**
     * This can be a statically defined fixture (at the top of the TestScript) or a dynamically set fixture created by responseId of the `action.operation` element.
     */
    sourceId?: string | undefined;
    _sourceId?: Element | undefined;
}
/**
 * This gives control to test-script writers to set headers explicitly based on test requirements.  It will allow for testing using:  - "If-Modified-Since" and "If-None-Match" headers.  See http://build.fhir.org/http.html#2.1.0.5.1 - "If-Match" header.  See http://build.fhir.org/http.html#2.1.0.11 - Conditional Create using "If-None-Exist".  See http://build.fhir.org/http.html#2.1.0.13.1 - Invalid "Content-Type" header for negative testing. - etc.
 */
export interface TestScriptSetupActionOperationRequestHeader extends BackboneElement {
    /**
     * If header element is specified, then field is required.
     */
    field: string;
    _field?: Element | undefined;
    /**
     * If header element is specified, then value is required.  No conversions will be done by the test engine e.g. "xml" to "application/fhir+xml".  The values will be set in HTTP headers "as-is".  Test engines do have to look for placeholders (${}) and replace the variable placeholders with the variable values at runtime before sending the request.
     */
    value: string;
    _value?: Element | undefined;
}
/**
 * The operation to perform.
 */
export interface TestScriptSetupActionOperation extends BackboneElement {
    /**
     * If this is specified, then test engine shall set the 'Accept' header to the corresponding value.  If you'd like to explicitly set the 'Accept' to some other value then use the 'requestHeader' element.
     */
    accept?: string | undefined;
    _accept?: Element | undefined;
    /**
     * If this is specified, then test engine shall set the 'Content-Type' header to the corresponding value.  If you'd like to explicitly set the 'Content-Type' to some other value then use the 'requestHeader' element.
     */
    contentType?: string | undefined;
    _contentType?: Element | undefined;
    /**
     * This has no impact on the verification itself.
     */
    description?: string | undefined;
    _description?: Element | undefined;
    /**
     * If multiple TestScript.destination elements are defined and operation.destination is undefined, test engine will report an error as it cannot determine what destination to use for the exchange.
     */
    destination?: number | undefined;
    /**
     * Whether or not to implicitly send the request url in encoded format. The default is true to match the standard RESTful client behavior. Set to false when communicating with a server that does not support encoded url paths.
     */
    encodeRequestUrl: boolean;
    _encodeRequestUrl?: Element | undefined;
    /**
     * This has no impact on the verification itself.
     */
    label?: string | undefined;
    _label?: Element | undefined;
    /**
     * The primary purpose of the explicit HTTP method is support of  HTTP POST method invocation of the FHIR search. Other uses will include support of negative testing.
     */
    method?: ('delete' | 'get' | 'options' | 'patch' | 'post' | 'put' | 'head') | undefined;
    _method?: Element | undefined;
    /**
     * If absent, test engine will send the message.  When present, test engine will not send the request message but will wait for the request message to be sent from this origin server.
     */
    origin?: number | undefined;
    /**
     * If "url" element is specified, then "targetId", "params", and "resource" elements will be ignored as "url" element will have everything needed for constructing the request url.  If "params" element is specified, then "targetId" element is ignored.  For FHIR operations that require a resource (e.g. "read" and "vread" operations), the "resource" element must be specified when "params" element is specified.  If "url" and "params" elements are absent, then the request url will be constructed from "targetId" fixture if present.  For "read" operation, the resource and id values will be extracted from "targetId" fixture and used to construct the url.  For "vread" and "history" operations, the versionId value will also be used.   Test engines would append whatever is specified for "params" to the URL after the resource type without tampering with the string (beyond encoding the URL for HTTP).  The "params" element does not correspond exactly to "search parameters".  Nor is it the "path".  It corresponds to the part of the URL that comes after the [type] (when "resource" element is specified); e.g. It corresponds to "/[id]/_history/[vid] {?_format=[mime-type]}" in the following operation: GET [base]/[type]/[id]/_history/[vid] {?_format=[mime-type]}  Test engines do have to look for placeholders (${}) and replace the variable placeholders with the variable values at runtime before sending the request.
     */
    params?: string | undefined;
    _params?: Element | undefined;
    /**
     * This gives control to test-script writers to set headers explicitly based on test requirements.  It will allow for testing using:  - "If-Modified-Since" and "If-None-Match" headers.  See http://build.fhir.org/http.html#2.1.0.5.1 - "If-Match" header.  See http://build.fhir.org/http.html#2.1.0.11 - Conditional Create using "If-None-Exist".  See http://build.fhir.org/http.html#2.1.0.13.1 - Invalid "Content-Type" header for negative testing. - etc.
     */
    requestHeader?: TestScriptSetupActionOperationRequestHeader[] | undefined;
    /**
     * If a requestId is supplied, then the resulting request (both headers and body) is mapped to the fixture ID (which may be entirely new and previously undeclared) designated by "requestId".  If requestId is not specified, it is the test engine's responsibility to store the request and use it as the requestId in subsequent assertions when assertion path and/or headerField is specified, direction is equal to request, and the requestId in not specified.
     */
    requestId?: string | undefined;
    _requestId?: Element | undefined;
    /**
     * If "url" element is specified, then "targetId", "params", and "resource" elements will be ignored as "url" element will have everything needed for constructing the request url.  If "params" element is specified, then "targetId" element is ignored. For FHIR operations that require a resource (e.g. "read" and "vread" operations), the "resource" element must be specified when "params" element is specified.  If "url" and "params" elements are absent, then the request url will be constructed from "targetId" fixture if present. For "read" operation, the resource and id values will be extracted from "targetId" fixture and used to construct the url. For "vread" and "history" operations, the versionId value will also be used.
     */
    resource?: string | undefined;
    _resource?: Element | undefined;
    /**
     * If a responseId is supplied, and the server responds, then the resulting response (both headers and body) is mapped to the fixture ID (which may be entirely new and previously undeclared) designated by "responseId".  If responseId is not specified, it is the test engine's responsibility to store the response and use it as the responseId in subsequent assertions when assertion path and/or headerField is specified and the responseId is not specified.
     */
    responseId?: string | undefined;
    _responseId?: Element | undefined;
    /**
     * The id of the fixture used as the body of a PUT or POST request.
     */
    sourceId?: string | undefined;
    _sourceId?: Element | undefined;
    /**
     * If "url" element is specified, then "targetId", "params", and "resource" elements will be ignored as "url" element will have everything needed for constructing the request url.  If "params" element is specified, then "targetId" element is ignored.  For FHIR operations that require a resource (e.g. "read" and "vread" operations), the "resource" element must be specified when "params" element is specified.  If "url" and "params" elements are absent, then the request url will be constructed from "targetId" fixture if present.  For "read" operation, the resource and id values will be extracted from "targetId" fixture and used to construct the url.  For "vread" and "history" operations, the versionId value will also be used.
     */
    targetId?: string | undefined;
    _targetId?: Element | undefined;
    /**
     * See http://build.fhir.org/http.html for list of server interactions.
     */
    type?: Coding | undefined;
    /**
     * Used to set the request URL explicitly.  If "url" element is defined, then "targetId", "resource", and "params" elements will be ignored.  Test engines would use whatever is specified in "url" without tampering with the string (beyond encoding the URL for HTTP).  Test engines do have to look for placeholders (${}) and replace the variable placeholders with the variable values at runtime before sending the request.
     */
    url?: string | undefined;
    _url?: Element | undefined;
}
/**
 * In order to evaluate an assertion, the request, response, and results of the most recently executed operation must always be maintained by the test engine.
 */
export interface TestScriptSetupActionAssert extends BackboneElement {
    /**
     * Thefhirpath expression to be evaluated against the expected fixture to compare to. Ignored if "assert.value" is used. The evaluation will be done before the assertion is evaluated.
     */
    compareToSourceExpression?: string | undefined;
    _compareToSourceExpression?: Element | undefined;
    /**
     * Id of the source fixture used as the contents to be evaluated by either the "source/expression" or "sourceId/path" definition.
     */
    compareToSourceId?: string | undefined;
    _compareToSourceId?: Element | undefined;
    /**
     * The XPath or JSONPath expression to be evaluated against the expected fixture to compare to. Ignored if "assert.value" is used. The evaluation will be done before the assertion is evaluated.
     */
    compareToSourcePath?: string | undefined;
    _compareToSourcePath?: Element | undefined;
    /**
     * If this is specified, then test engine shall confirm that the content-type of the last operation's headers is set to this value.  If "assert.sourceId" element is specified, then the evaluation will be done against the headers mapped to that sourceId (and not the last operation's headers).  If you'd like to have more control over the string, then use 'assert.headerField' instead.
     */
    contentType?: string | undefined;
    _contentType?: Element | undefined;
    /**
     * This has no impact on the verification itself.
     */
    description?: string | undefined;
    _description?: Element | undefined;
    /**
     * If the direction is specified as "response" (the default), then the processing of this assert is against the received response message. If the direction is specified as "request", then the processing of this assert is against the sent request message.
     */
    direction?: ('response' | 'request') | undefined;
    _direction?: Element | undefined;
    /**
     * If both "expression" and a "fixtureId" are specified, then the expression will be evaluated against the request or response body mapped to the fixtureId.  If "expression" is specified and a "fixtureId" is not, then the expression will be evaluated against the response body of the last operation.  Test engines are to store the request and response body and headers of the last operation at all times for subsequent assertions.
     */
    expression?: string | undefined;
    _expression?: Element | undefined;
    /**
     * If "headerField" is specified then "value" must be specified.  If "sourceId" is not specified, then "headerField" will be evaluated against the last operation's response headers.  Test engines are to keep track of the last operation's response body and response headers.
     */
    headerField?: string | undefined;
    _headerField?: Element | undefined;
    /**
     * This has no impact on the verification itself.
     */
    label?: string | undefined;
    _label?: Element | undefined;
    /**
     * Asserts that the response contains all the element/content in another fixture pointed to by minimumId.  This can be a statically defined fixture or one that is dynamically set via responseId.
     */
    minimumId?: string | undefined;
    _minimumId?: Element | undefined;
    /**
     * Asserts that the Bundle contains first, last, and next links.
     */
    navigationLinks?: boolean | undefined;
    _navigationLinks?: Element | undefined;
    /**
     * Operators are useful especially for negative testing.  If operator is not specified, then the "equals" operator is assumed; e.g. ```<code>   <assert>  <operator value="in" />  <responseCode value="200,201,204" />    </assert>    <assert>  <operator value="notEquals" />  <response value="okay"/>   </assert>    <assert>  <operator value="greaterThan" />    <responseHeader>     <field value="Content-Length" />     <value value="0" />    </responseHeader/>   </assert> </code> ```.
     */
    operator?:
        | (
              | 'equals'
              | 'notEquals'
              | 'in'
              | 'notIn'
              | 'greaterThan'
              | 'lessThan'
              | 'empty'
              | 'notEmpty'
              | 'contains'
              | 'notContains'
              | 'eval'
          )
        | undefined;
    _operator?: Element | undefined;
    /**
     * If both "path" and a "fixtureId" are specified, then the path will be evaluated against the request or response body mapped to the fixtureId.  If "path" is specified and a "fixtureId" is not, then the path will be evaluated against the response body of the last operation.  Test engines are to store the request and response body and headers of the last operation at all times for subsequent assertions.
     */
    path?: string | undefined;
    _path?: Element | undefined;
    /**
     * If "requestMethod" is specified then it will be used in place of "value". The "requestMethod" will evaluate against the last operation's request HTTP operation.
     */
    requestMethod?: ('delete' | 'get' | 'options' | 'patch' | 'post' | 'put' | 'head') | undefined;
    _requestMethod?: Element | undefined;
    /**
     * If "requestURL" is specified then it will be used in place of "value". The "requestURL" will evaluate against the last operation's full request URL path string.
     */
    requestURL?: string | undefined;
    _requestURL?: Element | undefined;
    /**
     * This will be expected resource type in response body e.g. in read, vread, search, etc.  See http://build.fhir.org/resourcelist.html for complete list of resource types; e.g. <assert > <resourceType value="Patient" </assert>.
     */
    resource?: string | undefined;
    _resource?: Element | undefined;
    /**
     * This is a shorter way of achieving similar verifications via "assert.responseCode".  If you need more control, then use "assert.responseCode"  e.g. <assert>  <contentType value="json" />  <response value="okay"/> </assert>.
     */
    response?:
        | (
              | 'okay'
              | 'created'
              | 'noContent'
              | 'notModified'
              | 'bad'
              | 'forbidden'
              | 'notFound'
              | 'methodNotAllowed'
              | 'conflict'
              | 'gone'
              | 'preconditionFailed'
              | 'unprocessable'
          )
        | undefined;
    _response?: Element | undefined;
    /**
     * To be used with "operator" attribute value. Asserts that the response code equals this value if "operator" is not specified.   If the operator is "in" or "notIn" then the responseCode would be a comma-separated list of values e.g. "200,201". Otherwise, it's expected to be a numeric value.   If "fixture" is not specified, then the "responseBodyId" value of the last operation is assumed.
     */
    responseCode?: string | undefined;
    _responseCode?: Element | undefined;
    /**
     * This can be a statically defined fixture (at the top of the testscript) or a dynamically set fixture created by responseId of the action.operation element.
     */
    sourceId?: string | undefined;
    _sourceId?: Element | undefined;
    /**
     * If this element is specified and it is true, then assertion failures should not stop the current test execution from proceeding.
     */
    stopTestOnFail: boolean;
    _stopTestOnFail?: Element | undefined;
    /**
     * The ID of a Profile fixture. Asserts that the response is valid according to the Profile specified by validateProfileId.
     */
    validateProfileId?: string | undefined;
    _validateProfileId?: Element | undefined;
    /**
     * The string-representation of a number, string, or boolean that is expected.  Test engines do have to look for placeholders (${}) and replace the variable placeholders with the variable values at runtime before comparing this value to the actual value.
     */
    value?: string | undefined;
    _value?: Element | undefined;
    /**
     * If this element is specified and it is true, then assertion failures can be logged by test engine but should not stop the test script execution from proceeding.  There are likely cases where the spec is not clear on what should happen. If the spec says something is optional (maybe a response header for example), but a server doesn’t do it, we could choose to issue a warning.
     */
    warningOnly: boolean;
    _warningOnly?: Element | undefined;
}
/**
 * An action should contain either an operation or an assertion but not both.  It can contain any number of variables.
 */
export interface TestScriptSetupAction extends BackboneElement {
    /**
     * In order to evaluate an assertion, the request, response, and results of the most recently executed operation must always be maintained by the test engine.
     */
    assert?: TestScriptSetupActionAssert | undefined;
    /**
     * The operation to perform.
     */
    operation?: TestScriptSetupActionOperation | undefined;
}
/**
 * A series of required setup operations before tests are executed.
 */
export interface TestScriptSetup extends BackboneElement {
    /**
     * An action should contain either an operation or an assertion but not both.  It can contain any number of variables.
     */
    action: TestScriptSetupAction[];
}
/**
 * An action should contain either an operation or an assertion but not both.  It can contain any number of variables.
 */
export interface TestScriptTestAction extends BackboneElement {
    /**
     * In order to evaluate an assertion, the request, response, and results of the most recently executed operation must always be maintained by the test engine.
     */
    assert?: TestScriptSetupActionAssert | undefined;
    /**
     * An operation would involve a REST request to a server.
     */
    operation?: TestScriptSetupActionOperation | undefined;
}
/**
 * A test in this script.
 */
export interface TestScriptTest extends BackboneElement {
    /**
     * An action should contain either an operation or an assertion but not both.  It can contain any number of variables.
     */
    action: TestScriptTestAction[];
    /**
     * A short description of the test used by test engines for tracking and reporting purposes.
     */
    description?: string | undefined;
    _description?: Element | undefined;
    /**
     * The name of this test used for tracking/logging purposes by test engines.
     */
    name?: string | undefined;
    _name?: Element | undefined;
}
/**
 * An action should contain either an operation or an assertion but not both.  It can contain any number of variables.
 */
export interface TestScriptTeardownAction extends BackboneElement {
    /**
     * An operation would involve a REST request to a server.
     */
    operation: TestScriptSetupActionOperation;
}
/**
 * A series of operations required to clean up after all the tests are executed (successfully or otherwise).
 */
export interface TestScriptTeardown extends BackboneElement {
    /**
     * An action should contain either an operation or an assertion but not both.  It can contain any number of variables.
     */
    action: TestScriptTeardownAction[];
}
/**
 * A structured set of tests against a FHIR server or client implementation to determine compliance against the FHIR specification.
 */
export interface TestScript extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'TestScript';
    /**
     * May be a web site, an email address, a telephone number, etc.
     */
    contact?: ContactDetail[] | undefined;
    /**
     * ...
     */
    copyright?: string | undefined;
    _copyright?: Element | undefined;
    /**
     * The (c) symbol should NOT be included in this string. It will be added by software when rendering the notation. Full details about licensing, restrictions, warrantees, etc. goes in the more general 'copyright' element.
     */
    copyrightLabel?: string | undefined;
    _copyrightLabel?: Element | undefined;
    /**
     * Note that this is not the same as the resource last-modified-date, since the resource may be a secondary representation of the test script. Additional specific dates may be added as extensions or be found by consulting Provenances associated with past versions of the resource.
     */
    date?: string | undefined;
    _date?: Element | undefined;
    /**
     * This description can be used to capture details such as comments about misuse, instructions for clinical use and interpretation, literature references, examples from the paper world, etc. It is not a rendering of the test script as conveyed in the 'text' field of the resource itself. This item SHOULD be populated unless the information is available from context (e.g. the language of the test script is presumed to be the predominant language in the place the test script was created).
     */
    description?: string | undefined;
    _description?: Element | undefined;
    /**
     * The purpose of this element is to define the profile of a destination element used elsewhere in the script.  Test engines could then use the destination-profile mapping to offer a filtered list of test systems that can serve as the receiver for the interaction.
     */
    destination?: TestScriptDestination[] | undefined;
    /**
     * Allows filtering of test scripts that are appropriate for use versus not.
     */
    experimental?: boolean | undefined;
    _experimental?: Element | undefined;
    /**
     * Fixture in the test script - by reference (uri). All fixtures are required for the test script to execute.
     */
    fixture?: TestScriptFixture[] | undefined;
    /**
     * Typically, this is used for identifiers that can go in an HL7 V3 II (instance identifier) data type, and can then identify this test script outside of FHIR, where it is not possible to use the logical URI.
     */
    identifier?: Identifier[] | undefined;
    /**
     * It may be possible for the test script to be used in jurisdictions other than those for which it was originally designed or intended.
     */
    jurisdiction?: CodeableConcept[] | undefined;
    /**
     * The required capability must exist and are assumed to function correctly on the FHIR server being tested.
     */
    metadata?: TestScriptMetadata | undefined;
    /**
     * The name is not expected to be globally unique. The name should be a simple alphanumeric type name to ensure that it is machine-processing friendly.
     */
    name: string;
    _name?: Element | undefined;
    /**
     * The purpose of this element is to define the profile of an origin element used elsewhere in the script.  Test engines could then use the origin-profile mapping to offer a filtered list of test systems that can serve as the sender for the interaction.
     */
    origin?: TestScriptOrigin[] | undefined;
    /**
     * See http://build.fhir.org/resourcelist.html for complete list of resource types.
     */
    profile?: Reference[] | undefined;
    /**
     * Usually an organization but may be an individual. The publisher (or steward) of the test script is the organization or individual primarily responsible for the maintenance and upkeep of the test script. This is not necessarily the same individual or organization that developed and initially authored the content. The publisher is the primary point of contact for questions or issues with the test script. This item SHOULD be populated unless the information is available from context.
     */
    publisher?: string | undefined;
    _publisher?: Element | undefined;
    /**
     * This element does not describe the usage of the test script. Instead, it provides traceability of ''why'' the resource is either needed or ''why'' it is defined as it is.  This may be used to point to source materials or specifications that drove the structure of this test script.
     */
    purpose?: string | undefined;
    _purpose?: Element | undefined;
    /**
     * The scope indicates a conformance artifact that is tested by the test(s) within this test case and the expectation of the test outcome(s) as well as the intended test phase inclusion.
     */
    scope?: TestScriptScope[] | undefined;
    /**
     * A series of required setup operations before tests are executed.
     */
    setup?: TestScriptSetup | undefined;
    /**
     * Allows filtering of test scripts that are appropriate for use versus not.
     */
    status: 'draft' | 'active' | 'retired' | 'unknown';
    _status?: Element | undefined;
    /**
     * A series of operations required to clean up after all the tests are executed (successfully or otherwise).
     */
    teardown?: TestScriptTeardown | undefined;
    /**
     * A test in this script.
     */
    test?: TestScriptTest[] | undefined;
    /**
     * This name does not need to be machine-processing friendly and may contain punctuation, white-space, etc.
     */
    title?: string | undefined;
    _title?: Element | undefined;
    /**
     * Can be a urn:uuid: or a urn:oid: but real http: addresses are preferred.  Multiple instances may share the same URL if they have a distinct version.
     * The determination of when to create a new version of a resource (same url, new version) vs. defining a new artifact is up to the author.  Considerations for making this decision are found in [Technical and Business Versions](resource.html#versions).
     * In some cases, the resource can no longer be found at the stated url, but the url itself cannot change. Implementations can use the [meta.source](resource.html#meta) element to indicate where the current master source of the resource can be found.
     */
    url?: string | undefined;
    _url?: Element | undefined;
    /**
     * When multiple useContexts are specified, there is no expectation that all or any of the contexts apply.
     */
    useContext?: UsageContext[] | undefined;
    /**
     * Variables would be set based either on XPath/JSONPath expressions against fixtures (static and response), or headerField evaluations against response headers. If variable evaluates to nodelist or anything other than a primitive value, then test engine would report an error.  Variables would be used to perform clean replacements in "operation.params", "operation.requestHeader.value", and "operation.url" element values during operation calls and in "assert.value" during assertion evaluations. This limits the places that test engines would need to look for placeholders "${}".  Variables are scoped to the whole script. They are NOT evaluated at declaration. They are evaluated by test engine when used for substitutions in "operation.params", "operation.requestHeader.value", and "operation.url" element values during operation calls and in "assert.value" during assertion evaluations.  See example testscript-search.xml.
     */
    variable?: TestScriptVariable[] | undefined;
    /**
     * There may be different test script instances that have the same identifier but different versions.  The version can be appended to the url in a reference to allow a reference to a particular business version of the test script with the format [url]|[version]. The version SHOULD NOT contain a '#' - see [Business Version](resource.html#bv-format).
     */
    version?: string | undefined;
    _version?: Element | undefined;
    /**
     * If set as a string, this is a FHIRPath expression that has two additional context variables passed in - %version1 and %version2 and will return a negative number if version1 is newer, a positive number if version2 and a 0 if the version ordering can't be successfully be determined.
     */
    versionAlgorithmString?: string | undefined;
    _versionAlgorithmString?: Element | undefined;
    /**
     * If set as a string, this is a FHIRPath expression that has two additional context variables passed in - %version1 and %version2 and will return a negative number if version1 is newer, a positive number if version2 and a 0 if the version ordering can't be successfully be determined.
     */
    versionAlgorithmCoding?: Coding | undefined;
}
/**
 * If the Transport.focus is a request resource and the transport is seeking fulfillment (i.e. is asking for the request to be actioned), this element identifies any limitations on what parts of the referenced request should be actioned.
 */
export interface TransportRestriction extends BackboneElement {
    /**
     * Note that period.high is the due date representing the time by which the transport should be completed.
     */
    period?: Period | undefined;
    /**
     * For requests that are targeted to more than one potential recipient/target, to identify who is fulfillment is sought for.
     */
    recipient?: Reference[] | undefined;
    /**
     * Indicates the number of times the requested action should occur.
     */
    repetitions?: number | undefined;
}
/**
 * Additional information that may be needed in the execution of the transport.
 */
export interface TransportInput extends BackboneElement {
    /**
     * If referencing a BPMN workflow or Protocol, the "system" is the URL for the workflow definition and the code is the "name" of the required input.
     */
    type: CodeableConcept;
    /**
     * The value of the input parameter as a basic type.
     */
    valueBase64Binary?: string | undefined;
    _valueBase64Binary?: Element | undefined;
    /**
     * The value of the input parameter as a basic type.
     */
    valueBoolean?: boolean | undefined;
    _valueBoolean?: Element | undefined;
    /**
     * The value of the input parameter as a basic type.
     */
    valueCanonical?: string | undefined;
    _valueCanonical?: Element | undefined;
    /**
     * The value of the input parameter as a basic type.
     */
    valueCode?: string | undefined;
    _valueCode?: Element | undefined;
    /**
     * The value of the input parameter as a basic type.
     */
    valueDate?: string | undefined;
    _valueDate?: Element | undefined;
    /**
     * The value of the input parameter as a basic type.
     */
    valueDateTime?: string | undefined;
    _valueDateTime?: Element | undefined;
    /**
     * The value of the input parameter as a basic type.
     */
    valueDecimal?: number | undefined;
    /**
     * The value of the input parameter as a basic type.
     */
    valueId?: string | undefined;
    _valueId?: Element | undefined;
    /**
     * The value of the input parameter as a basic type.
     */
    valueInstant?: string | undefined;
    _valueInstant?: Element | undefined;
    /**
     * The value of the input parameter as a basic type.
     */
    valueInteger?: number | undefined;
    /**
     * The value of the input parameter as a basic type.
     */
    valueInteger64?: string | undefined;
    _valueInteger64?: Element | undefined;
    /**
     * The value of the input parameter as a basic type.
     */
    valueMarkdown?: string | undefined;
    _valueMarkdown?: Element | undefined;
    /**
     * The value of the input parameter as a basic type.
     */
    valueOid?: string | undefined;
    _valueOid?: Element | undefined;
    /**
     * The value of the input parameter as a basic type.
     */
    valuePositiveInt?: number | undefined;
    /**
     * The value of the input parameter as a basic type.
     */
    valueString?: string | undefined;
    _valueString?: Element | undefined;
    /**
     * The value of the input parameter as a basic type.
     */
    valueTime?: string | undefined;
    _valueTime?: Element | undefined;
    /**
     * The value of the input parameter as a basic type.
     */
    valueUnsignedInt?: number | undefined;
    /**
     * The value of the input parameter as a basic type.
     */
    valueUri?: string | undefined;
    _valueUri?: Element | undefined;
    /**
     * The value of the input parameter as a basic type.
     */
    valueUrl?: string | undefined;
    _valueUrl?: Element | undefined;
    /**
     * The value of the input parameter as a basic type.
     */
    valueUuid?: string | undefined;
    _valueUuid?: Element | undefined;
    /**
     * The value of the input parameter as a basic type.
     */
    valueAddress?: Address | undefined;
    /**
     * The value of the input parameter as a basic type.
     */
    valueAge?: Age | undefined;
    /**
     * The value of the input parameter as a basic type.
     */
    valueAnnotation?: Annotation | undefined;
    /**
     * The value of the input parameter as a basic type.
     */
    valueAttachment?: Attachment | undefined;
    /**
     * The value of the input parameter as a basic type.
     */
    valueCodeableConcept?: CodeableConcept | undefined;
    /**
     * The value of the input parameter as a basic type.
     */
    valueCodeableReference?: CodeableReference | undefined;
    /**
     * The value of the input parameter as a basic type.
     */
    valueCoding?: Coding | undefined;
    /**
     * The value of the input parameter as a basic type.
     */
    valueContactPoint?: ContactPoint | undefined;
    /**
     * The value of the input parameter as a basic type.
     */
    valueCount?: Count | undefined;
    /**
     * The value of the input parameter as a basic type.
     */
    valueDistance?: Distance | undefined;
    /**
     * The value of the input parameter as a basic type.
     */
    valueDuration?: Duration | undefined;
    /**
     * The value of the input parameter as a basic type.
     */
    valueHumanName?: HumanName | undefined;
    /**
     * The value of the input parameter as a basic type.
     */
    valueIdentifier?: Identifier | undefined;
    /**
     * The value of the input parameter as a basic type.
     */
    valueMoney?: Money | undefined;
    /**
     * The value of the input parameter as a basic type.
     */
    valuePeriod?: Period | undefined;
    /**
     * The value of the input parameter as a basic type.
     */
    valueQuantity?: Quantity | undefined;
    /**
     * The value of the input parameter as a basic type.
     */
    valueRange?: Range | undefined;
    /**
     * The value of the input parameter as a basic type.
     */
    valueRatio?: Ratio | undefined;
    /**
     * The value of the input parameter as a basic type.
     */
    valueRatioRange?: RatioRange | undefined;
    /**
     * The value of the input parameter as a basic type.
     */
    valueReference?: Reference | undefined;
    /**
     * The value of the input parameter as a basic type.
     */
    valueSampledData?: SampledData | undefined;
    /**
     * The value of the input parameter as a basic type.
     */
    valueSignature?: Signature | undefined;
    /**
     * The value of the input parameter as a basic type.
     */
    valueTiming?: Timing | undefined;
    /**
     * The value of the input parameter as a basic type.
     */
    valueContactDetail?: ContactDetail | undefined;
    /**
     * The value of the input parameter as a basic type.
     */
    valueDataRequirement?: DataRequirement | undefined;
    /**
     * The value of the input parameter as a basic type.
     */
    valueExpression?: Expression | undefined;
    /**
     * The value of the input parameter as a basic type.
     */
    valueParameterDefinition?: ParameterDefinition | undefined;
    /**
     * The value of the input parameter as a basic type.
     */
    valueRelatedArtifact?: RelatedArtifact | undefined;
    /**
     * The value of the input parameter as a basic type.
     */
    valueTriggerDefinition?: TriggerDefinition | undefined;
    /**
     * The value of the input parameter as a basic type.
     */
    valueUsageContext?: UsageContext | undefined;
    /**
     * The value of the input parameter as a basic type.
     */
    valueAvailability?: Availability | undefined;
    /**
     * The value of the input parameter as a basic type.
     */
    valueExtendedContactDetail?: ExtendedContactDetail | undefined;
    /**
     * The value of the input parameter as a basic type.
     */
    valueDosage?: Dosage | undefined;
    /**
     * The value of the input parameter as a basic type.
     */
    valueMeta?: Meta | undefined;
}
/**
 * Outputs produced by the Transport.
 */
export interface TransportOutput extends BackboneElement {
    /**
     * The name of the Output parameter.
     */
    type: CodeableConcept;
    /**
     * The value of the Output parameter as a basic type.
     */
    valueBase64Binary?: string | undefined;
    _valueBase64Binary?: Element | undefined;
    /**
     * The value of the Output parameter as a basic type.
     */
    valueBoolean?: boolean | undefined;
    _valueBoolean?: Element | undefined;
    /**
     * The value of the Output parameter as a basic type.
     */
    valueCanonical?: string | undefined;
    _valueCanonical?: Element | undefined;
    /**
     * The value of the Output parameter as a basic type.
     */
    valueCode?: string | undefined;
    _valueCode?: Element | undefined;
    /**
     * The value of the Output parameter as a basic type.
     */
    valueDate?: string | undefined;
    _valueDate?: Element | undefined;
    /**
     * The value of the Output parameter as a basic type.
     */
    valueDateTime?: string | undefined;
    _valueDateTime?: Element | undefined;
    /**
     * The value of the Output parameter as a basic type.
     */
    valueDecimal?: number | undefined;
    /**
     * The value of the Output parameter as a basic type.
     */
    valueId?: string | undefined;
    _valueId?: Element | undefined;
    /**
     * The value of the Output parameter as a basic type.
     */
    valueInstant?: string | undefined;
    _valueInstant?: Element | undefined;
    /**
     * The value of the Output parameter as a basic type.
     */
    valueInteger?: number | undefined;
    /**
     * The value of the Output parameter as a basic type.
     */
    valueInteger64?: string | undefined;
    _valueInteger64?: Element | undefined;
    /**
     * The value of the Output parameter as a basic type.
     */
    valueMarkdown?: string | undefined;
    _valueMarkdown?: Element | undefined;
    /**
     * The value of the Output parameter as a basic type.
     */
    valueOid?: string | undefined;
    _valueOid?: Element | undefined;
    /**
     * The value of the Output parameter as a basic type.
     */
    valuePositiveInt?: number | undefined;
    /**
     * The value of the Output parameter as a basic type.
     */
    valueString?: string | undefined;
    _valueString?: Element | undefined;
    /**
     * The value of the Output parameter as a basic type.
     */
    valueTime?: string | undefined;
    _valueTime?: Element | undefined;
    /**
     * The value of the Output parameter as a basic type.
     */
    valueUnsignedInt?: number | undefined;
    /**
     * The value of the Output parameter as a basic type.
     */
    valueUri?: string | undefined;
    _valueUri?: Element | undefined;
    /**
     * The value of the Output parameter as a basic type.
     */
    valueUrl?: string | undefined;
    _valueUrl?: Element | undefined;
    /**
     * The value of the Output parameter as a basic type.
     */
    valueUuid?: string | undefined;
    _valueUuid?: Element | undefined;
    /**
     * The value of the Output parameter as a basic type.
     */
    valueAddress?: Address | undefined;
    /**
     * The value of the Output parameter as a basic type.
     */
    valueAge?: Age | undefined;
    /**
     * The value of the Output parameter as a basic type.
     */
    valueAnnotation?: Annotation | undefined;
    /**
     * The value of the Output parameter as a basic type.
     */
    valueAttachment?: Attachment | undefined;
    /**
     * The value of the Output parameter as a basic type.
     */
    valueCodeableConcept?: CodeableConcept | undefined;
    /**
     * The value of the Output parameter as a basic type.
     */
    valueCodeableReference?: CodeableReference | undefined;
    /**
     * The value of the Output parameter as a basic type.
     */
    valueCoding?: Coding | undefined;
    /**
     * The value of the Output parameter as a basic type.
     */
    valueContactPoint?: ContactPoint | undefined;
    /**
     * The value of the Output parameter as a basic type.
     */
    valueCount?: Count | undefined;
    /**
     * The value of the Output parameter as a basic type.
     */
    valueDistance?: Distance | undefined;
    /**
     * The value of the Output parameter as a basic type.
     */
    valueDuration?: Duration | undefined;
    /**
     * The value of the Output parameter as a basic type.
     */
    valueHumanName?: HumanName | undefined;
    /**
     * The value of the Output parameter as a basic type.
     */
    valueIdentifier?: Identifier | undefined;
    /**
     * The value of the Output parameter as a basic type.
     */
    valueMoney?: Money | undefined;
    /**
     * The value of the Output parameter as a basic type.
     */
    valuePeriod?: Period | undefined;
    /**
     * The value of the Output parameter as a basic type.
     */
    valueQuantity?: Quantity | undefined;
    /**
     * The value of the Output parameter as a basic type.
     */
    valueRange?: Range | undefined;
    /**
     * The value of the Output parameter as a basic type.
     */
    valueRatio?: Ratio | undefined;
    /**
     * The value of the Output parameter as a basic type.
     */
    valueRatioRange?: RatioRange | undefined;
    /**
     * The value of the Output parameter as a basic type.
     */
    valueReference?: Reference | undefined;
    /**
     * The value of the Output parameter as a basic type.
     */
    valueSampledData?: SampledData | undefined;
    /**
     * The value of the Output parameter as a basic type.
     */
    valueSignature?: Signature | undefined;
    /**
     * The value of the Output parameter as a basic type.
     */
    valueTiming?: Timing | undefined;
    /**
     * The value of the Output parameter as a basic type.
     */
    valueContactDetail?: ContactDetail | undefined;
    /**
     * The value of the Output parameter as a basic type.
     */
    valueDataRequirement?: DataRequirement | undefined;
    /**
     * The value of the Output parameter as a basic type.
     */
    valueExpression?: Expression | undefined;
    /**
     * The value of the Output parameter as a basic type.
     */
    valueParameterDefinition?: ParameterDefinition | undefined;
    /**
     * The value of the Output parameter as a basic type.
     */
    valueRelatedArtifact?: RelatedArtifact | undefined;
    /**
     * The value of the Output parameter as a basic type.
     */
    valueTriggerDefinition?: TriggerDefinition | undefined;
    /**
     * The value of the Output parameter as a basic type.
     */
    valueUsageContext?: UsageContext | undefined;
    /**
     * The value of the Output parameter as a basic type.
     */
    valueAvailability?: Availability | undefined;
    /**
     * The value of the Output parameter as a basic type.
     */
    valueExtendedContactDetail?: ExtendedContactDetail | undefined;
    /**
     * The value of the Output parameter as a basic type.
     */
    valueDosage?: Dosage | undefined;
    /**
     * The value of the Output parameter as a basic type.
     */
    valueMeta?: Meta | undefined;
}
/**
 * Record of transport of item.
 */
export interface Transport extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'Transport';
    /**
     * The date and time this transport was created.
     */
    authoredOn?: string | undefined;
    _authoredOn?: Element | undefined;
    /**
     * BasedOn refers to a higher-level authorization that triggered the creation of the transport.  It references a "request" resource such as a ServiceRequest or Transport, which is distinct from the "request" resource the Transport is seeking to fulfill.  This latter resource is referenced by FocusOn.  For example, based on a ServiceRequest (= BasedOn), a transport is created to fulfill a procedureRequest ( = FocusOn ) to transport a specimen to the lab.
     */
    basedOn?: Reference[] | undefined;
    /**
     * The title (eg "My Transports", "Outstanding Transports for Patient X") should go into the code.
     */
    code?: CodeableConcept | undefined;
    /**
     * Identifies the completion time of the event (the occurrence).
     */
    completionTime?: string | undefined;
    _completionTime?: Element | undefined;
    /**
     * The current location for the entity to be transported.
     */
    currentLocation: Reference;
    /**
     * A free-text description of what is to be performed.
     */
    description?: string | undefined;
    _description?: Element | undefined;
    /**
     * The healthcare event  (e.g. a patient and healthcare provider interaction) during which this transport was created.
     */
    encounter?: Reference | undefined;
    /**
     * If multiple resources need to be manipulated, use sub-transports.  (This ensures that status can be tracked independently for each referenced resource.).
     */
    focus?: Reference | undefined;
    /**
     * The entity who benefits from the performance of the service specified in the transport (e.g., the patient).
     */
    for?: Reference | undefined;
    /**
     * An identifier that links together multiple transports and other requests that were created in the same context.
     */
    groupIdentifier?: Identifier | undefined;
    /**
     * The transport event prior to this one.
     */
    history?: Reference | undefined;
    /**
     * This identifier is typically assigned by the dispenser, and may be used to reference the delivery when exchanging information about it with other systems.
     */
    identifier?: Identifier[] | undefined;
    /**
     * Additional information that may be needed in the execution of the transport.
     */
    input?: TransportInput[] | undefined;
    /**
     * The URL pointing to a *FHIR*-defined protocol, guideline, orderset or other definition that is adhered to in whole or in part by this Transport.
     */
    instantiatesCanonical?: string | undefined;
    _instantiatesCanonical?: Element | undefined;
    /**
     * The URL pointing to an *externally* maintained  protocol, guideline, orderset or other definition that is adhered to in whole or in part by this Transport.
     */
    instantiatesUri?: string | undefined;
    _instantiatesUri?: Element | undefined;
    /**
     * Insurance plans, coverage extensions, pre-authorizations and/or pre-determinations that may be relevant to the Transport.
     */
    insurance?: Reference[] | undefined;
    /**
     * This element is immutable.  Proposed transports, planned transports, etc. must be distinct instances.
     * In most cases, Transports will have an intent of "order".
     */
    intent:
        | 'unknown'
        | 'proposal'
        | 'plan'
        | 'order'
        | 'original-order'
        | 'reflex-order'
        | 'filler-order'
        | 'instance-order'
        | 'option';
    _intent?: Element | undefined;
    /**
     * The date and time of last modification to this transport.
     */
    lastModified?: string | undefined;
    _lastModified?: Element | undefined;
    /**
     * Principal physical location where this transport is performed.
     */
    location?: Reference | undefined;
    /**
     * Free-text information captured about the transport as it progresses.
     */
    note?: Annotation[] | undefined;
    /**
     * Outputs produced by the Transport.
     */
    output?: TransportOutput[] | undefined;
    /**
     * Transports may be created with an owner not yet identified.
     */
    owner?: Reference | undefined;
    /**
     * Not to be used to link an event to an Encounter - use Event.context for that.
     * [The allowed reference resources may be adjusted as appropriate for the event resource].
     */
    partOf?: Reference[] | undefined;
    /**
     * The kind of participant that should perform the transport.
     */
    performerType?: CodeableConcept[] | undefined;
    /**
     * Indicates how quickly the Transport should be addressed with respect to other requests.
     */
    priority?: ('routine' | 'urgent' | 'asap' | 'stat') | undefined;
    _priority?: Element | undefined;
    /**
     * This should only be included if there is no focus or if it differs from the reason indicated on the focus.
     */
    reasonCode?: CodeableConcept | undefined;
    /**
     * Transports might be justified based on an Observation, a Condition, a past or planned procedure, etc.   This should only be included if there is no focus or if it differs from the reason indicated on the focus.    Use the CodeableConcept text element in `Transport.reasonCode` if the data is free (uncoded) text.
     */
    reasonReference?: Reference | undefined;
    /**
     * This element does not point to the Provenance associated with the *current* version of the resource - as it would be created after this version existed.  The Provenance for the current version can be retrieved with a _revinclude.
     */
    relevantHistory?: Reference[] | undefined;
    /**
     * The desired or final location for the transport.
     */
    requestedLocation: Reference;
    /**
     * The creator of the transport.
     */
    requester?: Reference | undefined;
    /**
     * If the Transport.focus is a request resource and the transport is seeking fulfillment (i.e. is asking for the request to be actioned), this element identifies any limitations on what parts of the referenced request should be actioned.
     */
    restriction?: TransportRestriction | undefined;
    /**
     * A code specifying the state of the transport event.
     */
    status?: ('in-progress' | 'completed' | 'abandoned' | 'cancelled' | 'planned' | 'entered-in-error') | undefined;
    _status?: Element | undefined;
    /**
     * This applies to the current status.  Look at the history of the transport to see reasons for past statuses.
     */
    statusReason?: CodeableConcept | undefined;
}
/**
 * Concepts have both a ```display``` and an array of ```designation```. The display is equivalent to a special designation with an implied ```designation.use``` of "primary code" and a language equal to the [Resource Language](resource.html#language).
 */
export interface ValueSetComposeIncludeConceptDesignation extends BackboneElement {
    /**
     * This was added rather than increasing the cardinality of .use to 0..* in order to maintain backward compatibility.
     */
    additionalUse?: Coding[] | undefined;
    /**
     * In the absence of a language, the resource language applies.
     */
    language?: string | undefined;
    _language?: Element | undefined;
    /**
     * If no use is provided, the designation can be assumed to be suitable for general display to a human user.
     */
    use?: Coding | undefined;
    /**
     * The text value for this designation.
     */
    value: string;
    _value?: Element | undefined;
}
/**
 * The list of concepts is considered ordered, though the order might not have any particular significance. Typically, the order of an expansion follows that defined in the compose element.
 */
export interface ValueSetComposeIncludeConcept extends BackboneElement {
    /**
     * Expressions are allowed if defined by the underlying code system.
     */
    code: string;
    _code?: Element | undefined;
    /**
     * Concepts have both a ```display``` and an array of ```designation```. The display is equivalent to a special designation with an implied ```designation.use``` of "primary code" and a language equal to the [Resource Language](resource.html#language).
     */
    designation?: ValueSetComposeIncludeConceptDesignation[] | undefined;
    /**
     * The value set resource allows for an alternative display to be specified for when this concept is used in this particular value set. See notes in the value set narrative about the correct use of this element.
     */
    display?: string | undefined;
    _display?: Element | undefined;
}
/**
 * Selecting codes by specifying filters based on properties is only possible where the underlying code system defines appropriate properties. Note that in some cases, the underlying code system defines the logical concepts but not the literal codes for the concepts. In such cases, the literal definitions may be provided by a third party.
 */
export interface ValueSetComposeIncludeFilter extends BackboneElement {
    /**
     * In case filter.property represents a property of the system, the operation applies to the selected property. In case filter.property represents a filter of the system, the operation SHALL match one of the CodeSystem.filter.operator values.
     */
    op:
        | '='
        | 'is-a'
        | 'descendent-of'
        | 'is-not-a'
        | 'regex'
        | 'in'
        | 'not-in'
        | 'generalizes'
        | 'child-of'
        | 'descendent-leaf'
        | 'exists';
    _op?: Element | undefined;
    /**
     * A code that identifies a property or a filter defined in the code system.
     */
    property: string;
    _property?: Element | undefined;
    /**
     * Use regex matching with care - full regex matching on every SNOMED CT term is prohibitive, for example.
     */
    value: string;
    _value?: Element | undefined;
}
/**
 * All the conditions in an include must be true. If a system is listed, all the codes from the system are listed. If one or more filters are listed, all of the filters must apply. If one or more value sets are listed, the codes must be in all the value sets. E.g. each include is 'include all the codes that meet all these conditions'.
 */
export interface ValueSetComposeInclude extends BackboneElement {
    /**
     * The list of concepts is considered ordered, though the order might not have any particular significance. Typically, the order of an expansion follows that defined in the compose element.
     */
    concept?: ValueSetComposeIncludeConcept[] | undefined;
    /**
     * A copyright statement for the specific code system asserted by the containing ValueSet.compose.include element's system value (if the associated ValueSet.compose.include.version element is not present); or the code system and version combination (if the associated ValueSet.compose.include.version element is present).
     */
    copyright?: string | undefined;
    _copyright?: Element | undefined;
    /**
     * Selecting codes by specifying filters based on properties is only possible where the underlying code system defines appropriate properties. Note that in some cases, the underlying code system defines the logical concepts but not the literal codes for the concepts. In such cases, the literal definitions may be provided by a third party.
     */
    filter?: ValueSetComposeIncludeFilter[] | undefined;
    /**
     * If there are no codes or filters, the entire code system is included. Note that the set of codes that are included may contain abstract codes. See ''Coding.system'' for further documentation about the correct value for the system element.
     */
    system?: string | undefined;
    _system?: Element | undefined;
    /**
     * The value set URI is either a logical reference to a defined value set such as a [SNOMED CT reference set](https://terminology.hl7.org/SNOMEDCT.html), or a direct reference to a value set definition using ValueSet.url. The reference might not refer to an actual FHIR ValueSet resource; in this case, whatever is referred to is an implicit definition of a value set that needs to be clear about how versions are resolved.
     */
    valueSet?: string[] | undefined;
    _valueSet?: Element[] | undefined;
    /**
     * This is used when selecting the descendants of a concept - they may change between versions. If no version is specified, then the exact contents of the value set might not be known until a context of use binds it to a particular version. The special value '*' means all versions; It is at server discretion regarding expansions and which versions must be supported.
     */
    version?: string | undefined;
    _version?: Element | undefined;
}
/**
 * A set of criteria that define the contents of the value set by including or excluding codes selected from the specified code system(s) that the value set draws from. This is also known as the Content Logical Definition (CLD).
 */
export interface ValueSetCompose extends BackboneElement {
    /**
     * Usually this is used to selectively exclude codes that were included by subsumption in the inclusions. Any display names specified for the codes are ignored.
     */
    exclude?: ValueSetComposeInclude[] | undefined;
    /**
     * Note that in the FHIR terminology framework, "deprecated" does not mean inactive, but in some code systems, e.g. LOINC, "deprecated" does mean inactive. Code systems should define what codes are considered to be inactive. If this is not clearly defined (including in the FHIR code system resource), then all codes are assumed to be active.
     * The Value Set Definition specification defines an ActiveOnly element, which is the reverse of this element e.g. (ValueSet.compose.inactive=FALSE) is the same as (VSD.ActiveOnly=TRUE).
     */
    inactive?: boolean | undefined;
    _inactive?: Element | undefined;
    /**
     * All the conditions in an include must be true. If a system is listed, all the codes from the system are listed. If one or more filters are listed, all of the filters must apply. If one or more value sets are listed, the codes must be in all the value sets. E.g. each include is 'include all the codes that meet all these conditions'.
     */
    include: ValueSetComposeInclude[];
    /**
     * With a defined lockedDate the value set is considered "Locked". Otherwise, the value set may have different expansions as underlying code systems and/or value sets evolve.  The interpretation of lockedDate is often dependent on the context - e.g. a SNOMED CT derived value set with a lockedDate will have a different expansion in USA than in UK.  If a value set specifies a version for include and exclude statements, and also specifies a locked date, the specified versions need to be available that date, or the value set will not be usable.
     */
    lockedDate?: string | undefined;
    _lockedDate?: Element | undefined;
    /**
     * Note that property names can clash, so using a URI is recommended.
     */
    property?: string[] | undefined;
    _property?: Element[] | undefined;
}
/**
 * The server decides which parameters to include here, but at a minimum, the list SHOULD include all of the parameters that affect the $expand operation. If the expansion will be persisted all of these parameters SHALL be included. If the codeSystem on the server has a specified version then this version SHALL be provided as a parameter in the expansion (note that not all code systems have a version).
 */
export interface ValueSetExpansionParameter extends BackboneElement {
    /**
     * The names are assigned at the discretion of the server.
     */
    name: string;
    _name?: Element | undefined;
    /**
     * The value of the parameter.
     */
    valueString?: string | undefined;
    _valueString?: Element | undefined;
    /**
     * The value of the parameter.
     */
    valueBoolean?: boolean | undefined;
    _valueBoolean?: Element | undefined;
    /**
     * The value of the parameter.
     */
    valueInteger?: number | undefined;
    /**
     * The value of the parameter.
     */
    valueDecimal?: number | undefined;
    /**
     * The value of the parameter.
     */
    valueUri?: string | undefined;
    _valueUri?: Element | undefined;
    /**
     * The value of the parameter.
     */
    valueCode?: string | undefined;
    _valueCode?: Element | undefined;
    /**
     * The value of the parameter.
     */
    valueDateTime?: string | undefined;
    _valueDateTime?: Element | undefined;
}
/**
 * A property defines an additional slot through which additional information can be provided about a concept.
 */
export interface ValueSetExpansionProperty extends BackboneElement {
    /**
     * A code that is used to identify the property. The code is used in ValueSet.expansion.contains.property.code.
     */
    code: string;
    _code?: Element | undefined;
    /**
     * Reference to the formal meaning of the property. One possible source of meaning is the [Concept Properties](codesystem-concept-properties.html) code system.
     */
    uri?: string | undefined;
    _uri?: Element | undefined;
}
/**
 * A subproperty value for this concept.
 */
export interface ValueSetExpansionContainsPropertySubProperty extends BackboneElement {
    /**
     * A code that is a reference to ValueSet.expansion.property.code.
     */
    code: string;
    _code?: Element | undefined;
    /**
     * The value of this subproperty.
     */
    valueCode?: string | undefined;
    _valueCode?: Element | undefined;
    /**
     * The value of this subproperty.
     */
    valueCoding?: Coding | undefined;
    /**
     * The value of this subproperty.
     */
    valueString?: string | undefined;
    _valueString?: Element | undefined;
    /**
     * The value of this subproperty.
     */
    valueInteger?: number | undefined;
    /**
     * The value of this subproperty.
     */
    valueBoolean?: boolean | undefined;
    _valueBoolean?: Element | undefined;
    /**
     * The value of this subproperty.
     */
    valueDateTime?: string | undefined;
    _valueDateTime?: Element | undefined;
    /**
     * The value of this subproperty.
     */
    valueDecimal?: number | undefined;
}
/**
 * A property value for this concept.
 */
export interface ValueSetExpansionContainsProperty extends BackboneElement {
    /**
     * A code that is a reference to ValueSet.expansion.property.code.
     */
    code: string;
    _code?: Element | undefined;
    /**
     * A subproperty value for this concept.
     */
    subProperty?: ValueSetExpansionContainsPropertySubProperty[] | undefined;
    /**
     * The value of this property.
     */
    valueCode?: string | undefined;
    _valueCode?: Element | undefined;
    /**
     * The value of this property.
     */
    valueCoding?: Coding | undefined;
    /**
     * The value of this property.
     */
    valueString?: string | undefined;
    _valueString?: Element | undefined;
    /**
     * The value of this property.
     */
    valueInteger?: number | undefined;
    /**
     * The value of this property.
     */
    valueBoolean?: boolean | undefined;
    _valueBoolean?: Element | undefined;
    /**
     * The value of this property.
     */
    valueDateTime?: string | undefined;
    _valueDateTime?: Element | undefined;
    /**
     * The value of this property.
     */
    valueDecimal?: number | undefined;
}
/**
 * The codes that are contained in the value set expansion.
 */
export interface ValueSetExpansionContains extends BackboneElement {
    /**
     * This should not be understood to exclude its use for searching (e.g. by subsumption testing). The client should know whether it is appropriate for the user to select an abstract code or not.
     */
    abstract?: boolean | undefined;
    _abstract?: Element | undefined;
    /**
     * The code for this item in the expansion hierarchy. If this code is missing the entry in the hierarchy is a place holder (abstract) and does not represent a valid code in the value set.
     */
    code?: string | undefined;
    _code?: Element | undefined;
    /**
     * If the expansion uses this element, there is  no implication about the logical relationship between them, and the  structure cannot be used for logical inferencing. The structure  exists to provide navigational assistance for helping human users to  locate codes in the expansion.
     */
    contains?: ValueSetExpansionContains[] | undefined;
    /**
     * The designations provided must be based on the value set and code system definitions.
     */
    designation?: ValueSetComposeIncludeConceptDesignation[] | undefined;
    /**
     * The recommended display for this item in the expansion.
     */
    display?: string | undefined;
    _display?: Element | undefined;
    /**
     * This should only have a value if the concept is inactive.
     */
    inactive?: boolean | undefined;
    _inactive?: Element | undefined;
    /**
     * A property value for this concept.
     */
    property?: ValueSetExpansionContainsProperty[] | undefined;
    /**
     * An absolute URI which is the code system in which the code for this item in the expansion is defined.
     */
    system?: string | undefined;
    _system?: Element | undefined;
    /**
     * The exact value of the version string is specified by the system from which the code is derived.
     */
    version?: string | undefined;
    _version?: Element | undefined;
}
/**
 * Expansion is performed to produce a collection of codes that are ready to use for data entry or validation. Value set expansions are always considered to be stateless - they are a record of the set of codes in the value set at a point in time under a given set of conditions, and are not subject to ongoing maintenance.
 * Expansion.parameter is  a simplified list of parameters - a subset of the features of the [Parameters](parameters.html) resource.
 */
export interface ValueSetExpansion extends BackboneElement {
    /**
     * The codes that are contained in the value set expansion.
     */
    contains?: ValueSetExpansionContains[] | undefined;
    /**
     * Typically, this uri is a UUID (e.g. urn:uuid:8230ff20-c97a-4167-a59d-dc2cb9df16dd).
     */
    identifier?: string | undefined;
    _identifier?: Element | undefined;
    /**
     * Clients SHOULD use the next link, if present, to page through expansion results in preference to using the offset and count parameters. Due to the optional nature of the next link, its absence does not necessarily indicate that it is the last page of results. Instead, as the offset and count parameters SHALL be populated when paging, clients can reliably use the count/offset parameters to determine whether the whole expansion is returned.
     */
    next?: string | undefined;
    _next?: Element | undefined;
    /**
     * Paging only applies to flat expansions. If a filter is applied, the count is the number of concepts that matched the filter, not the number of concepts in an unfiltered view of the expansion.
     */
    offset?: number | undefined;
    /**
     * The server decides which parameters to include here, but at a minimum, the list SHOULD include all of the parameters that affect the $expand operation. If the expansion will be persisted all of these parameters SHALL be included. If the codeSystem on the server has a specified version then this version SHALL be provided as a parameter in the expansion (note that not all code systems have a version).
     */
    parameter?: ValueSetExpansionParameter[] | undefined;
    /**
     * A property defines an additional slot through which additional information can be provided about a concept.
     */
    property?: ValueSetExpansionProperty[] | undefined;
    /**
     * This SHOULD be a fully populated instant, but in some circumstances, value sets are expanded by hand, and the expansion is published without that precision.
     */
    timestamp: string;
    _timestamp?: Element | undefined;
    /**
     * Paging only applies to flat expansions.
     */
    total?: number | undefined;
}
/**
 * Description of the semantic space the Value Set Expansion is intended to cover and should further clarify the text in ValueSet.description.
 */
export interface ValueSetScope extends BackboneElement {
    /**
     * Criteria describing which concepts or codes should be excluded and why.
     */
    exclusionCriteria?: string | undefined;
    _exclusionCriteria?: Element | undefined;
    /**
     * Criteria describing which concepts or codes should be included and why.
     */
    inclusionCriteria?: string | undefined;
    _inclusionCriteria?: Element | undefined;
}
/**
 * A ValueSet resource instance specifies a set of codes drawn from one or more code systems, intended for use in a particular context. Value sets link between [CodeSystem](codesystem.html) definitions and their use in [coded elements](terminologies.html).
 */
export interface ValueSet extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'ValueSet';
    /**
     * The 'date' element may be more recent than the approval date because of minor changes or editorial corrections.
     */
    approvalDate?: string | undefined;
    _approvalDate?: Element | undefined;
    /**
     * An individiual or organization primarily involved in the creation and maintenance of the ValueSet.
     */
    author?: ContactDetail[] | undefined;
    /**
     * A set of criteria that define the contents of the value set by including or excluding codes selected from the specified code system(s) that the value set draws from. This is also known as the Content Logical Definition (CLD).
     */
    compose?: ValueSetCompose | undefined;
    /**
     * May be a web site, an email address, a telephone number, etc.
     */
    contact?: ContactDetail[] | undefined;
    /**
     * Frequently, the copyright differs between the value set and the codes that are included. The copyright statement should clearly differentiate between these when required.
     */
    copyright?: string | undefined;
    _copyright?: Element | undefined;
    /**
     * Note that this is not the same as the meta.lastUpdated which is specific to an instance of a value set resource on a server.  Additional specific dates may be added as extensions or be found by consulting Provenances associated with past versions of the resource.
     */
    date?: string | undefined;
    _date?: Element | undefined;
    /**
     * Description SHOULD contain instructions for clinical or administrative use and interpretation and information about misuse. Description SHOULD be populated unless the information is available from context. Description SHOULD be populated unless the value set is a contained resource (e.g. a value set defined solely in the context of a profile).
     */
    description?: string | undefined;
    _description?: Element | undefined;
    /**
     * An individual or organization primarily responsible for internal coherence of the ValueSet.
     */
    editor?: ContactDetail[] | undefined;
    /**
     * The effective period for a ValueSet determines when the content is applicable for usage and is independent of publication and review dates. For example, a valueset intended to be used for the year 2016 might be published in 2015.
     */
    effectivePeriod?: Period | undefined;
    /**
     * An individual or organization responsible for officially endorsing the ValueSet for use in some setting.
     */
    endorser?: ContactDetail[] | undefined;
    /**
     * Expansion is performed to produce a collection of codes that are ready to use for data entry or validation. Value set expansions are always considered to be stateless - they are a record of the set of codes in the value set at a point in time under a given set of conditions, and are not subject to ongoing maintenance.
     * Expansion.parameter is  a simplified list of parameters - a subset of the features of the [Parameters](parameters.html) resource.
     */
    expansion?: ValueSetExpansion | undefined;
    /**
     * Allows filtering of value sets that are appropriate for use versus not.
     */
    experimental?: boolean | undefined;
    _experimental?: Element | undefined;
    /**
     * Typically, this is used for identifiers that can go in an HL7 V3 II (instance identifier) data type, and can then identify this value set outside of FHIR, where it is not possible to use the logical URI.
     */
    identifier?: Identifier[] | undefined;
    /**
     * Normally immutability is set to 'false', which is the default assumption if it is not populated.  Note that the implication is that if this is set to 'true', there may be only one ValueSet version for this definition. Immutability tends to be set to 'true' in one of two cases: - Where the value set, by the nature of its usage, cannot change.  For example "All specializations of ACT in ActClassCode" - Where there's no safe way to express the "Purpose" such that someone else could safely make changes to the value set definition. Source workflow control must guarantee that the same URI always yields the same definition.
     */
    immutable?: boolean | undefined;
    _immutable?: Element | undefined;
    /**
     * It may be possible for the value set to be used in jurisdictions other than those for which it was originally designed or intended.
     */
    jurisdiction?: CodeableConcept[] | undefined;
    /**
     * If specified, this date follows the original approval date.
     */
    lastReviewDate?: string | undefined;
    _lastReviewDate?: Element | undefined;
    /**
     * The name is not expected to be globally unique. The name should be a simple alphanumeric type name to ensure that it is machine-processing friendly.A name should be provided unless the value set is a contained resource (e.g. an anonymous value set in a profile). Most registries will require a name.
     */
    name?: string | undefined;
    _name?: Element | undefined;
    /**
     * Usually an organization but may be an individual. This is not necessarily the same individual or organization that developed and initially authored the content. The publisher is the primary point of contact for questions or issues with the value set. This item SHOULD be populated unless the information is available from context.
     */
    publisher?: string | undefined;
    _publisher?: Element | undefined;
    /**
     * This element does not describe the usage of the value set. Instead, it provides traceability of ''why'' the resource is either needed or ''why'' it is defined as it is.  This may be used to point to source materials or specifications that drove the structure of this value set.
     */
    purpose?: string | undefined;
    _purpose?: Element | undefined;
    /**
     * Each related artifact is either an attachment, or a reference to another resource, but not both.
     */
    relatedArtifact?: RelatedArtifact[] | undefined;
    /**
     * An individual or organization primarily responsible for review of some aspect of the ValueSet.
     */
    reviewer?: ContactDetail[] | undefined;
    /**
     * Description of the semantic space the Value Set Expansion is intended to cover and should further clarify the text in ValueSet.description.
     */
    scope?: ValueSetScope | undefined;
    /**
     * Allows filtering of value sets that are appropriate for use versus not.See also the [valueset-workflowStatusDescription](extension-valueset-workflowstatusdescription.html) extension for additional status information related to the editorial process.
     */
    status: 'draft' | 'active' | 'retired' | 'unknown';
    _status?: Element | undefined;
    /**
     * This name does not need to be machine-processing friendly and may contain punctuation, white-space, etc.
     */
    title?: string | undefined;
    _title?: Element | undefined;
    /**
     * This element provides topical categorization and keywords for the ValueSet, as opposed to the more structured context-of-use information provided in the useContext element
     */
    topic?: CodeableConcept[] | undefined;
    /**
     * Can be a urn:uuid: or a urn:oid: but real http: addresses are preferred.  Multiple instances may share the same URL if they have a distinct version.
     * The determination of when to create a new version of a resource (same url, new version) vs. defining a new artifact is up to the author.  Considerations for making this decision are found in [Technical and Business Versions](resource.html#versions).
     * In some cases, the resource can no longer be found at the stated url, but the url itself cannot change. Implementations can use the [meta.source](resource.html#meta) element to indicate where the current master source of the resource can be found.
     */
    url?: string | undefined;
    _url?: Element | undefined;
    /**
     * When multiple useContexts are specified, there is no expectation that all or any of the contexts apply.
     */
    useContext?: UsageContext[] | undefined;
    /**
     * There may be different value set instances that have the same identifier but different versions.  The version can be appended to the url in a reference to allow a reference to a particular business version of the value set with the format [url]|[version]. The version SHOULD NOT contain a '#' - see [Business Version](resource.html#bv-format).
     */
    version?: string | undefined;
    _version?: Element | undefined;
}
/**
 * Information about the primary source(s) involved in validation.
 */
export interface VerificationResultPrimarySource extends BackboneElement {
    /**
     * Ability of the primary source to push updates/alerts (yes; no; undetermined).
     */
    canPushUpdates?: CodeableConcept | undefined;
    /**
     * Method for communicating with the primary source (manual; API; Push).
     */
    communicationMethod?: CodeableConcept[] | undefined;
    /**
     * Type of alerts/updates the primary source can send (specific requested changes; any changes; as defined by source).
     */
    pushTypeAvailable?: CodeableConcept[] | undefined;
    /**
     * Type of primary source (License Board; Primary Education; Continuing Education; Postal Service; Relationship owner; Registration Authority; legal source; issuing source; authoritative source).
     */
    type?: CodeableConcept[] | undefined;
    /**
     * When the target was validated against the primary source.
     */
    validationDate?: string | undefined;
    _validationDate?: Element | undefined;
    /**
     * Status of the validation of the target against the primary source (successful; failed; unknown).
     */
    validationStatus?: CodeableConcept | undefined;
    /**
     * Reference to the primary source.
     */
    who?: Reference | undefined;
}
/**
 * Information about the entity attesting to information.
 */
export interface VerificationResultAttestation extends BackboneElement {
    /**
     * The method by which attested information was submitted/retrieved (manual; API; Push).
     */
    communicationMethod?: CodeableConcept | undefined;
    /**
     * The date the information was attested to.
     */
    date?: string | undefined;
    _date?: Element | undefined;
    /**
     * When the who is asserting on behalf of another (organization or individual).
     */
    onBehalfOf?: Reference | undefined;
    /**
     * A digital identity certificate associated with the proxy entity submitting attested information on behalf of the attestation source.
     */
    proxyIdentityCertificate?: string | undefined;
    _proxyIdentityCertificate?: Element | undefined;
    /**
     * Signed assertion by the proxy entity indicating that they have the right to submit attested information on behalf of the attestation source.
     */
    proxySignature?: Signature | undefined;
    /**
     * A digital identity certificate associated with the attestation source.
     */
    sourceIdentityCertificate?: string | undefined;
    _sourceIdentityCertificate?: Element | undefined;
    /**
     * Signed assertion by the attestation source that they have attested to the information.
     */
    sourceSignature?: Signature | undefined;
    /**
     * The individual or organization attesting to information.
     */
    who?: Reference | undefined;
}
/**
 * Information about the entity validating information.
 */
export interface VerificationResultValidator extends BackboneElement {
    /**
     * Signed assertion by the validator that they have validated the information.
     */
    attestationSignature?: Signature | undefined;
    /**
     * A digital identity certificate associated with the validator.
     */
    identityCertificate?: string | undefined;
    _identityCertificate?: Element | undefined;
    /**
     * Reference to the organization validating information.
     */
    organization: Reference;
}
/**
 * Describes validation requirements, source(s), status and dates for one or more elements.
 */
export interface VerificationResult extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'VerificationResult';
    /**
     * Information about the entity attesting to information.
     */
    attestation?: VerificationResultAttestation | undefined;
    /**
     * The result if validation fails (fatal; warning; record only; none).
     */
    failureAction?: CodeableConcept | undefined;
    /**
     * Frequency of revalidation.
     */
    frequency?: Timing | undefined;
    /**
     * The date/time validation was last completed (including failed validations).
     */
    lastPerformed?: string | undefined;
    _lastPerformed?: Element | undefined;
    /**
     * The frequency with which the target must be validated (none; initial; periodic).
     */
    need?: CodeableConcept | undefined;
    /**
     * The date when target is next validated, if appropriate.
     */
    nextScheduled?: string | undefined;
    _nextScheduled?: Element | undefined;
    /**
     * Information about the primary source(s) involved in validation.
     */
    primarySource?: VerificationResultPrimarySource[] | undefined;
    /**
     * The validation status of the target (attested; validated; in process; requires revalidation; validation failed; revalidation failed).
     */
    status: 'attested' | 'validated' | 'in-process' | 'req-revalid' | 'val-fail' | 'reval-fail';
    _status?: Element | undefined;
    /**
     * When the validation status was updated.
     */
    statusDate?: string | undefined;
    _statusDate?: Element | undefined;
    /**
     * A resource that was validated.
     */
    target?: Reference[] | undefined;
    /**
     * The fhirpath location(s) within the resource that was validated.
     */
    targetLocation?: string[] | undefined;
    _targetLocation?: Element[] | undefined;
    /**
     * The primary process by which the target is validated (edit check; value set; primary source; multiple sources; standalone; in context).
     */
    validationProcess?: CodeableConcept[] | undefined;
    /**
     * What the target is validated against (nothing; primary source; multiple sources).
     */
    validationType?: CodeableConcept | undefined;
    /**
     * Information about the entity validating information.
     */
    validator?: VerificationResultValidator[] | undefined;
}
/**
 * Allows for adjustment on two axis.
 */
export interface VisionPrescriptionLensSpecificationPrism extends BackboneElement {
    /**
     * Amount of prism to compensate for eye alignment in fractional units.
     */
    amount: number;
    /**
     * The relative base, or reference lens edge, for the prism.
     */
    base: 'up' | 'down' | 'in' | 'out';
    _base?: Element | undefined;
}
/**
 * Contain the details of  the individual lens specifications and serves as the authorization for the fullfillment by certified professionals.
 */
export interface VisionPrescriptionLensSpecification extends BackboneElement {
    /**
     * Power adjustment for multifocal lenses measured in dioptres (0.25 units).
     */
    add?: number | undefined;
    /**
     * The limits are +180 and -180 degrees.
     */
    axis?: number | undefined;
    /**
     * Back curvature measured in millimetres.
     */
    backCurve?: number | undefined;
    /**
     * Brand recommendations or restrictions.
     */
    brand?: string | undefined;
    _brand?: Element | undefined;
    /**
     * Special color or pattern.
     */
    color?: string | undefined;
    _color?: Element | undefined;
    /**
     * Power adjustment for astigmatism measured in dioptres (0.25 units).
     */
    cylinder?: number | undefined;
    /**
     * Contact lens diameter measured in millimetres.
     */
    diameter?: number | undefined;
    /**
     * The recommended maximum wear period for the lens.
     */
    duration?: Quantity | undefined;
    /**
     * May also appear on the paper claim form or in the Medical Records as as OD (oculus dexter) for the right eye and OS (oculus sinister) for the left eye.
     */
    eye: 'right' | 'left';
    _eye?: Element | undefined;
    /**
     * Notes for special requirements such as coatings and lens materials.
     */
    note?: Annotation[] | undefined;
    /**
     * Contact lens power measured in dioptres (0.25 units).
     */
    power?: number | undefined;
    /**
     * Allows for adjustment on two axis.
     */
    prism?: VisionPrescriptionLensSpecificationPrism[] | undefined;
    /**
     * Identifies the type of vision correction product which is required for the patient.
     */
    product: CodeableConcept;
    /**
     * The value is negative for near-sighted and positive for far sighted.
     * Often insurance will not cover a lens with power between +75 and -75.
     */
    sphere?: number | undefined;
}
/**
 * An authorization for the provision of glasses and/or contact lenses to a patient.
 */
export interface VisionPrescription extends DomainResource {
    /** Resource Type Name (for serialization) */
    readonly resourceType: 'VisionPrescription';
    /**
     * The date this resource was created.
     */
    created: string;
    _created?: Element | undefined;
    /**
     * Jurisdictions determine the valid lifetime of a prescription. Typically vision prescriptions are valid for two years from the date written.
     */
    dateWritten: string;
    _dateWritten?: Element | undefined;
    /**
     * A reference to a resource that identifies the particular occurrence of contact between patient and health care provider during which the prescription was issued.
     */
    encounter?: Reference | undefined;
    /**
     * A unique identifier assigned to this vision prescription.
     */
    identifier?: Identifier[] | undefined;
    /**
     * Contain the details of  the individual lens specifications and serves as the authorization for the fullfillment by certified professionals.
     */
    lensSpecification: VisionPrescriptionLensSpecification[];
    /**
     * A resource reference to the person to whom the vision prescription applies.
     */
    patient: Reference;
    /**
     * The healthcare professional responsible for authorizing the prescription.
     */
    prescriber: Reference;
    /**
     * This element is labeled as a modifier because the status contains codes that mark the resource as not currently valid.
     */
    status: 'active' | 'cancelled' | 'draft' | 'entered-in-error';
    _status?: Element | undefined;
}
/**
 * Resource binding for generic use.
 */
export type FhirResource =
    | Account
    | ActivityDefinition
    | ActorDefinition
    | AdministrableProductDefinition
    | AdverseEvent
    | AllergyIntolerance
    | Appointment
    | AppointmentResponse
    | ArtifactAssessment
    | AuditEvent
    | Basic
    | Binary
    | BiologicallyDerivedProduct
    | BodyStructure
    | Bundle
    | CapabilityStatement
    | CarePlan
    | CareTeam
    | ChargeItem
    | ChargeItemDefinition
    | Citation
    | Claim
    | ClaimResponse
    | ClinicalImpression
    | ClinicalUseDefinition
    | CodeSystem
    | Communication
    | CommunicationRequest
    | CompartmentDefinition
    | Composition
    | ConceptMap
    | Condition
    | ConditionDefinition
    | Consent
    | Contract
    | Coverage
    | CoverageEligibilityRequest
    | CoverageEligibilityResponse
    | DetectedIssue
    | Device
    | DeviceDefinition
    | DeviceDispense
    | DeviceMetric
    | DeviceRequest
    | DeviceUsage
    | DiagnosticReport
    | DocumentManifest
    | DocumentReference
    | Encounter
    | Endpoint
    | EnrollmentRequest
    | EnrollmentResponse
    | EpisodeOfCare
    | EventDefinition
    | Evidence
    | EvidenceReport
    | EvidenceVariable
    | ExampleScenario
    | ExplanationOfBenefit
    | FamilyMemberHistory
    | Flag
    | FormularyItem
    | GenomicStudy
    | Goal
    | GraphDefinition
    | Group
    | GuidanceResponse
    | HealthcareService
    | ImagingSelection
    | ImagingStudy
    | Immunization
    | ImmunizationEvaluation
    | ImmunizationRecommendation
    | ImplementationGuide
    | Ingredient
    | InsurancePlan
    | InventoryReport
    | Invoice
    | Library
    | Linkage
    | List
    | Location
    | ManufacturedItemDefinition
    | Measure
    | MeasureReport
    | Medication
    | MedicationAdministration
    | MedicationDispense
    | MedicationKnowledge
    | MedicationRequest
    | MedicationUsage
    | MedicinalProductDefinition
    | MessageDefinition
    | MessageHeader
    | MolecularSequence
    | NamingSystem
    | NutritionIntake
    | NutritionOrder
    | NutritionProduct
    | Observation
    | ObservationDefinition
    | OperationDefinition
    | OperationOutcome
    | Organization
    | OrganizationAffiliation
    | PackagedProductDefinition
    | Parameters
    | Patient
    | PaymentNotice
    | PaymentReconciliation
    | Permission
    | Person
    | PlanDefinition
    | Practitioner
    | PractitionerRole
    | Procedure
    | Provenance
    | Questionnaire
    | QuestionnaireResponse
    | RegulatedAuthorization
    | RelatedPerson
    | RequestGroup
    | RequestOrchestration
    | Requirements
    | ResearchStudy
    | ResearchSubject
    | RiskAssessment
    | Schedule
    | SearchParameter
    | ServiceRequest
    | Slot
    | Specimen
    | SpecimenDefinition
    | StructureDefinition
    | StructureMap
    | Subscription
    | SubscriptionStatus
    | SubscriptionTopic
    | Substance
    | SubstanceDefinition
    | SubstanceNucleicAcid
    | SubstancePolymer
    | SubstanceProtein
    | SubstanceReferenceInformation
    | SubstanceSourceMaterial
    | SupplyDelivery
    | SupplyRequest
    | Task
    | TerminologyCapabilities
    | TestReport
    | TestScript
    | Transport
    | ValueSet
    | VerificationResult
    | VisionPrescription;
